/**
   adt::Stack -- the abstract data type stack


   Calling Sequence:

     adt::Stack([obj1, obj2, ... objn])


   Methods:

     push   -- push new elements on the stack
     pop    -- pop elements from stack
     top    -- top of the stack
     empty  -- stack empty ?
     depth  -- depth of the stack
     reverse  -- reverts the order of the stack


   Examples:

>> A := adt::Stack(1,2,3,4)

                                  Stack1

>> A::top()

                                     4

>> A::pop()

                                     4

>> A::top()

                                     3

>> A::push(42)

                                    42

>> A::depth()  

                                     4

>> A::empty()

                                   FALSE



**/

adt::Stack :=  proc() local DOM;
begin
  DOM := newDomain(genident("Stack")):
  DOM::STACK := [args()]:
		     
  DOM::pop :=
    proc()
      local popVal;
    begin
      if dom::empty() then return(FAIL) end_if;
      popVal := dom::STACK[nops(dom::STACK)];
      delete dom::STACK[nops(dom::STACK)];
      popVal
    end_proc:

  DOM::push := value -> (dom::STACK := append(dom::STACK, value); value);
  DOM::empty := () -> bool(nops(dom::STACK) = 0):
  DOM::top :=  proc()
	       begin
		 if dom::empty() then FAIL
		 else dom::STACK[nops(dom::STACK)]: end_if
	       end_proc:
  DOM::depth := () -> nops(dom::STACK):
  DOM::reverse := () -> (dom::STACK := revert(dom::STACK); null()):

  DOM
end_proc:

