// 23.1.94, kamo
/*++
 powerset(n) - powerset of a set

If n is an integer, then powerset(n) returns a set with all subsets of
{1,...,n}.  If n is a set, a set of all subsets of n is returned.
If n is a list, a set of all sublists is returned.
++*/

combinat::powerset:= 
proc(n) 
        local set, sublist, setExpected, rmMult;
begin
        /** sublist: create all sublists of a given list l
         *  l       - the list; multiple equal entries allowed
         *  rmDup   - function for removing equal entries in the powerset
         *  extargs - additional arguments for the rmDup function
         */
        sublist := proc(l : DOM_LIST, rmDup = id, extargs = null()) : DOM_LIST
            local res;
        begin
            if l = [] then [[]]
            elif nops(l) = 1 then [[], l]
            elif nops(l) = 2 then [[], [op(l, 1)], [op(l, 2)], l]
            else
                res := sublist([op(l, 2..nops(l))], rmDup, extargs);
                res := res . zip([[op(l, 1)] $ nops(res)], res, _concat);
                rmDup(res, extargs)
            end_if
        end_proc;

        if args(0) <> 1 then
                error("wrong number of arguments !");
        end_if;

        if n::dom::powerset <> FAIL then
          return(n::dom::powerset(n))
        end_if;
  
        /*
         * DOM_INT -> DOM_SET
         */
        if testtype(n,DOM_INT) then
            if n < 0 then
                error("argument must be a set or a non-negative integer!");
            end_if;
            case n
                of 0 do return({{}}); break;
                of 1 do return({{},{1}}); break;
                otherwise set := {$1..n};
            end_case;
        else
                set := n;
        end_if;

        /* rmMult :  
         * do we have to remove multiple equal entries in the powerset ?
         */
        rmMult := FALSE;
        if domtype(set) = DOM_SET then
            setExpected := TRUE;
            set := [op(set)];
        elif domtype(set) = DOM_LIST then
            setExpected := FALSE;
            set := sort(set);
            if nops({op(set)}) <> nops(set) then
                rmMult := TRUE;
            end_if;
        else
            error("argument must be a set, a list or a non-negative integer!");
        end_if;

        if rmMult then
           set := {op(sublist(set, listlib::removeDuplicates, KeepOrder))};
        else
           set := {op(sublist(set))};
        end_if;

        if setExpected then set := map(set, s -> {op(s)}) end_if;
        set 
end_proc:
