/*++
completeLDO.mu

Author: Marcus Hausdorf, Uni Karlsruhe
        (hausdorf@ira.uka.de)
Created: 	Fri Oct 22 15:28:06 1999
Last update: 	Thu Feb 24 17:46:37 2000
++*/

alias (DAT(x)=detools::data[x],
       CDAT(x)=contains(detools::data, x),
       SDAT(x,y)=sysassign(detools::data[x],y),
       S2DAT(x,y,z)= sysassign((detools::data[x][y]), z)):
       
// compares two multiindices: mi1 < mi2 <=>
// 1. order(mi1) < order(mi2)
// 2. if =, the rightmost nonzero entry of mi1 - mi2 is < 0
// (GRevLex)
detools::miLess := proc(mi1:DOM_LIST, mi2:DOM_LIST):DOM_BOOL
    local o1:DOM_INT, o2:DOM_INT, d:DOM_LIST, k:DOM_INT;
    begin
	o1 := _plus(op(mi1));
	o2 := _plus(op(mi2));
	if o1 = o2 then
	    d := zip(mi1, mi2, _subtract);
	    k := 1;
	    while ((k < DAT("nIvars")) and iszero(d[k])) do
		k := k + 1;
	    end_while;
	    return(bool(d[k] > 0));
	else
	    return(bool(o1 < o2));
	end_if;
    end_proc;

// class of a multiindex
detools::miClass := proc(mi:DOM_LIST):DOM_INT
    local k:DOM_INT;
    begin
	k := 1;
	while ((k < DAT("nIvars")) and iszero(mi[k])) do
	    k := k + 1;
	end_while;
	k;
    end_proc;
    
// returns [leading multiindex, its coefficient, column number] of a row
detools::rowLeader := proc(r:DOM_LIST):DOM_LIST
    local c_l:DOM_LIST, m:DOM_LIST, k:DOM_INT, pos:DOM_INT;
    begin
	r := map(r, (DAT("DF"))::dom2monomial @ (DAT("DF"))::lmonomial);
	c_l := map(r, op, 1);
	r := map(r, op, 2);
	m := r[nops(r)];
	pos := nops(r);
	for k from (nops(r) - 1) downto 1 do
	    if detools::miLess(m, r[k]) then
		m := r[k];
		pos := k;
	    end_if;
	end_for;
	[m, c_l[pos], pos];
    end_proc:

detools::prol := proc(eq:DOM_TABLE, v:DOM_INT):DOM_TABLE
    local der:DOM_LIST;
    begin
	eq["class"] := min(eq["class"], v);
	eq["order"] := eq["order"] + 1;
	der := (DAT("DF"))::monomial2dom(subsop([0 $ DAT("nIvars")], v = 1));
	eq["ops"] := map(eq["ops"], l -> (DAT("DF"))::_mult(der, l));
	if DAT("Compatibility") then
	    eq["rhsops"] := map(eq["rhsops"], l -> (DAT("DF"))::_mult(der, l));
	end_if;
	eq["leader"] := subsop(eq["leader"], v = eq["leader"][v] + 1);
	eq;
    end_proc:

detools::r2E := proc(r:DOM_LIST):DOM_TABLE
    local t:DOM_TABLE, tmp:DOM_LIST, tmp2, rr:DOM_INT, ss:DOM_INT;
    begin
	rr := DAT("R");
	ss := DAT("S");
	t := table();
	t["ident"] := genident("_lc_eq_");
	tmp := detools::rowLeader(r);
	t["leader"] := tmp[1];
	t["pos"] := tmp[3];
	if DAT("FractionFree") then
	    t["ops"] := r;
	    t["co"] := tmp[2];
	    if args(0) = 2 then
		t["rhsops"] := args(2);
	    end_if;
	else
	    if not iszero(tmp[2]) then
		if (DAT("CR"))::indets(tmp[2]) <> {} then
		    tmp2 := DAT("Pivots");
		    if contains(tmp2, (rr, ss)) then
			tmp2[rr, ss] := _union(tmp2[rr, ss], {tmp[2]});
		    else
			tmp2[rr, ss] := {tmp[2]};
	            end_if;  
		    SDAT("Pivots", tmp2);
		end_if;
		t["ops"] := map(r, (DAT("DF"))::Rdivide, tmp[2]);
		t["co"] := (DAT("CR"))::one;
		if args(0) = 2 then
		    t["rhsops"] := map(args(2), (DAT("DF"))::Rdivide, tmp[2]);
		end_if;
	    else 	
		t["ops"] := r;
		t["co"] := (DAT("CR"))::zero;
		if args(0) = 2 then
		    t["rhsops"] := args(2);
		end_if;
	    end_if;
	end_if;
	t["class"] := detools::miClass(tmp[1]);
	t["order"] := _plus(op(tmp[1]));
	t["lvl"] := 0;
	t["g_lvl"] := 0;
	t;
    end_proc:
    
detools::addEqs := proc(eq1:DOM_TABLE, eq2:DOM_TABLE):DOM_TABLE
    local tmp, tmp2, rr:DOM_INT, ss:DOM_INT;
    begin
	rr := DAT("R");
	ss := DAT("S");
	if DAT("FractionFree") then
	    if (DAT("Compatibility") 
		and contains(eq1, "rhsops")
		and contains(eq2, "rhsops"))
	    then
		tmp := zip(
		    map(eq1["rhsops"], (DAT("DF"))::multcoeffs, eq2["co"]),
		    map(eq2["rhsops"], (DAT("DF"))::multcoeffs, 
			(DAT("CR"))::_negate(eq1["co"])),
		    (DAT("DF"))::_plus);
	    else
		tmp := null();
	    end_if;	    
	    if (DAT("CR"))::indets(eq1["co"]) <> {} then
		tmp2 := DAT("Pivots");
		if contains(tmp2, [rr, ss]) then
		    tmp2[rr, ss] := _union(tmp2[rr, ss], {eq1["co"]});
		else
		    tmp2[rr, ss] := {eq1["co"]};
	        end_if;  
		SDAT("Pivots", tmp2);
	    end_if;	    
	    if (DAT("CR"))::indets(eq2["co"]) <> {} then
		tmp2 := DAT("Pivots");
		if contains(tmp2, [rr, ss]) then
		    tmp2[rr, ss] := _union(tmp2[rr, ss], {eq2["co"]});
		else
		    tmp2[rr, ss] := {eq2["co"]};
	        end_if;  
		SDAT("Pivots", tmp2);
	    end_if;
	    return(detools::r2E(zip(
		map(eq1["ops"], (DAT("DF"))::multcoeffs, eq2["co"]),
		map(eq2["ops"], (DAT("DF"))::multcoeffs, 
		    (DAT("CR"))::_negate(eq1["co"])),
		(DAT("DF"))::_plus), tmp));    
	else
	    if (DAT("Compatibility") 
		and contains(eq1, "rhsops")
		and contains(eq2, "rhsops"))
	    then
		tmp := zip(eq1["rhsops"], eq2["rhsops"], (DAT("DF"))::_subtract);
	    else
		tmp := null();
	    end_if;
	    return(detools::r2E(zip(eq1["ops"], eq2["ops"], (DAT("DF"))::_subtract), tmp));
	end_if;
    end_proc;

detools::addCC := proc(sys:DOM_LIST)
    local tmp:DOM_LIST, count:DOM_INT, k:DOM_INT;
    begin
	tmp := [(DAT("DF"))::zero $ nops(sys)];
	count := 0;
	for k from 1 to nops(sys) do
	    sys[k] := [sys[k], subsop(tmp, k = (DAT("DF"))::one)];
	end_for;
	SDAT("CCDOM", DAT("DF"));
	sys;
    end_proc; 
 
    
    




