/*++
completeLF.mu

Author: Marcus Hausdorf, Uni Karlsruhe
        (hausdorf@ira.uka.de)
Created: 	Tue Oct 26 12:39:23 1999
Last update: 	Thu Feb 24 17:47:40 2000
++*/

alias (DAT(x)=detools::data[x],
       CDAT(x)=contains(detools::data, x),
       SDAT(x,y)=sysassign(detools::data[x],y),
       S2DAT(x,y,z)= sysassign((detools::data[x][y]), z)):

detools::prol := proc(eq:DOM_TABLE, v:DOM_INT):DOM_TABLE
    begin
	eq["class"] := min(eq["class"], v);
	eq["order"] := eq["order"] + 1;
	eq["ops"] := (DAT("DF"))::totalDiff(eq["ops"], v);
	eq["leader"] := subsop(eq["leader"], v = eq["leader"][v] + 1);
	if DAT("Compatibility") then
	    eq["rhsops"] := (DAT("CCDOM"))::totalDiff(eq["rhsops"], 1);
	end_if;
	eq;
    end_proc;
 
detools::r2E := proc(r):DOM_TABLE
    local t:DOM_TABLE, tmp, tmp2, tmp3, rr:DOM_INT, ss:DOM_INT;
    begin
	rr := DAT("R");
	ss := DAT("S");
	t := table();
	t["ident"] := genident("_lc_eq_");
	tmp := (DAT("DF"))::lterm(r);
	if tmp = FAIL then
	    t["leader"] := [0 $ DAT("nIvars")];
	    t["pos"] := 0;
	    t["class"] := DAT("nIvars");
	    t["order"] := 0;
	else
	    t["leader"] := (DAT("DF"))::Variables::multiIndex(tmp);
	    t["pos"] := (DAT("DF"))::Variables::index(tmp);
	    t["class"] := (DAT("DF"))::Variables::class(tmp);
	    t["order"] := (DAT("DF"))::Variables::order(tmp);
	end_if;
	if DAT("FractionFree") then
	    t["ops"] := r;
	    t["co"] := (DAT("DF"))::lcoeff(r);
	    if args(0) = 2 then
		t["rhsops"] := args(2);
	    end_if;
	else
	    if not iszero((tmp2 := (DAT("DF"))::lcoeff(r))) then
		if (DAT("CR"))::indets(tmp2) <> {} then
		    tmp3 := DAT("Pivots");
		    if contains(tmp3, (rr, ss)) then
			tmp3[rr, ss] := _union(tmp3[rr, ss], {tmp2});
		    else
			tmp3[rr, ss] := {tmp2};
	            end_if;  
		    SDAT("Pivots", tmp3);
		end_if;
		t["ops"] := (DAT("DF"))::_divide(r, tmp2);
		t["co"] := (DAT("CR"))::one;
		if args(0) = 2 then
		    t["rhsops"] := (DAT("CCDOM"))::_divide(args(2), tmp2);
		end_if;
	    else
		t["ops"] := r;
		t["co"] := (DAT("CR"))::zero;
		if args(0) = 2 then
		    t["rhsops"] := args(2);
		end_if;
	    end_if;
	end_if;
	t["lvl"] := 0;
	t["g_lvl"] := 0;
	t;
    end_proc;
       
detools::addEqs := proc(eq1:DOM_TABLE, eq2:DOM_TABLE):DOM_TABLE
    local tmp, rr:DOM_INT, ss:DOM_INT;
    begin
	rr := DAT("R");
	ss := DAT("S");
	if DAT("FractionFree") then 
	    if (DAT("Compatibility") 
		and contains(eq1, "rhsops")
		and contains(eq2, "rhsops"))
	    then
		tmp := (DAT("CCDOM"))::_subtract(
		    (DAT("CCDOM"))::_mult(eq1["rhsops"], eq2["co"]),
		    (DAT("CCDOM"))::_mult(eq2["rhsops"], eq1["co"]));
	    else
		tmp := null();
	    end_if;
	    if (DAT("CR"))::indets(eq1["co"]) <> {} then
		tmp2 := DAT("Pivots");
		if contains(tmp2, [rr, ss]) then
		    tmp2[rr, ss] := _union(tmp2[rr, ss], {eq1["co"]});
		else
		    tmp2[rr, ss] := {eq1["co"]};
	        end_if;  
		SDAT("Pivots", tmp2);
	    end_if;	    
	    if (DAT("CR"))::indets(eq2["co"]) <> {} then
		tmp2 := DAT("Pivots");
		if contains(tmp2, [rr, ss]) then
		    tmp2[rr, ss] := _union(tmp2[rr, ss], {eq2["co"]});
		else
		    tmp2[rr, ss] := {eq2["co"]};
	        end_if;  
		SDAT("Pivots", tmp2);
	    end_if;
	    return(detools::r2E((DAT("DF"))::_subtract(
		(DAT("DF"))::_mult(eq2["co"], eq1["ops"]),
		(DAT("DF"))::_mult(eq1["co"], eq2["ops"])), tmp)); 
	else
	    if (DAT("Compatibility") 
		and contains(eq1, "rhsops")
		and contains(eq2, "rhsops"))
	    then
		tmp := (DAT("CCDOM"))::_subtract(eq1["rhsops"], eq2["rhsops"]);
	    else
		tmp := null();
	    end_if;
	    return(detools::r2E((DAT("DF"))::_subtract(eq1["ops"], 
		eq2["ops"]), tmp));
	end_if;
    end_proc;
        
detools::addCC := proc(sys:DOM_LIST):DOM_LIST
    local LFCC:DOM_DOMAIN, DV:DOM_DOMAIN, count:DOM_INT, k:DOM_INT, tmp;
    begin
	DV := (DAT("DF"))::Variables;
	LFCC := Dom::LinearDifferentialFunction(
	    Vars = [map(DV::indVariables, DV::expr), 
		    [genident("XC") $ i = 1..nops(sys)]],
	    Rest = [Types = {"Indep"}]);
	SDAT("CCDOM", LFCC);
	count := 0;
	tmp := (LFCC::Variables)::depVariables;
	for k from 1 to nops(sys) do
	    sys[k] := [sys[k], LFCC::convert(tmp[k])];
	end_for;
	sys;
    end_proc;

unalias(DAT, CDAT, SDAT, S2DAT):



