#++
detsys.mu
Author: Marcus Hausdorf
Created: November 17th, 1998
Last Update: 	Fri Aug 25 12:35:14 2000

detools::detSys(sys, <ind>, <dep>, Options) : Computes the determining system 
    of a system of partial differential equations. 
    
    sys : [DOM_EXPR] | [Domain]
	List of input equations; Domain must be Dom::LinearDifferentialFunction
	(DV, RDE), Dom::DifferentialPolynomial(DV, RDE) or Dom::Differential-
	Expressions(DV), where RDE = Dom::DifferentialExpression(Dom::Restric-
	tedDifferentialVariable(DV, Types = {"Indep", "Dep"})).
	
    ind, dep : [DOM_IDENT]  
	Names of the independent resp. dependent variables occurring in sys. 
	These arguments are only necessary if the equations are of type 
	DOM_EXPR.
	
    Options: (given in the form name = value) 
    
    Ansatz : DOM_EXPR | JVF
	User-defined ansatz. If a domain-type ansatz is given, the coefficient
	ring of JVF must be of type Dom::DifferentialPolynomial(DV, RDE) in
	the linear or polynomial case and of type Dom::DifferentialExpression
	in the general case.
	
    Params : [DOM_IDENT]
	For a user-defined ansatz, the names of the unknown functions must
	be specified.
	
    Interactive : TRUE | FALSE
	If TRUE, it is asked for every unsolved equation occurring during 
	computation, whether it is to be solved for its leader (by just typing
	"leader") or for some variable.
	
    Expr : TRUE | NoDiff | FALSE
	If TRUE, no domains for the output system are created. The returned 
	system is of type DOM_EXPR. For NoDiff, domains are created, but
	the result is still of type DOM_EXPR.
	
    Autoreduced : TRUE | FALSE
	If FALSE (default), no autoreduction is done. This is also the case for Expr = 
	TRUE.

    Selected : DOM_LIST
	Only the equations of sys whose indices are given in this list are taken into 
	account when computing the symmetries.
++#

alias (m_print(x) = print(Unquoted, x),
       DAT(x)=detools::data[x],
       CDAT(x)=contains(detools::data,x),
       SDAT(x,y)=sysassign(detools::data[x],y),
       T_ERR = error("wrong type of argument"),
       N_ERR = error("wrong number of arguments"),
       R_ERR = error("argument out of range"),
       D_ERR = error("unable to create DV/DF"),
       C_ERR = error("unable to convert equations"),
       S_ERR = error("unable to solve equations"),
       A_ERR = error("unable to create ansatz")):
       
detools::detSys := proc()
local  s_l, ans, sys:DOM_LIST, DV:DOM_DOMAIN, DVout:DOM_DOMAIN, DF:DOM_DOMAIN, 
    DFout:DOM_DOMAIN, JVF:DOM_DOMAIN, JDF:DOM_DOMAIN, v_l:DOM_LIST, eq, tmp;
begin
    // create the necessary domains and convert equations
    detools::getdatasymm(testargs(), args());
    ans := detools::getAnsatz();
    s_l := detools::simpSList(DAT("S_eqs"));
    sys := DAT("Eqs");
    DV := DAT("DVsymm");
    DF := DAT("DFsymm");
    JDF := DAT("JDFsymm");
    JVF := DAT("JVFsymm");
    s_l := map(s_l, l -> ([l[1], JDF::coerce(l[2])]));
    if CDAT("Selected") then
	if (not testtype(DAT("Selected"), Type::ListOf(Type::PosInt))
	    or max(op(DAT("Selected"))) > nops(sys))
	then
	    R_ERR;
	end_if;
	sys := [sys[i] $ i in DAT("Selected")];
    end_if;
    // determine domains for output
    if DAT("Expr") <> TRUE then
	DVout := Dom::DifferentialVariable(map(DV::indVariables . 
	    DV::depVariables, DV::expr), DAT("Params"));
	DFout := Dom::LinearDifferentialFunction(DVout, 
	    Dom::DifferentialExpression(Dom::RestrictedDifferentialVariable(
	    DVout, Order < 0)));
	SDAT("DVsymmout", DVout);
	SDAT("DFsymmout", DFout);
    end_if;
    // compute determiningSystem
    v_l := [op(select(_union(DF::indets(sys[i], DiffVars) $ i = 1..nops(sys)),
	l -> (DV::vartype(l) = "Deriv")))];
    tmp := JVF::prolong(ans, v_l);
    sys := map(sys, JVF::prolong(ans, v_l));
    for eq in s_l do
        sys := map(sys, JDF::subs, eq[1] = eq[2]);
    end_for;
    if DAT("Linear") or DAT("Polynomial") then
	sys := map(map(sys, JDF::coeff), (JDF::coeffRing)::expr);
    else
	sys := map(map(sys, ((l) -> (JDF::poly(l, [op(select(JDF::indets(l,hold(DiffVars)),
	    DV::vartype = "Deriv"))], hold(Expr))))), coeff);
    end_if;
    if DAT("Expr") = TRUE then
	if CDAT("Sorted") and DAT("Sorted") = TRUE then
	    sys := sort(sys, (l1, l2) -> (bool(length(l1) < length(l2))));
	end_if;
	return(sys);
    end_if;
    sys := map(sys, DFout::coerce);
    if DAT("Autoreduced") then
        sys := DFout::autoreduce(sys);
    end_if;
    if DAT("Expr") = hold(NoDiff) then
	sys := map(sys, DFout::expr);
    end_if;
    if CDAT("Sorted") and DAT("Sorted") = TRUE then
	sys := sort(sys, (l1, l2) -> (bool(length(l1) < length(l2))));
    end_if;
    sys;
end_proc:

unalias(m_print, DAT, CDAT, SDAT, T_ERR, N_ERR, R_ERR, D_ERR, C_ERR, S_ERR,
    A_ERR):

	    

	
	



