/*++
gendetsys.mu
Author : Marcus Hausdorf
Created : November 17th, 1998
Last Update : June 2nd, 1999

detools::ncDetSys(sys) : Compute the determining system for the gene-
    ralized symmetries of a system of partial differential equations.
    detools::genDetSys is called in the same way as detools::detSys, 
    with the following additional argument:
    
    - GenOrder : Order of generalized symmetry.
       
++*/

alias (m_print(x) = print(Unquoted, x),
       DAT(x)=detools::data[hold(x)],
       CDAT(x)=contains(detools::data,hold(x)),
       SDAT(x,y)=sysassign(detools::data[hold(x)],y),
       T_ERR = error("wrong type of argument"),
       N_ERR = error("wrong number of arguments"),
       R_ERR = error("argument out of range"),
       D_ERR = error("unable to create DV/DF"),
       C_ERR = error("unable to convert equations"),
       S_ERR = error("unable to solve equations"),
       A_ERR = error("unable to create ansatz")):

detools::genDetSys := proc(sys)
local k:DOM_INT, s_l, nIV:DOM_INT, nDV:DOM_INT, genOrder:DOM_INT, dv_l:
    DOM_LIST, ans, i:DOM_INT, neq_l:DOM_LIST, eq:DOM_LIST, dvo:
    DOM_INT, j2:DOM_INT, prolong2:DOM_PROC, maxOrder:DOM_INT, coeff_l:
    DOM_LIST, DV1:DOM_DOMAIN, DV2:DOM_DOMAIN, DF1:DOM_DOMAIN, DF2:DOM_DOMAIN,
    JVF:DOM_DOMAIN, j:DOM_INT;
begin
    prolong2 := proc(ind:DOM_LIST, de:DOM_LIST)
    local de2:DOM_LIST, i:DOM_INT;
    begin
        de2 := [DV1::totalDiff(de[1], ind[1]), 
                DF1::totalDiff(de[2], ind[1])];
        neq_l[k] := de2;
        k := k + 1;
        if nops(ind) + dvo < (maxOrder + genOrder) then
            for i from 1 to ind[1] do
                prolong2([i] . ind, de2);
            end_for;
        end_if;
    end_proc;

    // create the necessary domains and convert equations
    detools::getdata(testargs(), args());
    sys := DAT(Eqs);
    DV1 := DAT(DVin);
    DF1 := DAT(DFin);
    if not CDAT(Interactive) then SDAT(Interactive, FALSE) end_if;
    // solve equations
    if not CDAT(Solved) then
        s_l := [NIL $ nops(sys)];
    else
        if domtype((s_l := DAT(Solved))) <> DOM_LIST then 
            s_l := [s_l] end_if;
        s_l := s_l . [NIL $ (nops(sys) - nops(s_l))];
    end_if;
    s_l := detools::solveEquations(sys, s_l);
    // compute differential consequences
    nIV := DV1::numIndVar;
    nDV := DV1::numDepVar;
    maxOrder := max(op(map(sys, DF1::order)));    
    if not CDAT(GenOrder) then
        error("no order specified") end_if;
    genOrder := DAT(GenOrder);
    if domtype(genOrder) <> DOM_INT or genOrder < 1 then
        T_ERR end_if;
    neq_l := [NIL $ _plus((DV1::dim(maxOrder + genOrder - 
        DV1::order(s_l[i][1])) / nDV) $ i = 1..nops(s_l))];
    k := 1;    
    for eq in s_l do
        neq_l[k] := eq;
        k := k + 1;
        dvo := DV1::order(eq[1]);
        for j from 1 to nIV do
            prolong2([j], eq);
        end_for;
    end_for;
    if nops(s_l) = 1 then 
        for j from (nops(neq_l) - 1) downto 1 do
            for j2 from (j + 1) to nops(neq_l) do
                neq_l[j2][2] := DF1::diffSubs(neq_l[j2][2],
                    neq_l[j][1] = neq_l[j][2]);
            end_for;
        end_for;
        dv_l := sort(DV1::indVariables . 
            DV1::depVariables . 
            [op({op(DV1::derivatives(genOrder))} minus 
            _union({neq_l[i][1]} $ i = 1..nops(neq_l)))], 
            DV1::_less);
    else
        dv_l := sort(DV1::indVariables . 
        DV1::depVariables .
        [op({op(DV1::derivatives(genOrder))} minus
        (_union({neq_l[i][1], op(DF1::diffVars
        (neq_l[i][2]))} $ i = 1..nops(neq_l))))], 
        DV1::_less);
    end_if;
    // compute ansatz
    JVF := Dom::JetVectorField(DV1, DF1);
    SDAT(JVFsymm, JVF);
    if CDAT(Ansatz) then
        ans := JVF::coerce(DAT(Ansatz));
        if ans = FAIL then A_ERR end_if;
        if not CDAT(Coeff) then
            A_ERR end_if;
        coeff_l := DAT(Coeff);
        if (domtype(coeff_l) <> DOM_LIST
            or {op(map(coeff_l, domtype))} <> {DOM_IDENT})
        then A_ERR end_if;
	ans := JVF::evolutionaryForm(ans);
	DV2 := Dom::DifferentialVariable(map(dv_l, DV1::expr), coeff_l);
//    else
//        ans := JVF::coerce(
//            [[DF1((Q . i)(op(dv_l))), 
//            DV1::newMulti("Dep", i)] $ i = 1..nDV]);
//        DV2 := Dom::DifferentialVariable(map(dv_l, DV1::expr), 
//            [Q . i(op(dv_l)) $ i = 1..nDV]);	    
    end_if;
    SDAT(DVout, DV2);
    DF2 := Dom::LinearDifferentialFunction(DV2, 
        Dom::RestrictedDifferentialExpression(DV2, 
        "Indep"));
    SDAT(DFout, DF2);
    // call genDeterminingSystem
    sys := detools::genDeterminingSystem(sys, neq_l, ans, maxOrder, 
        genOrder);
    if not CDAT(Unsimplified) then SDAT(Unsimplified, FALSE) end_if;
    if not DAT(Unsimplified) then
	sys := DF2::autoReduce(sys);
    end_if;	
    detools::output(sys);    
end_proc;

detools::genDeterminingSystem := proc(sys:DOM_LIST, s_l:DOM_LIST, 
    ans, mOrd:DOM_INT, gOrd:DOM_INT)
local eq, DV1:DOM_DOMAIN, DF1:DOM_DOMAIN, DF2:DOM_DOMAIN, JVF:DOM_DOMAIN;
begin
    DV1 := DAT(DVin);
    DF1 := DAT(DFin);
    DF2 := DAT(DFout);
    JVF := DAT(JVFsymm);
    sys := map(sys, JVF::evolProlong(ans, gOrd + mOrd));
    for eq in s_l do
        sys := map(sys, DF1::subs, eq[1] = eq[2]);
    end_for;
    map(map(map(sys, 
        ((l) -> (DF1::poly(l, 
            select(DF1::diffVars(l), 
            ((m) -> (DV1::vartype(m) = "Deriv" and 
            DV1::order(m) > gOrd))), 
            hold(Expr))))), coeff), DF2::coerce);
end_proc:





