/*++
Author: Marcus Hausdorf, Uni Karlsruhe
        (hausdorf@ira.uka.de)
Created: 	Thu Aug 26 13:05:24 1999
Last update: 	Fri Aug 25 12:34:36 2000
++*/

alias (m_print(x) = print(Unquoted, x),
       DAT(x)=detools::data[x],
       CDAT(x)=contains(detools::data,x),
       SDAT(x,y)=sysassign(detools::data[x],y),
       T_ERR = error("wrong type of argument"),
       N_ERR = error("wrong number of arguments"),
       R_ERR = error("argument out of range"),
       D_ERR = error("unable to create DV/DF"),
       C_ERR = error("unable to convert equations"),
       A_ERR = error("unable to create ansatz")):

// auxiliary method; recognizes all differential variables
detools::subsDV := proc(de,DV)
local tmp;
begin
    tmp := DV::coerce(de);
    if tmp<>FAIL then return(tmp) end_if;
    case domtype(de)
    of DOM_INT do
    of DOM_RAT do
    of DOM_COMPLEX do
    of DOM_FLOAT do
    of DOM_BOOL do return(de);
    otherwise return(map(de, detools::subsDV,DV)); 
    end_case;
end_proc;

detools::getdatasymm := proc(test:DOM_BOOL)
local tmp, tmp2, sys:DOM_LIST, opt:DOM_EXPR, n:DOM_INT,
    DV:DOM_DOMAIN, DF:DOM_DOMAIN, RDV:DOM_DOMAIN, RDE:DOM_DOMAIN, 
    eq_l:DOM_LIST, s_l:DOM_LIST, k:DOM_INT;
begin
    sysassign(detools::data, table());		
    SDAT("Expr", TRUE);
    SDAT("Linear", FALSE);
    SDAT("Polynomial", FALSE);
    if domtype(args(2)) <> DOM_LIST then		
	SDAT("System", FALSE);
	tmp := [args(2)];
    else
	SDAT("System", TRUE);
	tmp := args(2);
    end_if;
    eq_l := [NIL $ nops(tmp)];
    s_l := [NIL $ nops(tmp)];
    for k from 1 to nops(tmp) do
	if type(tmp[k]) = "_equal" then
	    eq_l[k] := op(tmp[k], 1) - op(tmp[k], 2);
	    s_l[k] := [op(tmp[k], 1), op(tmp[k], 2)];
	else
	    eq_l[k] := tmp[k];
	end_if;
    end_for;
    DF := domtype(eq_l[1]);
    DV := DF::Variables;
    if (DV = FAIL 
	or DV::hasProp=FAIL 
	or not(DV::hasProp(Cat::DifferentialVariable))) 
    then
        if test and args(0) < 4 then
	    T_ERR
	end_if;
	DV := Dom::DifferentialVariable(args(3), args(4));
	eq_l := map(eq_l, detools::subsDV, DV);
	RDV := Dom::RestrictedDifferentialVariable(DV, 
	    Order < 1);
	RDE := Dom::DifferentialExpression(RDV);
	DF := Dom::LinearDifferentialFunction(DV, RDE);
	sys := map(eq_l, DF::coerce);
	if iszero(contains(sys, FAIL)) then
	    SDAT("Linear", TRUE);
	else	
	    DF := Dom::DifferentialPolynomial(DV, RDE);
	    sys := map(eq_l, DF::coerce);
	    if iszero(contains(sys,FAIL)) then
		SDAT("Polynomial", TRUE);
	    else
		DF := Dom::DifferentialExpression(DV);
		sys := map(eq_l, DF::coerce);
		if not iszero(contains(sys, FAIL)) then
		    error("unable to create appropiate domains");
		end_if;
	    end_if;
	end_if;
	SDAT("Expr",hold(NoDiff));
	n := 5;
    else					
	if test then
	    if not(DF::hasProp(Cat::DifferentialFunction(DV))) then
		error("inconsistent domains for differential equations");
	    end_if;
	end_if;
	sys := eq_l;
	SDAT("Expr", FALSE);
	if DF::linear then
	    SDAT("Linear", TRUE);
	elif DF::polynomial then
	    SDAT("Polynomial", TRUE);
	end_if;	    
	n := 3;
    end_if;
    for k from 1 to nops(s_l) do
	if s_l[k] = NIL then
	    tmp := DF::leader(sys[k]);
	    s_l[k] := [tmp, DF::solve(sys[k], tmp)];
	else
	    if (tmp := DV::coerce(s_l[k][1])) = FAIL then
		tmp := DF::leader(sys[k]);
		s_l[k] := [tmp, DF::solve(sys[k], tmp)];
	    else
		if (tmp2 := DF::coerce(s_l[k][2])) = FAIL then
		    T_ERR;
		else
		    s_l[k] := [tmp, tmp2];
		end_if;
	    end_if;
	end_if;
    end_for;
    if not iszero(contains(map(s_l, op, 2), FAIL)) then
	T_ERR;
    end_if;			
    SDAT("DVsymm", DV);
    SDAT("DFsymm", DF);
    SDAT("Eqs", sys);
    SDAT("S_eqs", s_l);
    tmp := split([args(n..args(0))], a -> bool(type(a) = "_equal"));
    for opt in tmp[1] do
	sysassign(detools::data[expr2text(op(opt,1))], op(opt,2));
    end_for;
    SDAT("Parms",tmp[2]);
    if not CDAT("Interactive") then SDAT("Interactive", FALSE) end_if;
    if not CDAT("Autoreduced") then SDAT("Autoreduced", FALSE) end_if;
    null();
end_proc;

detools::getAnsatz := proc()
local DV:DOM_DOMAIN, DF:DOM_DOMAIN, JVF:DOM_DOMAIN, JDF:DOM_DOMAIN, coeff_l:
    DOM_LIST, ans; 
begin
    DV := DAT("DVsymm");
    DF := DAT("DFsymm");
    if DAT("Linear") then
	JDF := Dom::DifferentialPolynomial(DV, Dom::DifferentialExpression(
	    Dom::RestrictedDifferentialVariable(DV, Order < 1)));
    else
	JDF := DF;
    end_if;
    if not CDAT("Ansatz") then
	JVF := Dom::JetVectorField(JDF, DF);
	SDAT("JDFsymm", JDF); 
	SDAT("JVFsymm", JVF);
	if CDAT("Params") then
	    if (domtype((coeff_l := DAT("Params"))) <> DOM_LIST
		or {op(map(coeff_l, domtype))} <> {DOM_IDENT}
		or nops(coeff_l) <> (DV::numIndVar + DV::numDepVar))
	    then
		A_ERR
	    end_if;
	else
	    coeff_l := [genident("XI") $ i = 1..DV::numIndVar] .
		[genident("PHI") $ i = 1..DV::numDepVar];
	end_if;
	SDAT("Params", coeff_l);
	ans := JVF::ansatz(coeff_l);
    else
	if (not CDAT("Params") 
	    or domtype((coeff_l := DAT("Params"))) <> DOM_LIST
            or {op(map(coeff_l, domtype))} <> {DOM_IDENT})
        then 
	    A_ERR 
	end_if; 
	ans := DAT("Ansatz");
	if ((ans::dom)::constructor = Dom::JetVectorField
	    and (ans::dom)::Variables = DV)
	then
	    JVF := domtype(ans); 
	    if JVF::coeffRing <> JDF then
		A_ERR;
	    end_if;
	    if JVF::opDomain <> DF then
		JVF := Dom::JetVectorField(JDF, DF);
		ans := JVF::convert(ans);
	    end_if;
	    SDAT("JDFsymm", JDF);
	    SDAT("JVFsymm", JVF);
	    return(ans);
	else
	    JVF := Dom::JetVectorField(JDF, DF);
	    if (ans := JVF::coerce(ans)) <> FAIL then
	    	SDAT("JDFsymm", JDF);
		SDAT("JVFsymm", JVF);
		return(ans);
	    else
		A_ERR;
	    end_if;
	end_if;
    end_if;
end_proc;

detools::simpSList := proc(s_l:DOM_LIST):DOM_LIST
local eq:DOM_LIST, eq2:DOM_LIST, os_l:DOM_LIST, ns_l:DOM_LIST,
    neq, dl, ds, DV:DOM_DOMAIN, JDF:DOM_DOMAIN;
begin
    DV := DAT("DVsymm");
    JDF := DAT("JDFsymm");
    if nops(s_l) = 1 then return(s_l) end_if;
    os_l := [];
    while s_l <> [] do
        ns_l := [];
        eq := s_l[1];
        os_l := append(os_l, eq);
        delete s_l[1];
        for eq2 in s_l do
            if eq[1] = eq2[1] then
                neq := eq2[2] - eq[2];
                if not JDF::iszero(neq) then
                    if DAT("Interactive") then
                        m_print("Solve ".expr2text(neq));
                        dl := input("for ?", dl);
                        if dl = hold(leader) then
                            dl := JDF::leader(neq);
                        else
                            if (dl := DV::coerce(dl)) 
                                = FAIL or iszero(contains
                                (JDF::diffVars(neq), dl))
                            then error
			        ("unable to convert input to DVin") 
                            end_if;
                        end_if;
                    else
                        dl := JDF::leader(neq);
                    end_if;
                    ds := JDF::solve(neq, dl);
                    if ds = FAIL then
                        error("unable to solve new equation")
                    end_if;
                    ns_l := append(ns_l, [dl, ds]);
                end_if;
            else
                ns_l := append(ns_l, [eq2[1],
                    JDF::subs(eq2[2], eq[1] = eq[2])]);
            end_if;
        end_for;
        s_l := ns_l;
    end_while;
    os_l;
end_proc;

unalias(m_print, DAT, CDAT, SDAT, T_ERR, N_ERR, R_ERR, D_ERR, C_ERR,
    A_ERR):





