/*++
hasHamiltonian checks whether a given system of ordinary differential
equations possesses a Hamiltonian. It returns a list of integrability
conditions which must be satisfied, if the system is to be Hamiltonian.  If
the list is empty, the system is unconditionally Hamiltonian.

Usage: hasHamiltonian([p,-F(q)],[q],[p]);
        First argument is the vector field representing the system (as a
        list of components).
        Second and third arguments are lists with the names of position and
        momentum coordinates, respectively.


Author: Werner M. Seiler, Uni Karlsruhe/Mannheim
	(wms@ira.uka.de, seiler@euler.math.uni-mannheim.de)
Created: November 16th, 1997
Last update: 	Mon Nov 22 20:52:11 1999
++*/


detools::hasHamiltonian := 
proc(sys:DOM_LIST, q:DOM_LIST, p:DOM_LIST):DOM_LIST
local i:DOM_INT, j:DOM_INT, n:DOM_INT, ic:DOM_LIST;
begin 
        n := nops(q);
	if testargs() then
            if args(0)<>3 then error("wrong number of arguments") end_if;
            if nops(p)<>n or nops(sys)<>2*n then
                error("incompatible dimensions");
            end_if;
        end_if;
	ic := 
          [ (diff(sys[i],p[j])-diff(sys[j],p[i]) $ j=i+1..n) $ i=1..n-1 ] .
          [ (diff(sys[n+i],q[j])-diff(sys[n+j],q[i]) $ j=i+1..n) $ i=1..n-1 ] .
          [ (diff(sys[i],q[j])+diff(sys[n+j],p[i]) $ j=1..n) $ i=1..n ];
        select(map(ic,simplify),_not@iszero); 
end_proc:

