/*++
hasPotential checks whether a given system of ordinary differential
equations possesses a potential. It returns a list of integrability
conditions which must be satisfied, if the system is to have a potential.  If
the list is empty, the system has always a potential.

Usage: hasPotential(vf,vars);
        vf is the vector field defining the system of ODEs given as a list
        of components; vars is a list of variables.


Author: Werner M. Seiler, Uni Karlsruhe/Mannheim
	(wms@ira.uka.de, seiler@euler.math.uni-mannheim.de)
Created: November 16th, 1997
Last update: 	Mon Mar  1 20:23:43 1999
++*/


detools::hasPotential := 
proc(sys:DOM_LIST,x:DOM_LIST):DOM_LIST
local i:DOM_INT,j:DOM_INT,n:DOM_INT,ic:DOM_LIST;
begin 
        if testargs() then
            if args(0)<>2 then error("wrong no of args") end_if;
            if nops(sys)<>nops(x) then
                error("incompatible dimensions");
            end_if;
        end_if;
        n := nops(x);
	ic := 
          [ (diff(sys[i],x[j])-diff(sys[j],x[i]) $ j=i+1..n) $ i=1..n-1 ];
        select(map(ic,simplify), _not@iszero);
end_proc:
