/*++
pdesolve(sys, <options>)  ---  master routine for solving PDEs

Currently not many methods have been implemented, but pdesolve has been
written in such a way that it can be easily extended.

With the option PolySol one can determine polynomial solutions.

Author: Werner M. Seiler, Uni Karlsruhe/Mannheim
	(wms@ira.uka.de, seiler@euler.math.uni-mannheim.de)
Created: Feb 22nd, 1999
Last update: 	Fri Apr 14 11:54:29 2000
++*/


alias(CANNOT_SOLVE=return(hold(detools::pdesolve)(args())),
      DAT(x)=detools::data[x],
      CDAT(x)=contains(detools::data,x),
      SDAT(x,y)=sysassign(detools::data[x],y),
      TYPE_ERR=error("wrong type of argument")):


detools::pdesolve :=
proc()
begin
    detools::getdata(testargs(),args());
    
    // Polynomial solutions
    
    if CDAT("PolySol") then
	if CDAT("Linear") and 			// linear constant coefficients
	   DAT("Linear") and CDAT("ConstCoeff") and DAT("ConstCoeff") 
        then
	    if domtype(DAT("PolySol"))<>DOM_LIST then
		SDAT("PolySol",[DAT("PolySol"),"a"]);
	    end_if;
	    if testargs() and
	       (not(testtype(DAT("PolySol")[1],Type::PosInt)) or
	        domtype(DAT("PolySol")[2])<>DOM_STRING) 
	    then TYPE_ERR end_if;
	    return((DAT("DFin"))::polysolve(DAT("Eqs"),
					    op(DAT("PolySol"))));
	else CANNOT_SOLVE end_if;		// general systems	
    end_if;
    
    // Series solutions
    if CDAT("Series") then CANNOT_SOLVE end_if;
    
    // Exact methods
    
    if CDAT("Linear") and DAT("Linear") then	// linear equations
	if CDAT("System") and DAT("System") then	// system
	    CANNOT_SOLVE;
	else						// single equation
	    SDAT("Order",(DAT("DFin"))::order(DAT("Eqs")[1]));
	    if DAT("Order")=1 and nops(DAT("Parms"))>1 then
		// try method of characteristics
		return(detools::charSolve(DAT("Eqs")[1],
					  op(DAT("Parms"))));
	    end_if;
	    CANNOT_SOLVE;
	end_if;
    else					// non-linear equations
	CANNOT_SOLVE;
    end_if;
end_proc:


unalias(CANNOT_SOLVE,DAT,CDAT,SDAT,TYPE_ERR):
