/*++
This file collects some combinatorial methods centering around the Hilbert
polynomial and the Cartan characters.

//detools::modStirling(q,n,k)  - computes the modified Stirling number
//                               sigma_k^{(n)}(q)
// (moved to combinat::modStirling)
detools::hilbert(alpha,r)    - computes the Hilbert polynomial H(q+r) as a
				polynomial in r given the Cartan characters
				of the q-th order symbol
detools::cartan(n,h)         - computes the Cartan characters for a given
				Hilbert polynomial (for a symbol in n
				independent variables)
detools::cartan(n,m,q,beta)  - computes Cartan characters from indices
				for symbol in n independent, m dependent
				variables and in order q
detools::arbFuns(q,alpha) - computes list with k-th entry denoting the
				number of arbitrary functions of k arguments
				appearing in the general solution of the
				q-th order differential equations with Cartan
				characters alpha
				

Author: Werner M. Seiler, Uni Karlsruhe/Mannheim
        (wms@ira.uka.de, seiler@euler.math.uni-mannheim.de)
Created: 	Fri Nov  5 19:12:16 1999
Last update: 	Tue Jan  4 10:35:49 2000
++*/

//(moved to combinat::modStirling)
//detools::modStirling :=
//    proc(q:DOM_INT,n:DOM_INT,k:DOM_INT):DOM_INT
//    option remember;
//    begin
//	if testargs() and ((k>n) or (n<1)) then
//	    error("illegal arguments");
//	end_if;
//	case k
//	of 0 do return(1);
//	of 1 do return((n^2+(2*q+1)*n)/2);
//	of n do return(_mult( i $ i=(q+1)..(q+n)));
//	otherwise
//	    return(detools::modStirling(q,n-1,k)+
//		   (q+n)*detools::modStirling(q,n-1,k-1));
//	end_case;
//    end_proc:


detools::hilbert :=
    proc(alpha:DOM_LIST,r:DOM_IDENT):DOM_POLY
    local n:DOM_INT, i:DOM_INT, k:DOM_INT;
    begin
	n := nops(alpha)-1;
	poly([ [_plus(alpha[k+1]*combinat::modStirling(0,k,k-i)/fact(k)
	              $ k=i..n),i] $ i=0..n ], [r], Dom::Rational);
    end_proc:


detools::cartan :=
    proc(n:DOM_INT):DOM_LIST
    local alpha:DOM_LIST, h:DOM_POLY, k:DOM_INT, i:DOM_INT, j:DOM_INT,
	  m:DOM_INT, q:DOM_INT, beta:DOM_LIST;
    begin
	if args(0)=2 then
	    h := args(2);
	    alpha := [ 0 $ n ];
	    for k from n downto 1 do
		alpha[k] := fact(k-1)*coeff(h,k-1) - 
				_plus(combinat::modStirling(0,i-1,i-k) *
				      alpha[i] / _mult( j $ j=k..(i-1) )
				      $ i=(k+1)..n);
	    end_for;
	elif args(0)=4 then
	    m := args(2);
	    q := args(3)-1;
	    beta := args(4);
	    alpha := [ m*binomial(q+n-k,q)-beta[k] $ k=1..n ];
	else error("wrong number of arguments") end_if;
	alpha;
    end_proc:


detools::arbFuns :=
    proc(q:DOM_INT,alpha:DOM_LIST):DOM_LIST
    local res:DOM_LST, n:DOM_INT, k:DOM_INT, i:DOM_INT, kf:DOM_INT;
    begin
	n := nops(alpha);
	res := [ alpha[k] $ k=1..n ];
	kf := fact(n);
	for k from n downto 1 do
	    kf := kf/k;
	    res[k] := res[k]+
		kf*_plus((alpha[i]*combinat::modStirling(0,i-1,i-k)-
		          res[i]*combinat::modStirling(q,i-1,i-k))/fact(i-1)
			  $ i=(k+1)..n);
	end_for;
	res;
    end_proc:
