// $Date: 1999/06/21 11:14:00 $ $Author: stefanw $ $Revision: 1.2 $

/* domains::discreteLog(one, mult, inv, equal, pow)

   given a group by its operations, this function returns
   a method that computes the discrete logarithm of a group element to
   a given base.
   The method returns FAIL if the group element has finite order, but is
   not a power of the given base.
   The method never terminates if the group element has
   infinite order and is not in the subgroup generated by the given base.

*/


// Source: BUchmann, Jacobson, Teske: Computational problems in finite
//  abelian Groups, Math. Comp. 66 (220), 1997, p. 1673

domains::discreteLog:=
proc(one, mult=_mult, inv=_invert, equal=bool@_equal, pow=_power)
  option escape;
  
begin

  /* return value */

  // a procedure taking a group element d, a base g, and an initial step
  //  width v
  
proc(d, g, v=2:Type::Even)
local
  h, a, b, c,f,    // : group elements
  y: DOM_INT,
  u: DOM_INT,
  s: DOM_INT,
  r: DOM_INT,
  R: DOM_TABLE;
  
begin
  // t=2, x=0 :not necessary
  s:=1; y:=u:=v; 
  h:=inv(g); 
  a:=one; b:=pow(g,v); c:=b; 
  R:=table();
  f:=inv(d);
  while TRUE do 
    for r from s to u do   // baby steps 
      a:=mult(a,h); 
      if s=1 then 
        if equal(a,f) then
          // x=r; t=1
          return(r) // break while
        else
          if equal(a, one) then
            // x=r; t=0
            return(FAIL)  // difference to Buchmann: does not return the order 
          else  
            R[a]:=r
          end_if
        end_if
      else
        R[a]:=r
      end_if
    end_for;
    while y< u^2 do   // giant steps
      R[mult(f,b)];
      if type(%)=DOM_INT then
        return(y+%) // t=1
      else
        R[b];
        if type(%)=DOM_INT then
          return(FAIL)
        else
          y:=y+u;
          b:=mult(b,c)
        end_if;
      end_if
    end_while;
    s:=u+1; u:=2*u;
    c:=mult(c,c)
  end_while
end_proc
          
              
  
  /* end of return value */

end_proc:
