// $Date: 1999/06/21 11:14:00 $ $Author: stefanw $ $Revision: 1.3 $

/* domains::grouporder(one, mult, inv, equal, pow)

   given a group by its operations, this function returns
   a method that computes the order of a group element 
   The method never terminates if the group element has
   infinite order.

*/


// Source: BUchmann, Jacobson, Teske: Computational problems in finite
//  abelian Groups, Math. Comp. 66 (220), 1997, p. 1667

domains::grouporder:=
proc(one, mult=_mult, inv=_invert, equal=bool@_equal, pow=_power)
  option escape;
  
begin

  /* return value */

  // a procedure taking a group element
  
proc(g, C=0:DOM_INT, v=2:DOM_INT)
local
  h, a, b, c,    // : group elements
  y: DOM_INT,
  u: DOM_INT,
  s: DOM_INT,
  r: DOM_INT,
  R: DOM_TABLE;
  
begin
  // unused: x:=0; 
  s:=1; y:=u:=v; 
  h:=inv(g); 
  a:=pow(h,C);b:=pow(g,v); c:=b; 
  R:=table(); 
  while TRUE do 
    for r from s to u do 
      a:=mult(a,h); 
      if s=1 then 
        if equal(a,one) then
          // x=r+C
          return(r+C)
          // break while
        else
          R[a]:=r
        end_if
      else
        R[a]:=r
      end_if
    end_for;
    while y< u^2 do
      R[b];
      if type(%)=DOM_INT then
        return(y+%+C)
      else
        y:=y+u;
        b:=mult(b,c)
      end_if
    end_while;
    s:=u+1; u:=2*u;
    c:=mult(c,c)
  end_while
end_proc
          
              
  
  /* end of return value */

end_proc:
