// $Date: 2000/05/21 14:53:45 $ $Author: bij $ $Revision: 1.10 $ 
/*++
Dom::ArithmeticalExpression -- the domain of arithmetical expressions

The domain of arithmetical expressions built up by the internal
operators '+', '*', etc. This domain has almost no algebraic
structure because expressions have no normal form (rational expressions
for example are not normalized to zero).

Use Dom::ExpressionField if you can supply a normalizing function
for a class of expressions which makes that class a field.

Entries:-
zero             - the element 0
one              - the element 1

Methods:-
iszero(e)        - returns TRUE iff e is zero
_plus(x1,x2,...) - returns the sum of the xi
_negate(x)       - returns -x
_subtract(x,y)   - returns x-y
_mult(x1,x2,...) - returns the product of the xi
_invert(x)       - returns 1/x
intmult(x,i)     - returns i times x
_power(x,y)      - returns x^y
_divide(x,y)     - returns x/y
max(x1,x2,...)   - the maximum of x1,x2,...
min(x1,x2,...)   - the minimum of x1,x2,...
norm(x)          - the absolute value of x

convert(e)       - convert the expression e into an Dom::ArithmeticalExpression,
		   ie. e itself
++*/

domain Dom::ArithmeticalExpression
  inherits Dom::Expression;
  category Cat::BaseCategory;
    axiom Ax::systemRep;

      zero:= 0;
      
      iszero:= iszero;
      
      one:= 1;

      _plus:= _plus;
      
      _negate:= _negate;
      
      _subtract:= proc(x,y) begin x - y end_proc;

      _mult:= _mult;

      _invert:= _invert;

      intmult:= _mult;

      _power:= _power;

      _divide:= _divide;

      D:= D;

      diff:= diff;

      max:= max;

      min:= min;

      norm:= abs;

      convert:=
      proc(e)
        local conv, result;
      begin
        conv:= proc(u)
                 begin
                   if type(u)="D" then
                     return(u)
                   end_if;
                 domtype(u);
                 if contains({DOM_INT, DOM_RAT, DOM_FLOAT, DOM_COMPLEX,
                              DOM_IDENT}, %) then
                   u
                 elif % = DOM_EXPR then
                   if contains(map({ op(u) }, conv), FAIL) then
                     FAIL
                   else
                     u
                   end_if
                 else
                   FAIL
                 end_if
               end_proc;
        result:=conv(expr(e));
        delete conv;
        result
    end_proc;

begin
  NIL
end_domain:

