/*++
DiffPoly.mu

Author: Marcus Hausdorf, Uni Karlsruhe
        (hausdorf@ira.uka.de)
Created: 	Mon Mar 29 10:17:24 1999
Last update: 	Thu Apr 20 17:46:34 2000
++*/
// 13.11.2000

/*
Dom::DifferentialPolynomials is the domain of polynomials with the variables 
being the dependent variables and derivatives from some Dom::Differential-
Variable and coefficients from a ring R. This must be at least in Cat::Ring, 
although it will usually be a Cat::PartialDifferentialRing. If elements of DV 
are to appear in R, R must be in Cat::DifferentialFunction(RDV), where RDV is some domain of restricted differential variables.
An element of Dom::DifferentialPolynomial(DV, R) has three slots:

    - a polynomial representing the data
    - the set of DVs appearing in the polynomial
    - the highest occurring order of a derivative
    
This domain is an element of Cat::PolyCat and of Cat::DifferentialFunction; 
it provides implementations for all the necessary entries.
*/

alias(REP(x) = extop(x, 1),
      VARS(x) = extop(x, 2),
      DEG(x) = extop(x, 3),
      VARSP(x) = op(extop(x, 1), 2),
      A_ERR = error("wrong number of arguments"),
      TYPE_ERR = error("wrong type of argument")):
      
domain Dom::DifferentialPolynomial()
local ConstCoeff:DOM_BOOL, RDV:DOM_DOMAIN, startDV:DOM_LIST, indVars:DOM_LIST, 
    depVars:DOM_LIST, nIV:DOM_INT, DVcoerce:DOM_PROC, DVorder:DOM_PROC, DVexpr:
    DOM_PROC, DVderivatives:DOM_PROC, DVvartype:DOM_PROC, DVconvto:DOM_PROC, 
    DVdimq:DOM_PROC, DVdim:DOM_PROC, DVgreater:DOM_PROC, DVindex:DOM_PROC, 
    DVrepeatedIndex:DOM_PROC, DVnewRepeated:DOM_PROC, DVtotalDiff:DOM_PROC, 
    RDVtest:DOM_PROC, RDVmode:DOM_PROC, RDVconvDV:DOM_PROC, RDVnewRepeated:
    DOM_PROC, RDVconvtoDV:DOM_PROC, Rcoerce:DOM_PROC, Rexpr:DOM_PROC, Riszero:
    DOM_BOOL, Rone:R, Rdiff:DOM_PROC, Rintmult:DOM_PROC, Rindets:DOM_PROC, 
    Rsubs:DOM_PROC, Rmult:DOM_PROC, Rpower:DOM_PROC, Rnormal:DOM_PROC, 
    Rsimplify:DOM_PROC, Rinvert:DOM_PROC, Rzero:R, RarbFunction:DOM_PROC, 
    Rnegate:DOM_PROC, diffP:DOM_PROC, DV, R, opts;
inherits Dom::BaseDomain;
category Cat::Polynomial(R), Cat::DifferentialFunction(DV);
//axiom (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if),
//    (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if);

//================
// Input / Output 
//================

subsDV := proc(de)
    local tmp;
    begin
	tmp := DVcoerce(de);
	if tmp <> FAIL then return(tmp) end_if;
	if domtype(de) = DOM_EXPR then 
	    return(map(de, dom::subsDV));
	else 
	    return(de); 
	end_if;
    end_proc;

convert := proc()
    local x, v_l, v2_l:DOM_LIST, vi_l:DOM_LIST, x1_l:DOM_LIST, x2_l:DOM_LIST, 
	tmp, ord:DOM_INT, k:DOM_INT, DOM:DOM_DOMAIN, ord2:DOM_INT, T:DOM_DOMAIN,
	cv:DOM_INT, m, n, nv:DOM_INT, dv, s_l:DOM_LIST, cres_l:DOM_LIST, 
	tres_l:DOM_LIST;
    begin	
	DOM := dom;
	if args(0) = 2 then
	    x := args(1); v_l := args(2);
	    if (domtype(x) <> DOM_LIST 
		or domtype(v_l) <> DOM_LIST)
	    then
		return(FAIL);
	    end_if;
	    v_l := map(v_l, DVcoerce);
	    if (not iszero(contains(v_l, FAIL)) 
		or not iszero(contains(map(v_l, not RDVtest), FALSE))
		or {op(map(x, nops))} <> {2} 
		or not iszero(contains((x1_l := 
		    map(map(x, op, 1), Rcoerce)), FAIL))
		or {op(map((x2_l := map(x, op, 2)), domtype))} <> {DOM_LIST} 
		or {op(map(x2_l, nops))} <> {nops(v_l)}
		or {op(map((tmp := map(x2_l, op)), domtype))} <> {DOM_INT}
		or {op(map(tmp, (l -> bool(l < 0))))} <> {FALSE})
	    then
		return(FAIL);
	    end_if;
	    ord := max(op(map(v_l, DVorder)));
	    v2_l := dom::getDVvars(ord);
	    vi_l := map(v_l, (l -> contains(v2_l, l)));
	    tmp := [0 $ nops(v2_l)];
	    ord2 := 0;
	    for k from nops(v_l) downto 1 do
		_plus(x2_l[i][k] $ i = 1..nops(x2_l));
		if % = 0 then
		    delete v_l[k];
		end_if;
	    end_for;
	    if (ord2 := max(op(map(v_l, DVorder)))) < ord then
		ord := ord2;
		v2_l := dom::getDVvars(ord);
	    end_if;
	    x2_l := map(x2_l, (l -> subsop(tmp,
		op(zip(vi_l, l, ((l1, l2) -> (l1 = l2)))))));
	    x := zip(x1_l, x2_l, [id]);
	    return(new(dom, poly(x, v2_l, R), {op(v_l)}, ord));
	else
	    x := args(1); 
	    case (T := domtype(x)) 
	    of dom do 
		return(x);
	    of DOM_INT do
	    of DOM_RAT do
	    of DOM_FLOAT do
	    of DOM_COMPLEX do
		if (x := Rcoerce(x)) <> FAIL then
		    return(new(dom, poly(x, startDV, R), {}, 0));
		end_if;	    
	    of DOM_IDENT do
	    of DOM_EXPR do
		v_l := {};
		x := dom::subsDV(expand(x));
		case type(x) 
		of "_plus" do
		    s_l := [map(op(x), l -> (if type(l) = "_mult" then [op(l)] 
		                             else [l] end_if))];
		    break;
		of "_mult" do
		    s_l := [[op(x)]];
		    break;
		otherwise
		    s_l := [[x]];
		end_case;
		for m from 1 to nops(s_l) do
		    tres_l := [];
		    cres_l := [];
		    for n from 1 to nops(s_l[m]) do
			if (type(s_l[m][n]) = "_power" 
			    and domtype((tmp := op(s_l[m][n], 1))) = DV
			    and not RDVtest(tmp))
			then
			    tres_l := append(tres_l, 
			                 hold(_power)(tmp, op(s_l[m][n], 2)));
			    v_l := v_l union {tmp}; 
			elif (domtype((tmp := s_l[m][n])) = DV
			    and not RDVtest(tmp))
			then
			    tres_l := append(tres_l, tmp);
			    v_l := v_l union {tmp};
			else
			    if ConstCoeff then
				if contains(map(indets(tmp), domtype), DV) then
				    return(FAIL);
				end_if;
			    end_if;
			    if (tmp := Rcoerce(tmp)) = FAIL then
				return(FAIL);
			    end_if;
			    cres_l := append(cres_l, tmp);
			end_if;
		    end_for;
		    tmp := [Rone, 1];
		    if cres_l <> [] then
			tmp[1] := Rmult(op(cres_l));
		    end_if;
		    if tres_l <> [] then
			if nops(tres_l) = 1 then
			    tmp[2] := op(tres_l);
			else
			    tmp[2] := hold(_mult)(op(tres_l));
			end_if;
		    end_if;
		    s_l[m] := tmp;
		end_for;
		if v_l = {} then
		    ord := 0;
		else
		    ord := max(op(map(v_l, DVorder)));
		end_if;
		tmp := dom::getDVvars(ord);
		return(new(dom, _plus(op(map(s_l, l -> multcoeffs(poly(l[2], tmp, R), l[1])))), v_l, ord));
		break;
	    of DV do
		if not RDVtest(x) then
		    tmp := DVorder(x);
		    return(new(dom, poly(x, dom::getDVvars(tmp), R), {x}, tmp));
		end_if;
		x := Rcoerce(x)
	    of R do
		if ConstCoeff then
		    if contains(map(indets(x), domtype), DV) then
			return(FAIL);
		    end_if;
		end_if;
		return(new(dom, poly(x, startDV, R), {}, 0));
	    of DOM_POLY do return(dom::convert(expr(x)));
	    of DOM_LIST do
		if ({op(map(x, domtype))} <> {DOM_LIST}
		    or {op(map(x, nops))} <> {2})
		then
		    return(FAIL);
		else
		    return(dom::convert(hold(_plus)
			(op(map(x, (l -> hold(_mult)(l[1], l[2])))))));
		end_if;
	    otherwise
		if (T::hasProp <> FAIL 
		    and T::hasProp(Cat::DifferentialFunction(DV))) 
		then
		    case T::constructor
		    of Dom::LinearDifferentialFunction do
			if T::iszero(x) then
			    return(dom::zero);
			elif ((x2_l := T::terms(x))) = [] then
			    return(new(dom, poly(T::inhomogeneity(x), startDV, 
				R), {}, 0));
			end_if;
			if T::coeffRing = R then
			    x1_l := T::coeff(x);
			    v_l := [0 $ nops(x2_l)];
			    tmp := [[x1_l[i], subsop(v_l, i = 1)] 
			        $ i = 1..nops(v_l)];
			    if nops(x1_l) <> nops(x2_l) then
			        tmp := tmp . [[x1_l[nops(x1_l)], v_l]];
			    end_if;
			    return(dom::convert(tmp, x2_l));
			else
			    return(dom::convert(T::expr(x)));
			end_if;
		    of Dom::DifferentialExpression do
			return(dom::convert(extop(x, 1)));
		    of Dom::DifferentialPolynomial do
			if (T::Variables <> DV
			    or RDVmode[1] <> "Less"
			    or (k := T::varSepOrder) = FAIL
			    or k > RDVmode[2])
			then
			    return(dom::convert(T::expr(x)));
			end_if;
			if k = RDVmode[2] then
			    x1_l := map(dom::poly2list(x), 
				l -> [Rcoerce(l[1]), l[2]]);
			    if not iszero(contains(map(x1_l, op, 1), FAIL)) then
				return(FAIL);
			    else
				return(new(dom, poly(x1_l, VARSP(x), R), 
				    VARS(x), DEG(x)));
			    end_if;
			else
			    x1_l := dom::poly2list(x);
			    if DEG(x) < RDVmode[2] then
				tmp := Rcoerce(hold(_plus)(op(
				    zip(map(map(x1_l, op, 1), T::coeffRing::expr),
				    map(map(x1_l, op, 2), l ->
				    DOM::list2term(l, VARSP(x))), 
				    hold(_mult)))));
				if tmp = FAIL then
				    return(FAIL);
				else
				    return(new(dom, poly(tmp, startDV, R), 
					{}, 0));
				end_if;
			    else
				cv := _plus(DVdimq(i) 
				    $ i = k..(RDVmode[2] - 1));
				x2_l := [];
				nv := nops(x1_l[1][2]);
				for m in x1_l do
				    tmp := Rcoerce(hold(_mult)(m[1], 
					op(zip([op(VARSP(x), 
					(nv - cv + 1)..nops(m[2]))], 
					[op(m[2], (nv - cv + 1)..nops(m[2]))],
					(l1, l2) -> (if iszero(l2) then 1 
					elif l2 = 1 then l1 
					else hold(_power)(l1, l2) end_if)))));
				    if tmp = FAIL then
					return(FAIL);
				    else
					x2_l := append(x2_l, [tmp, [op(m[2], 
					1..(nops(m[2]) - cv))]]);
				    end_if;
				end_for;
				return(new(dom, poly(x2_l, dom::getDVvars
				    (DEG(x)), R), select(VARS(x), 
				    DVorder >= RDVmode[2]), DEG(x)));
			    end_if;
			end_if;
		    end_case;
		end_if;
		if ((dv := DVcoerce(x)) <> FAIL 
		    and not RDVtest(dv))
		then
		    ord := DVorder(dv);
		    return(new(dom, 
			poly(dv, dom::getDVvars(ord), R), {dv}, ord));
		elif (dv := Rcoerce(x)) <> FAIL then
		    if Riszero(dv) then
			return(dom::zero);
		    else
			return(new(dom, poly(dv, startDV, R), {}, 0));
		    end_if;
		end_if;
		if T::expr <> FAIL then
		    return(dom::convert(T::expr(x)));
		end_if;
	    end_case;
	end_if;
	FAIL;
    end_proc;
    
convert_to := proc(x:dom, T)
    local res:DOM_EXPR, v, tmp, DOM:DOM_DOMAIN;
    begin
	DOM := dom;
	case T
	of dom do
	    return(x);
	of "diff" do
	of "D" do
	    res := dom::convert_to(x, DOM_EXPR);
	    for v in _union(VARS(x), select(dom::coeffVars(x), 
		DVvartype = "Deriv")) do
		res := subs(res, v = DVconvto(v, T));
	    end_for;
	    return(res);
	of DOM_LIST do 
	    return(map(dom::poly2list(x), l -> [l[1], DOM::list2term(l[2], 
		VARSP(x))]));
	of DOM_EXPR do
	    return(_plus(op(map(dom::poly2list(x), l -> 
		_mult(R::convert_to(l[1], DOM_EXPR), DOM::list2term
		(l[2], VARSP(x)))))));
	of DOM_POLY do
	    return(REP(x));
	of R do
	    if (VARS(x) <> {} 
		or dom::nterms(x) > 1)
	    then
		return(FAIL);
	    else
		return(dom::coeff(x));
	    end_if;
	of DV do
	    if (dom::nterms(x) > 1
		or dom::coeff(x) <> 1 
		or nops((v := expr(dom::lterm(x))) > 1)) 
	    then
		return(FAIL);
	    else 
		return(v);
	    end_if;
	otherwise
	    if (T::hasProp <> FAIL
		and T::hasProp(Cat::DifferentialFunction(DV)))
	    then
		case T::constructor
		of Dom::LinearDifferentialFunction do
		    if dom::isLinearFunction(x) then
			tmp := dom::convert_to(x, DOM_LIST);
			if tmp[nops(tmp)][2] = 1 then
			    delete tmp[nops(tmp)];
			    tmp := append(tmp, dom::tcoeff(x))
			end_if;
			return(T::convert(tmp));
		    end_if;
		of Dom::DifferentialExpression do
		    return(T::convert(dom::convert_to(x, DOM_EXPR)));
		otherwise
		    return(FAIL);
		end_case;
	    end_if;
            if domtype(T)=DOM_DOMAIN and T::hasProp <> FAIL then
            	T::convert(REP(x));
            else return(FAIL) end_if;
	end_case;
	FAIL;
    end_proc;

expr := proc(x:dom)
    local r_l:DOM_LIST, c_l:DOM_LIST, e_l:DOM_LIST, k:DOM_INT, v_l:DOM_LIST,
	DOM:DOM_DOMAIN;
    begin
	if dom::iszero(x) then
	    return(0);
	end_if;
	DOM := dom;
	v_l := map(VARSP(x), DVexpr);
	r_l := map(dom::poly2list(x), l -> [Rexpr(l[1]), 
	    DOM::list2term(l[2], v_l)]); 
	c_l := map(r_l, op, 1);
	e_l := map(r_l, op, 2);
	for k from 1 to nops(r_l) do
	    if c_l[k] = 1 then
		if e_l[k] = 1 then
		    r_l[k] := 1
		else 
		    r_l[k] := e_l[k];
		end_if;
	    else
		if e_l[k] = 1 then
		    r_l[k] := c_l[k];
		else
		    r_l[k] := hold(_mult)(c_l[k], e_l[k]);
		end_if;
	    end_if;
	end_for;
	if nops(r_l) = 1 then
	    r_l[1];
	else
	    hold(_plus)(op(r_l));
	end_if;
    end_proc; 
    
arbFunction := if RarbFunction <> FAIL then 
    proc(f, v_l)
    local k;
    begin
	if not ConstCoeff then
	    for k from 1 to nops(v_l) do
		if domtype(v_l[k]) = DV then
		    v_l[k] := RDVconvDV(v_l[k]);
		end_if;
	    end_for;
	end_if;
	dom::multcoeffs(dom::one, RarbFunction(f, v_l));
    end_proc;
else
    FAIL;
end_if;
	    
//=============
// Arithmetics 
//=============

zero := new(dom, poly(0, startDV, R), {}, 0);

one := new(dom, poly(1, startDV, R), {}, 0);

iszero := (x -> iszero(REP(x))); 

_plus := proc():dom
    local s:DOM_LIST, ord:DOM_INT, k:DOM_INT, v_l:DOM_SET;
    begin
	s := map([args()], dom::coerce);
	if testargs() then
	    if not iszero(contains(s, FAIL)) then
		TYPE_ERR;
	    end_if;
	end_if;
	case args(0)
	of 0 do
	    return(dom::zero);
	of 1 do
	    return(s[1]);
	otherwise 
	    ord := max(DEG(s[i]) $ i = 1..nops(s));
	    v_l := _union(VARS(s[i]) $ i = 1..nops(s));
	    for k from 1 to nops(s) do
		if DEG(s[k]) < ord then
		    s[k] := poly(dom::addZterms(dom::poly2list(s[k]), 
			DEG(s[k]), ord), dom::getDVvars(ord), R);
		else
		    s[k] := REP(s[k]);
		end_if;
	    end_for;
	    return(new(dom, _plus(op(s)), v_l, ord)); 
        end_case;
    end_proc;
    
_negate := (x -> new(dom, _negate(REP(x)), VARS(x), DEG(x)));
    
_mult := proc():dom
    local s:DOM_LIST, tmp, r_l:DOM_LIST, p_l:DOM_LIST, ord:DOM_INT, 
	k, pres:dom, rres:R, f, DOM:DOM_DOMAIN, f_l:DOM_LIST;
    begin
	DOM := dom;
	if args(0) = 0 then
	    return(dom::zero);
	elif args(0) = 1 then
	    return(dom::coerce(args(1)));
	else
	    s := [args()];
	    p_l := [];
	    r_l := [];
	    for f in s do
		if domtype(f) = dom then
		    p_l := append(p_l, f);
		elif domtype(f) = R then
		    r_l := append(r_l, f);
		elif (tmp := Rcoerce(f)) <> FAIL then
		    r_l := append(r_l, tmp);
		else
		    if (tmp := dom::coerce(f)) <> FAIL then
			p_l := append(p_l, tmp);
		    else
			TYPE_ERR;
		    end_if;
		end_if;
	    end_for;
	    case nops(p_l)
	    of 0 do
		pres := dom::one;
		break;
	    of 1 do
		pres := p_l[1];
		break
	    otherwise
		f_l := [];
		ord := max(DEG(p_l[i]) $ i = 1..nops(p_l));
		for k in p_l do
		    if dom::iszero(k) then
			return(dom::zero);
		    elif not dom::isone(k) then
			if DEG(k) < ord then
			    f_l := append(f_l, 
				poly(dom::addZterms(dom::poly2list(k), 
				DEG(k), ord), dom::getDVvars(ord), R));
			else
			    f_l := append(f_l, REP(k));
			end_if;
		    end_if;
		end_for;
		if f_l = [] then
		    pres := dom::one;
		else
		    pres := new(dom, _mult(op(f_l)), _union(VARS(p_l[i]) 
			$ i = 1..nops(p_l)), ord);
		end_if;
	    end_case;
	    case nops(r_l)
	    of 0 do
		return(pres);
		break;
	    of 1 do
		rres := r_l[1];
		break;
	    otherwise
		rres := Rmult(op(r_l));
	    end_case;
	    return(dom::multcoeffs(pres, rres));
	end_if;
    end_proc;
    
multcoeffs := proc(x, r):dom
    begin
	if domtype(r) = DOM_INT then
	    return(dom::intmult(x, r));
	end_if;
	x := dom::coerce(x);
	r := Rcoerce(r);
	if testargs() then
	    if (x = FAIL or r = FAIL) then
		TYPE_ERR;
	    end_if;
	end_if;
	if Riszero(r) then
	    return(dom::zero);
	end_if;
	new(dom, multcoeffs(REP(x), r), VARS(x), DEG(x));
    end_proc; 
    
intmult := proc(x, i):dom
    begin
	x := dom::coerce(x);
	if testargs() then
	    if (x = FAIL or domtype(i) <> DOM_INT) then
		TYPE_ERR;
	    end_if;
	end_if;
	if iszero(i) then
	    return(dom::zero);
	end_if;
	dom::mapcoeffs(x, Rintmult, i);
    end_proc;
    
_invert := (() -> FAIL);

_divide := (() -> FAIL);

//====================
// polynomial methods 
//====================

mapcoeffs := ((x, f) -> new(dom, mapcoeffs(REP(x), f), VARS(x), DEG(x)));
	        
coeff := proc(x:dom)
    local mv:DV, d:DOM_INT, x_l:DOM_LIST, pos:DOM_INT, mc:DOM_INT;
    begin
	if args(0) = 1 then
	    return(coeff(REP(x)));
	else
	    if args(0) = 2 then
		mv := dom::mainvar(x);
		d := args(2);
	    else
		mv := args(2);
		d := args(3);
	    end_if;
	    x_l := dom::poly2list(x);
	    pos := contains(VARSP(x), mv);
	    if iszero(pos) then
		error("wrong arguments");
	    end_if;
	    mc := select(x_l, l -> bool(d = l[2][pos]));
	    mc := map(mc, l -> [l[1], subsop(l[2], pos = 0)]);
	    return(new(dom, poly(mc, VARSP(x), R), VARS(x) minus {mv}, DEG(x))); 
	end_if;
    end_proc;	    

lcoeff := (x -> (if args(0) = 1 then lcoeff(REP(x), DegreeOrder)
    else lcoeff(REP(x), args(2..args(0))) end_if));

tcoeff := (x -> (if args(0) = 1 then tcoeff(REP(x), DegreeOrder)
    else tcoeff(REP(x), args(2..args(0))) end_if));

nthcoeff := ((x, n) -> (if args(0) = 2 then nthcoeff(REP(x), n, DegreeOrder)
    else nthcoeff(REP(x), n, args(3..args(0))) end_if));

nterms := (x -> nterms(REP(x)));

lterm := (x -> (if args(0) = 1 then new(dom, 
    lterm(REP(x), DegreeOrder), VARS(x), DEG(x)) else
    new(dom, lterm(REP(x), args(2..args(0))),
    VARS(x), DEG(x)) end_if));

nthterm := ((x, n) -> (if args(0) = 2 then new(dom, 
    nthterm(REP(x), n, DegreeOrder), VARS(x), DEG(x)) else
    new(dom, nthterm(REP(x), n, args(3..args(0)),
    VARS(x), DEG(x))) end_if));

lmonomial := (x -> (if args(0) = 1 then new(dom, 
    lmonomial(REP(x), DegreeOrder), VARS(x), DEG(x)) else
    new(dom, lmonomial(REP(x), args(2..args(0))),
    VARS(x), DEG(x)) end_if));

nthmonomial := ((x, n) -> (if args(0) = 2 then new(dom, 
    nthmonomial(REP(x), n, DegreeOrder), VARS(x), DEG(x)) else
    new(dom, nthmonomial(REP(x), args(2..args(0))),
    VARS(x), DEG(x)) end_if));

mainvar := dom::leader;

degreevec := (x -> (if args(0) = 1 then degreevec(REP(x), DegreeOrder)
    else degreevec(REP(x), args(2..args(0))) end_if));

degree := (x -> (if args(0) = 1 then degree(REP(x))
    else degree(REP(x), args(2..args(0))) end_if));

evalp := proc(x:dom):dom
    local eq;
    begin
	for eq in [args(2..args(0))] do
	    if (eq := R::coerce(eq)) = FAIL then
		return(FAIL);
	    end_if;
	    x := dom::subs(x, eq);
	end_for;
    end_proc;
    
pseudoRemainder := proc(f:dom, g:dom, y:DV):DOM_LIST
    local p:DOM_INT, m:DOM_INT, d:dom, s:DOM_INT, r:dom, q:dom, 
	y2:dom, ry:DOM_INT, lc:dom, y3:dom;
    begin
	p := dom::degree(f, y);
	m := dom::degree(g, y);
	if (iszero(p) or iszero(m) or p < m) then
	    error("cannot do pseudodivision");
	end_if;
	if DEG(f) < DEG(g) then
	    f := new(dom, poly(dom::addZterms(dom::poly2list(f), DEG(f), DEG(g)), VARSP(g),
		R), VARS(f), DEG(g));
	elif DEG(g) < DEG(f) then
	    g := new(dom, poly(dom::addZterms(dom::poly2list(g), DEG(g), DEG(f)), VARSP(f),
		R), VARS(g), DEG(f));
	end_if;
	d := dom::coeff(g, y, m);
	s := 0;
	r := f;
	q := dom::zero;
	y2 := dom::convert(y);
	while (not iszero(r) and (ry := dom::degree(r, y)) >= m) do
	    s := s + 1;
	    lc := dom::coeff(r, y, ry);
	    y3 := y2^(ry - m);
	    r := d * r - lc * g * y3;
	    q := d * q + lc * y3;
	end_while;
	return([r, q, d, s]);
    end_proc;

pseudoReduction := proc(f:dom, g:dom, y:DV)
    local c_l:DOM_LIST, p:DOM_INT, m:DOM_INT, f1:dom, f2:dom, tmp:DOM_LIST;
    begin
	c_l := [];
	p := dom::degree(f, y);
	m := dom::degree(g, y);
	if iszero(p) then
	    return([f, []]);
	elif iszero(m) then
	    return([g, []]);
	else
	    if p < m then
		f2 := f;
		f1 := g;
	    else
		f1 := f;
		f2 := g;
	    end_if;
	end_if;
	repeat
	    tmp := dom::pseudoRemainder(f1, f2, y);
	    f1 := f2;
	    f2 := tmp[1];
	    c_l := append(c_l, tmp[3]);
	until iszero(dom::degree(f2, y)) end_repeat;
	[f2, c_l];
    end_proc;
    
//======================
// methods from DFunCat 
//======================

indets := proc(x:dom):DOM_SET
    local opts:DOM_LIST, res:DOM_SET;
    begin	
	if args(0) > 1 then
	    opts := [args(2..args(0))];
	else 
	    opts := [];
        end_if;
	res := select(VARS(x), l -> not iszero(degree(REP(x), l)));
	if ConstCoeff then
	    if not iszero(contains(opts, hold(DiffVars))) then
		return(res);
	    else
		return(_union(res, map(dom::coeff(x), Rindets, op(opts))));
	    end_if;
	else
	    return(res union (map(_union(map(dom::coeff(x), 
		Rindets, op(opts))), l -> (if domtype(l) = RDV then 
		RDVconvtoDV(l) else l end_if))));
	end_if;
    end_proc;
    
diff := proc(x):dom
    local s:DOM_LIST, k:DOM_INT, v_l:DOM_LIST, tmp, DOM:DOM_DOMAIN, v;
    begin
	if args(0) = 1 then
	    return(x)
	end_if;
	s := [args(2..args(0))];
	x := dom::coerce(x);	    
	for k from 1 to nops(s) do
	    if (v := DVcoerce(s[k])) <> FAIL then
		s[k] := v;
	    end_if;
	end_for;
	if testargs() then
	    if x = FAIL then
	        TYPE_ERR;
	    end_if;
	end_if;
	DOM := dom;
	tmp := split(s, (l -> (domtype(l) = DV and not RDVtest(l)
	    and contains(VARS(x), l)))); 
	if tmp[1] <> [] then
	    v_l := map(tmp[1], l -> contains(VARSP(x), l));
	    x := new(dom, diffP(v_l, REP(x)), VARS(x), DEG(x));
	end_if;
	if tmp[2] <> [] then
	    if dom::iszero(x) then
		return(dom::zero);
	    end_if;
	    if Rdiff = FAIL then
		error("no partial differential ring");
	    end_if;
	    v_l := map(tmp[2], l -> (if domtype(l) = DV then RDVconvDV(l)
		else l end_if));
	    x := dom::mapcoeffs(x, l -> Rdiff(l, op(v_l)))
	end_if;
	x;
    end_proc;
    
totalDiff := proc(sys)
    local sysflag:DOM_BOOL, mu, tmp, res:DOM_LIST, j:DOM_INT, av_l:DOM_LIST, 
	ord:DOM_INT, p_l:DOM_LIST, v_l:DOM_SET, T1:DOM_TABLE, T2:DOM_TABLE, i:
	DOM_INT, c_l:DOM_LIST, v:DV, m:DOM_LIST, k:DOM_INT, e:DOM_LIST, nmi:
	DOM_LIST, nv:DV, pos:DOM_INT, nvars:DOM_INT, d_l:DOM_LIST, bound:
	DOM_INT, n:DOM_INT, p;
    begin
	sysflag := bool(domtype(sys) = DOM_LIST);
	if not sysflag then
	    sys := [sys];
	end_if;
	if domtype(args(2)) = DOM_LIST then
	    mu := args(2);
	elif args(0) > 2 then
	    mu := [args(2..args(0))];
	else 
	    mu := [args(2)];
	end_if;
	sys := map(sys, dom::coerce);
	if testargs() then
	    if not iszero(contains(sys, FAIL)) then
		TYPE_ERR;
	    end_if;
	end_if;
	for i from 1 to nops(mu) do
	    case domtype(mu[i])
	    of DOM_INT do
		if testargs() then
		    if mu[i] < 1 or mu[i] > nIV then
		        TYPE_ERR;
		    end_if;
                end_if;
		break;
	    of DV do
		if testargs() then
                    if DVvartype(mu[i]) <> "Indep" then
		        TYPE_ERR;
                    end_if;
                end_if;
		mu[i] := DVindex(mu[i]);
		break;
	    otherwise
		tmp := DVcoerce(mu[i]);
 		if testargs() then
		    if tmp = FAIL or DVvartype(tmp) <> "Indep" then
			TYPE_ERR;
		    end_if;
		end_if;
		mu[i] := DVindex(tmp);
	    end_case;
	end_for;
	res := [NIL $ nops(sys)];
	bound := max(0, RDVmode[2] - 1);
	for j from 1 to nops(sys) do
	    if dom::iszero(sys[j]) then
		res[j] := sys[j];
		break;
	    end_if;
	    p := sys[j];
	    ord := DEG(p);
	    p_l := dom::poly2list(p);
	    v_l := VARS(p);
	    av_l := [op(VARS(p))];
	    nvars := nops(p_l[1][2]);
	    tmp := map(av_l, (l -> nvars + 1 - contains(VARSP(p), l)));
	    T1 := table(op(zip(tmp, map(av_l, (ll -> 
		[DVrepeatedIndex(ll), DVindex(ll)])), ((l1, l2) -> l1 = l2))));
	    T2 := table(op(zip(av_l, tmp, ((l1, l2) -> l1 = l2))));
	    for n from 1 to nops(mu) do
		i := mu[n];
		p_l := dom::addZterms(p_l, ord, (ord := ord + 1));
		c_l := [];
		av_l := dom::getDVvars(ord);
		nvars := nops(av_l);
		for m in p_l do
		    for k from nvars downto 1 do
			if not iszero(m[2][k]) then
			    e := T1[nvars + 1 - k];
			    nmi := listlib::insert(e[1], i);
			    nv := DVnewRepeated("Deriv", e[2], nmi);
			    if not contains(T2, nv) then
				pos := nvars + 1 - contains(av_l, nv);
				T2[nv] := pos;
				T1[pos] := [nmi, e[2]];
				v_l := v_l union {nv};
			    else
				pos := T2[nv];
			    end_if;
			    c_l := append(c_l, [Rintmult(m[1], m[2][k]), 
				subsop(m[2], k = m[2][k] - 1, 
				(nvars + 1 - pos) = m[2][nvars + 1 - pos] 
				+ 1)]);
			end_if;
		    end_for;
		    if not ConstCoeff then
			d_l := Rindets(m[1], DiffVars);
			if contains(d_l, (v := RDVnewRepeated("Indep", i))) then
			    c_l := append(c_l, [Rdiff(m[1], v), m[2]]);
			end_if;
			d_l := select(d_l, l -> not contains({"Indep", "Const"},
			    DVvartype(l))); 
			for v in d_l do
			    if DVorder(v) < bound then
				c_l := append(c_l, [Rmult(Rdiff(m[1], v), 
				    RDVconvDV(DVtotalDiff(v, i))), m[2]]);
			    else
				nmi := listlib::insert(DVrepeatedIndex(v), i);
				nv := DVnewRepeated("Deriv", DVindex(v), nmi);
				if not contains(T2, nv) then
				    pos := nvars + 1 - contains(av_l, nv);
				    T2[nv] := pos;
				    T1[pos] := [nmi, DVindex(v)];
				    v_l := v_l union {nv};
				else
				    pos := T2[nv];
				end_if;
				c_l := append(c_l, [Rdiff(m[1], v),
				    subsop(m[2], (nvars + 1 - pos) = 
					m[2][nvars + 1 - pos] + 1)]);
			    end_if;
			end_for;
		    end_if;
		end_for;
		if n < nops(mu) then
		    p_l := poly2list(poly(c_l, av_l, R));
		    if nops(av_l) = 1 then 
			p_l := map(p_l, l -> [l[1], [l[2]]]);
		    end_if;
		end_if;
	    end_for;
	    res[j] := new(dom, poly(c_l, av_l, R), v_l, ord);	       
	end_for;
	if sysflag then
	    return(res);
	else
	    return(res[1]);
	end_if;
    end_proc;

subs := proc(x)
    local eq;
    begin
	if args(0) = 0 then
	    A_ERR;
	elif args(0) = 1 then
	    return(x);
	else
	    if (x := dom::coerce(x)) = FAIL then
		TYPE_ERR;
     	    end_if;
	    for eq in [args(2..args(0))] do
		x := dom::subsOne(x, eq);
	    end_for;
	    return(x);
        end_if;
    end_proc;
	
// warning: contains a grave mistake. Do you find it? 
subsOne := proc(x, eq:DOM_EXPR) // only substitution of DVs
    local ll, rr, tmp, p_l:DOM_LIST, ps_l:DOM_LIST, pos:DOM_INT, pos2:DOM_INT,
	res_l:DOM_LIST, k:DOM_INT, t:DOM_INT, ord:DOM_INT, v_l:DOM_LIST, _dummy,
	rr2:dom, d:DOM_INT, c_l:DOM_LIST;
    begin
	if args(0) < 2 then
	    A_ERR;
	end_if;
	if type(eq) <> "_equal" then
	    TYPE_ERR;
	end_if;
	if (ll := DVcoerce(op(eq, 1))) = FAIL then
	    return(dom::coerce(subs(dom::convert_to(x, DOM_EXPR), eq)));
	end_if;
	rr := op(eq, 2);
	if (tmp := DVcoerce(rr)) = FAIL then
	    if (tmp := Rcoerce(rr)) = FAIL then
		if (tmp := dom::coerce(rr)) = FAIL then
		    return(dom::coerce(subs(dom::convert_to(x, DOM_EXPR), ll = rr)));
		end_if;
	    end_if;
	end_if;
	rr := tmp;
	if RDVtest(ll) then
	    ll := RDVconvDV(ll);
	    if not contains(dom::coeffVars(x), ll) then
		return(x);
	    end_if;
	    if (domtype(rr) = R
		or (domtype(rr) = DV
		    and RDVtest(rr)))
	    then
		if domtype(rr) = DV then
		    rr := RDVconvDV(rr);
		end_if;
		return(dom::mapcoeffs(x, (l -> Rsubs(l, ll = rr))));
	    elif domtype(rr) = DV then   // the hard way
		_dummy := genident();
		return(dom::convert(subs(dom::expr(dom::mapcoeffs
		    (x, (l -> Rsubs(l, ll = R::convert(_dummy))))), 
		    _dummy = rr)));
	    else   // the *really* hard way
		_dummy := genident();
		return(dom::convert(subs(dom::expr(dom::mapcoeffs
		    (x, (l -> Rsubs(l, ll = R::convert(_dummy))))), 
		    _dummy = dom::expr(rr))));
	    end_if;
	else
	    if not contains(VARS(x), ll) then
		return(x);
	    end_if;
	    if iszero(degree(REP(x), ll)) then
		return(extsubsop(x, 2 = VARS(x) minus {ll}));
	    end_if;
	    if domtype(rr) = dom then
		p_l := dom::poly2list(x);
		d := dom::degree(x, ll);
		if DEG(x) < DEG(rr) then 
		    p_l := dom::addZterms(p_l, DEG(x), DEG(rr));
		    v_l := VARSP(rr);
		elif DEG(x) > DEG(rr) then
		    v_l := VARSP(x);
		    rr := new(dom, poly(dom::addZterms(dom::poly2list(rr),
			DEG(rr), DEG(x)), v_l, R), VARS(rr), DEG(x));
		else 
		    v_l := VARSP(x);
		end_if;
		pos := contains(v_l, ll);
		res_l := [];
		rr2 := rr;
		for k from 0 to d do
		    c_l := select(p_l, l -> bool(k = l[2][pos]));
		    if k > 1 then
			rr2 := dom::_mult(rr2, rr);
		    end_if;
		    if (not iszero(k) and c_l <> []) then
			ps_l := dom::poly2list(rr2);
			c_l := map(c_l, l -> op(map(ps_l, 
			    l2 -> [Rmult(l2[1], l[1]), 
				zip(l2[2], subsop(l[2], pos = 0), _plus)])));
		    end_if;
		    res_l := res_l . c_l;
		end_for;
		return(new(dom, poly(res_l, v_l, R), _union(VARS(x) minus {ll},
		    VARS(rr)), max(DEG(x), DEG(rr)))); 
	    else
		if (domtype(rr) = DV
		    and not RDVtest(rr))
		then
		    p_l := dom::poly2list(x);
		    ord := DVorder(rr);
		    if DEG(x) < ord then
			p_l := dom::addZterms(p_l, DEG(x), ord);
			v_l := dom::getDVvars(ord);
		    else
			v_l := VARSP(x)
		    end_if;
		    pos := contains(v_l, ll);
		    pos2 := contains(v_l, rr);
		    for k from 1 to nops(p_l) do
			if not iszero((t := p_l[k][2][pos])) then
			    p_l[k] := subsop(p_l[k], [2, pos2] = 
				t + p_l[k][2][pos2], [2, pos] = 0);
			end_if;
		    end_for;
		    return(new(dom, poly(p_l, v_l, R), _union((VARS(x) 
			minus {ll}), {rr}), max(DEG(x), ord)));
		else
		    if domtype(rr) = DV then
			rr := Rcoerce(rr);
		    end_if;
		    p_l := dom::poly2list(x); 
		    pos := contains(VARSP(x), ll);
		    for k from 1 to nops(p_l) do
			if not iszero(p_l[k][2][pos]) then
			    p_l[k] := [Rmult(Rpower(rr, p_l[k][2][pos]), 
				p_l[k][1]), subsop(p_l[k][2], pos = 0)];
			end_if;
		    end_for;
		    return(new(dom, poly(p_l, VARSP(x), R), VARS(x) minus
			{ll}, DEG(x)));
		end_if;
	    end_if;
	end_if;
    end_proc; 
    // you didn't? Well, you couldn't either, because I removed it on Dec 10th! 
    
solve := proc(x, dv)
    local co;
    begin
	x := dom::coerce(x);
	dv := DVcoerce(dv);
	if testargs() then
	    if (x = FAIL or dv = FAIL) then
		TYPE_ERR;
	    end_if;
	end_if;
	if not dom::isLinear(x, dv) then
	    return(FAIL);
	end_if;
	co := coeff(REP(x), dv, 1);
	if domtype(co) <> R then
	    co := coeff(co);
	end_if;
	x := dom::_negate(dom::subs(x, dv = Rzero));
	if co <> Rone then
	    if Rinvert = FAIL then
		return(FAIL);
	    end_if;
	    x := dom::multcoeffs(x, Rinvert(co));
	end_if;
	if not iszero(contains([args()], "Coeff")) then
	    return([x, co]);
	else
	    return(x);
	end_if;
    end_proc;
     	         
//==============================
// Groebner basis and reduction 
//==============================

// warning: not very extensively tested 

gbasis := if not R::hasProp(Cat::Field) then 
	FAIL; 
    else    
	proc(p_l:DOM_LIST):DOM_LIST
	local m:DOM_INT, r_l:DOM_LIST, a_l:DOM_LIST, DOM:DOM_DOMAIN;
	begin
	    DOM := dom;
	    if args(0) > 2 then
		a_l := [args(2..args(0))];
	    else
		a_l := [];
	    end_if;
	    m := max(op(map(p_l, l -> DEG(l))));
	    r_l := groebner::gbasis(
		map(p_l, l -> (
		    if DEG(l) < m then
			return(poly(DOM::addZterms
			    (DOM::poly2list(l), DEG(l), m),
			    DOM::getDVvars(m), R));
		    else
			return(REP(l));
		    end_if)), op(a_l));
	    map(r_l, l -> new(DOM, l, {}, m));
	end_proc;
    end_if;
    
normalf := if not R::hasProp(Cat::Field) then
	FAIL;
    else
	proc(p:dom, r_l:DOM_LIST)
	local a_l:DOM_LIST, m:DOM_INT, pos:DOM_INT, g_flag:DOM_BOOL, DOM:
	    DOM_DOMAIN;
	begin
	    g_flag := FALSE;
	    DOM := dom;
	    if args(0) > 2 then
		a_l := [args(2..args(0))];
		if not iszero((pos := contains(a_l, hold(CanNForm)))) then
		    delete a_l(pos);
		    g_flag := TRUE;
		end_if;
	    else 
		a_l := [];
	    end_if;
	    r_l := [p] . r_l;
	    m := max(op(map(r_l, l -> DEG(l))));
	    r_l := map(r_l, l -> (
		if DEG(l) < m then
		    return(poly(DOM::addZterms
			(DOM::poly2list(l), DEG(l), m),
			DOM::getDVvars(m), R));
		else
		    return(REP(l));
		end_if));
	    p := r_l[1];
	    delete r_l[1];
	    if g_flag then
		r_l := groebner::gbasis(r_l);
	    end_if;
	    new(dom, 
		groebner::normalf(p, r_l, op(a_l)), {}, m);
	end_proc;
    end_if;
	        
//===============
// miscellaneous 
//===============

equal := ((l1, l2) -> iszero(REP(l1 - l2)));

random := proc():dom
    local r:DOM_PROC, rr:DOM_PROC, o:DOM_INT, t:DOM_INT, d:DOM_INT, 
          v_l:DOM_LIST,	v2_l:DOM_LIST, n:DOM_INT, i:DOM_INT;
    begin
	r := random(1..4);
	o := RDVmode[2] + r();
	t := r()*r()+r();
	d := r();
	v_l := dom::getDVvars(o);
	rr := random(1..nops(v_l));
	n := rr();
	v2_l := [ 0 $ n ];
	for i from 1 to n do
	    v2_l[i] := v_l[rr()];
	end_for;
	new(dom, poly(polylib::randpoly(v2_l, R, Terms = t, Degree = d),
	    v_l, R), {op(v2_l)}, o);
    end_proc; 
	

has := proc(x:dom, ex):DOM_BOOL
    begin
	if domtype(ex) = DV then
	    if RDVtest(ex) then
		return(contains(dom::coeffVars, RDVconvDV(ex)));
	    else
		return(contains(VARS(x), ex));
	    end_if;
	else
	    return(has(REP(x), ex));
	end_if;
    end_proc;

simplify := (x -> dom::mapcoeffs(x, Rsimplify));

normal := (x -> dom::mapcoeffs(dom::normalPoly(x), Rnormal));
    
isLinear := proc(x:dom, dv):DOM_BOOL
    local pos:DOM_INT;
    begin
	dv := DVcoerce(dv);
	if testargs() then
	    if dv = FAIL then
		TYPE_ERR;
	    end_if;
	end_if;
	if iszero((pos := contains(VARSP(x), dv))) then
	    return(FALSE);
	end_if;
	x := dom::poly2list(x);
	if _plus(x[i][2][pos] $ i = 1..nops(x)) <> 1 then
	    return(FALSE);
	else
	    return(TRUE);
	end_if;
    end_proc;
    
isLinearFunction := proc(x:dom):DOM_BOOL
    local x_l:DOM_LIST;
    begin
	x_l := dom::poly2list(x);
	bool({1} = 
	    {_plus(x_l[i][2][j] $ i = 1..nops(x_l))
		$ j in map(select(VARS(x), not RDVtest), 
		l -> contains(VARSP(x), l))});
    end_proc;

constCoeff := ConstCoeff;

coeffRing := R;

polynomial := TRUE;

varSepOrder := if RDVmode[1] = "Less" then
    RDVmode[2] else FAIL end_if;
    
factor := (() -> FAIL);

gcd := (() -> FAIL);

length := (x -> length(REP(x)));

//=================
// local functions 
//=================

/* returns all dependent variables and derivatives up to order n. The use of a 
remember table and the recursive definition speed up computation. The result
list is ordered. */
getDVvars := 
    if RDVmode[1] = "Less" then
	proc(n)
	option remember;
	begin
	    if n <= RDVmode[2] then
		return(startDV);
	    else
		return(DVderivatives(n, 1) . dom::getDVvars(n - 1));
	    end_if;
	end_proc;
    else
	error("toBeDefined");
    end_if; 
    
/* remZTerms and addZterms remove resp. insert superfluous resp. necessary zeroes
into the exponent vectors in the list representation of polynomials, if their
order has changed*/
remZterms := proc(x, o1, o2)
    local d;
    begin
	if (o1 <= o2 
	    or o1 <= RDVmode[2])
	then
	    return(x);
	elif o2 < RDVmode[2] then
	    return(dom::remZterms(x, o1, RDVmode[2]));
	end_if;
	if (d := o1 - o2) = 1 then
	    d := DVdimq(o1);
	else
	    d := DVdim(o1) - DVdim(o2);
	end_if;
	map(x, (l -> [l[1], [op(l[2], (d + 1)..nops(x[1][2]))]]));
    end_proc;
    
addZterms := proc(x, o1, o2)
    local d;
    begin
	if (o1 >= o2
	    or o2 <= RDVmode[2])
	then
	    return(x);
	elif o1 < RDVmode[2] then
	    return(dom::addZterms(x, RDVmode[2], o2))
	end_if;
	if (d := o2 - o1) = 1 then
	    d := DVdimq(o2);
	else
	    d := DVdim(o2) - DVdim(o1);
	end_if;
	map(x, (l -> [l[1], [0 $ d] . l[2]]));
    end_proc;
    
// returns all DVs from v_l in the terms of x_l 
termVars := ((x_l, v_l) -> zip(v_l, [_plus(x_l[i][2][j] $ i = 1..nops(x_l))
	    $ j = 1..nops(x_l[1][2])], ((l1, l2) -> 
		(if l2 > 0 then l1 else null() end_if))));

/* takes an exponent vector and a list of DVs of equal length
and returns the corresponding term (as DOM_EXPR)*/
list2term := proc(e_l:DOM_LIST, v_l:DOM_LIST)
    begin
	map(zip(e_l, v_l, [id]), l -> (if iszero(l[1]) then null() 
	    elif l[1] = 1 then l[2] 
	    else hold(_power)(l[2], l[1])
	    end_if));
	if % = [] then
	    return(1);
	elif nops(%) = 1 then
	    return(%[1]);
	else
	    return(hold(_mult)(op(%)));
	end_if;
    end_proc;
	    
normalPoly := proc(x)
    local x_l, v_l, ord2;
    begin
	if (args(0) = 2
	    and args(2) = TRUE) 
	then
	    v_l := VARS(x);
	    if v_l = {} then
		ord2 := 0;
	    else
		ord2 := max(op(map(v_l, DVorder)));
	    end_if;
	else
	    // v_l := dom::termVars(x_l, VARSP(x));
	    v_l := dom::checkVars(x);
	    if v_l = {} then
		ord2 := 0;
	    else
		ord2 := max(op(map(v_l, DVorder)));
	    end_if;
	    // v_l := {op(v_l)};
	end_if;
	if ord2 < DEG(x) then
	    x_l := dom::poly2list(x);
	    return(new(dom, poly(dom::remZterms(x_l, 
		DEG(x), ord2), dom::getDVvars(ord2), R), v_l, ord2));
	elif ord2 > DEG(x) then
	    x_l := dom::poly2list(x);
	    return(new(dom, poly(dom::addZterms(x_l,
		DEG(x), ord2), dom::getDVvars(ord2), R), v_l, ord2));
	else
	    return(extsubsop(x, 2 = v_l));
	end_if;
    end_proc;
    
coeffVars := proc(x)
    begin
	if ConstCoeff then 
	    return({});
	end_if;
	_union(map(dom::coeff(x), Rindets, DiffVars));
    end_proc;

poly2list := (p -> (if nops(VARSP(p)) > 1 then return(poly2list(REP(p)))
    else return(map(poly2list(REP(p)), (l -> [l[1], [l[2]]]))) end_if));
    
checkVars := (x -> select(VARS(x), l -> not iszero(degree(REP(x), l)))); 
    
//================
// body of domain 
//================
	   	    
begin
    opts := select([args()], (a -> bool(type(a) = "_equal")));
    if iszero(nops(opts)) then
	if ((args(0) < 1) or (args(0) > 2)) then 
	    ARG_ERR 
	end_if;
	DV := args(1);
	if (domtype(DV) <> DOM_DOMAIN 
	    or DV::hasProp = FAIL 
	    or not(DV::hasProp(Cat::DifferentialVariable))) 
	then
	    error("first argument must be Cat::DifferentialVariable");
	end_if;
	if args(0) = 1 then 
	    R := Dom::ExpressionField();
	    ConstCoeff := TRUE; 
	else
	    R := args(2);
	    if (domtype(R) <> DOM_DOMAIN 
		or R::hasProp = FAIL 
		or not(R::hasProp(Cat::Rng))) 
	    then
		error("second argument must be a Cat::Rng");
	    end_if;
	    ConstCoeff := not(R::hasProp(Cat::DifferentialFunction));
	    if not(ConstCoeff) then
		RDV := R::Variables;
		if (domtype(RDV) <> DOM_DOMAIN 
		    or RDV::hasProp = FAIL 
		    or RDV::constructor <> Dom::RestrictedDifferentialVariable 
		    or RDV::Variables<>DV) 
		then
		    error("invalid coefficient domain");
		end_if;
	    end_if;
	end_if;
    else
	if nops(opts) <> args(0) then 
	    error("inconsistent arguments") 
	end_if;
	if args(0) > 2 then 
	    ARG_ERR 
	end_if;
	if op(opts[1], 1) <> hold(Vars) then 
	    TYPE_ERR 
	end_if;
	if (args(0) = 2 and op(opts[2], 1) <> hold(Rest)) then 
	    TYPE_ERR 
	end_if;
	opts := map(opts, (a -> if domtype(op(a, 2)) <> DOM_LIST then 
		    subsop(a, 2 = [op(a, 2)]) 
		  else a end_if));
	DV := Dom::DifferentialVariable(op(op(opts[1], 2)));
	if args(0) = 1 then
	    R := Dom::ExpressionField();
	    ConstCoeff := TRUE;
	else
	    RDV := Dom::RestrictedDifferentialVariable(DV, op(op(opts[2], 2)));
	    R := Dom::DifferentialExpression(RDV);
	    ConstCoeff := FALSE;
	end_if;
    end_if;
    indVars := revert(DV::indVariables);
    depVars := revert(DV::depVariables);
    DVderivatives := DV::derivatives;
    DVvartype := DV::vartype;
    if ConstCoeff then
	RDVtest := (l -> bool(DVvartype = "Indep"));
	RDVmode := ["Less", 0];
	RDVconvtoDV := id;
	RDVconvDV := id;
	startDV := depVars;
    else
	RDV := R::Variables;
	if not(RDV::constructor = Dom::RestrictedDifferentialVariable)
	   or not(RDV::Variables = DV)
	then
	    TYPE_ERR;
	end_if;
	RDVmode := RDV::mode;
	RDVconvDV := RDV::convDV;
	RDVconvtoDV := RDV::convtoDV;
	RDVtest := RDV::test;
	RDVnewRepeated := RDV::newRepeated;
	case RDVmode[1]
	of "Indep" do
	    RDVmode := ["Less", 0];
	of "Less" do
	    if iszero(RDVmode[2]) then
		startDV := depVars;
	    else
		startDV := DVderivatives(RDVmode[2], 1);
	    end_if;
	    break;
	of "Greater" do
	    error("toBeDefined");
	of "General" do
	    error("toBeDefined");
	end_case;  
    end_if;
    DVcoerce := DV::coerce;
    DVorder := DV::order;
    DVexpr := DV::expr;
    DVconvto := DV::convert_to;
    DVdimq := DV::dimq;
    DVdim := DV::dim;
    DVgreater := ((l1, l2) -> DV::_less(l2, l1));
    DVindex := DV::index;
    DVrepeatedIndex := DV::repeatedIndex;
    DVnewRepeated := DV::newRepeated;
    DVtotalDiff := DV::totalDiff;
    nIV := DV::numIndVar;
    Rcoerce := R::coerce;
    Rexpr := R::expr;
    Riszero := R::iszero;
    Rone := R::one;
    Rdiff := R::diff;
    Rintmult := R::intmult;
    Rindets := R::indets;
    if Rindets=FAIL then Rindets := indets end_if;
    Rsubs := R::subs;
    Rmult := R::_mult;
    Rpower := R::_power;
    Rnormal := R::normal;
    Rsimplify := R::simplify;
    Rinvert := R::_invert;
    Rzero := R::zero;
    RarbFunction := R::arbFunction;
    Rnegate := R::_negate;
    diffP := polylib::Dpoly;    
end_domain;

unalias(REP, VARS, DEG, VARSP, TYPE_ERR, A_ERR):


