// $Date: 2000/11/27 15:27:30 $ $Author: stefanw $ $Revision: 1.99 $


//   Interval - domain of intervals

//   Interval(a, b) : the open interval ]a,b[.
//   Interval([a] , b) : the semi-open interval [a,b[
//   Interval(a, [b]): the semi-open interval ]a,b] 

//   An interval consists of two operands:
//   the first is a relation of type <= or <
//   the second is a relation of type >= or >

//   The available methods are:
//   _plus, sign, _mult, _negate, _invert, _power, iszero, convert,
//   random, exp, _union, _intersect, and some special functions

// Zugriff auf den Operanden eines Intervalls
alias(IV(x) = extop(x, 1)): // Operand vom Typ 'property::IVnat'
alias(IVx = 1):             // derselbe Index fur 'subsop'

alias(Interval = Dom::Interval):
alias(IVnat = property::IVnat):

domain Interval
  
  inherits  Dom::BaseDomain;
  category  Cat::Set, Cat::AbelianMonoid;
  // in this order!! because otherwise there would be no search
  // beyond "toBeDefined", and the overloading mechanism of Cat::Set
  // would not work
  
// no axioms

// methods

  borders:= x ->[IVnat::left(IV(x)), IVnat::right(IV(x))];
  
  // Grenze links immer offen
  left:= x -> IVnat::left(IV(x));

  // Grenze links, offen oder [geschlossen]
  leftB:= x -> IVnat::leftB(IV(x));

  // Grenze rechts immer offen
  right:= x -> IVnat::right(IV(x));

  // Grenze rechts, offen oder [geschlossen]
  rightB:= x -> IVnat::rightB(IV(x));

  // TRUE/FALSE offen/geschlossen
  isleftopen:= x -> IVnat::lopen(IV(x));
  isrightopen:= x -> IVnat::ropen(IV(x));

  // beide Grenzen "float"
  isfloat:= x -> IVnat::isfloat(IV(x));
  
  // nur ein Punkt
  isPoint:= x -> IVnat::isPoint(IV(x));
  
  // nur intern: falls beide Grenzen "float",
  //             dann nur 'bool', sonst 'property::isrel'
  _is:=
    proc(EXPR, FL = FALSE)
    begin
      if FL then
        map(EXPR, float)
      else
        property::isrel(EXPR) = TRUE
      end_if
    end_proc;

  subs:=
    proc(x:dom)
    begin
      dom::new(subs(dom::leftB(x), args(2..args(0))),
               subs(dom::rightB(x), args(2..args(0))))
    end_proc;
  
  subsleft:=
    proc(x:dom, l, o = FAIL)
    begin
      new(dom, IVnat::subsleft(IV(x), l, o))
    end_proc;

  subsright:=
    proc(x:dom, r, o = FAIL)
    begin
      new(dom, IVnat::subsright(IV(x), r, o))
    end_proc;

  // substitute only the borders, but not the open/close information
  subsvals:=
    proc(x:dom, l, r)
    begin
      dom::subsright(dom::subsleft(x, l, FAIL), r, FAIL)
    end_proc;

  // interchange borders(x) - 
  // swaps the borders of x; the left border of the result is open iff
  // the right border of x is open, and the right border of the result is
  // open iff the left border of x is open.
  interchangeBorders:=
    proc(x:dom)
    begin
      extsubsop(x, IVx = IVnat::swap(IV(x)))
    end_proc;

  // expr - converts into a range
  // if two arguments are given, the second must be an identifier
  // in that case, expr returns an expression whose solution is
  // the given interval (this is necessary for the property mechanism)
  expr:=
    proc(iv:dom, x)
    begin
      case args(0)
        of 1 do
          if IVnat::isPoint(IV(iv))=TRUE then
            return({IVnat::left(IV(iv))})
          else
            return(IVnat::left(IV(iv))..IVnat::right(IV(iv)))
          end_if
        of 2 do
          if IVnat::isPoint(IV(iv))=TRUE then
            return(x = IVnat::left(IV(iv)))
          else
            if dom::left(iv) = -infinity then
              return(_if(dom::isrightopen(iv), hold(_less), hold(_leequal))(x, dom::right(iv)))
            elif dom::right(iv) = infinity then
              return(_if(dom::isleftopen(iv), hold(_less), hold(_leequal))(dom::left(iv), x))
            else  
              return(_if(dom::isrightopen(iv), hold(_less), hold(_leequal))(x, dom::right(iv)) and
                     _if(dom::isleftopen(iv), hold(_less), hold(_leequal))(dom::left(iv), x))
            end_if
          end_if
        otherwise
          error("wrong no of arguments")
      end_case
    end_proc;

  // overloading needed for _in

  set2expr:=
    proc(x:dom, u)
    begin
      dom::expr(x, u)
    end_proc;

  set2prop:=
    proc(S:dom)
    begin
      property::IV(dom::leftB(S), dom::rightB(S))
    end_proc;

  print:=
    proc(i:dom)
    begin
      IV(i)
    end_proc;
       
  Content :=
    proc(Out, i : dom)
      local Closure;
    begin
      if i::dom::isleftopen(i) then
        if i::dom::isrightopen(i) then
          Closure := "open"
        else
          Closure := "open-closed"
        end_if
      else
        if i::dom::isrightopen(i) then
          Closure := "closed-open"
        else
          Closure := "closed"
        end_if
      end_if;
      Out::Cinterval({hold(closure)=Closure}, 
                     Out(i::dom::left(i)), 
                     Out(i::dom::right(i))) 
    end_proc;

  equal:=
    proc(x:dom, y)
    begin
      if type(y) <> dom then
        FALSE
      else
        bool(IV(x) = IV(y))
      end_if
    end_proc;
  
  new:=
    proc(l, r)
      local iv;
    begin
      iv:= case args(0)
             of 1 do
               l:= dom::convert(l);
               if l = FAIL then
                 error("Illegal argument")
               else
                 l
               end_if;
               break
             of 2 do
               if map({args()}, domtype) minus
                 {DOM_INT, DOM_RAT, DOM_FLOAT, DOM_IDENT, DOM_EXPR, DOM_LIST,
                  stdlib::Infinity} <> {} then
                 return(FAIL)
               end_if;
               r:= IVnat::new(l, r);
               if r = property::Null then
                 {}
               else
                 new(dom, r)
               end_if;
               break
             otherwise
               FAIL
           end_case;
      if type(iv) = dom and dom::isPoint(iv)=TRUE then
        {IVnat::left(IV(iv))}
      else
        iv
      end_if
    end_proc;

  convert:=
    proc(l)
    begin
      case domtype(l)
        of dom do return(l)
        of Type do
          case type(l)
            of "IVnat" do
              return(new(dom, l))
            of "Rel" do // Rel -> IVnat
              new(dom, property::Rel::IVnat(l));
              break
            of "NRange" do // NRange -> IVnat
              if (l:= contains([Type::Real, Type::Negative,
                                Type::NonNegative, Type::Positive])) > 0 then
                return(dom::new(op([hold(-infinity, infinity),
                                    hold(-infinity, 0),
                                    hold([0], infinity),
                                    hold(0, infinity)], l)))
              else
                return(FAIL)
              end_if
            of "IV" do
              return(new(dom, property::IV::IVnat(l)))
            otherwise
              return(FAIL)
          end_case
        of DOM_INT do
        of DOM_RAT do
        of DOM_FLOAT do
        of DOM_IDENT do
          return(dom::new([l], [l]))
        of DOM_EXPR do
          case type(l)
            of "_range" do
              return(new(dom, IVnat::new(op(l, 1), op(l, 2))))
            of "_or" do
              return(hold(_union)(map(op(l), dom::convert)))
            of "_and" do
              return(hold(_intersect)(map(op(l), dom::convert)))
            otherwise
              return(new(dom, IVnat::new([l], [l])))
          end_case
        of DOM_SET do
          if nops(l) = 1 then
            return(dom::convert(op(l)))
          else
            return(FAIL)
          end_if;
        of DOM_LIST do
          if nops(l) <> 2 then
            return(FAIL)
          else
            return(dom::new([op(l,1)], [op(l,2)]))
          end_if
        of property do
          if l = property::Null then
            return({})
          else
            return(FAIL)
          end_if
        otherwise
          userinfo(1, "Cannot convert object of type '".
                   expr2text(domtype(l))."' to type 'interval'");
          return(FAIL)
      end_case;
      return(FAIL)
    end_proc;


  // emptycheck - returns the input interval if it is nonempty
  //              or the empty set otherwise
  emptycheck:=
    proc(x:dom)
      local l,r;
    begin
      if property::Fail(IV(x)) then
        return({})
      end_if;
      l:= dom::left(x); r:= dom::right(x);
      if l = -infinity and r = infinity then
        return(solvelib::BasicSet(Dom::Real))
      end_if;
      if l = r then
        if dom::isleftopen(x) or dom::isrightopen(x) then
          return({})
        else
          return({l})
        end_if
      elif is(_if(dom::isleftopen(x) or dom::isrightopen(x),
                  _leequal, _less)(r, l)) = TRUE then
        return({})
      end_if;
      return(x)
    end_proc;

  simplify:= x -> dom::emptycheck(x);

  bin_plus:=
    proc(r:dom, s:dom)
    begin
      new(dom, IVnat::_plus(IV(r), IV(s)))
    end_proc;

  mapplus:= (iv, x) -> dom::mapBorders(iv, _plus, x);
  
  inhomog_plus:=
  table(DOM_SET     = proc(iv, S)
                      begin
                        _union(op(map(S, _plus, iv)))
                      end_proc,

        DOM_INT     = dom::mapplus,
        DOM_RAT     = dom::mapplus,
        DOM_FLOAT   = dom::mapplus,

        DOM_COMPLEX = proc(iv, c)
                        local g;
                      begin
                        g:= genident();
                        Dom::ImageSet(c + g, g, iv)
                      end_proc,

        DOM_EXPR    = proc(iv, xpr)
                        local g;
                      begin
                        case type(xpr)
                          of "_union" do
                            return(map(xpr, _plus, iv))
                          of "_intersect" do
                          of "_minus" do
                            return(FAIL)
                        end_case;
                        if is(xpr, Type::Real) = TRUE then
                          Dom::Interval::mapplus(iv, xpr)
                        else
                          g:= genident();
                          Dom::ImageSet(xpr + g, g, iv)
                        end_if
                      end_proc,

        DOM_IDENT   = proc(iv, ident)
                        local g:DOM_IDENT;
                      begin
                        if is(ident, Type::Real)=TRUE then
                          Dom::Interval::mapplus(iv, ident)
                        else
                          g:=genident();
                          Dom::ImageSet(ident+g, g, iv)
                        end_if
                      end_proc
        );
    

  // sign - returns the set of possible signs
  sign:=
    proc(iv)
    begin
      if iv::dom = dom then
        IVnat::sign(IV(iv))
      else
        sign(iv)
      end_if
    end_proc;

  dirac:=
    proc(x:dom)
      local sgn;
    begin
      sgn:= dom::sign(x);
      if type(sgn) = DOM_SET then
        if contains(sgn, 0) then
          {0, dirac(0)}
        else
          {0}
        end_if
      else
        hold(dirac)(x)
      end_if
    end_proc;

  heaviside:=
    proc(x:dom)
      local sgn;
    begin
      sgn:=dom::sign(x);
      if type(sgn) = DOM_SET then
        map(sgn, heaviside)
      else
        hold(heaviside)(x)
      end_if
    end_proc;

  bin_mult:=
    proc(r:dom, s:dom)
      local ret;
    begin
      ret:= IVnat::_mult(IV(r), IV(s));
      if type(ret) = "_or" then
        hold(_union)(op(map(ret, dom::new)))
      else
        if property::Fail(ret) then
          dom::convert(ret)
        else
          dom::new(ret)
        end_if
      end_if
    end_proc;

  mapmult:= 
    proc(iv, x)
    begin
      if x > 0 then
        dom::mapBorders(iv, _mult, x);
      elif x=0 then
        {0}
      else
        dom::interchangeBorders(dom::mapBorders(iv, _mult, x))
      end_if
    end_proc;

  mapmultparam:=
    proc(iv,x)
      local gen;
    begin
      gen:= genident();
      piecewise([x > 0, dom::mapBorders(iv, _mult, x)],
                [x < 0, dom::interchangeBorders(dom::mapBorders(iv, _mult, x))],
                [x=0 , {0}],
                [Im(x) <> 0, Dom::ImageSet(x*gen, gen, iv) ])
    end_proc; 

  inhomog_mult:=
  table(DOM_SET      = proc(iv, S)
                       begin
                         _union(op(map(S, _mult, iv)))
                       end_proc,

        DOM_INT      = dom::mapmult,
        DOM_RAT      = dom::mapmult,
        DOM_FLOAT    = dom::mapmult,

        DOM_COMPLEX  = proc(iv, c)
                         local gen;
                       begin
                         gen:= genident();
                         Dom::ImageSet(c*gen, gen, iv)
                       end_proc,

        DOM_EXPR     = proc(iv, xpr)
                       begin
                         case type(xpr)
                           of "_union" do
                             return(map(xpr, _mult, iv))
                           of "_intersect" do
                           of "_minus" do
                             return(FAIL)
                         end_case;
                         Dom::Interval::mapmultparam(args())
                       end_proc,

        DOM_IDENT    = dom::mapmultparam);

  _negate:=
    proc(r:dom)
    begin
      new(dom, IVnat::_negate(IV(r)))
    end_proc;

  _subtract:=
    proc(r, s)
    begin
      if type(r) = dom and type((s:= dom::convert(s))) = dom then
        new(dom, IVnat::_subtract(IV(r), IV(s)))
      else
        r+_negate(s)
      end_if
    end_proc;
    
  _invert:=
    proc(r)
      local inv;
    begin
      if r = dom::zero then
        error("division by zero")
      else
        inv:= IVnat::_invert(IV(r));
        if type(inv) = "_or" then
          hold(_union)(op(map(inv, dom::new)))
        else
          new(dom, inv)
        end_if
      end_if
    end_proc;

  // _divide: necessary to handle case x/x correctly 
  _divide:= (x, y) -> x*_invert(y);


  bin_power:=
  proc(r:dom, n:dom)
  begin
    exp(n*dom::ln(r))
  end_proc;

  leftpownum:=
  proc(r:dom, n)
  begin
    if n=0 then
      dom::one
    elif n=1 then
      r
    elif n<0 then
      dom::leftpownum(dom::_invert(r), -n)
    else
      new(dom, IVnat::_power(IV(r), n))
    end_if
  end_proc;

  leftpowuneval:=
  proc(r:dom, n)
  begin
    genident();
    Dom::ImageSet(%^n, %, r)
  end_proc;
    
  inhomogleft_power:=
  table(
        DOM_INT= dom::leftpownum,
        DOM_RAT= dom::leftpownum,
        DOM_FLOAT= dom::leftpownum,

        DOM_EXPR = dom::leftpowuneval,
        DOM_IDENT = dom::leftpowuneval,

        DOM_SET =
        proc(r:Dom::Interval, S:DOM_SET)
        begin
          _union(op(map(S, u-> r^u)))
        end_proc

        );


    rightpowbyln:=
    proc(r, n:dom)
    begin
      exp(dom::_mult(n, ln(r)))
    end_proc;
    
    inhomogright_power:=
    table(
          DOM_INT= dom::rightpowbyln,
          DOM_RAT= dom::rightpowbyln,
          DOM_COMPLEX = dom::rightpowbyln,
          DOM_FLOAT= dom::rightpowbyln,
          DOM_EXPR=dom::rightpowbyln,
          DOM_IDENT=dom::rightpowbyln,

          DOM_SET=
          proc(S:DOM_SET, n:Dom::Interval)
          begin
            _union(op(map(S, _power, n)))
          end_proc
          
          );
    
  

  iszero:=
    proc(x)
    begin
      iszero(dom::left(x)) and iszero(dom::right(x))
    end_proc;

  random:=
    proc()
      local a, b, c, d;
    begin
      a:= tan(float(PI)*(random()/(10.0^12) - 0.5));
      // -infinity < a < infinity
      if random() mod 2 = 0 then
        a:= [a]
      end_if;
      b:= tan(float(PI/2)*(random()/(10.0^12)));
      // 0 <= b < infinity
      if random() mod 2 = 0 then
        d:= [a + b]
      else
        d:= a + b
      end_if;
      new(dom, IVnat::new(c, d))
    end_proc;

  one:= dom::new([1], [1]);

  zero:= dom::new([0], [0]);
    
  exp:=
    proc(x:dom)
      local l,r;
    begin
      if dom::left(x)=-infinity then
        l:=0
      else
        l:=exp(dom::left(x))
      end_if;
      if dom::right(x)=infinity then
        r:=infinity
      else
        r:=exp(dom::right(x))
      end_if;
      dom::subsvals(x, l, r)
    end_proc;

  bin_union:=
    proc(a:dom, b:dom)
      local ret;
    begin
      ret:= IVnat::_or(IV(a), IV(b));
      if type(ret) = "_or" then
        hold(_union)(op(map(ret, dom::new)))
      elif property::Fail(ret) then
        dom::convert(ret)
      else
        new(dom, ret)
      end_if
    end_proc;

  inhomog_union:=
    table(DOM_SET   = proc(iv, S)
                        local splitset, outerpts, l, r;
                      begin
                        splitset:= split(S, x->contains(iv, x));
                        if splitset[2] = {} and splitset[3] = {} then
                          iv
                        else
                          // test whether finite set contains border point of interval,
                          // e. g. iv = (-1, 1) and S = {1}
                          outerpts:= splitset[2] union splitset[3];
                          l:= Dom::Interval::left(iv);
                          r:= Dom::Interval::right(iv);
                          if contains(S, l) then
                            outerpts:= outerpts minus {l};
                            l:= [l]
                          end_if;
                          if contains(S, r) then
                            outerpts:= outerpts minus {r};
                            r:= [r]
                          end_if;
                          if outerpts = {} then
                            new(Dom::Interval, IVnat::new(l, r))
                          else
                            hold(_union)(iv, outerpts)
                          end_if
                        end_if;
                      end_proc, 

         piecewise  = piecewise::_union,
          
         DOM_EXPR   = proc(a, b)
                      begin
                        case type(b)
                          of "_intersect" do
                            return(map(b, _union, a))
                          of "_union" do
                            return(map(b, _union, a))
                            // better chance for simplification
                          otherwise
                            FAIL
                        end_case;
          end_proc
          );
 
  bin_intersect:=
    proc(a:dom, b:dom)
      local ret;
    begin
      ret:= IVnat::_and(IV(a), IV(b));
      if type(ret) = "_and" then
        hold(_intersect)(op(map(ret, dom::new)))
      elif property::Fail(ret) then
        dom::convert(ret)
      else
        dom::new(ret)
      end_if
    end_proc;

  inhomog_intersect:=
    table(DOM_SET   =
          proc(iv, S)
            local splitset, expandedset;
          begin
            splitset:= split(S, x -> contains(iv, x));
            if op(splitset, 3) = {} then
              splitset[1]
            else
              // transform any object s which may or may not be
              // in iv into a set that may have one or zero
              // elements depending on the condition s in iv
              expandedset:=map(splitset[3],
                               proc(s)
                                 local cond;
                               begin
                                 cond:= s in iv;
                                 piecewise([cond, {s}],
                                           [not cond, {}])
                               end_proc
                               );
              splitset[1] union _union(op(expandedset))
            end_if
          end_proc,

          piecewise = piecewise::_intersect,

          DOM_EXPR  = proc(a,b)
                        local intersections, c, d;
                      begin
                        case type(b)
                          of "_intersect" do
                            // to have maximal chance
                            // for simplification
                            return(map(b,_intersect, a))
                          of "_union" do
                            intersections:=map([op(b)], _intersect, a);
                            if contains(map(intersections, type),
                                        "_intersect")>0 then
                              return(FAIL)
                            else  
                              return(_union(op(intersections)))
                            end_if
                          of "_minus" do
                            [c, d]:= [op(b)];
                            // apply rule
                            // a intersect (c minus d) =
                            // (a intersect c) minus d =
                            // (a intersect c) minus (a intersect d)
                            // the latter may be simpler if the
                            // intersection of a and d can really be computed
                            intersections:= a intersect c;
                            a intersect d;
                            if type(%)<>"_intersect" then
                              return(intersections minus %)
                            else
                              return(intersections minus d)
                            end_if;
                        end_case;
                        FAIL
                      end_proc);
    
  bin_minus:=
    proc(A:dom, B:dom)
    begin
      // intersect with complement
      (IV(A) and IVnat::new(-infinity, dom::leftB(B)))
      or (IV(A) and IVnat::new(dom::rightB(B), infinity));
      if %=property::Null then
        {}
      else
        dom::new(%)
      end_if
    end_proc;

  // inhomogright_minus : table of methods for any_set minus interval
  inhomogright_minus:=
    table(DOM_SET = proc(A, B:Dom::Interval)
                      local d;
                    begin
                      d:= split(A, y -> Dom::Interval::contains(B, y));
                      if op(d, 3) = {} then
                        return(op(d, 2))
                      else
                        return(hold(_union)(op(d, 2), hold(_minus)(op(d, 3), B)))
                      end_if;
                    end_proc);

  inhomogleft_minus:=
    table(DOM_SET =
          proc(A:Dom::Interval, B)
            local intervals, y;
          begin
            intervals:= [A];
            for y in B do
              intervals:= map(intervals,
                              proc(J)
                              begin
                                if contains(J, y) = FALSE then
                                  J
                                else
                                  new(Dom::Interval, IVnat(Interval::left(J), y)),
                                  new(Dom::Interval, IVnat(y, Interval::right(J)))
                                end_if
                              end_proc)
          end_for;
          return(_union(op(intervals)))
          end_proc);
                      
  // is x an element of the interval X?
  contains :=
    proc(X, x)
    begin
      is(x in X)
    end_proc;

  // is c syntactically contained in iv?
  has:= (iv, c) -> bool(iv=c) or
        has(dom::left(iv), c) or has(dom::right(iv), c);


  // overload solvelib::Union
  // compute the union of all iv(x), where x runs through S
    
  Union:=
    proc(iv, x, S)
    begin
      if not dom::has(iv, x) then
        iv
      else
        // no further simplifications implemented at the moment
        hold(solvelib::Union)(iv, x, S)
      end_if
    end_proc;
  
  expr2set:=
    proc(X, x)
    begin
      expr(_if(dom::isleftopen(X), _less, _leequal)(dom::left(X), x)
           and _if(dom::isrightopen(X), _less, _leequal)(x, dom::right(X)))
    end_proc;

  getElement:=
  proc(iv:dom)
  begin
    if dom::isEmpty(iv)=FALSE then
      (dom::left(iv)+dom::right(iv))/2
    else
      FAIL
    end_if
  end_proc;
    
  isEmpty:=
  proc(iv:dom):DOM_BOOL
  begin
    if is(dom::right(iv) > dom::left(iv))=TRUE
      or dom::left(iv)=dom::right(iv) then
      FALSE
    else
      UNKNOWN
    end_if
  end_proc;


  isFinite:=
  proc(iv:dom):DOM_BOOL
  begin
    not is(dom::right(iv)> dom::left(iv))
  end_proc;
    
  // mapBorders: simply map the function to the borders

  // map: return the image of the interval under the mapping; inherited
  // from Cat::Set
    
  mapBorders:=
    proc(x)
      local lval, rval;
    begin
      [lval, rval]:= map([dom::left(x), dom::right(x)], args(2..args(0)));
      dom::subsvals(x, lval, rval)
    end_proc;

  zip:=
    proc(x, y)
      local lval, rval;
    begin
      [lval, rval]:= zip([dom::left(x), dom::right(x)],
                         [dom::left(y), dom::right(y)], args(3..args(0)));
      new(dom, IVnat::new(_if(dom::isleftopen(x), lval, [lval]),
                          _if(dom::isrightopen(x), rval, [rval])))
    end_proc;

  min:=
    proc()
      local l, r, DOM;
    begin
      l:= map([args()], dom::convert);
      if has(l, FAIL) then
        error("Could not convert arguments to type interval")
      end_if;
      DOM:= dom;
      r:= map(l, X -> [DOM::right(X), _if(DOM::isrightopen(X), 0, 1)]);
      l:= map(l, X -> [DOM::left(X), _if(DOM::isleftopen(X), 0, 1)]);
      r:= op(sort(r, (X, Y) -> op(X, 1) <= op(Y, 1) and op(X, 2) <= op(Y, 2)), 1);
      l:= op(sort(l, (X, Y) -> op(X, 1) <= op(Y, 1) and op(X, 2) >= op(Y, 2)), 1);
      new(dom, IVnat::new(_if(op(l, 2) = 1, DOM_LIST, id)(op(l, 1)),
                          _if(op(r, 2) = 1, DOM_LIST, id)(op(r, 1))))
    end_proc;

  max:=
    proc()
      local l, r, DOM;
    begin
      l:= map([args()], dom::convert);
      if has(l, FAIL) then
        error("Could not convert arguments to type interval")
      end_if;
      DOM:= dom;
      r:= map(l, X -> [DOM::right(X), _if(DOM::isrightopen(X), 0, 1)]);
      l:= map(l, X -> [DOM::left(X), _if(DOM::isleftopen(X), 0, 1)]);
      r:= op(sort(r, (X, Y) -> op(X, 1) >= op(Y, 1) and op(X, 2) >= op(Y, 2)), 1);
      l:= op(sort(l, (X, Y) -> op(X, 1) >= op(Y, 1) and op(X, 2) <= op(Y, 2)), 1);
      new(dom, IVnat::new(_if(op(l, 2) = 1, DOM_LIST, id)(op(l, 1)),
                          _if(op(r, 2) = 1, DOM_LIST, id)(op(r, 1))))
    end_proc;
    
  float:=
    proc(x)
    begin
      dom::subsvals(x, float(dom::left(x)), float(dom::right(x)))
    end_proc;
    
  ln:=
    proc(x)
      local l,r, imag, realp;
    begin
      l:= if not iszero(dom::left(x)) then
            ln(dom::left(x))
          else
            -infinity
          end_if;
      r:= if dom::right(x)=infinity then
            infinity
          elif iszero(dom::right(x)) then
            FAIL // does not matter
          else
            ln(dom::right(x))
          end_if;
            
      imag:=piecewise
             ([dom::left(x)<0,
               PI*I + dom::ln(-dom(dom::leftB(x), [0]))],
              [dom::left(x)>=0, {}]);
      realp:=piecewise
              ([dom::right(x)<=0, {}],
               [dom::right(x)>0 and dom::left(x)<=0,
                dom::subsvals(x, -infinity, r)],
               [dom::left(x)>0, dom::subsvals(x, l, r)]);
      imag union realp
    end_proc;

  abs:=
    proc(x)
    begin
      case sign(x)
        of {0} do
        of {1} do
        of {0,1} do
          return(x)
        of {-1} do
        of {0, -1} do
          return(dom::_negate(x))
        of {0, 1, -1} do
          return(dom::_union(new(dom, IVnat::new([0], dom::rightB(x))),
                             new(dom, IVnat::new([0], dom::rightB(dom::_negate(x))))))
        otherwise
          hold(abs)(x)
      end_case
    end_proc;

    Re:= id;  // intervals only contain reals

    Im:={0};
    
  arccos:=
    proc(x)
    begin
      if dom::_is(dom::left(x) < -1, dom::isfloat(x)) then
        error("arccos of number smaller than -1")
      elif dom::_is(dom::right(x) > 1, dom::isfloat(x)) then
        error("arccos of number greater than 1")
      end_if;
      dom::interchangeBorders(dom::mapBorders(x, arccos))
    end_proc;
   
  arccosh:=
    proc(x) 
    begin
      if dom::_is(dom::left(x) < 1, dom::isfloat(x)) then
        error("argument outside range")
      end_if;
      dom::mapBorders(x, arccosh)
    end_proc;

  arccot:=
    proc(x)
    begin
      if has(sign(x), 0) then 
        new(dom, IVnat::new([-PI/2], [PI/2]))
      else 
        dom::interchangeBorders(dom::mapBorders(x, arccot))
      end_if
    end_proc;

  arccoth:=
    proc(x) 
    begin
      if dom::_is(dom::right(x) < -1, dom::isfloat(x))
         or dom::_is(1 < dom::left(x), dom::isfloat(x)) then
        dom::interchangeBorders(dom::mapBorders(x, arccoth))
      else
        error("singularity")
      end_if
    end_proc;

  arcsin:=
    proc(x) 
    begin
      if dom::_is(dom::left(x) < -1, dom::isfloat(x))
         or dom::_is(1 < dom::right(x), dom::isfloat(x)) then
        error("argument outside range")
      end_if;
      dom::mapBorders(x, arcsin)
    end_proc;

  arcsinh:=
    proc(x)
    begin
      dom::mapBorders(x, arcsinh)
    end_proc;

  arctan:=
    proc(x)
    begin
      dom::mapBorders(x, arctan)
    end_proc;

  arctanh:=
    proc(x)
    begin
      if dom::_is(dom::left(x) < -1, dom::isfloat(x))
         or dom::_is(1 < dom::right(x), dom::isfloat(x)) then
        error("argument outside range")
      end_if;
      dom::mapBorders(x, arctanh)
    end_proc;

  cos:=
    proc(x) 
      local lo, hi, l, r, xl, xr, FL;
    begin
      FL:= dom::isfloat(x);
      xl:= dom::leftB(x);
      xr:= dom::rightB(x);
      l:= dom::left(x);
      r:= dom::right(x);
      // full period covered ?
      if l=-infinity or r=infinity or dom::_is(r - l > 2*PI, FL) then
        return(new(dom, IVnat([-1], [1])))
      end_if;
      hi:= 2*PI*ceil(l/2/PI); // next maxima following the left bound
      lo:= 2*PI*ceil((l - PI)/2/PI) + PI; // next minima to the right
      if dom::_is(hi - r <= 0, FL) then
        hi:= [1]
      elif dom::_is(cos(l) >= cos(r), FL) then
        hi:= subsop(xl, 1 = cos(l))
      elif dom::_is(cos(r) >= cos(l), FL) then
        hi:= subsop(xr, 1 = cos(r))
      else
        return(hold(cos)(x))
      end_if;
      if dom::_is(lo - r <= 0, FL) then
        lo:= [-1]
      elif dom::_is(cos(l) <= cos(r), FL) then
        lo:= subsop(xl, 1 = cos(l))
      elif dom::_is(cos(r) <= cos(l), FL) then
        lo:= subsop(xr, 1 = cos(r))
      else
        return(hold(cos)(x))
      end_if;
      new(dom, IVnat::new(lo, hi))
    end_proc;

  cosh:= 
    proc(x)
    begin 
      case sign(x)
        of {1} do
        of {0, 1} do
          return(dom::mapBorders(x, cosh))
        of {-1} do
        of {0, -1} do
          return(dom::mapBorders(dom::_negate(x), cosh))
        of {0} do
          return(dom::one)
        of {0, 1, -1} do
           return(dom::_union(dom::mapBorders(new(dom, IVnat::new([0], map(dom::leftB(x), _negate))), cosh),
                              dom::mapBorders(new(dom, IVnat::new([0], dom::rightB(x))), cosh)))
        otherwise 
           hold(cosh)(x)
      end_case
    end_proc;

  cot:=
    proc(x)
    begin 
      if sign(x) = {0} then 
        error("singularity")
      else 
        dom::interchangeBorders(dom::mapBorders(x, cot))
      end_if
    end_proc;
        
  coth:=
    proc(x)
    begin
      if has(sign(x), 0) then
        error("singularity")
      else
        dom::interchangeBorders(dom::mapBorders(x, coth))
      end_if
    end_proc;

  sin:=
    proc(x)
      local lo, hi, l, r, xl, xr, FL;
    begin
      FL:= dom::isfloat(x);
      l:= dom::left(x);
      r:= dom::right(x);
      // full period covered ?
      if l=-infinity or r=infinity or dom::_is(r - l > 2*PI, FL) then
        return(new(dom, IVnat([-1], [1])))
      end_if;
      xl:= dom::leftB(x);
      xr:= dom::rightB(x);
      hi:= 2*PI*ceil((l - PI/2)/2/PI) + PI/2;
      // next maxima following the left bound
      lo:= 2*PI*ceil((l + PI/2)/2/PI) - PI/2;
      // next minima to the right
      if dom::_is(hi - r <= 0, FL) then
        hi:= [1]
      elif dom::_is(sin(l) >= sin(r), FL) then
        hi:= subsop(xl, 1 = sin(l))
      elif dom::_is(sin(r) >= sin(l), FL) then
        hi:= subsop(xr, 1 = sin(r))
      else
        return(hold(sin)(x))
      end_if;
      if dom::_is(lo - r <= 0, FL) then
        lo:= [-1]
      elif dom::_is(sin(l) <= sin(r), FL) then
        lo:= subsop(xl, 1 = sin(l))
      elif dom::_is(sin(r) <= sin(l), FL) then
        lo:= subsop(xr, 1 = sin(r))
      else
        return(hold(sin)(x))
      end_if;
      new(dom, IVnat(lo, hi))
    end_proc;

  sinh:=
    proc(x)
    begin
      dom::mapBorders(x, sinh)
    end_proc;

  tan:=
    proc(x) 
      local a, b, k, tana, tanb;
    begin
      a:= dom::left(x);
      b:= dom::right(x);
      if a=-infinity or b=infinity then
        return(new(dom, IVnat(-infinity, infinity)))
      end_if;
      // tan(x + k*PI) = tan(x)
      k:= round(a/PI);
      a:= a - k*PI;
      b:= b - k*PI;
      if a = -PI/2 then // is?
        tana:= -infinity
      else
        tana:= tan(a)
      end_if;
      if b = PI/2 then // is?
        tanb:= infinity
      else
        tanb:= tan(b)
      end_if;
      if dom::_is(b > PI/2, dom::isfloat(x)) then
        if dom::_is(b - a > PI, dom::isfloat(x)) then
          return(new(dom, IVnat(-infinity, infinity)))
        else
          return(dom::_union(dom::subsvals(x, -infinity, tanb),
                             dom::subsvals(x, tana, infinity)))
        end_if
      else
        dom::subsvals(x, tana, tanb)
      end_if
    end_proc;

  tanh:=
    proc(x:dom)
    begin
      dom::mapBorders(x, tanh)
    end_proc;

begin

end_domain:
