/*
LinDiffOp.mu
author: Marcus Hausdorf
created:        Fri Apr 28 13:15:20 1999    
last update: 	Fri Mar 17 18:43:35 2000
*/
// 13.11.2000

alias(REP(x) = extop(x, 1)):

domain Dom::LinearDifferentialOperator(S, R = Dom::ExpressionField(normal))

local Rcoerce:DOM_PROC, Risone:DOM_PROC, nV:DOM_INT, var_l:DOM_LIST, Rdiff:
    DOM_PROC, Rmult:DOM_PROC, Rplus:DOM_PROC, Rinvert:DOM_PROC, Rexpr:
    DOM_PROC, Rintmult:DOM_PROC;
inherits Dom::DistributedPolynomial(var_l, R, Dom::MonomOrdering(DegRevLex(nV)));
category Cat::Ring, Cat::LeftModule(R);
axiom (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
      (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if);

//==============
// In- / Output 
//==============

convert := proc(x)
    local s_l:DOM_LIST;
    begin
	case domtype(x)
	of dom do
	    return(x);
	of DOM_LIST do
	    if ({op(map(x, domtype))} <> {DOM_LIST}
		or {op(map(x, nops))} <> {2})
	    then
		return(FAIL);
	    end_if;
	    if ({op(map(map(x, l -> l[2]), domtype))} <> {DOM_LIST}
		or {op(map(map(x, l -> l[2]), nops))} <> {nV}
		or {map(op(x[i][2]) $ i = 1..nops(x), testtype, 
		Type::NonNegInt)} <> {TRUE})
	    then
		return(FAIL);
	    end_if;
	    if has((x := map(x, l -> [Rcoerce(l[1]), l[2]])), FAIL) then
		return(FAIL);
	    end_if;
	    return(new(dom, poly(x, var_l, R)));
	of DOM_INT do
	of DOM_RAT do
	of DOM_FLOAT do
	of DOM_COMPLEX do
	of DOM_IDENT do
	of DOM_EXPR do
	    if iszero(x) then
		return(dom::zero);
	    end_if;
	    x := expand(x);
	    case type(x) 
	    of "_plus" do
		s_l := [map(op(x), l -> (if type(l) = "_mult" then [op(l)] else [l] end_if))];
		break;
	    of "_mult" do
		s_l := [[op(x)]];
		break;
	    otherwise
		s_l := [[x]];
	    end_case;
	    s_l := map(s_l, l -> map(l, l2 -> 
		(if type(l2) = "_power" then
		    if (testtype(op(l2, 2), Type::PosInt)
			and type(op(l2, 1)) = "D"
			and not iszero(contains(S, op(op(l2, 1), 1))))
		    then
			return(hold(_power)(var_l[contains(S, op(op(l2, 1), 1))], op(l2, 2)));
		    else
			return(l2);
		    end_if;
		elif (type(l2) = "D" 
		    and not iszero(contains(S, op(l2, 1))))
		then
		    return(var_l[contains(S, op(l2, 1))]);
		else
		    return(l2);
		end_if)));
	    if not iszero(contains(map(s_l, op), FAIL)) then
		return(FAIL);
	    end_if;
	    return(new(dom, poly(hold(_plus)(op(map(s_l, l -> hold(_mult)(op(l))))), var_l, R)));
	of DOM_POLY do
	    return(dom::convert(expr(x)));
	of R do
	    return(new(dom, poly(x, var_l, R)));
	otherwise
	    return(FAIL);
	end_case;
    end_proc;
  
convert_to := proc(x:dom, T)
    local DOM:DOM_DOMAIN;
    begin
	case T
	of dom do
	    return(x);
	of DOM_EXPR do
	    return(dom::expr(x));
	of DOM_POLY do
	    return(subs(REP(x), zip(var_l, map(S, D), (l1, l2) -> l1 = l2)));
	of DOM_LIST do
	    DOM := dom;
	    return(map(dom::poly2list(x), l -> [l[1], DOM::expvec2term(l[2])]));
	of R do
	    if iszero(dom::order(x)) then
		return(dom::coeff(x));
	    end_if;
	otherwise
	    return(FAIL);
	end_case;
    end_proc;
    
new := proc(x)
    begin
	dom::convert(x);
	if % = FAIL then
	    error("illegal arguments");
	else
	    return(%);
	end_if;
    end_proc;

/*expr := proc(x:dom)
    local tmp, i:DOM_INT;
    begin
	if dom::iszero(x) then
	    return(0);
	end_if;
	x := dom::poly2list(x);
	for i from 1 to nops(x) do
	    tmp := zip(x[i][2], S, (l, s) -> (if iszero(l) then null() 
		elif l = 1 then hold(D)(s) 
		else hold(_power)(hold(D)(s), l) end_if));  
	    if tmp = [] then
		x[i] := x[i][1];
	    elif nops(tmp) = 1 then
		if Risone(x[i][1]) then
		    x[i] := tmp[1];
		else
		    x[i] := hold(_mult)(Rexpr(x[i][1]), op(tmp));
		end_if;
	    else
		if Risone(x[i][1]) then
		    x[i] := hold(_mult)(op(tmp));
		else
		    x[i] := hold(_mult)(Rexpr(x[i][1]), op(tmp));
		end_if;
	    end_if;
	end_for;
	if nops(x) = 1 then
	    return(x[1]);
	else
	    return(hold(_plus)(op(x)));
	end_if;
    end_proc;*/

expr := proc(x:dom)
    begin
	if dom::iszero(x) then
	    return(0);
	end_if;
	subs(expr(REP(x)), op(zip(var_l, map(S, hold(D)), (l1, l2) -> (l1 = l2))));
    end_proc;

print := dom::expr;

printTerm := (dvec -> op(zip(revert(S), revert(dvec), (l1, l2) -> (if iszero(l2) then null()
    elif l2 = 1 then hold(D)(l1) else hold(_power)(hold(D)(l1), l2) end_if))));
    
expvec2term := proc(e_l:DOM_LIST):DOM_EXPR
    local tmp;
    begin
	tmp := map(zip(e_l, map(S, D), [id]), l -> (if iszero(l[1]) then null() 
	    elif l[1] = 1 then l[2] 
	    else hold(_power)(l[2], l[1])
	    end_if));
	if tmp = [] then
	    return(1);
	elif nops(tmp) = 1 then
	    return(tmp[1]);
	else
	    return(hold(_mult)(op(%)));
	end_if;
    end_proc;
    
poly2list := (p -> (if dom::iszero(p) then return([[0, [0 $ nV]]])
    elif nV > 1 then return(poly2list(REP(p)))
    else return(map(poly2list(REP(p)), (l -> [l[1], [l[2]]]))) end_if));
    
dom2monomial := (t -> (dom::poly2list(t); if % = [] then return([0, [0 $ nV]]) 
    else return(%[1]) end_if;));

monomial2dom := (mi -> (if args(0) = 1 then new(dom, poly([[1, mi]], var_l, R)) 
    else new(dom, poly([[args(2), mi]], var_l, R)) end_if));
    
numVars := nV;

coeffRing := R;
    
Variables := S;

//====================
// Polynomial Methods 
//====================

order := dom::degree;

ordervec := dom::degreevec;

coeff := proc(x:dom)
    local tmp;
    begin
	if args(0) = 1 then
	    return(coeff(REP(x)));
	else
	    tmp := coeff(REP(x), args(2..args(0)));
	    if domtype(tmp) = DOM_POLY then
		return(new(dom, tmp));
	    else
		return(tmp);
	    end_if;
	end_if;
    end_proc;

leader := dom::lterm;

classMonomial := proc(x:dom):DOM_INT
    local k;
    begin 
	x := dom::dom2monomial(x)[2];
	k := nV;
	while (k > 0 and iszero(x[k])) do
	    k := k - 1;
	end_while;
	if iszero(k) then
	    return(0);
	else
	    return(nV + 1 - k);
	end_if;
    end_proc;

class := (x -> dom::classMonomial(dom::lterm(x)));

//====================================
// Arithmetics / Differential Methods 
//====================================

zero := new(dom, poly(0, var_l, R));

one := new(dom, poly(1, var_l, R));

invertibleCoefficients := R::hasProp(Cat::Field);

characteristic := R::characteristic;

_mult := proc():dom
    local s:DOM_LIST, res1:dom, res2:R, DOM:DOM_DOMAIN, tmp;
    begin
	DOM := dom;
	s := [args(1..args(0))];
	tmp := split(s, l -> domtype(l) = DOM);
	res1 := dom::_mult2(op(tmp[1]));
	res2 := Rmult(op(tmp[2]));
	dom::multcoeffs(res1, res2); 
    end_proc; 

_mult2 := proc():dom
    local f1, f2, DOM:DOM_DOMAIN, i:DOM_INT, el:DOM_LIST, k:DOM_INT, res_l:DOM_LIST,
	res2_l:DOM_LIST;
    begin
	DOM := dom;
	case args(0)
	of 0 do
	    return(dom::one);
	of 1 do 
	    return(args(1));
	of 2 do
	    f1 := dom::poly2list(args(1));
	    f2 := dom::poly2list(args(2));
	    /*return(dom::_plus(dom::mapcoeffs(f2, l -> DOM::func_call(f1, l)), 
		new(dom, REP(f1) * REP(f2))));*/
	    res_l := [];
	    for el in f1 do
		res2_l := f2;
		for k from nV downto 1 do
		    if iszero(el[2][k]) then next end_if;
		    res2_l := map(res2_l, (l -> op([[Rintmult(binomial(el[2][k],i), 
			Rdiff(l[1], S[k] $ i)), subsop(l[2], k = l[2][k] + el[2][k] - i)] 
			$ i = 0..el[2][k]])));
		end_for;
		res_l := res_l . map(res2_l, l -> [Rmult(l[1], el[1]), l[2]]);
	    end_for;
	    return(new(dom, poly(res_l, var_l, R))); 
	otherwise
	    i := args(0) div 2;
	    return(dom::_mult2(dom::_mult2(args(1..i)),
			       dom::_mult2(args((i + 1)..args(0)))));
	end_case;
    end_proc;
    
mapcoeffs := (x -> new(dom, mapcoeffs(REP(x), args(2..args(0)))));

multcoeffs := (x -> new(dom, multcoeffs(REP(x), args(2))));

Rdivide := if not dom::invertibleCoefficients then FAIL else
    ((x, r) -> dom::multcoeffs(x, Rinvert(r))) end_if;
    
func_call := proc(x, r):R
    begin
	x := dom::coerce(x);
	r := Rcoerce(r);
	if testargs() then
	    if (x = FAIL or r = FAIL) then
		error("wrong type of arguments");
	    end_if;
	end_if;
	x := dom::poly2list(x);
	Rplus(op(map(x, l -> Rmult(l[1], Rdiff(r, S[i] $ l[2][i] 
	    $ i = 1..nops(l[2]))))));
    end_proc;
	        
commutator := ((x, y) -> (dom::_subtract(dom::_mult(x, y),
					 dom::_mult(y, x))));

diff := (x -> dom::mapcoeffs(x, Rdiff, args(2..args(0))));

D := ((i, x) -> dom::mapcoeffs(x, (f, i) -> R::D(i, f), i));

random := () -> new(dom, polylib::randpoly(var_l, R));

//===================
// adjoint operators 
//===================
    
adjointMonomial := proc(x:DOM_LIST):dom
    begin
	if iszero(x[1]) then
	    return(dom::zero);
	end_if;
	dom::_mult(new(dom, poly([[1, x[2]]], var_l, R)), 
	    dom::multcoeffs(dom::one, Rintmult((-1)^(_plus(op(x[2]))), x[1])));
    end_proc;
	    
adjoint := (x -> dom::_plus(op(map(dom::poly2list(x), 
    dom::adjointMonomial))));

adjointMatrix := proc(x)
    local m_flag:DOM_BOOL, mtype:DOM_DOMAIN;
    begin
	m_flag := FALSE;
	mtype := domtype(x);
	if (mtype::hasProp <> FAIL
	    and mtype::hasProp(Cat::Matrix) 
	    and mtype::coeffRing = dom)
	then
	    m_flag := TRUE;
	    x := mtype::convert_to(x, DOM_LIST);
	elif not testtype(x, Type::ListOf(Type::ListOf(dom))) then
	    error("wrong type of arguments");
	end_if;
	x := [[dom::adjoint(x[i][j]) $ i = 1..nops(x)] $ j = 1..nops(x[1])];
	if m_flag then
	    return(mtype(x));
	else
	    return(x);
	end_if;
    end_proc;

//================
// LDO <-> LinFun 
//================

makeSystem := proc(sys, LDF)
    local DV:DOM_DOMAIN, CR:DOM_DOMAIN, ind_l:DOM_LIST, tmp, z1:DOM_LIST, z2:
	DOM_LIST, DOM:DOM_DOMAIN, mtype:DOM_DOMAIN;
    begin
	DOM := dom;
	mtype := domtype(sys);
	if (mtype::hasProp <> FAIL
	    and mtype::hasProp(Cat::Matrix) 
	    and mtype::coeffRing = dom)
	then
	    sys := mtype::convert_to(sys, DOM_LIST);
	end_if;
	DV := LDF::Variables;
	CR := LDF::coeffRing;
	ind_l := map(DV::indVariables, DV::expr);
	if testargs() then
	    if (ind_l <> S 
		or nops({op((tmp := map(sys, nops)))}) <> 1
		or tmp[1] <> DV::numDepVar) 
	    then
		error("incompatible domains");
	    end_if;
	end_if;
	z1 := [0 $ nV];
	z2 := [i $ i = 1..DV::numDepVar];
	map(map(map(sys, l -> map(l, DOM::poly2list)), 
	    l2 -> zip(z2, l2, 
		(l2, l3) -> op(map(l3, l4 -> 
		(if l4[2] = z1 then 
		    return([CR::convert(l4[1]), DV::newMulti("Dep", l2)]) 
		 else 
		    return([CR::convert(l4[1]), 
			DV::newMulti("Deriv", l2, l4[2])]) end_if))))), 
	    LDF::convert);
    end_proc;
    
makeMatrix := proc(sys)
    local LDF:DOM_DOMAIN, DV:DOM_DOMAIN, R:DOM_DOMAIN, j:DOM_INT, res_l:
	DOM_LIST, res2_l:DOM_LIST, f, k, ind:DOM_INT, DOM:DOM_DOMAIN; 
    begin
	DOM := dom;
	LDF := domtype(sys[1]);
	DV := LDF::Variables;
	R := LDF::coeffRing;
	if testargs() then
	    if map(DV::indVariables, DV::expr) <> S then
		error("incompatible domains");
	    end_if;
	end_if;
	res_l := [[] $ nops(sys)];
	j := 0;
	for f in (map(sys, extop, 1)) do
	    j := j + 1;
	    res2_l := [[] $ DV::numDepVar];
	    for k in f do
		ind := DV::index(k[2]);
		res2_l := subsop(res2_l, ind = append(res2_l[ind], 
		    [Rcoerce(k[1]), DV::multiIndex(k[2])]));
	    end_for;
	    res_l[j] := map(res2_l, l -> new(DOM, poly(l, var_l, R)));
	end_for;
	if args(0) = 2 and args(2) = FALSE then
	    return(res_l);
	else
	    return((Dom::Matrix(dom))(res_l));
	end_if;
    end_proc;

//================
// body of domain 
//================

begin
    if args(0) < 1 or args(0) > 2 then
	error("wrong number of arguments");
    end_if;
    if (domtype(R) <> DOM_DOMAIN
	or R::hasProp = FAIL
	or not R::hasProp(Cat::PartialDifferentialRing))
    then
	error("second argument must be Cat::PartialDifferentialRing");
    end_if;
    if domtype(S) <> DOM_LIST then
	error("first argument must be list");
    end_if;
    nV := nops(S);
    var_l := [genident("_ldo_v_") $ i = 1..nV];
    Rcoerce := R::coerce;
    Risone := R::isone;
    Rdiff := R::diff;
    Rmult := R::_mult;
    Rplus := R::_plus;
    Rinvert := R::_invert;
    Rexpr := R::expr;
    Rintmult := R::intmult;
end_domain;

unalias(REP(x)):


