/*++
Dom::MonoidAlgebra(R,S) -- free algebra over the ring R with (finite) support
	in the ordered monoid S where one assumes that the elements of R and
	S commutes (otherwise see Dom::MonoidOperatorAlgebra(R,S))

This domain is a specialization of Dom::FreeModule(R,S). Especially, it
inherits its representation. The only additional operation is the
multiplication of elements of the domain. Furthermore we have now a bimodule.


Author: Werner M. Seiler, Uni Karlsruhe/Mannheim
        (wms@ira.uka.de, seiler@euler.math.uni-mannheim.de)
Created: July 3rd, 1997
Last update: 	Thu Mar  4 08:54:23 1999
++*/


alias(REP(x)=extop(x,1),
      TYPE_ERR=error("wrong type of argument")):


domain Dom::MonoidAlgebra(R,S)

    local Rmult:DOM_PROC, Rcoerce:DOM_PROC, Riszero:DOM_PROC,
          Rone:R, Smult:DOM_PROC, Sone:S, Spoly;
    inherits Dom::FreeModule(R,S);
    category
        (if R::hasProp(Cat::CommutativeRing) and
            S::hasProp(Cat::AbelianMonoid) then Cat::CommutativeRing end_if),
        Cat::Module(R), Cat::FiniteMonoidRing(R,S);
    axiom
        (if R::hasProp(Ax::canonicalRep) and
            S::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
        (if R::hasProp(Ax::normalRep) and
            S::hasProp(Ax::normalRep) then Ax::normalRep end_if);


    one := dom::generate1(Rone,Sone);


    poly :=
        if Spoly=FAIL then NIL else
            proc(a:dom):DOM_POLY
            begin
                _plus(op(map(REP(a), ((b,c) -> multcoeffs(Spoly(b[2],c),b[1])), R)))
            end_proc;
        end_if;


    _mult :=
        proc():dom
        local res,x1,x2,i,j,k,r,s,f;
        begin
	    case args(0)
	    of 0 do return(dom::one);
	    of 1 do return(args(1));
	    of 2 do
	        x1 := Rcoerce(args(1));
		if x1=FAIL then
                    x1 := dom::coerce(args(1));
                    if x1=FAIL then TYPE_ERR end_if;
		else
		    x2 := Rcoerce(args(2));
		    if x2=FAIL then
		        x2 := dom::coerce(args(2));
		        if x2=FAIL then TYPE_ERR end_if;
		        return(dom::smult(x1,x2));
		    else return(Rmult(x1,x2));
		    end_if;
		end_if;
	        x2 := Rcoerce(args(2));
                if x2=FAIL then
                    x2 := dom::coerce(args(2));
                    if x2=FAIL then TYPE_ERR end_if;
                else return(dom::smult(x2,x1)) end_if;
	        if not(iszero(contains([R,DOM_INT],domtype(x1)))) then
		    return(dom::smult(x1,x2));
	        elif not(iszero(contains([R,DOM_INT],domtype(x2)))) then
		    return(dom::smult(x2,x1));
	        end_if;

                res := [];
                for i in REP(x1) do
                    res := res .
                              map(REP(x2),((a,b) -> [Rmult(b[1],a[1]),
                                                     Smult(b[2],a[2])]), i);
                end_for;
                return(dom::generate(dom::normalizeRep(res)));
	    otherwise
	        r := [ 0 $ args(0) ];
                s := [ 0 $ args(0) ];
                i := 0;
                j := 0;
                for f in args() do
                    x1 := Rcoerce(f);
                    if x1=FAIL then
                        x1 := dom::coerce(f);
                        if x1=FAIL then TYPE_ERR
                        else
                            j := j+1;
                            s[j] := x1;
                        end_if;
                    else
                        i := i+1;
                        r[i] := x1;
                    end_if;
                end_for;
                case j
                of 0 do return(Rmult(op(r,1..i)));
                of 1 do
                    s := s[1];
                    break;
                of 2 do
                    s := dom::_mult(s[1],s[2]);
                    break;
                otherwise
                    k := j div 2;
                    s := dom::_mult(dom::_mult(op(s,1..k)),
                                    dom::_mult(op(s,k+1..j)));
                end_case;
                case i
                of 0 do return(s);
                of 1 do return(dom::smult(r[1],s));
                otherwise return(dom::smult(Rmult(op(r,1..i)),s));
                end_case;
	    end_case;
        end_proc;


begin

    if args(0)<>2 then error("wrong number of arguments") end_if;
    if (domtype(R)<>DOM_DOMAIN) or (domtype(S)<>DOM_DOMAIN) or
       (R::hasProp=FAIL) or (S::hasProp=FAIL) then
        error("arguments must be domains");
    end_if;
    if R::hasProp(Cat::Ring) then
        Rmult := R::_mult;
        Rcoerce := R::coerce;
        Riszero := R::iszero;
        Rone := R::one;
    else error("first argument must be a Cat::Ring") end_if;
    if S::hasProp(Cat::OrderedMonoid) then
        Smult := S::_mult;
        Sone := S::one;
    elif S::hasProp(Cat::OrderedAbelianMonoid) then
        Smult := S::_plus;
        Sone := S::zero;
    else error("second argument must be ordered monoid") end_if;
    Spoly := S::poly

end_domain:


unalias(REP,TYPE_ERR):
