/*++
Dom::TensorAlgebra(R,S,order) -- tensor algebra with coefficient ring R over
	the ordered set S of variables (S can also be a list; the ordering of
	the variables is then determined by the position in the list). The
	optional parameter order is a string naming the orderings used for
	the products.

Dom::TensorAlgebra(R,S,order) is a simple extension of Dom::MonoidAlgebra(R,
Dom::TensorProduct(S,order)). It is only needed in order to implement some
additional methods. Currently, this is only diff.


Author: Werner M. Seiler, Uni Karlsruhe/Mannheim 
	(wms@ira.uka.de, seiler@euler.math.uni-mannheim.de)
Created: March 19th, 1998
Last update: 	Tue Feb 15 14:28:39 2000
++*/


alias(REP(x)=extop(x,1)):


domain Dom::TensorAlgebra(R, S, order)

    local TP:DOM_DOMAIN, TPvar:DOM_PROC, TPpoly:DOM_PROC, TPdiff:DOM_PROC;
    inherits Dom::MonoidAlgebra(R,TP);
    category Cat::Module(R), Cat::FiniteMonoidRing(R,TP);
    axiom (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
	  (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if);


    variables :=
	proc(x:dom):DOM_LIST
	local vars:DOM_LIST, res:DOM_LIST, l:DOM_LIST;
	begin
	    if dom::iszero(x) then return([]) end_if;
	    vars := map(REP(x), ((l) -> TPvar(l[2])));
	    res := vars[1];
	    delete vars[1];
	    for l in vars do
		res := listlib::singleMerge(res,l,TP::Sless);
	    end_for;
	    res;
	end_proc;
	
	
    poly := 
	proc(x:dom):DOM_POLY
	begin
	    _plus(op(map(REP(x), ((l) -> multcoeffs(TPpoly(l[2]),l[1])))));
	end_proc;
    
    
    diff :=
	proc(x:dom):dom
	local res:DOM_LIST;
	begin
	    res := map(REP(x),
		       ((l) -> op(map(TPdiff(l[2],args(2..args(0))),
			          ((li,r) -> [r,li]), l[1]))),
		       args(2..args(0)));
	    if res=[] then dom::zero
	    else dom::generate(dom::normalizeRep(res)) end_if;
	end_proc;


begin

    if args(0)<2 or args(0)>3 then error("wrong number of arguments") end_if;
    if domtype(R)<>DOM_DOMAIN or (R::hasProp=FAIL) or
       not(R::hasProp(Cat::Ring)) then
	error("first argument must be a Cat::Ring");
    end_if;
    if domtype(S)=DOM_LIST then
	if S=[] then error("empty list") end_if;
    elif domtype(S)<>DOM_DOMAIN or (S::hasProp=FAIL) or
	 not(S::hasProp(Cat::OrderSet)) then
	error("second argument must be Cat::OrderedSet or list");
    end_if;
    
    if args(0)=2 then order := hold(DegreeOrder) end_if;
    TP := Dom::TensorProduct(S,order);
    TPvar := TP::variables;
    TPpoly := TP::poly;
    TPdiff := TP::diff;
    
end_domain:


unalias(REP):
