// kg, 26/06/96 

/*++
fp::unapply -- create a procedure from an expression

fp::unapply(e, [x1,...,xn])

e         - expression, regarded as function of x1,...,xn
x1,...,xn - (optional) indeterminates

fp::unapply views the expression e as a function of the indeterminates
x1,...,xn and returns a procedure computing that function.
If no indeterminates are given, any indeterminates of e found by indets
are used.

Examples:

fp::unapply(sin(x), x)		-> proc(x) begin sin(x) end_proc
fp::unapply(x+y, y, x)		-> proc(y,x) begin x+y end_proc
fp::unapply(2*x^2+1)		-> proc(x) begin 2 * x^2 + 1 end_proc
++*/

fp::unapply:=
proc(e)
  local Args, i;
begin
  if testargs() then
    if args(0) < 1 then
      error("wrong no of args")
    end_if;
    if args(0) > 1 then
      Args:= { args(2..args(0)) };
      if (nops(Args) <> args(0) - 1) or (Args <> indets(Args)) then
        error("wrong indeterminates")
      end_if
    end_if
  end_if;

  if e::dom::unapply <> FAIL then
    return(e::dom::unapply(args()))
  end_if;
  
  // get indeterminates 
  if args(0) = 1 then
    Args:= [ op(indets(e)) ]
  else
    Args:= [ args(2..args(0)) ]
  end_if;

  if Args = [] then
    subsop(proc() option arrow; begin end, 4=e, Unsimplified)
  else
    subsop(proc() option arrow; begin end, 1=op(Args),
           4=subs(e, [ Args[i]=DOM_VAR(0,1+i) $ i=1..nops(Args) ],
                  Unsimplified))
  end_if
end_proc:

// end of file 
