//**************************************************************************
//                     MACROFORT: FORTRAN code generator                      
//                                                                            
//                     Originally created by Claude Gomez                     
//                          at INRIA-Rocquencourt                             
//			 email: Claude.Gomez@inria.fr	                     
//                                                                            
//                       Adapted to MuPAD and Revised                         
//                              by Tony Scott                                 
//			 email: Tony.Scott@loria.fr	                     
//			        (Oct. 1998)                                  
//                                                                            
//		                Project POLKA                                
//                                                                            
//                                   LORIA                                    
//                            CAMPUS SCIENTIFIQUE                             
//                            VANDOEUVRE LES NANCY                            
//                                                                            
//**************************************************************************

//**************************************************************************
//  FUNCTIONS AVAILABLE FOR THE USER IN MACROFORT:                            
//  GENFOR INITGENFOR                                                         
//                                                                            
//  GLOBAL VARIABLES AVAILABLE FOR THE USER:                                  
//  COMMENT INPUTF OUTPUTF OPTIMIZED PRECISION                                
//**************************************************************************

//**************************************************************************
// internal functions directly related to MACROFORT : (macrofort::name)       
// callf closef commentf commonf continuef declaref dof do_m elsef endf       
// endiff equalf formatf functionf functionm gotof ifgotof ifthenelsem        
// ifthenf ifthenm matrixm openf openm parameterf programf programm readf     
// readm returnf subroutinef subroutinem untilm whilem writef writem          
//                                                                            
//  other internal functions:                                                 
// get_flabel get_label list listcom lprint parse_condition space             
//                                                                            
//  internal global variables:                                                
// nflab nflabel nlab nlabel nuntil nwhile                                    
//  internal tables                                                           
// flabel label macro                                                         
//**************************************************************************

alias(macrofort=generate::Macrofort):
macrofort := newDomain("generate::Macrofort");
macrofort::info := "Domain 'generate::Macrofort': FORTRAN Code Generator":
macrofort::interface := {hold(setPrecisionOption), hold(setOptimizedOption), 
                         hold(setAutoComment), hold(setIOSettings), 
                         hold(openOutputFile), hold(closeOutputFile),
			 hold(init), hold(genFor) };
alias(m_fortran=generate::m_fortran):
m_fortran := newDomain("generate::m_fortran");
// alias 
alias(m_fortrans=m_fortran::m_fortrans):
alias(m_Fortran=m_fortran::m_Fortran):
alias(space=macrofort::space):
alias(get_label=macrofort::get_label):
alias(get_flabel=macrofort::get_flabel):
alias(maclist=macrofort::list): 
alias(listcomp=macrofort::listcomp):
alias(listcom=macrofort::listcom):
alias(maclabel=macrofort::label): 
alias(macflabel=macrofort::flabel): 
alias(macnlabel=macrofort::nlabel): 
alias(macnflabel=macrofort::nflabel):
alias(macnwhile=macrofort::nwhile): 
alias(macnuntil=macrofort::nuntil): 
alias(parse_condition=macrofort::parse_condition):
alias(parse_condition1=macrofort::parse_condition1):
alias(maclprint=macrofort::lprint):
alias(genForm=macrofort::genForm):
alias(genFormdcl=macrofort::genFormdcl):
alias(macgenFor=macrofort::genfor):
alias(macmacro=macrofort::macro):
alias(macnlab=macrofort::nlab):
alias(macnextbreak=macrofort::nextbreak):

alias(m_statement=m_fortran::statement):
alias(writeln=m_fortran::writeln):
alias(arrayexpand=m_fortran::arrayexpand):
alias(m_expression=m_fortran::expression):
alias(m_write=m_fortran::write):
alias(m_precedence=m_fortran::precedence):
alias(m_float=m_fortran::float):
alias(m_integer=m_fortran::integer):
alias(m_function=m_fortran::function):
alias(m_function_IR1=m_fortran::function_IR1):
alias(m_function_R1=m_fortran::function_R1):
alias(m_function_R2=m_fortran::function_R2):
alias(m_function_name=m_fortran::function_name): 

// added routines 
alias(lextr=macrofort::lextr):
alias(myLength=macrofort::myLength):
alias(mysubstring=macrofort::mysubstring):
alias(mycontains=macrofort::mycontains):
alias(Indices=macrofort::Indices):
alias(assigned=macrofort::assigned):
alias(represent=macrofort::represent):
alias(iremm=macrofort::iremm):
alias(findsign=macrofort::findsign):
alias(cmp=macrofort::cmp):
alias(sortmult=macrofort::sortmult):
alias(sortplus=macrofort::sortplus):
alias(switchprec=macrofort::switchprec):

// global buffers 
alias(_istr=m_fortran::istr):
alias(_buffer=m_fortran::buffer):
// global settings 
alias(precision=macrofort::precision):
alias(optimized=macrofort::optimized):
alias(outputfile=macrofort::outputfile):
alias(fd=macrofort::fd):
alias(comment=macrofort::comment):
alias(inputf=macrofort::inputf):
alias(outputf=macrofort::outputf):

alias(PLUS=hold(slot)(macrofort, "PLUS")):
alias(MULT=hold(slot)(macrofort, "MULT")):
alias(SQRT=hold(slot)(macrofort, "SQRT")):

// changing defaults for macrofort

macrofort::setPrecisionOption:=proc(x:DOM_STRING)
begin
case x 
  of "single" do of "double" do of "quadruple" do 
      sysassign(precision, x): break
  otherwise 
  error("precision should be: single, double or quadruple");
end_case:
end_proc:

macrofort::setOptimizedOption:=proc(x:DOM_BOOL)
begin
  sysassign(optimized, x):
end_proc:

macrofort::setAutoComment:=proc(c:DOM_BOOL)
begin
  sysassign(comment, c):
end_proc:

macrofort::setIOSettings:=proc(i:DOM_INT,o:DOM_INT)
begin
  sysassign(inputf, i): 
  sysassign(outputf, o):
end_proc:

macrofort::openOutputFile:=proc(x:DOM_STRING)
local sname;
begin
  sysassign(outputfile, x):
  //sysassign(sname, myLength(x)):
  sname:=myLength(x):
  if mysubstring(x,sname-1,sname)=".f" then
     sysassign(outputfile, x):
     sysassign(fd, fopen(Text, outputfile, Write));
  else
     error(_concat("Improper file type for FORTRAN output file:",x));
  end_if:
end_proc:

macrofort::closeOutputFile:=proc()
begin
    fclose(fd):
end_proc:

lextr:=proc(l,n1,n2)
begin
 [op(l,n1..n2)]:
end_proc:

myLength:=proc(l)
begin
// version for 1.5.0 
case type(l)
  of DOM_STRING do of DOM_INT do return(length(l)):
  of DOM_IDENT do of "_index" do of DOM_ARRAY do return(length(expr2text(l))):
  otherwise
        error(_concat("myLength does not cover this case",expr2text(l))):
end_case:
end_proc:

mysubstring:=proc(s,a1,a2)
local a1m,a2m,temp;
begin
a1m:=a1-1:
a2m:=a2-1:
if domtype(s)=DOM_STRING then
 temp:=s:
else
 temp:=_concat("",s):
end_if:
substring(temp,a1m..a2m):
end_proc:

mycontains:=proc(f,st)
local i,verd;
begin
verd:=has(f,op(st,1)):
for i from 2 to nops(st) do
    verd:=bool(verd or has(f,op(st,i))):
end_for:
verd:
end_proc:

Indices := proc(data:DOM_ARRAY) local p;
begin
  p:= proc(x : DOM_LIST)
        local i;
      begin
        if nops(x) = op(data,[0,1]) then
	   x
        else
          p(x.[i]) $ i=op(data,[0,nops(x)+2])
        end_if
      end_proc;
  p([])
end_proc:

represent := proc(num)
local e,mantissa,exponent;
begin
   if not iszero(num) then
    e:=ceil(slot(ln,"float")(specfunc::abs(float(num)))/2.302585093):
    mantissa:=round(num*10^(DIGITS-e)):
    exponent:=e-DIGITS:
   else
    mantissa:=0:
    exponent:=0:
   end_if:
[mantissa,exponent]:
end_proc:

iremm:=proc(m,n)
local r;
begin
r := irem(m,n):
(m-r)/n:
end_proc:

findsign:=proc(e)
local temp;
begin
if contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(e)) then sign(e):
elif type(e)="_mult" and 
   contains({DOM_RAT, DOM_INT, DOM_FLOAT},domtype(op(e,nops(e)))) 
   then op(e,nops(e)):
elif type(e)="_power" or domtype(e)=DOM_IDENT then
   1:
else
   temp:=traperror(op(split(e, x -> bool(domtype(x)=DOM_INT)),1)*
      op(split(e, x -> bool(domtype(x)=DOM_FLOAT)),1)):
   if iszero(temp) then
     temp:=op(split(e, x -> bool(domtype(x)=DOM_INT)),1)*
           op(split(e, x -> bool(domtype(x)=DOM_FLOAT)),1):
     if contains({DOM_RAT,DOM_INT,DOM_FLOAT},domtype(temp)) and 
        temp > 0 then 
        temp:
     else
        1:
     end_if:
   else
    temp:=e: 
    if not(domtype(temp)=DOM_STRING) then temp:=expr2text(temp): end_if;
    error(_concat("cannot figure out the sign of ",temp)): 
   end_if:
end_if:
end_proc:


cmp:=proc(a,b)
begin
 not(bool((findsign(a)<0) and not(findsign(b)<0))):
end_proc:

sortmult:=proc(e)
local i,temp,cons1,cons2,top,bot;
begin
temp:=[op(e)]: 
cons1:=null(): cons2:=null():
bot:=null(): top:=null():
for i in temp do
 if contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(i)) then 
   cons1:=cons1,i:
 elif (i=PI) or (i=E) or (i=EULER) or (i=CATALAN) then
   cons2:=cons2,i:
 elif denom(i) <> 1 then
   bot:=bot,i:
 else
   top:=top,i:
 end_if:
end_for:
MULT(cons1,cons2,top,bot):
end_proc:

sortplus:=proc(e)
begin
if findsign(op(e,1)) < 0 then
  PLUS(op(sort([op(e)],cmp))):
else
  PLUS(op(e)):
end_if:
end_proc:

assigned := x -> bool(x <> FAIL): 

switchprec:=proc(a,b,c)
begin
case precision
 of "single" do return(a):
 of "double" do return(b):
 of "quadruple" do return(c):
 otherwise a: 
end_case:
end_proc:

// utilitary functions  

space := proc(n) 
begin
  if iszero(n) then "" else _concat(" " $n): end_if 
end_proc:

get_label := proc(flabel)
//global maclabel,macnlabel: 
begin
  if domtype(flabel)=DOM_INT then 
    _concat(space(4-myLength(flabel)),flabel)
  elif not(assigned(maclabel[flabel])) or 
       domtype(maclabel[flabel])=DOM_INT then
            maclabel[flabel]:
    else
       sysassign(macnlabel, macnlabel + 1):
       sysassign(maclabel[flabel],macnlabel + 1000):
       macnlabel + 1000:
  end_if
end_proc:

get_flabel := proc(flabel)
//global macnflabel,macflabel: 
begin
  if domtype(flabel)=DOM_INT then 
    _concat(space(4-myLength(flabel)),flabel)
  elif not(assigned(macflabel[flabel])) or 
       domtype(macflabel[flabel])=DOM_INT then
            macflabel[flabel]:
  else sysassign(macnflabel,macnflabel + 1):
       sysassign(macflabel[flabel],macnflabel + 2000):
       macnflabel + 2000:
  end_if
end_proc:

maclist := proc(l) 
begin
  if l = [] then return(l) end_if:
  if type(l[1])=DOM_LIST then l: else [l]; end_if;
end_proc:

listcomp := proc(list)
  local i, listc;
begin
  if list = [] then return("") end_if:
  if type(list[1])="_equal" then 
    listc:=_concat(m_fortrans(op(list[1],1)),"=",m_fortrans(op(list[1],2))):
  else listc := m_fortrans(list[1]) end_if: 
   for i in lextr(list,2,nops(list)) do
    if type(i)="_equal" then
      listc:=_concat("",listc,",",m_fortrans(op(i,1)),"=",m_fortrans(op(i,2))):
    else listc := _concat("",listc,",",m_fortrans(i)) end_if:
  end_for:
  listc:
end_proc:

listcom := proc(list)
  local i, listc;
begin
  if list = [] then return("") end_if:
  listc := m_fortrans(list[1]):
  for i in lextr(list,2,nops(list)) do
    listc := _concat("",listc,",",m_fortrans(i))
  end_for:
  listc
end_proc:

parse_condition := proc(condition)
  local c;
begin
  c:=parse_condition1(condition):
  if (mysubstring(c,1,1) = "(") then
    mysubstring(c,2,myLength(c)-1)
  else c
  end_if
end_proc:

parse_condition1 := proc(condition)
begin
  case type(condition) 
  of "_equal" do 
  return(_concat("(",m_fortrans(op(condition,1)),".eq.",m_fortrans(op(condition,2)),")")):
  of "_less"  do
  return(_concat("(",m_fortrans(op(condition,1)),".lt.",m_fortrans(op(condition,2)),")")):
  of "_leequal" do 
  return(_concat("(",m_fortrans(op(condition,1)),".le.",m_fortrans(op(condition,2)),")")):
  of "_unequal" do
  return(_concat("(",m_fortrans(op(condition,1)),".ne.",m_fortrans(op(condition,2)),")")):
  of DOM_LIST do
     case op(condition,1)
       of "NOT" do
       return(_concat("(.not.",parse_condition1(op(condition,2)),")")):
       of "AND" do
       return(_concat("(",parse_condition1(op(condition,2)),".and.",
            parse_condition1(op(condition,3)),")")):
       of "OR" do
       return(_concat("(",parse_condition1(op(condition,2)),".or.",
             parse_condition1(op(condition,3)),")")):
       otherwise return(m_fortrans(condition)):
     end_case:
   otherwise return(m_fortrans(condition)):
  end_case:
end_proc:

maclprint := proc(str) 
  local l;
begin
  l := myLength(str):
  if l <= 72 then fprint(Unquoted,fd,str);
  else fprint(Unquoted,fd,mysubstring(str,1,72));
       maclprint(_concat("     #",mysubstring(str,73,l))):
  end_if:
end_proc:

//**************************************************************************
// GENFOR : function generating FORTRAN code from its description             
//          by a list                                                         
//**************************************************************************

macrofort::new := proc(l) 
  local old_DIGITS, mres;
begin
  old_DIGITS := DIGITS:
  if assigned(precision) and 
     (contains(["single","double","quadruple"],precision)=0)
  then error(_concat("bad setting for precision ",precision)) end_if:
  case precision
  of "single" do DIGITS := 7:     break
  of "double" do DIGITS := 16:    break
  of "quadruple" do DIGITS := 32: break
  end_case:
  if not(type(optimized)=DOM_BOOL) then 
     error("optimization should be boolean") end_if:
  mres:=genForm(l):
  if type(mres)="_exprseq" then mres := [mres] end_if:
  macgenFor(mres,0):
  DIGITS := old_DIGITS:
  null()
end_proc:
macrofort::genFor := macrofort::new:


//***********************************************************************
// INIT_GENFOR() used to initialize global variables before every          
//    new FORTRAN code generation                                          
//***********************************************************************

macrofort::init := proc()
//global macnlab,macnflabel,macnlabel,
//     macnwhile,macnuntil,macflabel,
//     maclabel,comment,optimized,precision:
begin
  sysassign(macnlab, 0):
  sysassign(macnflabel, -1):
  sysassign(macnlabel, -1):
  sysassign(macnwhile, 1):
  sysassign(macnuntil, 1):
  sysassign(maclabel, table()):
  sysassign(macflabel, table()):
  sysassign(comment, TRUE):
  sysassign(inputf, 5):
  sysassign(outputf, 6):
  sysassign(outputfile, "macrofort.f"):
  sysassign(optimized, FALSE):
  sysassign(precision, "single"):
  null()
end_proc:

// macros expansion  
genForm := proc(l)
 local i,len,temp,opereff,operstr;
begin 
 if l = [] then return([]) end_if:
  if type(l)=DOM_LIST then
      if domtype(l[1])=DOM_STRING then
       operstr:=l[1]: 
      else
       operstr:=expr2text(l[1]):
      end_if:
      if operstr="programm" or operstr="subroutinem" or operstr="functionm" then
        opereff:=slot(macrofort,operstr):
        opereff(op(l,2..nops(l))):
      elif macmacro[operstr] = TRUE then
        len:=myLength(operstr):
        if mysubstring(operstr,len,len) <> "m" or operstr="program" then
           operstr:=_concat(operstr,"f"):
        end_if:
        opereff:=slot(macrofort,operstr):
        op(genForm(opereff(op(l,2..nops(l))))): 
      else 
        temp:=null():
        for i in l do
           temp:=temp,genForm(i):
        end_for:
        [temp]: 
      end_if
    else l:
  end_if:
end_proc:

// macros expansion for COMMONM and DECLAREM  
genFormdcl := proc(l,dcl,com)
  local i,opereff,operstr,dcloutput,comoutput,res,temp;
begin
  dcloutput:=dcl:
  comoutput:=com:
  if l = [] then return([[],dcloutput,comoutput]) end_if:
  if type(l)=DOM_LIST then
    if domtype(l[1])=DOM_STRING then
      operstr:=l[1]: 
    else
      operstr:=expr2text(l[1]):
    end_if:
    if operstr="declarem" then
      dcloutput:=[op(eval(dcloutput)),["declare",l[2],l[3]]]:
      res:=[]:
    elif operstr="commonm" then
      comoutput:=[op(eval(comoutput)),["common",l[2],l[3]]]:
      res:=[]:
    elif operstr="programm" or operstr="subroutinem" or operstr="functionm" then
      error("You cannot nest a PROGRAM, FUNCTION or SUBROUTINE"):
    elif macmacro[operstr] = TRUE then
      opereff:=slot(macrofort,operstr):
      ([res,dcloutput,comoutput]):=
             genFormdcl(opereff(op(l,2..nops(l))),dcloutput,comoutput):
      else 
        res:=null():
        for i in l do
          ([temp,dcloutput,comoutput]):=genFormdcl(i,dcloutput,comoutput):
          res:=res,temp:
        end_for:
        res:=[res]:
      end_if:
    else 
      res:=l:
  end_if:
[res,dcloutput,comoutput]:
end_proc:

// code generation when all macros have been expanded 
macgenFor := proc(l,Level)
  local e,len,opereff,operstr;
begin
  for e in l do 
    if e <> [] then
      if contains({DOM_IDENT,DOM_STRING,"_index",DOM_ARRAY},type(e)) then
        return(macgenFor([l],Level)) 
      else 
        if domtype(e[1])=DOM_STRING then
          operstr:=e[1]:
        else
          operstr:="":
        end_if:
      end_if:

     if operstr = "nextm" or operstr = "breakm" then
         error("NEXTM or BREAKM must only appear in a loop"):
     elif type(e[1])=DOM_LIST then
         macgenFor(e,Level+2);
     elif contains({"call","close","comment","common","continue","declare","do","else","end","endif","equal","format","function","goto","ifgoto","open","parameter","program","read","return","subroutine", "write","if_goto","if_then","auto_comment","matrixm","untilm","whilem","writem","ifthenelsem","ifthenm","openm","readm","programm"},operstr) then
         len:=myLength(operstr):
         if mysubstring(operstr,len,len) <> "m" or operstr="program" then
          operstr:=_concat(operstr,"f"):
         end_if:
         opereff:=slot(macrofort,operstr):
         opereff(Level,op(e,2..nops(e))): 
      else
         error(_concat(operstr," is not a macrofort operation")); 
      end_if;
    end_if;
  end_for
end_proc:

//**************************************************************************
// ELEMENTARY INSTRUCTIONS                                                    
//**************************************************************************

// for internal use only 
macrofort::auto_commentf := proc(Level,fstring)
begin
  if comment then
    fprint(Unquoted,fd,_concat("c",space(Level+5),fstring));
  end_if:
end_proc:

// [EQUALF,VARIABLE,EXPRESSION]              
// [EQUALF,[[VARIABLE,EXPRESSION],...]]      
macrofort::equalf := proc(Level,variable,expression) 
  local res,res2,l,v,temp;
begin
  res:=null(): res2:=null():
  if type(variable)=DOM_LIST then
    if optimized then
      for l in variable do
        temp:=m_fortrans(l[1]):
        if domtype(temp)=DOM_STRING then temp:=text2expr(temp): end_if:
        res:=res,temp=l[2]:
      end_for:
    else
      for l in variable do
        res:=res,m_fortrans(l[1])=l[2]:
      end_for:
    end_if:
    if optimized then 
      res:=generate::optimize([res]):
      for l in res do
        temp:=op(l,1):
        if not(domtype(temp)=DOM_STRING) then temp:=expr2text(temp): end_if:
        res2:=res2,_concat(space(Level),temp)=op(l,2):
      end_for:
      m_statement([res2]):
    else 
      for l in [res] do
        res2:=res2,_concat(space(Level),op(l,1))=op(l,2):
      end_for:
      m_Fortran([res2]):
    end_if
  else
    v:= _concat(space(Level),m_fortrans(variable)):
    m_Fortran([v=expression]):
  end_if
end_proc:

// [returnF] 
macrofort::returnf := proc(Level)
begin
  fprint(Unquoted,fd,_concat(space(Level+6),"return"));
end_proc:

// [ENDF]    
macrofort::endf := proc(Level)
begin
  fprint(Unquoted,fd,_concat(space(Level+6),"end"));
end_proc:

// [COMMENTF,CHAINE]  
macrofort::commentf := proc(Level,chaine)
begin
  fprint(Unquoted,fd,_concat("c",space(Level+5),chaine));
end_proc:

// [FORMATF,LABEL,LIST]   
macrofort::formatf := proc(Level,label,list)
begin
  maclprint(_concat(" ",get_flabel(label),
           space(Level+1),"format","(",
           listcom(list),")"))
end_proc:

// [READF,UNIT,LABEL,LIST]  
macrofort::readf := proc(Level,unit,label,list)
begin
  maclprint(_concat(space(Level+6),"read(",unit,",",
           get_flabel(label),") ",
           listcom(list)))
end_proc:

// [WRITEF,UNIT,LABEL,LIST]  
macrofort::writef := proc(Level,unit,label,list)
begin
  maclprint(_concat(space(Level+6),"write(",unit,",",
           get_flabel(label),") ",
           listcom(list)))
end_proc:
// [OPENF,UNIT,FILE,STATUS]  
macrofort::openf := proc(Level,unit,file,status)
begin
  maclprint(_concat(space(Level+6),"open(unit=",unit,
           ",file='",file,"',status='",status,"')"))
end_proc:

// [CLOSEF,UNIT]  
macrofort::closef := proc(Level,unit)
begin
  maclprint(_concat(space(Level+6),"close(",unit,")"))
end_proc:

// [IF_GOTO_F,CONDITION,LABEL]  
macrofort::if_gotof := proc(Level,condition,label)
begin
  maclprint(_concat(space(Level+6),"if (",
           parse_condition(condition),") goto ",
           get_label(label)))
end_proc:

// [IF_THEN_F,CONDITION]  
macrofort::if_thenf := proc(Level,condition)
begin
  maclprint(_concat(space(Level+6),"if (",
           parse_condition(condition),") then"))
end_proc:

// [ELSEF]  
macrofort::elsef := proc(Level)
begin
  fprint(Unquoted,fd,_concat(space(Level+6),"else"));
end_proc:

// [ENDIFF]  
macrofort::endiff := proc(Level)
begin
  fprint(Unquoted,fd,_concat(space(Level+6),"endif"));
end_proc:

// [GOTOF,LABEL]   
macrofort::gotof := proc(Level,label)
begin
  fprint(Unquoted,fd,_concat(space(Level+6),"goto ",get_label(label)));
end_proc:

// [DOF,LABEL,INDEX,STARTI,ENDI,STEP]   
// [DOF,LABEL,INDEX,STARTI,ENDI]        
macrofort::dof := proc(Level,label,index,starti,endi)
begin
   if args(0) = 6 then
    maclprint(_concat(space(Level+6),"do ",
             get_label(label),", ",index,"=",
             m_fortrans(starti),",",
             m_fortrans(endi),",",
             m_fortrans(args(args(0))))):
  else
    maclprint(_concat(space(Level+6),"do ",
             get_label(label),", ",index,"=",
             m_fortrans(starti),",",
             m_fortrans(endi)))
  end_if:
end_proc:

// [CONTINUEF,LABEL]   
macrofort::continuef := proc(Level,label)
begin
  fprint(Unquoted,fd,_concat(" ",get_label(label),
           space(Level+1),"continue"));
end_proc:

// [PROGRAMF,NAME]     
macrofort::programf := proc(Level,fname)
begin
  fprint(Unquoted,fd,_concat(space(Level+6),"program ",fname));
end_proc:

// [SUBROUTINEF,NAME,LIST]     
macrofort::subroutinef := proc(Level,fname,flist)
begin
  maclprint(_concat(space(Level+6),"subroutine ",fname,"(",
           listcom(flist),")"))
end_proc:

// [FUNCTIONF,TYPE,NAME,LIST]   
macrofort::functionf := proc(Level,ftype,fname,flist)
begin
  maclprint(_concat(space(Level+6),ftype," function ",fname,"(",
           listcom(flist),")"))
end_proc:

// [CALLF,NAME,LIST]    
macrofort::callf := proc(Level,fname,flist)
begin
  maclprint(_concat(space(Level+6),"call ",fname,"(",
           listcom(flist),")")):
end_proc:

// [COMMONF,NAME,LIST]   
macrofort::commonf := proc(Level,fname,flist)
begin
  maclprint(_concat(space(Level+6),"common/",fname,"/",
           listcom(flist)))
end_proc:

// [DECLAREF,TYPE,LIST]    
macrofort::declaref := proc(Level,ftype,flist)
begin
  maclprint(_concat(space(Level+6),ftype," ",
           listcom(flist)))
end_proc:

// [PARAMETERF,LIST]     
macrofort::parameterf := proc(Level,flist)
begin
  maclprint(_concat(space(Level+6),"parameter (",
           listcomp(flist),")"))
end_proc:

//***********************************************************************
// MACRO INSTRUCTIONS                                                      
//***********************************************************************

// [MATRIXM,VARIABLE,MATRIX]  
// MATRIXM is a macro instruction for FORTRAN but not for MACROFORT        
// we put "Level+1" for the case of Level = 0                              
macrofort::matrixm := proc(Level,variable,matrix)
local ABlist;
begin
  ABlist := [arrayexpand(variable,matrix)]:
  m_Fortran(ABlist):
end_proc:

// [DOM,INDEX,STARTI,ENDI,STEP,DO_LIST]  
// [DOM,INDEX,STARTI,ENDI,DO_LIST]       
macmacro["do_m"] := TRUE:
macrofort::do_m := proc(index,starti,endi,l)
//global macnlab: 
local breaklabel, dolist, dolabel, isbreak, isnext, res, fstep;
begin
  // isnext needs to be deleted for 1.5.0 
  delete isnext:
  if args(0) = 5 then
    dolist := args(5):
    fstep := l:
  else
    dolist := l:
    fstep := FALSE:
  end_if:
  dolabel := text2expr(_concat("m_label",macnlab)):
  sysassign(macnlab,macnlab+1):
  breaklabel := text2expr(_concat("m_label",macnlab)):
  sysassign(macnlab,macnlab+1):
  isbreak := FALSE:
  ([dolist,isbreak,isnext]):=
             macnextbreak(dolist,isbreak,isnext,breaklabel,dolabel):
  res := [["auto_comment",""]]:
  if fstep = FALSE then
    res := [res,["do",dolabel,index,starti,endi]]
  else res := [res,["do",dolabel,index,starti,endi,fstep]]
  end_if:
  res := [op(res),maclist(dolist),["continue",dolabel]]:
  if isbreak then
    res := [op(res),["continue",breaklabel]]:
  else sysassign(macnlab,macnlab-1): end_if;
  res := [op(res),["auto_comment",""]]:
end_proc:

// [NEXTM], [BREAKM]  
// generation of break and next into loops   
 macnextbreak:= proc(l,isbreak,isnext,breaklabel,nextlabel)  
  local i,oper,operstr,temp,ltemp,isbreaktemp,isnexttemp;
begin
  isbreaktemp:=isbreak:
  isnexttemp:=isnext:
  if l = [] then return([[],isbreaktemp,isnexttemp]) end_if:
  if type(l)=DOM_LIST then
      if domtype(l[1])=DOM_STRING then
       operstr:=l[1]: 
       // oper:=text2expr(operstr);
      else
       oper:=l[1]: operstr:=expr2text(oper);
      end_if:
      if operstr="do_m" or operstr="whilem" or operstr="untilm" 
      then 
       [l,isbreaktemp,isnexttemp]:
      elif operstr = "nextm" then
        isnexttemp := TRUE:
        [["goto",nextlabel],isbreaktemp,isnexttemp]:
      elif operstr = "breakm" then
        isbreaktemp := TRUE:
        [["goto",breaklabel],isbreaktemp,isnexttemp]:
      else 
      ltemp:=null():
      for i from 1 to nops(l) do
        ([temp,isbreaktemp,isnexttemp]):=
          macnextbreak(op(l,i),isbreaktemp,isnexttemp,breaklabel,nextlabel):
        ltemp:=ltemp,temp:
      end_for:
      [[ltemp],isbreaktemp,isnexttemp]: 
      end_if
  else [l,isbreaktemp,isnexttemp]:
  end_if
end_proc:

// [IF_THEN_ELSE_M,CONDITION,THEN_LIST,ELSE_LIST]  
macmacro["if_then_else_m"] := TRUE:
macrofort::if_then_else_m := proc(condition,then_list,else_list)
  local res;
begin
  res := [["if_then",condition],maclist(then_list)]:
  if else_list <> [] then
    res := [op(res),["else"],maclist(else_list),["endif"]]
  else res := [op(res),["endif"]]
  end_if:
  res
end_proc:

// [IF_THEN_M,CONDITION,THEN_LIST]  
macmacro["if_then_m"] := TRUE:
macrofort::if_then_m := proc(condition,then_list)
begin
  [["if_then",condition],maclist(then_list),["endif"]]
end_proc:

// [WHILEM,CONDITION,INIT_LIST,WHILE_LIST,MAX_WHILE]   
// [WHILEM,CONDITION,INIT_LIST,WHILE_LIST]   
macmacro["whilem"] := TRUE:
macrofort::whilem := proc(condition,init_list,while_list)
// global macnlab,macnwhile 
  local  breaklabel,isbreak,isnext,slabel,max_while,mxntl,
         nwhile,whf,inun,templist;
begin
  if args(0) = 4 then max_while:=args(4)
  else max_while:=FALSE end_if:
  slabel:=text2expr(_concat("m_label",macnlab)):
  sysassign(macnlab, macnlab + 1):
  breaklabel := text2expr(_concat("m_label",macnlab)):
// macnlab := macnlab:    
  sysassign(macnlab,macnlab+1) : // bug correction reported by Claude Gomez 
  isbreak := FALSE: isnext:=FALSE;
  if max_while = FALSE then inun:=null():
  else 
    mxntl:=text2expr(_concat("maxwhile",macnwhile)):
    nwhile:=text2expr(_concat("nwhile",macnwhile)):
    inun:=[["equal",mxntl,max_while],
           ["equal",nwhile,0]]:
  end_if:
  whf:=[["auto_comment"," "],
        ["auto_comment",
          _concat("WHILE  (",expr2text(condition),
            ") DO <WHILE_LIST> (",macnwhile,")")],
        ["auto_comment"," "]]:
  if (init_list <> []) or (max_while <> FALSE) then
    whf:=[op(whf),["auto_comment","WHILE LOOP INITIALIZATION"]]:
    if (init_list = []) or type(init_list[1])=DOM_LIST then
      whf:=[op(whf),[inun,op(init_list)]] 
    else whf:=[op(whf),[inun,init_list]] end_if:
  end_if:
  whf:=[op(whf),
       ["auto_comment"," "],
       ["auto_comment","WHILE LOOP BEGINNING"],
       ["continue",slabel],
       ["auto_comment"," "],
       ["auto_comment","WHILE LOOP TERMINATION TESTS"],
       ["if_then",condition]]:
//  addition to fix mapping: 
       ([templist,isbreak,isnext]):=
               macnextbreak(while_list,isbreak,isnext,breaklabel,slabel):
  if max_while <> FALSE then
    whf:=[op(whf),
         [["if_then",mxntl >= nwhile],
         [["auto_comment"," "],
          ["auto_comment","NEW LOOP ITERATION"],
          ["equal",nwhile,nwhile+1],
          ["auto_comment"," "],
          ["auto_comment","<WHILE_LIST>"]],
         maclist(templist),
         ["goto",slabel],
         ["else"],
         [["auto_comment"," "],
          ["auto_comment","WHILE LOOP TERMINATION :"],
	  ["auto_comment","BYPASSING THE MAXIMUM ITERATION NUMBER"],
          ["writem",outputf,[_concat("' ",mxntl," '")],[]]],
         ["endif"]]]:
  else
    whf:=[op(whf),
         [["auto_comment"," "],
          ["auto_comment","NEW LOOP ITERATION"],
          ["auto_comment"," "],
          ["auto_comment","<WHILE_LIST>"]],
         maclist(templist),
         ["goto",slabel]]:
  end_if:
  whf:=[op(whf),
       ["auto_comment"," "],
       ["auto_comment","NORMAL WHILE LOOP TERMINATION"],
       ["endif"]]:
  if isbreak then
    whf := [op(whf),["continue",breaklabel]]:
  else sysassign(macnlab,macnlab-1): end_if;
  whf := [op(whf),
          ["auto_comment",_concat("WHILE LOOP END (",macnwhile,")")]]:
  sysassign(macnwhile,macnwhile+1):
  whf:
end_proc:

// [UNTILM,CONDITION,INIT_LIST,UNTIL_LIST,MAX_UNTIL]   
// [UNTILM,CONDITION,INIT_LIST,UNTIL_LIST]             
macmacro["untilm"] := TRUE:
macrofort::untilm := proc(condition,init_list,until_list)
// global macnlab,macnuntil: 
  local breaklabel,isbreak,isnext,nextlabel,slabel,max_until,mxntl,
        nuntil,unf,inun,templist;
begin
  if args(0) = 4 then max_until:=args(4)
  else max_until:=FALSE end_if:  
  slabel:=text2expr(_concat("m_label",macnlab)):
  sysassign(macnlab, macnlab + 1):
  breaklabel := text2expr(_concat("m_label",macnlab)):
  sysassign(macnlab,macnlab+1):
  isbreak := FALSE:
  nextlabel := text2expr(_concat("m_label",macnlab)):
  sysassign(macnlab,macnlab+1):
  isnext := FALSE:
  if max_until = FALSE then inun:=null()
  else
    mxntl:=text2expr(_concat("maxuntil",expr2text(macnuntil))):
    nuntil:=text2expr(_concat("nuntil",expr2text(macnuntil))):
    inun:=[["equal",mxntl,max_until],
           ["equal",nuntil,0]]:
  end_if:
  unf:=[["auto_comment"," "],
        ["auto_comment",
          _concat("DO <UNTIL_LIST> UNTIL (",
            expr2text(condition),") (",macnuntil,")")],
        ["auto_comment"," "]]:
  if (init_list <> []) or (max_until <> FALSE) then
    unf:=[op(unf),["auto_comment","UNTIL LOOP INITIALIZATION"]]:
    if (init_list = []) or type(init_list[1])=DOM_LIST then
      unf:=[op(unf),[inun,op(init_list)]] 
    else unf:=[op(unf),[inun,init_list]] end_if:
  end_if:
  unf:=[op(unf),
       ["auto_comment"," "],
       ["auto_comment","UNTIL LOOP BEGINNING"],
       ["continue",slabel]]:
  if max_until <> FALSE then
       ([templist,isbreak,isnext]):=
               macnextbreak(until_list,isbreak,isnext,breaklabel,nextlabel):
    unf:=[op(unf),
         ["equal",nuntil,nuntil+1],
         ["auto_comment"," "],
         ["auto_comment","<UNTIL_LIST>"],
         maclist(templist),
         ["auto_comment"," "],
         ["auto_comment","UNTIL LOOP TERMINATION TESTS"]]:
    if (isnext) then
      unf:=[op(unf),["continue",nextlabel]]:
    end_if:
    unf:=[op(unf),
         ["if_then",["NOT",condition]],
         [["if_then",mxntl >= nuntil],
          [["auto_comment"," "],
           ["auto_comment","NEW LOOP ITERATION"],
           ["goto",slabel]],
          ["else"],
          [["auto_comment"," "],
           ["auto_comment","UNTIL LOOP TERMINATION :"],
	   ["auto_comment","BYPASSING THE MAXIMUM ITERATION NUMBER"],
           ["writem",outputf,[_concat("' ",mxntl," '")],[]]],
          ["endif"]]]:
  else
       ([templist,isbreak,isnext]):=
                macnextbreak(until_list,isbreak,isnext,breaklabel,nextlabel):
    unf:=[op(unf),
         ["auto_comment"," "],
         ["auto_comment","<UNTIL_LIST>"],
         maclist(templist),
         ["auto_comment"," "],
         ["auto_comment","UNTIL LOOP TERMINATION TESTS"]]:
    if (isnext) then
      unf:=[op(unf),["continue",nextlabel]]:
    end_if:
    unf:=[op(unf),
         ["if_then",["NOT",condition]],
         [["auto_comment"," "],
          ["auto_comment","NEW LOOP ITERATION"],
          ["goto",slabel]]]:
  end_if:
  unf:=[op(unf),
       ["auto_comment"," "],
       ["auto_comment","NORMAL UNTIL LOOP TERMINATION"],
       ["endif"]]:
  if isbreak then
    unf := [op(unf),["continue",breaklabel]]:
  end_if:
  unf := [op(unf),
          ["auto_comment",_concat("UNTIL LOOP END (",macnuntil,")")]]:
  sysassign(macnuntil,macnuntil+1):
  unf:
end_proc:

// [PROGRAMM,NAME,BODY_LIST]  
macmacro["programm"] := TRUE:
macrofort::programm := proc(fname,body_list)
  local com,dcl,l,temp;
begin
  com:=[]:
  dcl:=[]:
  ([l,dcl,com]):=genFormdcl(body_list,dcl,com):
  if type(l)="_exprseq" then l:=[l] end_if:
   temp:=fname:
   if not(domtype(temp)=DOM_STRING) then temp:=expr2text(temp): end_if;
  [["auto_comment"," "],["auto_comment",_concat("MAIN PROGRAM ",temp)],
   ["auto_comment"," "],
   ["program",fname],dcl,com,maclist(l),["end"]]
end_proc:

// [SUBROUTINEM,NAME,LIST,BODY_LIST]  
macmacro["subroutinem"] := TRUE:
macrofort::subroutinem := proc(fname,list,body_list)
  local com,dcl,l;
begin
  com:=[]:
  dcl:=[]:
  ([l,dcl,com]):=genFormdcl(body_list,dcl,com):
  if type(l)="_exprseq" then l:=[l] end_if:
  [["auto_comment"," "],["auto_comment",_concat("SUBROUTINE ",fname)],
   ["auto_comment"," "],
   ["subroutine",fname,list],dcl,com,maclist(l),["end"]]
end_proc:

// [FUNCTIONM,TYPE,NAME,LIST,BODY_LIST]  
macmacro["functionm"] := TRUE:
macrofort::functionm := proc(ftype,fname,flist,body_list)
  local com,dcl,l;
begin
  com:=[]:
  dcl:=[]:
  ([l,dcl,com]):=genFormdcl(body_list,dcl,com):
  if ftype(l)="_exprseq" then l:=[l] end_if:
 [["auto_comment"," "],["auto_comment",_concat("FUNCTION ",fname)],
   ["auto_comment"," "],
   ["function",ftype,fname,flist],dcl,com,maclist(l),["end"]]
end_proc:

// [COMMONM,NAME,LIST]  
// [DECLAREM,NAME,LIST] 

// [READM,UNIT,FORMAT_LIST,VAR_LIST]  
macmacro["readm"] := TRUE:
macrofort::readm := proc(unit,format_list,var_list)
//global macnflabel: 
begin
  sysassign(macnflabel,macnflabel+1):
  [["read",unit,2000 + macnflabel,var_list],
   ["format",2000 + macnflabel,format_list]]
end_proc:

// [WRITEM,UNIT,FORMAT_LIST,VAR_LIST]  
macmacro["writem"] := TRUE:
macrofort::writem := proc(unit,format_list,var_list)
//global macnflabel: 
begin
  sysassign(macnflabel,macnflabel+1):
  [["write",unit,2000 + macnflabel,var_list],
   ["format",2000 + macnflabel,format_list]]
end_proc:

// [OPENM,UNIT,FILE,STATUS,BODY_LIST]  
macmacro["openm"] := TRUE:
macrofort::openm := proc(unit,file,status,body_list)
begin
  [["open",unit,file,status],body_list,["close",unit]]
end_proc:

macmacro["do_m"]           := TRUE:
macmacro["if_then_else_m"] := TRUE:
macmacro["if_then_m"]      := TRUE:
macmacro["whilem"]         := TRUE:
macmacro["untilm"]         := TRUE:
macmacro["programm"]       := TRUE:
macmacro["subroutinem"]    := TRUE:
macmacro["functionm"]      := TRUE:
macmacro["readm"]          := TRUE:
macmacro["writem"]         := TRUE:
macmacro["openm"]          := TRUE:


//**************************************************************************
//                                                                            
//      M_FORTRAN function (modification of MAPLE FORTRAN function)           
//                                                                            
//**************************************************************************

m_Fortran := proc(x) 
local statseq;
begin
        if contains({DOM_BOOL,"_equal","_unequal","_less","_leequal",
            "_not","_and","_or"},domtype(x)) then statseq := [hold(m_boolean)(x)] 
        elif optimized then statseq := generate::optimize(x)
        elif testtype(x,
        Type::ListOf(Type::Equation(Type::Union(DOM_IDENT,"_index",DOM_STRING),
        Type::Arithmetical)))
             then statseq := x:
        elif testtype(x,Type::ListOf(Type::Equation("function",Type::Arithmetical)))
             then statseq := x:
	elif domtype(x)=DOM_STRING then statseq := [x]:
	elif testtype(x,Type::Arithmetical) then statseq := [x]:
	else error(_concat("invalid argument ",expr2text(x))):
	end_if:
        m_statement(statseq): 
	return()
end_proc:

m_fortrans := proc(x) local statseq;
//global _istr: 
begin
  sysassign(_istr, TRUE):
        //check Type Boolean
        if contains({DOM_BOOL,"_equal","_unequal","_less","_leequal",
            "_not","_and","_or"},domtype(x)) then statseq := [hold(m_boolean)(x)] 
        elif testtype(x,Type::ListOf(Type::Equation(DOM_IDENT,Type::Arithmetical)))
             then statseq := x:
	elif type(x)=DOM_ARRAY then statseq := [x]:
 	elif domtype(x)=DOM_STRING 
             then statseq := [x]:
 	elif testtype(x,Type::Arithmetical) 
             then statseq := [x]:
	else error(_concat("invalid argument ",expr2text(x))):
	end_if:
        m_statement(statseq):
  sysdelete(_istr):
  mysubstring(_buffer,6,myLength(_buffer)):
end_proc:

writeln := proc()
begin
  if _istr <> TRUE then fprint(Unquoted,fd,_buffer); end_if:  
end_proc:

m_write := proc(e) 
local l,d;
//global _buffer: 
begin
   if _istr = TRUE then
     sysassign(_buffer, _concat("",_buffer,e)): 
   else
	l := myLength(e):
	d := myLength(_buffer) + l - 72:
	if d <= 0 then 
           sysassign(_buffer, _concat("",_buffer,e)): return() 
        end_if;
	sysassign(_buffer, _concat(_buffer,mysubstring(e,1,l-d))):
         fprint(Unquoted,fd,_buffer);
	sysassign(_buffer, _concat("     #",mysubstring(e,l-d+1,l))):
  end_if:
end_proc:

arrayexpand := proc(A,B)
local l,i;
begin
 if not (contains({DOM_IDENT,"_index",DOM_ARRAY},domtype(A)) 
    and domtype(B)=DOM_ARRAY) then return(A,B) end_if:
     l:=Indices(B): 
     A[op(op(l,i))]=B[op(op(l,i))]$i=1..nops(l):  
end_proc:

m_statement := proc(s) local k;
//global _buffer:
begin
for k in s do
	if _istr <> TRUE then 
          sysassign(_buffer, "      ");
        else 
          sysassign(_buffer, "ISTR "):
        end_if:
        if testtype(k,"function") and op(k,0)=hold(m_boolean) then
	     m_expression(op(k,1))
 	elif domtype(k)=DOM_STRING then m_expression(k): 
 	elif testtype(k,Type::Arithmetical) then m_expression(k): 
	elif type(k)="_equal" then
             if type(op(k,1))="_index" then m_write("  "): end_if:
	     m_expression(op(k,1)):
	     m_write(" = "): 
	     m_expression(op(k,2)):
	else 
             error("not implemented yet"):
	end_if:
	writeln():
end_for:
end_proc:

m_expression := proc(e) local k,p,s,es,lsign;
begin
case e
 of PI do 
 of EULER do 
 of E do
 of CATALAN do
    m_write(m_float(float(e))): return():
 of TRUE do m_write(".true."): return():
 of FALSE do m_write(".false."): return():
end_case;

case type(e)
of DOM_FLOAT do m_write(m_float(float(e))): return():
of DOM_IDENT do of DOM_STRING do m_write(e): return(): 
of DOM_INT do m_write(m_integer(e)): return():
of "_mult" do
        es:=sortmult(e); 
//       fix for difference in convention  
        lsign:=op(es,1); 
	if lsign + 1 = 0 then
		m_write("-"):
		p := nops(e) = 2 and m_precedence(lsign) <= 50:
		if p then m_write("(") end_if:
		m_expression(_mult(op(subsop(es,1=1)))):
		if p then m_write(")") end_if:
		return():
	elif domtype(lsign)=DOM_RAT and (specfunc::abs(op(lsign,1))=1) then
		if lsign < 0 then m_write("-") end_if:
		p := nops(e) = 2 and m_precedence(lsign) <= 50:
		if p then m_write("(") end_if:
		m_expression(_mult(op(subsop(es,1=1)))):
		if p then m_write(")") end_if:
		m_write(_concat("/",(op(lsign,2)))):
		return():
	end_if:
 	p := m_precedence(es,1): 
 	if p then m_write("(") end_if:
 	m_expression(lsign):  
	if p then m_write(")") end_if: 
	for k from 2 to nops(op(es)) do 
		if type(op(es,k))="_power" and 
                contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(op(op(es,k),2)))
			and sign(op(op(es,k),2)) = -1 
		then s := op(op(es,k),1)^(-op(op(es,k),2)): 
                  m_write("/")
		else s := op(es,k): m_write("*")
		end_if:
		p := m_precedence(s) < m_precedence(es):
		if p then m_write("(") end_if:
		m_expression(s):
		if p then m_write(")") end_if:
	end_for:
        return(): 
of "_power" do
        if (domtype(op(e,2))=DOM_RAT or domtype(op(e,2))=DOM_INT) and op(e,2) < 0 then
		p := m_precedence(op(e,1)):
		m_write("1/"):
		if p < 70 then m_write("(") end_if:
		m_expression(op(e,1)^(-op(e,2))):
		if p < 70 then m_write(")") end_if:
	elif domtype(op(e,1))=DOM_IDENT and domtype(op(e,2))=DOM_INT then
		m_write(_concat("",op(e,1),"**",op(e,2))):
	elif domtype(op(e,2))=DOM_RAT and op(op(e,2),2)=2 then
		s := op(e,1)^op(op(e,2),1):
		//m_expression(sqrt(s))
		m_expression(SQRT(s)):
	else	p := m_precedence(e,1):
		if p then m_write("(") end_if:
		m_expression(op(e,1)):
		if p then m_write(")") end_if:
		m_write("**"):
		p := m_precedence(e,2):
		if p then m_write("(") end_if:
		m_expression(op(e,2)):
		if p then m_write(")") end_if:
	end_if:
        return():
of "_plus" do
        es:= sortplus(e); 
	p := m_precedence(es,1): 
	if p then m_write("(") end_if:
	m_expression(op(es,1)):
	if p then m_write(")") end_if:
	for k from 2 to nops(es) do
                lsign:=findsign(op(es,k)):
                if not(contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(lsign)) 
                   and lsign < 0)
		then m_write("+") end_if:
		p := m_precedence(es,k):
		if p then m_write("(") end_if:
		m_expression(op(es,k)):
		if p then m_write(")") end_if:
	end_for:
        return():

of "_index" do
//  fix for indexed arrays      
	m_expression(op(e,1)):
	m_write("("):
 	m_expression(op(e,2)): 
        if nops(e) > 2 then
	for k from 3 to nops(e) do
		m_write(","):
		m_expression(op(e,k))
	end_for:
        end_if:
	m_write(")"):
        return():
of DOM_RAT do
	m_expression(float(op(e,1))):
	m_write("/"):
	m_expression(float(op(e,2))):
        return():
of "_not" do
	m_write(" .not. "):
	p := m_precedence(e,1):
	if p then m_write("(") end_if:
	m_expression(op(e,1)):
	if p then m_write(")") end_if:
        return():
end_case;

if testtype(e,"function") then
        m_function(e):
 elif contains({DOM_BOOL,"_equal","_unequal","_less","_leequal",
            "_and","_or"},domtype(e)) then
	p := m_precedence(e,1):  
	if p then m_write("(") end_if:
	m_expression(op(e,1)):
	if p then m_write(")") end_if:
        case type(e)
         of "_or" do m_write(" .or. "): break
	 of "_and" do m_write(" .and. "): break
	 of "_equal" do m_write(" .eq. "): break
	 of "_less" do m_write(" .lt. "): break
	 of "_leequal" do m_write(" .le. "): break
	 of "_unequal" do m_write(" .ne. "): break
        end_case:
	p := m_precedence(e,2):
	if p then m_write("(") end_if:
	m_expression(op(e,2)):
	if p then m_write(")") end_if:
else 
error("unable to translate"); 
end_if:

end_proc:


m_precedence := proc(e,k)
begin
case args(0) 
 of 1 do
    if (domtype(e)=DOM_INT or domtype(e)=DOM_FLOAT) and e > 0 then return(99):
    elif (domtype(e)=DOM_INT or domtype(e)=DOM_RAT) then return(70): 
    end_if:
    case type(e) 
      of DOM_IDENT  do return(99):
      of "_mult"   do return(70):
      of "_plus"   do return(50):
      of "function" do
           case op(e,0) 
             of MULT do return(70):
             of PLUS do return(50):
             otherwise  return(99):
           end_case:
           break:
      of "_index"   do return(99):
      of "_power"   do return(80):
      of DOM_RAT    do return(70): 
      of DOM_INT    do return(60): 
      of DOM_FLOAT  do return(60):
      of "_equal"   do return(40):
      of "_less"    do return(40):
      of "_leequal" do return(40):
      of "_unequal" do return(40):
      of "_not"     do return(30):
      of "_and"     do return(20):
      of "_or"      do return(10):
    end_case:
    //  addition for sorted products 
    if testtype(e,"function") then return(99);
    else return(0):
    end_if
//of 2 do 
otherwise
   if type(e)="_mult" and k=nops(e) and 
   contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(op(e,k))) then
     return(FALSE): 
   end_if;
    if k=1 then
     case type(e)
      of "function" do
        if op(e,0)=MULT and contains({DOM_RAT,DOM_INT,DOM_FLOAT},domtype(op(e,1))) 
        then 
          return(FALSE):
        end_if:
        next;
      of "_power" do
        if type(op(e,1))="_power" then return(TRUE): end_if:
      next;
      otherwise return(m_precedence(e) > m_precedence(op(e,1)));
     end_case:
    else return(m_precedence(e) > m_precedence(op(e,k)));
    end_if:
end_case;

end_proc:

m_float := proc(f) local mantissa,exponent,letter,preend_ifx;
begin
	if f = 0
	then mantissa := 0: exponent := 0
	else ([mantissa,exponent]):=represent(f): end_if;
	if exponent = 0 and mantissa < 1000000 then
                case precision
		of "single" do
                  return(_concat("",mantissa,".0")): 
		of "double" do
                  return(_concat("",mantissa,".D0")): 
		of "quadruple" do
                  return(_concat("",mantissa,".Q0")):
                end_case: 
	end_if:
	case precision 
            of "single" do    letter := "E": break
	    of "double" do    letter := "D": break
	    of "quadruple" do letter := "Q": break
        end_case:
	if mantissa < 0 then
		preend_ifx := "-0.":
		mantissa := -mantissa
	else preend_ifx := "0."
	end_if:
	while irem(mantissa,10) = 0 do
		mantissa := iremm(mantissa,10):
		exponent := exponent+1:
	end_while:
	exponent := exponent+myLength(mantissa):
	_concat("",preend_ifx,mantissa,letter,exponent):
end_proc:

m_integer := proc(e)
begin
  if myLength(e) > 8 then
    m_float(float(e))
  else expr2text(e):
  end_if
end_proc:

m_function := proc(e)
  local f,fstring,k,n,offset;
begin
// fix for functions 
  if type(e)="_index" then offset:=1: else offset:=0; end_if;
  f:=op(e,offset): 
  fstring:=expr2text(f): 
  if fstring="generate::Macrofort::SQRT" or fstring = "SQRT" then fstring := "sqrt": end_if:
  n:=nops([op(e)]): 
  if fstring="abs" or fstring="dabs" or fstring="qabs" then 
    if n <> 1 then 
       error(_concat(fstring," function must have one and only one argument")): 
    end_if:
       m_function_IR1(m_function_name(fstring),op(e,1+offset)):
  elif mycontains(fstring,["sqrt","exp","ln","log","acos","asin","cos","cosh","sin","sinh","tan","tanh"]) then
    if n <> 1 then 
        error(_concat(fstring," function must have one and only one argument")): 
    end_if:
    m_function_R1(m_function_name(fstring),op(e,1+offset)):
  elif fstring="atan" or fstring="dtan" or fstring="qtan" then
    if n=1 then m_function_R1(m_function_name(fstring),op(e,1+offset)):
    elif n=2 then 
        m_function_R2(m_function_name(_concat(fstring,"2")),op(e,1+offset),op(e,2+offset)):
    else 
        error(_concat(fstring," function must have one or two arguments")):
    end_if
  elif fstring="Hold" then 
   m_write("("):   
   m_write(expr2text(op(e))):  
   m_write(")"): 
  else
    m_expression(f):
    m_write("("):
     if n > 0 then
       m_expression(op([op(e)],1+offset)):
       if n > 1 then
	for k from (2+offset) to n do
	  m_write(","):
	  m_expression(op([op(e)],k)):
	end_for:
       end_if
     end_if:
     m_write(")"):
  end_if:
end_proc:

m_function_IR1 := proc(f,a)
begin
// 1 integer or real argument  
  m_write(f):
  m_write("("):
  m_expression(a):
  m_write(")"):
end_proc:

m_function_R1 := proc(f,a)
// 1 real argument            
local va;
begin
  m_write(f):
  m_write("("):
   va:=a:
    if contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(a)) then 
    if iszero(a) then 
       case precision 
         of "single" do va:="0.0E0": break
         of "double" do va:="0.0D0": break
         of "quadruple" do va:="0.0Q0": break
       end_case:
    else va:=float(a) end_if:
  end_if:
  m_expression(va):
  m_write(")"):
end_proc:

m_function_R2 := proc(f,a,b)
// 2 real arguments  
local v;
begin
  m_write(f):
  m_write("("):
  v:=a:
   if contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(a)) then 
    if iszero(a) then 
       case precision 
         of "single" do v:="0.0E0": break
         of "double" do v:="0.0D0": break
         of "quadruple" do v:="0.0Q0": break
       end_case:
    else v:=float(a) end_if:
  end_if:
  m_expression(v):
  m_write(","):
  v:=b:
   if contains({DOM_RAT, DOM_INT, DOM_FLOAT}, domtype(b)) then 
    if iszero(b) then 
       case precision 
         of "single" do v:="0.0E0": break
         of "double" do v:="0.0D0": break
         of "quadruple" do v:="0.0Q0": break
       end_case:
    else v:=float(b) end_if:
  end_if:
  m_expression(v):
  m_write(")"):
end_proc:

m_function_name:=proc(e)
begin
case e
 of "exp"   do return(switchprec("exp","dexp","qexp")):
 of "abs"   do return(switchprec("abs","dabs","qabs")):
 of "acos"  do return(switchprec("acos","dacos","qacos")):
 of "asin"  do return(switchprec("asin","dasin","qasin")):
 of "atan"  do return(switchprec("atan","datan","qatan")):
 of "atan2" do return(switchprec("atan2","datan2","qatan2")):
 of "cos"   do return(switchprec("cos","dcos","qcos")):
 of "sin"   do return(switchprec("sin","dsin","qsin")):
 of "tan"   do return(switchprec("tan","dtan","qtan")):
 of "sinh"  do return(switchprec("sinh","dsinh","qsinh")):
 of "cosh"  do return(switchprec("cosh","dcosh","qcosh")):
 of "ln"    do return(switchprec("log","dlog","qlog")):
 of "log"   do return(switchprec("log","dlog","qlog")):
 of "sqrt"  do return(switchprec("sqrt","dsqrt","qsqrt")):
end_case:
end_proc:

// unalias 
unalias(space):                            
unalias(get_label):                       
unalias(get_flabel):                     
unalias(maclist):                       
unalias(listcomp):                     
unalias(listcom):                     
// following 2 aliases not allowed in 1.4.0 
unalias(maclabel): 
unalias(macflabel): 
unalias(macnlabel): 
unalias(macnflabel):
unalias(macnwhile): 
unalias(macnuntil): 
unalias(parse_condition):        
unalias(parse_condition1):      
unalias(maclprint):            
unalias(genForm):             
unalias(genFormdcl):         
unalias(macgenFor):         
unalias(equalf):          
unalias(macmacro):        
unalias(macnlab): 
unalias(macnextbreak):    
unalias(m_statement):                
unalias(writeln):                   
unalias(arrayexpand):              
unalias(m_expression):            
unalias(m_write):                  
unalias(m_precedence):          
unalias(m_float):              
unalias(m_integer):           
unalias(m_function):         
unalias(m_function_IR1):    
unalias(m_function_R1):    
unalias(m_function_R2):   
unalias(m_function_name): 
unalias(macrofor_string):            
// added routines 
unalias(lextr):                 
unalias(myLength):                   
unalias(mysubstring):               
unalias(mycontains):               
unalias(Indices):                 
unalias(assigned):               
unalias(represent):             
unalias(iremm):                
unalias(findsign):           
unalias(cmp):               
unalias(sortmult):         
unalias(sortplus):        
unalias(switchprec):   

// global buffers 
unalias(_istr):
unalias(_buffer):
// global settings 
unalias(precision):
unalias(optimized):
unalias(outputfile):
unalias(fd):
unalias(inputf): 
unalias(outputf): 
unalias(comment):

unalias(PLUS):
unalias(MINUS):
unalias(SQRT):

//************************************************************************
