/*--
generate::sortSum  --  rearranges the succession of the summands in
                       a sum. The first summand is positive if
                       possible. 

generate::sortSum( SUM )

       SUM  --   a "_plus"- expression

Puts the first nonnegative summand in the first position of
the sum.

>> -a*b*3-c*d*(-2);

                              - 3 a b + 2 c d

>> generate::sortSum(%);

                               2 c d - 3 a b

--*/

generate::isNeg :=
proc(arg1)
  local lastarg;
begin
  if testtype(arg1, "_mult") then
    lastarg := op(arg1, nops(arg1));
    if testtype(lastarg, Type::Real) then
      if lastarg < 0 then
        return(TRUE);
      end_if;
    end_if;
    return(FALSE);
  end_if;
  FALSE;
end_proc:


generate::sortSum :=
proc(SuM)
  local i, list;
begin
  if testargs() then
    if not testtype(SuM, "_plus") then
      error("\"_plus\"-expression expected");
    end_if;
  end_if;
  if nops(SuM) = 1 then
    return(SuM);
  end_if;
  list := [op(SuM)];
  if generate::isNeg( op(list,1)) then
    for i from 2 to nops(list) do
      if not generate::isNeg(op(list, i)) then
        return(hold(_plus) (op(subsop(list, 1=op(list, i), i=op(list, 1)))));
      end_if;
    end_for;
  end_if;
  SuM;
end_proc:

