/*
 groebner::factor(l) takes as input a list of polynomials (for example
	the output of groebner::gbasis) describing a variety, and factors
	the variety if possible.
 Input: a list of polynomials
 Output: a set of list of polynomials
 Example: 
>> l:=[x^2 + y + z - 3,x + y^2 + z - 3,x + y + z^2 - 3]:
>> g:=groebner::gbasis(l,LexOrder):
>> groebner::factor(g);

{[x - 1, y - 1, z - 1], [x + 3, y + 3, z + 3],

                          2       2
   [x + y - 1, - y + z + y  - 2, z  - 2],

                                   2
   [x + z - 1, y + z - 1, - 2 z + z  - 1]}
*/
groebner::factor :=
proc(ll, reord)
  local i,s,p,n,g,_t,q,unk,one,post,unk0,d,T,l;
begin
  if args(0)=2 then
    // second argument must be Reorder
    if args(2) <> hold(Reorder) then
      error("Wrong type of second argument")
    end_if
  else
    reord:= null()
  end_if;
      
  if ll=[] then
    return({[]})
  end_if;
  n:=nops(ll);
  if type(ll[1])<>DOM_POLY then
    // to ensure the order of variables remains the same 
    l:=map(ll,poly,[op(indets(ll,PolyExpr))]);
  else
    l:=ll
  end_if;
  s:={};
  p:=l[n]; // last polynomial 
  _t:=genident("t");
  unk0:=op(p,2); unk:=[_t,op(unk0)]; d:=op(p,3);
  T:=poly(_t,unk,d);
  l:=map(l,poly,unk,d);
  one:=poly(1,unk,d);
  p:=factor(p);
  p:= Factored::convert_to(p,DOM_LIST);
  if nops(p)=1 then
    return({args(1)}) // degree 0 case 
  // elif nops(p)=3 then
    // return({subsop(args(1),n=p[2])}) // one factor only 
  end_if;

  post:=proc()
        begin
          if has(op(args(1),1),_t) then
            null()
          else
            poly(args(1),unk0,d)
          end_if
      end_proc;

  p:=[p[2*i] $ i = 1..nops(p) div 2];
  if type(p[1]) = DOM_POLY then
    p:=map(p, poly, unk, op(p[1], 3))
  end_if;
  for i from 1 to nops(p) do
    q:=_mult(op(subsop(p, i = T))) - one;
    // variables must *not* be reordered, since this would
    // confuse the solver
    g:=groebner::gbasis(append(subsop(l, n = p[i]), q), LexOrder, reord)
;
    g:=map(g, post);
    if nops(g) <> 1 or expr(g[1]) <> 1 then
      s:=s union {g}
    end_if
  end_for;
  s

end_proc:
