// $Date: 2000/11/30 12:14:40 $ $Author: fakler $ $Revision: 1.48 $ 

/*--
intlib::defInt( f,x=a..b, options )

--*/

// tests xrng used to be x=a..b 
intlib::xrngTest:=
  proc(xrng)
    name intlib;
  begin
    if type(xrng) <> "_equal" then
       error("Second argument must be of form x=a..b")
    elif domtype(op(xrng,1)) <> DOM_IDENT then
       error("Integration variable is not an identifier")
    elif type(op(xrng,2)) <> "_range" then
       error("Second argument must be of form x=a..b")
    elif has(op(xrng,2), op(xrng,1)) then
       error("Range depends upon integration variable")
    end_if;
    null()
  end_proc:
    
// main procedure for definite integration
intlib::defInt:=
  proc(f :Type::Arithmetical, xrng) // options
    local prop,C,F,options,x,a,b,f0,x0;
    save _X_; // necessary to make assumption on integration variable
              // I choose _X_ instead of _X as in intlib::int, since this is
              // more efficient if intlib::int is called
  begin
    f0:=f;
    if testargs() then f:=eval(f); intlib::xrngTest(xrng) end_if;
    
    x:=op(xrng,1); a:=op(op(xrng,2),1); b:=op(op(xrng,2),2);
    options:=intlib::getOptions(args(3..args(0)));

//    if has(f,int) then 
      // in that case subs at the end of this procedure may lead to
      // a wrong result
//      return(hold(int)(f,x=a..b,intlib::printOptions(options)))
//    end_if;
    if has(f,abs) then
      // rewriting abs here is necessary since currently e.g. under
      // the assumption assume(x,Type::Interval(0,2*PI)): abs(sin(x)+1/2);
      // would be automatically simplified, but this is bad for integration.
      f:=eval(subs(f,hold(abs)=(u -> sign(u)*u)));
    end_if;
    
    C:=genident("__X");
    if property::hasprop(_X_) then
      // (prop1:=getprop(_X_))<>hold(_X_) then //assume(C,prop1) end_if;
      //assume(C,prop1)  // does'nt work due to an error in assume
      zip([C],[getprop(_X_)],assume);
    end_if;
    delete _X_;
    if property::hasprop(x) then // x has property
      prop:= getprop(x);
      if property::IV::_and(prop,Type::Interval([a,b]))=Type::Interval([a,b])
        then
        //assume(_X_,prop); // properties are consistent
        zip([_X_],[prop],assume);
      else // properties may be inconsistent
        prop:=Type::Interval([a,b]);
        if intlib::printWarningsFlag then
          warning("While integrating, we will assume ".expr2text(x).
                  " has property ".expr2text(prop).
                  " instead of given property ".expr2text(getprop(x)).".");
        end_if;
        //assume(_X_,prop); // does'nt work due to an error in assume
        zip([_X_],[prop],assume);
      end_if;
    else
      //assume(_X_,Type::Interval(a,b));
      zip([_X_],[Type::Interval([a,b])],assume);
    end_if;

    if has(f,int) then 
      // in that case the following subs of this procedure may lead to
      // a wrong result, since different integration variables may become
      // the same
      if getprop(_X_)=Type::Interval([a,b]) then
        // perhaps it is now possible to solve the nested integrals
        x0:=genident("_X_");
        zip([x0],[Type::Interval([a,b])],assume);
        f:=eval(subs(f,x=x0));
        if has(f,int) then
          return(hold(int)(f0,x=a..b,intlib::printOptions(options)))
        end_if;
        f:=eval(subs(f,x0=x));
        delete x0;
      else
        return(hold(int)(f0,x=a..b,intlib::printOptions(options)))
      end_if;
    end_if;

    if x<>_X_ then
      f:=subs(f,[_X_=C,x=_X_]);
      a:=subs(a,[_X_=C,x=_X_]);
      b:=subs(b,[_X_=C,x=_X_]);
    end_if;
    F:=intlib::defInt_intern(f,_X_=a..b,options);
//    if type(F)<>"int" then
    if not has(F,int) then
      F:=subs(F,[_X_=x,C=_X_]);
    else
      F:=hold(int)(f0,xrng,intlib::printOptions(options));
    end_if;
    unassume(C);
    F    
  end_proc:

intlib::defInt_intern :=
  proc(f,xrng)  // options
    local i,integral,g,fac,options,x,b,a; 
  begin
    if testargs() then f:=eval(f); intlib::xrngTest(xrng) end_if;
 
    x:=op(xrng,1); a:=op(op(xrng,2),1); b:=op(op(xrng,2),2);
    options:=intlib::getOptions(args(3..args(0)));
    if testtype(f,Type::PolyExpr(x)) and not options["PrincipalValue"] then 
       // no singularity possible 
       f:=poly(f,[x]);
       f:=_plus(coeff(f,i)*x^(i+1)/(i+1) $ i=0..degree(f));
       return(intlib::int_eval(f,x=b)-intlib::int_eval(f,x=a))
    elif is(b<a)=TRUE then 
       return(-intlib::defInt_intern(f,x=a..b,options))
    else 
      // test if integrand contains special functions that can be treated
      integral:=intlib::specialFunctionDef(f,x,a,b,options);
      if integral<>FAIL then return( integral )
      end_if;
    end_if;
    // first try the lookup method 
    // remove constant factors before lookup method 
    if type(f)="_mult" then
      g:=select(f,has,x); fac:=select(f,not has,x)
    else g:=f; fac:=1
    end_if;
    i:=intlib::defLookup(g,x=a..b,options);
    if i<>FAIL then return(fac*i) end_if;
    userinfo(1,"definite lookup method failed,try to use an antiderivative");
    // then try using an antiderivative 
    intlib::antiderivative(f,x=a..b,options);
  end_proc:

intlib::antiderivative:=
  proc(f :Type::Arithmetical,xrng) // options
    save calledFromWithinDefInt;
    local integral,g,g0,s,y,eps,lr,ia,ib,x,a,b,options,props;
  begin
    
    // first do some tests and extract from arguments
    if testargs() then f:=eval(f); intlib::xrngTest(xrng) end_if;
    
    x:=op(xrng,1); a:=op(op(xrng,2),1); b:=op(op(xrng,2),2);
    options:=intlib::getOptions(args(3..args(0)));

    // set flag to prevent some simplifications done in indefinite integration
    sysassign(calledFromWithinDefInt,TRUE);
    
    // compute the antiderivative
    g:=int(f,x);

    // try to evaluate a sum(....z=RootOf(..))
    g0:=g;
    if type(g)="sum" then
      g0:=intlib::expandSum(g,x);
    elif has(g,sum) then
      g0:=map(g,intlib::expandSum,x);
    end_if;
    // test heuristically, if complexity of g0 is not too big,
    // otherwise exponential growth in memory usage may occur
    if length(g)*intlib::memGrowthFactor > length(g0) then g:=g0 end_if;
    if has(g,I) then
      g:=intlib::simplifyComplexIntegral(g,x,hold(Unique));
    end_if;

    // substitute remaining two-argument arcus tangents
    // otherwise we could run into a bad branch and getting a wrong answer
    if has(g,arctan) then
      g:=eval(subs(g,hold(arctan)=proc(x,y)
                     begin
                       if args(0)=1 then
                         arctan(x)
                       else
                         -I * ln((x+I*y)/(abs(x+I*y)))
                       end_if
                          end_proc));
    end_if;
    if has(g,arg) then
      g:=eval(subs(g,hold(arg)=((x,y)->-I*ln((x+I*y)/(abs(x+I*y))))));
    end_if;

    userinfo(2,"antiderivative is",g);
    if type(g)<>"int" then
      if options["Continuous"] then // g is assumed to be continuous
        //return( intlib::int_eval(g,x=b)-intlib::int_eval(g,x=a) )
        s:={};
        userinfo(2,"antiderivative is assumed to be continuous");
      else
         s:=NIL;
        // discontinuities in a..b
        if hastype(g,"RootOf") then // this would yield an error within discont
          s:=Type::Interval(a,b);
        elif not has(f,dirac) then
          if has(g,I) then traperror((g:=simplify(g);),intlib::simplifyTime);
          end_if;
          userinfo(2,"searching for discontinuities of ",g,"in ",a..b);
          traperror((s:=discont(g,x=a..b, (if has(g,I) then Dom::Complex
                                           else Dom::Real end_if));),
                    intlib::discontTime);
        else s:={};
        end_if;
        if s=NIL and intlib::debugWarningsFlag then
          props:=solvelib::showprops(args());
          if props<>null() then
            warning("error occured in discont(".
                    expr2text(g,x=a..b, (if has(g,I) then Dom::Complex
                                       else Dom::Real end_if)).
                    ") and properties ".expr2text(props));
          else
            warning("error occured in discont(".
                    expr2text(g,x=a..b, (if has(g,I) then Dom::Complex
                                         else Dom::Real end_if)).")")
          end_if;
          s:=Type::Interval(a,b);
        end_if;
      end_if;
      userinfo(2,"potential discontinuities of the antiderivative are",s);
      if type(s)<>DOM_SET then
        if intlib::printWarningsFlag then
          //warning("potential discontinuities of the antiderivative are ".
          //        expr2text(s));
          warning("Found potential discontinuities of the antiderivative.".
                  "\nTry option 'Continuous' or use properties (?assume).");
        end_if;
      else
        if contains(s,a) then
          lr:=intlib::boundaryTest(g,x=a,Right);
          if not lr[1] and intlib::printWarningsFlag then
            warning("Antiderivative is unbounded at lower limit");
          end_if;
          ia:=lr[2];
        else // traperror is necessary due to possible bugs in discont and
             // option Continuous
          ia:=NIL;
          traperror((ia:=intlib::int_eval(g,x=a);));
          if ia=NIL then
            if intlib::printWarningsFlag then
            warning("Antiderivative is discontinuous at lower limit");
            end_if;
            ia:=intlib::boundaryTest(g,x=a,Right)[2];
          end_if;
        end_if;
        if contains(s,b) then
          lr:=intlib::boundaryTest(g,x=b,Left);
          if not lr[1] and intlib::printWarningsFlag then
            warning("Antiderivative is unbounded at upper limit");
          end_if;
          ib:=lr[2];
        else // traperror is necessary due to possible bugs in discont and
             // option Continuous
          ib:=NIL;
          traperror((ib:=intlib::int_eval(g,x=b);));
          if ib=NIL then
            if intlib::printWarningsFlag then
            warning("Antiderivative is discontinuous at upper limit");
            end_if;
            ib:=intlib::boundaryTest(g,x=b,Left)[2];
          end_if;
        end_if;
        integral:=ib-ia;
        s:=s minus {a,b};
        for y in s do
          if options["PrincipalValue"] then // compute principal value 
            eps:= genident("eps");
            integral:=integral
                       +intlib::limit(subs(g,x=y-eps)-subs(g,x=y+eps),
                                      eps=0,Right);
          else
            lr:=intlib::poleLRTest(g,x=y);
            if has(lr,{infinity,-infinity,undefinded}) then
              return(undefined)
            elif lr[1] then integral:=integral+lr[2]-lr[3];
              //else return(hold(int)(f,x=a..b))
              // otherwise there should not be a discontinuity
            end_if;
          end_if
        end_for;
        return(integral)
      end_if
    end_if;
    if a=b then 0 else hold(int)(f,x=a..b,intlib::printOptions(options)) end_if
  end_proc:

intlib::boundaryTest:= // test, if lower or upper bound of given range is
  proc(f,x,dir)        // bounded
    local t,r;
  begin
    userinfo(6,"test, if antiderivative is bounded at ".expr2text(x));
    r:=intlib::limit(f,x,dir);
  //t:=(if has(r,{infinity,-infinity,undefined}) then FALSE else TRUE end_if);
    t:=not(bool(has(r,{infinity,-infinity,undefined})));
    [t,r]
  end_proc:

intlib::poleLRTest:=  // test, if limit's result is correct most probably 
  proc(f,x)           // for inner discontinuous points
    local r,t,s;
  begin
    userinfo(6,"test, if antiderivative is indeed discontinuous at ".
               (s:=expr2text(x);));
    //if type(op(x,2))=stdlib::Infinity then
    if has(op(x,2),{infinity,-infinity}) then
      r:=intlib::limit(f,x);
      r:=[r,r];
    else
      r:=[intlib::limit(f,x,Left),intlib::limit(f,x,Right)]
    end_if;
    //t:=(if has(r,{infinity,-infinity,undefined}) or r[1]<>r[2] then TRUE
    //  else FALSE
    //    end_if);
    t:=bool(has(r,{infinity,-infinity,undefined}) or r[1]<>r[2]);
    userinfo(10, (if t then "is discontinuous at ".s
                 else "not discontinuous at ".s
                 end_if;));
    [t, op(r)]
  end_proc:

// wrapper function for limit which secures that only a valid value
// like an arithmetical expression or a symbolic limit-call is returned.
intlib::limit:=
  proc()
    local lim,props;
  begin
    lim:=NIL;
    traperror((lim:=limit(args())));
    if lim=NIL and intlib::debugWarningsFlag then
      props:=solvelib::showprops(args());
      if props<>null() then
            warning("error occured in limit(".
                    expr2text(args()).
                    ") and properties ".expr2text(props));
      end_if;
      return(hold(limit)(args()));
    end_if;
    case type(lim)
      of Dom::Interval do lim:=undefined; break;
      of DOM_FAIL do lim:=hold(limit)(args()); break;
    end_case;
    lim;
  end_proc:
        

/*--
	intlib::int_eval(f,x=a,dir,typ) 
  
  	evaluates f in x=a with direction dir (Left or Right) and typ=TRUE 
        iff a is a singular point 
--*/
intlib::int_eval := proc(f,x) 
begin
   //if type(op(x,2))=stdlib::Infinity then 
   if has(op(x,2),{infinity,-infinity}) then
       intlib::limit(f,x)
   elif args(0)>=4 and args(4) then 
       intlib::limit(f,x,args(3))
   else 
       eval(subs(f,x))
   end_if
end_proc:
