// $Date: 2000/10/31 13:22:30 $ $Author: fakler $ $Revision: 1.42 $ 

/* intlib::defLookup(f,t=a..b,options) computes int(f,t=a..b,options),
  and return FAIL if unable.
   References: 

   [1] Evaluation of Classes of Definite Integrals Involving
   Elementary Functions via Differentiation of Special Functions,
   by K.O. Geddes, M.L. Glasser, R.A. Moore and T.C. Scott,
   Applicable Algebra in Engineering, Communication and Computing (AAECC)
   volume 1, pages 149-165, 1990.

   [2] Explicit Evaluation of Certain Definite Integrals Involving
       Powers of Logarithms, by K.S. Ko"lbig, JSC 1, 1985, pages 109-114.

   The implementation here first checks for the classes in [1], and then
   uses the formulas in [2] for the special cases not treated in [1].
   All the examples given in [2] (4.1 to 4.9) work.

   Currently only the first two classes in [1] are implemented. The third
   class (generated by the incomplete gamma function) needs the
   Meijer G function, which is not yet implemented in MuPAD.
*/

alias(Match=intlib::match):
alias(Match2=intlib::matchassign):
alias(indep(a,x)=_pattern(hold(a),not has,x)): /* NOTE: same alias is
                                                   defined in lookup.mu */
//alias(indep2(a,x,y)=_pattern(hold(a), ((X)->(not (has(X,x) or bool(X=y)))))):
intlib::indep2:=
proc(a,x,y)
  option hold;
  local x0,y0;
begin
   x0:=context(x); y0:=context(y); 
  _pattern(a, ((X)->(not (has(X,x) or bool(X=y)))));
end_proc:
alias(indep2=intlib::indep2):
alias(_p=_pattern):

intlib::iseven := proc(f,t)
option remember;
begin
   bool(f=eval(subs(f,t=-t)))
end_proc:

intlib::isodd := proc(_f_,_t_)
option remember;
begin
   bool(-_f_=eval(subs(_f_,_t_=-_t_)))
end_proc:

// NOTE: Please do not use the following ugly technique to simulate
//       dynamical scoping within static scoping and to keep user given
//       properties.

intlib::defLookup :=
  proc(f :Type::Arithmetical,trng) // options
    local vl,sl,bl,hl,e,prop,C,t,ta,tb,options;
    save s,F,u,_s,t0,w,a,b,z,_w,c,d,_z,g,i,k,l,m,n,mu,m1,p,nu,m2,_n,r,_X,
         s0,F0,u0,_s0,t00,w0,a0,b0,z0,_w0,c0,d0,_z0,g0,i0,k0,l0,m0,n0,mu0,
         m10,p0,nu0,m20,_n0,r0;
  begin
    if testargs() then f:=eval(f); intlib::xrngTest(trng) end_if;

    t:=op(trng,1); ta:=op(op(trng,2),1); tb:=op(op(trng,2),2);
    options:=intlib::getOptions(args(3..args(0)));

    prop:=getprop(t);
    C:=genident();
    if property::hasprop(_X) then //assume(C,prop1) end_if;
      //assume(C,prop1)  // does'nt work due to an error in assume
      zip([C],[getprop(_X)],assume);
    end_if;
    delete _X;
    if prop<>t then
      if not(property::Fail((prop:=prop and Type::Real))) then
        if prop=Type::Zero then
          prop:=Type::Complex;
          if intlib::printWarningsFlag then
            warning("Cannot integrate when ".expr2text(t).
                    " has property ".expr2text(getprop(t)).".\n".
                    "While integrating, we will assume ".expr2text(t).
                    " has property ".expr2text(prop).".");
          end_if;
        elif is(t,Type::Rational)=TRUE then
          prop:=Type::Real;
          print(Unquoted,"Cannot integrate when ".expr2text(t).
                " has property ".expr2text(getprop(t)).".\n".
                "While integrating, we will assume ".expr2text(t).
                " has property ".expr2text(prop).".")
        end_if;
        //assume(_X,prop) // does'nt work due to an error in assume
        zip([_X],[prop],assume);
      else
        print(Unquoted,"Cannot assume ".expr2text(t).
              " is a real variable,\n"."since that would be inconsistent to ".
              "user given properties")
      end_if
    else
      assume(_X,Type::Real)
    end_if;
    if t<>_X then f:=subs(f,[_X=C,t=_X]) end_if;
    delete s0,F0,u0,_s0,t00,w0,a0,b0,z0,_w0,c0,d0,_z0,g0,i0,k0,l0,m0,n0,mu0,
           m10,p0,nu0,m20,_n0,r0;
    vl:=((indets(f) union indets({ta,tb})) minus {t})
       intersect {s,F,u,_s,t0,w,a,b,z,_w,c,d,_z,g,i,k,l,m,n,mu,m1,p,nu,m2,_n,r,
                  s0,F0,u0,_s0,t00,w0,a0,b0,z0,_w0,c0,d0,_z0,g0,i0,k0,l0,m0,
                  n0,mu0,m10,p0,nu0,m20,_n0,r0};
    hl:=vl intersect {s0,F0,u0,_s0,t00,w0,a0,b0,z0,_w0,c0,d0,_z0,g0,i0,k0,l0,
                      m0,n0,mu0,m10,p0,nu0,m20,_n0,r0};
    vl:=vl minus hl;
    if iszero(nops(vl)) then sl:=[]: bl:=[]
    else sl:=[op(map(vl, e->e=_concat(e,0)))]; 
    end_if;
    if nops(hl)<>0 then
      hl:=[op(map(hl, e->e=genident()))];
      sl:=hl.sl 
    end_if;
    bl:=[op(map(sl,e->op(e,2)=op(e,1)))];
    properties(op(e,2),properties(op(e,1))) $ e in sl ;
    delete s,F,u,_s,t0,w,a,b,z,_w,c,d,_z,g,i,k,l,m,n,mu,m1,p,nu,m2,_n,r;
    f:=subs(subs(intlib::defLookup_intern(subs(f,sl),_X,subs(ta..tb,sl),
                                          options),bl),_X=t);
    map(hl,e->(unassume(op(e,2)); e)); // delete properties
    f
  end_proc:


intlib::defLookup_intern := proc(_f,t,rng,options) local n;
// rng is a range a..b 
save  _X,g,s,nu,b,a,c,p,i,F,mu;
begin
  delete _X,g,s,nu,b,a,c,p,F,mu;
  if (g:=intlib::defLookup1(subs(_f,t=_X),rng))<>FAIL then return(g) end_if;
   userinfo(1,"trying to get a closed form for",hold(int)(_f,t=rng));
   case rng
     of 0..infinity do
      if (i:=intlib::intdef1(_f,t))<>FAIL then return(i)
      elif (i:=intlib::intdef2c(_f,t))<>FAIL then return(i)
      elif (i:=intlib::intdef4(_f,t))<>FAIL then return(i)
      end_if;
      // first try to recognize a Laplace transform 
      userinfo(100,"try to recognize a Laplace transform");
      if Match2(_f,_p(hold(g))* //exp(-indep2(s,t,0)*t))
                   exp(-_p(hold(s),X->(not (has(X,t) or bool(X=0))))*t))
        then
         userinfo(101,"...for special formula");
         // Gradstein and Ryzhik, page 319, formula 8 
         if Match2(g,t^indep(nu,t)*(t+indep(b,t))^indep(nu,t)) then
            nu:=nu+1; mu:=s;
            if (nu<>1 and is(Re(nu)>0) and is(Re(mu)>0))=TRUE
               and arg(Re(b),Im(b))<>PI
              then
               return((b/mu)^(nu-1/2)/PI^(1/2)*exp(b*mu/2)*gamma(nu)*
		besselK(1/2-nu,b*mu/2))
            end_if
         end_if;
         if is(Re(s)>=0)=TRUE then
           // otherwise there is no chance to fulfill
           // the condition that |g(t)|<=K*exp(c*t) with Re(s)>c
           i:=transform::laplace(g,t,hold(s)); // prevent value of s
           if not hastype(i,"transform::laplace") then
             // use the value of s
             return(eval(i))
           end_if;
         end_if;
         delete s;
      end_if;
      // then try to recognize a Cosine fourier transform 
      userinfo(100,"try to recognize a Cosine fourier transform");
      if intlib::iseven(_f,t) and Match2(_f,_p(hold(g))*cos(indep(s,t)*t))
        then
         userinfo(2,"recognized a Cosine Fourier transform");
         i:=transform::fourier(g,t,s);
         if not has(i,transform::fourier) then return(i/2) end_if;
         delete s;
      end_if; 
      // then a Mellin transform 
      userinfo(100,"try to recognize a Mellin transform");
      if Match2(_f,_p(hold(g))* //t0^(indep2(s,t,1)-1))
                t^(_p(hold(s),X->(not (has(X,t) or bool(X=1))))-1))
      then
         i:=transform::mellin(g,t,s);
         if not has(i,transform::mellin) then return(i) end_if;
      end_if;
      // lookup method 
      userinfo(100,"lookup method");
      if Match2(_f,sin(indep(a,t)*t)*sin(indep(b,t)*t)/t^2) then
         return(min(a,b)*PI/2)
      elif Match2(_f,sin(indep(a,t)*t)^indep(b,t)*t^indep(c,t)) then
	// sin(a*t)^b*t^c 
         if a=0 or b=0 then FAIL
         elif testtype(c,Type::NegInt) then // gamma(c+1)->singularity 
	    return((if c=-b then
               // c(k,l) = int(sin(x)^k/x^l,x=0..infinity) 
               c:=proc(k,l) option remember; begin
                  if l<=2 then binomial(k-l,k/2-l/2)*PI/2^(k+1-l)
                  else k*((k-1)*c(k-2,l-2)-k*c(k,l-2))/(l-1)/(l-2)
                  end_if
               end_proc;
               c(b,b)*a^(b-1)
            elif b=3 and c=-1 then PI*sign(a)/4
            elif b=3 and c=-2 then 3/4*a*ln(3)
            elif c<-1 and is(b+c<=-1)=TRUE then sign(a^b)*infinity
            else FAIL end_if))
         else return(sin((c+1)*PI/2)*gamma(c+1)*sign(a)/abs(a)^(c+1))
         end_if
      // t^a*(b+c*sin(i)+p*cos(s)) 
      elif Match2(_f,t^indep(a,t)*(indep(b,t)+indep(c,t)
	*sin(_p(hold(i)))+indep(p,t)*cos(_p(hold(s))))) then
         if a=-2 and c=0 and p=-b and not has(s/t,t) then
	    // int((1-cos(p*t))/t^2,t=0..infinity)=1/2*Pi*abs(p) 
            return(1/2*PI*abs(s/t))
         elif b=0 and p=0 and Match2(i,t^indep(s,t)) then
            return(c*sin((a+1)*PI/2/s)*gamma((a+1)/s)/s)
         elif b=0 and c=0 and Match2(s,t^indep(i,t)) then
	    return(p*cos((a+1)*PI/2/i)*gamma((a+1)/i)/i)
         elif is(limit(_f*t,t=0,Right)>0)=TRUE then return(infinity)
         end_if
      // (exp(a*t^p)-exp(b*t^p))/t 
      elif Match2(_f,
           (exp(indep(a,t)*t^indep(p,t))-exp(indep(b,t)*t^indep(p,t)))/t)
         then
	   return((if (is(a<0) and is(b<0) and is(p>0))=TRUE then ln(b/a)/p
         	   else FAIL end_if))
      // case (F(a*t)+c*F(b*t))/t 
      elif Match2(_f,(indep(F,t)(indep(a,t)*t)
	+indep(c,t)*indep(F,t)(indep(b,t)*t))/t) then
         return((if F=hold(cos) and c=-1
		   then ln(b)-ln(a)+PI/2*sign(b)-PI/2*sign(a)
                 elif F=hold(tan) and c=-1 then ln(a/b)
                 elif F=hold(tanh) then
	           if c=-1 then ln(a/b) elif is(a*b>0)=TRUE then
		     sign(a)*sign(c+1)*infinity else FAIL end_if
                 else FAIL end_if))
      // (sech(a*t)-sech(b*t))/t 
      elif Match2(_f,(1/cosh(indep(a,t)*t)-1/cosh(indep(b,t)*t))/t) then
         return((if is(a*b>0)=TRUE then ln(b/a) else FAIL end_if))
      // g/sinh(a*t) 
      elif Match2(_f,_pattern(hold(g))/sinh(indep(a,t)*t)) then
	 return((if g=t then sign(a)*PI^2/4/a^2
		 elif Match2(g,sin(indep(b,t)*t)) then
                    PI*tanh(b*PI/2/abs(a))/2/a
                 else FAIL end_if))
      elif Match2(_f,1/(t^2+_pattern(a,not has,t))) then return(PI/2/a^(1/2))
      // g*cos(b*t) 
      elif Match2(_f,cos(indep(b,t)*t)*_pattern(hold(g))) then
         return((
           // g=1/exp(a*t^2) 
           if Match2(g,1/exp(indep(a,t)*t^2)) then
             PI^(1/2)/2/exp(b^2/4/a)/a^(1/2)
           elif Match2(g,1/cosh(indep(a,t)*t)) then PI*sech(b*PI/2/a)/2/a
           elif Match2(g,sin(indep(a,t)*t)/t) then PI/4*(sign(a+b)-sign(b-a))
           else FAIL end_if))
      // 1/cosh(a*t) 
      elif Match2(_f,1/cosh(indep(a,t)*t)) then return(PI/2/abs(a))
      // sinh(a*t)/(1+exp(b*t)) 
      elif Match2(_f,sinh(indep(a,t)*t)/(1+exp(indep(b,t)*t))) then
         return((if is(b>0)=TRUE then PI*csc(a*PI/b)/2/b-1/2/a
		 else FAIL end_if))
      // t^n*J(nu,a*t)*J(mu,b*t) 
      elif Match2(_f,t^indep(n,t)*besselJ(indep(nu,t),
	          indep(a,t)*t)*besselJ(indep(mu,t),indep(b,t)*t)) then
          return((
	      if (is(Re(1-n)>0) and is(Re(mu+nu+1-n)>0) and is(a>0) and
                 is(b>0))=TRUE then
	           if is(a>b)=TRUE
                     then a; a:=b; b:=%2; nu; nu:=mu; mu:=%2 end_if;
                   (nu+mu+1-n)/2; (nu-mu-n+1)/2;
	           a^mu/2^n*b^(n-mu-1)/gamma(mu+1)/gamma(%1+n)*gamma(%2)
	           *hypergeom([%2,%1],[mu+1],a^2/b^2)
              else FAIL end_if))
      end_if;
      break
     of -infinity..0 do
        return( int(eval(subs(_f,t=-t)),t=0..infinity,options) )
     of 0..PI/2 do
          return((
            if (i:=intlib::intdef2a(_f,t))<>FAIL then i
            // case 1/(a*cos(t)+1) 
            elif Match2(_f,1/(indep(a,t)*cos(t)+1)) then
	      if a=1 then 1
              elif a=-1 then infinity
	      else 2*arctanh((a-1)/(a^2-1)^(1/2))/(a^2-1)^(1/2) end_if
            // case 1/(tan(t)^a+1) 
            elif Match2(_f,1/(tan(t)^indep(a,t)+1)) then PI/4
            // case (a*cos(t)^2+b*sin(t)^2)^c 
	    elif Match2(_f,
                 (indep(a,t)*cos(t)^2+indep(b,t)*sin(t)^2)^indep(c,t)) then
              if c=-1 then PI/2/abs(a^(1/2)*b^(1/2))
              elif c=-2 then PI/4*(a+b)/a^(3/2)/b^(3/2)
              else FAIL end_if
            // case (arctan(a*t)-arctan(b*t))/t 
            elif Match2(_f,(arctan(indep(a,t)*t)-arctan(indep(b,t)*t))/t) then 
	      FAIL // PI/2*ln(a/b) is wrong 
            // case (1-a*sin(t)^2)^c 
	    elif Match2(_f,(1-indep(a,t)*sin(t)^2)^indep(c,t)) then
              return((
                 if c=-1/2 then ellipticK(a^(1/2))
                 elif c=1/2 then ellipticE(a^(1/2))
                 elif c=-3/2 then ellipticPi(a,a^(1/2))
                 else FAIL end_if))
            else FAIL end_if))
   of -1..0 do if not intlib::iseven(_f,t) then break end_if
   // else go into the case 0..1 
   of 0..1 do
      if (i:=intlib::intdef2b(_f,t))<>FAIL then return(i)
      elif Match2(_f,ln(1/t)^indep(a,t)) then return(gamma(a+1))
      elif Match2(_f,t^indep(b,t)/(1+t^indep(a,t))*ln(ln(1/t))) then
         /* ISSAC'97 proceedings, A Class of Logarithmic Integrals, by
	   Victor Adamchik */
         if b=a-1 and is(Re(a)>0)=TRUE // Corollary 2 page 4 
	    then return(-ln(2)*ln(2*a^2)/2/a)
	 elif b=2*a-1 and is(Re(a)>0)=TRUE // Corollary 3 page 4 
	    then return((ln(2)^2+2*(ln(2)-1)*ln(a)-2*EULER)/2/a)
	 end_if
      elif Match2(_f,t^(indep(a,t)-1)*ln(ln(1/t))) then
         // eq (33) from Adamchik's paper 
         if is(Re(a)>0)=TRUE then return(-(ln(a)+EULER)/a) end_if
      elif Match2(_f,t^indep(b,t)/(1+t^indep(a,t))^2*ln(ln(1/t))) then
         if b=a-1 and is(Re(a)>0)=TRUE then // Prop. 5 page 6 
	    return(-1/2/a*(EULER+ln(2*a/PI)))
	 end_if
      elif Match2(_f,t^indep(b,t)/(1+t^indep(a,t))^3*ln(ln(1/t))) then
         if b=a-1 and is(Re(a)>0)=TRUE then // Prop. 6 page 7 
	    return(
               -3*ln(a)/8/a-(10*ln(2)-6*ln(PI)+9*EULER+36*D(zeta)(-1))/24/a)
	 end_if
      end_if; break
   of -infinity..infinity do
      // recognizes Fourier transforms 
      if Match2(_f,_p(hold(g))*exp(-I*indep(s,t)*t)) and s<>0 then
         userinfo(2,"recognized an exponential Fourier transform");
         i:=transform::fourier(g,t,s);
         if not has(i,transform::fourier) then return(i) end_if
      end_if;
      delete s;
      if options["PrincipalValue"] then
      // try to compute integral in the sense of Cauchy
        //limit(int(_f,t=-s..0,PrincipalValue)
        //      +int(_f,t=0..s,PrincipalValue),s=infinity)
        s:=genident("R");
        i:=int(_f+eval(subs(_f,t=-t)),t=0..s,options);
        if type(i)="int" then
          return( limit(hold(int)(_f,t=-s..0,intlib::printOptions(options))
                        +hold(int)(_f,t=0..s,intlib::printOptions(options)),
                        s=infinity))
        else return( limit(i,s=infinity) ) end_if
      end_if;
      if intlib::iseven(_f,t) and
        //type(limit(_f,t=0))<>stdlib::Infinity
        not(has(limit(_f,t=0),{infinity,-infinity})) then
        return(2*int(_f,t=0..infinity,options))
      // the following may be wrong since there is no check of discontinuities 
      //elif intlib::isodd(_f,t) then 
      //   if limit(t*_f,t=infinity)=0 then return(0) end_if
      end_if; break
   of 0..PI do
      if Match2(_f,ln(indep(a,t)+indep(b,t)+indep(c,t)*cos(t))) then
         if iszero(expand(c^2-4*a*b)) or a*b=0 then
            return(PI*ln(1/2*(a+b+(a^2+b^2+c)^(1/2))))
         end_if
      end_if; break
   of 0..2*PI do
      if Match2(_f,1/(indep(a,t)*cos(t)+1)) then
         if (is(-1<a) and is(a<1))=TRUE then return(2*PI/(1-a^2)^(1/2)) end_if
      elif Match2(_f,sin(t+indep(a,t)+sin(t))) then
	 return(-2*PI*sin(a)*besselJ(1,1))
      end_if; break
   end_case;
   FAIL
end_proc:

intlib::defLookup1:=proc() begin FAIL end_proc :
//intlib::defLookup1(cos(_X),0..infinity):=undefined:
intlib::defLookup1(_X/sin(_X),0..PI/2):=2*hold(CATALAN):
intlib::defLookup1(ln(_X)/(1+_X),0..1):=-PI^2/12:
intlib::defLookup1(ln(_X)/(1-_X),0..1):=-PI^2/6:
intlib::defLookup1(ln(1+_X)/_X,0..1):=PI^2/12:
intlib::defLookup1(ln(_X)*ln(1+_X),0..1):=-2*ln(2)-PI^2/12+2:
intlib::defLookup1(ln(_X)/(1-_X^2),0..1):=-PI^2/8:
intlib::defLookup1(ln((1+_X)/(1-_X))/_X,0..1):=PI^2/4:
intlib::defLookup1(ln((1+exp(_X))/(exp(_X)-1)),0..infinity):=PI^2/4:
intlib::defLookup1(_X*ln(sin(_X)),0..PI):=-PI^2*ln(2)/2:
// (29) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(1/(1+_X^2)*ln(ln(1/_X)),0..1):=PI/2*ln((2*PI)^(1/2)*
    gamma(3/4)/gamma(1/4)):
// (30) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(_X/(1+_X^4)*ln(ln(1/_X)),0..1):=PI/4*ln(PI^(1/2)*
    gamma(3/4)/gamma(1/4)):
// (31) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(1/(1+_X^3)*ln(ln(1/_X)),0..1):=ln(2)/6*ln(3/2)-
    PI*(ln(54)-8*ln(2*PI)+12*ln(gamma(1/3)))/6/3^(1/2):
// (32) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(1/(1-_X+_X^2)*ln(ln(1/_X)),0..1):=
    PI*(5*ln(2*PI)-6*ln(gamma(1/6)))/3/3^(1/2):
// from Victor Adamchik paper in ISSAC'97, 3.5 page 6 
intlib::defLookup1(1/(1+_X+_X^2)*ln(ln(1/_X)),0..1):=
    PI*(-3*ln(3)+8*ln(2*PI)-12*ln(gamma(1/3)))/6/3^(1/2):
// from Victor Adamchik paper in ISSAC'97, 3.6 page 6 
intlib::defLookup1(_X/(1+_X+_X^2+_X^3+_X^4)*ln(ln(1/_X)),0..1):=
    PI/5*(cot(2*PI/5)*ln(2*PI)-2*ln(gamma(1/5)/gamma(4/5))*sin(PI/5)
    +2*ln(gamma(2/5)/gamma(3/5))*sin(2*PI/5)):
// (42) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(_X^(1/2)/(1+_X)^2*ln(ln(1/_X)),0..1):=
    EULER/2-3/2*ln(2)+ln(4*gamma(3/4)/gamma(1/4))+PI/2*ln(2*PI^(1/2)*
    gamma(3/4)/gamma(1/4)):
// (43) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(_X^(1/2)/(1+_X^3)^2*ln(ln(1/_X)),0..1):=
    -EULER/6-1/3*ln(6^(1/2)*gamma(3/4)/gamma(1/4))+PI/6*ln(2*(PI/3)^(1/2)*
    gamma(3/4)/gamma(1/4)):
// (44) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(_X/(1-_X+_X^2)^2*ln(ln(1/_X)),0..1):=
    -EULER/3-1/3*ln(6*3^(1/2)/PI)+PI*3^(1/2)/27*(5*ln(2*PI)-6*ln(gamma(1/6))):
// (47) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(_X^(1/2)/(1+_X)^3*ln(ln(1/_X)),0..1):=
    -CATALAN/2/PI+PI/8*ln(2*PI^(1/2)*gamma(3/4)/gamma(1/4)):
// (48) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(1/(1+_X^2)^3*ln(ln(1/_X)),0..1):=
    1/32*(-16*ln(2)+3*PI*ln(2*PI)+8*CATALAN/PI-8*EULER+2*(3*PI-8)*
    ln(gamma(3/4)/gamma(1/4))):
// (49) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1((1-6*_X^2+_X^4)/(1+_X^2)^3*ln(ln(1/_X)),0..1):=
    2*CATALAN/PI:
intlib::defLookup1(sech(_X)*tanh(_X)/_X,0..infinity):=4*CATALAN/PI:
// (50) from Victor Adamchik paper in ISSAC'97 
intlib::defLookup1(tanh(_X)/_X/(1-2*cosh(2*_X))^2,0..infinity):=
    (-2*3^(1/2)*PI^2-3*PI*ln(3)+3^(1/2)*psi(1/6,1))/18/PI:

/* first class of integrals (generated by the gamma function),
   recognizes int(t^w*ln(b*t^d)^m*exp(-u*t^s)*cos(c*t^s),t=0..infinity)
   where the cos can be replaced by sin, formula (3) p. 151
*/
intlib::intdef1 := proc(f,t) local pat,l,typ,fac,i,a;
save w,b,d,m,u,_s,c;
begin
   delete w,b,d,m,u,_s,c;
   typ:=" ";
   pat:=t^_p(w)*ln(indep(b,t)*t^_p(d))^_p(m,Type::NonNegInt)
        *exp(-indep(u,t)*t^_p(_s));
   // rewrites a^(b*x) to exp(b*x*ln(a)) 
   f:=misc::maprec(f,{"_power"}=proc(X) begin (if not has(op(X,1),t) and not 
	has(op(X,2)/t,t) then exp(op(X,2)*ln(op(X,1)))
	else X end_if) end_proc );
   l:=Match(f,pat);
   if l=FAIL then
      if (l:=Match(f,pat*cos(indep(c,t)*t^_p(_s))))=FAIL then
         l:=Match(f,pat*sin(indep(c,t)*t^_p(_s)));
         if l=FAIL then return(FAIL)
         else;
            userinfo(2,"matches int(t^w*ln(b*t^d)^m*exp(-u*t^s)*sin(c*t^s),".
		       "t=0..infinity)");
            typ:="sin" 
         end_if;
      else
         userinfo(2,"matches int(t^w*ln(b*t^d)^m*exp(-u*t^s)*cos(c*t^s),".
                    "t=0..infinity)");
         typ:="cos"
      end_if;
   else
      userinfo(2,"matches int(t^w*ln(b*t^d)^m*exp(-u*t^s),t=0..infinity)");
      c:=0 // no cos nor sin factor 
   end_if;
   userinfo(3,"with",op(l));
   l:=op(l);
   m:=subs(m,l); w:=subs(w,l); _s:=subs(_s,l); b:=subs(b,l); d:=subs(d,l);
   c:=subs(c,l);
   u:=subs(u,l);
   if is(_s,Type::Real)<>TRUE then FAIL
   elif _s=0 or is(u>0)<>TRUE then
	if u=0 and m=0 and typ="cos" then
           if is(w<=-1)=TRUE then infinity
	   elif w=0 and c>0 then
             if _s=1 then undefined 
             elif _s>1 then // Gradsteyn, Ryzhik: 3.712
               cos(PI/2/_s)*gamma(1/_s)/abs(c)^(1/_s)/_s
	     else FAIL end_if
           else FAIL end_if
	elif u=0 and m=0 and typ="sin" and w=0 and c>0 then
           if _s=1 then undefined 
           elif _s>1 then // Gradsteyn, Ryzhik: 3.712
	     sin(PI/2/_s)*gamma(1/_s)/abs(c)^(1/_s)/_s*sign(c) 
           else FAIL end_if
	else FAIL end_if
   else
      a:=(w+1)/_s;
      if is(Re(a)>0)<>TRUE then
         if a=0 and m=0 and typ="sin" then return(arctan(c/u)/_s) end_if;
         return(FAIL)
      elif (u:=u-I*c)<>1 then // change of variable u*t^s -> t^s 
         fac:=u^(-a); b:=b*u^(-d/_s);
      else fac:=1
      end_if;
      // now the integrand is fac*t^w*ln(b*t^d)^m*exp(-t^s) 
      // formula (1) and (2) page 151 
      i:=fac*sign(_s)*_plus(binomial(m,i)*ln(b)^(m-i)*d^i*_s^(-i-1)*
         eval(subs(intlib::dgamma(i,t),t=a)) $ i=0..m);
      if typ="cos" then Re(i) elif typ="sin" then Im(i) else i end_if
   end_if;
end_proc:

intlib::dgamma:=proc(n,t)
option remember;
begin
   if n=0 then gamma(t) else diff(intlib::dgamma(n-1,t),t) end_if
end_proc:

/* second class of integrals (generated by the beta function)
   matches int(sin(t)^r*ln(sin(t))^n*cos(t)^s*ln(cos(t))^m,t=0..PI/2),
   formula (6) p. 152 */
intlib::intdef2a := proc(f,t) local l;
save a,r,n,s,m;
begin
   delete a,r,n,s,m;
   /* on 0..PI/2, cos(t)>=0 and sin(t)>=0 therefore ln(1/cos(t))=-ln(cos(t))
     and ln(1/sin(t))=-ln(sin(t)) */
   f:=subs(f,ln(1/cos(t))=-ln(cos(t)),ln(1/sin(t))=-ln(sin(t)),
	1-sin(t)^2=cos(t)^2,sqrt(cos(t)^2)=cos(t));
   if has(f,ln(tan(t))) then
      f:=expand(subs(f,ln(tan(t))=ln(sin(t))-ln(cos(t))));
      if type(f)="_plus" then return(map(f,intlib::intdef2a,t)) end_if
   end_if;
   l:=Match(f,indep(a,t)*sin(t)^indep(r,t)*
              ln(sin(t))^_p(n,Type::NonNegInt)*cos(t)^indep(s,t)*
	      ln(cos(t))^_p(m,Type::NonNegInt));
   if l=FAIL then FAIL
   else
      r:=subs(r,l); s:=subs(s,l);
      if (is(Re(r)>-1) and is(Re(s)>-1))<>TRUE then FAIL
      else
         subs(a*2^(-n-m-1),l)*intlib::evalbeta(subs((m,n),l),(r+1)/2,(s+1)/2)
      end_if
   end_if
end_proc:

/* returns diff(beta(z,w),w$m),z$n) at z=r and w=s
  where Re(r)>0 and Re(s)>0
*/
intlib::evalbeta := proc(/*m,n,r,s*/) local _l;
save _z,_w;
begin
   delete _z,_w; 
   _l:=diff(diff(beta(_z,_w),_w$args(1)),_z$args(2));
   _l:=subs(_l,beta(_z,_w)=gamma(_z)*gamma(_w)/gamma(_z+_w));
   eval(subs(_l,_z=args(3),_w=args(4)))
end_proc:

/* matches int(t^r*ln(t)^n*(1-t^u)^s*(ln(1-t^u))^m,t=0..1), formula (8) p. 152
  extended with the change of variable t^u -> t with u>0 */
intlib::intdef2b := proc(f,t) local v,fac;
save r,_n,u,s,m;
begin
   delete r,_n,u,s,m;
   // first transforms ln(1/t)^r to (-1)^r*ln(t)^r 
   f:=misc::maprec(f,{"_power"}=proc() begin (if op(args(1),1)=ln(1/t) then
	(-1)^op(args(1),2)*ln(t)^op(args(1),2) else args(1) end_if) end_proc );
   fac:=(if type(f)="_mult" then split(f,has,t); f:=%[1]; %2[2] else 1 end_if);
   userinfo(2,"trying to match int(t^r*ln(t)^n*(1-t^u)^s*ln(1-t^u)^m,t=0..1)");
   if not Match2(f,t^indep(r,t)*ln(t)^indep(_n,t)*(1-t^indep(u,t))^indep(s,t)
	*(ln(1-t^indep(u,t)))^_p(m,Type::NonNegInt)) then FAIL
   else
      userinfo(3,"matches with",hold(r)=r,hold(_n)=_n,hold(u)=u,
                 hold(s)=s,hold(m)=m);
      if [s,m]=[0,0] and testtype(_n,Type::PosInt) then
         // case int(t^r*ln(t)^n,t=0..1) 
         return(fac*(-1)^_n*(r+1)^(-_n-1)*gamma(_n+1))
      elif not testtype(_n,Type::NonNegInt) then return(FAIL)
      end_if;
      if is(u>0)<>TRUE then return(FAIL)
      elif u=1 then // fac unchanged 
      else // change t^u -> t 
         r:=(r+1)/u-1; fac:=fac/u^(_n+1);
      end_if;
      if r=-1 and s=0 and m>0 then // formula (3.1) of Ko"lbig 
         fac*fact(_n)*fact(m)*_plus(intlib::C(_n-v,1,1,0,0)*
				    _plus(binomial(v+r+1,r)
	    *intlib::C(m-r,1,1,0,0)*intlib::C(v+r+1,1,0,0,1)$ r=0..m)$v=0.._n)
      elif (is(Re(r)<=-1) or is(Re(s)<=-1))=TRUE then FAIL
      else fac*intlib::evalbeta(m,_n,r+1,s+1)
      end_if
   end_if
end_proc:

intlib::c := proc( k, a, a1, a2, a3) option remember; 
begin
   if k=1 then -ln(a)-EULER*(a1+a2-a3)
   else (-1)^k*(a1^k+a2^k-a3^k)*zeta(k) end_if
end_proc:

intlib::C := proc(k, a, a1, a2, a3) local i; option remember; 
begin
   if k=0 then 1 
   else expand(1/k*
	    _plus(intlib::c(i,a,a1,a2,a3)*intlib::C(k-i,a,a1,a2,a3) $ i=1..k))
   end_if
end_proc:

// matches int(t^r*ln(t)^m/(1+b*t^s)^l,t=0..infinity), formula (9) p. 153 
intlib::intdef2c := proc(f,t) local z0,w0,a,n;
save m,b,s,l,w,z,r,m1,m2;
begin
   delete m,b,s,l,w,z,r,m1,m2;
   userinfo(2,"trying to match int(t^r*ln(t)^m/(1+b*t^s)^l,t=0..infinity)");
  //indep2(z,t,0)
   if not Match2(f,t^indep(r,t)*ln(t)^_p(m,Type::NonNegInt)/ 
                    (_p(hold(z),X->(not (has(X,t) or bool(X=0))))
                   +indep(b,t)*t^indep(s,t))^indep(l,t))
     then FAIL
   elif z<>1 then 1/z^l*intlib::intdef2c(t^r*ln(t)^m/(1+b/z*t^s)^l,t)
   else
      userinfo(2,hold(m)=m,hold(b)=b,hold(r)=r,hold(s)=s,hold(l)=l);
      if not testtype(m,Type::NonNegInt) then FAIL
      elif s=0 then FAIL
      else
         delete z; 
         z0:=(r+1)/s; w0:=l-z0; // typo in the paper: w0=l-z0 and not 1-z0 
         if (is(Re(w0)>0) and is(Re(z0)>0) and not is(b<0))=TRUE then
            f:=sign(s)*beta(z,w)/s^(m+1)/b^z;
            while m>0 do f:=diff(f,z)-diff(f,w); m:=m-1 end_while;
            expand(eval(subs(f,z=z0,w=w0)))
         elif b=-1 and s=1 then // try formula (17) 
            a:=1-l; b:=l-r-1;
            userinfo(2,
		     "trying to match int(t^r*ln(t)^m/(1-t)^l,t=0..infinity)");
            userinfo(3,"a=",a,"b=",b);
            if (is(a>0) and is(b>0))=TRUE then
               l:=diff(beta(z,w),z$m); m:=diff((-1)^(m+1-z)*beta(z,w),w$m);
               eval(subs(l,z=r+1,w=a)+subs(m,z=a,w=b))
	    elif (testtype((n:=r),Type::NonNegInt) and
		  testtype(l-r-2,Type::NonNegInt)
		 and is(m-l+1>=0))=TRUE then // (3.6) of Ko"lbig 
                (-1)^(m+n+l)*fact(m)/fact(l-1)*
                _plus(combinat::stirling1(n+1,m1+1)
		*_plus((-1)^m2*combinat::stirling1(l-n-1,m2+1)*(2*PI)^(m-m1-m2)
		*abs(bernoulli(m-m1-m2))/fact(m-m1-m2)$m2=0..l-n-2)$m1=0..n)
	    elif (testtype((n:=r+1/2),Type::NonNegInt) and
                 testtype(l-n-1,Type::NonNegInt) 
		 and is(m-l+1>=0))=TRUE then // (3.8) of Ko"lbig 
               userinfo(3,"using formula (3.8) of Koelbig");
               (-1)^(m+n+l+1)*fact(m)/fact(l-1)*
	       _plus((-1)^m1*intlib::Shat(m1,n)
	       *_plus(intlib::Shat(m2,l-n-1)*(2*PI)^(m-m1-m2+1)
		      *(2^(m-m1-m2+1)-1)*abs(bernoulli(m-m1-m2+1))
		      /fact(m-m1-m2+1)$m2=0..l-n-1)$m1=0..n)
            else FAIL
            end_if
         elif iszero(z0) and is(Re(w0)>0)=TRUE then // try with limit 
            f:=sign(s)*beta(z,w)/s^(m+1)/b^z;
            while m>0 do f:=diff(f,z)-diff(f,w); m:=m-1 end_while;
            f:=expand(expand(eval(subs(f,w=w0))));
            limit(f,z=0,Right)
         elif m=0 and b=1 and s=1 and l=1 then
            return((if (is(r<=-1) or is(0<=r))=TRUE then infinity else FAIL 
		end_if))
         else FAIL
         end_if
      end_if
   end_if
end_proc:

// int(x^k/(exp(x)+1),x=0..infinity) = k!*(1-2^(-k))*zeta(k+1) 
intlib::intdef4 := proc(f,x) local l;
save k;
begin
   delete k;
   l:=Match(f,x^indep(k,x)/(exp(x)+1));
   if l=FAIL then FAIL
   else
      k:=subs(k,op(l));
      if k=0 then ln(2)
      elif is(Re(k)>-1)=TRUE then gamma(k+1)*(1-2^(-k))*zeta(k+1)
      else infinity
      end_if
   end_if
end_proc:

// notation \hat{S} in Ko"lbig's paper, page 110 
intlib::Shat:=proc(p,q) local j;
option remember;
begin
   _plus((-1)^j*combinat::stirling1(q,j)*binomial(j,p)*2^(p-j)$j=p..q)
end_proc:
