// $Date: 2000/11/13 08:38:20 $ $Author: fakler $ $Revision: 1.22 $ 

/*++ ---------------- RischInt.mu ---------------------

Author: Winfried Fakler,  University of Paderborn
Date Created: 12 December 1998
Last Update : 13 November 2000


Description:
This file contains the toplevel functions for symbolic integration of
elementary functions. Currently, it is only implemented for transcendental
functions.
This function is mainly a cut - copy - paste implementation extracted from
int.mu due to Ralf Kraume and Paul Zimmermann.
							  
Functions: 

 - used Parameter:
    ++ f   :DOM_EXPR
    ++ x   :DOM_IDENT				  
    ++ f    = elementary function
    ++ x    = indeterminate of f					  

 - intlib::rischInt(f, x <, Unsimplified>)
    ++ returns an elementary antiderivative of f wrt. x or FAIL.
    ++ Currently, only the transcendental case is implemented.
    ++ If option Unsimplified is given, the complex integral is returned.
    
See:
 
Examples:
>> intlib::rischInt(x*sin(x^2-1),x);
					      
++*/

 intlib::rischInt :=
  proc(f :Type::Arithmetical,x :DOM_IDENT,o=" ")
    local opt,i,integral,tosubs,hassincos,hasI,
          vl,sl,bl,hl,e;
    save  n,k,k_sub,k_ident,k_diff,k_deg,k_alg,Q,Q_sub,
          n0,k0,k_sub0,k_ident0,k_diff0,k_deg0,k_alg0,Q0,Q_sub0;
  begin
    //if testargs() then f:=eval(f) end_if;
    delete n0,k0,k_sub0,k_ident0,k_diff0,k_deg0,k_alg0,Q0,Q_sub0;
    vl:=((indets(f) minus indets(x)) union {hold(k)})
         intersect {n,k,k_sub,k_ident,k_diff,k_deg,k_alg,Q,Q_sub,
                    n0,k0,k_sub0,k_ident0,k_diff0,k_deg0,k_alg0,Q0,Q_sub0};
    hl:=vl intersect {n0,k0,k_sub0,k_ident0,k_diff0,k_deg0,k_alg0,Q0,Q_sub0};
    vl:=vl minus hl;
    if iszero(nops(vl)) then sl:=[]: bl:=[]
    else sl:=[op(map(vl, e->e=_concat(e,0)))]; 
    end_if;
    if nops(hl)<>0 then
      hl:=[op(map(hl, e->e=genident()))];
      sl:=hl.sl 
    end_if;
    bl:=[op(map(sl,e->op(e,2)=op(e,1)))];
    properties(op(e,2),properties(op(e,1))) $ e in sl ;
    delete n,k,k_sub,k_ident,k_diff,k_deg,k_alg,Q,Q_sub;
    f:=eval(subs(f,sl));
    x:=eval(subs(x,sl));
    opt:=hold(Exp);
    // the following two lines are used to simplify the result afterwards
    hassincos:=has(f,{sin,cos,tan});
    hasI:=has(f,I);
    // initialize tower of extension fields of _f_ over rational numbers
    n:=1; k:=[x];
    k_sub:=[x]; k_ident:=[genident()]; k_diff:=[1]; k_deg:=[0]; k_alg:=[0];
    Q:=[]; Q_sub:=[];
    // substitution and construction of the field extensions over the 
    // rational numbers 
    if hassincos then f:=combine(f,sincos) end_if; // produces a simpler tower
    //f:=intlib::subst(f,opt);
    f:=rewrite(rewrite(f,ln),exp);  // did not work for f=sqrt(k(x))
    if has(f,exp) then f:=combine(expand(f),exp) end_if;
    userinfo(2,"after substitution, integrand is",f);
    repeat i:=intlib::tower(f) until i<>FALSE end_repeat;
    if i=FAIL then
      userinfo(1,"construction of tower extension failed");
      return(FAIL)
    end_if;
    userinfo(2,"field extension of rational numbers",Q,Q_sub);
    userinfo(2,"tower extension of functions",k,k_sub,k_alg);
    userinfo(2,"after field extensions, integrand is",i);
    // determine integral 
    integral:=intlib::intt(i,n);
    // back substitution 
    tosubs:=[k_sub[n-i]$ i=0..n-2, Q_sub[nops(Q)-i]$ i=0..nops(Q)-1];
    while has(integral,tosubs) do
      // take care with eval here, for example in int(1/(x^2+k^2),x) !!! 
      if traperror((integral:=subs(integral,k_sub[n-i]=k[n-i]$ i=0..n-2,
                       Q_sub[nops(Q)-i]=Q[nops(Q)-i]$ i=0..nops(Q)-1,
		       hold(k)=hold(_intlib_k),hold(n)=hold(_intlib_n),
		       hold(i)=hold(_intlib_i));))<>0 then
        userinfo(1,"something is wrong, probably there is an undetected ".
               "algebraic relation between some functions of the integrand.".
                "\nPerhaps rewriting the integrand e.g. with ".
                "'rewrite(integrand,tan)' will help.");
        integral:=FAIL;
      end_if;
    end_while;
    integral:=eval(subs(integral,hold(_intlib_k)=hold(k),
			hold(_intlib_n)=hold(n),
	                hold(_intlib_i)=hold(i)));
    userinfo(2,"after substitution, integral is",integral);
    if integral=FAIL then return(FAIL) end_if;
    // try to simplify integral 
    if o<>Unsimplified then
      if hasI=FALSE and has(integral,I) then
        userinfo(2,"try to compute real part of integral");
        integral:=intlib::simplifyComplexIntegral(integral,x,
                                                  if intlib::hasLnExp(f) then
                                                    hold(Unique)
                                                  end_if)
      end_if;
      if hassincos then
        // the following line needs too much time e.g. for
        // combine(expand(cos(10*x)))
        // integral:=combine(expand(intlib::to_sincos(integral,x)),sincos)
        traperror((integral:=simplify(integral);),intlib::simplifyTime);
      end_if;
    end_if;
    delete n,k,k_sub,k_ident,k_diff,k_deg,k_alg,Q,Q_sub;
    integral:=eval(subs(integral,bl));
    map(hl,e->(unassume(op(e,2)); e));
    integral
  end_proc:

/*
f:=sin(t)^2*(cos(t)*(-1) + cos(t)^2*3 + sin(t)^2*3)*(cos(t)*(-6) + cos(t)^2*9\
 + sin(t)^2*9 + 1)^(-2);

*/
