// $Date: 2000/09/15 06:59:58 $ $Author: fakler $ $Revision: 1.10 $ 

/*++
  Synopsis: changevar(f,eq) changes the variables in f according to eq.
  Author: Paul Zimmermann, 8 Feb 1996
          Winfried Fakler
  Examples:
  >> intlib::changevar(hold(int)(ln(x)*2, x=0..1),x=exp(t));

                     int(2 t exp(t), t = -infinity..0)
++*/

intlib::changevar :=
  proc(i,eq=FAIL,t)
    local _x,xoft,u,tofx,s,f;
  begin
    if not( type(i)="int" ) then
      error("First argument must be of type \"int\"");
    end_if;
    if type(eq)=DOM_FAIL then
      error("Missing equation for substitution");
    elif type(eq)<>"_equal" then
      error("Second argument must be an equation describing the substitution");
    end_if;
        
    if type((_x:=op(i,2)))="_equal" then _x:=op(_x,1) end_if;
    if not has(eq,_x) then
      error("Missing variable "._x." in substitution equation");
    end_if;
    if args(0)<3 then // try to find the new variable
      t:=indets(eq) minus {_x};
      if nops(t)<>1 then
        error("Please give the new variable")
      end_if;
      t:=op(t)
    end_if;
    if not has(eq,t) then
      error("Missing new variable ".t." in substitution equation");
    end_if;
    //assume(t,Type::Real);
    s:=solvelib::discreteSolve(eq,_x); //,PrincipalValue;
    if type(s)<>DOM_SET or s={} then
      error("Cannot solve the change equation for "._x)
    end_if;
    xoft:=op(s,1);
    for u from 2 to nops(s) do
      if length(op(s,u))<length(xoft) then xoft:=op(s,u) end_if
    end_for;
    if type(op(i,2))="_equal" then
      tofx:=solvelib::discreteSolve(eq,t); //,PrincipalValue;
      if type(tofx)<>DOM_SET or tofx={} then
        error("Cannot solve the change equation for ".t)
      end_if;
      tofx:=op(tofx,1);
      u:=
       t=limit(tofx,_x=op(i,[2,2,1]),Right)..limit(tofx,_x=op(i,[2,2,2]),Left)
    else u:=t
    end_if;
    f:=eval(subs(op(i,1),_x=xoft));
    traperror((f:=simplify(f);),intlib::simplifyTime);
    hold(int)(f*diff(xoft,t),u)
  end_proc:

// end of file 
