// $Date: 2000/10/31 13:22:29 $ $Author: fakler $ $Revision: 1.70 $ 

/*--
intlib::int(f,x) - computes the antiderivative of the function f(x) with
                   respect to the variable x.
--*/

// the following (protected) identifier is used to detect the recursion level
// of intlib::int. This becomes necessary to initialize the set intlib::tries
// correctly. Only at top level intlib::tries is new initialized.
// Note: This is only secure for using intlib::int, if one calls
//       intlib::int_intern instead wrong counting will occure.
//sysassign(intlibDepthLevel,0):

intlib::int:=
  proc(f :Type::Arithmetical,x :DOM_IDENT)
    local prop,C,F;
    save _X; //necessary to make assumption on integration variable
  begin
    //if testargs() then f:=eval(f) end_if;
    C:=genident("__X");
    if property::hasprop(_X) then 
      //assume(C,prop1)  // does'nt work due to an error in assume
      zip([C],[getprop(_X)],assume);
    end_if;
    delete _X;
    if property::hasprop(x) then
      prop:=getprop(x);
      if prop=Type::Complex then
        //assume(_X,prop) // does'nt work due to an error in assume
        zip([_X],[Type::Complex],assume);        
      elif not(property::Fail((prop:=prop and Type::Real))) then
        if prop=Type::Zero then
          prop:=Type::Complex;
          if intlib::printWarningsFlag then
            warning("Cannot integrate when ".expr2text(x).
                    " has property ".expr2text(getprop(x)).".\n".
                    "While integrating, we will assume ".expr2text(x).
                    " has property ".expr2text(prop).".");
          end_if;
        elif is(x,Type::Rational)=TRUE then
          prop:=Type::Real;
          if intlib::printWarningsFlag then
            warning("Cannot integrate when ".expr2text(x).
                    " has property ".expr2text(getprop(x)).".\n".
                    "While integrating, we will assume ".expr2text(x).
                    " has property ".expr2text(prop).".");
          end_if;
        end_if;
        //assume(_X,prop) // does'nt work due to an error in assume
        zip([_X],[prop],assume);
      elif intlib::printWarningsFlag then
        warning("Cannot assume ".expr2text(x).
               " is a real variable,\n"."since that would be inconsistent to ".
                "user given properties")
      end_if
    else
      assume(_X,Type::Real)
    end_if;
    if x<>_X then f:=subs(f,[_X=C,x=_X]) end_if;
    F:=subs(intlib::int_intern(f,_X),[_X=x,C=_X]);
    unassume(C);
    F    
  end_proc:


intlib::int_intern:=
  proc(_F_,_x_)
  //option remember; // could lead to conflicts with user given properties
    local integral,Float,_f_,F0;
    save intlibDepthLevel;//make counter secure for error or exception handling
  begin
    F0:=_F_;
    if intlibDepthLevel = hold(intlibDepthLevel) then 
       sysassign(intlib::tries,{});
       sysassign(intlibDepthLevel,0):
    end_if;
    sysassign(intlibDepthLevel,intlibDepthLevel+1);
    // First, transform each floating point number to its
    // rational approximation 
    _f_:=misc::maprec(_F_,
         {DOM_FLOAT,DOM_COMPLEX}=(x -> numeric::rationalize(x,Minimize)) );
    Float:=(if _f_ <> _F_ then float else id end_if);
  
    // substitute remaining two-argument arcus tangents
    // otherwise we could run into trouble with the lookup mechanism
    // Also doing some more rewritings, anyway maybe
    // this should be done in specialFunctionInt, but would cause there
    // more function calls
    if has(_f_,arctan) then
      _f_:=eval(subs(_f_,hold(arctan)=proc(x,y)
                     begin
                       if args(0)=1 then
                         arctan(x)
                       else
                         -I * ln((x+I*y)/(abs(x+I*y)))
                       end_if
                          end_proc));
    end_if;
    if has(_f_,arg) then
      _f_:=eval(subs(_f_,hold(arg)=((x,y)->-I*ln((x+I*y)/(abs(x+I*y))))));
    end_if;
    if has(_f_,log) then
      _f_:=eval(subs(_f_,hold(log)=((b,x) -> ln(x)/ln(b))));
    end_if;
                        
    // test if integrand contains special functions that can be treated
    integral:=intlib::specialFunctionInt(_f_,_x_);
    if integral<>FAIL then return(Float(integral)) end_if;
                        
    // lookup method 
    userinfo(2,"trying lookup method");
    integral:=intlib::lookup(_f_,_x_);
    if integral<>FAIL then return(Float(integral)) end_if;
                        
    // trying Risch Integration 
    userinfo(2,"lookup failed, trying Risch integration");
    integral:=intlib::rischInt(_f_,_x_);
    if integral<>FAIL then return(Float(intlib::expandExp(integral))) end_if;
                        
    // try to simplify the integrand or doing a last hack :-)) 
  //  userinfo(2,"Risch's algorithm failed, try to simplify integrand");
  //  if not contains(intlib::tries,_f_) then
  //    sysassign(intlib::tries,intlib::tries union {_f_});
  //  end_if;
  //  integral:=intlib::simplifyIntegrand(_f_,_x_);
  //  if integral<>FAIL then return(Float(intlib::expandExp(integral))) end_if;
  
    // using linearity of integration 
    userinfo(2,"try to integrate term by term");
    integral:=intlib::termByTerm(_f_,_x_);
    sysassign(intlibDepthLevel,intlibDepthLevel-1);
    if integral<>FAIL then return(Float(intlib::expandExp(integral))) end_if;
    hold(int)(F0,_x_)
  end_proc:

intlib::tries:={}:

intlib::expandExp:=
  proc(f)
  begin
    if has({f},exp) then
      misc::maprec(f,{"exp"}=proc() begin expand(exp(op(args()))) end_proc )
    else
      f
    end_if;
  end_proc:

intlib::simplifyIntegrand:=
  proc(f0,x)
    local f, F;
  begin
    f:=simplify(f0);   // for the following a normalform for f would be needed.
    if f<>f0 and not contains(intlib::tries,f) then
      // to avoid loops
      sysassign(intlib::tries,intlib::tries union {f});
      F:=intlib::int(f,x);
      if type(F)<>"int" then return(F) end_if;
    end_if;
    FAIL
  end_proc:
    
/* if both the lookup method and Risch algorithm failed,
  and if the function is a sum, it can be that some
  terms can be integrated by the lookup method and some
  other by Risch algorithm, like for example int(diff(g(x),x)+x+2,x) */
intlib::termByTerm := proc(f,x)
local f0,e,i,r;
begin
   f0:=f;		
   if type(f)="_mult" and contains(map({op(f)},type),"_plus") then
      f:=expand(f)
   end_if;
   if type(f)="_plus" then
     r:=0;
     for e in [op(f)] do
       i:=intlib::int(e,x);
       if hastype(i,"int") then
         return(FAIL)
       else
         r:=r+i
       end_if
     end_for;
     return(r);
   end_if;
   FAIL
end_proc:

