// $Date: 2000/11/24 11:46:17 $ $Author: tonner $ $Revision: 1.22 $ 
// 
// frankp, 23.06.1997 

Pref:= newDomain("Pref"):
Pref::info:= "The domain of preferences [try ?Pref for details]":
Pref::interface:= 
    {hold(verboseRead), hold(callBack), hold(callOnExit), hold(echo),
    hold(floatFormat), hold(maxTime), hold(maxMem), hold(output), 
    hold(postInput), hold(postOutput), hold(noProcRemTab), hold(prompt), 
    hold(report), hold(typeCheck), hold(keepOrder), hold(kernel), 
    hold(promptString), hold(userOptions), // hold(moduleTrace),
    hold(matrixSeparator), hold(ignoreNoDebug), hold(trailingZeroes),
    hold(timesDot), hold(alias), hold(warnChanges),
    hold(warnLexProcEnv), hold(warnDeadProcEnv), hold(ansi)}:

Pref::new:=
    // Pref()     -  print table of preferences and corresponding values 
    // Pref(NIL)  -  reset preferences to their default values           
    // 
    // Note: A new preference must be entered in the set Pref::interfaces, 
    //       otherwise 'Pref()' and 'Pref(NIL)' will not work correctly!   
    proc()
        local s, l, e, sf;
	save PRETTYPRINT, TEXTWIDTH;
    begin
        PRETTYPRINT:= FALSE;
        TEXTWIDTH:= 32000:

        case args(0)
        of 0 do
            sf:= map( Pref::interface,expr2text );
            l:= max( op(map(sf,length)) ) + 6;

            s:= "";
            for e in sf do
                s:= s . stringlib::format( "Pref::".e,l,Left ) . ": " . expr2text( slot(Pref,e)() ) . "\n"
            end_for;
            print( Unquoted,s );
            return(null());
        of 1 do 
            if args(1) <> NIL then error("expecting 'NIL' as argument") end_if;

            // reset preferences to their default value 
            sf:= map( Pref::interface,expr2text );
            for e in sf do
                case (s:= slot( slot( Pref,e ),"default" ))
                of FAIL do break; // read-only 
                of "NIL" do // default value is NIL 
                    slot( Pref,e )(NIL); break
                otherwise
                    slot( Pref,e )(s)
                end_case
            end_for;
            return( TRUE )
        otherwise
            error("expecting either none or one argument")
        end_case;
    end_proc:

proc()
    option noDebug;
    local path;
begin

path := pathname("PREF");

Pref::verboseRead:= 
    loadproc(Pref::verboseRead,path,"verbread"):

Pref::alias:=
    loadproc(Pref::alias,path,"alias"):

Pref::callBack:=
    loadproc(Pref::callBack,path,"callback"):

Pref::callOnExit:=
    loadproc(Pref::callOnExit,path,"callexit"):

Pref::allComplete := 
    loadproc(Pref::allComplete, path, "complall"):
Pref::noComplete  := 
    loadproc(Pref::noComplete,  path, "complall"):

Pref::echo:=
    loadproc(Pref::echo,path,"echo"):

Pref::ansi:=
    loadproc(Pref::ansi,path,"ansi"):

Pref::floatFormat:=
    loadproc(Pref::floatFormat,path,"float"):

Pref::maxTime:=
    loadproc(Pref::maxTime,path,"maxtime"):

Pref::maxMem:=
    loadproc(Pref::maxMem,path,"maxmem"):

Pref::output:=
    loadproc(Pref::output,path,"output"):

Pref::postInput:=
    loadproc(Pref::postInput,path,"postin"):

Pref::postOutput:=
    loadproc(Pref::postOutput,path,"postout"):

Pref::noProcRemTab:=
    loadproc(Pref::noProcRemTab,path,"procrem"):

Pref::prompt:=
    loadproc(Pref::prompt,path,"prompt"):

Pref::report:=
    loadproc(Pref::report,path,"report"):

Pref::typeCheck:=
    loadproc(Pref::typeCheck,path,"typcheck"):

Pref::keepOrder:=
    loadproc(Pref::keepOrder,path,"keepOrd"):

Pref::kernel:=
    loadproc(Pref::kernel,path,"kernel"):

Pref::promptString:=
    loadproc(Pref::promptString,path,"prostr"):

Pref::userOptions:=
    loadproc(Pref::userOptions,path,"useropt"):

Pref::moduleTrace:=
    loadproc(Pref::moduleTrace,path,"modtrac"):

Pref::matrixSeparator:=
    loadproc(Pref::matrixSeparator,path,"matrsep"):

Pref::ignoreNoDebug:=
    loadproc(Pref::ignoreNoDebug,path,"inodebug"):

Pref::trailingZeroes:=
    loadproc(Pref::trailingZeroes,path,"delzero"):

Pref::timesDot:=
    loadproc(Pref::timesDot,path,"printdot"):

Pref::warnChanges:=
    loadproc(Pref::warnChanges,path,"warn15"):

Pref::warnLexProcEnv:=
    loadproc(Pref::warnLexProcEnv,path,"wrnLexPE"):

Pref::warnDeadProcEnv:=
    loadproc(Pref::warnDeadProcEnv,path,"wrnDeadPE"):

stdlib::LoadlibState["Pref"]:= TRUE:

null()
end_proc():

// end of file 

