/*++
The library 'detools' contains methods for treating differential equations.

Author: Werner M. Seiler, Uni Mannheim (seiler@euler.math.uni-mannheim.de)
	(with contributions by Jay Belanger and Marcus Hausdorf)
Date created: May 14th, 1998
Last update: 	Thu May  4 16:43:38 2000
++*/


detools:= newDomain("detools"):
detools::info:= "Library 'detools': methods for differential equations":
detools::interface:= { 
    hold(arbFuns), hold(autoreduce), hold(cartan), hold(characteristics), 
    hold(charODESystem), hold(charSolve), hold(complete), hold(derList2Tree),
    hold(detSys), hold(euler), hold(genDetSys), hold(hamilton),
    hold(hasHamiltonian), hold(hasPotential), hold(hilbert), hold(modode), 
 // hold(modStirling),  // <- moved to combinat::modStirling
    hold(ncDetSys),
 // hold(odeplot),  // obsolete because of plot::ode. Keep them as
 // hold(phaseplot),// undocumented functions for TCI 
    hold(pdesolve), 
    hold(transform)
}:

// The global table data is filled by the method getdata.
// It contains the basic data extracted from the arguments given by the user.
// This includes especially all options; each of them becomes a key.
// The main keys are:
//	DVin :	    	differential variable domain used for input
//	DFin :		differential function domain used for input
//	DVout :	    	differential variable domain used for output
//	DFout :		differential function domain used for output
//	Expr :		TRUE ->   output as DOM_EXPR
//			NoDiff -> output as DOM_EXPR but without diff
//			FALSE ->  output using DFout
//	System :	TRUE, if system is passed; FALSE for single equation
//	Linear :	TRUE, if linear equations are given
//	Polynomial :	TRUE, if polynomial domain can be used
//	Eqs :		the given equation converted to DFin
//	Parms :		further parameters given by the user; this includes 
//			everything that is not an option or an equation
// Some methods might use further keys.
detools::data := table():

// used by derList2Tree
detools::D_Table := NIL:


// MECHanics subdirectory
alias(path = pathname("DETOOLS","MECH")):
detools::euler := loadproc(detools::euler,path,"Euler"):
// detools::hamilton := loadproc(detools::hamilton,path,"Ham"):
detools::hasHamiltonian := loadproc(detools::hasHamiltonian,path,"HasHam"):
detools::hasPotential := loadproc(detools::hasPotential,path,"HasPot"):

// COMPLETE subdirectory
unalias(path):
alias(path = pathname("DETOOLS","COMPLETE")):
detools::complete := loadproc(detools::complete, path, "complete"):

// NUMerics/PLOT subdirectory
unalias(path):
alias(path = pathname("DETOOLS","NUMPLOT")):
detools::odeplot := loadproc(detools::odeplot,path,"ODEPlot"):
detools::phaseplot := loadproc(detools::phaseplot,path,"PhasePlot"):
detools::modode := loadproc(detools::modode,path,"ModODE"):

// LIE symmetry subdirectory
unalias(path):
alias(path = pathname("DETOOLS","LIE")):
detools::detSys := loadproc(detools::detSys,path,"detsys"):
detools::genDetSys := loadproc(detools::genDetSys,path,"gendetsys"):
detools::ncDetSys := loadproc(detools::ncDetSys,path,"ncdetsys"):
detools::genDeterminingSystem := loadproc(detools::genDeterminingSystem,
				      path,"gendetsys"):
detools::getAnsatz := loadproc(detools::getAnsatz,path,"symmisc"):
detools::getdatasymm := loadproc(detools::getdatasymm,path,"symmisc"):
detools::simpSList := loadproc(detools::simpSList,path,"symmisc"):
//detools::solveEquations := loadproc(detools::solveEquations,path,"symmisc"):

// SOLVER subdirectory
unalias(path):
alias(path = pathname("DETOOLS","SOLVER")):
detools::pdesolve := loadproc(detools::pdesolve,path,"PDESolve"):
detools::charODESystem := loadproc(detools::charODESystem,path,"linchar"):
detools::charSolve := loadproc(detools::charSolve,path,"linchar"):
detools::characteristics := loadproc(detools::characteristics,path,"linchar"):

// UTILITY subdirectory
unalias(path):
alias(path = pathname("DETOOLS","UTILITY")):
detools::autoreduce := loadproc(detools::autoreduce,path,"AutoRed"):
detools::derList2Tree := loadproc(detools::derList2Tree,path,"derivTree"):
detools::transform := loadproc(detools::transform,path,"transform"):
// detools::makeproc := loadproc(detools::makeproc,path,"MakeProc"):
//detools::modStirling moved to combinat::modStirling
//detools::modStirling := loadproc(detools::modStirling,path,"combinat"):
detools::hilbert := loadproc(detools::hilbert,path,"combinat"):
detools::cartan := loadproc(detools::cartan,path,"combinat"):
detools::arbFuns := loadproc(detools::arbFuns,path,"combinat"):
detools::getdata := loadproc(detools::getdata,path,"GetData"):
detools::output := loadproc(detools::output,path,"Output"):


stdlib::LoadlibState["detools"]:= TRUE:
null():

// end of file 




