// $Date: 2000/11/28 18:50:21 $  $Author: fakler $  $Revision: 1.180 $ #
// stdlib.mu -- lib-file for standard library STDLIB

stdlib::exported:= {
hold(D),hold(asympt),hold(anames),hold(export),hold(unexport),hold(info),hold(iquo),
hold(irem),hold(plot2d),hold(plot3d),hold(random),
hold(O),hold(taylor),hold(gcd),hold(lcm),hold(content),
hold(loadlib),hold(normal),hold(numer),
hold(denom), hold(pade), hold(version),
hold(factor),hold(gcdex),hold(partfrac),
hold(sum),hold(piecewise),hold(series),hold(powermod),
hold(infinity),hold(undefined),hold(complexInfinity),
hold(universe),hold(int),hold(maprat),hold(setuserinfo),
hold(expose),hold(radsimp),hold(combine),hold(linsolve),
hold(RootOf),hold(revert),hold(rationalize),hold(irreducible),
hold(rewrite),hold(discont),
hold(plotfunc),
hold(plotfunc2d),
hold(plotfunc3d),
hold(assign),
hold(hastype),hold(conjugate),
hold(simplify),hold(rectform),hold(Re),hold(Im),
hold(product),hold(max),hold(min),
hold(freeze),hold(unfreeze), hold(limit),
hold(solve),hold(is),hold(getprop),hold(unassume),hold(assume),hold(expand),
hold(diff),hold(isprime),hold(genident),hold(expr),
hold(icontent),hold(iquo),hold(irem),hold(ilcm),hold(igcd),hold(igcdex),hold(ithprime),
hold(protocol),hold(read),hold(print),hold(expr2text),hold(text2expr),hold(open),
hold(close),hold(fprint),hold(system),hold(length),
hold(substring),hold(strmatch),
hold(time),hold(rtime),hold(bytes),
hold(loadlib),hold(map),hold(zip),hold(contains),hold(split),hold(select),
hold(coeff),hold(nthcoeff),hold(lcoeff),hold(collect),hold(package),
hold(_in),hold(lhs),hold(rhs),hold(coerce),hold(Factored)
}:
// Note: do not include 'matrix' (see below) in order to be consistent
// with 'array', 'table', etc.

stdlib::interface:= stdlib::exported:

/*--
  If a library routine often uses protected variables, these
  statements may be executed by syseval.
  This routine is used by loadlib and loadproc, so it has to
  be defined before the first execution of both.
 
  Usage:
 
     stdlib::syseval( expr )

  !!! stdlib::syseval is used inside of loadproc. Do not move !!!!
--*/
stdlib::syseval := 
proc()
  option hold, noDebug;
  local oldAssign, res;
begin
  oldAssign := _assign;
  sysassign(_assign, sysassign);
  res := context(args());
  sysassign(_assign, oldAssign);
  res
end_proc:

/*--
  The following sets are used for the protect mechanism of the
  MuPAD library. The variables in stdlib::ENVIRONMENT_VARIABLES
  and stdlib::SYSTEM_CONSTANTS are not protected, the variables
  in stdlib::LIBRARY_CONSTANTS are protected.
  The protection is done at the end of .mupadsysinit.
--*/
// use strings here in order to avoid subsequent evaluation of
// library package initialization files:
stdlib::LIBRARIES:= {
  hold(import), hold(Ax), hold(Cat), hold(Dom), hold(Network), hold(RGB),
  hold(Series), hold(property), hold(Type), hold(adt), hold(intlib),
  hold(combinat), hold(fp), hold(generate), hold(groebner), hold(linalg),
  hold(listlib), hold(linopt), hold(matchlib), hold(module), hold(numeric),
  hold(numlib), hold(ode), hold(orthpoly), hold(output), hold(plot), hold(polylib),
  hold(prog), hold(solvelib), hold(stats), hold(stringlib), hold(student), hold(transform),
  hold(stdlib), hold(specfunc), hold(detools)
}:

stdlib::ENVIRONMENT_VARIABLES:= {          
  hold(DIGITS), hold(HISTORY), 
  hold(LEVEL), hold(LIBPATH), hold(MAXDEPTH), hold(MAXLEVEL), 
  hold(ORDER), hold(PRETTYPRINT), hold(READPATH), 
  hold(ROUNDING), hold(SEED), hold(TEXTWIDTH), hold(WRITEPATH),
  hold(TESTPATH), hold(_mod)
}:


stdlib::SYSTEM_CONSTANTS := {
  hold(CATALAN),
  hold(EULER), hold(FALSE), hold(I), hold(PI), hold(TRUE), 
  hold(UNKNOWN)
}:

stdlib::LIBRARY_CONSTANTS := { 
  hold(infinity), hold(MSDOS), hold(NO_CONVERGENCE), hold(NO_SOLUTION), 
  hold(toBeDefined), hold(UNBOUNDED), hold(undefined), hold(EMPTY), hold(OPTIMAL),
  hold(complexInfinity),
  hold(Global) // global property
}:

stdlib::OPTIONS := {
  hold(Above), hold(Adaptive), hold(All), hold(AllExponents), hold(Alldata),
  hold(AcceptSingularity),
  hold(Always), hold(Ansatz), hold(Any), 
  hold(Append), hold(Approx), /* hold(Args), */
  hold(Arrows), hold(Attached), hold(Automatic),
  hold(Axes), hold(AxesOrigin), hold(AxesScaling), 
  hold(AndMesh), hold(AndULine), hold(AndVLine),

  hold(BackGround), hold(BackSubstitution), 
  hold(Beams), // Option fuer plotlib::dataplot
  hold(Bin), hold(Bottom),
  hold(Box), hold(Banded), hold(BUTCHER6), hold(Binary),
  
  hold(CameraPoint), hold(Capacity), hold(Center), hold(CK45), hold(CK54),
  hold(Closed), hold(Coeff),
  hold(Coeffs), hold(Color), hold(ColorPatches), hold(Colors),  
  hold(Columns), // Option fuer plotlib::dataplot
  hold(Complete), hold(Concave), hold(Constrained),
  hold(Connected), // Option fuer plot::Pointset
  hold(Continuous), // Option fuer int
  hold(Contours), hold(Convex), 
  hold(Corner), hold(Crit),
  hold(Curve),
  hold(Curves), // Option fuer plotlib::dataplot
  hold(CurvesPoints), // Option fuer plotlib::dataplot
  hold(Cyclic),
  
  hold(DashedLines), hold(DefaultXRange), hold(DefaultYRange),
  hold(DegInvLexOrder), hold(Degree), hold(DegreeOrder),
  hold(DegInvLex),
  hold(DegLex), hold(DegRevLex),
  hold(Delete), hold(Diagonal),
  hold(Diagonalization), hold(DiffVars),
  hold(Dimension), hold(Discont), hold(Domain), hold(DomainsOnly), 
  hold(DOPRI45),
  hold(DOPRI54), hold(DualPrices),
  hold(DrawMode), // Option fuer plot::Pointset
  hold(Duplicate),
  
  hold(Error), hold(EULER1), hold(Eweight), hold(Exact), hold(Expr), 
  hold(Extended), 
  
  hold(Filled), hold(Circles), hold(FilledCircles), hold(First), hold(Flat), 
  hold(FixedPrecision),
  hold(FloatFormat), hold(FocalPoint), hold(FontSize), hold(Forced),
  hold(ForeGround), hold(Format), hold(Frames), hold(Frobenius), 
  hold(FilledSquares),
  
  hold(Gauss), hold(GaussChebyshev), hold(GaussLegendre), 
  hold(GaussTschebyscheff), hold(Generic),
  hold(GenOrder), hold(Gif), hold(Grid), hold(GridLines),
  hold(GridLinesColor), hold(GridLinesWidth), hold(GridLinesStyle),
  hold(GC), hold(GL), hold(GT),
  
  hold(Height), hold(HiddenLine), hold(Hlode), hold(HlodeOverRF),
  hold(Homogeneous),

  hold(IgnoreSpecialCases), hold(Impulses),
  hold(Include), hold(Independent), hold(Index), hold(IndexList), 
  hold(Infinity), hold(InputOnly), hold(Interpolation), hold(IntMod), 
  hold(Interactive), hold(Irreducible), hold(InSet),

  hold(KeepOrder), hold(Krylov),
  
  hold(Labeling), hold(Labels), hold(Left), hold(Length), hold(LexOrder), 
  hold(LieGroupAction), hold(Lin), 
  hold(Line), // Option fuer Turtleplot
  hold(Lines), // Option fuer plotlib::dataploat
  hold(LinesPoints), // Option fuer plotlib::dataplot
  hold(LineStyle), hold(LineWidth), hold(List), hold(Lode),
  hold(LodeOverRF), hold(Log), hold(Logic), 
  
  hold(MaxDegree), hold(MaxRandomPoints), hold(MaxCalls),
  hold(Mem), hold(Mesh), hold(MinorExpansion), hold(Mode), 
  hold(ModuleTrace), hold(Multiple), 
  hold(MultiSolutions), // for numeric::fsolve
  hold(Minimize),
  
  hold(Name), hold(Named), hold(Nary), hold(Natural), hold(New),
  hold(NewtonCotes), hold(NoArgs), hold(NoCheck), hold(NoErrors), 
  hold(NoLeftVectors), hold(NonNested), hold(NonNegative), hold(None),
  hold(NoNL), hold(NoRightVectors), hold(Normal), hold(NotAKnot), hold(NC),
  hold(Not), hold(NoWarning),
  
  hold(Off), hold(On), hold(Only), hold(Open), hold(Origin), 
  hold(Order), hold(OutSet), hold(OutputDomain), hold(OldFromNew),

  hold(Path), hold(Periodic), hold(Piechart), hold(Piechart3d), hold(Plain),
  hold(PlotDevice), hold(Prefix), hold(Postfix),
  hold(Points), // Option fuer plotlib::dataplot
  hold(PointStyle), hold(PointWidth), hold(PolyExpr), hold(PostMap),
  hold(PreMap), hold(Pretty), hold(Primitive), 
  hold(PrincipalValue), hold(Proc), hold(Properties),
  
  hold(Quiet), hold(Quo), hold(QDet),
  
  hold(Random), //replaces option Seed = s in numeric::fsolve
  hold(Raster), hold(RatExpr), hold(RATIO_YX), hold(Real),
  hold(RealsOnly), 
  hold(RealValuesOnly),
  hold(Recursive),
  hold(RelativeError), hold(Rem), hold(Reorder), 
  hold(RestrictedSearch), hold(Restore),
  hold(RK4), hold(RKF43), hold(RKF34), hold(RKF54a), hold(RKF45a),
  hold(RKF54b), hold(RKF45b), hold(RKF87), hold(RKF78), hold(Right),
  hold(Root), hold(Rest),

  hold(Sample), hold(Scales), hold(Scaling),
  hold(SceneOptions),
  hold(Scene2d), hold(Scene3d), hold(Smoothness),
  hold(SearchLevel),
  hold(ShowAssumptions),
  hold(Solved), hold(Special), hold(Steps),
  hold(SquareFree),
  hold(StartingValues), // Option fuer stats::reg
  hold(Stepsize), hold(Style), hold(Subset), hold(Superset),
  hold(Surface), hold(Symbolic),
  hold(SymbolicAnalysis),
  hold(Symmetric), hold(Squares),
  hold(System), hold(ShowTitle), hold(SolidLines),
  
  hold(TaylorExpansion),
  hold(Term), hold(Terms), hold(Test), hold(Text), hold(Ticks), hold(Title), 
  hold(TitlePosition), hold(Titles), hold(ToNearest), hold(ToInfinity), 
  hold(ToMinusInfinity), hold(ToZero), hold(TrailingZeroes), hold(Transform),
  hold(Transposed), hold(Tree), hold(Transparent),
  hold(Types),

  hold(UnConstrained), hold(Unique), hold(Unquoted), hold(Unrestricted),
  hold(UnrestrictedSearch), // for numeric::fsolve
  hold(User), hold(Unsimplified), hold(Unsorted), hold(Up), hold(UsePrimeTab),
  hold(ULine),
  
  hold(Values), hold(VectorOf), hold(ViewingBox), hold(Vweight), hold(VLine),
  hold(Vars),

  hold(Laurent), hold(Taylor), hold(Puiseux),
  
  hold(Warning), hold(Write), hold(WireFrame),
  
  hold(X11), hold(xCK54), hold(xCK45), hold(xDOPRI54), hold(xDOPRI45),
  hold(xRKF43), hold(xRKF34), hold(xRKF54a), hold(xRKF45a),
  hold(xRKF54b), hold(xRKF45b), hold(xRKF78), hold(xRKF87),
  hold(XMax), hold(XMin),
  
  hold(YMax), hold(YMin),

  hold(ZMax), hold(ZMin),
  
  hold(sincos), hold(sinhcosh),

  // fuer plotlib::implicitplot
  hold(Contours),
  hold(Splines), 
  hold(Factor),  // <- do protect, because the function Factor will disappear!
  hold(MinStepsize), hold(MaxStepsize), hold(StartingStepsize),
  hold(Precision)
}:

stdlib::SYSTEM_TABLES := {
  hold(stdlib::Alias), hold(stdlib::LoadlibState)
}:

// alias/unalias -- in .mupadsysinit definiert

alias(path = pathname("STDLIB")):

D:= loadproc(D, path, "D"):
Im:= loadproc(Im, pathname("STDLIB","IM"), "Im"):
O:= loadproc(O, path, "O"):
pade := loadproc(pade, path, "pade"):
Re:= loadproc(Re, pathname("STDLIB","RE"), "Re"):
taylor:= loadproc(taylor, path, "taylor"):
_fnest:= loadproc(_fnest, path, "fnest"):
_in:= loadproc(_in, path, "in"):
_print_userinfo:= loadproc(_print_userinfo, path, "userinfo"):
anames:= loadproc(anames, path, "anames"):
assign:= loadproc(assign, path, "assign"):
asympt:= loadproc(asympt, path, "asympt"):
collect:= loadproc(collect, path, "collect"):
combine:= loadproc(combine, path, "combine"):
complexInfinity:= loadproc(complexInfinity, path, "cinfty"):
conjugate:= loadproc(conjugate, pathname("STDLIB","CONJ"), "conj"):
content:= loadproc(content, path, "content"):
denom:= loadproc(denom, path, "denom"):
discont:= loadproc(discont, path, "discont"):
evalassign:= loadproc(evalassign, path, "evalass"):
export:= loadproc(export, path, "export"):
expose:= loadproc(expose, path, "expose"):
factor:= loadproc(factor, path, "factor"):
freeze:= loadproc(freeze, path, "freeze"):
fun:= loadproc(fun, path, "fun"):
func:= loadproc(func, path, "func"):
gcd:= loadproc(gcd, path, "gcd"):
gcdex:= loadproc(gcdex, path, "gcdex"):
stdlib::hasaccess:= loadproc(stdlib::hasaccess, path, "hasaccess"):
hastype:= loadproc(hastype, path, "hastype"):
stdlib::hasmsign:= loadproc(stdlib::hasmsign, path, "hasmsign"):
infinity:= loadproc(infinity, path, "infinity"):
info:= loadproc(info, path, "info"):
int:= loadproc(int, path, "int"):
iquo:= loadproc(iquo, path, "iquo"):
irem:= loadproc(irem, path, "irem"):
irreducible:= loadproc(irreducible, path, "irreduc"):
ithprime:=loadproc(ithprime, path, "ithprime"):
lcm:= loadproc(lcm, path, "lcm"):
lhs := loadproc(lhs,path,"lhsrhs"):
linsolve:= loadproc(linsolve, path, "linsolve"):
loadlib:= loadproc(loadlib, path, "loadlib"):
maprat:= loadproc(maprat, path, "maprat"):
match:= loadproc(match, pathname("MATCHLIB"), "matchrec"):
max:= loadproc(max, path, "minmax"):
min:= loadproc(min, path, "minmax"):
normal:= loadproc(normal, path, "normal"):
stdlib::normalizesign:= loadproc(stdlib::normalizesign, path, "normalizesign"):
stdlib::normal:= loadproc(stdlib::normal, path, "normal"):
numer:= loadproc(numer, path, "numer"):
operator:= loadproc(operator, path, "operator"):
package := loadproc(package, path, "package"):
partfrac:= loadproc(partfrac, path, "partfrac"):
plot2d:= loadproc(plot2d, path, "plot2d"):
plot3d:= loadproc(plot3d, path, "plot3d"):
plotfunc2d:= loadproc(plotfunc2d, path, "plotfun2"):
plotfunc3d:= loadproc(plotfunc3d, path, "plotfun3"):
plotfunc:= loadproc(plotfunc, path, "plotfunc"):
powermod:= loadproc(powermod, path, "powermod"):
product:= loadproc(product, path, "product"):
radsimp:= loadproc(radsimp, path, "radsimp"):
stdlib::radnormal:= loadproc(stdlib::radnormal, path, "radsimp"):
random:= loadproc(random, path, "random"):
rationalize := loadproc(rationalize, path, "maprat"):
rectform:= loadproc(rectform, pathname("STDLIB","RECTFORM"), "rectform"):
revert := loadproc(revert, path, "revert"):
rewrite:= loadproc(rewrite, path, "rewrite"):
rhs := loadproc(rhs,path,"lhsrhs"):
series:= loadproc(series, path, "series"):
setuserinfo:= loadproc(setuserinfo, path, "setuinfo"):
simplify:= loadproc(simplify, pathname("STDLIB","SIMPLIFY"), "simplify"):
sum:= loadproc(sum, path, "sum"):
Sum:= loadproc(Sum, path, "sum"):
sysevalassign:= loadproc(sysevalassign, path, "sysevass"):
undefined:= loadproc(undefined, path, "undef"):
unexport:= loadproc(unexport, path, "unexport"):
unfreeze:= loadproc(unfreeze, path, "freeze"):
universe:= loadproc(universe, path, "universe"):
version:= loadproc(version, path, "version"):
expr:= loadproc(expr, path, "expr"):
ifactor:= loadproc(ifactor, path, "ifactor"):
coerce:= loadproc(coerce, path, "coerce"):
matrix:= loadproc(matrix, path, "matrix"):
Factored:= loadproc(Factored, path, "Factored"):


/* -------- RootOf -------- */
RootOf := loadproc(RootOf, path, "RootOf"):
// the following methods are used in 'sum':
stdlib::RootOf_polysum:= loadproc(stdlib::RootOf_polysum, path, "RootOf" ):
stdlib::RootOf_evala:= loadproc(stdlib::RootOf_evala, path, "RootOf" ):
stdlib::RootOf_expand_power:= loadproc(stdlib::RootOf_expand_power, path, "RootOf" ):

/* ------- limit --------- */
limit:= loadproc(limit, path, "limit"):
stdlib::limit:= loadproc(stdlib::limit, path, "limit"):

/* -------  solve --------- */
solve                  := loadproc(solve,                  path,"solve"):

/* ------- piecewise ------- */
piecewise:= loadproc(piecewise, path, "piecewis"):
stdlib::branch:= loadproc(stdlib::branch, path, "piecewis"):

/*-------- export some of the functions of property ----- */
unalias(path):
alias(path = pathname("PROP")):

is		:= loadproc(is,		path, "Propis" ):
getprop		:= loadproc(getprop,	path, "Propget"):
unassume	:= loadproc(unassume,	path, "Propdel"):
assume		:= loadproc(assume, 	path, "Propset"):

/*------ CONTINUE -----------*/
protect:= proc() name protect; option hold;
begin
    if args(0) = 2 then
        protected(args(1), context(args(2)))
    else
        protected(args(1), Warning)
    end_if
end_proc:

unprotect:= proc() name unprotect; option hold;
begin
    protected(args(1), None)
end_proc:

// Domain fuer spez. Ausgabe
stdlib::Exposed:= newDomain("stdlib::Exposed"):
stdlib::Exposed::op:= proc(x, rg = null()) begin op(extop(x, 2..extnops(x)), rg) end_proc:
stdlib::Exposed::subsop:= proc(x) begin subsop(extop(x, 2..extnops(x)), args(2..args(0))) end_proc:
stdlib::Exposed::nops:= (x)-> extnops(x) - 1:
stdlib::Exposed::print:= proc(x) begin extop(x, 1)(extop(x, 2..extnops(x))) end_proc:
// erstes 
stdlib::Exposed::new:=
  proc(x)
  begin
    if args(0) = 1 or not(contains({DOM_PROC, DOM_FUNC_ENV}, type(x))) then
      new(stdlib::Exposed, extop, args())
    else
      new(stdlib::Exposed, args())
    end_if
  end_proc:
    
_pref(hold(Alias)=[TRUE,
                   proc()
                     local config;
                   begin
                     config := _parser_config();
                     if config = null() then
                       return(table())
                     else
                       config := op(config,2);
                       config := map([op(config)],
                                     x -> (x, eval(op(x, 1))=op(x, 2)));
                       return(table(op(config)))
                     end_if;
                   end_proc]):

// temporary library implementation of _seqin

/* (a $ b in c) is parsed into _seqin(a,b,c),
  which evaluates to eval(subs(a,b=op(c,1))),...,eval(subs(a,b=op(c,nops(c))))
*/
_seqin := proc() local i, _0_expr; option hold;
begin
   if not contains({DOM_IDENT, DOM_VAR}, type(args(2))) then 
      error("illegal arguments") end_if;
   _0_expr:= context(args(3));
   context(subs(args(1),args(2)=op(_0_expr,i),Unsimplified)) $ 
                           i=1..nops(_0_expr)
end_proc:
_seqin := funcenv(_seqin, builtin(1125, 5, " $ ", "_seqin"),
                  table("type"="_seqin")):


// initialisiere Completion
_pref(hold(Complete) = 
proc(x)
    name Complete;
    option noDebug;
    local globals, libs, funcEnvs, interface, all, makeNames, 
          num, num1, domName, DOMAIN;
begin
  makeNames := proc(d, s)
    name makeNames;
    local i;
  begin
     d."::".op(s,i) $ i = 1..nops(s);
  end_proc:

  if contains({"", ":", "::"}, x) then return("") end_if;

  // select only matching globals to determine DOM_DOMAIN and DOM_FUNC_ENV
  globals := map(select(stdlib::anames(3), X -> strmatch(expr2text(X), x."\*")),
                 X -> context(hold(val)(X)));
  [libs,funcEnvs,globals] := split(globals, X -> op([UNKNOWN, TRUE, FALSE],
                                                    contains([DOM_DOMAIN, DOM_FUNC_ENV], type(X)) + 1));
  // All
  all:= select(Pref::allComplete(), strmatch, x."\*");

  // if given a ':' or '::'
  num:= op(strmatch((()->(_concat(x[length(x) - i] $ i=1..length(x))))(), ":", Index), 1);
  if num = TRUE then
    // only ':'
    return("")
  elif type(num) = DOM_INT then
    // possibly domain
    num:= length(x) - num;
    if x[num - 2] = ":" then
      // two ':'
      if num > 2 and x[num - 3] = ":" then
        // more as two ':'
        return("")
      else
        num:= num - 1
      end_if
    end_if;
    domName := substring(x, 0, num - 1);
    // KEYWORDS keywords - don't evaluate
    if contains({"and", "begin", "break", "case", "delete", "div", "do",
                 "downto", "elif", "else", "end", "end_case", "end_for" , "end_if",
                 "end_proc", "end_repeat", "end_while", "for", "from", "if", "in",
                 "intersect", "local", "minus", "mod", "name", "next", "not", "of",
                 "option", "or", "otherwise", "proc", "quit", 
                 "repeat", "step", "then", "to", "union", "until", "while"}, domName) then
      return("")
    else
      text2expr(domName);
      if traperror((DOMAIN := eval(%))) <> 0 then
        return("");
      end_if;
      if domtype(DOMAIN) = DOM_DOMAIN then
        // DOMAIN is a domain
        if domtype(DOMAIN::interface) = DOM_SET then
          // if an interface exists, only complete interface
          interface := {makeNames(domName, map(DOMAIN::interface, expr2text))};
        elif DOMAIN::allEntries <> FAIL then
          // DOMAIN from domains package    
          interface := {makeNames(domName, DOMAIN::allEntries())};
        else
          // no interface, complete all operands
          interface := {makeNames(domName, map({op(DOMAIN)}, op, 1))};
        end_if;
        if domtype(DOMAIN::completions) = DOM_SET then
          interface:= interface union DOMAIN::completions
        end_if
      elif domtype(DOMAIN) = DOM_FUNC_ENV then
        // DOMAIN is a function environment
        if domtype(op(DOMAIN,3)) = DOM_TABLE then
          // there are slots defined
          if domtype(DOMAIN::interface) = DOM_SET then
            interface := {makeNames(domName, DOMAIN::interface)};
          else
            interface := {makeNames(domName, map({op(op(DOMAIN,3))}, op, 1))};
            end_if;
        else
          interface := {};
        end_if;
      else
        // DOMAIN is no domain
        interface := {};
      end_if
    end_if  
  else
    // normal name completion, no ':' given
    libs:= select(libs, x->x::interface <> {});
    libs:= map(libs minus {stdlib} /*union funcEnvs*/, expr2text);
    if nops(libs) > 1 then
      // additionally the domain without '::'
      all:= all minus libs
    end_if;
    interface := map(libs, _concat, "::")
  end_if;  

  op(sort(select([op((interface union all) minus Pref::noComplete())], strmatch, x."\*")))
end_proc):


// special environments used for integration
// _pattern -- defining patterns used in intlib's pattern matcher
_pattern:=funcenv(_pattern):
_pattern:=slot(_pattern,"type","_pattern"): // for pattern variable 
// counter to detect recursion level of method int (used in INTLIB/int.mu
protect(intlibDepthLevel,Error):
protect(intlibLookupDepthLevel,Error):
// global flag used in definite integration to prevent some simplifications
protect(calledFromWithinDefInt,Error):

// global flag for ode solver, it needs to be global for using save which is
// necessary to be correct if errors/exceptions occurs
protect(calledFromWithinDSolve,Error):

stdlib::LoadlibState["stdlib"]:= TRUE:
null():

