// 
// $Date: 2000/05/21 14:55:22 $  $Author: bij $   $Revision: 1.8 $ 
// 
// frankp, 16/08/94 

/*++
  addcol.mu

	linalg::addCol() -- form linear combinations of matrix columns

	addCol(A,p1,p2,s)

	A     : matrix, square matrix
	p1,p2 : positive integers
	s     : any

	After s is converted into an element of A::coeffRing, 
	'addCol' returns a copy of matrix A in which column 
	p2 of A is replaced by col(A,p1)*s + col(A,p2).
++*/

linalg::addCol := proc(A,p1,p2,s)
    local R, r, DA;
begin
    DA:= A::dom;
    if testargs() then
	if args(0) <> 4 then error("expecting 4 arguments") end_if;
	if DA::hasProp( Cat::Matrix ) <> TRUE then
	    error ("first argument is not of 'Cat::Matrix'")
	end_if;
	if not testtype( p1,Type::PosInt ) or not testtype( p2,Type::PosInt ) then
	    error("invalid indices")
	end_if;
	R := op( DA::matdim(A),2 );
	if p1 > R or p2 > R then
	    error("indices larger than upper matrix bound")
	end_if;
	if not testtype( s,DA::coeffRing ) then
	    error("unable to convert ".expr2text(s))
	end_if
    end_if;

    R:= DA::coeffRing;
    if domtype(s) <> R then s:= R::coerce(s) end_if;

    r:= DA::col( A,p1 );
    DA::setCol( A,p2,_plus(DA::map(r,R::_mult,s),DA::col( A,p2 )) )
end_proc:

// end of file 
