// 
// $Date: 2000/05/21 14:55:22 $  $Author: bij $   $Revision: 1.7 $ 
// 
// frankp, 16/08/94 

/*++
     addrow.mu

	linalg::addRow()    -- form linear combinations of matrix rows

	addRow(A,p1,p2,s)

	A     : matrix, square matrix
	p1,p2 : positive integers
	s     : any

	After s is converted into an element of A::coeffRing, 'addRow'
	returns a copy of matrix A in which row p2 of A is replaced
	by row(A,p1)*s + row(A,p2).
++*/

linalg::addRow := proc(A,p1,p2,s)
    local R, r;
begin
    if testargs() then
	if args(0) <> 4 then error("expecting 4 arguments") end_if;
	if A::dom::hasProp( Cat::Matrix ) <> TRUE then
	    error("first argument is not of 'Cat::Matrix'")
	end_if;
	if not testtype( p1,Type::PosInt ) or not testtype( p2,Type::PosInt ) then
	    error("invalid indices")
	end_if;
	R := op( A::dom::matdim(A),1 );
	if p1 > R or p2 > R then
	    error("indices larger than upper matrix bound")
	end_if;
	if not testtype( s,A::dom::coeffRing ) then
	    error("unable to convert ".expr2text(s))
	end_if
    end_if;

    R:= A::dom::coeffRing;
    if domtype(s) <> R then s:= R::coerce(s) end_if;

    r:= A::dom::row( A,p1 );
    A::dom::setRow( A,p2,_plus(A::dom::map( r,R::_mult,s ),A::dom::row( A,p2 )) )
end_proc:

// end of file 
