// $Date: 1999/10/21 09:58:24 $  $Author: frankp $   $Revision: 1.6 $
// frankp, 31/10/94

/*
        linalg::divergence -- computes the divergence of a vector function
			      over fields

        divergence(x,m,[ogSystem])

        x       : list or vector of arithmetical expressions 
                  (the vector function)
        m       : list of (indexed) identifiers
	ogSystem: (optional) name or list of the scale factors of an 
                  orthogonally curvilinear coordinate system

	This function computes the divergence of x in the variables given 
        in m. If only two arguments are given linalg::divergence operates on
        the cartesian coordinate system. Hence the result is the sum
	diff(x[i],m[i]) as i ranges over the length of x.

        Otherwise, linalg::divergence operates on the orthogonally 
	curvilinear coordinate system specified by a list of scale 
	factors or by a name which scale factors are defined in table 
	linalg::ogCoordTab.
*/

linalg::divergence := proc(x,m)
    local g, i, R, Rmult, Rinvert, Rnormal;
begin
    if testargs() then
	if args(0) <> 2 and args(0) <> 3 then
	    error("expecting 2 or 3 arguments")
	end_if;

	if testtype( x,linalg::VectorOf(Type::AnyType) ) then
	    if not (x::dom::coeffRing)::hasProp( Cat::Field ) then
                error("expecting vectors over 'Cat::Field'")
            end_if
        elif not testtype( x,Type::ListOf(Type::Arithmetical) ) then
            error("expecting list of arithemtical expressions")
        end_if;

	if not testtype( m,Type::ListOf(Type::Unknown,nops(x) $ 2) ) then
            error("invalid number or type of variables")
	end_if;

	if args(0) = 3 then
	    if nops(m) <> 3 then
	        error("expecting vector function with 3 components")
	    elif domtype(args(3)) <> DOM_LIST and 
                 type(linalg::ogCoordTab[args(3),hold(Scales)]) = "_index"
            then
                error("illegal or undefined coordinate system")
            end_if
	end_if
    end_if;

    if args(0) = 2 then 
	return( _plus( diff(x[i],m[i]) $ i=1..nops(m) ) )
    else
        g:= args(3);
        if domtype(g) <> DOM_LIST then
            g:= linalg::ogCoordTab[g,hold(Scales)](m[i] $ i=1..3)
        end_if;

        if domtype(x) <> DOM_LIST then
	    R:= x::dom::coeffRing;
	    for i from 1 to 3 do
	        if (g[i]:= R::coerce(g[i])) = FAIL then
		    userinfo(1,"unable to compute the divergence over ",R);
                    return( FAIL )
	        end_if
	    end_for;
	    Rmult:= R::_mult; 
	    Rinvert:= R::_invert;
	    Rnormal:= id
        else
	    Rmult:= _mult;
	    Rinvert:= 1/id;
	    Rnormal:= normal
        end_if;

        Rnormal(Rmult( Rinvert(g[i]) $ i=1..3, _plus( 
            diff(Rmult(g[2],g[3],x[1]),m[1]),
            diff(Rmult(g[3],g[1],x[2]),m[2]),
            diff(Rmult(g[1],g[2],x[3]),m[3]) 
        ) ))
    end_if
end_proc:

