// $Date: 2000/03/29 08:18:10 $  $Author: stefanw $   $Revision: 1.16 $
// frankp, 30/03/95

/*
	linalg::eigenvalues -- computes the eigenvalues of a matrix

	linalg::eigenvalues(A [,Multiple])
	
	A: square matrix
	 
	The call eigenvalues(A) returns a set of the eigenvalues of A.
	If option Multiple is given, then a list of sublists 
        is returned, where each sublist contains an eigenvalue and its 
        algebraic multiplicity.

	If the coefficient ring of A is Dom::Float then numeric::eigenvalues 
        is used. In this case and for the option 'Multiple', the list e of 
        eigenvalues is sorted which means that e[i]<=e[j] for i<j.
*/

linalg::eigenvalues := proc(A)
    local p, x, e, R;
begin
    if testargs() then
        if args(0) < 1 or args(0) > 2 then
	    error("expecting a matrix and optionally 'Multiple' as option")
        elif A::dom::hasProp( Cat::Matrix ) <> TRUE then
	    error("first argument is not of 'Cat::Matrix'")
	elif args(0) = 2 and args(2) <> hold(Multiple) then
            error("expecting option 'Multiple'")
	end_if
    end_if;

    R:= A::dom::coeffRing;
    if R = Dom::Float then
        userinfo(1,"call 'numeric::eigenvalues'");
        p:= numeric::eigenvalues( A ); 
        if args(0) = 1 then
            p:= select( map({op(p)},R::coerce),_not@_equal,FAIL );
            return( p )
        else
            // numeric::eigenvalues already returns multiple eigenvalues,
            // hence collect multiple eigenvalues:
            x:= [];
            while p <> [] do
                p:= split( p,_equal,p[1] );
                if (e:= R::coerce(p[1][1])) <> FAIL then
                    x:= append( x,[e,nops(p[1])] )
                end_if;
                p:= p[2]
            end_while;
            return( x )
        end_if
    else
        x:= genident("x"):
        userinfo(1,"compute characteristic polynomial p(A,x)");
        p:= linalg::charpoly( A,x );

        userinfo(1,"compute zeros of p(A,x) in x");
        if args(0) = 2 then
            // with option 'Multiple', return a list of lists:
            return( [op(solve(p,x,hold(Multiple), hold(MaxDegree)=4))] )
        else
            return( solve(p,x, hold(MaxDegree)=4) )
        end_if
    end_if
end_proc:

