// $Date: 1999/10/30 15:23:55 $  $Author: frankp $   $Revision: 1.7 $
// frankp, 1/11/94

/*
        linalg::grad -- computes the vector gradient of an expression

        grad(x,m,[ogSystem])

        x       : arithmetical expression
        m       : list of (index) identifiers
        ogSystem: (optional) name or list of the scale factors of an 
                  orthogonally curvilinear coordinate system

        This function computes the gradient of x in respect to m in form of 
        a list. If only two arguments are given then 'linalg::grad' operates 
        on cartesian coordinate system. 
        Hence the result is the vector v of partial derivatives whose i-th 
        entry is defined by v[i]:=diff( x,m[i] ).

        The vector is of type Dom::Matrix().

        Otherwise 'linalg::grad' operates on the orthogonally curvilinear
        coordinate system specified by a list of scale factors or by a name 
        which scale factors are defined in table 'linalg::ogCoordTab'.

        The scale factors g_i are defined by g_i := |diff( T(u),u_i )|,
        where T describes the orthogonal transformation x=T(u) with x 
        the cartesian coordinates and u the orthogonally curvilinear
        coordinates.
*/

linalg::grad:= proc(f,x)
    local g, i;
begin
    // type checking:
    if testargs() then
        if args(0) <> 2 and args(0) <> 3 then
            error("expecting 2 or 3 arguments")
        elif not testtype( f,Type::Arithmetical ) then
            error("1st argument must be of 'Type::Arithmetical'")
        elif not testtype( x,Type::ListOf(Type::Unknown) ) then 
            error("2nd argument must be a list of (indexed) identifiers")
        end_if;
        if args(0) = 3 then
            g:= args(3);
            if nops(x) <> 3 then
                error("expecting 3 variables")
            elif domtype(g) <> DOM_LIST 
                 and type(linalg::ogCoordTab[g,hold(Scales)]) = "_index" 
            then
                error("illegal or undefined coordinate system")
            end_if
        end_if
    end_if;

    if args(0) = 3 then
        g:= args(3);
        if domtype(g) <> DOM_LIST then
            g:= linalg::ogCoordTab[g,hold(Scales)](op(x))
        end_if;
        return(Dom::Matrix()( 3,1,
           [1/g[1]*diff(f,x[1]),1/g[2]*diff(f,x[2]),1/g[3]*diff(f,x[3])] 
        ))
    else
        return(Dom::Matrix()( nops(x),1,[diff(f,x[i]) $ i=1..nops(x)] ))
    end_if
end_proc:

