// $Date: 2000/04/04 15:21:26 $  $Author: frankp $   $Revision: 1.8 $
// frankp, 31/10/94

/*
        linalg::isUnitary -- tests for unitary matrices

        isUnitary(A)

        A: square matrix

        This function tries to find out if A is unitary.

        Matrix A is orthogonal iff A^(-1) = transpose(conjugate(A)). This is 
        equivalent to the fact that the columns of A build an orthonormal system 
        (in respect to the scalar product defined by linalg::scalarProduct).
        
        Note that one can only be sure that A is not an unitary matrix
        if the axiom 'canonicalRep' is stated for the coefficient domain 
        of A. In respect to axiom 'canonicalRep' is stated or not, FALSE
        and UNKNOWN is returned, respectively.

        The coefficient domain of A has to be an integral domain.
*/

linalg::isUnitary:= proc(A)
    local i, j, n, R, c, col, ret, scalarProduct, Riszero, 
          Rone, Requal, Rzero;
begin
    if testargs() then
        if args(0) <> 1 then
            error("expecting one argument")
        elif A::dom::hasProp( Cat::Matrix ) <> TRUE then
            error("argument is not of 'Cat::Matrix'")
        elif not (A::dom::coeffRing)::hasProp( Cat::IntegralDomain ) then
            error("expecting matrix over 'Cat::IntegralDomain'")
        end_if
    end_if;
   
    n:= A::dom::matdim(A);
    if n[1] <> n[2] then 
        error("not a square matrix")
    end_if;

    R:= A::dom::coeffRing;

    scalarProduct:= linalg::scalarProduct;
    col:= A::dom::col;
    Requal:= R::equal;
    Rone:= R::one;
    Rzero:= R::zero;
    Riszero:= R::iszero;
    scalarProduct:= linalg::scalarProduct;

    n:= n[1];
    c:= [FAIL $ n]; // the columns of A
    for i from 1 to n do
        if c[i] = FAIL then c[i]:= col(A,i) end_if;
        if (ret:= Requal( scalarProduct( c[i] $ 2 ),Rone )) <> TRUE then
            return( ret )
        end_if;
        for j from i+1 to n do
            if c[j] = FAIL then c[j]:= col(A,j) end_if;
            if (ret:= Requal( scalarProduct( c[i],c[j] ),Rzero )) <> TRUE then
                return( ret )
            end_if
        end_for
    end_for;

    return( TRUE )
end_proc: 

