// $Date: 2000/04/17 13:53:21 $  $Author: frankp $   $Revision: 1.3 $
// K. F. Gehrs, 2000/02/25

/*
    linalg::minpoly -- returns the minimal polynomial of a square matrix

    linalg::minpoly(A,x)

    A: square matrix over a field
    x: identifier
     
    Computes the minimal polynomial of the matrix A in x, i.e. the polynomial, 
    not zero, of the smallest degree, which annihilates the matrix A.
    The algorithm mentioned below from the book Matrix Analysis is used in 
    a modified form here. 

    The coefficient domain of A has to be a field, i.e. a domain of category 
    'Cat::Field'.

    Reference: Matrix Analysis,
               by Roger A. Horn (The Johns Hopkins University) 
               by Charles R. Johnson (Clemson University)
               algorithm page 148/149 problem 5
               Cambridge University Press 1985 
*/

linalg::minpoly := proc(A,x)
    local Mat, T, L, B, h, i, j, l;
begin
    if args(0) <> 2 then
        error("expecting two arguments, a matrix of 'Cat::Matrix' and an identifier")
    end_if;

    Mat := A::dom;
    if testargs() then
        if Mat::hasProp( Cat::Matrix ) <> TRUE then
            error("expecting a matrix of 'Cat::Matrix'")
        elif not (Mat::coeffRing)::hasProp( Cat::Field ) then
            error("expecting matrix over 'Cat::Field'")
        end_if;
        if domtype(x) <> DOM_IDENT then
            error("2nd argument must be an identifier")
        end_if;
    end_if;
    l:= Mat::matdim(A);
    if l[1] <> l[2] then
        error("not a square matrix")
    end_if;

    l:= l[1];
    B:= Mat::identity(l);
    T:= array( 1..l^2,1..l^2+1, [[ Mat::coeffRing::zero $ l^2+1 ] $ l^2] );
    for h from 0 to l do
        ( ( T[(j-1)*l+i,h+1]:= B[i,j] ) $ i=1..l ) $ j=1..l;
        B:= B * A
    end_for;

    L:= linalg::nullspace( Mat::create(T) );

    return(Dom::DistributedPolynomial([x],Mat::coeffRing)(
            poly( [[ L[1][i],i-1 ] $ i=1..l^2],[x],Mat::coeffRing )
    ))
end_proc:

