// $Date: 1999/10/21 09:58:31 $  $Author: frankp $   $Revision: 1.4 $
// frankp, 06/09/94

/*
    linalg::orthog  --  returns an orthogonal system of vectors

	orthog( S )

	S : list or set of vectors

	Orthogonalize the vectors in S using the Gram-Schmidt process.

	The vectors must be of same dimensions and defined over a field. 
*/

linalg::orthog:= proc(S)
    local R, s, OS, bi, bj, aj, scalarProduct,
	  Riszero, viszero, vplus, Rmult, Rnegate, Rdivide;
begin
    if testargs() then
	if args(0) <> 1 then error("expecting one argument") end_if;
	OS:= Type::ListOf;
	case domtype(S)  
	of DOM_SET  do 
	    OS:= Type::SetOf
	of DOM_LIST do
	    if nops(S) = 0 then return( S ) end_if;

	    bi:= op(S,1);
	    if not testtype( bi,linalg::VectorOf(Type::AnyType) ) then
		error("expecting list or set of vectors")
	    end_if;
	    R:= bi::dom::coeffRing;
	    if not R::hasProp( Cat::Field ) then
		error("expecting vectors over a 'Cat::Field'")
	    end_if;
	    s:= max( op(bi::dom::matdim(bi)) );
	    if not testtype( S,OS(linalg::VectorOf(R,s)) ) then
		error("vectors are not compatible")
	    end_if;
	    break
	otherwise
	    error("expecting a list or set of vectors")
	end_case
    end_if;

    if nops(S) = 0 then return( S ) end_if;

    aj:= op(S,1);
    vplus:= aj::dom::_plus;
    viszero:= aj::dom::iszero;

    R:= aj::dom::coeffRing;
    Rmult:= R::_mult;
    Riszero:= R::iszero;
    Rnegate:= R::_negate;
    Rdivide:= R::_divide;
    scalarProduct:= linalg::scalarProduct;

    if domtype( S ) = DOM_LIST then delete S[1] else S:= subsop(S,1=null()) end_if;

    OS:= [aj];
    for aj in S do
        bj:= aj;
	for bi in OS do
	    s:= scalarProduct(bi,bi);
	    if Riszero( s ) then next end_if;
            s:= Rnegate( Rdivide(scalarProduct(bi,aj),s) );
	    if not Riszero( s ) then
                bj:= vplus( bj,map( bi,Rmult,s ) )
	    end_if
        end_for;
	if not viszero(bj) then OS:= append( OS,bj ) end_if
    end_for;
    
    if domtype(S) = DOM_LIST then OS else {op(OS)} end_if
end_proc:

