// $Date: 1999/10/22 15:34:19 $ $Author: frankp $ $Revision: 1.7 $
//  stefanw, 26.1.98

/*
	linalg::permanent(x)

	x - square matrix over a commutative ring


        computes the permanent of a matrix
        (Note: the permanent is defined like the determinant, but
	with the signs of the permutations not entering the definition.)
*/

linalg::permanent:= proc(x)
  local R,n,i;
begin
   if testargs() then
    if args(0) <> 1 then
      error("expecting 1 argument")
    end_if;
    if x::dom::hasProp(Cat::Matrix) <> TRUE then
      error("argument is not of 'Cat::Matrix'")
    end_if;
    if  not (x::dom::coeffRing)::hasProp(Cat::CommutativeRing) then
      error("expecting matrix over a 'Cat::CommutativeRing'")
    end_if;
    n:=x::dom::matdim(x);
    if n[1] <> n[2] then
      error("not a square matrix")
    end_if
  end_if;

  R:=x::dom::coeffRing;
  n:= x::dom::matdim(x)[1];
  case n
    of 1 do
      return( x[1,1] )
    of 2 do
      return(R::_plus( R::_mult(x[1,1], x[2,2]), R::_mult(x[1,2], x[2,1]) ))
    of 3 do
      return(R::_plus(
        R::_mult(x[1, 1], x[2, 2], x[3, 3]), 
        R::_mult(x[1, 1], x[2, 3], x[3, 2]), 
        R::_mult(x[2, 1], x[1, 2], x[3, 3]),
        R::_mult(x[2, 1], x[1, 3], x[3, 2]), 
        R::_mult(x[3, 1], x[1, 2], x[2, 3]),
        R::_mult(x[3, 1], x[1, 3], x[2, 2])
      ))
     otherwise
      return(R::_plus( R::_mult(x[i,1], 
	linalg::permanent(x::dom::delRow(x::dom::delCol(x,1), i))) 
	$ i = 1..n
      ) )
  end_case
end_proc:

