// $Date: 2000/05/25 08:35:58 $  $Author: tonner $   $Revision: 1.7 $
// frankp, 29/07/94

/*
    	linalg::scalarProduct(u,v) -- returns the Euclidian scalar product
				      of u and v where u and v are
				      vectors over the same coeffRing. 

	The Euclidian scalar product of two vectors with respect to
        the standard basis defined by

	    u[1]*conjugate(v[1]) + u[2]*conjugate(v[2]) + ...

        If the component ring of the vectors do not define
        an entry "conjugate" the scalar product

            u[1]*v[1] + u[2]*v[2] + ...

        is used.
*/

linalg::scalarProduct := proc(u,v)
    local i, R, Rmult, Rnormal;
begin
    if testargs() then
	if args(0) <> 2 then error("expecting 2 arguments") end_if;
	if not testtype( u,linalg::VectorOf(Type::AnyType) ) then
	    error("expecting two vectors")
	end_if;
	if not testtype(v,linalg::VectorOf(u::dom::coeffRing,u::dom::nops(u))) then
	    error("vectors are not compatible")
	end_if
    end_if;

    R:= u::dom::coeffRing;
    Rmult:= R::_mult;
    if (Rnormal:= R::normal) = FAIL then
    	if R::hasProp(Ax::systemRep) then Rnormal:= normal else Rnormal:= id end_if
    end_if;

    if R::conjugate = FAIL then
        return( Rnormal(R::_plus(
            Rmult(u[i],v[i]) $ i=1..nops(u)
        )) )
    else
        return( Rnormal(R::_plus( 
            Rmult(u[i],R::conjugate(v[i])) $ i=1..nops(u)
        )) )
    end_if
end_proc:

linalg::scalarProduct := funcenv(linalg::scalarProduct):
linalg::scalarProduct::Content := 
   (Out, data) ->
   Out::Capply(Out::Cscalarproduct, map(op(data), Out)):
