/*- 
 addEdge - adds one or several edges to a Network
 addEdge(G, e)
 addEdge(G, e, <Eweight=c>, <Capacity=t>)
 addEdge(G, [e1,..,en])
 addEdge(G, [e1,..,en], <Eweight=[c1,..,cn]>,<Capacity=[t1,..,tn]>)
-*/
Network::addEdge := proc(G)
local Ed,Ew,Ecap,Epr,Epo,i,a,b,c,e,f,Lhs,Rhs;
begin
   if testargs() then
      if args(0) < 2 or args(0) > 4 then
	     error("Wrong number of arguments!")
      end_if;
      Network::graph_check(G);
   end_if;
   Ed :=  Network::edge(G);
   Ew := Network::eWeight(G);
   Ecap := Network::eCapacity(G);
   Epo := Network::epost(G);
   Epr := Network::epre(G);
   
   if testtype(args(2), Type::ListOf(DOM_LIST)) then
      a := [args()];
      f := proc() begin testtype(args(1), DOM_LIST) end_proc ;
      b := op(select(a, f));
      f := proc() begin testtype(args(1), "_equal") end_proc ;
      c := select(a,f);
      /*-
        check whether edges are specified correctly
      -*/
       if testargs() then
   	      if not testtype(b, Type::ListOf(DOM_LIST)) then
   	         error("Wrong edge specification!")
   	      end_if;
   	      for e in b do
   	         if not testtype(e, Type::ListOf(Type::AnyType,2,2)) then
   		        error("Wrong edge specification!");
   	         end_if;
		  if not Network::isVertex(G,e[1]) or not Network::isVertex(G,e[2]) then
                     error("specified node is not part of the Network!");
                  end_if;
                  if contains(Ed,e) <> 0 then
                    error("specified edge already contained in Network!");
                  end_if;
   	      end_for;
       end_if;	
 
      Ed := Ed.b;
      for e in b do
          if type(Epo[e[1]])<>DOM_LIST then Epo[e[1]]:=[] end_if;
          Epo[e[1]] := append(Epo[e[1]], e[2]);
          if type(Epr[e[2]])<>DOM_LIST then Epr[e[2]]:=[] end_if;
          Epr[e[2]] := append(Epr[e[2]], e[1]);
	  Ew[e] := 1;
	  Ecap[e] := 1;
      end_for;

      for i in c do
         Lhs := op(i,1); Rhs := op(i,2);
         case Lhs
            of hold(Eweight) do
		/*--
   	        if not testtype(Rhs, Type::ListOf(Type::Numeric)) then
   		       error("Non numeric edge costs!");
   	        end_if;
		--*/
   	        if nops(Rhs) <> nops(b) then 
   		       error("Length of lists must be equal!");
   	        end_if;
   	        for i from 1 to nops(b) do
   		       Ew[b[i]] :=  Rhs[i]
   	        end_for;
   	        break;
      	    of hold(Capacity) do
		/*--
   	        if not testtype(Rhs, Type::ListOf(Type::Numeric)) then
   		       error("Non numeric edge capacities!");
   	        end_if;
		--*/
   	        if nops(Rhs) <> nops(b) then 
   		       error("Length of lists must be equal!");
   	        end_if;
   	        for i from 1 to nops(b) do
   		       Ecap[b[i]] :=  Rhs[i]
   	        end_for;
   	        break;
   	    otherwise
   	        error("Unknown type!");
   	 end_case;
       end_for;


   else //- one single edge is changed -
	a := [args()];
        f := proc() begin testtype(args(1), DOM_LIST) end_proc ;
        b := op(select(a, f));
        f := proc() begin testtype(args(1), "_equal") end_proc ;
        c := select(a,f);
   
        /*-
        check whether edge is specified correctly
        -*/
        if testargs() then
   	       if not testtype(b, Type::ListOf(Type::AnyType,2,2)) then
   	          error("Wrong edge specification!")
   	       end_if;
	       if not Network::isVertex(G,b[1]) or not Network::isVertex(G,b[2]) then
                  error("specified node is not part of the Network!");
               end_if;
               if contains(Ed,b) <> 0 then
                  error("specified edge already contained in Network!");
               end_if;
        end_if;	
        Ed := append(Ed,b);
	Ew[b] := 1;
        Ecap[b] := 1;
        if type(Epo[b[1]])<>DOM_LIST then Epo[b[1]]:=[] end_if;
        Epo[b[1]] := append(Epo[b[1]], b[2]);
        if type(Epr[b[2]])<>DOM_LIST then Epr[b[2]]:=[] end_if;
        Epr[b[2]] := append(Epr[b[2]], b[1]);
   	
   	for i in c do
   	      Lhs := op(i,1); Rhs := op(i,2);
   	      case Lhs
      	   	 of hold(Eweight) do
			/*--
   		        if not testtype(Rhs, Type::Numeric) then
   			       error("Non numeric edge cost!");
   		        end_if;
			--*/
   		        Ew[b] :=  Rhs;
   		        break;
      	   	 of hold(Capacity) do
			/*--
   		        if not testtype(Rhs, Type::Numeric) then
   			       error("Non numeric edge capacitiy!");
   		        end_if;
			--*/
   			Ecap[b] :=  Rhs;
   		        break;
   	         otherwise
   		        error("Unknown type!");
   	       end_case;
	    end_for;
	 end_if;
     	
     new(Network, Network::vertex(G), Ed, Network::vWeight(G), Ecap, Ew, Epo, Epr);
end_proc:
