/*- 
 addVertex - adds one or several vertices to a Network
 addVertex(G,x)
 addVertex(G,x,Vweight=n)
 addVertex(G,[x1,..,xn])
 addVertex(G,[x1,..,xn], Vweight=[v1,..vn])
-*/
Network::addVertex := proc(G)
local V,i,Vw,Lhs,Rhs;
begin
   if testargs() then
      if args(0) < 2 or args(0) > 3 then
	     error("Wrong number of arguments!")
      end_if;
      Network::graph_check(G);
   end_if;
   
   case args(0) 
     of 2 do
	if testtype(args(2), DOM_LIST) then
	    V := Network::vertex(G).args(2);
	    Vw := Network::vWeight(G);
	    for i in args(2) do
		Vw[i] := 0
      	    end_for;
	else
	   V := append(Network::vertex(G),args(2));
	   Vw := Network::vWeight(G);
	   Vw[args(2)] := 0;
	end_if;
	break;
     of 3 do
	if not testtype(args(3), "_equal") then
	   error("Wrong vertex weights specification!");
	end_if;
	Lhs := op(args(3),1); Rhs := op(args(3),2);
	if Lhs <> hold(Vweight) then
	   error("Vertex weight specification!")
	end_if;

	if testtype(args(2), DOM_LIST) then
	   /*--
	   if not testtype(Rhs, Type::ListOf(Type::Numeric)) then
		error("Non numeric vertex weights!");
	   end_if;
	   --*/
	   if nops(args(2)) <> nops(Rhs) then
		error("Length of lists must be equal!");
	   end_if;
	   V := Network::vertex(G).args(2);
	   Vw := Network::vWeight(G);
	   for i from 1 to nops(Rhs) do
		Vw[args(2)[i]] := Rhs[i];
	   end_for;
	else
	   /*--
	   if not testtype(Rhs, Type::Numeric) then
		error("Non numeric vertex weight!");
	   end_if;
	   --*/
	   V := append(Network::vertex(G),args(2));
	   Vw := Network::vWeight(G);
	   Vw[args(2)] := Rhs;
	end_if;
	break;
     otherwise
	error("Wrong number of arguments!")
     end_case;
     new(Network, V, Network::edge(G), Vw, Network::eCapacity(G), 
	 Network::eWeight(G), Network::epost(G), Network::epre(G));
end_proc:
