/*- 
 admissibleFlow - checks a flow for admissibility
 admissibleFlow(N,f)
 Parameters:
    N - Network
    f - flow in N (a table with f[e] = c, when c units flow over the edge e)
 Returns TRUE, if f is an admissible flow in N. FALSE otherwise.
-*/
Network::admissibleFlow := proc(N,f)
local Ed,Ecap,Eset,item,edge,value;
begin 
   if testargs() then
      if args(0) <> 2 then
         error("Wrong number of arguments!");
      end_if;
      Network::graph_check(N);
      if not testtype(f, DOM_TABLE) then
         error("Illegal argument (flow must be a DOM_TABLE)!");
      end_if;
      if not testtype(f, Type::TableOfIndex(DOM_LIST/*,2,2*/)) then
         error("Illegal argument (flow must be a DOM_TABLE with edges as indices)!");   
      end_if;
      if not testtype(f, Type::TableOfEntry(Type::Numeric)) then
         error("Illegal argument (flow values must be numeric values)!");
      end_if;
   end_if;
   
   Ed := Network::edge(N);
   Ecap := Network::eCapacity(N); 
   Eset := [];
   for item in f do 
      edge := op(item,1);
      Eset := append(Eset, edge);
      value := op(item,2);
      if not Network::isEdge(N, edge) then
         return(FALSE);
      end_if;
      if value < 0 or value > Ecap[edge] then
         return(FALSE);
      end_if;
   end_for;
   if {op(Eset)} <> {op(Ed)} then
      return(FALSE);
   end_if;
   TRUE
end_proc:
