/*- 
 changeEdge - change cost/capacity of one or several edges in a Network
 changeEdge(G, e, <Eweight=c>, <Capacity=t>)
 changeEdge(G, [e1,..,en], <Eweight=[c1,..,cn]>,<Capacity=[t1,..,tn]>)
-*/
Network::changeEdge := proc(G)
local Ed,Ew,Ecap,i,a,b,c,e,f,Lhs,Rhs;
begin
   if testargs() then
      if args(0) < 3 or args(0) > 4 then
	     error("Wrong number of arguments!")
      end_if;
      Network::graph_check(G);
   end_if;
   
   Ed :=  Network::edge(G);
   Ew := Network::eWeight(G);
   Ecap := Network::eCapacity(G);
   
   if testtype(args(2), Type::ListOf(DOM_LIST)) then
      a := [args()];
      f := proc() begin testtype(args(1), DOM_LIST) end_proc ;
      b := op(select(a, f));
      f := proc() begin testtype(args(1), "_equal") end_proc ;
      c := select(a,f);
   
      /*-
        check whether edges are specified correctly
      -*/
       if testargs() then
   	      if not testtype(b, Type::ListOf(DOM_LIST)) then
   	         error("Wrong edge specification!")
   	      end_if;
   	      for e in b do
   	         if not testtype(e, Type::ListOf(Type::AnyType,2,2)) then
   		        error("Wrong edge specification!");
   	         end_if;
   	         if not Network::isEdge(G,e) then
   		        error("Specified edge is not part of the Network!");
   	         end_if;
   	      end_for;
       end_if;	
   	
   	   for i in c do
   	      Lhs := op(i,1); Rhs := op(i,2);
   	      case Lhs
      	   	 of hold(Eweight) do
			/*--
   		        if not testtype(Rhs, Type::ListOf(Type::Numeric)) then
   			       error("Non numeric edge costs!");
   		        end_if;
			--*/
   		        if nops(Rhs) <> nops(b) then 
   			       error("Length of lists must be equal!");
   		        end_if;
   		        for i from 1 to nops(b) do
   			       Ew[b[i]] :=  Rhs[i]
   		        end_for;
   		        break;
      	   	 of hold(Capacity) do
			/*--
   		        if not testtype(Rhs, Type::ListOf(Type::Numeric)) then
   			       error("Non numeric edge capacities!");
   		        end_if;
			--*/
   		        if nops(Rhs) <> nops(b) then 
   			       error("Length of lists must be equal!");
   		        end_if;
   		        for i from 1 to nops(b) do
   			       Ecap[b[i]] :=  Rhs[i]
   		        end_for;
   		        break;
   	         otherwise
   		        error("Unknown type!");
   	       end_case;
	    end_for;
   else //- one single edge is changed -
	a := [args()];
        f := proc() begin testtype(args(1), DOM_LIST) end_proc ;
        b := op(select(a, f));
        f := proc() begin testtype(args(1), "_equal") end_proc ;
        c := select(a,f);
   
        /*-
        check whether edge is specified correctly
        -*/
        if testargs() then
   	       if not testtype(b, Type::ListOf(Type::AnyType,2,2)) then
   	          error("Wrong edge specification!")
   	       end_if;
   	       if not Network::isEdge(G,b) then
   		      error("Specified edge is not part of the Network!");
   	       end_if;
        end_if;	
   	
   	    for i in c do
   	      Lhs := op(i,1); Rhs := op(i,2);
   	      case Lhs
      	   	 of hold(Eweight) do
			/*--
   		        if not testtype(Rhs, Type::Numeric) then
   			       error("Non numeric edge cost!");
   		        end_if;
			--*/
   		        Ew[b] :=  Rhs;
   		        break;
      	   	 of hold(Capacity) do
			/*--
   		        if not testtype(Rhs, Type::Numeric) then
   			       error("Non numeric edge capacitiy!");
   		        end_if;
			--*/
   			Ecap[b] :=  Rhs;
   		        break;
   	         otherwise
   		        error("Unknown type!");
   	       end_case;
	    end_for;
	 end_if;
     	
     new(Network, Network::vertex(G), Ed, Network::vWeight(G), 
	 Ecap, Ew, Network::epost(G), Network::epre(G));
end_proc:
