/*-
 Network::convertSSQ - converts a Network to a single source, single sink Network
 convertSSQ(N,q,s)
 Parameters:
   N - Network
   q - source (new node)
   s - sink (new node)
 Converts a Network to a single source, single sink Network.  The given nodes q and s 
 are inserted in the Network. The are connected to the other nodes in the following way:
   [q,i] for every i with Network::vWeight[i] > 0
   [i,s] for every i with Network::vWeight[i] < 0
   eCapacity[[q,i]] := vWeight[i]   eWeight[[q,i]] := 0
   eCapacity[[i,s]] := eWeight[i]   eWeight[[i,s]] := 0
-*/
Network::convertSSQ := proc(N,q,s)
local V,Vw,Ed,Ew,Ecap,f,Q,S,i,new_Edq,new_Eds;
begin
   if testargs() then
      Network::graph_check(N);
   end_if;

   V := Network::vertex(N);

   if testargs() then
      if contains(V,q) <> 0 or contains(V,s) <> 0 then
	     error("Specified nodes are already nodes of the Network!");
      end_if;
   end_if;

   Vw := Network::vWeight(N);
   f := proc() begin bool(Vw[args(1)] > 0) end_proc ;
   Q := select(V,f);
   f := proc() begin bool(Vw[args(1)] < 0) end_proc ;
   S := select(V,f);
   Ed := Network::edge(N);
   new_Edq := [[q,Q[i]] $ i=1..nops(Q)];
   new_Eds := [[S[i],s] $ i=1..nops(S)];
   Ew := Network::eWeight(N);
   Ecap := Network::eCapacity(N);
   Vw[q] := 0;
   Vw[s] := 0;
   for i in Q do 
	Ew[[q,i]] := 0;
	Ecap[[q,i]] := Vw[i];
	Vw[q] := Vw[q] + Vw[i];
	Vw[i] := 0;
   end_for;
   for i in S do 
	Ew[[i,s]] := 0;
	Ecap[[i,s]] := -Vw[i];
	Vw[s] := Vw[s] + Vw[i];
	Vw[i] := 0;
   end_for;
   Ed := Ed.new_Edq.new_Eds;
   V := V.[q,s];
   Network(V,Ed, hold(Vweight) = Vw, hold(Eweight) = Ew, hold(Capacity)=Ecap);
end_proc:



