/*-
 residualNetwork - computes the residual network
 residualNetwork(N,f <,mode>)
 Parameters:
    N - network
    f - flow in N (a table with f[e] = c, when c units flow over the edge e)
    mode - (optionally) mode = Extended, then also those edges with a zero
           residual capacity are contained. Otherwise those edges are omitted.
-*/
Network::residualNetwork := proc(N,flow)
local Ed,e,Ecap,de,der,dec,dew,ce,cec,cew,N1,Ew,reverse;
begin
  if testargs() then
    if args(0) < 2 or args(0) > 3 then
       error("Wrong number of arguments!");
    end_if;
    Network::graph_check(N);
    if not Network::admissibleFlow(N,flow) then
       error("Specified flow is inadmissible in the given network!");
    end_if;   
    if args(0) = 3 then 
       if args(3) <> hold(Extended) then
          error("Illegal argument!");
       end_if;
    end_if; 
  end_if; 
        
  reverse := proc() begin [args(1)[2],args(1)[1]] end_proc ;
  
  Ed := Network::edge(N);
  Ecap := Network::eCapacity(N);
  Ew := Network::eWeight(N);
  if args(0) = 3 then   // Extended network: don't omit empty edges 
      der := []; dec := []; dew := [];
      ce := []; cec := []; cew := [];
      for e in Ed do
          if flow[e] > 0 then
             ce := append(ce,e);
             cec := append(cec, Ecap[e]-flow[e]);
             if not Network::isEdge(N, reverse(e)) then
                der := append(der, reverse(e));
                dec := append(dec, flow[e]);
                dew := append(dew, -Ew[e]); 
             end_if;
          end_if;
      end_for;
      N1 := N;
      if nops(der) > 0 then
          N1 := Network::addEdge(N1, der, hold(Capacity)=dec, hold(Eweight)=dew);
      end_if;
      if nops(ce) > 0 then
          N1 := Network::changeEdge(N1, ce, hold(Capacity)=cec);
      end_if;
  else // omit empty edges 
      de := []; der := []; dec := []; dew := [];
      ce := []; cec := []; cew := [];
      for e in Ed do
          if flow[e] > 0 then
             if flow[e] = Ecap[e] then
                de := append(de,e);
             else
                ce := append(ce,e);
                cec := append(cec, Ecap[e]-flow[e]);
             end_if;
             if not Network::isEdge(N, reverse(e)) then
                der := append(der, reverse(e));
                dec := append(dec, flow[e]);
                dew := append(dew, -Ew[e]); 
             end_if;
          end_if;
      end_for;
      N1 := N;
      if nops(de) > 0 then
         N1 := Network::delEdge(N1,de);
      end_if;
      if nops(der) > 0 then
          N1 := Network::addEdge(N1, der, hold(Capacity)=dec, hold(Eweight)=dew);
      end_if;
      if nops(ce) > 0 then
          N1 := Network::changeEdge(N1, ce, hold(Capacity)=cec);
      end_if;
  end_if;
  N1;
end_proc:

