
/*

 numeric::testfloat(f, a, b)
 numeric::testfloat(f, a, b, s)

 f    - eine Prozedur
 a, b - reelle Zahlen mit rl < ru
 s    - ganze Zahl > 0, optional - Defaultwert ist 100

 Berechnet werden die Teilintervalle von [a, b], 
 in denen f(x) reellwertig ist. s gibt dabei die
 Anzahl der Funktionsauswertungen an, d.h. es wird
 f(x) f.a. x = a+n*i, n=(b-a)/s, i=0..s ausgewertet.

 Zurueckgegeben wird eine Liste 
   [ ri, nri, fmin, fmax ] wobei
 ri und nri Listen der Form [[a_1,b_1],...,[a_n,b_n]]
 sind. ri enthaelt die Teilintervalle in denen f(x)
 reellwertig ist, nri die Teilintervalle in denen f(x)
 nicht reellwertig ist. ri bzw. nri koennen auch die 
 Form [[]] haben. 
 fmin = min f(x), fmax = max f(x) f.a. x aus ri

*/

numeric::testfloat := proc( f : DOM_PROC, 
                            a : Type::Real, 
			    b : Type::Real,
			    s : DOM_INT     )
  local nargs;
begin 
  nargs := args(0);
  if nargs > 2 and nargs < 5 then
     if nargs = 3 then
        s := 100;
     end_if;
  else
     error("Wrong number of arguments");
  end_if;
  if a >= b then
     error("a < b expected");
  end_if;
  numeric::testfloat_rec(f, a, b, (b-a)/s, [], [], infinity, -infinity) 
end_proc:

numeric::testfloat_rec := proc(f, rl, ru, s, ri, nri, ymin, ymax)
   local a, b, eps, x, xi, y;
begin
   /* ri contains the intervals, where f(x) is real */
   a := float(rl); b := float(ru); eps := 10^(-DIGITS+1);
   x := a; xi := a;
   while x <= b do
       y := float(f(x));
       if not testtype(y, Type::Real) then
          if x > a then
	     [xi, ymin, ymax] := 
	          numeric::bisection_testfloat(f, x-s, x, ymin, ymax);
	     if abs(a-xi) > eps then
                ri := append(ri, [a, xi]);
	     end_if
	  end_if;
	  x := x+s;
          while x <= b and not testtype(float(f(x)), Type::Real) do
              x := x+s
          end_while;
          if x < b then
             [x, ymin, ymax] := 
	         numeric::bisection_testfloat(f, x, x-s, ymin, ymax);
	     if abs(xi-x) > eps then
	        nri := append(nri, [xi, x]);
 	     end_if;
             [ri, nri, ymin, ymax] := 
	          numeric::testfloat_rec(f, x, b, s, ri, nri, ymin, ymax);
	  else
	     if abs(xi-b) > eps then
	        nri := append(nri, [xi, b]);
	     end_if
          end_if;
          return( [ ri , nri, ymin, ymax ] )
       else
          x := x+s;
          if y < ymin then 
	     ymin := y
          end_if;
	  if y > ymax then
	     ymax := y
	  end_if;
       end_if
   end_while;
   return( [ append(ri, [a, b]) , nri , ymin, ymax ] )
end_proc:

/*
x0 - real number
x1 - non real number 
*/
numeric::bisection_testfloat := proc(f, x0, x1, ymin, ymax)
   local c, eps, y;
begin
   eps := 10^(-DIGITS-5);
   while abs(x1-x0) > eps do
     c := (x0+x1)/2;
     y := float(f(c));
     if testtype(y, Type::Real) then
        x0 := c; 
	if y < ymin then 
	   ymin := y
	elif y > ymax then
	   ymax := y
	end_if;
     else
        x1 := c;
     end_if
   end_while;
   return( [ x0, ymin, ymax ] )
 end_proc:
