
/*++ ---------------- SimpSol.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 13 March   1998
Last Update : 13 March   1998


Description:
This file contains functions to simplify solutions of ordinary differential 
equations. 
NOTE: THIS FUNCTION IS UNDER CONSTRUCTION!

Functions: 

 - used Parameter:
    ++ sols   :{DOM_EXPR,DOM_LIST,DOM_SET}
    ++ sols    = a set or a list or a sequence of expressions

 - ode::simplifySolutions(sols)
    ++ returns simpler solution expressions equivalent to sols. 
    ++ Currently only exponential functions
    ++ are combined recursively, integrals are evaluated and the whole
    ++ solution expressions are factorized.
 
See:
 
Examples:
>> Ly:=diff(y(x),x$2)-(x^2-x)*diff(y(x),x)+x*y(x);

                                                        2
            x y(x) + diff(y(x), x, x) - diff(y(x), x) (x  - x)


>> ode::liouvillianSolutions(Ly,y(x)); 

{      /    /    /        2    3        \ \    \    /  2    3 \
{      |    |    | x - 2 x  + x  - 2    | |    |    | x    x  |
{ x int| exp| int| -----------------, x | |, x | exp| -- - -- |
{      \    \    \       x - 1          / /    /    \ 4    6  /

   /    /    2 \    /  3 \ \1/2      /    /    /        2    3        \ \
   |    |   x  |    | x  | |         |    |    | x - 2 x  + x  - 2    | |
   | exp| - -- | exp| -- | |    - int| exp| int| -----------------, x | |,
   \    \   2  /    \ 3  / /         \    \    \       x - 1          / /

     \    /  2    3 \ /    /    2 \    /  3 \ \1/2
     |    | x    x  | |    |   x  |    | x  | |
   x | exp| -- - -- | | exp| - -- | exp| -- | |   ,
     /    \ 4    6  / \    \   2  /    \ 3  / /

        /  2    3 \ /    /    2 \    /  3 \ \1/2
        | x    x  | |    |   x  |    | x  | |
   x exp| -- - -- | | exp| - -- | exp| -- | |    -
        \ 4    6  / \    \   2  /    \ 3  / /

      /  2    3 \ /    /    2 \    /  3 \ \1/2 }
      | x    x  | |    |   x  |    | x  | |    }
   exp| -- - -- | | exp| - -- | exp| -- | |    }
      \ 4    6  / \    \   2  /    \ 3  / /    }


>> ode::simplifySolutions(%); 

                {                   /    /  3    2 \    \ }
                {                   |    | x    x  |    | }
                {                   | exp| -- - -- |    | }
                {                   |    \ 3    2  /    | }
                { x - 1, (x - 1) int| --------------, x | }
                {                   |   2               | }
                {                   \  x  - 2 x + 1     / }

++*/

sysassign(ode::simplifySolutions, proc(s)
  local sols;
  begin
    sols := s; // this statement performs some evaluations 
    sols := misc::maprec(sols,{"int"}=proc()
	   begin intlib::int(expand(op(args(),1)),op(args(),2)) end_proc );
//    sols := misc::maprec(sols,{"_mult","_power"}=fun(combine(args(),exp)));
    sols := misc::maprec(sols,{"_mult"}=proc() begin combine(_mult(args()),exp) end_proc );
    sols := misc::maprec(sols,{"_power"}=proc() begin _power(combine(op(args(),1),exp),
                                                      op(args(),2)) end_proc );
    map(sols, e->Factored::convert_to(factor(simplify(e)),DOM_EXPR))
  end_proc):

