// $Date: 2000/11/24 11:59:01 $ $Author: fakler $ $Revision: 1.12 $ 

/*  Input: eq is a first-order ODE in y(x).
   Output: a solution or FAIL.
   Example: ode::clairaut((x*diff(y(x),x)-y(x))^2-diff(y(x),x)^2-1,y,x); 
   Reference: Zwillinger chap. 38 page 158
*/

ode::clairaut := proc(eq,y,x) 
local f;
save Z, yp;
begin
   delete yp;
   eq:=expand(subs(eq,diff(y(x),x)=yp,y(x)=x*yp-Z));
   if not has(eq,{x,y}) and type(eq)="_plus" then
      f:=select(eq,has,Z);
      if not has(f,yp) then
         userinfo(1,"Clairaut's equation");
         return(ode::clairautEq(subs(f,Z=x),subs(f-eq,yp=x),x))
      end_if
   end_if;
   FAIL
end_proc:

/* solves f(x*y'-y) = g(y').
   A general solution is given by f(xC-y)=g(C) where C is a constant.
   Example: ode::clairautEq(x^2,x^2-1,x);
*/
ode::clairautEq := proc(_f_,_g_,x) // f and g are expressions in x 
local C,gensol,eq,partsol;
save y, yp;
begin
   delete y,yp;
   userinfo(3,"f=",_f_,"g=",_g_);
   // search for a general solution 
   C:=genident("C");
   gensol:=solvelib::discreteSolve(subs(_f_,x=x*C-y)=subs(_g_,x=C),y);
   //lev:=max(2-ode::depth,ode::maxdegree);
   //gensol:=solvelib::discreteSolve(subs(_f_,x=x*C-y)=subs(_g_,x=C),y,
   //                                MaxDegree=lev);
   if gensol=FAIL then
     userinfo(3,"finding general solution failed while solving ".
              expr2text(subs(_f_,x=x*C-y)=subs(_g_,x=C))." in ".expr2text(y));
     return(FAIL);
   else
     userinfo(3,"general solution is",gensol);
   end_if;
   // search for a singular solution 
   eq:=subs(diff(_f_,x),x=x*yp-y)*x-subs(diff(_g_,x),x=yp);
   partsol:=solvelib::discreteSolve(eq,yp,MaxDegree=1);
   //partsol:=solvelib::discreteSolve(eq,yp,MaxDegree=ode::maxdegree);
   userinfo(3,"singular solution for y' is",partsol);
   //if type(partsol)="solve" then partsol:={} end_if;
   // ugly poor work around
   if type(partsol)="solve" or partsol=FAIL then partsol:={} end_if;
   if partsol<>{} then
      partsol:=op(partsol,1);
      eq:=numer(subs(_f_,x=x*partsol-y)-subs(_g_,x=partsol));
      partsol:=solvelib::discreteSolve(eq,y);
      //partsol:=solvelib::discreteSolve(eq,y,MaxDegree=lev);
      userinfo(3,"particular solution is",partsol);
      if partsol=FAIL then partsol:={} end_if; // ugly poor work around
   end_if;
   gensol union partsol
end_proc:
