// $Date: 2000/05/21 14:57:12 $ $Author: bij $ $Revision: 1.12 $ 

/* ode::solvable_for_y(eq,y,x) tries to solve the first-order ODE eq=0 wrt y(x)
   using the method of "solvable in y" equations (cf Zwillinger pages 305-306).
   There is a typo in the example given by Zwillinger: equation 
   (82.6) should be
   p'=p/x/(1-p^2) instead of p'=px/(1-p^2), but the solution (82.7) is correct.

   Output: either FAIL or a set of solutions.

   Example:
>> ode::solvable_for_y(y(x)-x-x*diff(y(x),x),y,x);

                           {x + x (C5 - ln(x))}

>> trace(ode::solve_eq):
>> ode::solvable_for_y(x-y(x)*diff(y(x),x)+x*diff(y(x),x)^2,y,x);
trace, enter ode::solve_eq with args=, p(x)-x*diff(p(x), x) + x*p(x)^2*diff(p(x), x), p, x, {}
trace, exit ode::solve_eq with result=, solve(C1 + 1/p*x*exp(p^2*1/2), p)
*/

ode::solvable_for_y := proc(eq,y,x) local p,yy,sol;
begin
   userinfo(2,"trying to recognize an equation solvable for y");
   p:=genident("p");
   eq:=subs(eq,diff(y(x),x)=p,y(x)=y);
   // Note, I use MaxDegree only, since otherwise solve needs too much time
   sol:=solvelib::discreteSolve(eq,y,MaxDegree=1); //,MaxDegree=1);
   if not ode::isResultOfSolveOK(sol,"NoRootOf") then return(FAIL) end_if;
   //if type(sol)="solve" or sol=FAIL then return(FAIL) end_if;
   //sol:=select(sol,proc() begin bool(type(args(1))<>"RootOf") end_proc );
   //if nops(sol)=0 then return(FAIL) end_if;
   userinfo(1,"equation solvable for y");
   yy:=op(sol,1); // yy=f(x,p) 
   sol:=numer(diff(subs(yy,p=p(x)),x)-p(x));
   sysassign(ode::depth,ode::depth+1);
   sol:=ode::solve_eq(sol,p,x,{});
   sysassign(ode::depth,ode::depth-1);
   if type(sol)<>DOM_SET or has(sol,FAIL) then
      userinfo(1,"solvable for y method failed");return(FAIL) 
   end_if;
   sol:=map(sol,proc() begin eval(subs(eq,p=args(1))) end_proc );
   sol:=map(sol,solvelib::discreteSolve,y);
   userinfo(1,"solvable for y method worked");
   map(sol,()->(if type(args(1))="solve" or type(args(1))="RootOf" then
                  args(1) else op(args(1)) end_if))
end_proc:
