// $Date: 2000/11/28 18:53:49 $ $Author: fakler $ $Revision: 1.23 $ 

/* main function to solve a single differential equation eq=0 in y(z) 
  with initial conditions inits */


ode::solve_eq := proc(eq,y,z,inits={}:DOM_SET)
local _l,i,den,s;
// global ode::depth; 
begin
   if ode::depth>ode::maxDepth then return({FAIL}) end_if;
   if type(eq)="_equal" then eq:=op(eq,1)-op(eq,2) end_if;
   if has(eq,exp) then eq:=combine(eq,exp) end_if; // simplify exp(t)*exp(-t) 
   if iszero(eq) then return({y(z)}) end_if;
   den:=denom(eq); // must not vanish 
   eq:=numer(eq);
   userinfo(2,"trying to factor");
   _l:=maprat(eq,factor); // to avoid problems with gcd 
   userinfo(2,"factorization ".
	(if nops(_l)=3 then "failed" else "worked" end_if));
   s:=_union(ode::solve_eq_irred(_l[2*i],y,z,inits,den) $ i=1..nops(_l) div 2);
   if calledFromWithinDSolve = TRUE then
     if ode::isResultOfSolveOK(s,"AllowImplicit") 
       then s
     else // try to extract partial solutions which are ok
       if type(s)="_union" and (s:=select([op(s)],testtype, DOM_SET))<>[]
         then _union(op(s))
       else return({FAIL})
       end_if;
     end_if;
   else s
   end_if;
end_proc:


// returns a list of solutions or [] if no solution found 

ode::solve_eq_irred :=
proc(eq,y,z,inits,den)
  local n,s,_l,sol,Lhs,_a,i,csts,islin;
  save yp;
begin
  islin := FALSE;
   userinfo(2,"solving ODE",eq,"with respect to",y(z));
   delete yp;
   if not testtype(eq,Type::ODE(y,z)) then
      error("not an ordinary differential equation in the given variable")
   end_if;
   n:=ode::order(eq,{y});
   if n=-1 then return({}) end_if; // no y(..) 
   userinfo(2,"ordinary differential equation of order",n);
   if n=0 then
      s:=subs(eq,y(z)=yp);
      sol:=solvelib::discreteSolve(s,yp);
      //# solve up to quadratics here, don't use ode::maxdegree #
      //sol:=solvelib::discreteSolve(s,yp,MaxDegree=2);
   elif (sol:=ode::separate(eq,y,z,n))=FAIL then
      // first try to recognize linear equations 
      _l:=[diff(y(z),z$i) $ i=0..n];
      if (_l:=Type::Linear(eq,_l))<>FALSE then islin:=TRUE;
        sol:=ode::solve_linear(_l,z,n)
      else // non linear 
	 userinfo(2,"non linear ordinary differential equation");
	 sol:=ode::autonomousEq(eq,y,z,n);
	 // exact n-th order equations 
	 if sol=FAIL then
	    if n=1 then
               sol:=ode::exact_first_order(eq,y,z);
               if sol=FAIL then 
                  sol:=ode::homogeneous(eq,y,z) end_if;
               if sol=FAIL then 
                  sol:=ode::int_factors(eq,y,z) end_if;
               if sol=FAIL then 
                  sol:=ode::lagrange(eq,y,z) end_if;
               if sol=FAIL then 
                  sol:=ode::riccati(eq,y,z) end_if;
               //if sol=FAIL then 
               //   sol:=ode::solvable_for_x(eq,y,z) end_if;
               //solvable_for_y currently not used because it can return only
               //parametric solutions, which are not yet dealt with 
               //if sol=FAIL then 
               //  sol:=ode::solvable_for_y(eq,y,z) end_if; 
	    elif n=2 then sol:=ode::exact_second_order(eq,y,z)
	    end_if
	 end_if;
	 if sol=FAIL then sol:=ode::bernoulli(eq,y,z) end_if;
	 if sol=FAIL then sol:=ode::clairaut(eq,y,z) end_if;
	 if sol=FAIL then sol:=ode::differentiation(eq,y,z) end_if;
	 if sol=FAIL and n=1 then sol:=ode::contact(eq,y,z) end_if;
	 if sol=FAIL and n=1 then sol:=ode::solvable_for_x(eq,y,z) end_if;
      end_if;
      // currently factoring of diff. operator only implemented for linear eqs 
      if sol=FAIL and islin=TRUE then 
         sol:=ode::factor(eq,y,z,n) end_if;
      if sol=FAIL then sol:=ode::ymissing(eq,y,z,n) end_if;
      if sol=FAIL then sol:=ode::equidimensional_in_y(eq,y,z,n) end_if;
      if sol=FAIL then sol:=ode::equidimensional_in_x(eq,y,z,n) end_if;
      // methods that call other methods and/or call solve_eq recursively 
      if sol=FAIL then 
         sol:=ode::interchange(eq,y,z,n) end_if;
      // scale_invariant calls equidimensional_in_x 
      if sol=FAIL then 
         sol:=ode::scale_invariant(eq,y,z,n) end_if;
   end_if;
   userinfo(2,"solution is",sol);
   if sol=FAIL then return({FAIL}) // do not return FAIL because of _concat 
   elif inits={} then return(sol)
   end_if;
   userinfo(1,"trying to fulfill initial conditions");
   userinfo(2,"which are",inits);
   // do not deal with implicit solutions for BVPs 
   if type(sol)="solve" then 
   elif type(sol)<>DOM_SET then return({FAIL}) end_if;
   s:={};
   csts:=indets(sol) minus (indets(eq) union {y,z});
   for eq in inits do
      if has(op(eq,2),y) then 
         error("dependent variable not allowed in rhs of initial conditions") 
      end_if;
      Lhs:=op(eq,1); i:=0; _a:=op(Lhs); 
      Lhs:=op(Lhs,0); // should be y or D(...D(y)...) 
      case type(Lhs)
      of DOM_IDENT do break
      of "D" do repeat Lhs:=op(Lhs); i:=i+1 until type(Lhs)<>"D" end_repeat; 
                break
      otherwise error("invalid initial conditions")
      end_case;
      if Lhs<>y then error("invalid initial conditions") end_if;
      s := s union {[i,_a,op(eq,2)]}; // [i,a,b] means (D@@i)(f)(a)=b 
      if eval(subs(den,z=_a,op(inits)))=0 then
         userinfo(1,"denominator ".expr2text(den).
                    " vanishes for given initial conditions");
         return({})
      end_if;
   end_for;
   if type(sol)="solve" then
      if op(sol,2)=y and nops(s)=1 and op(s,[1,1])=0 and nops(csts)=1 then
         eq:=op(sol,1); csts:=op(csts,1); s:=op(s,1);
         _a:=solvelib::discreteSolve(subs(eq,z=op(s,2),y=op(s,3)),csts);
         if type(_a)<>"solve" and nops(_a)=1 then
            return(solvelib::discreteSolve(subs(eq,csts=op(_a,1)),y))
         end_if
      end_if;
      return({FAIL})
   else sol:=map(sol,ode::solvinit,s,csts,z);
     if sol<>{FAIL} then sol:= sol minus {FAIL} end_if;
     sol;
   end_if
end_proc:

// inits is a set of triples [i,a,b] 
ode::solvinit := proc(sol,inits,csts,z)
local s,t,i,a,definiteInt;
begin
   userinfo(3,"general solution is",sol);
   if type(sol)="RootOf" then 
      sol:=subs(sol,extop(sol,2)=extop(sol,2)(z)) end_if;
   userinfo(3,"initial conditions are",inits);
   s:={};
   if traperror((for t in inits do
     s:=s union {eval(subs(diff(sol,z$t[1]),
			   hold(int)=proc() begin 0 end_proc ,
			   z=t[2]))=t[3]};
   end_for))<>0 then return(FAIL) end_if;
   if hastype(s,"RootOf") then return(FAIL) end_if;
   if csts={} then return(FAIL) end_if; 
   userinfo(3,"system to solve is",s,"with respect to",csts);
   s:=solvelib::discreteSolve(s,csts);
   userinfo(3,"solution for initial conditions is",s);
   s:=select(s, ()->contains({DOM_LIST,DOM_SET},type(args(1))) );
   //userinfo(3,"solution for initial conditions is",s);
   if type(s)<>DOM_SET then FAIL
   //elif nops(s)=0 then null() # no solution #  // solve changed once more
   elif s={} or s={[]} then null() // no solution 
   else // s is a set of solutions 
     a:=select(inits, e->e[1]=0);
     if a={} then a:=op(inits,1)[2] else a:=op(a,1)[2] end_if;
     t:=genident("t");
     definiteInt:=proc()
                    local l;
                  begin
                    l:=subs([op(args())],z=t);
                    l[2]:= t=a..z;
                    int(op(l))
                  end_proc;
     sol:=misc::maprec(sol,{"int"}=definiteInt);
     sol:=[subs(sol,op(s,i)) $ i=1..nops(s)];
     if has(sol,{sin,cos}) then sol:=combine(expand(sol),sincos) end_if;
     op(sol)
   end_if
end_proc:







