// $Date: 2000/05/21 14:57:37 $ $Author: bij $ $Revision: 1.4 $ 
// tonner, 23/06/98 

/*++
output::CatPresentation -- Category of MathML presentation outputs
++*/

alias(Priority=output::Priority);


/** Creating Presentation trees
 *
 *   This domain implements the infrastructure to create Presentation
 *   trees
 */

category output::CatPresentation
  category output::CatUtils;

  outputType := "Presentation";

  // methods to access the internal representation
  priority; mmlType; attributes; entries;

  /* MML Presentation token elements and layout schemata */

  mi; mn; mo; mtext; mspace; ms;

  mrow; mfrac; mroot; msqrt; mstyle; merror; mpadded; mphantom; mfenced;

  msub; msup; msubsup; munder; mover; munderover; mmultiscripts; mprescripts;
    
  none;       // ???

  mtable; mtr; mtd; maligngroup; malignmark; maction;

  placeholder;  // Presentation is used to generate templates with placeholders

  keyword := () -> dom::mtext("color"="red", args());
  statement := () -> dom::mtext(expr2text(args()));

  
  parenthesisOut :=
  proc(prio : Priority, x : DOM_LIST)
      local child;
    begin 
      child := dom(op(x));
      if dom::priority(child) > prio or 
       (Priority::_equal(prio, Priority::Stmt) and
	Priority::_equal(dom::priority(child), 
			 Priority::Stmt)) then
       child
     else
       dom::parenthesis(child)
     end_if;
    end_proc;


  // print a standard function f(a,b)
  stdFunc :=
  proc(x)
    local i;
  begin
    dom::mrow(dom(op(x,0)),
	      dom::mo("&ApplyFunction;"),
	      dom::mfenced(dom(op(x, i)) $ i=1..nops(x))
	      );
  end_proc;
  
  // print in parenthesis
  parenthesis := dom::mfenced;
  
  
  // print in braces
  braces := () -> dom::mfenced("open"="{", "close"="}", args());
  
  
  // print in brackets
  brackets := () -> dom::mfenced("open"="[", "close"="]", args());


// generate a output method for a infix operator
  infixOperator :=
  proc(str : DOM_STRING)
    option escape;
  begin
    proc(Out: DOM_DOMAIN)
      local i, arg1, prio;
    begin
      arg1 := args(2..args(0));   // arg1 can be _expreseq
      case nops(arg1)
	of 0 do
	  Out::mrow(Out::mi(expr2text(op(arg1,0))),
		    Out::mo("&ApplyFunction;"),
		    Out::mfenced());
	  break;
	of 1 do
	  Out::mrow(Out::mi(expr2text(op(arg1,0))),
		    Out::mo("&ApplyFunction;"),
		    Out::mfenced(Out(op(arg1))));
	  break;
	otherwise
	  prio := Priority(arg1);
	  Out::mrow(Out::parenthesisOut(prio, [op(arg1,1)]),
		    (Out::mo(str),
		     Out::parenthesisOut(prio, [op(arg1,i)]))
		    $ i = 2..nops(arg1))
      end_case;
    end_proc;
  end_proc;

// generate a output method for a prefix operator
  prefixOperator :=
  proc(str : DOM_STRING)
    option escape;
  begin
    proc(Out: DOM_DOMAIN)
      local i, arg1, prio;
    begin
      arg1 := args(2..args(0));
      case nops(arg1)
	of 0 do
	  Out::mrow(Out::mi(expr2text(op(arg1,0))),
		    Out::mo("&ApplyFunction;"),
		    Out::mfenced());
	  break;
	otherwise
	  prio := Priority(arg1);
	  Out::mrow(Out::mo(str),
		    (Out::parenthesisOut(prio, [op(arg1,i)]))
		    $ i = 1..nops(arg1))
      end_case;
    end_proc;
  end_proc;

// generate a output method for a postfix operator
  postfixOperator :=
  proc(str : DOM_STRING)
    option escape;
  begin
    proc(Out: DOM_DOMAIN)
      local i, arg1, prio;
    begin
      arg1 := args(2..args(0));
      case nops(arg1)
	of 0 do
	  Out::mrow(Out::mi(expr2text(op(arg1,0))),
		    Out::mo("&ApplyFunction;"),
		    Out::mfenced());
	  break;
	otherwise
	  prio := Priority(arg1);
	  Out::mrow((Out::parenthesisOut(prio, [op(arg1,i)]))
		    $ i = 1..nops(arg1),
		    Out::mo(str))
      end_case;
    end_proc;
  end_proc;
  
end_category:

// end of file 
