/* $Date: 2000/11/23 08:57:56 $ $Author: tonner $ $Revision: 1.30 $ */
/* tonner, 23/06/98 */

/** output::CatUtils -- Utilies for the output generation
 */

alias(Priority=output::Priority);


/**  Utilies for the output generation
 *
 *   This domain provides utilities for Content and Presentation
 *   based outputs.
 */
category output::CatUtils
  
/** Name : DOM_STRING
 *
 *  Every domain derived from output::CatUtils has to provide a Name
 *  entry which is used as slot name to store output methods for this
 *  output type.  E.g. if a domain output::MMLcontent derived from
 *  output::CatContent has the Name entry "MMLContent" the output 
 *  method for lists for the output type output::MMLcontent is stored
 *  and searched for in DOM_LIST::MMLContent.
 */
  Name;
  
/** outputType : DOM_STRING
 *
 *  outputType may be one of the strings "Content" or "Presentation"
 *  and defines which kind of output should be generated.
 */
  outputType;
  

/** create and print
 *
 *  These methods define how to create the internal representation
 *  and how to print it on the screen 
 */
  create;  print;

/** setAliasTab
 *
 *  set a table for alias replacements
 */
  setAliasTab := x -> sysassign(dom::aliasTab, x);
  aliasTab := table();
  
/** Method new - converts MuPAD data into its output representation
  *
  * hier ist das Auffinden von Methoden implementiert.  
  * Sollte in den Kern !!!!
  */
  new :=
  proc()
    local argType, method, result;
  begin
    // look for alias replacements
    if contains(dom::aliasTab, args()) then
      return(dom::new(dom::aliasTab[args()]))
    end_if;
    
    // do not assign args() to a local variable.  If
    // args() is an unnamed procedure, it will receive the name
    // of the local variable as a side effect !!!
    argType := (args())::dom;

    if (method := slot(argType, dom::Name)) <> FAIL then
      /*
       * a special output function is defined; use it
       */
      result := method(dom, args());
    elif (method := slot(argType, dom::outputType)) <> FAIL then
      /*
       * 
       */
      sysassign( slot(argType, dom::outputType), method);
      result := method(dom, args());
    elif (method := argType::print) <> FAIL then
      /*
       *  und nun alte "print"-Methode Aufrufen und deren Ergebnis ausgeben  
       */
      result := dom(method(args()));
      if domtype(result) = DOM_STRING then
      /* we want to see the output unquoted */
        result := hold(``).result
      end_if;
    else
      result := dom::unknownType(args());
    end_if;

    if dom::outputType = "Presentation" and dom::priority(result) = "none" then
      //
      // Hier waere es schoen, wenn man per _set_slot einfach den Priority-
      // Eintrag aendern koennte.
      //
      result := new(dom, Priority(args()), extop(result,2..extnops(result)));
    end_if;
    return(result)
  end_proc;
  
  testtype := FALSE;

  stdFunc;  // Domains of this type have to define a method 
            // stdFunc for the output of generic functions

  statement;  // defines the output of statements
  
  // used for elements of domains without output methods
  unknownType :=
  () -> dom::stdFunc(hold(new)(domtype(args()), extop(args())));
  

  // called once per output -- e.g.  set LaTeX output in \[ ... \] 
  enclose := id;


 //++++++++++++++++ a collection of utility routines  ++++++++++++++

  // decide whether x is an operator
  isOperator :=
  proc(x)
  begin
    if domtype(x) = DOM_IDENT then
      x := eval(x);
    end_if;
    if domtype(x) = DOM_FUNC_ENV then
      if domtype(op(x,[2,3])) = DOM_STRING then
        TRUE else FALSE
      end_if;
    else
      FALSE
    end_if;
  end_proc;

    
  isNeg :=                        // needed for sortSum and in _plus 
  proc(arg1)
    local argType, lastarg;
  begin
    argType := domtype(arg1);
    if argType::isNeg <> FAIL then
      return(argType::isNeg(arg1));
    end_if; 
    if testtype(arg1, Type::Real) and arg1 < 0 then
      return(TRUE)
    end_if;
    if testtype(arg1, "_mult") then
      lastarg := op(arg1, nops(arg1));
      if testtype(lastarg, Type::Real) then
        if lastarg < 0 then
          return(TRUE);
        end_if;
      end_if;
      return(FALSE);
    end_if;
    if type(arg1) = DOM_COMPLEX then
      if op(arg1, 1) = 0 and op(arg1, 2) < 0 then
        return(TRUE);
      end_if;	
      return(FALSE);
    end_if;
    FALSE;
  end_proc;
 
  sortSum :=            // rearrange a sum without - at the beginnig 
  proc(SuM)
    local i, list;
  begin 
    if testargs() then 
      if not testtype(SuM, "_plus") then
        error("\"_plus\"-expression expected");
      end_if;
    end_if;
    if nops(SuM) = 1 then
      return(SuM);
    end_if;
    list := [op(SuM)];
    if dom::isNeg(list[1]) then
      for i from 2 to nops(list) do
        if not dom::isNeg(list[i]) then
          return([op(subsop(list, 1=list[i], i=list[1]))]);
        end_if;
      end_for;
    end_if;
    SuM;
  end_proc;

  splitProduct :=
  //  analyse a _mult-expr and return a list of a 1 number and 2 
  //  lists; the number is the sign of the product 1 or -1, the  
  //  1. list contains the numerator and the 2. the denominator. 

  proc(multExpr)
    local numList, denumList, sigN, num, i, expo, normalForm;
  begin 
    if testargs() then
      if type( multExpr ) <> "_mult" then
        error("_mult-expr expected");
      end_if;
    end_if;

    normalForm :=
      (
       num := select({op(multExpr)}, testtype, Type::Numeric);
       if nops(num) = 0 then TRUE 
       elif nops(num) = 1 and  // the last one is the number ? 
	 testtype(op(multExpr, nops(multExpr)), Type::Numeric) then 
	 TRUE
       else FALSE end_if;
       ):   // if normalForm = FALSE don't change the order of terms 
    if nops(multExpr) = 1 then  // _mult(a+b) 
      return([1, [op(multExpr,1)], []]);
    end_if;

    sigN := 1;
    numList := [];
    denumList := [];
    
    for i in multExpr do
      if testtype( i, Type::Real ) or i::dom::isNeg <> FAIL then
        if dom::isNeg(i) then
	  i := -i;
          sigN := sigN * (-1);
	  if i = 1 then next end_if; 
        end_if;
	if testtype( i, DOM_RAT) then  // fraction 
          num := op(i,1);
          if num <> 1 then  // ignore 1 ( a/2 -> [[a],[2]] not [[1,a],[2]] ) 
	    if normalForm then
              numList := [num].numList;
	    else
	      numList := numList.[num];
	    end_if;
	  end_if;
          if normalForm then  
	    denumList := [op(i,2)].denumList;
	  else
	    denumList := denumList.[op(i,2)];
	  end_if;
        else 
              // real number, but no fraction 
          if normalForm then  
            numList := [i].numList;
	  else
	    numList := numList.[i];
	  end_if;
        end_if;
      elif testtype( i, DOM_COMPLEX ) then  // insert at the beginning 
        if normalForm then  
          numList := [i].numList;
        else
          numList := numList.[i];
        end_if;
      elif testtype( i, "_power" ) then
	 		// no number, so insert at the end 
        // take a look at the exponent 
        expo := op(i ,2);
        if testtype( expo, "_mult") then
          if testtype( op(expo, nops(expo)), Type::Real ) then
            if  op(expo, nops(expo)) < 0 then
              denumList := denumList.[1/i];
            else
              numList := numList.[i];
            end_if; // product contains a real number 
          else
            // exponent is a product with no real number 
            numList := numList.[i];
          end_if;
         // exponent is no product 
         // if exponent is less zero, so insert 1/i in denumerator 
         // but not if base an exponent are numbers                
         elif testtype(expo, Type::Real) and
              not testtype(op(i, 1), Type::Real) then
	   if expo = -1 then  
             // special case for -1 because of internal normal form 
	     denumList := denumList.[op(i, 1)];
	   elif expo < 0 then
	     denumList := denumList.[1/i];
	   else
	     numList := numList.[i];
	   end_if
         else   // just insert 
	   numList := numList.[i];
         end_if;
      else   // factor is no _power, just insert in numerator 
	numList := numList.[i];
      end_if;	       
    end_for;

    if numList = [] then   // dont't return empty numerator 
      numList := [ 1 ];
    end_if;

    return( [sigN, numList, denumList] );
  end_proc;
//++++++++++++++++ a collection of utility routines  ++++++++++++++
 
end_category:

/*
 *
 *   And now some default ouput methods for the basis data types
 *
 */
   
alias(path = pathname("OUTPUT", "DOMAINS")):

DOM_ARRAY::Content    := loadproc(DOM_ARRAY::Content, path, "array"):
DOM_BOOL::Content     := proc(Out, x) 
			 begin 
			   if x = TRUE then
			     Out::Ccn(hold(`&true;`))
			   elif x = FALSE then
			     Out::Ccn(hold(`&false;`))
			   else 
			     Out::Ccn(hold(`&UNKNOWN;`))
			   end_if;
			 end_proc:
DOM_COMPLEX::Content  := loadproc(DOM_COMPLEX::Content, path, "complex"):
DOM_DOMAIN::Content   := 
  proc(Out, x) 
    local domName;
  begin 
    if x::Name <> FAIL then 
      domName :=  Out(x::Name)
    else
      domName :=  Out(x::key)
    end_if:
    if domtype(domName) = DOM_STRING then
      domName := hold(``).domName
    end;
    domName
  end:
DOM_EXEC::Content     := (Out, x) -> 
                         if domtype(op(x,4)) = DOM_STRING then
                           Out(text2expr(op(x,4)))
                         elif domtype(op(x,3)) = DOM_STRING then
                           Out(text2expr(op(x,3)))
                         else
                           Out(hold(builtin)(op(x)))
                         end_if;
DOM_EXPR::Content     := loadproc(DOM_EXPR::Content, path, "expr"):
// What else ?
DOM_FAIL::Content     := (Out, x) -> Out::Cci(hold(`&FAIL;`)):
DOM_FLOAT::Content    := proc(Out, x)
                           local exponent, hasSign, signLessX, res;
                         begin
                           hasSign := if x<0 then TRUE else FALSE end;
                           signLessX := abs(x);
                           if signLessX <> 0.0 and
                              (signLessX > 10^DIGITS or 
                               signLessX < 10^(-DIGITS)) then
                             exponent := floor(ln(signLessX)/ln(10.0));
                             res := Out(hold(_mult)(signLessX*10^(-exponent), 
                                                    hold(_power)(10, exponent)));
                             if hasSign then
                               return(Out::Cminus(res))
                             else
                               return(res)
                             end_if;
                           else
                             Out::Ccn(x):
                           end_if:
                         end_proc:
DOM_FUNC_ENV::Content := loadproc(DOM_FUNC_ENV::Content, path, "funcenv"):
DOM_IDENT::Content    := (Out, x) -> Out::Cci(x):
DOM_INT::Content      := (Out, x) -> Out::Ccn(x):
DOM_LIST::Content     := proc(Out, x) local i;
                         begin
/*                           Out::Clist(Out(i) $ i in x) */
                           Out::Clist(Out(op(x,i)) $ i=1..nops(x)) 
                         end_proc:
DOM_NIL::Content      := (Out, x) -> Out::Cci(hold(`&NIL;`)):
DOM_NULL::Content     := (Out, x) -> Out::Cci(hold(`&NULL;`)):
DOM_POINT::Content    := (Out, x) -> 
                         if op(x, nops(x)) = NIL then
                            Out(hold(point)(op(x, 1..nops(x)-1)))
                         else
                            Out(hold(point)(op(x, 1..nops(x)-1), 
                                            hold(Color)=op(x, nops(x))))
                         end:
DOM_POLYGON::Content  := 
  proc(Out, x)
    local Nops, options;
  begin
    Nops := nops(x);
    if op(x, Nops-2) = NIL then
      options := null()
    else 
      options := hold(Color)=op(x, Nops-2);
    end;
    if op(x, Nops-1) = TRUE then
      options := options, hold(Closed)=TRUE;
    end:
    if op(x, Nops) = TRUE then
      options := options, hold(Filled)=TRUE;
    end:

    Out(hold(polygon)(op(x, 1..Nops-3), options));
  end:
DOM_POLY::Content     := 
  proc(Out, data)
    local list, coeffRing, varList;
  begin
    varList := op(data, 2);
    coeffRing := op(data, 3);
    if coeffRing = hold(Expr) then
      coeffRing := null()
    end_if;
    list := poly2list(data);
    if nops(varList) = 1 then // univariate polynomial
      list := zip(list, [op(varList) $ nops(list)], 
                  (x, y) -> [op(x,1), [_power(y, op(x,2))]])
    else                      // multivariate polynomial
      list := zip(list, [varList $ nops(list)], 
                  proc(x, y) 
                    local i; 
                  begin
                     [op(x,1), select([_power(op(y, i), 
                                       op(x,[2,i])) $ 
                                    i = 1..nops(varList)],
                                   x -> x <> 1)]
                  end);
    end_if;
    list := map(list, 
                   x -> if x[1] = 1 then 
                           if nops(x[2]) = 1 then op(x[2])
                           else hold(_mult)(op(x[2])) end
                        elif op(x[2]) = 1 then x[1]
                        elif testtype(x[1], Type::Numeric) then 
                             hold(_mult)(op(x[2]), x[1])
                        else hold(_mult)(x[1],op(x[2])) end);
    if nops(list) = 1 then
      Out(hold(poly)(op(list), varList, coeffRing))
    else
      Out(hold(poly)(hold(_plus)(op(list)), varList, coeffRing))
    end_if;
  end_proc:
DOM_PROC::Content     := 
  (Out, data) ->
    if contains({op(data,3)}, hold(arrow)) then
      Out::Capply(Out::Cproc, 
                  Out(op(data,1)), 
                  Out(subs(op(data,4), 
                           DOM_VAR(0,i+1)=op(data,[1,i]) $ 
                              i=1..nops(op(data,1)), 
                           Unsimplified)
                     ));
    else
      Out(hold(``).DOM_PROC::print(data)):
    end_if:
DOM_RAT::Content      := (Out, x) -> 
                         if x < 0 then 
                           Out(hold(_negate)(-x)):
                         else
                           Out::Cdivide(Out(op(x,1)), Out(op(x,2))):
                         end_if:
DOM_SET::Content      := proc(Out, x : DOM_SET)
                           local i;
                         begin
                           Out::Cset(Out(i) $ i in DOM_SET::sort(x))
                         end_proc:
DOM_STRING::Content   := (Out, x) -> x:
DOM_TABLE::Content    := proc(Out, tbl)
  local i;
begin
    Out::Ctable(
		 (Out(op(tbl, [i,1])),
                 Out::Crhs(Out(op(tbl, [i, 2]))))
                         $ i = 1..nops(tbl)
                 )
end_proc:
DOM_VAR::Content      := () -> error("not yet implemented"):
infinity:
stdlib::Infinity::Content := proc(Out, data)
                               local res;
                             begin
                               res := Out::Ccn({type="constant"}, hold(`&infin;`)):
                               if data > 0 then
                                 res
                               else
                                 Out::Cminus(res)
                               end_if
                             end_proc:



DOM_ARRAY::Presentation    := loadproc(DOM_ARRAY::Presentation, path, "array"):
DOM_BOOL::Presentation     := (Out, x) -> Out::keyword(expr2text(x)):
DOM_COMPLEX::Presentation  := loadproc(DOM_COMPLEX::Presentation, path, "complex"):
DOM_DOMAIN::Presentation   := loadproc(DOM_DOMAIN::Presentation, path, "domain"):
DOM_EXEC::Presentation     := loadproc(DOM_EXEC::Presentation, path, "exec"):
DOM_EXPR::Presentation     := loadproc(DOM_EXPR::Presentation, path, "expr"):
DOM_FAIL::Presentation     := Out -> Out::keyword("FAIL"):
DOM_FLOAT::Presentation    := (Out, x) -> Out::mn(expr2text(x)):
DOM_FUNC_ENV::Presentation := loadproc(DOM_FUNC_ENV::Presentation, path, "funcenv"):
DOM_IDENT::Presentation    := (Out, x) -> Out::mi(expr2text(x)):
DOM_INT::Presentation      := (Out, x) -> Out::mn(expr2text(x)):
DOM_LIST::Presentation     := (Out, x) -> Out::brackets(Out(op(x))):
DOM_NIL::Presentation      := Out -> Out::keyword("NIL"):
DOM_NULL::Presentation     := Out -> Out::mtext("null()"):
DOM_POINT::Presentation    := loadproc(DOM_POINT::Presentation, path, "point"):
DOM_POLY::Presentation     := (Out, data) -> Out(hold(poly)(op(data))):
DOM_POLYGON::Presentation  := loadproc(DOM_POLYGON::Presentation, path, "polygon"):
DOM_PROC::presentation     := (Out, data) -> Out::statement(data):
DOM_RAT::Presentation      := (Out, x) -> Out::mfrac(Out(op(x,1)),
                                                Out(op(x,2))):
DOM_SET::Presentation      := (Out, x) ->  Out::braces(Out(op(x))):
DOM_STRING::Presentation   := (Out, x) -> Out::ms(x):
DOM_TABLE::Presentation    := loadproc(DOM_TABLE::Presentation, path, "table"):
DOM_VAR::Presentation      := () -> error("not yet implemented"):

stdlib::Infinity::Presentation := (Out, x) -> 
                                  if x = -infinity then
                                    Out(hold(_mult)(infinity, -1))
                                  else
                                    Out::mi("&infin;"):
                                  end_if:
unalias(path):

// end of file 
