// $Date: 2000/11/30 13:40:58 $ $Author: tonner $ $Revision: 1.48 $ 
// tonner, 23/06/98 

/*++
output::Content -- Domain of Content markup
++*/

output:

/** Creating Content trees
 *
 *   This domain implements the infrastructure to create Content
 *   trees
 */
domain Content
  category output::CatContent;
  inherits Dom::BaseDomain;
  
  Name := "Content";

  create :=
  proc(typString : DOM_STRING, typInfo : DOM_STRING, attributes : DOM_STRING)
    local typ;
  begin
    if testargs() then
      if args(0) < 3 then
        error("internal output error")
      end_if;
      if map({args(4..args(0))}, domtype) minus
        {DOM_STRING, DOM_INT, DOM_IDENT, DOM_FLOAT, Content} <> {} then
        error("children may only be idents, numbers, strings or elements of Content");
      end_if;
    end_if;
    
    typ := dom::ContentTable[typString];
    if type(typ) = "_index" then
      error("No template \"".typString."\" defined");
    end_if;
    
    new(dom, args())
  end_proc;

  // access element entries
  typString := proc(c)
  begin
    if contains({DOM_INT,DOM_STRING,DOM_FLOAT,DOM_IDENT},domtype(c)) then
        expr2text(domtype(c))
    else
        extop(c, 1)
    end
  end;

  attributes := proc(c)
  begin
    if contains({DOM_INT,DOM_STRING,DOM_FLOAT,DOM_IDENT},domtype(c)) then 
        ""
    else
        extop(c, 2)
    end
  end;

  infos := proc(c)
  begin
    if contains({DOM_INT,DOM_STRING,DOM_FLOAT,DOM_IDENT},domtype(c)) then 
      ""
    else
      extop(c, 3)
    end
  end;
    
  op := proc(c,i=-1)
               begin
                 if i = -1 then
                   extop(c, 4..extnops(c))
                 elif testtype(i, "_range") then
                   extop(c, 3+op(i,1)..3+op(i,2))
                 else
                   extop(c, 3+i)
                 end_if
               end_proc;
  nops := c -> extnops(c) - 3;

  Content2tree :=
  proc(x)
    local res;
  begin
    if domtype(x) <> dom then [x]
    else
      res := [dom::typString(x).
       ", info: ".expr2text(dom::infos(x)).", attrib : ".
       expr2text(dom::attributes(x))];
      if nops(x)=0 then res
      else append(res, map(op(x), dom::Content2tree));
      end_if;
    end_if
  end_proc;

  print := 
  proc(x)
  begin
    output::tree(dom::Content2tree(x), {hold(Small)});
  end_proc;
    
  /* output in XML notation */
  xml := proc(arg1) begin dom::do_print_xml(arg1) end_proc;
  
  do_print_xml :=
  proc(arg1)
    name output::print;
    local DOM, process, Print,
          outString:DOM_STRING, indentLevel:DOM_INT;
  begin
    indentLevel := 0;
    outString := "";
    DOM := dom;
    
// extract args
  Print :=
  proc(d)
    local attributes, infos, Indent, Empty;
  begin
    Indent := {};
    Empty := { "int" };
    if contains({DOM_STRING, DOM_INT, DOM_IDENT, DOM_FLOAT, output::Indent} , domtype(d)) then
       case domtype(d) 
         of DOM_STRING do
	return(d, DOM::newline());
            return("<cstring>".d."</cstring>", DOM::newline())
         of DOM_INT do
            return("<cn type=\"integer\"> ".expr2text(d)." </cn>", DOM::newline())
         of DOM_FLOAT do
            return("<cn> ".expr2text(d)." </cn>", DOM::newline())
         of DOM_IDENT do
            return("<ci> ".expr2text(d)." </ci>", DOM::newline())
	 of output::Indent do
            return(d)
       end_case;
    end_if;

    attributes := DOM::attributes(d);
    infos := DOM::infos(d);
    "<", DOM::typString(d),
    (if contains(Empty, DOM::typString(d)) then "/>"
     else
       ">",
     DOM::newline(), 
     DOM::indent(
       (if attributes <> "" then
          "<annotation encoding=\"MuPAD-attributes\">", 
          DOM::indent(attributes), DOM::newline(),
          "</annotation>"
        else
          null()
        end_if),
       (if infos <> "" then
          "<annotation encoding=\"MuPAD-infos\">", 
          DOM::indent(infos),  DOM::newline(),
          "</annotation>"
        else
          null()
        end_if),
       DOM::op(d)), DOM::newline(),
     "</", DOM::typString(d),">"
     end_if),
    DOM::newline()
  end_proc;

    process :=
    proc(x)
      name  output::print::process;
      local Oseq, i, entries;
    begin
      Oseq := [Print(x)];
// print("process", x, domtype(x), Print, Oseq);
      for i in Oseq do
	if domtype(i) = DOM_STRING then
	  outString := outString.DOM::translate(i);
	elif domtype(i) = output::Indent then
	  if output::Indent::isNewline(i) then
	    outString := _concat(outString, "\n", ("  "$indentLevel));
	  else
	    indentLevel := indentLevel+1;
	    outString := outString."  ";
	    if (entries := output::Indent::entries(i)) <> null() then
	      map(entries,process);
	    end_if;
	    indentLevel := indentLevel-1;
	  end_if;
	elif domtype(i) = dom then
	  process(i);  
	else
// print("process", i, domtype(i));
	  error("output element expected");
	end_if;
      end_for;
    end_proc;

    process(dom::enclose(arg1));
    outString;
  end_proc;

  translate := id;

  // formatting of source output
  newline := proc() option remember; begin output::Indent("newline") end_proc;
  indent :=  () -> output::Indent("indent", args());

  expr :=
  proc(c)
    local i, cType, attributes;
    save XxX;
  begin
    delete XxX;
    
    cType := domtype(c);

    // here we need a special conversion from backquoted identifiers
    // representing MML-constants to MuPAD constants
    if cType = DOM_IDENT and contains(dom::MML2ident, c) then
       return(dom::MML2ident[c]);
    end_if;

    if not contains({DOM_STRING, DOM_INT, DOM_IDENT, DOM_FLOAT}, cType) and domtype(c) <> dom then
      error("wrong type")
    end_if;

    if nops(dom::ContentTable[dom::typString(c)]) > 1 then
      attributes := table(text2expr(dom::attributes(c)));
      return(dom::ContentTable[dom::typString(c)][2](c, attributes, dom::infos(c)));
    else
      subs(XxX(dom::expr(op(c,i)) $ i = 1 .. nops(c)),  XxX=op(dom::ContentTable[dom::typString(c)], 1), Unsimplified)
    end_if;
  end_proc;


/** Tables for translations from MML-constants to MuPAD constants
    and vice versa.
  */ 
  MML2ident := table();

/** Table for translations from MML-constants to MuPAD constants
  */ 
  ident2MML := table();


/** fills the tables MML2ident and ident2MML in the init section
  */ 
  translateIdents :=
  proc(equation : Type::Equation(DOM_IDENT))
  begin
    dom::MML2ident[op(equation, 1)] := op(equation, 2);
    dom::ident2MML[op(equation, 2)] := op(equation, 1);
  end_proc;



/**  Table of all known Content types
 *
 *   Format of the table entries:
 *   
 *   index = [mupadType, backTranslate (optional)]
 *
 *   index              : string which adresses the template in MuPAD
 *
 *   mupadType          : type of the MuPAD object which is represented
 *                        by this Content
 *
 *   backTranslate (opt): MuPAD function which translates a Content 
 *                        element of the type index into a valid MuPAD
 *                        object
 */
  ContentTable := table();

/* Define new entries in Content::ContentTable */
  set :=
  proc(NAME : DOM_STRING,
       mupadType,
       backTranslate = null() : DOM_PROC)
    option hold;
  begin
    dom::ContentTable[NAME] := [mupadType, backTranslate];
  end_proc;
  
  Ccn:=       // Content Number
  proc() 
    local num; 
  begin 
    num := args(args(0));
    if testtype(num, Type::Real) and num < 0 then 
      dom(hold(_negate)(-num))
    else
      num
    end_if
  end_proc;

  Cci:= () -> if contains(dom::ident2MML, args()) then
		 dom::Ccn({type="constant"}, dom::ident2MML[args()]);
	      else
                 args();      // Content Identifier
              end_if;

  /**  Capply - the MathML apply construct
   *  The implementation of the MathML apply construct for MuPAD Content
   *  output is quite complicated, but makes it easy to define the Content
   *  slot for MuPADs expression types.  E.g. 
   */
  Capply:=
  proc()
  begin
    if testargs() and args(1) = FAIL then
       error("trying to apply undefined entity");
    end_if;
    args(1)(args(2..args(0)))
  end_proc;   

  Cfn := x -> dom(x);      // user-defined function
  Cinterval := // interval constructor
     proc(x, left, right)
     begin
       if op(x,[1,2]) = "open" then
         dom::create("intervaloo", "", "", left, right);
       elif op(x,[1,2]) = "open-closed" then
         dom::create("intervaloc", "", "", left, right);
       elif op(x,[1,2]) = "closed-open" then
         dom::create("intervalco", "", "", left, right);
       else
         dom::create("intervalcc", "", "", left, right);
       end_if;
     end_proc;

//  Cinverse;   // functional inverse
  Ccondition // properties of an identifier
	 := id;
//  Cdeclare;   // declaration
//  Clambda;    // user defined functions
  
//  Csep;       // separator
  Ccompose :=   // composition operator
          () -> dom::create("compose", "", "", args());
  Cident  :=    () -> dom::create("ident", "", "", args());// identity function

  // Arithmetic, Algebra and Logic
  Cquotient :=  () -> dom::create("quotient", "", "", args());
  Cexp :=       data -> if data = 1 then hold(`&ee;`) else 
                        dom::Cpower(hold(`&ee;`), data) end_if;   
  Cfactorial := () -> dom::create("factorial", "", "", args());
  Cdivide :=    () -> dom::create("divide", "", "", args());
  Cmax :=       () -> dom::create("max", "", "", args());
  Cmin :=       () -> dom::create("min", "", "", args());
  Cminus :=     () -> dom::create("plus", "Signs=\"-\"", "", args());
  Cplus :=      proc()
                  local DOM, list, Signs;
                begin
                  DOM := dom;
                  /* here we have to translate all operands of type
                     "minus" into out special sign encoding for _plus */
                  list := map([args()], contents ->
                                if domtype(contents) = DOM and 
                                   DOM::typString(contents) = "plus" and 
                                   DOM::attributes(contents) = "Signs=\"-\"" then
                                  [op(contents), "-"]
                                elif contains({DOM_INT, DOM_RAT, DOM_FLOAT}, domtype(contents)) and contents < 0 then
                                  [-contents, "-"]   
                                else
                                  [contents, "+"]
                                end_if);
                  Signs := _concat(op(map(list, op, 2)));
                  list := map(list, op, 1);
                  if Signs = "+" or nops(list) = 0 then
                    dom::create("apply", "", "", hold(_plus), op(list))
                  else
                    dom::create("plus", "Signs=".expr2text(Signs), "", 
                                op(list));
                  end_if;
                end_proc;
  Cpower :=     () -> dom::create("power", "", "", args());
  Crem   :=     () -> dom::create("rem", "", "", args());
  Ctimes :=     () -> dom::create("times", "", "", args());
  Croot :=      () -> dom::create("root", "", "", args());
  Csqrt :=      () -> dom::create("sqrt", "", "", args());
//  Cgcd;
  Cand :=       () -> dom::create("and", "", "", args());
  Cor :=        () -> dom::create("or", "", "", args());
//  Cxor;
  Cnot :=       () -> dom::create("not", "", "", args());
//  Cimplies;  Cforall;  Cexists;
  Cabs :=       () -> dom::create("abs", "", "", args());
  Cconjugate :=       () -> dom::create("conjugate", "", "", args());

  // Relations
  Ceq  :=     () -> dom::create("eq", "", "", args());
  Cneq :=    () -> dom::create("neq", "", "", args());
//  Cgt;
  Clt  := () -> dom::create("lt", "", "", args());
//  Cgeq;
  Cleq := () -> dom::create("leq", "", "", args());

  // Calculus
  Cln := () -> "ln";
  Clog := () -> dom::create("log", "", "", args());
  Cint := proc()
          begin
            if args(0) = 4 then
              // definite integral
              dom::create("definiteint", "", "", args(2..3), args(4), args(1))
            else
              // indefinite integral
              dom::create("int", "", "", args(2), args(1))
            end_if
          end_proc;
//  Cdiff;  
  Cpartialdiff := () ->
      if args(0) = 2 then
        dom::create("partialdiff", "", "", args());
      else
        dom::create("partialdiff1", "", "", args());
      end_if;
  Clowlimit := () -> args();
  Cuplimit  := () -> args();
  Cbvar := () -> args();
  Cdegree := () -> args();

  // Theory of sets
  Cset := () -> if args(0) = 0 then hold(`&emptyv;`) else 
              dom::create("set", "", "", args()) end_if;
  Clist := () -> dom::create("list", "", "", args());
  Cunion := () -> dom::create("union", "", "", args());
  Cintersect := () -> dom::create("intersect", "", "", args());
  Cin := () -> dom::create("in", "", "", args()); 
// Cnotin;  Csubset;
//  Cprsubset;  Cnotsubset;
  Csetdiff := () -> dom::create("setdiff", "", "", args());

  // Sequences and Series
  Csum := proc()
          begin
            if args(0) = 4 then
              dom::create("sum", "", "", args())
            elif args(0) = 3 then
              dom::create("sum2", "", "", args())
            else
              dom::create("sum1", "", "", args())
            end_if
          end_proc;
                  
  Cproduct := proc()
          begin
            if args(0) = 4 then
              dom::create("product", "", "", args())
            else
              dom::create("product1", "", "", args())
            end_if
          end_proc;

  Climit := () -> if domtype(args(2)) = DOM_STRING then
                    if args(2) = "Direction = Right" then
                      dom::create("limitRight", args(2), "", args(1), args(3..args(0)) );
                    else  // Left
                      dom::create("limitLeft", args(2), "", args(1), args(3..args(0)) );
                    end_if;
                  else 
                    dom::create("limit", "", "", args());
                  end_if;

  Ctendsto := proc(x: "_equal")
		option escape;
                local direction;
              begin
		if contains({"above", "below", "two-sided"}, op(x,2)) then
		  if op(x,2) = "above" then 
		    direction := "Direction = Right"
                  elif op(x,2) = "below" then 
		    direction := "Direction = Left"
 	 	  else
		    direction := null()
		  end_if;
		  proc(bvar, tendsto)
		  begin
		    direction, tendsto
		  end_proc
		else
		  id
		end_if;
              end_proc;

  // Trigonometry
  Csin    := () -> "sin";
  Ccos    := () -> "cos";
  Ctan    := () -> "tan";
  Csec    := () -> "sec";
  Ccsc    := () -> "csc";
  Ccot    := () -> "cot";
  Csinh   := () -> "sinh";
  Ccosh   := () -> "cosh";
  Ctanh   := () -> "tanh";
  Csech   := () -> "sech";
  Ccsch   := () -> "csch";
  Ccoth   := () -> "coth";
  Carcsin := () -> "arcsin";
  Carccos := () -> "arccos";  
  Carctan := () -> "arctan";

  // Statistics
//  Cmean;  Csdev;  Cvar;  Cmedian;  Cmode;  Cmoment;

  // Linear Algebra
//  Cvector;
  Cmatrix :=
  proc()
    local cols, rows; 
  begin
    cols := nops(args(1)); rows := args(0);
    dom::create("matrix", expr2text(hold(Rows)=expr2text(rows), hold(Cols)=expr2text(cols)), "Dom::Matrix()", op(_concat(args())));
  end_proc;
  Cmatrixrow := () -> [args()];
  Cdeterminant := () -> dom::create("det", "", "", args());  
  Ctranspose := () -> dom::create("transpose", "", "", args());
//  Cselect;
//  Cvectorproduct;
  Cscalarproduct := () -> dom::create("scalarproduct", "", "", args()); 
//  Couterproduct;

  // Semantic Mapping Elements
//  Cannotation;  Csemantics;  CannotationXml;

  Cexprseq := () -> dom::create("seq", "", "", args());
  Cproc := () -> dom::create("proc", "", "", args());
  Cfnest := () -> dom::create("fnest", "", "", args());
  Cceil := () -> dom::create("ceil", "", "", args());
  CbesselI := () -> dom::create("besselI", "", "", args());
  CbesselJ := () -> dom::create("besselJ", "", "", args());
  CbesselK := () -> dom::create("besselK", "", "", args());
  CbesselY := () -> dom::create("besselY", "", "", args());
  CD := () -> if args(0) = 1 then
                dom::create("D", "", "", args())
              else
                dom::create("Dpara", "", "", args())
              end_if;
  Cfloor := () -> dom::create("floor", "", "", args());
  Cslot :=  () -> dom::create("slot", "", "", args());
  Cpolylog := () -> dom::create("polylog", "", "", args());
  Cnorm := () -> if args(0) = 1 then
                   dom::create("norm", "", "", args());
                 else
                   dom::create("normx", "", "", args());
                 end_if;
  Cindex := () -> dom::create("index", "", "", args());
  Cbinomial := () -> dom::create("binomial", "", "", args()); 
  Crange := () -> dom::create("range", "", "", args()); 
  Cimageset := () -> dom::create("imageset", "", "", args());
  Cpiecewise :=  () -> dom::create("piecewise",  expr2text(hold(Rows)=expr2text(args(0) div 2), hold(Cols)="2"), "", args());
  Cpwcondition :=  () -> dom::create("pwcondition", "", "", args());
  Ctable := () -> dom::create("table",  expr2text(hold(Rows)=expr2text(args(0) div 2), hold(Cols)="2"), "", args());
  // right hand side of a formatted equation ; 
  // used in table etc
  Crhs :=  () -> dom::create("rhs", "", "", args());

  // equation system;  will be translated into a set of equations
  CEQS := () -> dom::create("EQS", "", "", args());
  Ctableau :=
  proc()
    local options; 
  begin
    if args(0)>0 and domtype(args(1)) = DOM_SET then
      options := table(op(args(1)));
    else
      error("Attributes missing");
    end;
    
    dom::create("tableau", expr2text(op(options)), "", args(2..args(0)));
  end_proc;

  end_domain:


// initialize identifier translation tables
map( [ hold(`&ii;`) = I, hold(`&pi;`) = PI, hold(`&ee;`) = E, 
       hold(`&infin;`) = infinity, hold(`&NotANumber;`) = RD_NAN, 
       hold(`&gamma;`) = EULER,  hold(`&true;`) = TRUE, 
       hold(`&false;`) = FALSE, hold(`&UNKNOWN;`) = UNKNOWN,
       hold(`&NIL;`) = NIL, hold(`&FAIL;`) = FAIL, 
       hold(``) = null(), hold(`&emptyv;`) = {}, // only for back translations
       hold(`&NULL;`) = null(), hold(`&Zopf;`) = hold(Z_),
       hold(`&Qopf;`) = hold(Q_), hold(`&Ropf;`) = hold(R_), 
       hold(`&Copf;`) = hold(C_), 
       hold(`&Im;`) = hold(Im), hold(`&Re;`) = hold(Re), 
       hold(`&zeta;`) = hold(zeta) , hold(`&delta;`) = hold(dirac), 
       hold(`&Gamma;`) = hold(igamma), hold(`&Gamma;`) = hold(gamma)], 
     Content::translateIdents):

Content::set("plus", _plus, 
                        proc(plusExpr, attributes : DOM_TABLE)
                          name convert_plus;
                          local i, signs;
                          save XxX;
                        begin
                          signs := attributes[Signs];
                          if signs = "-" then
                            _negate(Content::expr(op(plusExpr, 1)))
                          else
                            subsop(XxX((Content::expr(op(plusExpr, i));
                                    if signs[i-1] = "-" then
                                      _negate(%)
                                    else
                                      %
                                    end_if) $ i = 1 .. nops(plusExpr)),
                               0=hold(_plus), Unsimplified)
                          end_if
                        end_proc):
Content::set("times",   _mult):  /* ! keine Simplifizierung */
Content::set("divide",_divide):  /* ! keine Simplifizierung */
Content::set("power",_power):  /* ! keine Simplifizierung */
Content::set("DOM_INT",DOM_INT, x -> x):
Content::set("DOM_IDENT",DOM_IDENT, x -> x):
Content::set("DOM_FLOAT",DOM_FLOAT, x -> x):
Content::set("DOM_STRING",DOM_STRING, x -> x):
Content::set("list" ,DOM_LIST, proc(x)
                                         local i;
                                       begin
                                         [Content::expr(op(x, i)) $ i = 1 ..nops(x)]
                                       end_proc):
Content::set("set",   DOM_SET, proc(x)
                                         local i;
                                       begin
                                         {Content::expr(op(x,i)) $ i = 1 .. nops(x)}
                                       end_proc): 
Content::set("EQS",   DOM_LIST, proc(x)
                                         local i;
                                       begin
                                         [Content::expr(op(x,i)) $ i = 1 .. nops(x)]
                                       end_proc): 
Content::set("diff"    ,diff):
Content::set("partialdiff"   , diff):
Content::set("partialdiff1"   , diff):
Content::set("eq"  , _equal, proc(x)
                                         local i;
                                       begin
                                         _equal(Content::expr(op(x,1)),  Content::expr(op(x,2)))
                                       end_proc):
Content::set("lt"   , _less):
Content::set("leq", _leequal):
Content::set("neq", _unequal):
Content::set("gt", _less, proc() begin error("not yet implemented") end_proc):
Content::set("geq", _leequal, proc() begin error("not yet implemented") end_proc):
Content::set("and"    , _and):
Content::set("or"     ,  _or):
Content::set("not"    , _not):
Content::set("union", _union):
Content::set("intersect", _intersect):
Content::set("setdiff", _minus):
Content::set("ln"   , ln):
Content::set("log"   , log):
Content::set("conjugate"     ,  conjugate):
Content::set("definiteint"   , int, proc(x)
                                       begin
                                         hold(int)(Content::expr(op(x,3)), Content::expr(op(x,4))=Content::expr(op(x,1))..Content::expr(op(x,2)))
                                       end_proc):
Content::set("int"   , int):
Content::set("sum"    , sum, proc(x)
                                       begin
                                         hold(sum)(Content::expr(op(x,4)), Content::expr(op(x,1))=Content::expr(op(x,2))..Content::expr(op(x,3)))
                                       end_proc):   
Content::set("sum1"    , sum, proc(x)
                                       begin
                                         hold(sum)(Content::expr(op(x,2)), Content::expr(op(x,1)))
                                       end_proc):   
Content::set("sum2"    , sum, proc(x)
                                       begin
                                         hold(sum)(Content::expr(op(x,3)), Content::expr(op(x,1))=Content::expr(op(x,2)))
                                       end_proc): 
Content::set("product"    , product, proc(x)
                                       begin
                                         hold(product)(Content::expr(op(x,4)), Content::expr(op(x,1))=Content::expr(op(x,2))..Content::expr(op(x,3)))
                                       end_proc):   
Content::set("product1"    , product, proc(x)
                                       begin
                                         hold(product)(Content::expr(op(x,2)), Content::expr(op(x,1)))
                                       end_proc):
Content::set("root"    , _power, 
  proc(x)
  begin
    hold(_power)(Content::expr(op(x,2)), 1/Content::expr(op(x,1)))   
  end_proc):   
Content::set("sqrt"    , _power, 
  proc(x)
  begin
    hold(_power)(Content::expr(op(x)), 1/2)
  end_proc):   
Content::set("limit"  , limit, 
  (x, attributes) ->
  hold(limit)(Content::expr(op(x,3)), 
              Content::expr(op(x,1))=Content::expr(op(x,2)))):  
Content::set("limitRight"  , limit, 
  (x, attributes) ->
  hold(limit)(Content::expr(op(x,3)), 
              Content::expr(op(x,1))=Content::expr(op(x,2)), hold(Right))):  
Content::set("limitLeft"  , limit, 
  (x, attributes) ->
  hold(limit)(Content::expr(op(x,3)), 
              Content::expr(op(x,1))=Content::expr(op(x,2)), hold(Left))):  
Content::set("fenced"  , id, x -> Content::expr(op(x,1))):
Content::set("factorial"   , fact):
Content::set("in"   , _in):
Content::set("abs"   , abs):
Content::set("norm"   , norm):
Content::set("normx"   , norm):
Content::set("in"   , _in):
Content::set("det"   , det):
Content::set("ceil"   , ceil):
Content::set("floor"   , floor):
Content::set("rem"   , _mod):
Content::set("quotient"   , _div):
Content::set("fnest"   , _fnest):
Content::set("proc"   , error):
Content::set("slot"   , slot, x -> hold(slot)(Content::expr(op(x,1)), expr2text(op(x, 2)))):
Content::set("polylog"   , polylog):
Content::set("D"   , D):
Content::set("Dpara"   , error):
Content::set("besselI"   , besselI):
Content::set("besselJ"   , besselJ):
Content::set("besselK"   , besselK):
Content::set("besselY"   , besselY):

Content::set("binomial"   , binomial):
Content::set("range"   , _range):
Content::set("compose"   , _fconcat):

// back translation still to be done
Content::set("imageset"   , error):
Content::set("pwcondition"   , error):
Content::set("piecewise"   , error):
Content::set("rhs"   , error):
Content::set("table"   , error):
Content::set("intervaloo"   , error):
Content::set("intervaloc"   , error):
Content::set("intervalco"   , error):
Content::set("intervalcc"   , error):

Content::set("apply", fp::apply, 
                              proc(x)
                                save XxX;
                              begin
                                delete XxX;
                                
                                subs(XxX(Content::expr(op(x, i)) $
                                         i = 2 ..nops(x)),
                                     XxX=Content::expr(op(x,1)), Unsimplified)
                              end_proc):
Content::set("scalarproduct", linalg::scalarProduct):
Content::set("transpose", linalg::transpose):
Content::set("matrix", DOM_ARRAY, 
                         proc(m, attributes, infoString)
                           local rows, cols, i, j;
                         begin
                           rows := text2expr(attributes[Rows]);
                           cols := text2expr(attributes[Cols]);
                           array(1..rows, 1..cols,
                                 (i,j) = Content::expr(op(m,(i-1)*cols+j))
                                 $ j=1..cols $ i=1..rows);
                           if infoString <> "" then
                             eval(text2expr(infoString))(%)
			   else
			     Dom::Matrix()(%);
			   end_if;
                         end_proc):
Content::set("tableau", Tableau, () -> error("not yet implemented")):
Content::set("seq", _exprseq):
Content::set("index", _index):

// end of file 
