// $Date: 2000/05/21 14:57:38 $ $Author: bij $ $Revision: 1.6 $ 
// tonner, 23/06/98 

/*++
output::MMLpresentation -- Domain of MathML presentation outputs
++*/

alias(MathML=output::MMLpresentation):
alias(Priority=output::Priority);


domain MathML
  category output::CatPresentation;
  inherits Dom::BaseDomain;
    
  Name := "MathMLpresentation";
    
  create :=
  proc(mlType : DOM_STRING)
    local argList;
  begin
    argList := split([args(2..args(0))], x -> testtype(x, "_equal"));
    new(dom, "none", mlType, table(op(op(argList,1))),
        op(op(argList,2)))
  end_proc;

  // access to element entries
  priority   := elem -> extop(elem, 1);
  mmlType    := elem -> extop(elem, 2);
  attributes := elem -> extop(elem, 3);
  entries    := elem -> op(elem, 4..extnops(elem));

  // output representation tree -> DOM_STRING
  //
  //    !!!!!  DIESE FUNKTION MUSS IN DEN KERN  !!!!!
  //
  print := proc(arg1) begin dom::do_print(arg1) end_proc;
  
  do_print :=
  proc(arg1)
    name output::print;
    local DOM, process, Print,
          outString:DOM_STRING, indentLevel:DOM_INT;
  begin
    indentLevel := 0;
    outString := "";
    DOM := dom;
    
// extract args
  Print :=
  proc(d)
    name MathML::Print;
    local attributes, i, Indent, Empty;
  begin
    Indent := {"mrow", "mfrac", "mroot", "msqrt", "mstyle", "mpadded",
	       "mphantom", "mfenced", "msub", "msup", "msubsup", "munder",
	       "mover", "munderover", "mmultiscripts", "mtable", "mtr",
	       "mtd", "maction"};
    Empty := {"mspace", "maligngroup", "malignmark", "mprescripts",
	       "none"};
    attributes := DOM::attributes(d);
    "<", DOM::mmlType(d),
    (if attributes <> table() then
       (" ", op(attributes, [i,1]), "=",
	expr2text(op(attributes, [i,2]))) $ i=1..nops(attributes)
     else
       null()
     end_if),
    (if contains(Empty, DOM::mmlType(d)) then "/>"
     else
       ">",
       (if contains(Indent, DOM::mmlType(d)) then
	  DOM::newline(), DOM::indent(DOM::entries(d)), DOM::newline()
	else
	  DOM::entries(d)
	end_if),
     "</", DOM::mmlType(d),">"
     end_if),
    DOM::newline()
  end_proc;

    process :=
    proc(x)
      name  output::print::process;
      local Oseq, i, entries;
    begin
      Oseq := [Print(x)];
// print("process", x, domtype(x), Print, Oseq);
      for i in Oseq do
	if domtype(i) = DOM_STRING then
	  outString := outString.DOM::translate(i);
	elif domtype(i) = output::Indent then
	  if output::Indent::isNewline(i) then
	    outString := _concat(outString, "\n", ("  "$indentLevel));
	  else
	    indentLevel := indentLevel+1;
	    outString := outString."  ";
	    if (entries := output::Indent::entries(i)) <> null() then
	      map(entries,process);
	    end_if;
	    indentLevel := indentLevel-1;
	  end_if;
	elif domtype(i) = dom then
	  process(i);  
	else
// print("process", i, domtype(i));
	  error("output element expected");
	end_if;
      end_for;
    end_proc;

    process(dom::enclose(arg1));
    outString;
  end_proc;

  translate := id;
  
  mi            := () -> dom::create("mi",     args());
  mn            := () -> dom::create("mn",     args());
  mo            := () -> dom::create("mo",     args());
  mtext         := () -> dom::create("mtext",  args());
  mspace        := () -> dom::create("mspace", args());
  ms            := () -> dom::create("ms",     args());
  
  mrow          := () -> dom::create("mrow",  args());
  mfrac         := () -> dom::create("mfrac", args());
  mroot         := () -> dom::create("mroot", args());
  msqrt         := () -> dom::create("msqrt", args());
  mstyle        := () -> dom::create("mstyle", args());
  merror        := () -> dom::create("merror",  args());
  mpadded       := () -> dom::create("mpadded", args());
  mphantom      := () -> dom::create("mphantom", args());
  mfenced       := () -> dom::create("mfenced", args());

  msub          := () -> dom::create("msub", args());
  msup          := () -> dom::create("msup", args());
  msubsup       := () -> dom::create("msubsup", args());
  munder        := () -> dom::create("munder", args());
  mover         := () -> dom::create("mover", args());
  munderover    := () -> dom::create("munderover", args());
  mmultiscripts := () -> dom::create("mmultiscripts", args());
  mprescripts   := () -> dom::create("mprescripts", args());
  none          := () -> dom::create("none", args());
  
  mtable        := () -> dom::create("mtable", args());
  mtr           := () -> dom::create("mtr", args());
  mtd           := () -> dom::create("mtd", args());
  maligngroup   := () -> dom::create("maligngroup", args());
  malignmark    := () -> dom::create("malignmark", args());

  maction       := () -> dom::create("maction", args());

  placeholder   := () -> dom::create("placeholder", dom::brackets(args()));

  keyword := () -> dom::create("mtext", "color"="red", args());
  statement := () -> dom::create("mtext", expr2text(args()));

  // formatting of source output
  newline := proc() option remember; begin output::Indent("newline") end_proc;
  indent :=  () -> output::Indent("indent", args());

end_domain:
  
// end of file 
