/**
  output::isNeg  :  

  Domains with own representation which may contain signed values
  have to provide a isNeg method in order to produce good looking 
  outputs.  It order to check some operands of an element these
  methods may call this function.


  WARNING:  This is a library equivalent of an internal kernel
            function.  Changes have to be done in MIO_is_neg as
            well !!!
*/

output::isNeg := 
proc(x) : DOM_BOOL
  local isneg, re, negate;
begin
  negate := x::dom::_negate;
  if (isneg := x::dom::isNeg) <> FAIL and negate <> FAIL then
    return(isneg(x));
  end_if;
  
  case domtype(x)
    of DOM_INT do
    of DOM_RAT do
    of DOM_FLOAT do
       return(bool(x<0));
       break;

    of DOM_COMPLEX do
       re := op(x,1);
       if re <> 0 then
         return(bool(re<0));
       else
         return(bool(op(x,2)<0));
       end_if;
       break;

    of DOM_EXPR do
       case type(x) 
           of "_negate" do
               return(TRUE);
	       break;

           of "_plus" do
               return(_and(map(op(x), output::isNeg)));
	       break;

	   of "_mult" do
	       if contains({DOM_COMPLEX, DOM_FLOAT, DOM_INT, DOM_RAT}, 
			   domtype(op(x, nops(x)))) then
		  return(output::isNeg(op(x, nops(x))));
               end_if;
	       if contains({DOM_COMPLEX, DOM_FLOAT, DOM_INT, DOM_RAT}, 
			   domtype(op(x, 1))) then
		  return(output::isNeg(op(x, 1)));
               end_if;
               return(FALSE);
               break;
       end_case;
   end_case;
   FALSE
end_proc:
