// $Date: 2000/11/14 10:05:11 $  $Author: frankp $   $Revision: 1.24 $
// igel
// completely rewritten by frankp (21.09.2000)

// TODO: Fuellen schneller gestalten, sehr langsam!

/* -----------------------------------------------------------
    Ellipse2d -- The graphical primitive for two-dimensional ellipses

    Syntax:

    Ellipse2d(m, r1, r2, <op1, op2, ...>)

    m               : either list or point (specifiying the center)
    r1, r2          : arithmetical expressions
    op1, op2, ...   : plot2d options, see plot2d and plot3d for details.

    Example:
    >> plot::Ellipse2d( [1, 2], 3, 3 )

    >> plot::Ellipse2d( [1, 2], 3, 3, Filled = TRUE )

    >> plot::Ellipse2d( [1, 2], 3, 3, Color = RGB::Green )

----------------------------------------------------------------*/
alias( DEFAULTOPTIONS = plot::Ellipse2d::defaultOptions ):
alias( OPTIONNAMES = plot::Ellipse2d::optionNames ):

plot::Ellipse2d:= newDomain("plot::Ellipse2d"):
plot::Ellipse2d::info:= "plot::Ellipse2d -- graphical primitive for ellipses":
plot::Ellipse2d::type:= "graphprim":
plot::Ellipse2d::dimension:= () -> 2:

plot::Ellipse2d::new := proc(m, r1, r2)
    local o, i, opt, options, ellipse;
begin
    if args(0) < 3 then
        error("expecting at least two arguments");
    end_if:

    case domtype(m)
    of DOM_LIST do
        if nops(m) <> 2 or not testtype(m[1], Type::Arithmetical) or not  testtype(m[2], Type::Arithmetical) then
            error("1st argument: expecting a list of two arithmetical expressions")
        end_if;
        break
    of DOM_POINT do
        if nops(m) <> 3 then
            error("1st argument: expecting a two-dimensional point")
        end_if;
        m:= [op(m,1..2)];
        break
    of plot::Point do
        if nops(m) <> 2 then
            error("1st argument: expecting a two-dimensional point")
        end_if;
        m:= [op(m)];
        break
    otherwise
        error("1st argument: expecting a list of two-arithmetical expressions, or a two-dimensional point")
    end_case;

    if not testtype(r1, Type::Arithmetical) then
        error("2nd argument: expecting an arithmetical expression")
    elif not testtype(r2, Type::Arithmetical) then
        error("3rd argument: expecting an arithmetical expression")
    end_if:

    options:= DEFAULTOPTIONS;  // default options (a table)
    opt:= [args(4..args(0))];  // user options (a list)
    // check user options and override the default options:
    for i from 1 to nops(opt) do
        if (o:= plot::Ellipse2d::checkOption(opt[i]))[1] = FALSE then
            error(o[2])
        else
            options[o[2]]:= o[3]
        end_if
    end_for;
        
    ellipse:= newDomain( expr2text(genident("plot::Ellipse2d_")) );
        
    // attributes:
    ellipse::center         := m;
    ellipse::radius1        := r1;
    ellipse::radius2        := r2;
    ellipse::range          := 0..2*PI:
    ellipse::refreshPlotdata:= TRUE;
    ellipse::plotdata       := [];
    ellipse::options        := options;

    // return the new curve:
    return(new(plot::Ellipse2d, ellipse));
end_proc:

plot::Ellipse2d::print := proc(e)
begin
    e:= extop(e,1);
    hold(plot::Ellipse2d)(e::center,e::radius1,e::radius2)
end_proc:

// Note: Do not return an error message in the case of an invalid call,
//       because then the object would be destroyed! Hence a warning 
//       message is issued, and the call of this method has no effect 
//       on the object.
plot::Ellipse2d::slot:= proc(o,s,v=null())
    local e, t, _s;
begin
    if args(0) = 3 then
        // slot assignment:
        e:= extop(o,1);

        // first handle all user-accessable attributes:
        case s 
        of "refreshPlotdata" do
            if v <> TRUE and v <> FALSE then
                warning("attribute 'refreshPlotdata': expecting either 'TRUE' or 'FALSE'; assignment ignored");
                return( o )
            else
                e::refreshPlotdata:= v;
                return( extsubsop(o,1=e) )
            end_if
        of "center"  do
            if not testtype(v, Type::ListOf(Type::Arithmetical,2,2)) then
                warning("attribute 'center': expecting a list of two arithmetical expressions; assignment ignored");
                return( o )
            else
                e::center:= v;
                e::refreshPlotdata:= TRUE;
                return( extsubsop(o,1=e) )
            end_if
        of "radius1" do
        of "radius2" do
            if not testtype(v, Type::Arithmetical) then
                warning("attribute '".s."': expecting an arithmetical expression; assignment ignored");
                return( o )  
            else
                e::refreshPlotdata:= TRUE;
                return( extsubsop(o,1=slot(e,s,v)) )
            end_if
        of "range" do
            if not testtype(v, Type::Arithmetical..Type::Arithmetical) then
                if not testtype(v, Type::ListOf(Type::Arithmetical,2,2)) then
                    warning("attribute 'range': expecting form 'a..b'; assignment ignored");
                    return( o )
                else
                    // convert v into the form a..b:
                    v:= op(v,1)..op(v,2)
                end_if
            end_if;
            e::refreshPlotdata:= TRUE;
            return( extsubsop(o,1=slot(e,s,v)) )
        of "options" do
            if domtype(v) <> DOM_TABLE then
                warning("attribute 'options': expecting a table of plot options; assignment ignored");
                return( o )
            else
                e::options:= v;
                e::refreshPlotdata:= TRUE;
                return( extsubsop(o,1=e) )
            end_if
        end_case;

        // an option given?
        t:= plot::Ellipse2d::checkOption(text2expr(s) = v);
        if t[1] = FALSE then
            warning( t[2]."; assignment ignored" );
            return(o)
        else
            e::options[t[2]]:= t[3];
            e::refreshPlotdata:= TRUE;
            return( extsubsop(o,1=e) )
        end_if
    else
        // slot access:
        e:= extop(o,1);
        if (t:= slot(e,s)) <> FAIL then
            return(t)
        else
            _s:= text2expr(s);
            if not contains(OPTIONNAMES,_s) then
                error("unknown attribute or option '".s."'")
            elif type((t:= e::options[_s])) <> "_index" then
                return(t)
            else
                return(FAIL)
            end_if
        end_if
    end_if
end_proc:

plot::Ellipse2d::getPlotdata := proc(c)
    local e, p, a, b, s, i, h, x, range, grid, curve, options;
begin
    e:= extop(c,1);
    if e::refreshPlotdata = FALSE then return( e::plotdata ) end_if;
    
    p    := e::center;
    a    := e::radius1;
    b    := e::radius2;
    range:= e::range;

    // remove specific Ellipse2d options and options with value NIL:
    // create 2d-object:
    if e::options[hold(Filled)] = FALSE then 
        options:= e::options;
        delete options[hold(Filled)];
        // convert e::options into a list:
        options:= [op(options)];

        // create a plot::Curve2d:
        x:= genident("x");
        curve:= plot::Curve2d([p[1] + a*cos(x), p[2] + b*sin(x)], x = range, op(options))
    else
        // create a plot::Polygon:
        options:= e::options;
        if type((grid:= options[hold(Grid)])) = "_index" then
            grid:= [100] // use the default value if the option was deleted
        end_if;

        // remove specific Curve2d and Ellipse2d options:
        delete options[hold(Filled)], options[hold(Grid)], options[hold(Smoothness)], options[hold(Style)];
        // convert e::options into a list:
        options:= [op(options)];

        s:= op(range, 1);
        h:= (op(range, 2) - s)/grid[1];

        curve:= plot::Polygon(
            p, plot::Point( p[1] + a*cos(s+i*h),  p[2] + b*sin(s+i*h) ) $ i = 0..grid[1],
            hold(Closed) = TRUE, hold(Filled) = TRUE, op(options)
        );
    end_if;

    e::refreshPlotdata:= FALSE;
    e::plotdata:= curve::dom::getPlotdata(curve)
end_proc:

plot::Ellipse2d::copy := proc(c)
begin
    return(extsubsop(c, 1=newDomain(genident("plot::Ellipse2d_"), extop(c, 1))))
end_proc:

plot::Ellipse2d::modify := proc(c)
    local m, o, options;
begin
    m := plot::Ellipse2d::copy(c);
    options := [args(2..args(0))];

    ( slot(m, expr2text(op(o, 1)), op(o, 2)) ) $ o in options;
   
    m::refreshPlotdata := TRUE;
    return(m)
end_proc:

plot::Ellipse2d::op := proc(c, n)
begin
    if args(0) < 1 or args(0) > 2 then
        error("invalid number of arguments")
    end_if;

    c:= extop(c,1);
    c:= [c::center,c::radius1,c::radius2];
    if args(0) = 1 then op(c) else op(c, n) end_if
end_proc:

plot::Ellipse2d::nops := c -> 3:

plot::Ellipse2d::_index := proc(c:plot::Ellipse2d, n:Type::PosInt)
begin
    if args(0) <> 2 then
        error("expecting two arguments")
    end_if;
    
    c:= extop(c,1);
    c:= [c::center,c::radius1,c::radius2];
    if n > nops(c) then
        error("index out of range")
    else
        return( c[n] )
    end_if
end_proc:

// Note: Do not return an error message in the case of an invalid call,
//       because then the object would be destroyed! Hence a warning 
//       message is issued, and the call of this method has no effect 
//       on the object.
plot::Ellipse2d::set_index := proc(c:plot::Ellipse2d, n:Type::PosInt, v)
    local e;
begin
    if args(0) <> 3 then
        warning("wrong number of arguments; assignment ignored");
        return(c)
    end_if; 

    e:= extop(c,1);
    case n
    of 1 do
        if v <> e::center then 
            if not testtype(v, Type::ListOf(Type::Arithmetical,2,2)) then
                warning("3rd argument: expecting a list of two arithmetical expressions; assignment ignored");
                return(c)
            end_if;
            e::refreshPlotdata:= TRUE;
            e::center:= v;
        end_if;
        return( extsubsop(c,1=e) )
    of 2 do
        if v <> e::radius1 then 
            if not testtype(v, Type::Arithmetical) then
                warning("3rd argument: expecting an arithmetical expression; assignment ignored");
                return(c)
            end_if;
            e::refreshPlotdata:= TRUE;
            e::radius1:= v;
        end_if;
        return( extsubsop(c,1=e) )
    of 3 do
        if v <> e::radius2 then 
            if not testtype(v, Type::Arithmetical) then
                warning("3rd argument: expecting an arithmetical expression; assignment ignored");
                return(c)
            end_if;
            e::refreshPlotdata:= TRUE;
            e::radius2:= v;
        end_if;
        return( extsubsop(c,1=e) ) 
    otherwise
        warning("index out of range; assignment ignored");
        return(c)
    end_case
end_proc:

plot::Ellipse2d::checkOption:= proc(opt)
    local value;
begin
    if type(opt) <> "_equal" then
        return( [FALSE,"options must have the form 'OptionName = value'"] )
    end_if;

    value:= op(opt,2); opt:= op(opt,1);
    if not contains( OPTIONNAMES,opt ) then
        return( [FALSE,"unknown option name '".expr2text(opt)."'"] )
    else
        // First, handle special cases for certain options:
        // Color: Beside the standard format 'Color = [{Height,Flat},[r,g,b]]',
        //        allow 'Color = [r,g,b]' which corresponds to '[Flat,[r,g,b]]'.
        if opt = hold(Color) then
            if (value:= plot::getColorFormat(value)) = FAIL then
                return( [FALSE,"option 'Color': invalid color specification"] )
            end_if
        end_if;

        return( [TRUE,opt,value] )
    end_if
end_proc:

OPTIONNAMES:= {
  // options for 2-dimensional objects:
  hold(Color),hold(LineStyle),hold(LineWidth),hold(PointStyle),
  hold(PointWidth),hold(Title),hold(TitlePosition),
  // options for 2d-curves:
  hold(Grid),hold(Smoothness),hold(Style),
  // options for plot::Ellipse2d:
  hold(Filled)
}:

DEFAULTOPTIONS:= table():
// Default options for objects of the domain plot::Curve2d:
DEFAULTOPTIONS[hold(Color)] := [hold(Flat),RGB::Red]: // needed if Filled=TRUE
DEFAULTOPTIONS[hold(Grid)]  := [100]:
DEFAULTOPTIONS[hold(Filled)]:= FALSE:

