// $Date: 2000/11/14 12:37:03 $  $Author: frankp $   $Revision: 1.4 $
// frankp

/* -----------------------------------------------------------
    NullObject -- The graphical primitive representing the 
                  two dimensional graphical null object
----------------------------------------------------------------*/ 
plot::NullObject:= newDomain("plot::NullObject"):
plot::NullObject::info:= "plot::NullObject -- primitive for representing graphical null objects":
plot::NullObject::type:= "graphprim":
plot::NullObject::dimension:= g -> (extop(g,1))::dimension:

plot::NullObject::new:= () -> error("you can not create new objects of this domain"):
    
plot::NullObject::print:= e -> (extop(e,1))::key:

plot::NullObject::op:= proc()
begin
    if args(0) < 1 or args(0) > 2 then
        error("invalid number of arguments")
    else
        return(FAIL)
    end_if
end_proc:

plot::NullObject::nops:= g -> 0:

plot::NullObject::_index:= proc()
begin
    if args(0) <> 2 then
        error("expecting two arguments")
    else
        error("invalid index")
    end_if
end_proc:

// Note: Do not return an error message in the case of an invalid call,
//       because then the object would be destroyed! Hence a warning 
//       message is issued, and the call of this method has no effect 
//       on the object.
plot::NullObject::set_index := proc(n)
begin
    if args(0) <> 3 then
        warning("expecting three arguments; assignment ignored");
        return(n)
    else
        warning("invalid index; assignment ignored");
        return(n)
    end_if
end_proc:

plot::NullObject::getPlotdata := () -> []:

// Note: Do not return an error message in the case of an invalid call,
//       because then the object would be destroyed! Hence a warning 
//       message is issued, and the call of this method has no effect 
//       on the object.
plot::NullObject::slot:= proc(o, s)
begin
    if args(0) = 3 then
        // slot assignment:
        warning("unknown attribute '".s."'; assignment ignored");
        return(o)
    else
        case s
        of "dimension" do
            return((extop(o,1))::dimension)
        of "plotdata" do
            return([])
        otherwise
            error("unknown attribute '".s."'")
        end_case
    end_if
end_proc:

// create the 2-dimensional and 3-dimensional graphical null objects:
plot::NullObject2d:= proc()
    local null2d;
begin
    null2d:= newDomain("plot::NullObject2d");

    null2d::dimension:= 2;
    null2d::plotdata := [];

    // return the object:
    return( new(plot::NullObject, null2d) )
end_proc():

plot::NullObject3d:= proc()
    local null3d;
begin
    null3d:= newDomain("plot::NullObject3d");

    null3d::dimension:= 3;
    null3d::plotdata := [];

    // return the object:
    return( new(plot::NullObject, null3d) )
end_proc():

