/* -------------------------------------------------------------
    area -- function that returns a two-dimensional graphical 
            primitive of type plot::Polygon describing an area

    Syntax:

    area(nodes, f <, g><, graphoptions>)

    nodes        : list of arithmetical expressions 
    f, g         : DOM_PROC 
    Options      : DOM_IDENT
    graphoptions : list of plot2d options, see plot2d and plot3d for details.

    Description:
    
    The functions f and g form the boundary of the area, it's approximated 
    by a polygon given by the list of nodes (the depending values to build 
    a vertex of the polygon are calculated automatically).

    The default value of g is the zero-function.

    Example:
    >> plot::area( [ i/10 $ i = 0..10 ], x -> cos(x) )

    >> plot::area( [ i/10 $ i = 0..10 ], x -> cos(x), x -> sin(x) )

    >> plot::area( [ i/10 $ i = 0..10 ], x -> cos(x), x -> sin(x), Color = RGB::Blue )

----------------------------------------------------------------*/


plot::area := proc(stuetzen:Type::ListOf(Type::Arithmetical), f:DOM_PROC, g = 0, options = [])
    local knoten;
begin
    case args(0) 
        of 0 do
        of 1 do 
            error("at least 2 arguments expected");
        of 2 do
            break:
        of 3 do
            options := [];
            g := 0;
        break:
        otherwise
            if not domtype(g) = DOM_PROC then
                error("3rd argument: procedure expected");
            end_if:
            options := [args(4..args(0))];
        break:
    end_case:
    
    if options = [] then
        options := [hold(Closed) = TRUE, hold(Filled) = TRUE];
    else
        if not contains({op( map(options, op, 1)) }, hold(Filled)) then    
            options := append( options, hold(Filled) = TRUE ); 
        end_if:
    end_if:

    stuetzen := (sort@float)(stuetzen);
    knoten := map(stuetzen, i -> [i, f(i)]);
    stuetzen := sort(map(stuetzen, _mult, -1.0));
    knoten := knoten . map(stuetzen, i -> [-i, g(-i)]);

    return( plot::Polygon(op(knoten), op(options)) )
end_proc:
