// $Date: 2000/07/05 09:21:31 $  $Author: frankp $   $Revision: 1.1 $

/*++
    line -- graphical object for lines

    Syntax:

    Line(p1, p2 <, op1, op2, ...>)

    p1, p2       : either graphical objects of domain type plot::Point or
                   lists of two or three arithmetical expressions 
    op1, op2, ...: plot2d or plot3d options, see plot2d and plot3d for details.

    Description:
    
    line(p1,p2) returns the line joining the points p1 and p2 as an object of
    domain type plot::Polygon. 

    Example:
    >> plot::line( [ 1, 2 ], [ 3, 4 ] )

    >> plot::line( [ 1, 2 ], [ 3, 4 ], Color = RGB::Blue, LineStyle = Dashedlines )
++*/
plot::line := proc(x0, y0)
begin
    if args(0) < 2  then
        error("at least two arguments expected")
    end_if;
    
    if (x0:= plot::Point::convert(x0)) = FAIL then
        error("1st argument: two- or three-dimensional point expected")
    elif (y0:= plot::Point::convert(y0)) = FAIL then
        error("2nd argument: two- or three-dimensional point expected")
    else
        return( plot::Polygon(x0, y0, args(3..args(0))) )
    end_if
end_proc:

