// $Date: 2000/11/03 16:32:31 $  $Author: frankp $   $Revision: 1.3 $

/*++

    plot::cylindrical - generates graphics in cylindrical coordinates

    Call:
    cylindrical([f1, f2, f3], u = a..b, v = c..d<, options>)

    Parameters:
    f1, f2, f3: functional expressions in u and v (r(u,v), phi(u,v), z(u,v))
    u, v      : identifiers
    a, b, c, d:
    options   : plot options for three-dimensional graphical objects

    Return Value:
    A graphical object of the domain type plot::Surface3d.

    Examples:
    >> s:= plot::cylindrical(
         [1, u, z], u = -PI..PI,  z = -1..1,
         Grid = [20, 20], Style = [HiddenLine, Mesh]
       );
    >> plot(s, Axes = Box, Ticks = 0)
    >> s1:= plot::cylindrical(
         [u, u, z], u = -PI..PI, z = -PI..PI,
         Grid = [30, 30], Color = [Height],
         Style = [ColorPatches, AndMesh]
       ):
       s2:= plot::cylindrical(
         [-u, u, z], u = -PI..PI, z = -2..2,
         Grid = [30, 30], Color = [Height],
         Style = [ColorPatches, AndMesh]
       ):
    >> plot(s1, s2, Axes = Box, Ticks = 0)
++*/
plot::cylindrical:= proc(surf,u,v)
begin
    if args(0) < 3 then
        error("expecting at least three arguments")
    else
        plot::Surface3d( 
            [surf[1]*cos(surf[2]),surf[1]*sin(surf[2]),surf[3]],
            u, v, args(4..args(0))
        )
    end_if
end_proc:

