// $Date: 2000/10/30 18:01:45 $  $Author: frankp $   $Revision: 1.4 $

/*++
    plot::density - generates density plots

    Call:
    density( [f1, f2, f3], u = a..b, v = c..d<, options> )

    Parameters:
    f1, f2, f3: functional expressions in u and v
    u, v      : identifiers
    a, b, c, d: arithmetical expressions
    options   : plot options for three-dimensional graphical objects

    Return Value:
    A graphical scene (!)

    Details:
    This isn't really a density plot. It is only a special
    view at this 3D scene. See camera and focal point.

    Examples:
    >> d:= plot::density([u,v,1/2*sin(u*u + v*v)], u=[0,PI], v=[0, PI]):
    >> plot(d)
++*/
plot::density:= proc(f,ur,vr)
    local options, s;
begin
    if args(0) < 3 then
        error("at least three arguments expected")
    end_if;

    options:= [args(4..args(0))];
    if not has(options,hold(Color)) then
        options:= append(options, hold(Color)=[hold(Height)])
    end_if;
    if not has(options,hold(Style)) then
        options:= append(options, hold(Style)=[hold(ColorPatches),hold(Only)])
    end_if;

    s:= plot::Surface3d(f,ur,vr,op(options));
    plot::Scene( s,
        hold(CameraPoint) = [0,-100,100000000], 
        hold(FocalPoint)  = [0,0,0],
        hold(Axes)=hold(Box), 
        hold(Ticks)=[5,5,0], 
        hold(Labels)=[expr2text(op(ur,1)),expr2text(op(vr,1)),""]
    )
end_proc:

