// $Date: 2000/10/30 18:01:46 $  $Author: frankp $   $Revision: 1.2 $

/*++

    plot::polar - generates graphics in polar coordinates

    Call:
    polar([f1, f2], phi = pmin..pmax<, options>)

    Parameters:
    f1, f2 : functional expressions in phi
    phi    : identifier (the angle)
    pmin   : minimum value for the range in that phi is evaluated
    pmax   : maximum value for the range in that phi is evaluated
    options: plot options for two-dimensional graphical objects

    Return Value:
    A graphical object of the domain type plot::Curve2d.

    Examples:
    >> c:= plot::polart([phi, phi], phi = [0, 4*PI], Grid = [100]):
    >> plot(c, Axes = None, Ticks = 0)
++*/
plot::polar:= proc(curve,phi)
begin
    if args(0) < 2 then
        error("expecting at least two arguments")
    else
        plot::Curve2d( 
            [curve[1]*cos(curve[2]),curve[1]*sin(curve[2])],phi,args(3..args(0))
        )
    end_if
end_proc:

