// $Date: 2000/11/03 16:32:31 $  $Author: frankp $   $Revision: 1.3 $

/*++

    plot::spherical - generates graphics in spherical coordinates

    Call:
    spherical([f1, f2, f3], u = a..b, v = c..d<, options>)

    Parameters:
    f1, f2, f3: functional expressions in u and v (r(u,v), phi(u,v), theta(u,v))
    u, v      : identifiers
    a, b, c, d: arithmetical expressions
    options   : plot options for three-dimensional graphical objects

    Return Value:
    A graphical object of the domain type plot::Surface3d.

    Examples:
    >> s:= plot::spherical(
         [1, u, v], u = -PI..PI,  v = 0..PI,
         Grid = [20, 20], Style = [HiddenLine, Mesh]
       );
    >> plot(s, Axes = Box, Ticks = 0)
++*/
plot::spherical:= proc(surf,u,v)
begin
    if args(0) < 3 then
        error("expecting at least three arguments")
    else
        plot::Surface3d( 
            [surf[1]*cos(surf[2])*sin(surf[3]),
             surf[1]*sin(surf[2])*sin(surf[3]),
             surf[1]*cos(surf[3])
            ],u,v,args(4..args(0))
        )
    end_if
end_proc:

