// kg, 08/07/94
// Turtle -- 2d turtle graphic primitives

/*
Operands of a turtle:
1 - x-position
2 - y-position
3 - direction radian (0..2*PI)
4 - list of polygons of two points
5 - actual color, a list [r,g,b]
6 - state stack
*/

domain plotlib::Turtle

// a new turtle starts at (0,0) heading north, green color, empty stack
new:= proc() begin
    if args(0) <> 0 then error("no args allowed") end_if;
    new(dom, 0.0, 0.0, 0.5*float(PI), [], [0.0,1.0,0.0], [])
end_proc;

// print -- print turtle
print:= proc(t:dom) begin
    "(".expr2text(extop(t,1)).",".expr2text(extop(t,2)).") -> ".
    expr2text(extop(t,3)*180.0/float(PI))
end_proc;

// testtype -- test if expression is turtle
testtype:= proc(e,T) begin
    if T = dom then
	if domtype(e) = dom then
	    return(TRUE)
	end_if
    end_if;
    FAIL
end_proc;

// color -- set actual turtle color
color:= proc(t:dom, r:Type::Real, g:Type::Real, b:Type::Real) begin
    r:= float(r);
    g:= float(g);
    b:= float(b);
    if testargs() then
	if max(r,g,b) > 1.0 then
	    error("wrong color value")
	end_if;
	if max(r,g,b) < 0.0 then
	    error("wrong color value")
	end_if;
    end_if;
    extsubsop(t, 5=[r,g,b])
end_proc;

// left -- turn left about d without changing position
left:= proc(t:dom, d:Type::Real) begin
    extsubsop(t, 3=(extop(t,3) + float(d*PI/180.0)))
end_proc;

// right -- turn right about d without changing position
right:= proc(t:dom, d:Type::Real) begin
    extsubsop(t, 3=(extop(t,3) - float(d*PI/180.0)))
end_proc;

// move -- move about length l without drawing
move:= proc(t:dom, l:Type::Real) begin
    l:= float(l);
    extsubsop(t, 1 = extop(t,1) + l*cos(extop(t,3)),
		 2 = extop(t,2) + l*sin(extop(t,3)))
end_proc;

// line -- draw line with length l and actual color #
line:= proc(t:dom, l:Type::Real) local x, y, x0, y0; begin
    l:= float(l);
    x0:= extop(t,1);
    y0:= extop(t,2);
    x:= x0 + l*cos(extop(t,3));
    y:= y0 + l*sin(extop(t,3));
    extsubsop(t, 1 = x, 2 = y, 
    	4 = append(extop(t,4), polygon(point(x0,y0), point(x,y), Color=extop(t,5))))
end_proc;

// push -- push actual state onto stack
push:= proc(t:dom) begin
    extsubsop(t, 6 = [[extop(t,1), extop(t,2), extop(t,3), extop(t,5)], extop(t,6)])
end_proc;

// pop -- pop new turtle state from stack
pop:= proc(t:dom) local stack, state; begin
    stack:= extop(t,6);
    if nops(stack) = 0 then return(t) end_if;
    state:= stack[1];
    new(dom, state[1], state[2], state[3], extop(t,4), state[4], stack[2])
end_proc;

// pos -- return actual position
pos:= proc(t:dom) begin
    (extop(t,1), extop(t,2))
end_proc;

// dir -- return actual direction
dir:= proc(t:dom) begin
    extop(t,3)*180.0/float(PI)
end_proc;

// path -- return turtle path in a form suitable for a plot2d-command
path:= proc(t:dom) begin
    [ Mode=List, extop(t,4) ]
end_proc;

// plot -- draw turtle path
plot:= proc(t:dom) begin
    plot2d( Axes=None, dom::path(t) )
end_proc;

end_domain:

// end of file
