/*
Eingabe: Argumente gemaess der erweiterten Spezifikation von plot2d
Ausgabe: Modifizierte Argumente konform zur urspruenglichen Spezifikation von 
         plot2d und der internen Funktion plot. 
	 Untersucht wird:
	 1) Clipping gemaess ViewingBox, sofern die ViewingBox angegeben wurde.
	 2) Untersuchung, in welchen Teilintervallen die angegebenen Funktionen
	    reellwertig sind, sofern RealValuesOnly bzw. RealsOnly auf TRUE
	    gesetzt wurde.
	 3) Untersuchung von Unstetigkeitsstellen, sofern Discont auf TRUE
	    gesetzt wurde.
	 4) Aufsplitten von geschlossenen und/oder gefuellten Polygonzuegen
	    in mehrer Polygonzuege, die jeweils aus genau aus 3 Punkten bestehen.
*/

// alias(concat_list(a, b) = append(a, b)):
alias(concat_list(a, b) = [b].a):

plotlib::check_args := proc()
  local check_discont, check_functions, clipping, grid,
        gridlines, gridcolor, gridstyle, gridwidth,
	i, IVB, l, l1, labeling, LogXY, n, na, no,  
        new_args, new_list, obj_def, obj_empty, obj_nr,   
        p, q, t, setVB, ticks, vbox;
begin

  new_args := [args()]; na := new_args; 
  clipping := FALSE; check_functions := FALSE; 
  check_discont := FALSE; setVB := FALSE;
  gridlines := FALSE;
  obj_def := FALSE; obj_empty := TRUE;

  labeling  := TRUE;        // Labels on/off
  grid      := 100;         // Number of grid points
  ticks     := [5, 5];      // Number of Tickmarks
  gridcolor := RGB::Gray;   // Color of grid lines
  gridstyle := DashedLines; // Style of grid lines
  gridwidth := 1;           // Width of grid lines, integer > 0

  vbox := [ XMin, XMax, YMin, YMax ];
  IVB := [infinity, -infinity, infinity, -infinity];

  t := select(new_args, has, ViewingBox);
  if t <> [] and type(t[1]) = "_equal" then
     // Loesche die Option ViewingBox = ... aus den Argumenten
     new_args := subs(new_args, op(t)=null());
     na:=new_args; 
     // t = [ViewingBox=lhs]
     vbox := plotlib::check_vb(op(t[1],2));
  end_if;

  LogXY := [FALSE, FALSE]; 

  t := select(new_args, has, AxesScaling);
  if t <> [] then 
     if not ( type(t[1]) = "_equal" and type((l:=op(t[1], 2))) = DOM_LIST and 
              nops(l) = 2 and (l[1] = Lin or l[1] = Log) and 
       	      (l[2] = Lin or l[2] = Log) 
 	    ) then
        error(expr2text(op(t[1], 1)).": [x, y] where x and y are Lin or Log expected.")
     else
        if l[1] = Log then LogXY[1] := TRUE; setVB := TRUE end_if;
        if l[2] = Log then LogXY[2] := TRUE; setVB := TRUE end_if;
     end_if;
  end_if;

  t := select(new_args, has, Ticks);
  if t <> [] then 
     if not ( type(t[1]) = "_equal" ) then
        error("Illegal Ticks.")
     end_if;
     // Ersetze die Option Ticks = ... durch neuen Wert
     p := plotlib::check_ticks(op(t[1],2), vbox, ticks, LogXY);
     new_args := subs(new_args, op(t)=p);
     na:=new_args; 
  end_if;

  if vbox[1] = XMin then vbox[1] := -infinity end_if;
  if vbox[2] = XMax then vbox[2] :=  infinity end_if;
  if vbox[3] = YMin then vbox[3] := -infinity end_if;
  if vbox[4] = YMax then vbox[4] :=  infinity end_if;
  if not testtype(vbox, Type::ListOf(Type::Arithmetical, 4,4)) or
     vbox[1] >= vbox[2] or vbox[3] >= vbox[4] then
     error("Illegal ViewingBox.")
   end_if;
   vbox := [vbox[1], vbox[3], vbox[2], vbox[4]];
   if vbox <> [ -infinity, -infinity, infinity, infinity ] then
      clipping := TRUE;
   end_if;

  t := select(new_args, has, {RealsOnly, RealValuesOnly});
  if t <> [] then 
     if not ( type(t[1]) = "_equal" and 
              domtype((check_functions :=op(t[1], 2))) = DOM_BOOL and
              check_functions <> UNKNOWN 
	    ) then
        error(expr2text(op(t[1], 1)).": TRUE or FALSE expected.")
     end_if;
     // Loesche die Option RealValuesOnly = ... aus den Argumenten
     new_args := subs(new_args, op(t)=null());
     na:=new_args; 
  end_if;

  t := select(new_args, has, Discont);
  if t <> [] then 
     if not ( type(t[1]) = "_equal" and 
              domtype((check_discont :=op(t[1], 2))) = DOM_BOOL and
              check_discont <> UNKNOWN 
	    ) then
        error(expr2text(op(t[1], 1)).": TRUE or FALSE expected.")
     end_if;
     // Loesche die Option Discont = ... aus den Argumenten
     new_args := subs(new_args, op(t)=null());
     na:=new_args; 
  end_if;

  t := select(new_args, has, GridLines);
  if t <> [] then 
     if nops(t) <> 1 then 
        error("GridLines declared twice.")
     end_if;
     // t =[GridLines=...] or
     // t =[[Mode=GridLines, ...]]
     if type(t[1]) = DOM_LIST then 
        if not ( type(t[1][1]) = "_equal" and 
                 op(t[1][1],1) = Mode and op(t[1][1],2) = GridLines ) then
           error("[Mode=GridLines, ...] expected.")
        end_if
     elif type(t[1]) = "_equal" and op(t[1], 1) = GridLines then
        l := op(t[1], 2);
	if not has(l, [0, None,[0,0]]) then
           gridlines := l; 
	   setVB := TRUE;
	end_if;
	// Loesche die Option GridLines = ... aus den Argumenten
        new_args := subs(new_args, op(t)=null());
        na:=new_args; 
     else
        error(t[1]." : Illegal specification of GridLines.")
     end_if;
  end_if;

  t := select(new_args, has, GridLinesStyle);
  if t <> [] then
     if type(t[1]) <> "_equal" then
        error("Illegal GridLinesStyle.")
     end_if;
     gridstyle := op(t[1],2);
     new_args := subs(new_args, op(t)=null());
     na:=new_args;
  end_if;

  t := select(new_args, has, GridLinesColor);
  if t <> [] then
     if type(t[1]) <> "_equal" then
        error("Illegal GridLinesColor.")
     end_if;
     gridcolor := op(t[1],2);
     new_args := subs(new_args, op(t)=null());
     na:=new_args;
  end_if;

  t := select(new_args, has, GridLinesWidth);
  if t <> [] then
     if type(t[1]) <> "_equal" then
        error("Illegal GridLinesWidth.")
     end_if;
     gridwidth := op(t[1],2);
     new_args := subs(new_args, op(t)=null());
     na:=new_args;
  end_if;

  t := select(new_args, has, Labeling);
  if t = [] then
     new_args := [Labeling = labeling].new_args // Labeling: Set default value
  end_if;

/*
  t := select(new_args, has, SetViewingBox);
  if t <> [] then 
     if not ( type(t[1]) = "_equal" and 
              domtype((setVB :=op(t[1], 2))) = DOM_BOOL and
              setVB <> UNKNOWN 
	    ) then
        error(expr2text(op(t[1], 1)).": TRUE or FALSE expected.")
     end_if;
     // Loesche die Option SetViewingBox = ... aus den Argumenten
     new_args := subs(new_args, op(t)=null());
     na:=new_args; 
  end_if;
*/

/*
  if hastype(new_args, DOM_POLYGON) or 
     has(new_args, {Mode=Curve}) then
*/

     // Es wird nach geschlossenen und/oder gefuellten
     // Polygonen gesucht, die aus mehr als 3 Punkten
     // bestehen. Diese koennen durch die grundlegende
     // plot-Routine nicht behandelt werden. Um dies
     // jedoch in plot2d zu ermoeglichen, werden solche
     // Polygonzuege mittels plotlib::polygonplot
     // triangularisiert, d.h. in mehrere Polygone mit
     // jeweils nur 3 Punkten aufgeteilt.

     // Da VCam das erste Objekt zuletzt und das letzte
     // Objekt zuerst zeichnet, werden hier die Objekte
     // umsortiert, d.h. [Mode=obj1,...],...,[Mode=objN,...]
     // wird zu [Mode=objN,...],...,[Mode=obj1,...]
     // Ist dieses Verhalten nicht gewuenscht, so kann der
     // Alias concat_list umgesetzt werden.
     // Reihenfolge der Eingabe beibehalten:
     //      alias(concat_list(a, b) = append(a, b)):
     // Reihenfolge der Eingabe umkehren:
     //      alias(concat_list(a, b) = [b].a):

     na := []; // Liste der Szene-Einstellungen
     no := []; // Liste der Objekte Mode=...

     for l in new_args do
         if type(l) = DOM_LIST and type(l[1]) = "_equal" and
            op(l[1], 1) = Mode then
	    
	   if op(l[1], 2) = List then
            // [Mode=List, l2], l2 = [...]
            new_list := [];
            for p in l[2] do
/*
                if domtype(p) = DOM_POLYGON and 
                   (n := nops(p)) > 6       and 
                   (op(p, n-1) or op(p, n)) then
                   // Gefuelltes oder geschlossenes Polygon 
                   // bestehend aus mehr als 3 Punkten
                   q := (plotlib::polygonplot)::PlotScene([p]);
                   // q = [Mode=List, [...]]
                   new_list := append(new_list, op(q[2]));
*/
                if domtype(p) = DOM_POLYGON then
		   n := nops(p);

		   // Nehme alle Punkte einzeln auf, die Farbinformationen
		   // enthalten.
                   // new_list := new_list . select([op(p, i)$i=1..n-3], 
		   //                               x-> testtype(op(x,3), DOM_LIST));
		   if domtype(op(p, n-2)) <> NIL then
		      // polygon definiert selbst eine Farbe
		      // Loesche alle Farbinformationen in point,
		      // d.h. point(x,y,Color=...) -> point(x,y)
		      p := subsop(p, [i,nops(op(p,i))]=NIL $ i=1..n-3)
		   end_if;
		   if n > 6 and (op(p, n-1) or op(p, n)) then
                      // Gefuelltes oder geschlossenes Polygon 
                      // bestehend aus mehr als 3 Punkten
                      q := (plotlib::polygonplot)::PlotScene([p]);
                      // q = [Mode=List, [...]]
                      new_list := append(new_list, op(q[2]));
                   else
		      new_list := append(new_list, p);
		   end_if
                else
                   new_list := append(new_list, p);
                end_if
            end_for;
	    no := concat_list(no, [Mode=List, new_list, l[i]$i=3..nops(l)])
	  elif op(l[1], 2) = Curve then
            if domtype(l[2][1]) = Factored then
	       l[2][1] := expr(l[2][1])
	    end_if;
            if domtype(l[2][2]) = Factored then
	       l[2][2] := expr(l[2][2])
	    end_if;
	    if not has(l, Grid) then
	       no := concat_list(no, l.[Grid=[grid]])
	    else
	       no := concat_list(no, l)
	    end_if
	  else
            na := append(na, l);
          end_if 
         else
            na := append(na, l);
         end_if
     end_for;

     na := na.no;

  // end_if;

 if not (clipping or setVB or ((check_functions or check_discont) and has(na, Curve))) then 
    return(op(na)) 
 end_if;

 setVB := TRUE;

 obj_nr := 0; new_args := []; 

 for l in na do

     if type(l) = DOM_LIST and type(l[1]) = "_equal" and
        op(l[1], 1) = Mode then 

        case op(l[1], 2)

          of Curve do
             // [ Mode = Curve, [fx(t), fy(t)], t=[a,b], ... ] 
             obj_def := TRUE; obj_nr := obj_nr + 1;
             l1 := plotlib::clip2d(Curve, l, vbox, obj_nr, clipping,
	                           check_functions, check_discont, LogXY, 
				   FALSE, setVB);
             if setVB and l1 <> null() then IVB := plotlib::IVB(IVB) end_if;
             if l1 <> null() then obj_empty := FALSE end_if;
	     new_args := append(new_args, l1);
             break;
        
          of List do
             // [ Mode = List, [... ] ] 
             obj_def := TRUE;
	     if clipping or setVB then
	        l1 := plotlib::clip2d(List, l, vbox, FALSE, TRUE, setVB);
		if setVB and l1 <> null() then IVB := plotlib::IVB(IVB) end_if;
                if l1 <> null() then obj_empty := FALSE end_if;
                new_args := append(new_args, l1);
	     else
                obj_empty := FALSE;
	        new_args := append(new_args, l);
	     end_if;
             break;

          otherwise
             error("Unknown Mode: Curve or List expected.");

         end_case;

     else

        new_args := append(new_args, l)

     end_if

   end_for;

   if not obj_def then
      error("No graphical object defined.\n[Mode = Curve, ...] or [Mode = List, ...] expected.")
   end_if;

   if (t:=select(new_args, has, AxesOrigin)) <> [] then
      t := t[1];
      if type(t) = "_equal" and 
         type((t:=float(op(t, 2)))) = DOM_LIST and nops(t) = 2 then 
         if type(t[1]) = DOM_FLOAT then
	    IVB[1] := min(IVB[1], t[1]); IVB[2] := max(IVB[2], t[1]);
	 end_if;
	 if type(t[2]) = DOM_FLOAT then
	    IVB[3] := min(IVB[3], t[2]); IVB[4] := max(IVB[4], t[2]);  
	 end_if
      end_if
   end_if;

   // Ist der minimale und maximale Y-Wert gleich, so wird
   // die ViewingBox vergroessert, da sonst keine Y-Achse
   // gezeichnet wird. Dies kann z.B. vorkommen, wenn die
   // zu zeichnende Funktion konstant ist, also 
   // plotfunc2d(c), c eine reelle Zahl. Ist der Y-Wert
   // gleich 0, so werden die Default-Y-Werte genommen
   // (-5 und 5), ansonsten +/- 5/3 * |y|, wobei 5/3
   // ein beliebig gewaehlter Wert ist.

   if iszero(IVB[4]-IVB[3]) then
      if iszero(IVB[3]) then
         IVB[3] := op(plotlib::Options[DefaultYRange], 1); 
         IVB[4] := op(plotlib::Options[DefaultYRange], 2); 
      else
         IVB[3] := -5/3 * abs(IVB[3]);
         IVB[4] :=  5/3 * abs(IVB[4]);
      end_if
   end_if;

   if (t:=select(new_args, has, Ticks)) = [] then
      ticks := plotlib::check_ticks(Automatic, IVB, ticks, LogXY);
      new_args := [ ticks ] . new_args
   else
      ticks := plotlib::check_ticks(op(t[1],2), IVB, ticks, LogXY);
      new_args := subs(new_args, op(t)=ticks)
   end_if;
   n := 10^5;
   if iszero(IVB[4]-IVB[3]) or iszero(IVB[2]-IVB[1]) 
      then t := 1/n;
      else t := abs(IVB[2]-IVB[1]) / abs(IVB[4]-IVB[3]);
   end_if;
   if t=undefined or t < 1/n or t > n then
      t := "Ratio between x- and y-range is greater than ".expr2text(n)."\n";
      t := t."x-range: ".expr2text(IVB[1])." .. ".expr2text(IVB[2])."\n";
      t := t."y-range: ".expr2text(IVB[3])." .. ".expr2text(IVB[4])."\n";
      //t := t."Use option 'ViewingBox', or specify a second range\n";
      if stdlib::ShowPlotWarning = TRUE then warning(t) end_if
   end_if;

   if gridlines <> FALSE then
      gridlines := plotlib::gridlines(gridlines, gridcolor, 
                                      gridstyle, gridwidth, 
				      op(ticks, 2), IVB, LogXY);
      new_args := new_args . [ gridlines ] 
   end_if;
/*
   if select(new_args, has, Axes) = [] then
      // Default: Axes = Corner
      new_args := [Axes=Corner] . new_args
   end_if;
*/
   return(op(new_args))

end_proc:

