
plotlib::testfloat := proc( 
                    fx,   /* DOM_PROC   */
                    fy,   /* DOM_PROC   */ 
		    a,    /* Type::Real */
		    b,    /* Type::Real */
		    l,    /* DOM_LIST   */
		    steps /* DOM_INT    */ 
		    )
   local ivc, ivx, ivy, mess, xmin, xmax, ymin, ymax;
begin
   a := float(a); b := float(b);
   [ivx, ivc, xmin, xmax] := numeric::testfloat(fx, a, b, steps);
   if ivx = [] then
      // fx(t) liefert im Interval [a, b] nur nicht reelle Werte
      if l <> [] then
         mess := expr2text(l[2][1]) . " returns no real numbers in\n";
         mess := mess . expr2text([a, b]);
         userinfo(1, mess);
      end_if;
      return( [ [], xmin, xmax, ymin, ymax ] );
   end_if;
   [ivy, ivc, ymin, ymax] := numeric::testfloat(fy, a, b, steps);
   if ivy = [] then
      // fy(t) liefert im Interval [a, b] nur nicht reelle Werte
      if l <> [] then
         mess := expr2text(l[2][2]) . " returns no real numbers in\n";
         mess := mess . expr2text([a, b]);
         userinfo(1, mess);
      end_if;
      return( [ [], xmin, xmax, ymin, ymax ] );
   else
      return( [ plotlib::combine_intervals(ivx, ivy, a, b, l),
                xmin, xmax, ymin, ymax ] );
   end_if;
end_proc:

plotlib::combine_intervals := proc(ivx, ivy, a, b, l)
   local cx, cy, dx, dy, iv, mess, ny, x, y;
begin
   // l = [ Mode = Curve, [fx(t), fy(t)], t=[a,b], ... ]
   // oder l = []

   // ivx und ivy sind von der Form [[a_1,b_1],...,[a_n,b_n]]
   // und Teilintervalle von [a, b]. ivx enthaelt die Teil-
   // intervalle in denen fx(t) in [a,b] reellwertig ist,
   // ivy enthaelt die Teilintervalle in denen fy(t) in [a,b] 
   // reellwertig ist

   if nops(ivx) = 1 and nops(ivy) = 1 then
      cx := max(ivx[1][1], ivy[1][1]);
      dx := min(ivx[1][2], ivy[1][2]);
      if cx > dx then
         if l <> [] then
            mess := expr2text(l[2]) . "\nNo common interval where both";
 	    mess := mess . " functions return real numbers";
	    userinfo(1, mess);
	 end_if;
	 return( [] )
      else
         if (cx > a or dx < b) and l <> [] then
	    mess := expr2text(l[2]) . "\nCommon interval where both";
	    mess := mess . " functions return real numbers is\n" . expr2text([cx, dx]);
	    userinfo(mess);
	 end_if;
         return( [[cx, dx]] );
      end_if;
   else // nops(ivx) >= 1 and nops(ivy) >= 1

      // Die Intervallisten ivx und ivy werden kombiniert, sodass eine 
      // Intervalliste ir = [[a_1, b_1],...,[a_n, b_n]] entsteht und 
      // es fuer alle i aus 1..n und alle t aus [a_i, b_i] gilt: 
      // fx(t) ist reell und fy(t) ist reell.
      
      iv := []; y := 1; ny := nops(ivy);
      for x from 1 to nops(ivx) do
          cx := ivx[x][1]; dx := ivx[x][2];
          repeat
	    cy := ivy[y][1]; dy := ivy[y][2];
	    if cx > dy then 
	       y := y+1;
	    elif cy > dx then
	       break; // repeat-Schleife verlassen
	    else
	       iv := append(iv, [max(cx, cy), min(dx, dy)]);
	       if dx >= dy then
	          y := y+1
	       else
	           break
               end_if;
	    end_if
	  until y > ny end_repeat;
 	  if y > ny then 
	     break; // for-Schleife verlassen
	  end_if
       end_for;
       if iv = [] then
          if l <> [] then
             mess := expr2text(l[2]) . "\nNo common interval where both ";
 	     mess := mess . "functions return real numbers";
	     userinfo(1, mess);
	  end_if;
	  return( [] )
       else
          if l <> [] then
             mess := expr2text(l[2]) . "\nCommon intervals where both ";
	     mess := mess . "functions return real numbers are " . expr2text(iv);
	     userinfo(1, mess);
	  end_if;
	  return( iv )
       end_if
   end_if;
end_proc: