
plotlib::check_ticks := proc(Rhs, vbox : DOM_LIST, default_ticks : DOM_LIST, LogXY)
begin
  plotlib::check_ticks_gridlines(Rhs, vbox, default_ticks, "Ticks", LogXY)
end_proc:

plotlib::check_ticks_gridlines := proc(Rhs, vbox, default_ticks, optionname, LogXY)

   local d, i, l;

begin

   d := nops(default_ticks); // 2D or 3D

   if Rhs = None then
      Rhs := [0 $ d]
   elif Rhs = Automatic then
      Rhs := default_ticks
   elif type(Rhs) = DOM_INT and Rhs >= 0 then
      // Ticks = n
      Rhs := [Rhs $ d]
   elif type(Rhs) <> DOM_LIST or nops(Rhs) <> d then
      error("Illegal ".optionname.": ".expr2text(Rhs));
   end_if;

   l := [];
   for i from 1 to d do
       l := l . [plotlib::check_tick(Rhs[i], vbox[2*i-1], vbox[2*i], 
                                     default_ticks[i], optionname, 
				     LogXY[i])]
   end_for;

   return( Ticks = l )

end_proc:

plotlib::check_tick := proc(l, a, b, default_ticks, optionname, LogScale)
   local c, d, t;
begin
   // a: DOM_FLOAT or XMin or YMin or ZMin
   // b: DOM_FLOAT or XMax or YMax or ZMax
   if l = Automatic then
      l := default_ticks
   end_if;
   
   a := float(a); b := float(b);

   if type(l) = "_equal" then
      if op(l, 1) = Steps then
         l := op(l, 2);
	 if type((t:=float(l))) = DOM_FLOAT and l <> 0.0 then
	    // Steps = dx
	    return( [a..b, t, 1] )
	 elif type(l) = DOM_LIST and 
	      type((t:=float(l[1]))) = DOM_FLOAT and t <> 0.0 and 
	      type(l[2]) = DOM_INT and l[2] >= 0 then
	      // Steps = [dx, n]
              return( [a..b, t, l[2]+1] )
	 end_if;
      end_if;
      error("Illegal ".optionname.": ".expr2text(l))
   elif type(l) = DOM_LIST then
      if nops(l) > 0 then
         if type(l[1]) <> "_range" then
            for c from 1 to nops(l) do
                if type(l[c]) ="_equal" then
	           l[c] := float(op(l[c],1)) = op(l[c],2)
	        else
	           l[c] := float(l[c])
	        end_if
            end_for
	 else
	    // [a..b, c, d]
	    if nops(l) <> 3 or
	       type(float(l[2])) <> DOM_FLOAT or
	       type(l[3]) <> DOM_INT or l[3] < 0 then
	       error()
	    end_if
         end_if
      end_if
   elif type(l) <> DOM_INT then
      error("Illegal ".optionname.": ".expr2text(l))
   end_if;

   if type(a) <> DOM_FLOAT or type(b) <> DOM_FLOAT then
      return(l)
   end_if;

   if LogScale then
      a := exp(a); b := exp(b); 
   else
      // Smooth endpoints
      d := abs(b-a)/10^4; a := a-d; b:=b+d;
   end_if;

   if type(l) = DOM_INT then
      if LogScale then
         return(l)
      else
         return( [a..b, plotlib::calc_ticks(a, b, l), 1] ) 
      end_if
   elif type(l) = DOM_LIST then
      if nops(l) > 0 and type((t:=l[1])) = "_range" then
         // [ al..bl, d, s ]
         if nops(l) <> 3 then
            error("Illegal ".optionname.": ".expr2text(l))
         end_if; 
         c := float(op(t, 1)); d := float(op(t, 2));
         if type(c) = DOM_FLOAT then a := min(a, c) end_if;
         if type(d) = DOM_FLOAT then b := max(b, d) end_if;
	 if LogScale then
	    d := l[2]/l[3];
	    a := trunc(a/d);
	    if a < 0 then a := a*d else a := a*(d+1) end_if;
	    l := [];
	    while a <= b do
	       a := a + d; l := append(l, a)
	    end_while;
	    return( l )
	 else
            return( [a..b, l[2], l[3]] )
	 end_if
      else
         // [ k_1, ..., k_n ]
         // k_i -> r_i oder k_i -> r_i = s_i
         return( l )
         // if a < min(op(l)) then l1 := append(l1, a) end_if;
         // if b > max(op(l)) then l1 := append(l1, b) end_if;
      end_if
   else
      error("Illegal ".optionname.": ".expr2text(l))
   end_if;
     
end_proc:

plotlib::calc_ticks := proc(a : Type::Real, 
                            b : Type::Real, 
                            n : Type::NonNegInt )
    local d, eps, l, s;
begin
    a := float(a); b := float(b);
    if n = 0 then
       // Return a value greater than max(|a|,|b|)
       // because [a..b, n, 1] , n=0, is not possible.
       // This workaround forces to draw a axes line
       // from a to b without ticks.
       return( max(abs(a), abs(b))+1 )
    end_if;
    
    if abs(b-a) < 10^(-DIGITS) then
       return( 1.0 )
    end_if;
    l := b - a;
    if n = 1 then
       return( l )
    end_if;
 
    eps := 0.005;
    d := l / (n-1);
    s := 1.0;
    if d > 1.0 + eps then
	   while d > 1.0 + eps do
	    d := d / 10.0;
	    s := s * 10.0;
       end_while
    elif d < 0.1 - eps then
	   while d < 0.1 - eps do
	    d := d * 10.0;
	    s := s / 10.0;
	   end_while
    end_if;
    // tick marks may be at 0.1, .125, .2, .25, .5 and 1.
    if d < 0.125 - eps then
	   d := 0.1 * s;
    elif d < 0.2 - eps then
	   d := 0.125 * s;
    elif d < 0.25 - eps then
	   d := 0.2 * s;
    elif d < 0.5 - eps then
	   d := 0.25 * s;
    elif d < 1.0 - eps then
	   d := 0.5 * s;
    else
	   d := s;
    end_if;
    return( d );
end_proc:
