/* 
   This procedure checks the option ViewingBox
   It returns a list of 4 values [xa, xb, ya, yb]
   where xa < xb and ya < yb. xa, xb, ya, yb are
   real numbers (DOM_FLOAT) or xa = XMin, xb = XMax
   or ya = YMin or yb = YMax.
*/

plotlib::check_vb := proc(t)
   // t must be ViewingBox = rhs
begin
   // ViewingBox = Automatic or
   // ViewingBox = [x, y]    or
   // ViewingBox = [xa, xb, ya, yb]
   // x: Automatic or xa..xb
   // y: Automatic or ya..yb
   // xa: float(xa) = DOM_FLOAT or XMin
   // xb: float(xb) = DOM_FLOAT or XMax
   // ya: float(ya) = DOM_FLOAT or YMin
   // yb: float(yb) = DOM_FLOAT or YMax
   if t = Automatic then
      t := [XMin, XMax, YMin, YMax];
   elif type(t) = DOM_LIST then
      if nops(t) = 2 then
	 if type(t[1]) = "_range" then
	    t := [ op(t[1], 1), op(t[1], 2), t[2] ]
	 elif t[1] = Automatic then
	    t := [ XMin, XMax, t[2] ]
	 end_if;
         if type(t[3]) = "_range" then
	    t := [ t[1], t[2], op(t[3], 1), op(t[3], 2) ]
         elif t[3] = Automatic then
	    t := [ t[1], t[2], YMin, YMax ]
	 else
	    error("Illegal ViewingBox: ".expr2text(t[3]))
	 end_if;
      end_if;
   end_if;
   return( float(t) )
end_proc:
