
plotlib::clip2d := proc(mode) 
begin
   if mode = Curve then
      plotlib::clip2d_Curve(args(2..args(0)))
   elif mode = List then
      plotlib::clip2d_List(args(2..args(0)))
   else
      error("First argument must be Curve or List")
   end_if
end_proc:

plotlib::clip2d_Curve := proc(l             : DOM_LIST, 
                              vbox          : DOM_LIST, 
                              obj_nr        : DOM_INT,
			      clip          : DOM_BOOL,
			      checkr        : DOM_BOOL,
			      checkd        : DOM_BOOL,
			      LogXY         : DOM_LIST,
			      IntervalsOnly = FALSE,
			      setVB         = FALSE             
			     )

  local a, b, color, fx, fxo, fyo, fy, grid, iv, j, k,
        l1, maxx, maxy, minx, miny, r, s, smooth, t,
	xa, xb, ya, yb, XMin, XMax, YMin, YMax, IVB;
begin
 
 minx := vbox[1]; maxx := vbox[3]; 
 miny := vbox[2]; maxy := vbox[4];

 // [ Mode = Curve, [fx(t), fy(t)], t=[a,b], ... ] 

 grid := 100; smooth := 0; color := FALSE;

 // 2nd argument must be of the form [fx(t), fy(t)] 

 k := l[2];

 if not testtype(k, Type::ListOf(Type::AnyType,2,2)) then
    error("List of 2 expressions expected")
 end_if;
 fx := k[1]; fy := k[2];

 // 3rd argument must be of the form t = [a, b] 

 r := l[3];
 if not (testtype(r, "_equal") and
    testtype(float(op(r,2)), Type::ListOf(Type::Real,2,2))) then
    error("Specification of the form t = [a,b] expected (a, b real numbers)");
 end_if;
 t := op(r, 1); a := op(r, [2,1]); b := op(r, [2,2]);

 for j from 4 to nops(l) do
     k := l[j];
     if not testtype(k, "_equal") then
        error("Specification of the form option = value expected");
     end_if;
     case op(k, 1)
       of Color do
          color := TRUE;
          break;
       of Grid do
          // Grid = [n] or Grid = n
          grid := op(k, 2);
	  if not testtype(grid, Type::PosInt) then
            if not testtype(grid, Type::ListOf(Type::PosInt,1,1)) then
              error("Grid: positive integer expected")
            end_if;
            grid := grid[1];
	  end_if;
          break;
       of Smoothness do
          // Smoothness = [m] 
          smooth := op(k, 2);
          if not testtype(smooth, Type::ListOf(Type::PosInt,1,1)) then
             error("Smoothness: positive integer expected")
          end_if;
          smooth := smooth[1];
          break;
     end_case
  end_for;

  // Convert fx to an executable object
  if not contains({DOM_PROC, DOM_EXEC, DOM_FUNC_ENV,piecewise}, domtype(fx)) then 
     if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT}, domtype(fx)) then
        // misc::subsFreeVars nicht mehr vorhanden.
        // fx := subsop(proc() begin 1 end_proc, 4=misc::subsFreeVars(fx, t=hold(args(1))))
	if LogXY[1] then
           fx := fp::unapply(ln(fx), t);
	else
	   fx := fp::unapply(fx, t);
	end_if
      else
        error("Illegal argument: ".expr2text(fx))
      end_if
  else
      if domtype(fx) = piecewise then
         fx := fp::unapply(piecewise(op(fx)),t)
      elif checkd and stdlib::ShowPlotWarning = TRUE then
         warning("Discont = TRUE: For plotting with expressions only.")
      end_if;
      if LogXY[1] then
        fxo := fx; fx := fp::unapply(ln(hold(fxo)(t)), t);
     end_if
  end_if;
 
  // Convert fy to an executable object
  if not contains({DOM_PROC, DOM_EXEC, DOM_FUNC_ENV,piecewise}, domtype(fy)) then 
     if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT}, domtype(fy)) then
        // misc::subsFreeVars nicht mehr vorhanden.
        // fy := subsop(proc() begin 1 end_proc, 4=misc::subsFreeVars(fy, t=hold(args(1))))
	if LogXY[2] then
	   fy := fp::unapply(ln(fy), t);
	else
           fy := fp::unapply(fy, t);
	end_if
      else
        error("Illegal argument: ".expr2text(fy))
      end_if 
  else
      if domtype(fy) = piecewise then
         fy := fp::unapply(piecewise(op(fy)),t)
      elif checkd and stdlib::ShowPlotWarning = TRUE then
         warning("Discont = TRUE: For plotting with expressions only.")
      end_if;
      if LogXY[2] then
        fyo := fy; fy := fp::unapply(ln(hold(fyo)(t)), t);
     end_if
  end_if;
 
  IVB := [ -infinity, infinity, -infinity, infinity ];

  iv := [[a, b]]; 

  // Die Reihenfolge der folgenden Check-Funktionen ist einzuhalten.

  if checkr or (setVB and not clip) then
     // Berechne, sofern die plot2d-Option RealValuesOnly=TRUE 
     // gesetzt wurde, die Teilintervalle von [a,b], in denen 
     // fx(t) und fy(t) reellwertig sind, oder
     // benutze plotlib::testfloat um die interne ViewingBox
     // zu berechnen, falls setVB=TRUE ist. Ist nur setVB 
     // gesetzt, nicht aber checkr, checkd oder clip, dann
     // wird ein Fehler ausgeloest, wenn plotlib::testfloat
     // selbst ein Fehler liefert. Grund: Spaetestens in 
     // plot wuerde sonst ein Fehler erzeugt.
     if traperror((
          [ iv, XMin, XMax, YMin, YMax ] := 
	        plotlib::testfloat(fx, fy, a, b, l, grid*(smooth+1))
        )) = 0 then
        if not checkr and iv <> float([[a,b]]) then
           t := "Non-real values detected in\n".expr2text(l);
	   userinfo(1, t);
 	   t := "Non-real values detected (try option RealValuesOnly = TRUE)";
	   error(t)
	end_if;
        if iv = [] then
           return( null() )
        end_if;
	IVB := [ XMin, XMax, YMin, YMax ]
     else
        if checkr then
           userinfo(1, "RealValuesOnly check failed.")
	else
           t := "Non-real values detected in\n".expr2text(l);
	   userinfo(1, t);
 	   t := "Non-real values detected (try option RealValuesOnly = TRUE)";
	   error(t)
	end_if
     end_if
  end_if;

  if checkd then
     XMin := infinity; XMax := -infinity;
     YMin := infinity; YMax := -infinity;
     r := [];
     for j from 1 to nops(iv) do
         [s, xa, xb, ya, yb] := 
	     plotlib::testdiscont( fx, fy, t, iv[j][1], iv[j][2], 
	                           plotlib::Options[RATIO_YX], 
			           grid*(smooth+1), 
			           minx, maxx, miny, maxy) ;
         XMin := min(XMin, xa); XMax :=  max(XMax, xb);
	 YMin := min(YMin, ya); YMax :=  max(YMax, yb);
         r := r . s
     end_for;
     if XMin =  infinity then XMin := IVB[1] end_if;
     if XMax = -infinity then XMax := IVB[2] end_if; 
     if YMin =  infinity then YMin := IVB[3] end_if; 
     if YMax = -infinity then YMax := IVB[4] end_if; 
     IVB := [ XMin, XMax, YMin, YMax ];
     iv := r;
  end_if;

  if clip then
     XMin := infinity; XMax := -infinity;
     YMin := infinity; YMax := -infinity;
     r := [];
     for j from 1 to nops(iv) do
         [ s, xa, xb, ya, yb ] := 
	      plotlib::clip_2( fx, fy, iv[j][1], iv[j][2], 
	                       grid*(smooth+1), minx, maxx, miny, maxy);
	 XMin := min(XMin, xa); XMax := max(XMax, xb);
         YMin := min(YMin, ya); YMax := max(YMax, yb);
	 IVB := [ XMin, XMax, YMin, YMax ];
         r := r . s	                           
     end_for;
  else
     r := iv;
  end_if;

  if setVB then
     //if nops(r) = 0 then
     //   sysassign( plotlib::InternalViewingBox , 
     //              [ undefined, undefined, undefined, undefined ] )
     //else
        if IVB[1] = -infinity then IVB[1] := a end_if;
        if IVB[2] =  infinity then IVB[2] := b end_if; 
        sysassign( plotlib::InternalViewingBox , IVB )
     //end_if
  end_if;

  if nops(r) = 0 then 
     return( null() ) 
  end_if;

  if IntervalsOnly then return( r ) end_if; 

  if /* nops(r) > 1 and */ not color then
     l := append(l, Color = [Flat, RGB::ColorList[obj_nr]])
  end_if;

  l1 := null();

  for iv in r do
      l1 := l1, subsop(l, [3,2,1]=iv[1], [3,2,2]=iv[2]);
  end_for;

  return(l1);

end_proc:

plotlib::clip_1 := proc(f, xs, xe, grid, miny, maxy)
   local d, eps, mx, my, r, t, x, xl, y, YMin, YMax;
begin

   my := miny; r := []; d := float((xe-xs)/grid);
   xs := float(xs); xe := float(xe); xl := null();
   eps := 10^(-DIGITS+1); 
   YMin := infinity; YMax := -infinity;

   for x from xs to xe step d do
       y := float(f(x));
       if y > YMax then YMax := y end_if;
       if y < YMin then YMin := y end_if;
       if y > maxy then
          if type(xl) <> DOM_NULL then
	     t := plotlib::bisection(f, mx, my, x, y, maxy);
	     if abs(xl-t) > eps then
                r := append(r, [ xl, t ]);
	     end_if;
             xl := null()
          end_if
       elif y < miny then
          if type(xl) <> DOM_NULL then
	     t := plotlib::bisection(f, mx, my, x, y, miny);
             if abs(xl-t) > eps then
                r := append(r, [ xl, t ]);
	     end_if;
             xl := null()
          end_if
       elif type(xl) <> DOM_NULL then
          if my > maxy then
	     t := plotlib::bisection(f, mx, my, x, y, maxy);
	     if abs(xl-t) > eps then
                r := append(r, [ xl, t ]);
	     end_if;
             xl := null()
          elif my < miny then
	     t := plotlib::bisection(f, mx, my, x, y, miny);
	     if abs(xl-t) > eps then
                r := append(r, [ xl, t ]);
	     end_if;
             xl := null()
          end_if
       elif my > maxy then
          xl := plotlib::bisection(f, mx, my, x, y, maxy)
       elif my < miny then
          xl := plotlib::bisection(f, mx, my, x, y, miny)
       else
          xl := x
       end_if;
       mx := x; my := y;
  end_for;
  if type(xl) <> DOM_NULL and abs(xl-mx) > eps then
     r := append(r, [xl, mx])
  end_if;
  if miny <> -infinity then YMin := miny end_if;
  if maxy <>  infinity then YMax := maxy end_if;
  [ r, YMin, YMax ]
end_proc:

plotlib::clip_2 := proc(fx, fy, a, b, grid, minx, maxx, miny, maxy)
  local iv, l, r, s, XMin, XMax, YMin, YMax, ya, yb;
begin
  [l, XMin, XMax] := plotlib::clip_1(fx, a, b, grid, minx, maxx);
  if l = [] then 
     return([ l, minx, maxx, miny, maxy ]) 
  end_if;
  r := []; YMin := infinity; YMax := -infinity;
  for iv in l do
      [s, ya, yb ] := plotlib::clip_1(fy, iv[1], iv[2], grid, miny, maxy);
      YMin := min(YMin, ya); YMax := max(YMax, yb);
      r := r . s
  end_for;
  return([ r, XMin, XMax, YMin, YMax ])
 end_proc:

plotlib::bisection := proc(f, x0, y0, x1, y1, bound)
  local c, eps;
begin
   if y0 < bound then
      c := x0; x0 := x1; x1 := c;
   end_if; 
   eps := 10^(-DIGITS);
   // y0 = f(x0) > bound, y1 = f(x1) < bound
   while abs(x1-x0) > eps do
      c := (x0+x1)/2;
      if float(f(c)) > bound then
         x0 := c;
      else
         x1 := c;
      end_if
   end_while;
   x0;
end_proc:

plotlib::clip2d_List := proc(l             : DOM_LIST,
                             vbox          : DOM_LIST,
			     IntervalsOnly = FALSE,
			     clip          = TRUE,
			     setVB         = FALSE
			    )

  local col, i, ind, maxx, maxy, minx, miny, 
        n, new_list, opts, p, px, py, q, r, plist;

begin

  // [ Mode = List, [... ] ] 

  if domtype(l[2]) <> DOM_LIST then
     error("Mode = List: Second argument must be a list.")
  end_if;

  new_list := l[2]; ind := 1;

  if clip then

  for p in l[2] do

      opts := null(); col := null();

      if domtype(p) = DOM_POINT then
         if plotlib::in_vbox(p, vbox) then
	    new_list[ind] := p;
            ind := ind + 1
         end_if;

      elif domtype(p) = DOM_POLYGON then
         // polygon(p1,...,pn, color, closed, filled)
         // color  : list of 3 reals
         // closed : TRUE or FALSE
         // filled : TRUE or FALSE
         n := nops(p);
         plist := [op(p,q) $ q = 1..n-3];
         if op(p, n-2) <> NIL then
            // Color = ...
            col  := Color=op(p, n-2);
            opts := col;
         end_if;
         if op(p, n-1) then
            // Closed = TRUE  
            // Die Option Closed=TRUE wird umgesetzt,
            // indem der Startpunkt als zusaetzlicher
            // Endpunkt in das Polygon eingefuegt wird.
            plist := append(plist, op(p, 1));
            opts := opts, Closed=TRUE
         end_if:
         if op(p, n) then
             // Filled = TRUE
             if n > 6 then
                error("Only triangles can be filled.")
             end_if;

             if nops(plist) = n-3 then
                // Option Closed=TRUE nicht gesetzt,
                // d.h. Startpunkt wurde nicht ergaenzt.
                // Muss beim Clipping ergaenzt werden, da
                // sonst falsches Resultat erzielt wird.
                plist := append(plist, op(p, 1));
             end_if;

             r := plotlib::clip2d_filled_polygon(plist, vbox);

             if r <> [] then  
                p := [polygon(op(r), opts, Filled=TRUE)];
                p := (plotlib::polygonplot)::PlotScene(p); 
                // p = [Mode=List, [...]]              
                // Achtung: p[2] kann eine Sequenz von Polygonen
                // sein. Hier wird davon ausgegangen, das intern
                // geflattet wird!
                new_list[ind] := op(p[2]); ind := ind + nops(p[2])
             end_if
          else
             // Filled = FALSE
             r := plotlib::clip2d_polygon(plist, vbox);
             if r <> [] then
                // Hier nicht opts verwenden, da opts u.U.
                // den Eintrag Closed=TRUE enthaelt. Dies
                // fuehrt zu einem Fehler, da ein Polygon
                // hier mehr als 3 Punkte enthalten kann.
                // Die Option Closed=TRUE wurde umgesetzt,
                // indem der Startpunkt als zusaetzlicher
                // Endpunkt in das Polygon eingefuegt wurde.
                p := polygon(op(r[i]), col) $ i=1..nops(r);
                // Achtung: p kann eine Sequenz von Polygonen
                // sein. Hier wird davon ausgegangen, das intern
                // geflattet wird!
                new_list[ind] := p; ind := ind + nops(r)
             end_if;
          end_if;

     else

        error("point or polygon expected.")

     end_if

  end_for;

  new_list := [ new_list[i] $ i = 1..ind-1 ];

  end_if; // if clip then

  if new_list = [] then
     if setVB then
        sysassign( plotlib::InternalViewingBox , 
        //           [ undefined, undefined, undefined, undefined ] )
                     [ vbox[1], vbox[3], vbox[2], vbox[4] ] )
     end_if;
     return( null() )
  end_if;
  
  if setVB then
     if clip then
        [minx, miny, maxx, maxy] := vbox;
	if minx = -infinity then minx :=  infinity end_if;
	if miny = -infinity then miny :=  infinity end_if;
        if maxx =  infinity then maxx := -infinity end_if;
	if maxy =  infinity then maxy := -infinity end_if;
     else
        [minx, miny, maxx, maxy] := 
	        [infinity, infinity, -infinity, -infinity]
     end_if;
  
     if not clip or has(vbox, {infinity, -infinity}) then
        for p in new_list do
         if domtype(p) = DOM_POINT then
	    px := op(p, 1); py := op(p, 2);
            if px < minx then 
	       minx := px 
	    end_if;
	    if px > maxx then 
	       maxx := px 
	    end_if;
	    if py < miny then 
	       miny := py
 	    end_if;
	    if py > maxy then 
	       maxy := py 
	    end_if
	 else // DOM_POLYGON
	    for q from 1 to nops(p)-3 do
                px := op(p, [q, 1]); py := op(p, [q, 2]);
                if px < minx then 
	           minx := px
 	        end_if;
	        if px > maxx then 
	           maxx := px 
	        end_if;
	        if py < miny then 
	           miny := py
 	        end_if;
	        if py > maxy then 
                   maxy := py 
	        end_if;
             end_for
	  end_if
         end_for
      end_if;
      sysassign( plotlib::InternalViewingBox , 
                 [ minx, maxx, miny, maxy ]    )
  end_if;

  if IntervalsOnly then
     return( new_list )
  else
     return( [Mode=List, new_list, l[i] $ i=3..nops(l)] )
  end_if

end_proc:

//
// point_list - [ point(x_1,y_1), ... , point(x_n, y_n) ] , n >= 1
// vbox       - [ minx, miny, maxx, maxy ] 
//
// Rueckgabe: [] - Polygon liegt vollstaendig ausserhalb der ViewingBox
//            [plist_1, ..., plist_m] , m > 0, plist_i sind Listen der
//                 Form [point(x_i_1, y_i_1),...,point(x_i_ni, y_i_ni)]
//                 mit ni >= 1. Die einzelnen Listen plist_i bilden die
//                 Polygone, die innerhalb der ViewingBox liegen.
// 

plotlib::clip2d_polygon := proc(point_list : DOM_LIST, 
                                vbox       : DOM_LIST)

   local i, j, n, p, polygon_list, s; 

begin

   n := nops(point_list);

   if n = 1 then
      if plotlib::in_vbox(point_list[1], vbox) then
         return([point_list])
      else
         return([])
      end_if
   end_if;

   i := 1; polygon_list := null();

   while i <= n and not plotlib::in_vbox(point_list[i], vbox) do
      if i > 1 then
         p := [plotlib::intersect_vbox( point_list[i-1], point_list[i], vbox, TRUE)];
         if p <> [] then 
            polygon_list := polygon_list, p 
         end_if;
      end_if;
      i := i+1;
   end_while;

   if i > n then
      return( [polygon_list] );
   elif i = 1 then
      p := null(); s := 1;
   else // 1 < i <= n 
      p:=plotlib::intersect_vbox(point_list[i-1], point_list[i], vbox, FALSE);
      if p = point_list[i] then
         p := null()
      end_if;
      s := i
   end_if;

   i := i+1;
   while i <= n and plotlib::in_vbox(point_list[i], vbox) do
      i := i+1;
   end_while;

   if i > n then
      // Polygon point_list[s],...,point_list[n]
      // liegt innerhalb der ViewingBox
      return( [polygon_list, [p, point_list[j] $ j =s..n]] );
   else
      polygon_list := 
         polygon_list, 
          [ p, point_list[j] $ j=s..i-1,
            plotlib::intersect_vbox(point_list[i], point_list[i-1], vbox, FALSE) 
          ];
      if i = n then
         return( [polygon_list] );
      else
         return( [polygon_list] . 
                 plotlib::clip2d_polygon( [point_list[j] $ j=i..n], vbox)
         );
      end_if;
   end_if;
end_proc: 

//
// p    - point(x, y)
// vbox - [minx, miny, maxx, maxy]
//

plotlib::in_vbox := proc(p, vbox)
   local eps;
begin
   eps := 10^(-DIGITS+1);
   if op(p, 1) >= vbox[1]-eps and op(p, 1) <= vbox[3]+eps and
      op(p, 2) >= vbox[2]-eps and op(p, 2) <= vbox[4]+eps then
      return(TRUE)
   else
      return(FALSE)
   end_if
end_proc:

//
// p_out       - point(x1, y1) 
// p_in        - point(x2, y2) 
// vbox        - [minx, miny, maxx, maxy]
// both_points - TRUE : Ausgabe beider Punkte
//             - FALSE : Ausgabe des Schnittpunktes
//
// Ist both_points = FALSE, dann muss p_out ausserhalb 
// und p_in innerhalb der ViewingBox vbox liegen.
// Ist both_points = TRUE, dann liegen p_out und p_in
// beide ausserhalb der ViewingBox vbox
//
// Rueckgabe:
// 1) Ein Punkt point(x, y) oder 
// 2) Sequenz zweier Punkte point(x3, y3), point(x4,y 4)
//

plotlib::intersect_vbox := proc(p_out, p_in, vbox, both_points : DOM_BOOL)
   local eps, lb, m, p, p2, r, t, 
         ub, x1, x2, y1, y2;
begin 

   x1 := op(p_out, 1); y1 := op(p_out, 2);
   x2 := op(p_in, 1);  y2 := op(p_in, 2);
   eps := 10^(-DIGITS+1); // x := genident();

   if abs(x1-x2)<eps then
      // Beiden Punkte definieren eine Senkrechte
      if x1 >= vbox[1] and x1 <= vbox[3] then
      // Die Senkrechte liegt bzgl. der x-Koordinate in vbox
         if both_points and
            ((y1 < vbox[2] and y2 < vbox[2]) or 
             (y1 > vbox[4] and y2 > vbox[4])) then
            return( null() );
         end_if;
         p := point(x1, min(max(y1, vbox[2]), vbox[4]));
         if both_points then
           p := p, point(x2, min(max(y2, vbox[2]), vbox[4]))
         end_if;
         return( p )
      else
         return( null() )
      end_if
   elif abs(y1-y2)<eps then
      // Beiden Punkte definieren eine Waagerechte
      if y1 >= vbox[2] and y1 <= vbox[4] then
         // Die Waagerechte liegt bzgl. der y-Koordinate in vbox
         if both_points and
            ((x1 < vbox[1] and x2 < vbox[1]) or 
             (x1 > vbox[3] and x2 > vbox[3])) then
            return(null())
         end_if;
         p := point(min(max(x1, vbox[1]), vbox[3]), y1);
         if both_points then
            p := p, point(min(max(x2, vbox[1]), vbox[3]), y2)
         end_if;
         return( p )
      else
         return( null() )
      end_if
  end_if;

  m := (y2-y1)/(x2-x1); t := y1-m*x1;

/*
 * solve ist *teuer* (und hier voelliger Overkill)
  f := m*x+t;

  r := solve({f >= vbox[2], f <= vbox[4]},x);

  if r = {} then return(null()) end_if;

  // Bisheriges solve lieferte r = {lb <= x , x <= ub} 
  // lb := max(op(r, [1,1,1]), vbox[1]); 
  // ub := min(op(r, [1,2,2]), vbox[3]);

  // Aktuelle Version liefert Dom::Interval
  lb := max(op(r, [1,1,1]), vbox[1]); 
  ub := min(op(r, [1,1,3]), vbox[3]);

 */

/* m <> 0 ist gesichert. */

  r := (-1/m*(- vbox[2] + t), 1/m*(vbox[4] - t));

  lb := max(min(r), vbox[1]);
  ub := min(max(r), vbox[3]);

  if lb > vbox[3] then
     return(null())
  end_if;

  if x1 < lb then
     p := point(lb, m*lb+t)
  elif x1 > ub then
     p := point(ub, m*ub+t)
  else
     p := point(x1,y1)
  end_if;

  if both_points then
     if x2 < lb then
        p2 := point(lb, m*lb+t)
     elif x2 > ub then
        p2 := point(ub, m*ub+t)
     else
        p2 := point(x2,y2)
     end_if;
     if plotlib::in_vbox(p, vbox) and plotlib::in_vbox(p2, vbox) then
        if p <> p2 then
           p := p, p2
        end_if
     else
        p := null()
     end_if
  end_if;

  return( p ); 

end_proc:


//
// point_list - [ point(x_1,y_1), ... , point(x_4, y_4) ] 
// vbox       - [ minx, miny, maxx, maxy ] 
//
// 
// Es muss gelten: x_1 = x_4 und y_1 = y_4, d.h. es
// liegt ein geschlossener Polygonzug vor, der ein
// Dreieck bildet. Dreieck kann entartet sein, d.h.
// auch eine Strecke bilden.
//

plotlib::clip2d_filled_polygon := proc(point_list : DOM_LIST, 
                                       vbox       : DOM_LIST)

   local m, n, p, p1, p2, polygon_list;

begin

   polygon_list := plotlib::clip2d_polygon(point_list, vbox);
   
   n := nops(polygon_list);
   case n
     of 0 do
        return( null() ) 
     of 1 do
        p := [op(polygon_list[1])]; 
        break;
     of 2 do
     of 3 do
        m  := nops(polygon_list[1]);
        p1 := polygon_list[1][m];
        p2 := polygon_list[2][1];
        p  := [ op(polygon_list[1]), plotlib::add_edge(p1, p2, vbox), 
                op(polygon_list[2]) ];

        if n = 2 then break end_if;
        
        m  := nops(polygon_list[2]);
        p1 := polygon_list[2][m];
        p2 := polygon_list[3][1];
        p  := append(p, plotlib::add_edge(p1, p2, vbox), op(polygon_list[3]));

   end_case;

   m  := nops(polygon_list[n]);
   p1 := polygon_list[1][1];
   p2 := polygon_list[n][m];
   if p1 <> p2 then
      p := append(p, plotlib::add_edge(p1, p2, vbox), p1);
   end_if;

   return( p );

end_proc:  

plotlib::add_edge := proc(p1, p2, vbox)
   local eps, p, x1, x2, y1, y2;
begin

   x1 := op(p1, 1); y1 := op(p1, 2);
   x2 := op(p2, 1); y2 := op(p2, 2);
   eps := 10^(-DIGITS+1);

   if abs(x1-x2)>eps and abs(y1-y2)>eps then
      if x1 = vbox[1] then
         if y2 < y1 then
            p := point(vbox[1], vbox[2])
         else
            p := point(vbox[1], vbox[4])
         end_if
      elif x1 = vbox[3] then
         if y2 < y1 then
            p := point(vbox[3], vbox[2])
         else
            p := point(vbox[3], vbox[4])
         end_if
      else
         p := plotlib::add_edge(p2, p1, vbox);
      end_if
   else
      p := null()
   end_if;

   return( p );

end_proc:

plotlib::testdiscont := proc(fx, fy, t, a, b, ratio, grid, minx, maxx, miny, maxy)
  local iv, l, r, s, xmin, xmax, ya, yb, ymin, ymax;
begin
  [l, xmin, xmax] := numeric::get_intervals(fx, t, a, b, 
                                            ratio, grid, a, b);
  ymin := infinity; ymax := -infinity;
  if l = [] then 
     return( [ l, xmin, xmax, ymin, ymax ] ) 
  elif l = FAIL then
     error("Automatic range detection failed. Please use option ViewingBox.")
  end_if;
  r := []; 
  for iv in l do
      [s, ya, yb] := numeric::get_intervals(fy, t, iv[1], iv[2], 
                                            ratio, grid, miny, maxy);
      if s = FAIL then
         error("Automatic range detection failed. Please use option ViewingBox.")
      end_if;
      ymin := min(ya, ymin); ymax := max(yb, ymax);
      r := r . s
  end_for;
  return( [ r, xmin, xmax, ymin, ymax ] )
 end_proc:

plotlib::IVB := proc( IVB )
begin
  if plotlib::InternalViewingBox[1] <> -infinity then
     IVB[1] := min(plotlib::InternalViewingBox[1], IVB[1])
  end_if;
  if plotlib::InternalViewingBox[2] <> infinity then
    IVB[2] := max(plotlib::InternalViewingBox[2], IVB[2])
  end_if;
  if plotlib::InternalViewingBox[3] <> -infinity then
     IVB[3] := min(plotlib::InternalViewingBox[3], IVB[3])
  end_if;
  if plotlib::InternalViewingBox[4] <> infinity then
     IVB[4] := max(plotlib::InternalViewingBox[4], IVB[4])
  end_if;
  return(IVB)
end_proc:
