
/*

Zeichne Gitterlinien 

1) Vertikale Linien von xmin bis xmax 
   a) Ist xgrid eine reelle Zahl > 0, dann werden vertikale 
      Linien im Abstand xgrid gezeichnet. 
   b) Ist xgrid eine Liste, dann werden Linien von
      (xmin, xgrid[i]) bis (xmax, xgrid[i]), i = 1..nops(xgrid)
   c) Ist xgrid = 0 oder xgrid = [], so werden keine Linien
      gezeichnet.
2) Horizontale Linien von ymin bis ymax
   a) Ist ygrid eine reelle Zahl > 0, dann werden horizontale
      Linien im Abstand ygrid gezeichnet.
   b) Ist ygrid eine Liste, dann werden Linien von
      (ygrid[i], ymin) bis (ygrid[i], ymax), i = 1..nops(ygrid)
   c) Ist ygrid = 0 oder ygrid = [], so werden keine Linien
      gezeichnet.

*/
 
plotlib::calc_gridlines := proc(xmin, xmax, ymin, ymax, 
                           xgrid, ygrid, 
                           gridcolor, gridstyle, gridwidth ) 

   local xminf, xmaxf, yminf, ymaxf, ix, iy, 
         xLines, yLines, xList, yList, ixmax, iymax;
begin 
   xminf:=float(xmin): xmaxf:=float(xmax):
   yminf:=float(ymin): ymaxf:=float(ymax): 

   if type(xgrid) = DOM_LIST then
      xList := xgrid;
   else
      if xgrid = 0 then
         xList := []
      else
         ixmax:=ceil((xmaxf-xminf)/xgrid);
         xList:=[xminf+ix*xgrid $ ix=0..ixmax];
      end_if
   end_if;

   if type(ygrid) = DOM_LIST then
      yList := ygrid;
   else
      if ygrid = 0 then
         yList := []
      else
         iymax:=ceil((ymaxf-yminf)/ygrid);
         yList:=[yminf+iy*ygrid $ iy=0..iymax];
      end_if
   end_if;
      
   if xList = [] and yList = [] then
      return( null() )
   end_if;

   yLines:=polygon(point(xList[iy],yminf),
                   point(xList[iy],ymaxf)) $ iy=1..nops(xList):
   xLines:=polygon(point(xminf,yList[ix]),
                   point(xmaxf,yList[ix])) $ ix=1..nops(yList): 
   return( [Mode=GridLines, [xLines,yLines],
            LineStyle=gridstyle, LineWidth=gridwidth,
            Color = [Flat, gridcolor] ] );
end_proc:


plotlib::gridlines := proc(Rhs, gridcolor, gridstyle, gridwidth, ticks, IVB, LogXY)
                           
   local xgrid, xmin, xmax, ygrid, ymin, ymax;

begin
   // ticks: [x, y]
   // x,y  : [a..b, d, n] or [a1, ..., aN]
   if Rhs = None then
      return( null() )
   elif Rhs <> Automatic then
      ticks := plotlib::check_gridlines(Rhs, IVB, [5,5], LogXY)
   end_if;
   [xgrid, xmin, xmax] := plotlib::convert_ticks(ticks[1], IVB[1], IVB[2], LogXY[1]);
   [ygrid, ymin, ymax] := plotlib::convert_ticks(ticks[2], IVB[3], IVB[4], LogXY[2]);
   return(plotlib::calc_gridlines(xmin, xmax, ymin, ymax, xgrid, ygrid,
	                          gridcolor, gridstyle, gridwidth))
end_proc:

plotlib::check_gridlines := proc(Rhs, vbox : DOM_LIST, default_ticks : DOM_LIST, LogXY)
begin
  op(plotlib::check_ticks_gridlines(Rhs, vbox, default_ticks, "GridLines", LogXY), 2)
end_proc:

plotlib::convert_ticks := proc(ticks : DOM_LIST, tmin, tmax, LogScale)
   local i, d, l, s, Ticks_delta, Ticks_min, Ticks_max, Ticks_step;
begin
   if LogScale then
      tmin := exp(tmin); tmax := exp(tmax);
   end_if;
   if type(ticks) = DOM_INT then
      return([ticks, tmin, tmax])
   elif type(ticks[1]) = "_range" then
      Ticks_delta := ticks[3];
      Ticks_step  := ticks[2];
      Ticks_min   := op(ticks[1], 1);
      Ticks_max   := op(ticks[1], 2);
      if Ticks_step > max(abs(Ticks_min), abs(Ticks_max)) then
         // [a..b, n, 1] with n > max(|a|,|b|) is a
	 // workaround for [a..b, n, 1] , n=0
         // This workaround forces to draw a axes line
         // from a to b without ticks. Therefore draw no
	 // gridlines
	 return( [0, Ticks_min, Ticks_max] )
      end_if;
/*
      if frac(Ticks_min/Ticks_step) <> 0.0 then
	 if Ticks_min < 0 then
            Ticks_min := (trunc(Ticks_min/Ticks_step)-1) * Ticks_step;
	 else
	    Ticks_min := trunc(Ticks_min/Ticks_step) * Ticks_step;
         end_if
      end_if;
      if frac(Ticks_max/Ticks_step) <> 0.0 then
	 if Ticks_max > 0 then
	    Ticks_max := (trunc(Ticks_max/Ticks_step)+1) * Ticks_step;
	 else
            Ticks_max := trunc(Ticks_max/Ticks_step) * Ticks_step;
         end_if               
      end_if;
      return([Ticks_step/Ticks_delta, Ticks_min, Ticks_max])
*/
      l := []; 
      s := floor(Ticks_max/Ticks_step) * Ticks_step;
      if abs(s-Ticks_max) > 10^(-DIGITS) then
         while s > Ticks_max do s := s - Ticks_step end;
      end_if;
      d := Ticks_step/Ticks_delta;
      while s + d <= Ticks_max do s := s + d end;
      while s >= Ticks_min do
         l := append(l, s);
	 s := s - d
      end_while;
      return([l, Ticks_min, Ticks_max])
   else
      for i from 1 to nops(ticks) do
          if type(ticks[i]) = "_equal" then
	     ticks[i] := op(ticks[i], 1)
	  end_if
      end_for;
      // VCam automatically resize the bounding box w.r.t.
      // the minimal and maximal value of ticks
      tmin := min(tmin, op(ticks)); tmax := max(tmax, op(ticks));
      return([ticks, tmin, tmax])
   end_if
end_proc:
