/* implicit 2-dimensional plot
  
  Syntax: implicitplot(f,rx,ry <,n>)

  f     : procedure or function with two parameters or
          a list of procedures or functions
  rx,ry : ranges
  n     : integer (optional, default is 6)

  Examples: plotlib::implicitplot((x,y)->(x^2+y^2-1),-1..1,-1..1):

            plotlib::implicitplot([(x,y)->(sin(x*y)), func(x+2*y,x,y)],
                                  -PI..PI, -2..2):
            

  The algorithm used is the BisectEx algorithm of
  Jean-Pierre Dedieu and Jean-Claude Yakoubsohn,
  see Numerical Algorithms, 4 (1993), pages 1-24.
*/

plotlib::implicitplot :=
proc(f,rx,ry,n) 
// f is a fun or a list of fun's, rx, ry are ranges, n an integer 
  local g;
begin
  if args(0)<4 then n:=6 end_if; // default depth 
  if type(f) = DOM_LIST then
    plot2d(
           [Mode=List,[plotlib::bisect(g,Dom::Interval(float(rx)),
                                       Dom::Interval(float(ry)),n)]] $ g in f
      )
   else 
      plot2d([Mode=List,[plotlib::bisect(f,Dom::Interval(rx),Dom::Interval(ry),n)]])
   end_if
end_proc:

plotlib::bisect :=
proc(f,ix,iy,n)
  local v,mx,my,xl,xr,yl,yr;
begin
   v:=sign(f(ix,iy));
   if contains(v, 0)<>TRUE then
     null() // exclusion 
   else
        xl:=ix::dom::left(ix); xr:=ix::dom::right(ix);
        yl:=iy::dom::left(iy); yr:=iy::dom::right(iy);
      if n=0 then  
        polygon(point(xl,yl),point(xr,yl),point(xr,yr),Filled=TRUE),
        polygon(point(xl,yl),point(xl,yr),point(xr,yr),Filled=TRUE)
      else // bisection 
        mx:=(xl+xr)/2; my:=(yl+yr)/2;
        xl:=ix::dom::new([xl,mx]); xr:=ix::dom::new([mx,xr]);
        yl:=iy::dom::new([yl,my]); yr:=iy::dom::new([my,yr]);
        plotlib::bisect(f,xl,yl,n-1), plotlib::bisect(f,xl,yr,n-1),
        plotlib::bisect(f,xr,yl,n-1), plotlib::bisect(f,xr,yr,n-1)
      end_if
   end_if
end_proc:

