/*-
  implplot.mu

  Eine neue Implementation von implicitplot fr Nullstellenmengen
glatter Funktionen R^2->R.  Die Funktion mu auf der Nullstellenmenge
fast berall regulr sein.

-*/


plotlib::implicitplot :=
proc(F,xrx,yry)
  local f,c,x,y,xmin,xmax,ymin,ymax,plotfactor,
  Lf,Lstart,newton2d,clean_startpoints,PlotList,
  currcolor, Options;
begin

  // testtype(xrx,"_equal")...
  ([x,xmin])    := [op(xrx)];
  ([xmin,xmax]) := map([op(xmin)],float);
  ([y,ymin])    := [op(yry)];
  ([ymin,ymax]) := map([op(ymin)],float);

  Options := table(
	hold(Grid) = [5,5],
	hold(Colors) = [[1.0,0.0,0.0],[0.0,1.0,0.0],[0.0,0.0,1.0],
                 [1.0,1.0,0.0],[1.0,0.0,1.0],[0.0,1.0,1.0],[1.0,1.0,1.0]],
	hold(MinStepsize) = min(xmax-xmin,ymax-ymin)*1.0e-6,
	hold(MaxStepsize) = min(xmax-xmin,ymax-ymin)*1.0e-2,
	hold(StartingStepsize) = min(xmax-xmin,ymax-ymin)*1.0e-3,
	hold(Precision) = 10.0^(2-DIGITS),
	hold(Ticks)=[10,10],
	hold(Contours)=[0],
	hold(Splines)=FALSE,
	hold(Factor)=FALSE,
	args(4..args(0)));

  if nops(Options[hold(Grid)]) < 2 then
	Options[hold(Grid)] := [Options[hold(Grid)],Options[hold(Grid)]];
  end_if;

// xxx Argument checking!!

  currcolor := 1;
  
  newton2d :=
  proc(f,Df,x0,y0,h)
    name plotlib::implicitplot::newton2d;
    local i,fxy,Df_square,Dfxy;
  begin
    i := 0;
    fxy := float(subs(f,x=x0,y=y0));
    while (abs(fxy)>Options[hold(Precision)]) and i<500 do
      Dfxy := [ float(subs(Df[1],x=x0,y=y0)), float(subs(Df[2],x=x0,y=y0)) ];
      Df_square := Dfxy[1]^2+Dfxy[2]^2;
      if abs(Df_square/fxy)<1e-15 then 
	// Rang(Df) not maximal. Make a small perturbation instead.
	userinfo(20,"Perturbating...");
        x0 := x0 + h/(random(-100..100)()+0.5);
        y0 := y0 + h/(random(-100..100)()+0.5);
      else
        x0  := x0 - h*fxy*Dfxy[1]/Df_square;
        y0  := y0 - h*fxy*Dfxy[2]/Df_square;
      end_if:
      fxy := float(subs(f,x=x0,y=y0));
      i := i+1;
    end_while:
    return([x0,y0,i]);
  end_proc:

  clean_startpoints :=
  proc(last_x, last_y, cur_x, cur_y, epsilon)
    name plotlib::implicitplot::clean_startpoints;
    local i, min_x, max_x, min_y, max_y, a, b, dist;
    option escape;
  begin
    min_x := min(last_x, cur_x) - epsilon;
    max_x := max(last_x, cur_x) + epsilon;
    min_y := min(last_y, cur_y) - epsilon;
    max_y := max(last_y, cur_y) + epsilon;
    for i from 1 to nops(Lstart) do
      if(min_x <= Lstart[i][1]) then
        break;
      else
        if Lstart[i][1]>max_x then return(); end_if;
      end_if;
    end_for;

    a := last_y - cur_y;
    b := cur_x - last_x;
    dist := sqrt(a^2+b^2);
    if iszero(dist) then return() end_if;
    a := a/dist;
    b := b/dist;

    while(i<=nops(Lstart) and Lstart[i][1]<max_x) do
      if (Lstart[i][2]>min_y and Lstart[i][2]<max_y
          and abs(a*(Lstart[i][1]-cur_x)+b*(Lstart[i][2]-cur_y))<epsilon )
      then
        userinfo(25,"Deleting ".expr2text(Lstart[i]));
        delete Lstart[i];
      else
        i := i+1;
      end_if;
    end_while;
  end_proc;
  
  plotfactor :=
  proc(f)
    name plotlib::implicitplot::plotfactor;
    local Df, plotsingle, startx, starty, x_i, y_i,
          dummy, Lsol;
  begin
    plotsingle :=
    proc(cur_x, cur_y)
      name plotlib::implicitplot::plotfactor::plotsingle;
      local direction, h, Df_x, Df_y, Df_xy, dx, dy,
            res, res_x, res_y, old_nres, old_Dx, old_Dy,
            last_x, last_y, n, t, minmax;
    begin
      minmax := proc() begin [min(args()),max(args())]; end;
      res := [[startx,starty,0.0]];
      for direction from -1 to 1 step 2 do
        old_nres := nops(res);
        cur_x := startx;
        last_x := startx;
        last_y := starty;
        cur_y := starty;
        h := Options[hold(StartingStepsize)];
	t := 0.0;
        old_Dx:=0;
        old_Dy:=0;
// xxx Schutz vor periodischen Lsungen verbessern!
        while xmin <= cur_x and cur_x <= xmax
          and ymin <= cur_y and cur_y <= ymax
          and (nops(res)-old_nres<5
               or (((cur_x-startx)^2+(cur_y-starty)^2)>h^2))
          and nops(res) < 1000 do

          Df_x := float(subs(Df[1],x=cur_x,y=cur_y));
          Df_y := float(subs(Df[2],x=cur_x,y=cur_y));
// Mit der QR-Zerlegung von numeric::factorQR ist det(Q)=
//
//                 3                            2
//                b                            a  b
//  - --------------------------- - ---------------------------,
//                 /    2    \1/2                /    2    \1/2
//      2    2 3/2 |   b     |        2    2 3/2 |   b     |
//    (a  + b )    | ------- |      (a  + b )    | ------- |
//                 |  2    2 |                   |  2    2 |
//                 \ a  + b  /                   \ a  + b  /
//
// also sign(det(Q))=-sign(b), falls (a,b)<>(0,0).  Weiterhin gilt
//      R = ((Df_xy[1]^2+Df_xy[2]^2)^(1/2)),
// also det(R)>=0.  Somit ist
//     ( diff(f,x)  diff(f,y) )
// sign(   dx          dy     ) = direction
//
// erfllt fr

          Df_xy := sqrt(Df_x^2+Df_y^2);
          if iszero(Df_xy) then
            break;
          end_if;
          dx    := direction * -Df_y/Df_xy;
          dy    := direction * Df_x/Df_xy;

          old_Dx := old_Dx * Df_x;
          old_Dy := old_Dy * Df_y;
          if old_Dx <= 0 and old_Dy <=0
            and not(iszero(old_Dx) and iszero(old_Dy))
            then
            // possible bifurcation point
                break;
          end_if;
          old_Dx := Df_x;
          old_Dy := Df_y;

          last_x := cur_x;
          last_y := cur_y;
          
          ([cur_x,cur_y,n]) := newton2d(f,Df,cur_x+h*dx,cur_y+h*dy,1);
//        t := t + direction * h * Df_xy;
          t := t + direction * sqrt((cur_x-last_x)^2+(cur_y-last_y)^2);
          clean_startpoints(last_x,last_y,cur_x,cur_y,h);
          res := res.[[cur_x,cur_y,t]];
          h := /*min(1.0, Df_xy)**/ h*5/(n+1);
          h := min(max(h,Options[hold(MinStepsize)]),
                   Options[hold(MaxStepsize)]);
       end_while;
      end_for; // direction
      if nops(res) = 2 then
          ([cur_x,cur_y,n]) := newton2d(f,Df,(res[1][1]+res[2][1])/2.0,
                                             (res[1][2]+res[2][2])/2.0,
                                             1);
          res := res.[[cur_x,cur_y,(res[1][3]+res[2][3])/2.0]];
      end_if;
        
      userinfo(20, "Found ".expr2text(nops(res))." points on component.");
// xxx this is too much work.  --> collect the points differently
      res := sort(res, (a,b)->bool(op(a,3)<op(b,3)));
      if Options[hold(Splines)] then
        res_y := numeric::cubicSpline(op(map(res, 
              x -> [op(x,3),op(x,2)])), Natural);
        res_x := numeric::cubicSpline(op(map(res,
              x -> [op(x,3),op(x,1)])), Natural);
        t := genident();
        dx := minmax(op(map(res,op,1)));
        dx := dx[2] - dx[1];
        dy := minmax(op(map(res,op,2)));
        dy := dy[2] - dy[1];
        dx := min(5*nops(res),50,
                          ceil(100*(max(dx/(xmax-xmin),dy/(ymax-ymin)))));
        dx := max(dx, 3);
          userinfo(20, "Using ".expr2text(dx)." points on curve");
        return([hold(Mode)=hold(Curve),[res_x(t),res_y(t)],
                t=minmax(op(map(res,op,3))),
                hold(Grid)=[dx],
                hold(Color)=[hold(Flat),Options[hold(Colors)][currcolor]]]
//             ,[hold(Mode)=hold(List),map(res,x->point(op(x,1),op(x,2)))]
                  );
        else
          return(polygon(_seqgen(point(res[i][1],res[i][2]),i,1..nops(res))));
        end_if;
    end_proc:

    Df:=[diff(f,x), diff(f,y)];

    Lstart := [];
    userinfo(10,"Calculating startpoints");
    for x_i from xmin to xmax step ((xmax-xmin)/Options[hold(Grid)][1]) do
      for y_i from ymin to ymax step ((ymax-ymin)/Options[hold(Grid)][2]) do
        Lstart := Lstart.[[op(newton2d(f,Df,x_i,y_i,1),1..2)]];
      end_for;
    end_for;

    Lsol :=[];
//    Lstart:=sort(Lstart, (a,b)->bool(a[1]<b[1] or (a[1]=b[1] and a[2]<b[2])));
    userinfo(15,"Calculated ".expr2text(nops(Lstart))." points");
    while nops(Lstart)>0 do
      [startx,starty] := Lstart[1];
      delete Lstart[1];
      ([startx,starty,dummy]) := newton2d(f,Df,startx,starty,1);
      if(startx>=xmin and startx <= xmax
         and starty >= ymin and starty <= ymax) then
        userinfo(20,"Starting at (".
                 expr2text(startx).",".expr2text(starty).")");
        Lsol := Lsol.[plotsingle(startx,starty)];
      end_if;
    end_while;
    return(Lsol);
  end_proc:

  if not(testtype(F,DOM_LIST)) then F := [F] end_if:
  PlotList := [];
  for f in F do
    userinfo(5,"Plotting ".expr2text(f));
    for c in Options[hold(Contours)] do
      if Options[hold(Factor)] then
        Lf := factor(f-c);
        Lf := Factored::convert_to(Lf,DOM_LIST);
        Lf := [Lf[2*i]$i=1..(nops(Lf)-1)/2];
      else
        Lf := [f-c];
      end_if;
      PlotList := PlotList.[op(map(Lf,op@plotfactor))];
      currcolor := modp(currcolor,nops(Options[hold(Colors)])) + 1;
    end_for;
  end_for;
  userinfo(5,"Calling plot2d with ".expr2text(nops(PlotList))." curves");
  if Options[hold(Splines)] then
    plot2d(hold(Axes)=hold(Origin),
  	   hold(Labels)=[expr2text(x),expr2text(y)],
  	   plotlib::check_ticks(Options[hold(Ticks)],[xmin,ymin,xmax,ymax],
                                [10,10]),
  	   op(PlotList)
  	   );
  else
    plot2d(hold(Axes)=hold(Origin),
  	   hold(Labels)=[expr2text(x),expr2text(y)],
  	   plotlib::check_ticks(Options[hold(Ticks)],[xmin,ymin,xmax,ymax],
                                [10,10]),
           [hold(Mode)=hold(List),PlotList]
  	   );
  end_if;
  null();
end_proc:

/*-

 Beispiele:

- sin(x*y),x=-5..5,y=-5..5

[dev] > plotlib::implicitplot(sin(x*y),x=-5..5,y=-5..5)


                                                               Domain: DOM_NULL
                                                               Properties: None
                                                               Time: 223.01 sec

- sin(5*sin(x)*y)

- sin(x^3-x*y),x=-5..5,y=-5..5

- y*(3*x^2-y^2)-(x^2+y^2)^2, x=-1..1, y=-1..1



-*/
