/*++

plotlib::plotsetup(dev)

dev -- device ( X11 or Term )

plotsetup is used for plotting with mupad (terminal version of MuPAD).
If mupad is running in an X11 environment, plotsetup(X11) modify the
functions plot2d and plot3d to display the graphic in a separate
window on the workstation display. 
plotdevice(Term) resets plot2d and plot3d.
plotdevice is only available on Unix based operating systems.
 
++*/

plotlib::plotsetup_showplot := proc()
begin
  if plotlib::plotsetup_DEVICE = hold(X11) then
    if system("mxvcam save.mp &") <> 0 then
      print("Can't spawn external previewer.") ;
    else
      return( null() )
    end_if;
    print("Plotdata saved in binary file save.mp") ;
  end_if ;
  FAIL ;
end_proc:

plotlib::plotsetup := proc(dev)
  local plot2d_ext, plot3d_ext, plotfunc2d_ext, plotfunc3d_ext;
begin
  if sysname() <> "UNIX" then
     error("this function only runs on a UNIX platform");
  end_if;

/* ------- local routines ------- */
  plot3d_ext := proc()
      local r ;
      name plot3d;
  begin
    if ( r:=plotlib::plotsetup_plot3d(args()) ) = FAIL then
      plotlib::plotsetup_showplot() ;
    else
      r ;
    end_if ;
  end_proc:

  plot2d_ext := proc()
      local r ;
      name plot2d;
  begin
    if ( r:=plotlib::plotsetup_plot2d(args()) ) = FAIL then
      plotlib::plotsetup_showplot() ;
    else
      r ;
    end_if ;
  end_proc:

  plotfunc3d_ext := proc()
      local r ;
      name plotfunc3d;
  begin
    if ( r:=plotlib::plotsetup_plotfunc3d(args()) ) = FAIL then
      plotlib::plotsetup_showplot() ;
    else
      r ;
    end_if ;
  end_proc:

  plotfunc2d_ext := proc()
      local r ;
      name plotfunc2d;
  begin
    if ( r:=plotlib::plotsetup_plotfunc2d(args()) ) = FAIL then
      plotlib::plotsetup_showplot() ;
    else
      r ;
    end_if ;
  end_proc:

/* ------------------------------- */

  case dev

   of hold(Term) do

     // if .. <> dev abfragen, sonst rekursive Definition bei
     // mehrfachen Aufrufen !
     if plotlib::plotsetup_DEVICE <> dev then
        sysassign( plotlib::plotsetup_DEVICE, dev );
     end_if;
     if plotlib::plotsetup_plot2d <> FAIL then
        sysassign( plot2d, plotlib::plotsetup_plot2d );
        sysassign( plot3d, plotlib::plotsetup_plot3d );
     end_if;
     break;

  of hold(X11) do

     // if .. <> dev abfragen, sonst rekursive Definition bei
     // mehrfachen Aufrufen !
     if plotlib::plotsetup_DEVICE <> dev then 
        sysassign( plotlib::plotsetup_DEVICE, dev);
     end_if;
     if plotlib::plotsetup_plot2d = FAIL then
        sysassign( plotlib::plotsetup_plot2d, plot2d );
        sysassign( plotlib::plotsetup_plot3d, plot3d );
        sysassign( plotlib::plotsetup_plotfunc2d, plotfunc2d );
        sysassign( plotlib::plotsetup_plotfunc3d, plotfunc3d );
     end_if;
     sysassign( plot2d, plot2d_ext );
     sysassign( plot3d, plot3d_ext );
     sysassign( plotfunc2d, plotfunc2d_ext );
     sysassign( plotfunc3d, plotfunc3d_ext );
     break;
 
  otherwise

    error("Unknown device '".dev."'.");
 
  end_case;
  
  null()
  
end_proc:

// end of file 
