// $Date: 2000/05/21 14:57:55 $ $Author: bij $ $Revision: 1.6 $ 


/*++
	cyclotomic(n,x) 

	computes the n-th cyclotomic polynomial, expressed
	in the indeterminate x

++*/


polylib::cyclotomic:=
proc(n: Type::PosInt, x: DOM_IDENT)
  local numerator, denominator, divsn,pos,neg, deg,
  modulus, i, signoflargef;

begin
  deg:=numlib::phi(n);
  modulus:=poly(x^(deg+1),[x]);
// result will have degree deg
  divsn:=select(numlib::divisors(n), i -> numlib::moebius(n/i)<>0);
  
/* in principle, we would have to compute
  \prod_{mu(n/d)=1}(x^d-1) / \prod_{mu(n/d)=-1}(x^d-1) ;
  but we can do the computation modulo x^(deg+1), and
  therefore replace each factor of high degree by -1 */

  split(divsn, _less, deg+1);
  signoflargef:=nops(op(%,2));
  divsn:=op(%2,1);
  userinfo(10, expr2text(signoflargef)." large factors left out");
  [pos,neg]:=[op(split(divsn, i -> numlib::moebius(n/i)=1), 1..2)];
  numerator:=poly(1,[x]);
  for i in pos do
    numerator:=divide(numerator*poly(x^i-1,[x]), modulus, Rem)
  end_for;
  userinfo(10, "Numerator is ".expr2text(numerator));
  denominator:=poly(1,[x]);
  for i in neg do
    denominator:=divide(denominator*poly(x^i-1,[x]), modulus, Rem)
  end_for;
  userinfo(10, "denominator is ".expr2text(denominator));
  divide(multcoeffs(numerator, (-1)^signoflargef)*
         op(gcdex(denominator, modulus), 2), modulus, Rem)
end_proc: 

