// $Date: 2000/07/06 09:43:33 $ $Author: kg $ $Revision: 1.7 $ 


/*
    polylib::sortMonomials(f, ord)

    returns a list of monomials sorted in *descending* order w.r.t.
    the monomial order ord.

    ord may be the identifier LexOrder, DegreeOrder, DegInvLexOrder;
    or an object of type Dom::MonomOrdering

*/


polylib::sortMonomials:=

proc(f:Type::Union(Type::Arithmetical, DOM_POLY), ord=hold(LexOrder))
  local nvars, o, monomlist, i;
begin
  if args(0)=0 then
    error("sortMonomials called without arguments")
  end_if;
  if args(0)>4 then
    error("Wrong number of arguments")
  end_if;

  if f::dom::sortMonomials <> FAIL then
    return(f::dom::sortMonomials(args()))
  end_if;
  
  if type(ord)=DOM_LIST then
    if testtype(f, Type::Arithmetical) then
      return(expr(polylib::sortMonomials(poly(f,ord), args(3..args(0)))))
    else
      return(polylib::sortMonomials(poly(f, ord), args(3..args(0))))
    end_if
  end_if;
  nvars:=nops(op(f,2));
  monomlist:=[polylib::nthmonomial(f, i) $i=1..nterms(f)];
  
  if type(ord)=DOM_IDENT then
    case ord
      of hold(LexOrder) do
        return(monomlist)
      of hold(DegreeOrder) do
        o:=Dom::MonomOrdering(DegLex(nvars)); break;
      of hold(DegInvLexOrder) do
        o:=Dom::MonomOrdering(DegInvLex(nvars)); break;
      otherwise
        error("Unknown monomial ordering")
    end_case
  elif type(ord)="function" then
    o:=Dom::MonomOrdering(ord)
  else
    o:=ord
  end_if;
  sort(monomlist, (x,y) -> bool(o(degreevec(x), degreevec(y))>0))
end_proc:
      
