// $Date: 2000/06/06 20:03:03 $ $Author: frankp $ $Revision: 1.11 $
// kg, 16/12/94

/*++
polylib::sqrfree -- square-free factorization of polynomials

sqrfree(p)

p - polynomial or polynomial expression
Recollect - determines whether factors with the same power are collected

Returns the square-free factorization of a polynomial or expression
over the rationals.

If the polynomial is of type Expr ist coefficients must be rationals.
If the polynomial is of type IntMod(n) then n must be a prime number.
If the coefficients are from a domain that domain must be a UFD of 
characteristic 0 or a finite field.
++*/

polylib::sqrfree:= proc(p, Recollect=TRUE:DOM_BOOL)
  local t, i, j, recollect;
begin

  // recollect - local method
  recollect:=
  if Recollect then
    proc(f)
      local t;
    begin
      // re-collect factors with equal powers
      t:= table();
      for i from 3 to nops(f) step 2 do
        if contains(t, f[i]) then
          t[f[i]]:= t[f[i]] * f[i-1]
        else
          t[f[i]]:= f[i-1]
        end_if
      end_for;
      [ f[1], (op(t, [j,2]), op(t, [j,1])) $ j=1..nops(t) ]
    end_proc;

  else
    // do not recollect
    id
  end_if;


  // m a i n   p r o g r a m
  
  if args(0) > 2 then
    error("wrong no of args")
  end_if;
  if p::dom::sqrfree <> FAIL then
    p:= p::dom::sqrfree(p);
    if domtype(p) = DOM_LIST then
      return(Factored::create( p,"squarefree" ))
    else
      return(p)
    end_if
  end_if;
  if domtype(p) = DOM_POLY then
    if testargs() then
      t:= op(p,3);
      if domtype(t) = DOM_DOMAIN then
        if t::hasProp <> FAIL then
          if t::hasProp(Cat::FactorialDomain) then
            if t::characteristic <> 0 then
              if t::hasProp(Cat::Field) then
                if not t::hasProp(Dom::IntegerMod) then
                  if not t::hasProp(Dom::GaloisField) then
                    if not t::hasProp(Dom::AlgebraicExtension) then
                      error("not a finite field")
                    elif t::degreeOverPrimeField=UNKNOWN then
                      error("not a finite field")
                    end_if
                  end_if
                end_if
              else
                error("not a field")
              end_if
            end_if
          else
            error("not a factorial domain")
          end_if
        end_if
      else
        if t<>hold(Expr) and not isprime(op(t,1)) then
          error("no prime modulus")
        end_if
      end_if
    end_if;
    p:= recollect(faclib::sqrfree_poly(p));
    return(Factored::create( p,"squarefree" ))
  else
    p:= recollect(faclib::factor_expr(p, faclib::sqrfree_poly));
    return(Factored::create( p,"squarefree" ))
  end_if
    
end_proc:

