// $Date: 2000/10/04 08:28:21 $ $Author: kg $ $Revision: 1.39 $

//
// This file contains informations about changes
// between MuPAD 1.4.2 and MuPAD 2.0
// 

prog::Changes:= newDomain("prog::Changes"):
prog::Changes::info:= "Changes between MuPAD 1.4.2 and MuPAD 2.0":
alias(Change = prog::Changes):

// old plotfunc -> plotfunc2d/plotfunc3d
// old Puiseux
// old funcattr, func_env, strlen
// old RealNum, PRETTY_PRINT

alias(path = pathname("PROG")):
Change::renamed:=      loadproc(Change::renamed,      path, "CH_renamed"):
Change::renamedLibs:=  loadproc(Change::renamedLibs,  path, "CH_renamed"):
Change::renamedSlots:= loadproc(Change::renamedSlots, path, "CH_renamed"):
Change::removed:=      loadproc(Change::removed,      path, "CH_removed"):

Change::newKeywords:=  loadproc(Change::newKeywords,  path, "CH_new"):
Change::newLibs:=      loadproc(Change::newLibs,      path, "CH_new"):
Change::newFunctions:= loadproc(Change::newFunctions, path, "CH_new"):

Change::undocumented:=
  table(hold(System)   = [hold(stdmod),hold(util) /* modules */, hold(plotlib),
                          hold(sysassign), hold(sysevalassign),
                          hold(sysdelete),
                          hold(clientCall), hold(clientObject),
                          hold(rdabs), hold(rddivide),
                          hold(rdfloat2text), hold(rdgetsign),
                          hold(rdint2float), hold(rdisfinite), hold(rdmult),
                          hold(rdnegate), hold(rdplus), hold(rdround),
                          hold(rdsubtract), hold(rdtext2float),
                          hold(_new_bytecode), hold(outputFormat), hold(outputType),
                          hold(_op_fconcat), hold(_op_plusmult),
                          hold(_op_power), hold(_lazy_and), hold(_lazy_or),
                          hold(_constructor), hold(_act_proc_env),
                          hold(_check_global), hold(_eval_entry),
                          hold(_mult_intern), hold(_parser_callback),
                          hold(_parser_config), hold(_pattern),
                          hold(_print_userinfo), hold(_userinfo_level),
                          hold(radnormal), hold(C_), hold(R_), hold(Z_),
                          hold(Q_), hold(Content), hold(MathContent)]
        ):

prog::changes:=
  proc(OPT = hold(Warning), QU = hold(Interactive))
    option hold;
    local k, LIB, FUNC;
  begin
    if OPT = hold(Info) then
      return(prog::printChanges(hold(All)))
    elif OPT = hold(Remove) or OPT = NIL then
      return(prog::remove(hold(All), QU))
    elif OPT = hold(Warning) or OPT = hold(Error) then
      return(prog::_changes(OPT, QU))
    elif OPT = hold(Quiet) then
      return(prog::_changes(hold(Warning), hold(Quiet)))
    elif type(OPT) = "slot" then
      return(prog::findChanges(QU, context(op(OPT))))
    else
      return(prog::findChanges(QU, context(OPT)))
    end_if
  end_proc:

prog::_changes:=
  proc(OPT = hold(Warning), QU)
    local k, LIB, FUNC;
  begin
    sysassign(prog::COMPAT, TRUE);
    sysassign(Change::oldlibs, []);
    // changes
    for k in Change::renamed do
      // [hold(stdlib), ifft] = [hold(numeric), invfft]
      LIB:= op(k, [1, 1]);
      FUNC:= /*eval*/(op(k, [1, 2]));
      if not contains({DOM_DOMAIN, DOM_FUNC_ENV}, domtype(eval(LIB)))
         and LIB <> hold(Sysvar) and LIB <> hold(System) then
        userinfo(1, "Creating non-existing library '".expr2text(LIB)."' [prog::changes]");
        sysassign(Change::oldlibs, append(Change::oldlibs, LIB));
        if QU <> hold(Quiet) then
          protected(LIB, hold(Error))
        end_if;
        sysevalassign(LIB, newDomain(expr2text(LIB)), 2);
        LIB:= eval(LIB)
      end_if;
      if LIB = hold(Sysvar) then
        // Sonderfall Variablen
        userinfo(1, "Protect obsolete system variable '".prog::getname(FUNC)."' [prog::changes]");
        if QU <> hold(Quiet) then
          protected(FUNC, hold(Error))
        end_if;
        sysevalassign(FUNC,
                      subsop(hold(hold((warning("system variable '".NIL.
                                                "' was renamed, please use '".NIL."'");NIL))),
                             _if(OPT = hold(Error), [1,1,0] = error, null()),
                             [1,1,1,2] = expr2text(op(k, [1, 2])),
                             [1,1,1,4] = expr2text(op(k, [2, 2])), [1,2] = op(k, [2, 2]),
                             Unsimplified), 2)
                      //hold(hold(error))("obsolete system variable, use '".expr2text(op(k, [2, 2]))."'"))
      else
        // re-define old functions
        LIB:= eval(LIB);
        userinfo(1, "Redefine obsolete function '"._if(LIB <> hold(System) and LIB <> hold(Sysvar), expr2text(LIB)."::", "").prog::getname(FUNC)."' [prog::changes]");
        if FUNC = hold(unassign) then
          // Sonderfall!
          sysassign(unassign, proc()
                                option hold;
                              begin
                                warning("'unassign' was removed. Use the new keyword 'delete'");
                                eval(hold(_delete)(args()))
                              end)
        else
          prog::redefine(LIB, FUNC, op(k, [2, 1]), op(k, [2, 2]), OPT, "", QU);
        end_if;
        if LIB = stdlib then
          sysevalassign(FUNC, slot(LIB, expr2text(FUNC)), 2)
        end_if
      end_if
    end_for;

    // removed
    for k in Change::removed do
      // [hold(System), "domain"] = [hold(System), newDomain, "use 'newDomain' instead"]
      LIB:= op(k, [1, 1]);
      FUNC:= /*eval*/(op(k, [1, 2]));
      if not contains({DOM_DOMAIN, DOM_FUNC_ENV}, domtype(eval(LIB)))
         and LIB <> hold(Sysvar) and LIB <> hold(System) then
        userinfo(1, "Creating non-existing library '".expr2text(LIB)."' [prog::changes]");
        if QU <> hold(Quiet) then
          protected(LIB, hold(Error))
        end_if;
        sysevalassign(LIB, newDomain(expr2text(LIB)), 2);
        LIB:= eval(LIB)
      end_if;
      if LIB = hold(Sysvar) then
        // Sonderfall Variablen
        userinfo(1, "Protect obsolete system variable '".prog::getname(FUNC)."' [prog::changes]");
        if QU <> hold(Quiet) then
          protected(FUNC, hold(Error))
        end_if;
        sysevalassign(FUNC,
                      subsop(hold(hold((warning("system variable '".NIL.
                                                "' was removed".NIL);NIL))),
                             _if(OPT = hold(Error), [1,1,0] = error, null()),
                             [1,1,1,2] = expr2text(op(k, [1, 2])),
                             [1,1,1,4] = _if(op(k, [2, 3]) <> "", ", ".op(k, [2, 3]), ""),
                             [1,2] = op(k, [2, 2]),
                             Unsimplified), 2)
                      //hold(hold(error))("obsolete system variable, use '".expr2text(op(k, [2, 2]))."'"))
      else
        // re-define old functions
        LIB:= eval(LIB);
        userinfo(1, "Redefine obsolete function '"._if(LIB <> hold(System), expr2text(LIB)."::", "").prog::getname(FUNC)."' [prog::changes]");
        if op(k, [2, 2]) = FAIL then
          prog::redefine(LIB, FUNC, op(k, [2, 1]), /*hold(procname)*/ "test", OPT,
                         _if(nops(op(k, 2)) > 2, op(k, [2, 3]), FAIL), QU)
        else
          prog::redefine(LIB, FUNC, op(k, [2, 1]), op(k, [2, 2]), OPT,
                         _if(nops(op(k, 2)) > 2, op(k, [2, 3]), FAIL), QU)
        end_if;
        if LIB = stdlib then
          sysevalassign(FUNC, slot(LIB, expr2text(FUNC)), 2)
        end_if
      end_if
    end_for;
    if QU <> hold(Quiet) then
        fprint(Unquoted, 0, "\nInfo: Obsolete functions of MuPAD version 1.4 are 'restored'.");
      if OPT = hold(Error) then
        fprint(Unquoted, 0, "      Any call to an obsolete function will produce a hint");
        fprint(Unquoted, 0, "      and an error.");
      else
        fprint(Unquoted, 0, "      Any call to an obsolete function will produce a warning.");
        fprint(Unquoted, 0, "      A function from the present library with the same or a  ");
        fprint(Unquoted, 0, "      similar functionality will be called automatically.");
      end_if
    end_if;
    // keep state
    sysassign(Change::state, (OPT, QU));
    null()
  end_proc:

prog::redefine:=
  proc(LIB, FUNC, NLIB, NFUNC, OPT, MSG = "", QU = hold(Interactive))
    local PR, NM, PNM;
  begin
    if NLIB = hold(System) or NLIB = hold(Sysvar) or NLIB = stdlib or NLIB = hold(stdlib) then
      NM:= prog::getname(NFUNC);
      PNM:= FUNC;
      if type(FUNC) = DOM_STRING then
//         if QU <> hold(Quiet) then
//           warning("Cannot redefine function '".FUNC."'; it is now a keyword in MuPAD\n         '".FUNC."' was changed to '".NM."'");
//         end_if;
        return()
      end_if
    else
      if domtype(FUNC) = DOM_STRING then
        NM:= expr2text(NLIB)."::".prog::getname(NFUNC);
        PNM:= hold(slot)(LIB, FUNC);
        NFUNC:= hold(slot)(NLIB, expr2text(NFUNC));
//         if QU <> hold(Quiet) then
//           warning("Only use '".FUNC."' as string in 'slot(".expr2text(NLIB).", \"".FUNC."\")';\n         You cannot type '".expr2text(PNM)."'\n         '".FUNC."' is now a keyword in MuPAD")
//         end_if;
      else
        NM:= expr2text(NLIB)."::".prog::getname(NFUNC);
        if LIB <> hold(System) then
          PNM:= hold(slot)(LIB, expr2text(FUNC));
          FUNC:= expr2text(FUNC)
        else
          PNM:= FUNC
        end_if;
        NFUNC:= hold(slot)(NLIB, expr2text(NFUNC))
      end_if
    end_if;
    if LIB = hold(System) then
      PR:= subsop(proc()
                    option hold;
                  begin
                    warning("");
                    id(context(args()))
                  end_proc,
                  _if(OPT = hold(Error), ([4, 1, 0] = error, [4, 2] = null()), null()),
                  [4, 1, 1] = "'".expr2text(FUNC).
                  _if(MSG = "", "' was changed to ".
                      _if(NM = "CATALAN", "the constant ", "")."'".NM."'",
                                  "' was removed"._if(MSG <> FAIL, ", ".MSG, "")),
                  _if(OPT = hold(Warning),
                      [4, 2, 0] = NFUNC, null()),
                  6 = PNM);
      sysevalassign(FUNC, PR, 2)
    else
      PR:= subsop(proc()
                    option hold;
                  begin
                    warning("");
                    id(context(args()))
                  end_proc,
                  _if(OPT = hold(Error), ([4, 1, 0] = error, [4, 2] = null()), null()),
                  [4, 1, 1] = "'".expr2text(LIB)."::".FUNC.
                  _if(MSG = "", "' was changed to '".NM."'",
                                "' was removed, ".MSG),
                  _if(OPT = hold(Warning),
                      [4, 2, 0] = NFUNC, null()),
                  6 = PNM);
      sysassign(slot(LIB, FUNC), PR)
    end_if
  end_proc:

prog::remove:=
  proc(OPT = hold(All), QU)
    option hold;
    local k, FUNC;
  begin
    //
    if eval(OPT) = hold(All) then
      // delete changed functions
      for k in Change::renamed do
        if type(op(k, [1, 2])) <> DOM_IDENT then
          // keywords
        elif contains({hold(System), hold(Sysvar)}, op(k, [1, 1])) then
          userinfo(1, "remove redefined function/variable '".expr2text(op(k, [1, 2]))."'");
          eval(hold(sysdelete)(op(k, [1, 2])))
        else
          FUNC:= hold(slot)(op(k, [1, 1]), expr2text(op(k, [1, 2])));
          userinfo(1, "remove redefined function '".expr2text(FUNC)."'");
          eval(hold(sysdelete)(FUNC))
        end_if
      end_for;
      // delete removed functions
      for k in Change::removed do
        if type(op(k, [1, 2])) <> DOM_IDENT then
          // keywords
        elif contains({hold(System), hold(Sysvar)}, op(k, [1, 1])) then
          userinfo(1, "remove redefined function/variable '".expr2text(op(k, [1, 2]))."'");
          eval(hold(sysdelete)(op(k, [1, 2])))
        else
          FUNC:= hold(slot)(op(k, [1, 1]), expr2text(op(k, [1, 2])));
          userinfo(1, "remove redefined function '".expr2text(FUNC)."'");
          eval(hold(sysdelete)(FUNC))
        end_if
      end_for;
      // delete old libs
      for k in Change::oldlibs do
        eval(hold(sysdelete)(k))
      end_for;
      if QU <> hold(Quiet) then
        fprint(Unquoted, 0, "\nInfo: All redefined functions and domains are removed.")
      end_if
    else
      eval(hold(sysdelete)(OPT))
    end_if
  end_proc:


prog::printChanges:=
  proc(OPT = hold(ALL))
    local newLibs, newFunctions, newVariables, newOptions,
          changedLibraries, changedFunctions, changedVariables,
          changedOptions, printLine, printSet,
          OUTFILE;
  begin
    OUTFILE:= 0;
    
    

// Change::newLibs
    newLibs:=
      proc()
      begin
      end_proc;
    
// Change::newFunctions
    newFunctions:=
      proc()
      begin
      end_proc;
    
    newVariables:=
      proc()
      begin
      end_proc;
    
    newOptions:=
      proc()
      begin
      end_proc;
// Change::newKeywords
    
// Change::renamedLibs
    changedLibraries:=
      proc()
      begin
      end_proc;
        
// Change::renamed
// Change::renamedSlots
    changedFunctions:=
      proc()
        local k, TABLE;
      begin
        TABLE:= ["obsolete function", "was renamed to", " ", " "];
        for k in sort([op(Change::renamed)], (a,b) ->_less(expr2text(op(a, [1,1])), expr2text(op(b, [1,1])))) do
          // [hold(stdlib), ifft] = [hold(numeric), invfft]
          LIB:= op(k, [1, 1]);
          FUNC:= /*eval*/(op(k, [1, 2]));

          if LIB = hold(Sysvar) then
            // Sonderfall Variablen
            TABLE:= append(TABLE, "'".expr2text(op(k, [1, 2]))."'",
                                  "'".expr2text(op(k, [2, 2]))."'")
          else
            // re-define old functions
            TABLE:= append(TABLE, "'".expr2text(op(k, [1, 1]))."::".expr2text(op(k, [1, 2]))."'",
                           "'".expr2text(op(k, [2, 1]))."::".expr2text(op(k, [2, 2]))."'")
          end_if
        end_for;
        printSet("changed Functions", TABLE, 2)
      end_proc;
    
    changedVariables:=
      proc()
      begin
      end_proc;
    
    changedOptions:=
      proc()
      begin
      end_proc;
    
// Change::removed

// Change::undocumented


    printLine:=
      proc()
      begin
      end_proc;
    
    printSet:=
      proc(NAME, SET, COL)
      begin
        fprint(hold(Unquoted), OUTFILE, NAME, ":\n");
        output::tableForm(SET, " ", {hold(Unquoted), hold(Column) = COL})
      end_proc;

    changedFunctions()
    
  end_proc:

prog::findChanges:=
  proc(QU, LIB = hold(System), FUNC = FAIL)
    //option hold;
    local FIND, CNL, CRM, NME, X, PZEILE;
  begin

    PZEILE:=
      proc(OOBJ, STAT, NOBJ = FAIL, DIR = 0)
      begin
        if NOBJ <> FAIL then
          if DIR = 0 then
            if QU <> hold(Quiet) then
              fprint(Unquoted, 0, "Info: '", OOBJ, "' is ", STAT, ".")
            else
              return(stringlib::subsop(STAT, 0 = "old"))
            end_if
          else
            if QU <> hold(Quiet) then
              fprint(Unquoted, 0, "Info: '", NOBJ, "' is ", STAT, ".")
            else
              return(stringlib::subsop(STAT, 0 = "new"))
            end_if
          end_if;
          if QU <> hold(Quiet) then
            fprint(Unquoted, 0, "      '", OOBJ, "' is changed to '", NOBJ, "'. [prog::changes]")
          end_if
        else
          if QU <> hold(Quiet) then
            fprint(Unquoted, 0, "Info: '", OOBJ, "' is ", STAT, ". [prog::changes]")
          else
            STAT
          end_if
        end_if
      end_proc;
    
    CNL:=
      proc(X)
      begin
        prog::getname(op(X, 1))."::".prog::getname(k) $ k in op(X, 2)
      end_proc;

    CRM:=
      proc(X, NR = 1)
      begin
        prog::getname(op(X, [NR, 1]))."::".prog::getname(op(X, [NR, 2]))
      end_proc;

    CRN:=
      proc(X, NR = 1)
      begin
        prog::getname(op(X, [NR, 1]))
      end_proc;

    NME:=
      proc(LIB, FUNC)
      begin
        if domtype(LIB) <> DOM_STRING then
          LIB:= prog::getname(LIB)
        end_if;
        if domtype(FUNC) <> DOM_STRING then
          FUNC:= prog::getname(FUNC)
        end_if;
        if contains({"System", "stdlib", "Sysvar"}, LIB) then
          FUNC
        else
          LIB."::".FUNC
        end_if
      end_proc;

    FIND:=
      proc(NLIB, NFUNC)
      begin
        if contains(Change::newKeywords, NFUNC) > 0 then
          // Change::newKeywords
          return([NFUNC, "a new keyword"])
        elif contains(map(Change::newLibs, expr2text), NFUNC) > 0 then
          // Change::newLibs
          return([NFUNC, "a new library"])
        elif contains(map({op(Change::newFunctions)}, CNL), NLIB."::".NFUNC) then
          // Change::newFunctions
          return([NME(NLIB, NFUNC), "a new function"])
        elif (X:= contains(map([op(Change::renamedLibs)], CRN, 1), NFUNC)) > 0 then
          // Change::renamedLibs
          return([NFUNC, "a renamed library", op(map([op(Change::renamedLibs)], CRN, 2), X)])
        elif (X:= contains(map([op(Change::renamedLibs)], CRN, 2), NFUNC)) > 0 then
          // Change::renamedLibs
          return([op(map([op(Change::renamedLibs)], CRN, 1), X), "a renamed library", NFUNC, 1])
        elif (X:= contains(map([op(Change::renamed)], CRM), NLIB."::".NFUNC)) > 0 then
          // Change::renamed
          return([NME(NLIB, NFUNC), "a renamed function",
                  op(map([op(Change::renamed)], X -> NME(op(op(X, 2)))), X)])
        elif (X:= contains(map([op(Change::renamed)], CRM, 2), NLIB."::".NFUNC)) > 0 then
          // Change::renamed
          return([op(map([op(Change::renamed)], X -> NME(op(op(X, 1)))), X),
                  "a renamed function", NME(NLIB, NFUNC), 1])
        elif FALSE then
          // Change::renamedSlots
        elif contains(map({op(Change::removed)}, CRM), NLIB."::".NFUNC) then
          // Change::removed
          return([NME(NLIB, NFUNC), "removed"])
        elif contains(map({op(Change::removed)}, CRM), "Sysvar"."::".NFUNC) then
          // Change::removed
          return([NFUNC, "a removed environment variable"])
        elif contains(map({op(Change::undocumented)}, CNL), NLIB."::".NFUNC) then
          // Change::undocumented
          return([NME(NLIB, NFUNC), "undocumented"])
        else
          type(eval(OFUNC))
        end_if
      end_proc;

    // ======== main ======== //
    if FUNC = FAIL then
      FUNC:= LIB;
      LIB:= hold(System)
    end_if;

    ret:= FIND(prog::getname(LIB), prog::getname(FUNC));
    if domtype(ret) <> DOM_LIST then
      ret:= FIND("stdlib", prog::getname(FUNC));
      if domtype(ret) <> DOM_LIST then
        if QU <> hold(Quiet) then
          fprint(Unquoted, 0, "Info: The given object seems not to be changed. [prog::changes]");
          return()
        else
          return(FAIL)
        end_if
      end_if
    end_if;
    return(PZEILE(op(ret)))
  end_proc:

prog::_generateCchanges:= proc()
    local fd, p, pp, l, i, j;
begin
    p:= proc() begin fprint(NoNL, fd, args()) end;
    pp:= proc(n) begin 
        if n[1] <> System and n[1] <> Sysvar then p(n[1]); p("::") end;
        p(n[2])
        end;
    fd:= fopen(Text, "changes.c", Write);
    p("/* changes.c -- generated by prog::_generateCchanges() */\n");
    // removed names
    l:= split(map([op(prog::Changes::removed)], op, 1), 
              x -> op(x,1)=hold(Sysvar) or op(x,1)=hold(System));
    p("const char* gRemovedSys[] = {\n");
    for i in l[1] do
        p("\""); p(i[2]); p("\",\n");
    end;
    p("NULL\n};\n");
    p("const char* gRemovedPack[] = {\n");
    for i in l[2] do
        p("\""); p(i[1]); p("::"); p(i[2]); p("\",\n");
    end;
    p("NULL\n};\n");
    // new names
    p("const char* gNewLib[] = {\n");
    for i in prog::Changes::newLibs do
        p("\""); p(i); p("\",\n");
    end;
    p("NULL\n};\n");
    p("const char* gNewSys[] = {\n");
    for i in prog::Changes::newFunctions[System] do
        p("\""); p(i); p("\",\n");
    end;
    p("NULL\n};\n");
    l:= select([op(prog::Changes::newFunctions)], x -> op(x,1) <> System);
    p("const char* gNewPack[] = {\n");
    for i in l do
        for j in op(i,2) do
            p("\""); p(op(i,1)); p("::"); p(j); p("\",\n");
        end;
    end;
    p("NULL\n};\n");
    // changed names
    l:= split([op(prog::Changes::renamed)],
              x -> op(x,[1,1])=hold(Sysvar) or op(x,[1,1])=hold(System));
    p("const SubsInfo gRenamedSys[] = {\n");
    for i in l[1] do
        p("{\""); p(op(i,[1,2])); p("\", \""); pp(op(i,2)); p("\"},\n");
    end;
    p("{NULL, NULL}\n};\n");
    p("const SubsInfo gRenamedPack[] = {\n");
    for i in l[2] do
        p("{\""); pp(op(i,1)); p("\", \""); pp(op(i,2)); p("\"},\n");
    end;
    p("{NULL, NULL}\n};\n");
    p("const SubsInfo gRenamedLib[] = {\n");
    for i in [op(prog::Changes::renamedLibs)] do
        p("{\""); p(op(i,[1,1])); p("\", \""); p(op(i,[2,1])); p("\"},\n");
    end;
    p("{NULL, NULL}\n};\n");
    fclose(fd);
end_proc:

