
/*++
prog::testinit -- init testing

prog::testinit([f])

f - file name (default is "test")

Opens the test result file 'f.res'. If the variable 'TESTPATH' has a
value, it is used as a prefix for the path name.
++*/

prog::testinit:=
  proc(f, OPT = hold(None))
    local UOPT, LogFile, inp, S, NM;
  begin
    if args(0) = 0 then
      f:= "test"
    elif args(0) = 1 then
      if domtype(f) <> DOM_STRING then
        error("illegal file name prefix")
      end_if
    else error("wrong no of args") end_if;
    if TESTPATH <> hold(TESTPATH) then
      f:= TESTPATH.f
    end_if;
    sysassign(prog::TestFile, fopen(Text, f.".res", Write));

    // older results
    UOPT:= text2expr(Pref::userOptions());
    // MuPAD Option: -U TESTPATH=\"PROP/TEST/property.err\",NAME=\"galois\"
    if domtype(UOPT) <> DOM_NULL then
      // gesetzte 'Useroption'
      if nops(UOPT) = 2 then
        if type(op(UOPT, 1)) = "_equal" and op(UOPT, [1, 1]) = hold(TESTPATH) then
          // Dateiname relativ zum LIBPATH
          if type(op(UOPT, 2)) = "_equal" and op(UOPT, [2, 1]) = hold(NAME) then
            NM:= op(UOPT, [2, 2])
          else
            NM:= "noname"
          end_if;
          S:= table("Name" = NM);
          UOPT:= op(UOPT, [1, 2]); // Name der Log-Datei
          if traperror((LogFile:= fopen(UOPT))) = 0 and LogFile <> FAIL then
            while (inp:= finput(LogFile)) <> null() do
              if op(inp, 1) = "Time" then
                // save global timing information
                S["Time", op(inp, 2)]:= op(inp, 3)
              else
                S[inp]:= FALSE
              end_if
            end_while;
            fclose(LogFile);
          else
            userinfo(1, "Yesterdays log-file not found")
          end_if;
        end_if
      end_if;
      // write older results
      if contains(S, ("Time", NM)) then
        S["Time", 0]:= S["Time", NM]; // vorige Timings
        delete S["Time", NM]
      else
        S["Time", 0]:= 0
      end_if;
      sysassign(prog::TestOldErrors, S);
      sysassign(prog::TestErrors, table());
      sysassign(prog::TestLogFile, UOPT)
    end_if;
    
    if prog::TestFile = FAIL then
      error("could not open results file")
    end_if;
    sysassign(prog::TestNo, 1);
    sysassign(prog::TestFunc, FAIL);
    sysassign(prog::TestInitTime, time()); // gesamte Zeit
    sysassign(prog::TestTime, 0); // Rechenzeit
    sysassign(prog::TestErrorCount, 0);
    sysassign(prog::TestCount, 0);
    sysassign(prog::TestQuiet, bool(OPT = hold(Quiet)));
    null()
  end_proc:

/*++
prog::testexit -- exit testing

prog::testexit()

Close test results file.
++*/

prog::TestFile:= FAIL:
prog::TestLogFile:= FAIL:
prog::TestOldErrors:= FAIL:
prog::TestErrors:= FAIL:
prog::TestErrorCount:= 0:
prog::TestQuiet:= FALSE:

prog::testexit:=
  proc()
    local LogFile, TIME, k, NAME, F;
  begin
    F:= X -> stringlib::formatf(float(X), 2);
    if domtype(prog::TestFile) = DOM_INT then
      fclose(prog::TestFile)
    end_if;
    // Meldung ausgeben
    if not prog::TestQuiet then
      TIME:= time() - prog::TestInitTime;
      //Info: property::assume - 26 examples,  2 errors, 12 warnings, time: 18.10/23.45 s
      fprint(Unquoted, 0, "Info: ",
             expr2text(prog::TestCount),
             " test"._if(prog::TestCount = 1, "", "s").", ",
             expr2text(prog::TestErrorCount),
             " error"._if(prog::TestErrorCount = 1, "", "s").", ",
             " time: ", F(float(prog::TestTime)/1000), " /", F(float(TIME)/1000), " s");
    end_if;
    // Log-Datei schreiben
    if prog::TestLogFile <> FAIL then
      fprint(Unquoted, 0, "Info: All tests ", expr2text(prog::TestCount), " [prog::testexit]");
      NAME:= prog::TestOldErrors["Name"];
      if traperror((LogFile:= fopen(Text, prog::TestLogFile, Write))) = 0
         and LogFile <> FAIL then
        for k in prog::TestErrors do
          if op(k, 2) = TRUE then
            fprint(LogFile, op(k, 1))
          end_if
        end_for;
        for k in prog::TestOldErrors do
          if op(op(k, 1), 1) = "Time" and op(k, [1, 2]) <> 0 then
            fprint(LogFile, [op(op(k, 1)), op(k, 2)])
          end_if
        end_for;
        fprint(LogFile, ["Time", NAME, prog::TestTime]);
        fclose(LogFile)
      else
        warning("could not open log file ".prog::TestLogFile)
      end_if;
      // Time check
      if prog::TestOldErrors["Time", 0] > 0 then // alte timings existieren
        if abs((prog::TestOldErrors["Time", 0] - prog::TestTime)/prog::TestOldErrors["Time", 0]) > .9 then
          warning("Time ".F(float((prog::TestTime - prog::TestOldErrors["Time", 0])/
                                  prog::TestOldErrors["Time", 0]*100)))
        else
          userinfo(1, "Time changing ".F(float((prog::TestTime - prog::TestOldErrors["Time", 0])/
                                               prog::TestOldErrors["Time", 0]*100))
                                      ."%")
        end_if
      end_if
    end_if;
    sysassign(prog::TestFile, FAIL);
    sysassign(prog::TestLogFile, FAIL);
    sysassign(prog::TestNo, 1);
    sysassign(prog::TestOldErrors, FAIL);
    sysassign(prog::TestFunc, FAIL);
    sysassign(prog::TestErrorCount, 0);
    sysassign(prog::TestCount, 0);
    null()
  end_proc:

/*++
prog::testfunc -- init testing of function

prog::testfunc(f)

f - expression

prog::testfunc(f) initializes the testing of function f. The argument
is not evaluated. If the test result file has not been opened it is
opened by prog::testfunc. If the variable 'TESTPATH' has a
value, it is used as a prefix for the path name.
++*/

prog::testfunc:=
  proc(FUNC)
    option hold;
  begin
    if args(0) <> 1 then
      error("wrong no of args")
    end_if;
    if prog::TestFile = FAIL then
      prog::testinit(expr2text(FUNC))
    end_if;
    sysassign(prog::TestNo, 1);
    sysassign(prog::TestFunc, FUNC);
    sysassign(prog::TestFuncName, expr2text(FUNC));
    null()
  end_proc:

/*++
prog::test -- test function

prog::test()
prog::test(a)
prog::test(a, b)
prog::test(a, b, c)

a, b - expressions
c    - string

prog::test prints - depending upon the parameters - the current function
and test number onto the test results file 'test.res'. This signals
an error in the current test.

prog::test() prints the current function and test number unconditional.

prog::test(a) prints the current function and test number if a is not
equal to TRUE or if an error occures during the evaluation of a.

prog::test(a, b) prints the current function and test number if a is not
equal to b (in the sense of _equal) of if an error occures during the
evaluation of a or b.

prog::test(a,b,c) works like prog::test(a, b) but append string c to 
the output.
++*/

prog::test:=
  proc(a, b = TRUE, c = "")
    option hold;
    local r, ERR, i, lprint, TE, REG, ARGM, PREF, IND, WARN, TIME;
    save PRETTYPRINT, TEXTWIDTH;
  begin
    // evaluate the third argument
    c := context(c):
    lprint:=
      proc(COND, MSG)
        name prog::test::lprint;
      begin
        if bool(COND) then // error!
          if domtype(prog::TestFile) = DOM_INT then
            fprint(Unquoted, prog::TestFile, prog::TestFunc, " ",
                   prog::TestNo, ":\t", args(i)$i=2..args(0))
          end_if;
          IND:= "";
          if REG then
            IND:= "*reg*"
          elif prog::TestOldErrors <> FAIL and
               not contains(prog::TestOldErrors, [prog::TestFunc, expr2text(MSG)]) then
            IND:= "*new*"
          end_if;
  
          fprint(Unquoted, 0, "Error", IND, " in test '", prog::TestFunc, "' ",
                 prog::TestNo, ":\t", args(i)$i=2..args(0));
          if prog::TestErrors <> FAIL then
            sysassign(prog::TestErrors[[prog::TestFunc, expr2text(MSG)]], TRUE)
          end_if;
          sysassign(prog::TestErrorCount, prog::TestErrorCount + 1)
        else // no error
//           if prog::TestOldErrors <> FAIL and
//              contains(prog::TestOldErrors, [prog::TestFunc, expr2text(MSG)]) then
            
//             fprint(0, "Info: Error '".expr2text(prog::TestErrorCount)."' corrected [prog::test]")
//           end_if
        end_if
      end_proc:

    WARN:= warning;
    sysassign(warning,
              proc()
              begin
                if not hastype(map(prog::NOWARN,
                                   (X, Y) -> stringlib::match(Y, "[".
                                                              _if(type(X) = DOM_STRING, X, expr2text(X)).
                                                              "]"), args()),
                               DOM_INT) then
                  WARN(args())
                end_if
              end_proc);

    if domtype(prog::TestFile) <> DOM_INT then
      //warning("results file has not been opened")
    end_if;
    if prog::TestFunc = FAIL then
      //warning("no function to test");
      sysassign(prog::TestFunc, "interactive");
      sysassign(prog::TestNo, 0)
    end_if;
    // Tests global zahlen
    sysassign(prog::TestCount, prog::TestCount + 1);
    
    //PRETTYPRINT:= FALSE;
    //TEXTWIDTH:= 2048;
    PREF:= Pref::typeCheck(Always), testargs(TRUE);

    ARGM:= 0; REG:= FALSE;
    if bool(args(args(0)) = hold(Quiet)) then
      c:= op(["", c], args(0) - 2);
      if args(0) < 3 then delete b end_if;
      REG:= TRUE;
      ARGM:= 1
    end_if;

    if type(args(args(0) - ARGM)) = hold(NoWarning) then
      sysassign(prog::NOWARN, [prog::TestFuncName]);
      ARGM:= ARGM + 1
    elif type(args(args(0) - ARGM)) = "_equal"
         and op(args(args(0) - ARGM), 1) = hold(NoWarning) then
      sysassign(prog::NOWARN, op(args(args(0) - ARGM), 2));
      if not type(prog::NOWARN) in {DOM_SET, DOM_LIST} then
        sysassign(prog::NOWARN, [prog::NOWARN])
      end_if;
      ARGM:= ARGM + 1
    end_if;
    
    if type(b) = "_equal" and op(b, 1) = hold(TrapError) then
      TE:= op(b, 2);
    else
      TE:= 0
    end_if;
    
    TIME:= 0;
    r:= FAIL;
    userinfo(Text, 1, "testing '", prog::TestFunc, "' ", prog::TestNo);
    case args(0) - ARGM
      of 0 do
        print(prog::TestNo);
        sysassign(prog::TestNo, prog::TestNo - 1);
        break
      of 1 do
        userinfo(Text, 2,"  for ",a," = ",_if(TE > 0, "Error No. ".expr2text(TE), "TRUE"));
        ERR:= traperror((TIME:= time((r:= context(a)))));
        lprint(ERR <> TE or (ERR = 0 and r <> TRUE),
               hold(_equal)(a, TRUE), " [", _if(ERR > 0, ("TrapError ", ERR), ("<> ", r)), "]");
        break
      of 2 do
        userinfo(Text, 2,"  for ",a," = ",_if(TE > 0, "Error No. ".expr2text(TE), b));
        ERR:= traperror((TIME:= time((r:= context(a)))));
        b:= context(b);
        lprint(ERR <> TE or (ERR = 0 and r <> b),
               hold(_equal)(a, b), " [", _if(ERR > 0, ("TrapError ", ERR), ("<> ", r)), "]");
        break
      of 3 do
        userinfo(Text, 2,"  for ",a," = ",_if(TE > 0,  "Error No. ".expr2text(TE), b));
        ERR:= traperror((TIME:= time((r:= context(a)))));
        b:= context(b);
        lprint(ERR <> TE or (ERR = 0 and r <> b),
               hold(_equal)(a, b),  " [", _if(ERR > 0, ("TrapError ", ERR), ("<> ", r)), "]", " (", c, ")");
        break
      otherwise error("wrong no of args");
    end_case;
    sysassign(prog::TestNo, prog::TestNo + 1);
    sysassign(prog::TestTime, prog::TestTime + TIME);
    Pref::typeCheck(op(PREF, 1));
    testargs(op(PREF, 2));
    sysdelete(prog::NOWARN);
    sysassign(warning, WARN);
    null()
  end_proc:

// end of file 
