      
/*(
   Divs - domain of divisors

    1, 0
   2     - teilbar durch 2^1

    1, 3
   2     - teilbar durch 2^1, nicht teilbar durch 2^3

)*/

alias(Divs = (property::Divs)):
alias(Ar(x) = [5, x]):
alias(AR = 5):
alias(Arg(x, y) = op(extop(x, 5), y)):
alias(NArgs(x) = nops(extop(x, 5))):
alias(Args(x) = op(extop(x, 5))):
alias(ARG(x) = op(extop(x, 5))):
alias(EX(x) = not(op(extop(x, 6)))):

Divs:= newDomain("property::Divs"):
Divs::info:= "Domain of divisors":
Divs::domtype:= ()->(Type):
Divs::type:= "Divs":
Divs::prop:= TRUE:
Divs::propdom:= slot(property, "Divs"):
Divs::autoload:= TRUE:
Divs::testtype:= Type::testtype:

Divs::noTrace:= {hold(print), hold(testtype), hold(domtype)}:

Divs::ax:= []:

Divs::new:=
	proc(XX = 1, Opt = null())
	begin
	  if property::Fail(XX) then
	    return(property::Null)
	  end_if;
	  if args(0) > 0 then
	    new(Divs, "Divs", FALSE, FALSE, FALSE,
		       [table(Divs::stein(XX, Opt))], bool(Opt = hold(Exact)))
	  else
	    property::Null
	  end_if
	end_proc:

//  

Divs::stein:=
	proc(XX, Opt = null())
	  local FACT, RET, i;
	begin
	  if domtype(XX) = DOM_INT then
	    FACT:= stdlib::ifactor(XX)
	  elif type(XX) = "_not" then
	    RET:= Divs::stein(op(XX));
	    for i from 2 to nops(RET) do
	      RET:= subsop(RET, [i, 2] = [0, op(RET, [i, 2, 1])])
	    end_for;
	    return(RET)
	  else
	    FACT:= factor(XX)
	  end_if;
	  if abs(op(FACT, 1)) > 1 then
	    RET:= Divs::stein(op(FACT, 1))
	  else
	    RET:= (0 = op(FACT, 1))
	  end_if;
	  for i from 1 to floor((nops(FACT)-1)/2) do
//	    RET:= RET, op(FACT, 2*i) = Dom::Interval([1], [op(FACT, 2*i+1)])
	    RET:= RET, op(FACT, 2*i) = [op(FACT, 2*i+1),
					_if(Opt = hold(Exact), op(FACT, 2*i+1) + 1, 0)]
	  end_for;
	  if type(RET) = "_equal" then
	    RET, 1 = [1, 0]
	  else
	    RET
	  end_if
	end_proc:

// TRUE  - alle Primteiler von d2 auch von d1 
// FALSE - ein Primteiler von d2 NICHT-Printeiler von d1 
Divs::is:=
	proc(d1, d2)
	  local ret, ret1, di, dj, k, TT;
	begin
          if type(d1) = DOM_INT then d1:= DOM_INT::Divs(d1) end_if;
          if type(d1) <> "Divs" or type(d2) <> "Divs" then
	    return(UNKNOWN)
	  end_if;
	  ret:= TRUE;
	  di:= map(op(ARG(d1)), op, 1);
	  dj:= map(op(ARG(d2)), op, 1);
	  if (TT:= {op(di)} intersect {op(dj)} minus {0}) = {} then
	    if Divs::_sign(d2) = 0 then
	      return(UNKNOWN and EX(d1))
	    elif Divs::_sign(d2) < 0 then
	      return(UNKNOWN or not(EX(d1)))
	    else
	      return(UNKNOWN)
	    end_if
	  end_if;
	  if {op(dj)} minus {op(di)} <> {} then
	    ret1:= UNKNOWN
	  else
	    ret1:= UNKNOWN;
	    for k in TT do
	      ret1:= TRUE;
	      ret:= ret and ((bool(op(ARG(d1)[k], 1) - op(ARG(d2)[k], 1) >= 0)
			      and bool(op(ARG(d1)[k], 1)*op(ARG(d2)[k], 1) <> 0))
			     or (bool(op(ARG(d2)[k], 2) - op(ARG(d1)[k], 2) >= 0)
				 and bool(op(ARG(d2)[k], 2)*op(ARG(d1)[k], 2) <> 0)));
	      if ret <> TRUE then
		break
	      end_if
	    end_for
	  end_if;
	  //print("0", EX(d1), EX(d2));
	  //print("1", ret1, ret);
	  if (ret1 and ret) = TRUE then return(TRUE) end_if;

	  ret1:= UNKNOWN; ret:= TRUE;
	  for k in TT do
	    ret1:= TRUE;
	    ret:= ret and ((bool(op(ARG(d2)[k], 1) - op(ARG(d1)[k], 2) >= 0)
			    and bool(op(ARG(d2)[k], 1)*op(ARG(d1)[k], 2) <> 0))
			   or (bool(op(ARG(d1)[k], 1) - op(ARG(d2)[k], 2) >= 0)
			       and bool(op(ARG(d1)[k], 1)*op(ARG(d2)[k], 2) <> 0)));
	    if ret <> TRUE then
	      break
	    end_if
	  end_for;
	  //print("2", ret1, ret);
	  if (ret1 and ret) = TRUE then return(FALSE) end_if;
	  
	  UNKNOWN and EX(d1)
	end_proc:

Divs::sign:=
	proc(d1)
	begin
	  ARG(d1)[0]
	end_proc:

// Teiler / not Teiler 
Divs::_sign:=
	proc(d1)
	  local ret, di, k, i, TT;
	begin
	  di:= map(op(ARG(d1)), op, 1);
	  ret:= ARG(d1);
	  TT:= {op(di)} minus {0};
	  i:= sign(op(ret[extop(TT, 1)], 1)) - sign(op(ret[extop(TT, 1)], 2));
	  for k in TT do
	    if op(ret[k], 1) <> 0 then
	      if i = -i  then return(0) else i:= 1 end_if
	    end_if;
	    if op(ret[k], 2) <> 0 then
	      if i = -i then return(0) else i:= -1 end_if
	    end_if
	  end_for;
	  i
	end_proc:

// Teiler entfernen (_divide) 
Divs::Ipl:=
	proc(r1, r2)
	  local m;
	begin
	  if op(r1, 2) = 0 then
	    if op(r2, 2) = 0 then
	      if (m:= op(r1, 1) - op(r2, 1)) >= 0 then
		return([m, 0])
	      end_if
	    end_if
	  end_if;
	  [0, 0]
	end_proc:

// Teiler entfernen (_divide) 
Divs::Imi:=
	proc(r1, r2)
	  local m;
	begin
	  if op(r1, 2) = 0 then
	    if op(r2, 2) = 0 then
	      if (m:= op(r1, 1) - op(r2, 1)) >= 0 then
		return([m, 0])
	      end_if
	    end_if
	  end_if;
	  [0, 0]
	end_proc:

// gcd/lcm 
Divs::Iun:=
	proc(r1, r2, PR = max)
	  local m, n;
	begin
	  if op(r1, 2) = 0 then                       // [x,0]         gcd         lcm
	    if op(r2, 2) = 0 then                     // [x,0] [x,0]   [min, 0]    [max, 0]
	      return([PR(op(r1, 1), op(r2, 1)), 0])
	    elif op(r2, 1) = 0 then                   // [x,0] [0,x]   [0, x]      [x, 0]
	      if op(r1, 1) < op(r2, 2) then                  
		return([op(r1, 1), op(r2, 2)])
	      else                                    //
		return([0, 0])
	      end_if
	    else                                      // [x,0] [x,y]   [0, y]      [max, 0]
	      m:= PR(op(r1, 1), op(r2, 1));
	      if m < op(r2, 2) then
		return([m, op(r2, 2)])
	      else                                    //
		return([0, 0])
	      end_if
	    end_if
	  elif op(r1, 1) = 0 then                     // [0,x]
	    if op(r2, 2) = 0 then                     // [0,x] [x,0]   [0, x]      [x, 0]
	      if op(r2, 1) < op(r1, 2) then
		return([op(r2, 1), op(r1, 2)])
	      else                                    // 
		return([0, 0])
	      end_if
	    elif op(r2, 1) = 0 then                   // [0,x] [0,x]   [0, min]    [0, max]
	      return([0, PR(op(r1, 1), op(r2, 1))])
	    else                                      // [0,x] [x,y]   [0, min]    [x, 0]
	      m:= PR(op(r1, 2), op(r2, 2));
	      if op(r2, 1) < m then
		return([op(r2, 1), m])
	      else                                    //
		return([0, 0])
	      end_if
	    end_if
	  else                                        // [x,y]
	    if op(r2, 2) = 0 then                     // [x,y] [x,0]   [0, y]      [max, 0]
	      m:= PR(op(r1, 1), op(r2, 1));
	      if m < op(r1, 2) then
		return([m, op(r1, 2)])
	      else                                    //
		return([0, 0])
	      end_if
	    elif op(r2, 1) = 0 then                   // [x,y] [0,x]   [0, min]    [x, 0]
	      m:= PR(op(r1, 2), op(r2, 2));
	      if op(r1, 1) < m then
		return([op(r1, 1), m])
	      else                                    //
		return([0, 0])
	      end_if
	    else                                      // [x,y] [x,y]   [min, max]    [max, min]
	      m:= PR(op(r1, 1), op(r2, 1));
	      n:= PR(op(r1, 2), op(r2, 2));
	      if m < n then
		return([m, n])
	      else                                    //
		return([0, 0])
	      end_if
	    end_if
	  end_if
	end_proc:

// Gesamt-Wert 
Divs::val:=
	proc(XX)
	  local k, n, r, rr;
	begin
	  n:= {map(op(ARG(XX)), op, 1)} minus {0};
	  r:= ARG(XX)[0];
	  for k in n do
	    r:= r*k^(op(ARG(XX)[k], 1))
	  end_for;
	  rr:= 1;
	  for k in n do
	    rr:= rr*k^(op(ARG(XX)[k], 2))
	  end_for;
	  [r, rr]
	end_proc:

// Gesamt-Wert 
Divs::factor:=
	proc(XX)
	  local k, n, r, rr;
	begin
	  n:= {map(op(ARG(XX)), op, 1)} minus {0};
	  r:= ARG(XX)[0]; rr:= null();
	  for k in n do
	    if op(ARG(XX)[k], 1) > 0 then
	      r:= r, k, op(ARG(XX)[k], 1)
	    end_if
	  end_for;
	  for k in n do
	    if op(ARG(XX)[k], 2) > 0 then
	      rr:= rr, k, not(op(ARG(XX)[k], 2))
	    end_if
	  end_for;
	  [r, rr]
	end_proc:

// gcd mit not-Operanden 
Divs::gcd:=
	proc(XX, YY = null())
	begin
	  if args(0) < 2 then return(XX) end_if;
	  if XX = 0 then return(YY) end_if;
	  if args(0) > 2 then
	    return(Divs::gcd(args(1), Divs::gcd(args(2..args(0)))))
	  else
	    return(/*Divs(op*/(Divs::_gcd(args(1), args(2)))/*)*/)
	  end_if;
	end_proc:

Divs::Igcd:=
	proc(r1, r2)
	begin
	  if op(r1, 1) = 0 then
	    if op(r2, 1) = 0 then
	      [0, min(op(r1, 2), op(r2, 2))]
	    else
	      [0, op(r1, 2)]
	    end_if
	  else
	    if op(r2, 1) = 0 then
	      [0, op(r2, 2)]
	    else
	      [min(op(r1, 1), op(r2, 1)), 0]
	    end_if
	  end_if
	end_proc:

Divs::_gcd:=
	proc(d1, d2)
	  local ret, di, dj, k, TT;
	begin
	  if type(d2) <> "Divs" then
	    return(Divs(1))
	  end_if;
	  di:= map(op(ARG(d1)), op, 1);
	  dj:= map(op(ARG(d2)), op, 1);
	  if (TT:= {op(di)} intersect {op(dj)} minus {0}) = {} then
	    if Divs::_sign(d1) + Divs::_sign(d2) = -2 then
	      return(hold(_and)(d1, d2))
	    end_if
	  end_if;
	  ret:= table();
	  for k in TT do
	    ret[k]:= Divs::Igcd(ARG(d1)[k], ARG(d2)[k])
	  end_for;
	  for k in ({op(di)} minus TT minus {0}) do
	    if op(ARG(d1)[k], 2) > 0 then
	      ret[k]:= subsop(ARG(d1)[k], 1 = 0)
	    end_if
	  end_for;
	  for k in ({op(dj)} minus TT minus {0}) do
	    if op(ARG(d2)[k], 2) > 0 then
	      ret[k]:= subsop(ARG(d2)[k], 1 = 0)
	    end_if
	  end_for;
	  ret[0]:= ARG(d1)[0]*ARG(d2)[0];
	  if nops(ret) = 1 then
	    Divs(1)
	  else
	    return(subsop(d1, 1 = hold(Divs), AR = [ret]))
	  end_if
	end_proc:

// lcm mit not Operanden 
Divs::lcm:=
	proc(XX)
	begin
	  if args(0) < 2 then return(XX) end_if;
	  if args(0) > 2 then
	    return(Divs::lcm(args(1), Divs::lcm(args(2..args(0)))))
	  else
	    return(/*Divs(op*/(Divs::_lcm(args(1), args(2)))/*)*/)
	  end_if;
	end_proc:

Divs::Ilcm:=
	proc(r1, r2)
	begin
	  if op(r1, 1) = 0 then
	    if op(r2, 1) = 0 then
	      [0, max(op(r1, 2), op(r2, 2))]
	    else
	      [op(r2, 1), 0]
	    end_if
	  else
	    if op(r2, 1) = 0 then
	      [op(r1, 1), 0]
	    else
	      [max(op(r1, 1), op(r2, 1)), 0]
	    end_if
	  end_if
	end_proc:

Divs::_lcm:=
	proc(d1, d2)
	  local ret, di, dj, k, TT;
	begin
	  if type(d2) <> "Divs" then
	    return(Divs(1))
	  end_if;
	  di:= map(op(ARG(d1)), op, 1);
	  dj:= map(op(ARG(d2)), op, 1);
	  if Divs::_sign(d1) + Divs::_sign(d2) = -2 then
	    if (nops(ARG(d1)) > 2 or nops(ARG(d2)) > 2) and (di <> dj) then
	      return(Divs(1))
	    end_if
	  end_if;
	  if (TT:= {op(di)} intersect {op(dj)} minus {0}) = {} then
	    ret:= ARG(d1);
	    for k in {op(dj)} do
	      ret[k]:= ARG(d2)[k]
	    end_for;
	    ret[0]:= ARG(d1)[0]*ARG(d2)[0];
	    return(subsop(d1, 1 = hold(Divs), AR = [ret]))
	  end_if;
	  ret:= table();
	  for k in TT do
	    ret[k]:= Divs::Ilcm(ARG(d1)[k], ARG(d2)[k])
	  end_for;
	  for k in ({op(di)} minus TT minus {0}) do
	    if op(ARG(d1)[k], 1) > 0 then
	      ret[k]:= subsop(ARG(d1)[k], 2 = 0)
	    end_if;
	  end_for;
	  for k in ({op(dj)} minus TT minus {0}) do
	    if op(ARG(d2)[k], 1) > 0 then
	      ret[k]:= subsop(ARG(d2)[k], 2 = 0)
	    end_if
	  end_for;
	  ret[0]:= ARG(d1)[0]*ARG(d2)[0];
	  return(subsop(d1, 1 = hold(Divs), AR = [ret]));
	end_proc:

Divs::_or:=
	proc()
	begin
	  property::_Op
	  (Divs, _or,
	   proc(d1, d2)
	     local ret, di, dj, TT, k, n;
	   begin
	     if (di:= Divs::_sign(d1)) + (dj:= Divs::_sign(d2)) = -2 then // pos
	       return(Divs::_lcm(d1, d2))
	     elif di + dj = -2 then
	       di:= map(op(ARG(d1)), op, 1);
	       dj:= map(op(ARG(d2)), op, 1);
	       if di = dj then // gleiche not
		 return(Divs::_lcm(d1, d2))
	       end_if;
	     end_if;
	     di:= map(op(ARG(d1)), op, 1);
	     dj:= map(op(ARG(d2)), op, 1);
	     if (TT:= {op(di)} intersect {op(dj)} minus {0}) = {} then
	       ret:= ARG(d1);
	       for k in {op(dj)} do
	         ret[k]:= ARG(d2)[k]
	       end_for;
	       ret[0]:= ARG(d1)[0]*ARG(d2)[0];
	       return(subsop(d1, 1 = hold(Divs), AR = [ret]))
	     end_if;
	     ret:= table();
	     for k in TT do
	       if (n:= Divs::Ian(ARG(d1)[k], ARG(d2)[k])) <> FAIL then
		 ret[k]:= n
	       end_if
	     end_for;
	     for k in ({op(di)} minus TT minus {0}) do
	       ret[k]:= ARG(d1)[k]
	     end_for;
	     for k in ({op(dj)} minus TT minus {0}) do
	       ret[k]:= ARG(d2)[k]
	     end_for;
	     if nops(ret) < 2 then return(property::Null) end_if; // Widerspruch
	     ret[0]:= ARG(d1)[0]*ARG(d2)[0];
	     return(subsop(d1, 1 = hold(Divs), AR = [ret]));
	   end_proc, args())
	end_proc:

Divs::Ian:=
	proc(r1, r2)
	begin
	  r1:= [max(op(r1, 1), op(r2, 1)), min(op(r1, 2), op(r2, 2))];
	  if op(r1, 1) >= op(r1, 2) then
	    [op(r1, 1), 0]
	  else
	    r1
	  end_if
	end_proc:

Divs::_and:=
	proc()
	begin
	  property::_Op
	  (Divs, _and,
	   proc(d1, d2)
	     local ret, di, dj, TT, k, n;
	   begin
	     if (di:= Divs::_sign(d1)) + (dj:= Divs::_sign(d2)) = 2 then // pos
	       return(Divs::_lcm(d1, d2))
	     elif di + dj = -2 then
	       di:= map(op(ARG(d1)), op, 1);
	       dj:= map(op(ARG(d2)), op, 1);
	       if di = dj then // gleiche not
		 return(Divs::_lcm(d1, d2))
	       end_if;
	     end_if;
	     di:= map(op(ARG(d1)), op, 1);
	     dj:= map(op(ARG(d2)), op, 1);
	     if (TT:= {op(di)} intersect {op(dj)} minus {0}) = {} then
	       ret:= ARG(d1);
	       for k in {op(dj)} do
	         ret[k]:= ARG(d2)[k]
	       end_for;
	       ret[0]:= ARG(d1)[0]*ARG(d2)[0];
	       return(subsop(d1, 1 = hold(Divs), AR = [ret]))
	     end_if;
	     ret:= table();
	     for k in TT do
	       if (n:= Divs::Ian(ARG(d1)[k], ARG(d2)[k])) <> FAIL then
		 ret[k]:= n
	       end_if
	     end_for;
	     for k in ({op(di)} minus TT minus {0}) do
	       ret[k]:= ARG(d1)[k]
	     end_for;
	     for k in ({op(dj)} minus TT minus {0}) do
	       ret[k]:= ARG(d2)[k]
	     end_for;
	     if nops(ret) < 2 then return(property::Null) end_if; // Widerspruch
	     ret[0]:= ARG(d1)[0]*ARG(d2)[0];
	     return(subsop(d1, 1 = hold(Divs), AR = [ret]));
	   end_proc, args())
	end_proc:

// Invertieren (fur not) 
Divs::Iin:=
	proc(r1)
	begin
	  if op(r1, 1) = 0 then
	    [op(r1, 2), 0]
	  elif op(r1, 2) = 0 then
	    [0, op(r1, 1)]
	  else
	    FAIL
	  end_if
	end_proc:

// not 
Divs::_not:=
	proc(d1)
	  local k, di, ret, n;
	begin
	  di:= {map(op(ARG(d1)), op, 1)} minus {0};
	  ret:= null();
	  for k in di do
	    if (n:= Divs::Iin(ARG(d1)[k])) <> FAIL then
	      ret:= ret, subsop(d1, 1 = hold(Divs), AR = [table(0 = ARG(d1)[0], k = n)])
	    end_if
	  end_for;
	  hold(_or)(ret)
	end_proc:

Divs::_plus:=
	proc()
	begin
	  property::_Op
	  (Divs, _plus,
	   proc(d1, d2)
	     local s;
	   begin
	     if Divs::_sign(d1) + Divs::_sign(d2) = 2 then
	       Divs::_gcd(d1, d2)
	     elif (s:= Divs::sign(d1)) = Divs::sign(d2) then
	       s*Type::PosInt
	     else
	       Type::Integer
	     end_if
	   end_proc, args())
	end_proc:

Divs::_subtract:=
	proc()
	begin
	  property::_Op(Divs, _subtract,
	    proc(d1, d2)
	    begin
	      Divs::_plus(d1, Divs::_negate(d2))
	    end_proc, args())
	end_proc:

Divs::_negate:=
	proc(d1)
	  local ret;
	begin
	  ret:= ARG(d1);
	  ret[0]:= -ret[0];
	  subsop(d1, AR = [ret])
	end_proc:

Divs::Imu:=
	proc(r1, r2)
	begin
	  if op(r1, 1) = 0 then
	    if op(r2, 1) = 0 then
	      [0, op(r1, 2) + op(r2, 2) - 1]
	    else
	      [op(r2, 1), 0]
	    end_if
	  else
	    if op(r2, 1) = 0 then
	      [op(r1, 1), 0]
	    else
	      [(op(r1, 1) + op(r2, 1)), 0]
	    end_if
	  end_if
	end_proc:

Divs::_mult:=
	proc()
	begin
	  property::_Op
	  (Divs, _mult,
	   proc(d1, d2)
	     local di, ret, ret1, k;
	   begin
	     if type(d2) <> "Divs" then
	       if property::_is(d2, Type::Integer) = TRUE then
		 return(d1)
	       end_if
	     end_if;
	     di:= {map(op(ARG(d2)), op, 1)} minus {0};
	     ret:= ARG(d1); ret1:= ARG(d2);
	     for k in di do
	       if contains(ret, k) then
		 ret[k]:= Divs::Imu(ret[k], ARG(d2)[k])
	       else
		 ret[k]:= ret1[k]
	       end_if
	     end_for;
	     return(subsop(d1, AR = [ret]))
	   end_proc, args())
	end_proc:
	
Divs::_divide:=
	proc()
	begin
	  property::_Op(Divs, _divide,
	    proc(d1, d2)
	      local ret, dj, k, i;
	    begin
	      if type(d2) <> "Divs" then
	        return(Divs(1))
	      end_if;
	      dj:= map(op(ARG(d2)), op, 1);
	      if {op(dj)} minus {op(map(op(ARG(d1)), op, 1))} <> {} then
		return(Divs())
	      end_if;
	      ret:= ARG(d1);
	      for k in ({op(dj)} minus {0}) do
		if (i:= Divs::Imi(ARG(d1)[k], ARG(d2)[k])) <> [0, 0] then
	          ret[k]:= i
		else
		  delete ret[k]
		end_if
	      end_for;
	      return(subsop(d1, AR = [ret]))
	    end_proc, args())
	end_proc:
	
Divs::_invert:=
	proc(XX)
	begin
	  Divs::sign(XX)*Type::PosRat
	end_proc:

Divs::_power:=
	proc(d1, pw)
	  local ret, di, k;
	begin
	  if domtype(d1) = DOM_INT then
	    if abs(d1) = 1 then
	      if Divs::is(pw, Type::Even) = TRUE then
		return(abs(d1))
	      elif Divs::is(pw, Type::Odd) = TRUE then
		return(d1)
	      end_if
	    end_if
	  end_if;

	  if domtype(pw) = DOM_INT then
	    di:= {map(op(ARG(d1)), op, 1)} minus {0};
	    ret:= ARG(d1);
	    if pw > 0 then
	      for k in di do
	        ret[k]:= [_if(op(ret[k], 1) > 0, op(ret[k], 1) + pw, 0),
			  _if(op(ret[k], 2) > 0, op(ret[k], 2) + pw, 0)]
	      end_for;
	      subsop(d1, AR = [ret]);
	      return(subsop(d1, AR = [ret]))
	    elif pw < 0 then
	      Divs::_invert(d1)
	    else
	      return(Divs(1))
	    end_if
	  end_if;
	  property::_Op
	  (Divs, _power,
	   proc(d1, d2)
	   begin
	     if Divs::sign(d1) = -1 then
	       if Divs::is(d2, Type::Even) = TRUE then
		 return(Divs::abs(d1))
	       end_if
	     end_if;
	     FAIL
	   end_proc, args())
	end_proc:

// Methoden
Divs::abs:=
	proc(d1)
	  local ret;
	begin
	  ret:= ARG(d1);
	  ret[0]:= 1;
	  subsop(d1, AR = [ret])
	end_proc:

Divs::NRange:=
	proc(XX)
	  local _ret;
	begin
	  _ret:= _mult(op(Divs::val(XX)));
	  if domtype(_ret) = DOM_INT then
	    return(Type::Integer)
	  end_if;
	  return(FAIL)
	end_proc:

Divs::print:=
        proc(XX)
	  local n, r, rr;
	begin
	  if (n:= /*expr2text*/(op(XX, 1))) <> "Divs" then
	    n
	  else
	    n:= Divs::val(XX); r:= op(n, 1); rr:= op(n, 2);
	    "Divs("._if(r <> 1, expr2text(r), "").
	             _if(rr <> 1, _if(r <> 1, ", not ", "not ").expr2text(rr),
			 _if(r*rr = 1, "1", "")).")"
	  end_if
	end_proc:

unalias(Ar):
unalias(Arg):
unalias(Args):
unalias(ARG):
unalias(Divs):
