

/*(
   IV     - Domain of numerical property 'Intervall'

)*/

alias(IV = property::IV):
alias(IVnat = property::IVnat):
alias(Arg(x) = op(extop(x, 5), 1)):
alias(RG(x) = op(extop(x, 5), 2)):
alias(Ar = [5, 1]): alias(Rg = [5, 2]):

IV:= newDomain("property::IV"):
IV::info:= "Domain of numerical property 'Interval of a number'":
IV::domtype:= (()->(Type)):
IV::type:= "IV":
IV::prop:= TRUE:
IV::propdom:= slot(property, "IV"):
//IV::autoload:= TRUE:
IV::testtype:= Type::testtype:

IV::noTrace:= {hold(print), hold(testtype), hold(domtype)}:

IV::ax:= []:

IV::new:= 
  proc()
    local iv, rg;
  begin
    if type(args()) = "id" then
      slot(Type, "Interval")(args())
    else
      if contains({"NRange", "Residue"}, type(args(args(0))))then
        rg:= args(args(0));
        iv:= 1
      else
        rg:= Type::Real;
        iv:= 0
      end_if;
      iv:= IVnat(args(1..(args(0) - iv)));

      IV::check(iv, rg)
    end_if
  end_proc:

// 
IV::check:=
  proc(iv, rg)
  begin
    if iv = property::Null then return(property::Null) end_if;
    //if iv = Dom::Interval(-infinity, infinity) then return(rg) end_if;
    if property::implies(rg, Type::Integer) = TRUE then
      iv:= IVnat::Integer(iv);
      if property::Fail(iv) then return(iv) end_if;
    end_if;
    Type::Interval(hold(id)([iv, rg]))
  end_proc:

IV::_check:=
  proc(iv, rg = Type::Real)
  begin
    if domtype(iv) = DOM_EXPR then
      subsop(hold(id)(IV::check(op(iv, k), rg) $ k = 1..nops(iv)), 0 = op(iv, 0), Unsimplified)
      //map(iv, IV::check, rg) // wird evaluiert!
    elif type(iv) = "IVnat" then
      IV::check(iv, rg)
    else // error
      iv
    end_if
  end_proc:

// TRUE if only one point
IV::isPoint:= iv->IVnat::isPoint(Arg(iv)):

IV::op:=
  proc(iv, opr = 1..3)
  begin
    op((IV, IVnat::leftB(Arg(iv)), IVnat::rightB(Arg(iv)), RG(iv)), map(opr, _plus, 1))
  end_proc:

// _in
IV::testtype_in:=
  proc(x, iv, rg)
  begin
    if testtype(x, Type::Real) then
      if testtype(x, rg) then
        bool(op([hold(_less), hold(_leequal)], IVnat::_lopen(iv) + 1)(IVnat::left(iv), x))
        and bool(op([hold(_less), hold(_leequal)], IVnat::_ropen(iv) + 1)(x, IVnat::right(iv)))
      else
        FALSE
      end_if
    else
      FAIL
    end_if
  end_proc:

// point or FAIL
IV::unequalPoint:=
  proc(iv1, iv2)
  begin
    if IVnat::leftB(Arg(iv1)) = -infinity
       and IVnat::rightB(Arg(iv2)) = infinity
       and IVnat::ropen(Arg(iv1))
       and IVnat::lopen(Arg(iv2))
       and is(IVnat::rightB(Arg(iv1)) = IVnat::leftB(Arg(iv2))) = TRUE then
      return(IVnat::rightB(Arg(iv1)))
    elif IVnat::leftB(Arg(iv2)) = -infinity
       and IVnat::rightB(Arg(iv1)) = infinity
       and IVnat::ropen(Arg(iv2))
       and IVnat::lopen(Arg(iv1))
       and is(IVnat::rightB(Arg(iv2)) = IVnat::leftB(Arg(iv1))) = TRUE then
      return(IVnat::leftB(Arg(iv1)))
    end_if;
    FAIL
  end_proc:

IV::simplify:=
  proc(ivs)
    local l, lo, r, ro, rg, iv;
  begin
    iv:= Arg(ivs);
    if type(iv) <> "IVnat" then
      return(iv)
    end_if;
    rg:= RG(ivs);
    l:= IVnat::left(iv);
    lo:= IVnat::_lopen(iv);
    r:= IVnat::right(iv);
    ro:= IVnat::_ropen(iv);
    if property::RESET = 0 then
      if rg = Type::Real then // Relation
	if r = infinity then
          if l = -infinity then
            return(rg)
          else
            return(property::Rel(-op([hold(_less, _leequal)], lo + 1), l))
          end_if
	elif l = -infinity then
	  return(property::Rel(op([hold(_less, _leequal)], ro + 1), r))
	end_if
      else // Zahlenbereiche
	if r = infinity then
          if l = -infinity then
            return(rg)
	  elif l = 0 then // > 0
	    if rg::dom::sign(rg) = {-1} then
	      return(property::Null)
	    else
	      return(rg::dom::_and(op([Type::Positive, Type::NonNegative], lo + 1), rg))
            end_if
	  end_if
	elif l = -infinity then
	  if r = 0 then // < 0
	    if rg::dom::sign(rg) = {0} then
	      return(property::Null)
	    else
	      return(rg::dom::_and(Type::Negative, rg))
	    end_if
	  end_if
	end_if
      end_if
    end_if;
    IV::check(iv, rg)
  end_proc:

IV::IVnat:=
  proc(iv)
  begin
    Arg(iv)
  end_proc:

IV::is:=
  proc(iv1, iv2)
  begin
    if type(iv1) <> "IV" then
      return(UNKNOWN)
    end_if;

    // NRange
    if type(iv2) = "NRange" then
      if property::implies(RG(iv1), iv2) = TRUE then
        return(TRUE)
      else
        return(property::implies(IV::NRange(iv1), iv2))
      end_if
    elif type(iv2) <> "IV" then
      return(UNKNOWN)
    end_if;

    return(IVnat::is(Arg(iv1), Arg(iv2)) and property::NRange::is(RG(iv1), RG(iv2)))
  end_proc:

// Type::Constant
//IV::constant:=
//	proc(iv)
//	begin
//          FALSE
//	end_proc:

IV::_intersect:= hold(_and):

// Schnitt beider Mengen
IV::_and:=
  proc()
  begin
    property::_Op(IV, _and,
      proc(iv1, iv2)
      begin
	IV::_check(IVnat::_and(Arg(iv1), Arg(iv2)), _and(RG(iv1), RG(iv2)))
      end_proc, args())
  end_proc:

IV::_union:= hold(_or):

// Vereinigung beider Mengen
IV::_or:=
  proc()
  begin
    property::_Op(IV, _or,
      proc(iv1, iv2)
      begin
        IV::_check(IVnat::_or(Arg(iv1), Arg(iv2)), _or(RG(iv1), RG(iv2)))
      end_proc, args())
  end_proc:

IV::sign:=
  proc(iv)
  begin
    property::IVnat::sign(Arg(iv))
  end_proc:

IV::_plus:=
  proc()
  begin
    property::_Op(IV, _plus,
	          proc(iv1, iv2)
                  begin
                    IV::_check(Arg(iv1) + Arg(iv2), RG(iv1) + RG(iv2))
	          end_proc, args())
  end_proc:

IV::_subtract:=
  proc()
  begin
    property::_Op(IV, _subtract,
	          proc(iv1, iv2)
	          begin
                    IV::_check(Arg(iv1) - Arg(iv2), RG(iv1) - RG(iv2))
	          end_proc, args())
  end_proc:

IV::_negate:=
  proc(iv)
  begin
    IV::_check(-Arg(iv), -RG(iv))
  end_proc:

IV::_mult:=
  proc()
  begin
    property::_Op(IV, _mult,
	          proc(iv1, iv2)
	          begin
                    IV::_check(Arg(iv1)*Arg(iv2), RG(iv1)*RG(iv2))
	          end_proc, args())
  end_proc:

IV::_divide:=
  proc()
  begin
    property::_Op(IV, _divide,
      proc(iv1, iv2)
      begin
        IV::_check(Arg(iv1)/Arg(iv2), RG(iv1)/RG(iv2))
      end_proc, args())
  end_proc:

IV::_invert:=
  proc(iv)
  begin
    IV::_check(1/Arg(iv), 1/RG(iv))
  end_proc:

IV::_power:=
  proc(iv1, iv2)
  begin
    if type(iv2) <> IV then
      IV::_check(Arg(iv1)^iv2, RG(iv1)^iv2)
    else
      IV::_check(Arg(iv1)^Arg(iv2), RG(iv1)^RG(iv2))
    end_if
  end_proc:

// 
// IV::ln:=
//   proc(iv)
//   begin
//     IV::_check(ln(Arg(iv)), ln(RG(iv)))
//   end_proc:

IV::NRange:=
  proc(iv, OPT = hold(Approx))
    local Ex;
  begin
    Ex:=
      proc(RET, NULL, INF)
      begin
        if OPT = hold(Exact) and (is(NULL = 0) <> TRUE or type(INF) <> stdlib::Infinity) then
          FAIL
        else
          RET
        end_if
      end_proc;
    case IVnat::sign(Arg(iv))
      of {1} do
        Ex(property::NRange::_and(property::NRange("Positive"), RG(iv)),
           IVnat::left(Arg(iv)), IVnat::right(Arg(iv))); break
      of {-1} do
        Ex(property::NRange::_and(property::NRange("Negative"), RG(iv)),
           IVnat::right(Arg(iv)), IVnat::left(Arg(iv))); break
      of {-1, 0} do
        hold(_or)(property::NRange::_and(property::NRange("Negative"), RG(iv)), Type::Zero); break
      of {0, 1} do
        property::NRange::_and(property::NRange("NonNegative"), RG(iv)); break
      otherwise
        Ex(property::NRange::_and(property::NRange("Real"), RG(iv)), 1, 1)
    end_case
  end_proc:

IV::Rel:=
  proc(iv, Opt = hold(Exact), fi = FAIL)
    local BOR, BORo, bo;
  begin
    //if IV::constant(iv) then
    //  property::Rel(_equal, IVnat::left(Arg(iv), 1))
    //end_if;
    if Opt = hold(Exact) and property::implies(RG(iv), Type::Rational) = TRUE then
      if RG(iv) = Type::Zero then
        return(property::Rel(_equal, 0))
      else
        // cannot simplify subset of Real
        return(iv)
      end_if
    end_if;
    BOR:= IVnat::left(Arg(iv)), IVnat::right(Arg(iv));
    BORo:= IVnat::_lopen(Arg(iv)) + 1, IVnat::_ropen(Arg(iv)) + 1;
    // one point
    if _equal(BOR) then
      return(property::Rel(_equal, op(BOR, 1)))
    end_if;
    fi:= contains(["<", ">", hold(Left), hold(Right)], fi); // (can 1, 2), (must 3, 4)
    if fi = 0 then
      if op(BOR, 1) = -infinity then
        fi:= 4
      elif op(BOR, 2) = infinity then
        fi:= 3
      end_if
    end_if;
    if fi <> 0 then
      bo:= op(BOR, fi);
      if fi > 2 then // nur eine Grenze!
        return(property::Rel(op([-1, 1], fi - 2)*op([hold(_less), hold(_leequal)],
                                                    op(BORo, fi - 2)),
                             op(BOR, fi - 2)))
      else
        if property::_is(bo = (op([-1, 1], fi))*infinity) = TRUE then
          return(property::Rel(op([1, -1], fi)*op([_less, _leequal], op(BORo, 3 - fi)),
                               op(BOR, 3 - fi)))
        end_if;
        bo:= op(BOR, 3 - fi);
        if property::_is(bo = (op([1, -1], fi))*infinity) = TRUE then
          return(property::Rel(op([-1, 1], fi)*op([_less, _leequal], op(BORo, fi)),
                               op(BOR, fi)))
        end_if;
        if property::_is(_equal(BOR)) = TRUE and _mult(BORo) = 4 then
          return(property::Rel(_equal, op(BOR, 1)))
        end_if
      end_if
    end_if;
    hold(_and)(property::Rel(-op([_less, _leequal], op(BORo, 1)),
                             op(BOR, 1)),
               property::Rel(op([_less, _leequal], op(BORo, 2)),
                             op(BOR, 2)))
  end_proc:

IV::Complex:=
  proc(iv)
  begin
    property::Complex(iv, Type::Zero)
  end_proc:

IV::prop2set:=
  proc(iv)
  begin
    if type(iv) = "_or" then
      hold(_union)(map(op(iv), IV::prop2set))
    else
      Dom::Interval::convert(iv)
    end_if
  end_proc:

unalias(IV):

// Zeichen fur 'property::Func'
property::IV::Z:= "in":

property::IV::print:=
  proc(iv)
    //local x, i;
  begin
    expr2text(Arg(iv)) . " " . (RG(iv))::dom::Z . " " . expr2text(RG(iv))
  end_proc:

unalias(RG):
unalias(Rg):
unalias(Ar):
unalias(Arg):
