// $Date: 2000/12/04 16:18:55 $  $Author: bij $  $Revision: 1.34 $

//
//  Complex - Domain of property
//            'Properties of complex identifiers'
//

alias(Complex = property::Complex):
alias(RE(x) = op(extop(x, 5), 1)):
alias(IM(x) = op(extop(x, 5), 2)):
alias(_RE = [5, 1]):
alias(_IM = [5, 2]):

Complex:= newDomain("property::Complex"):
Complex::info:= "Properties of complex identifiers";
Complex::domtype:= ()->(Type):
Complex::type:= "Complex":
Complex::prop:= TRUE:
Complex::propdom:= slot(property, "Complex"):
Complex::testtype:= Type::testtype:

Complex::noTrace:= {hold(print), hold(testtype), hold(domtype)}:

//Complex::ax:= []:

Complex::new:= 
  proc(re = Type::Real, im = Type::Real)
  begin
    if contains({Type::Imaginary, Type::Complex}, re) then
      error("the real part can only be real")
    elif contains({Type::Imaginary, Type::Complex}, im) then
      error("the imaginary part can only be real")
    end_if;
    new(Complex, "Complex", TRUE, FALSE, FALSE, [re, im])
  end_proc:

Complex::_new:=
  proc(re = Type::Real, im = Type::Real)
  begin
    if contains({Type::Imaginary, Type::Complex}, re) then
      re:= re and Type::Real
    end_if;
    if contains({Type::Imaginary, Type::Complex}, im) then
      im:= im and Type::Real
    end_if;
    new(Complex, "Complex", TRUE, FALSE, FALSE, [re, im])
  end_proc:

Complex::is:= 
  proc(FU1, FU2)
    local i;
  begin
    if type(FU1) = "NRange" then
      FU1:= property::NRange::Complex(FU1)
    end_if;

    if type(FU2) = "NRange" then
      if property::implies(IM(FU1), Type::Zero) = TRUE then // nur Realteil
        return(property::implies(RE(FU1), FU2))
      //elif property::implies(RE(FU1), Type::Zero) = TRUE then // nur Imaginarteil
      //  return(property::implies(IM(FU1), FU2))
      end_if
    end_if;

    if type(FU2) <> "Complex" then
      return(UNKNOWN)
    end_if;
    if (i:= contains([RE(FU2), IM(FU2)], FAIL)) > 0 then
      if i = 2 then
        return(property::implies(RE(FU1), RE(FU2)))
      else
        return(property::implies(IM(FU1), IM(FU2)))
      end_if
    end_if;
    
    property::implies(RE(FU1), RE(FU2)) and property::implies(IM(FU1), IM(FU2))
  end_proc:

// Uberladung
Complex::contains:=
        proc(CP, nm)
        begin
          if type(CP) <> "Complex" then
            FALSE
          else
            bool(property::_is(nm, CP) = TRUE)
          end_if
        end_proc:

Complex::_and:=
  proc()
  begin
    property::_Op
    (Complex, _and,
     proc(FU1, FU2)
       local re, im, ret;
     begin
       ret:= _if(RE(FU1) = Type::Real, null(), RE(FU1)),
             _if(RE(FU2) = Type::Real, null(), RE(FU2));
       re:= eval(_if((re:= _and(ret)) = TRUE, Type::Real, re));
       ret:= _if(IM(FU1) = Type::Real, null(), IM(FU1)),
             _if(IM(FU2) = Type::Real, null(), IM(FU2));
       im:= eval(_if((im:= _and(ret)) = TRUE, Type::Real, im));
       if property::implies(im, Type::Zero) = TRUE then return(re) end_if;
       if property::Fail(re, im) then
         return(property::Null)
       else
         return(Complex::_new(re, im))
       end_if
     end_proc, args())
  end_proc:

Complex::_or:=
  proc()
  begin
    //return(hold(_or)(args()));
    property::_Op
    (Complex, _or,
     proc(FU1, FU2)
       local re, im, ret;
     begin
       ret:= _if(RE(FU1) = Type::Real, null(), RE(FU1)),
             _if(RE(FU2) = Type::Real, null(), RE(FU2));
       re:= eval(_if((re:= _or(ret)) = FALSE, Type::Real, re));
       ret:= _if(IM(FU1) = Type::Real, null(), IM(FU1)),
             _if(IM(FU2) = Type::Real, null(), IM(FU2));
       im:= eval(_if((im:= _or(ret)) = FALSE, Type::Real, im));
       //if property::implies(im, Type::Zero) then return(re) end_if;
       return(Complex::_new(re, im))
     end_proc, args())
  end_proc:

Complex::_plus:=
  proc()
  begin
    property::_Op
    (Complex, _plus,
     proc(FU1, FU2)
     begin
       Complex::_new(RE(FU1) + RE(FU2), IM(FU1) + IM(FU2))
       //hold(_plus)(FU1, FU2)
     end_proc, args())
  end_proc:

Complex::_subtract:=
  proc()
  begin
    property::_Op
    (Complex, _subtract,
     proc(FU1, FU2)
     begin
       Complex::_plus(FU1, Complex::_negate(FU2))
       //hold(_subtract)(FU1, FU2)
     end_proc, args())
  end_proc:

Complex::_negate:=
  proc(FU)
  begin
    subsop(FU, _RE = _negate(RE(FU)), _IM = _negate(IM(FU)))
  end_proc:

Complex::_mult:=
  proc()
  begin
    property::_Op
    (Complex, _mult,
     proc(FU1, FU2)
     begin
       Complex::_new(RE(FU1)*RE(FU2) - IM(FU1)*IM(FU2),
                     RE(FU1)*IM(FU2) + IM(FU1)*RE(FU2))
       //Type::Complex
       //hold(_mult)(FU1, FU2)
     end_proc, args())
  end_proc:

Complex::_divide:=
  proc()
  begin
    property::_Op
    (Complex, _divide,
     proc(FU1, FU2)
     begin
       Type::Complex
       //hold(_divide)(FU1, FU2)
     end_proc, args())
  end_proc:

Complex::_invert:=
  proc(FU)
    local x;
  begin
    x:= RE(FU)^2 + IM(FU)^2;
    if property::implies(RE(FU), property::Rel(hold(_unequal), 0)) = TRUE then
      Complex::_new(RE(FU)/x and property::Rel(hold(_unequal), 0), IM(FU)/x)
    elif property::implies(IM(FU), property::Rel(hold(_unequal), 0)) = TRUE then
      Complex::_new(RE(FU)/x, IM(FU)/x and property::Rel(hold(_unequal), 0))
    else
      Complex::_new(RE(FU)/x, IM(FU)/x)
    end_if
    //hold(_invert)(FU1)
  end_proc:

Complex::_power:=
  proc(FU1, POW)
  begin // 
    if type(FU1) <> "Complex"
       and type((FU1:= property::convert_to(FU1, Complex))) <> "Complex" then
      return(hold(_power)(FU1, POW))
    end_if;
    if property::_is(POW, Type::Even) = TRUE then
      if property::_is(RE(FU1), property::Rel(hold(_equal), 0)) = TRUE then
        if property::_is(POW, Type::Residue(0, 4)) = TRUE then
          return(IM(FU1)^POW)
        elif property::_is(POW, Type::Residue(2, 4)) = TRUE then
          return(-IM(FU1)^POW)
        end_if;
        return(hold(_or)(IM(FU1)^2, -IM(FU1)^2))
      else
        return(Type::Complex)
      end_if
    elif property::_is(POW, Type::Real) = TRUE then
      // ???
      return(Type::Complex)
    end_if;
    hold(_power)(FU1, POW)
  end_proc:

Complex::NRange:= 
  proc(CP)
  begin
    if property::implies(IM(CP), Type::Zero) = TRUE then
      return(property::convert_to(RE(CP), property::NRange))
    elif property::implies(RE(CP), Type::Zero) = TRUE then
      case property::implies(IM(CP), Type::NonZero)
        of TRUE do
          return(property::NRange("Imaginary", "Zero"));
          break
        of FALSE do
          return(Type::Zero)
        otherwise
          return(Type::Imaginary)
      end_case
    end_if;
    FAIL
  end_proc:

//Complex::Residue:= 
//  proc(CP)
//  begin
//    FAIL
//  end_proc:

Complex::Rel:=
  proc()
  begin
    property::Null
  end_proc:

alias(SB = solvelib::BasicSet):
alias(DI = Dom::ImageSet):
Complex::prop2set:=
  proc(CP)
    local ID1, ID2;
  begin
    if IM(CP) = Type::Zero then
      property::prop2set(RE(CP))
    elif RE(CP) = Type::Real and IM(CP) = Type::Real then
      SB(Dom::Complex)
    else
      ID1:= genident("X");
      ID2:= genident("Y");
      DI(ID1 + ID2*I, [ID1, ID2], [property::prop2set(RE(CP)), property::prop2set(IM(CP))])
    end_if
  end_proc:
unalias(DI):
unalias(SB):

Complex::Re:=
  proc(CP)
  begin
    RE(CP)
  end_proc:

Complex::Im:=
  proc(CP)
  begin
    IM(CP)
  end_proc:

Complex::exp:=
  proc(CP)
  begin
    CP
  end_proc:

Complex::abs:=
  proc(CP)
  begin
    Type::NonNegative
  end_proc:

Complex::print:=
  proc(XX)
    local x;
  begin
    x:= (_if(RE(XX) = Type::Real, 0, 1) + (_if(IM(XX) = Type::Real, 0, 2)));
    if x = 0 then
      "Type::Complex"
    else
      (_if(x mod 2 = 1, ("Re(.) " .
                         (_if((RE(XX))::dom::Z <> FAIL, (RE(XX))::dom::Z . " ", ""))
                         . expr2text(RE(XX))), "")) .
      (_if(x = 3, " and ", "")).
      (_if(x - 1 > 0, ("Im(.) " .
                       (_if((IM(XX))::dom::Z <> FAIL, (IM(XX))::dom::Z . " ", ""))
                       . expr2text(IM(XX))), ""))
    end_if
  end_proc:

unalias(Complex):
