
/*( Prule - Domain of 'A Rule of Properties'
           


)*/

//alias(POp = property::POp):
alias(Prule = property::Prule):
alias(Arg(x, y) = op(extop(x, 5), y)):

Prule:= newDomain("property::Prule"):
Prule::info:= "A Rule of Properties":
Prule::type:= "Prule":
  
Prule::new:=
	proc()
	  local _op, _fir, _sec, _res;
	begin
	  if args() = null() then return(null()) end_if;
	  if contains({DOM_EXEC, DOM_PROC}, domtype(args(1)))
	     or contains({"Pfunc", "P-func"}, args(args(0))) then
	    return(new(Prule, args(1),
		              _if(args(0) > 1,
		                  _if(type(args(2)) = DOM_STRING,
		                      args(2),
		                      expr2text(args(2))),
		                  "no name"),
		              _if(args(0) > 2,
		                  _if(type(args(3)) = DOM_STRING,
		                      args(3),
		                      expr2text(args(3))),
		                  "no information"),
	                      "Pfunc"))
	  end_if;
	  if type(args(1)) = "_equal" then
	    _op:= op(args(1), [1, 0]);
	    _fir:= op(args(1), [1, 1]);
	    _sec:= op(args(1), [1, 2]);
	    _res:= op(args(1), 2)
	  else
	    if domtype(args(1)) = DOM_EXPR and args(0) < 2 then
	      _op:= op(args(1), 0);
	      _fir:= op(args(1), 1);
	      _sec:= op(args(1), 2);
	      _res:= "??"
	    else
	      _op:= args(1);
	      _fir:= args(2);
	      _sec:= args(3);
	      _res:= _if(args(0) = 3, "??", args(4))
	    end_if
	  end_if;
	  new(Prule, context(_op), context(_fir),
		     _if(_sec = NIL, FAIL, context(_sec)), context(_res))
	end_proc:

Prule::expr:=
	proc()
	begin
	  op(args(), 1)(op(args(), 2..3)) = op(args(), 4)
	end_proc:

//Prule::dom:=
//	proc()
//	begin
//	  return([domtype(op(args(), 2)), domtype(op(args(), 3))])
//	end_proc:

Prule::opr:=
	proc()
	begin
	  op(args(), 1)
	end_proc:

Prule::first:=
	proc()
	begin
	  [op(args(), 1), op(args(), 2)]
	end_proc:

Prule::fir:=
	proc()
	begin
	  op(args(), 2)
	end_proc:

Prule::sec:=
	proc()
	begin
	  op(args(), 3)
	end_proc:

Prule::result:=
	proc()
	begin
	  op(args(), 4)
	end_proc:

Prule::_equiv:=
	proc(X, Y, ex = null())
	begin
	  if ex = "Exact" then
	    bool(op(X, 1) = op(Y, 1)) and
	    bool(op(X, 2) = op(Y, 2)) and
	    bool(op(X, 3) = op(Y, 3))
	  else
	    bool(op(X, 1) = op(Y, 1)) and
	    bool(op(X, 2) = op(Y, 2))
	  end_if
	end_proc:

Prule::_equif:=
	proc(X, Y, ex = null())
	begin
	  if ex = "Exact" then
	    bool(op(X, 1) = op(Y, 1))
	  else
	    bool(op(X, 2) = op(Y, 2))
	  end_if;
	end_proc:

Prule::_name:=
	proc(fn, nm = null(), nf = null())
	begin
	  if Prule::_func(fn) then
	    if args(0) > 1 then
		extsubsop(fn,
			  _if(nm <> "", 2 = nm, null()),
			  _if(args(0) > 2, 3 = nf, null()))
	    else
	      op(fn, 2) . " - " . op(fn, 3)
	    end_if
	  else
	    FAIL
	  end_if
	end_proc:

//Prule::_info:=
	//proc()
	//begin
	  //if Prule::_func(args()) then
	    //op(args(), 4)
	  //else
	    //FAIL
	  //end_if
	//end_proc:

Prule::_func:=
	proc()
	begin
	  //bool(contains({DOM_EXEC, DOM_PROC, DOM_FUNC_ENV}, type(op(args(1), 1))))
	  bool(op(args(), 4) = "Pfunc")
	end_proc:

// Argumente: (P-func, P-rule) oder (P-func, _op, a, b) 
Prule::exec:=
	proc()
	begin
	  if not(Prule::_func(args(1))) then
	    userinfo(1, "Not a P-func!");
	    return(FAIL)
	  end_if;
	  if domtype(args(2)) = Prule then
	    op(args(1), 1)(op(args(2), 1..3))
	  else
	    op(args(1), 1)(args(2..args(0)))
	  end_if
	end_proc:

Prule::Pid:= newDomain("Prule::Pid"):
Prule::Pid::new:=
	proc()
	begin
	  new(Prule::Pid, args())
	end_proc:

Prule::Pid::expr:=
	proc()
	begin
	  map(op(args()), expr)
	end_proc:
	
Prule::Pid::_subtract:=
	proc() begin hold(_subtract)(args()) end_proc:

Prule::Pid::_mult:=
	proc() begin hold(_mult)(args()) end_proc:

Prule::Pid::_divide:=
	proc() begin hold(_divide)(args()) end_proc:

Prule::Pid::_and:=
	proc() begin hold(_and)(args()) end_proc:

Prule::Pid::_or:=
	proc() begin hold(_or)(args()) end_proc:

Prule::Pid::_not:=
	proc() begin hold(_not)(args()) end_proc:

Prule::Pid::print:=
	proc()
	begin
	  expr2text(op(args()))
	end_proc:

Prule::print:=
	proc(rule)
	begin
	  if Prule::_func(rule) then
	    "P-func: " . Prule::fir(rule) . " - " . Prule::sec(rule)
	  else
	    if Prule::sec(rule) <> FAIL then
	      expr2text(Prule::opr(rule)
		        (Prule::Pid(Prule::fir(rule)),
	                 Prule::Pid(Prule::sec(rule)))
		         = Prule::result(rule))
	    else
	      expr2text(Prule::opr(rule)
		        (Prule::Pid(Prule::fir(rule)))
		         = Prule::result(rule))
	    end_if
	  end_if
	end_proc:

Prule::fprint:= 
	proc(_DAT)
	begin
	  fprint(hold(Unquoted), _DAT, Prule::_fprint(args(2), "nice").":")
	end_proc:

Prule::_fprint:=
	proc(rule, nc = null())
	begin
	  if nc = "nice" then
	    if Prule::_func(rule) then
	      expr2text(op(rule, 1..3))
	    else
	      
	      expr2text(op(rule, 1)(
	      Prule::Pid(expr2text(domtype(op(rule, 2))).
	               "(".expr2text(op(op(rule, 2))).")"),
	      Prule::Pid(expr2text(domtype(op(rule, 3))).
	               "(".expr2text(op(op(rule, 3))).")")) =
	      Prule::Pid(expr2text(domtype(op(rule, 4))).
	               "(".expr2text(op(op(rule, 4))).")"))

	    end_if
	  else
	    if Prule::_func(rule) then
	      //expr2text(op(rule, 1..3))
	      expr2text(op(rule, 1)(
	      Prule::Pid(expr2text(domtype(op(rule, 2))).
	               "(".expr2text(op(op(rule, 2))).")"),
	      Prule::Pid(expr2text(domtype(op(rule, 3))).
	               "(".expr2text(op(op(rule, 3))).")")) =
	      Prule::Pid(expr2text(domtype(op(rule, 4))).
	               "(".expr2text(op(op(rule, 4))).")"))
	    else
	      expr2text(op(rule, 1)).", ".
	      expr2text(domtype(op(rule, 2))).
	      "(".expr2text(op(op(rule, 2)))."), ".
	      expr2text(domtype(op(rule, 3))).
	      "(".expr2text(op(op(rule, 3)))."), ".
	      expr2text(domtype(op(rule, 4))).
	      "(".expr2text(op(op(rule, 4))).")"
	    end_if
	  end_if
	end_proc:

unalias(Arg):
unalias(Prule):
unalias(POp):
