// $Date: 2000/12/04 16:18:33 $ $Author: bij $ $Revision: 1.75 $

/*(
   Rel     - Domain of Relations
   Testversion: Identifier entfernt

   op(Rel) - Identifier
             Relation:   1 = "_less"
                         2 = "_leequal"
                         3 = "_unequal"
                         4 = "_equal"
                         5 = "_moequal"
                         6 = "_more"
             Border
)*/

alias(Rel = property::Rel):
alias(OPR(x) = op(extop(x, 1), 1)): // Operator
alias(OP = [1, 1]):
alias(BOR(x) = op(extop(x, 1), 2)): // Border
alias(BO = [1, 2]):

alias(LES = 1):
alias(LEE = 2):
alias(UNE = 3):
alias(EQU = 4):
alias(MOE = 5):
alias(MOR = 6):

Rel:= newDomain("property::Rel"):
Rel::info:= "Domain of algebraic property 'Relations of Identifiers'":
Rel::domtype:= ()->Type:
Rel::type:= "Rel":
Rel::prop:= TRUE:
Rel::propdom:= slot(property, "Rel"):
Rel::autoload:= TRUE:
Rel::testtype:= Type::testtype:

Rel::noTrace:= {hold(print), hold(testtype), hold(domtype)}:

Rel::ax:= []:

Rel::__op:= [hold(_less),        // 1 
             hold(_leequal),     // 2 
             hold(_unequal),     // 3 
             hold(_equal),       // 4 
             -hold(_leequal),    // 5 
             -hold(_less)]:      // 6 

Rel::__po:= ["<",              // 1 
             "<=",             // 2 
             "<>",             // 3 
             "=",              // 4 
             ">=",             // 5 
             ">"]:             // 6 

// operator, border 
Rel::new:= 
        proc(oper = hold(_equal), border = FAIL)
          local x, ret;
        begin
          ret:= if type(oper) = "_mult" then // negiert
                  if contains({hold(_equal), hold(_unequal)},
                              (x:= op(oper, 3 - contains([op(oper)], -1)))) then
                    new(Rel, [contains(Rel::__op, x), border])
                  else
                    if contains(Rel::__op, x) > 0 then
                      new(Rel, [7 - contains(Rel::__op, x), border])
                    else
                      if (x:=contains(Rel::__op,
                                        text2expr(op(x, 3)["type"]))) > 0 then
                        new(Rel, [7 - x, border])
                      else
                        error("unexpected arguments")
                      end_if
                    end_if
                  end_if
                else // nicht negiert
                  if (x:= contains(Rel::__op, oper)) > 0 then
                    new(Rel, [x, border])
                  else
                    if (x:= contains(Rel::__op, 
                                       (oper:= text2expr(op(oper, 3)["type"])))) > 0 then
                      new(Rel, [x, border])
                    else
                      error("unexpected arguments")
                    end_if
                  end_if
                end_if;
          if OPR(ret) <> EQU and OPR(ret) <> UNE and Rel::real(ret) = FALSE then
            error("relation with complex border")
          else
            ret
          end_if
        end_proc:

// Ergebnis-Tabelle fur Operatoren, wenn `Border' gleich sind 
//            "<"       "<="      "<>"      "="       ">="      ">"     
Rel::__xx:= [[TRUE,     TRUE,     TRUE,     FALSE,    FALSE,    FALSE   ],
/*( "<=" )*/   [UNKNOWN,  TRUE,     UNKNOWN,  UNKNOWN,  UNKNOWN,  FALSE   ],
/*( "<>" )*/   [UNKNOWN,  UNKNOWN,  TRUE,     FALSE,    UNKNOWN,  UNKNOWN ],
/*( "="  )*/   [FALSE,    TRUE,     FALSE,    TRUE,     TRUE,     FALSE   ],
/*( ">=" )*/   [FALSE,    UNKNOWN,  UNKNOWN,  UNKNOWN,  TRUE,     UNKNOWN ],
/*( ">"  )*/   [FALSE,    FALSE,    TRUE,     FALSE,    TRUE,     TRUE    ]]:

Rel::LESS:= (x, y) -> y < x:
Rel::LEEQUAL:= (x, y) -> y <= x:

// Verkn"upfungs-Tabelle f"ur Operatoren, wenn `Border' ungleich sind 
//            "<"       "<="      "<>"      "="       ">="      ">"     
Rel::__xy:= [[_leequal, _leequal, _leequal, _leequal,  UNKNOWN,  UNKNOWN ],
/*( "<=" )*/   [_less,    _leequal, _less,    _less,  UNKNOWN,  UNKNOWN ],
/*( "<>" )*/   [/*UNKNOWN,  UNKNOWN,  _equal,   UNKNOWN,  UNKNOWN,  UNKNOWN*/],
/*( "="  )*/   [/*_less,   _leequal, _unequal,  _equal,   _leequal, _less  */],
/*( ">=" )*/   [UNKNOWN,  UNKNOWN,  _less,    Rel::LESS,  _leequal, _less /*_leequal?*/],
/*( ">"  )*/   [UNKNOWN,  UNKNOWN,  _leequal, Rel::LEEQUAL,  _leequal, _less   ]]:

Rel::is:=
        proc(rel1, rel2)
          local o, NCONST, BOR1, BOR2;
        begin
          if args(0) < 2 then return(UNKNOWN) end_if;

          // fast alle Relationen reell
          if property::implies(Type::Real, rel2) = TRUE and (o:= Rel::real(rel1)) = TRUE then
            return(TRUE)
          end_if;

          // Intervall 
          if type(rel1) = "IV" then
            if OPR(rel2) > 4 then
              return(Rel::is(property::IV::Rel(rel1, FAIL, hold(Left)), rel2))
            elif OPR(rel2) < 3 then
              return(Rel::is(property::IV::Rel(rel1, FAIL, hold(Right)), rel2))
            else // 3, 4
              return(op([NIL, NIL, _or, _and], OPR(rel2))
                     (Rel::is(property::IV::Rel(rel1, FAIL, hold(Left)), rel2),
                      Rel::is(property::IV::Rel(rel1, FAIL, hold(Right)), rel2)))
            end_if
          end_if;
          if type(rel2) = "IV" then
            if OPR(rel1) > 4 then
              return(Rel::is(rel1, property::IV::Rel(rel2, FAIL, hold(Left))))
            elif OPR(rel1) < 3 then
              return(Rel::is(rel1, property::IV::Rel(rel2, FAIL, hold(Right))))
            else // 3, 4
              return(op([NIL, NIL, _or, _and], OPR(rel1))
                     (Rel::is(rel1, property::IV::Rel(rel2, FAIL, hold(Left))),
                      Rel::is(rel1, property::IV::Rel(rel2, FAIL, hold(Right)))))
            end_if
          end_if;

          if type(rel1) <> "Rel" and type(rel2) <> "Rel" then
            return(UNKNOWN)
          end_if;

          // numerisch 
          if type(rel1) <> "Rel" then
            // eval notwendig? 
            o:= map([/*eval*/(rel1), /*eval*/(BOR(rel2))], property::carefloat);
            if map({op(o)},
                   (()->(bool(type(args()) = DOM_FLOAT)
                       or contains({infinity, -infinity}, args())))) = {TRUE} then
              if OPR(rel2) < 5 then
                return(bool(op(Rel::__op, OPR(rel2))
                              (op(o, 1), op(o, 2))))
              else
                return(bool(op(Rel::__op, Rel::inv(OPR(rel2)))
                              (op(o, 2), op(o, 1))))
              end_if
            end_if;
            return(UNKNOWN)
          end_if;

          // Ident ohne property

          //
          if type(rel2) <> "Rel" then return(UNKNOWN) end_if;

          // Comparison with complex arguments
          if contains({LES, LEE, MOE, MOR}, OPR(rel2)) and FALSE then
            return(FALSE)
          end_if;

          BOR1:= property::carefloat(BOR(rel1));
          BOR2:= property::carefloat(BOR(rel2));
          if BOR1 <> BOR2 then
            NCONST:= not(testtype(BOR1, Type::Constant) and testtype(BOR1, Type::Constant))
                     and UNKNOWN;
            if OPR(rel1) = EQU then
              return(property::_is(Rel::expr(rel2, BOR1)))
            elif OPR(rel1) = UNE then
              return(UNKNOWN)
            elif OPR(rel2) = EQU then
              return(property::_is(Rel::expr(rel1, BOR2)) and UNKNOWN)
            elif (o:= op(op(Rel::__xy, OPR(rel1)), OPR(rel2))) <> UNKNOWN then
              if (OPR(rel1) > 4) or (OPR(rel2) > 4) then
                return(property::_is(o(BOR2, BOR1))
                       or UNKNOWN)
              else
                return(property::_is(o(BOR1, BOR2))
                       or UNKNOWN)
              end_if
            elif contains({LES*MOE, LES*MOR, LEE*MOE, LEE*MOR}, OPR(rel1)*OPR(rel2)) then
              // !!! excl. 6, 10 ???
              //versch. Richtungen
              o:= eval(op(Rel::__op, OPR(rel2)));
              if OPR(rel2) >= MOE then
                o:= eval(op(Rel::__op, OPR(rel2) - 4));
                return((not(property::_is(o(BOR1, BOR2)))
                        and UNKNOWN) or NCONST)
              else // <= LEE
                return((not(property::_is(o(BOR2, BOR1)))
                        and UNKNOWN) or NCONST)
              end_if
            //elif OPR(rel2) <> 4 then
              //return(property::_is(
                           //op(Rel::__op, OPR(rel2))
                              //BOR2, BOR1))
            end_if;
            return(UNKNOWN)
          end_if;
          return(op(op(Rel::__xx, OPR(rel1)), OPR(rel2)))
        end_proc:

Rel::op:=
        proc(rel, ind)
        begin
          if args(0) = 1 then
            BOR(rel)
          elif ind = 0..1 then
            eval(Rel::__op[OPR(rel)]), BOR(rel)
          else
            op([eval(Rel::__op[OPR(rel)]), BOR(rel)], ind + 1)
          end_if
        end_proc:

// Typ einer Relation 
Rel::reltype:=
        proc(rel)
        begin
          if OPR(rel) < 5 then
            Rel::__op[OPR(rel)]
          else
            _negate(Rel::__op[7 - OPR(rel)])
          end_if
        end_proc:

Rel::_sort:=
        proc(rel1, rel2)
        begin
          bool(op([1,2,5,6,4,3], OPR(rel1)) < op([1,2,5,6,4,3], OPR(rel2)))
        end_proc:

// Typ einer Relation 
Rel::open:=
        proc(oper)
        begin
          contains({1, 3, 6}, OPR(oper))
        end_proc:

// Typ einer Relation 
Rel::closed:=
        proc(oper)
        begin
          contains({2, 4, 5}, OPR(oper))
        end_proc:

// Typ einer Relation 
Rel::operator:=
        proc()
        begin
          if args(0) > 1 then
            Rel(args(2), BOR(args(1)))
          else
            Rel::reltype(args())
          end_if
        end_proc:

// ret open border (rel) or subs open border (rel, bor)
Rel::border:=
        proc()
        begin
          if args(0) > 1 then
            subsop(args(1), BO = args(2))
          else
            BOR(args())
          end_if
        end_proc:

// indets 
Rel::indets:=
        proc(rel)
        begin
          indets(BOR(rel))
        end_proc:

// Type::Numeric
Rel::Numeric:=
        proc(rel, OPT = FAIL)
        begin
          if OPT <> FAIL then
            if testtype(BOR(rel), Type::Numeric) then
              BOR(rel)
            else
              property::carefloat(BOR(rel))
            end_if
          else
            OPR(rel) = 4 and testtype(property::carefloat(BOR(rel)), Type::Numeric)
          end_if
        end_proc:

// Type::Constant 
Rel::constant:=
        proc(rel)
        begin
          if OPR(rel) = 4 then
            property::_is(BOR(rel), Type::Constant)
          else
            UNKNOWN
          end_if;
        end_proc:

// TRUE, if rel 'real', FALSE, if rel 'complex'
Rel::real:=
        proc(rel)
        begin
          if OPR(rel) = UNE then
            if is(BOR(rel), Type::Real) <> TRUE then
              FALSE
            else
              UNKNOWN
            end_if
          elif testtype(BOR(rel), Type::Constant) then
            if contains({DOM_INT, DOM_RAT, DOM_FLOAT}, domtype(BOR(rel))) then
              TRUE
            else // border complex
              bool(OPR(rel) <> EQU) and UNKNOWN
            end_if
          //elif property::_is(BOR(rel), Type::Real) = TRUE then
          //  TRUE
          elif domtype(BOR(rel)) = stdlib::Infinity then
            TRUE
          else
            UNKNOWN
          end_if
        end_proc:

// TRUE, wenn </<= .. >=/> oder >=/> .. </<= 
Rel::neq:=
        proc(opr1, opr2)
        begin
          bool(opr1 < 3 and opr2 > 4) or
          bool(opr1 > 4 and opr2 < 3)
          //bool(OPR(args(1)) < 3 and OPR(args(2)) > 4) or
          //bool(OPR(args(1)) > 4 and OPR(args(2)) < 3)
        end_proc:

//  
Rel::inv:=
        proc(oper)
        begin
          op([6, 5, 3, 4, 2, 1], oper)
        end_proc:

//  
Rel::_maxop:=
        proc(opr1, opr2)
        begin
          if abs(opr1 - opr2) <= 1 then
            return(max(opr1, opr2))
          end_if;
          0
        end_proc:

//  
Rel::_minop:=
        proc(opr1, opr2)
        begin
          if abs(opr1 - opr2) <= 1 then
            return(min(opr1, opr2))
          end_if;
          0
        end_proc:

// Signum analog zu Intervallen 
Rel::sign:=
        proc(rel)
        begin
          if property::_is(BOR(rel) > 0) = TRUE then
            op([{-1, 0, 1}, {-1, 0, 1}, {-1, 0, 1}, {1}, {1}, {1}], OPR(rel))
          elif property::_is(BOR(rel) < 0) = TRUE then
            op([{-1}, {-1}, {-1, 0, 1}, {-1}, {-1, 0, 1}, {-1, 0, 1}], OPR(rel))
          elif property::_is(BOR(rel) = 0) = TRUE then
            op([{-1}, {-1, 0}, {-1, 1}, {0}, {0, 1}, {1}], OPR(rel))
          else
            {-1, 0, 1}
          end_if;
        end_proc:

// internes Signum
Rel::_sign:=
        proc(rel)
        begin
          if property::_is(BOR(rel) > 0) = TRUE then
            op([0, 0, 0, 1, 1, 1], OPR(rel))
          elif property::_is(BOR(rel) < 0) = TRUE then
            op([-1, -1, 0, -1, 0, 0], OPR(rel))
          elif property::_is(BOR(rel) = 0) = TRUE then
            op([-1, -0.5, 0, 0, 0.5, 1], OPR(rel))
          else
            0
          end_if;
        end_proc:

Rel::_not:=
        proc(rel)
        begin
          case OPR(rel)
          of 4 do
            return(subsop(rel, OP = 3))
          of 3 do
            return(subsop(rel, OP = 4))
          end_case;
          hold(_not)(rel)
          //return(hold(_or)(not Type::Real, subsop(rel, OP = modp((OPR(rel) + 4), 8))))
        end_proc:

// Verkn"upfungs-Tabelle f"ur _and, wenn `Border' gleich sind 
//            "<"   "<="  "<>"  "="   ">="  ">" 
Rel::__ax:= [[LES,  LES,  LES,  FAIL, FAIL, FAIL],
/*( "<=" )*/   [LES,  LEE,  LES,  EQU,  EQU,  FAIL],
/*( "<>" )*/   [LES,  LES,  UNE,  FAIL, MOR,  MOR ],
/*( "="  )*/   [FAIL, EUQ,  FAIL, EQU,  EQU,  FAIL],
/*( ">=" )*/   [FAIL, EQU,  MOR,  EQU,  MOE,  MOR ],
/*( ">"  )*/   [FAIL, FAIL, MOR,  FAIL, MOR,  MOR ]]:

// #( Verkn"upfungs-Tabelle f"ur _and -> Null, wenn `Border' ungleich sind )#
// #(            "<"            "<="       "<>"  "="   ">="  ">" )#
// Rel::__ax:= [[FAIL,          FAIL                                   ],
// #( "<=" )#   [FAIL,          FAIL                                   ],
// #( "<>" )#   [                           FAIL,                      ],
// #( "="  )#   [_not@_leequal, _not@_less, FAIL, TRUE, _less, _leequal],
// #( ">=" )#   [                                       FAIL,  FAIL    ],
// #( ">"  )#   [                                       FAIL,  FAIL    ]]:

Rel::_and:=
        proc()
          local ARGS, i;
        begin
          // ausser fur Ungleichungen!
          if (i:= contains((ARGS:= [args()]), Type::Real)) > 0 then
            // Achtung: map von alias OPR!!!
            if contains(map(subsop(ARGS, i = null()), X->op(extop(X, 1), 1)), 4) = 0 then
              return(Rel::_and(op(subsop(ARGS, i = null()))))
            end_if
          end_if;
          // umsortieren der Argumente - Relationen mit gleichen Grenzen zuerst
          if args(0) > 2 then
            ARGS:= split(ARGS, X -> type(X) = "Rel");
            ARGS:= subsop(ARGS, 1 = sort(op(ARGS, 1),
                                         (X, Y) -> property::isrel(BOR(X) >= BOR(Y)) = TRUE));
            for i from 1 to nops(op(ARGS, 1)) - 1 do
              if BOR(op(ARGS, [1, i])) = BOR(op(ARGS, [1, i + 1])) then
                ARGS:= subsop(ARGS, [1, i] = FAIL,
                                    [1, i + 1] = Rel::_and(op(ARGS, [1, i]), op(ARGS, [1, i + 1])));
              end_if
            end_for;
            if contains(op(ARGS, 1), property::Null) > 0 then
              return(property::Null)
            end_if;
            ARGS:= subs(op(ARGS, 1), FAIL = null()) . op(ARGS, 2)
          end_if;

          property::_Op(Rel, _and,
            proc(rel1, rel2)
              local __op, __pq, __ar;
            begin
              if (__op:= contains([TRUE, FALSE], property::implies(rel1, rel2))) > 0 then
                return(op([rel1, property::Null], __op))
              end_if;
              if (__op:= contains([TRUE, FALSE], property::implies(rel2, rel1))) > 0 then
                return(op([rel2, property::Null], __op))
              end_if;
              __ar:= BOR(rel1), BOR(rel2);
              if property::_is(_equal(__ar)) = TRUE then
                if (__op:= op(Rel::__ax, [OPR(rel1), OPR(rel2)]))
                   <> FAIL then
                  return(subsop(rel1, OP = __op))
                else
                  return(property::Null)
                end_if
              end_if;
              __pq:= OPR(rel1), OPR(rel2);
//            if (__op:= contains([__pq], 4)) > 0 then
//                 if (k:= OPR(args(3 - __op))) = 4 then
//                   return(property::Null)
//                 elif k <> 3 then
//                   if property::_is(Rel::expr(subsop(args(3 - __op),
//                                                     OP = OPR(args(3 - __op))),
//                                              op(__ar, __op))) = FALSE then
//                     return(property::Null)
//                   end_if
//                 end_if
//            end_if;
              if (__pq:= {__pq}) minus {1, 2} = {} then
                if (__op:= (contains([TRUE, FALSE], property::_is(_less(__ar))))) > 0  then
                  return(args(__op))
                end_if
              end_if;
              if __pq minus {5, 6} = {} then
                if (__op:= (contains([TRUE, FALSE], property::_is(_less(__ar))))) > 0  then
                  return(args(3 - __op))
                end_if
              end_if;
// max/min nicht schlau genug
//            if __pq intersect {1, 2} = {} then
//              if type((__op:= max(__ar))) <> "max" then
//                return(args(contains([__ar], __op)))
//              end_if
//            end_if;
//            if __pq intersect {5, 6} = {} then
//              if type((__op:= min(__ar))) <> "min" then
//                return(args(contains([__ar], __op)))
//              end_if
//            end_if;
              //if (__op:= property::_is(BOR(rel1) < BOR(rel2)))
                 //= TRUE then
                //if (__op:= property::_is(BOR(rel1) <= BOR(rel2)))
                   //= TRUE then
                  
                //end_if
              //elif (__op:= property::_is()) = TRUE then
                //if (__op:= property::_is()) = TRUE then
                  
                //end_if
              //end_if;
              //if (OPR(rel1) > 4 and OPR(rel2) < 3) then
                //return()
              //elif (OPR(rel1) > 4 and OPR(rel2) < 3) then
                //return()
              //end_if;
              //return(_and(Rel::IV(rel1), Rel::IV(rel2)))
              return(hold(_and)(args()))
            end_proc, op(ARGS))
            //        ARGS! umsortierte Argumente
        end_proc:

// Verkn"upfungs-Tabelle f"ur _and, wenn `Border' gleich sind 
//            "<"   "<="  "<>"  "="   ">="  ">" 
Rel::__ox:= [[LES,  LEE,  UNE,  LEE,  FAIL, UNE ],
/*( "<=" )*/   [LEE,  LEE,  UNE,  LEE,  FAIL, FAIL],
/*( "<>" )*/   [UNE,  FAIL, UNE,  FAIL, FAIL, UNE ],
/*( "="  )*/   [LEE,  LEE,  FAIL, EQU,  MOE,  MOE ],
/*( ">=" )*/   [FAIL, FAIL, FAIL, MOE,  MOE,  MOE ],
/*( ">"  )*/   [UNE,  FAIL, UNE,  MOE,  MOE,  MOR ]]:

Rel::_or:=
        proc()
        begin
          property::_Op(Rel, _or,
            proc(rel1, rel2)
              local __op, __pq, __ar, i;
            begin
              if (__op:= contains([TRUE, FAIL/*FALSE*/], property::implies(rel1, rel2))) > 0 then
                return(op([rel2, Type::Real], __op))
              end_if;
              if (__op:= contains([TRUE, FAIL/*FALSE*/], property::implies(rel2, rel1))) > 0 then
                return(op([rel1, Type::Real], __op))
              end_if;
              __ar:= BOR(rel1), BOR(rel2);
              if property::_is(_equal(__ar)) = TRUE then
                if {OPR(rel1), OPR(rel2)} = {EQU, UNE} then
                  return(Type::Complex)
                elif (__op:= op(Rel::__ox, [OPR(rel1), OPR(rel2)]))
                   <> FAIL then
                  return(subsop(rel1, OP = __op))
                else
                  return(Type::Real)
                end_if
              end_if;
              __pq:= OPR(rel1), OPR(rel2);
              if (i:= contains([__pq], 3)) > 0 then
                if property::_is(Rel::expr(args(3 - i), BOR(args(i)))) = TRUE  then
                  return(Type::Real)
                end_if
              end_if;
              
              if (__pq:= {__pq}) intersect {3, 4} = {} then
                if __pq intersect {1, 2} = {} then
                  if (__op:= (contains([TRUE, FALSE], property::_is(_less(__ar))))) > 0  then
                    return(args(3 - __op))
                  end_if
                end_if;
                if __pq intersect {5, 6} = {} then
                  if (__op:= (contains([TRUE, FALSE], property::_is(_less(__ar))))) > 0  then
                    return(args(__op))
                  end_if
                end_if;
                if property::_is(Rel::expr(rel1, BOR(rel2))) = TRUE then
                  return(Type::Real)
                end_if
              end_if;
// min/max nicht schlau genug
//            if __pq intersect {1, 2} = {} then
//              if type((__op:= min(__ar))) <> "min" then
//                return(args(contains([__ar], __op)))
//              end_if
//            end_if;
//            if __pq intersect {5, 6} = {} then
//              if type((__op:= max(__ar))) <> "max" then
//                return(args(contains([__ar], __op)))
//              end_if
//            end_if;
              //return(_or(Rel::IV(rel1), Rel::IV(rel2)))
              return(hold(_or)(args()))
            end_proc, args())
        end_proc:

Rel::_plus:=
        proc()
        begin
          property::_Op(Rel, _plus,
            proc(rel1, rel2)
            begin
              if OPR(rel1) = OPR(rel2) then
                if OPR(rel1) <> 3 then
                  return(subsop(rel1, BO = BOR(rel1) + BOR(rel2)))
                end_if
              // 1+2=3,1*2=2; 5+6mod8=3,5*6mod25=5 
              elif modp(OPR(rel1) + OPR(rel2), 8) = 3 then
                return(subsop(rel1, OP = _if(OPR(rel1) < 3, 1, 6),
                                       BO = BOR(rel1) + BOR(rel2)))
              elif OPR(rel1) = 4 then
                return(subsop(rel2, BO = BOR(rel1) + BOR(rel2)))
              elif OPR(rel2) = 4 then
                return(subsop(rel1, BO = BOR(rel1) + BOR(rel2)))
              end_if;
              return(Type::Real)
              //return(hold(_plus)(args()))
            end_proc, args())
        end_proc:

Rel::_subtract:=
        proc()
        begin
          property::_Op(Rel, _subtract,
            proc(rel1, rel2)
            begin
              if Rel::neq(OPR(rel1), OPR(rel2)) then
                                         // prov.
                return(subsop(rel1, OP = op(Rel::__ax, [OPR(rel1), Rel::inv(OPR(rel2))]),
                                    BO = BOR(rel1) - BOR(rel2)))
              // < > 
              elif OPR(rel1) = 4 then
                return(subsop(rel2, OP = Rel::inv(OPR(rel2)),
                                       BO = BOR(rel1) - BOR(rel2)))
              elif OPR(rel2) = 4 then
                return(subsop(rel1, BO = BOR(rel1) - BOR(rel2)))
              end_if;
              return(Type::Real)
              //return(hold(_subtract)(args()))
            end_proc, args())
        end_proc:

Rel::_negate:=
        proc(rel1)
        begin
          return(subsop(rel1, OP = Rel::inv(OPR(rel1)),
                                 BO = -(BOR(rel1))))
        end_proc:

// _leequal eines Nulloperanden 
// sign_1, sign_2, op_1, op_2, op_alt 
Rel::__muop:=
        proc(sign_1, sign_2, op_1, op_2, op_alt)
        begin
          if sign_1 = 0.5 and (op_1 mod 5) <> 1 then op_1
          elif sign_1 = -0.5 and (op_1 mod 5) <> 1 then Rel::inv(op_1)
          elif sign_2 = 0.5 and (op_2 mod 5) <> 1 then op_2
          elif sign_2 = -0.5 and (op_2 mod 5) <> 1 then Rel::inv(op_2)
          else op_alt
          end_if
        end_proc:

Rel::_mult:=
        proc()
        begin
          property::_Op
           (Rel, _mult,
            proc(rel1, rel2)
              local __ar, __s1, __s2;
            begin
              __s1:= Rel::_sign(rel1);
              __s2:= Rel::_sign(rel2);
              __ar:= [OPR(rel1), OPR(rel2)];

              if OPR(rel1) = 4 then
                return(subsop(rel2, _if(__s1 = 0, OP = 4, null()),
                                    _if(__s1 = -1, OP = 7 - OPR(rel2), null()),
                                       BO = BOR(rel1)*BOR(rel2)))
              elif OPR(rel2) = 4 then //!!! _sign(= I) = 0 !!!
                if __s2 = 0 and Rel::real(rel2) = FALSE then
                  return(property::Complex::_mult(property::convert_to(rel1, property::Complex),
                                                  property::convert_to(rel2, property::Complex)))
                else
                  return(subsop(rel1, _if(__s2 = 0, OP = 4, null()),
                                      _if(__s2 = -1, OP = 7 - OPR(rel1), null()),
                                      BO = BOR(rel1)*BOR(rel2)))
                end_if
              end_if;

              if __s1 > 0 then
                if __s2 > 0 then
                  if {op(__ar)} minus {5, 6} = {} then
                    return(subsop(rel1,
                           OP = Rel::__muop(__s1, __s2, op(__ar), max(op(__ar))),
                           BO = BOR(rel1)*BOR(rel2)))
                  end_if
                elif __s2 < 0 then
                  if {op(__ar, 1)} minus {5, 6} = {} and
                     {op(__ar, 2)} minus {1, 2} = {} then
                    return(subsop(rel2,
                           OP = Rel::__muop(__s1, __s2, op(__ar),
                                     min(Rel::inv(op(__ar, 1)), op(__ar, 2))),
                           BO = BOR(rel1)*BOR(rel2)))
                  end_if
                end_if
              elif __s1 < 0 then
                if __s2 > 0 then
                  if {op(__ar, 2)} minus {5, 6} = {} and
                     {op(__ar, 1)} minus {1, 2} = {} then
                    return(subsop(rel1,
                           OP = Rel::__muop(__s1, __s2, op(__ar),
                                     max(Rel::inv(op(__ar, 2)), op(__ar, 1))),
                           BO = BOR(rel1)*BOR(rel2)))
                  end_if
                elif __s2 < 0 then
                  if {op(__ar)} minus {1, 2} = {} then
                    return(subsop(rel1,
                           OP = Rel::__muop(__s1, __s2, op(__ar),
                                     Rel::inv(min(op(__ar)))),
                           BO = BOR(rel1)*BOR(rel2)))
                  end_if
                end_if
              end_if;

              return(Type::Real)
              //return(hold(_mult)(args()))
            end_proc, args())
        end_proc:

Rel::_divide:=
        proc()
        begin
          property::_Op
           (Rel, _divide,
            proc(rel1, rel2)
              local __ar, __s1, __s2;
            begin
              __s2:= Rel::_sign(rel2);
              // Null Teiler 
              if abs(__s2) < 1 then
                if contains({2, 4, 5}, OPR(rel2)) then
                  return(property::Null)
                elif OPR(rel2) = 3 and
                     property::_is(BOR(rel2) <> 0) = TRUE then
                  return(property::Null)
                end_if
              end_if;

              __s1:= Rel::_sign(rel1);
              __ar:= [OPR(rel1), OPR(rel2)];
              // =x 
              if OPR(rel1) = 4 then
                if OPR(rel2) = 4 and Rel::_sign(rel2) <> 0 then
                  return(subsop(rel1, BO = BOR(rel1)/BOR(rel2)))
                end_if;
                if abs(__s2) = 1 then
                  if sign(__s1*__s2) < 0 then
                    return(subsop(rel2, OP = 1, BO = 0))
                  elif sign(__s1*__s2) > 0 then
                    return(subsop(rel2, OP = 6, BO = 0))
                  elif property::_is(BOR(rel1) = 0) = TRUE then
                    return(subsop(rel1, BO = 0))
                  end_if
                end_if
              elif OPR(rel2) = 4 then
                if abs(__s1) = 1 then
                  if sign(__s1*__s2) < 0 then
                    return(subsop(rel1, OP = 1, BO = 0))
                  elif sign(__s1*__s2) > 0 then
                    return(subsop(rel1, OP = 6, BO = 0))
                  elif property::_is(BOR(rel1) = 0) = TRUE then
                    return(subsop(rel2, BO = 0))
                  end_if
                end_if
              end_if;

              if __s1 > 0 then
                if __s2 > 0 then
                  if {op(__ar)} minus {5, 6} = {} then
                    return(subsop(rel1, OP = 6, BO = 0))
                  end_if
                elif __s2 < 0 then
                  if {op(__ar, 1)} minus {5, 6} = {} and
                     {op(__ar, 2)} minus {1, 2} = {} then
                    return(subsop(rel2, OP = 1, BO = 0))
                  end_if
                end_if
              elif __s1 < 0 then
                if __s2 > 0 then
                  if {op(__ar, 2)} minus {5, 6} = {} and
                     {op(__ar, 1)} minus {1, 2} = {} then
                    return(subsop(rel1, OP = 1, BO = 0))
                  end_if
                elif __s2 < 0 then
                  if {op(__ar)} minus {1, 2} = {} then
                    return(subsop(rel1, OP = 6, BO = 0))
                  end_if
                end_if
              end_if;

              // genau (un)gleich 
              if {op(__ar)} minus {3, 4} = {} then
                if {op(__ar)} = {4} then
                  return(subsop(rel1, BO = BOR(rel1)/BOR(rel2)))
                elif {op(__ar)} = {3, 4} then
                  return(subsop(rel1, OP = 3, BO = BOR(rel1)/BOR(rel2)))
                end_if
              end_if;

              // <> 0
              if OPR(rel1) = 3 and property::_is(BOR(rel1) = 0) = TRUE then
                return(rel1)
              end_if;

              return(Type::Real)
              //return(hold(_divide)(args()))
            end_proc, args())
        end_proc:

Rel::_invert:=
        proc(rel)
          local k;
        begin
          if Rel::_sign(rel) > 0 then
            if OPR(rel) = 4 then
              return(subsop(rel, BO = 1/BOR(rel)))
            elif OPR(rel) > 4 then
              if property::_is(BOR(rel) >= 1) = TRUE then
                return(property::IV(0, _if(OPR(rel) > 5, 1/BOR(rel), [1/BOR(rel)])))
              else
                return(subsop(rel, OP = 6, BO = 0))
              end_if
            end_if
          elif Rel::_sign(rel) < 0 then
            if OPR(rel) = 4 then
              return(subsop(rel, BO = 1/BOR(rel)))
            elif OPR(rel) < 3 then
              if property::_is(BOR(rel) <= -1) = TRUE then
                return(property::IV(_if(OPR(rel) < 2, 1/BOR(rel), [1/BOR(rel)]), 0))
              else
                return(subsop(rel, OP = 1, BO = 0))
              end_if
            end_if
          end_if;
          if (k:= property::_is(BOR(rel) = 0)) = FALSE then
            k:= property::IV(-1, 1) and rel;
            if type(k) = "IV" then
              k:= property::IV::sign(k)
            elif k = property::Null then
              k:= {0}
            else
              k:= {-1, 0, 1}
            end_if;
            if k = property::Null then
              return(subsop(rel, BO = 1/BOR(rel)))
            elif not contains(k, -1) then
              return(property::Rel(-hold(_less), 0))
            elif not contains(k, 1) then
              return(property::Rel(hold(_less), 0))
            else
              return(property::IV(-infinity, infinity))
            end_if
          elif k = TRUE then
            if OPR(rel) = 3 then
              return(subsop(rel, BO = 0))
            end_if
          end_if;
          return(property::Null)
        end_proc:

Rel::_power:=
        proc(rel, pow)
          local s, sp, REAL;
        begin
          if type(rel) <> "Rel" then
            if type((s:= property::_convert(FAIL, pow, Rel))) <> "Rel" then
              return(hold(_power)(rel, pow))
            else
              return(Rel::_power(s, pow))
            end_if
          end_if;

          s:= Rel::_sign(rel);
          if type(pow) <> "Rel" then
            sp:= property::_is(pow > 0);
            // Quadrat einer Relation
            if (REAL:= property::_is(pow, Type::Even)) = TRUE then
              // > 0
              if OPR(rel) = UNE then
                return(rel)
              end_if;
              if s > 0 then
                if sp = TRUE then // pow > 0
                  return(subsop(rel, BO = BOR(rel)^pow))
                elif sp = FALSE then // pow < 0
                  if BOR(rel) <> 0 then
                    return(subsop(rel, BO = 1/(BOR(rel)^-pow)))
                  else
                    return(Rel::_invert(Rel::_power(rel, -pow)))
                  end_if
                else
                  return(subsop(rel, BO = BOR(rel)^pow))
                end_if
              // < 0
              elif s < 0 then
                if sp = TRUE then // pow > 0
                  return(subsop(rel, OP = 7 - OPR(rel), BO = BOR(rel)^pow))
                elif sp = FALSE then
                  if BOR(rel) <> 0 then
                    return(subsop(rel, BO = 1/(BOR(rel)^-pow)))
                  else
                    return(Rel::_invert(Rel::_power(rel, -pow)))
                  end_if
                else
                  return(subsop(rel, BO = BOR(rel)^pow))
                end_if
              elif testtype(BOR(rel), Type::Real) then
                return(Rel::abs(rel))
              end_if
            // pow ungerade
            elif REAL = FALSE then
              // > 0
              if s > 0 then
                if sp = TRUE then // pow > 0
                  return(subsop(rel, BO = BOR(rel)^pow))
                elif sp = FALSE then // pow < 0
                  if BOR(rel) <> 0 then
                    //return(subsop(rel, BO = 1/(BOR(rel)^-pow)))
                  else
                    //return(Rel::_invert(Rel::_power(rel, -pow)))
                  end_if
                else
                  //return(subsop(rel, BO = BOR(rel)^pow))
                end_if
              // < 0
              elif s < 0 then
                if sp = TRUE then // pow > 0
                  //return(subsop(rel, OP = 7 - OPR(rel), BO = BOR(rel)^pow))
                elif sp = FALSE then
                  if BOR(rel) <> 0 then
                    //return(subsop(rel, BO = 1/(BOR(rel)^-pow)))
                  else
                    //return(Rel::_invert(Rel::_power(rel, -pow)))
                  end_if
                else
                  return(subsop(rel, BO = 0))
                end_if
              end_if
            end_if;

            // Wurzel 
            if domtype(pow) = DOM_RAT and op(pow, 2) = 2 then
              return(Rel::sqrt(rel, op(pow, 1)))
            end_if;
            // hoch Null 
            if property::_is(pow = 0) = TRUE then
              return(subsop(rel, BO = BOR(rel)^0))
            elif property::_is(pow < 0) = TRUE then
              return(Rel::_invert(Rel::_power(rel, -pow)))
            end_if;

            // pow numerisch
            if testtype(pow, Type::Constant) and testtype(property::carefloat(pow), Type::Real) then
              if s > 0 then
                // > 0
                if pow >= 0 then
                  return(subsop(rel, BO = BOR(rel)^pow))
                else // pow < 0
                  return(Rel::_invert(subsop(rel, BO = BOR(rel)^-pow)))
                end_if
              elif s < 0 then
                // <= 0
                if domtype(pow/2) = DOM_INT then
                  if pow >= 0 then
                    return(subsop(rel, BO = BOR(rel)^pow, OP = Rel::inv(OPR(rel))))
                  else // pow < 0
                    return(Rel::_invert(Rel::_power(rel, -pow)))
                  end_if
                else
                  if pow >= 0 then
                    return(subsop(rel, BO = BOR(rel)^pow))
                  else // pow < 0
                    return(Rel::_invert(subsop(rel, BO = BOR(rel)^-pow)))
                  end_if
                end_if
              else // Type::Real
                if domtype(pow/2) = DOM_INT then
                  return(property::Rel(-_leequal, 0))
                elif testtype(BOR(rel), Type::Real) then
                  return(subsop(rel, BO = BOR(rel)^pow))
                else
                  return(hold(_power)(rel, pow))
                end_if
              end_if
            // pow numerisch
            elif testtype(pow, Type::Numeric) then // Complex
              return(Type::Complex)
            // pow NRange
            elif type(pow) = "NRange" then
              REAL:= property::NRange::is(pow, Type::Real);
              //return(property::NRange::_power(Rel::NRange(rel), pow));
              if not contains((sp:= property::NRange::sign(pow)), 1) then
                // pow >= 0
                if not contains(sp, 0) and property::NRange::is(pow, Type::Integer) = TRUE then
                  // pow >= 1
                  if s > 0 then // rel > 0
                    return(rel)
                  elif s < 0 then // rel < 0
                    return(Type::Real)
                  else
                    return(Type::Complex)
                  end_if
                else // pow >= 0
                  if s > 0 then // rel > 0
                    return(subsop(rel, BO = 0))
                  elif s < 0 then // rel < 0
                    return(Type::Complex)
                  else
                    return(Type::Complex)
                  end_if
                end_if
              elif not contains(sp, -1) then
                // pow <= 0
                return(Rel::_invert(Rel::_power(rel, -pow)))
              elif property::NRange::is(pow, Type::Integer) = TRUE then
                // pow - integer
                if s > 0 then // rel >= 0
                  return(subsop(rel, BO = 0))
                elif s < 0 then // rel < 0
                  return(Type::Real)
                else 
                  return(Type::Real)
                end_if
              elif property::NRange::is(pow, Type::Real) = TRUE then
                // pow - real
                if s > 0 then // rel >= 0
                  return(subsop(rel, BO = 0))
                elif s < 0 then // rel < 0
                  return(Type::Complex)
                else 
                  Type::Real
                end_if
              else
                return(Type::Complex)
              end_if
            elif type(pow) = "Residue" then
              if property::Residue::constant(pow) = TRUE then
                return(Rel::_power(rel, property::Residue::remainder(pow)))
              else
                // Verallgemeinerung
                return(Rel::_power(rel, property::Residue::range(pow)))
              end_if
            else
            end_if;
          else // rel^rel
            sp:= Rel::_sign(pow);
            if s > 0 then
              // > 0
              if sp >= 0 then
                return(subsop(rel, BO = BOR(rel)^BOR(pow)))
              else // pow < 0
                return(Rel::_invert(subsop(rel, BO = BOR(rel)^-BOR(pow))))
              end_if
            elif s < 0 then
              // <= 0
              if sp >= 0 then
                return(subsop(rel, BO = BOR(rel)^BOR(pow)))
              else // pow < 0
                return(Rel::_invert(subsop(rel, BO = BOR(rel)^-BOR(pow))))
              end_if
            end_if
          end_if;
          return(hold(_power)(args()))
        end_proc:

Rel::evaluate:=
        proc(rel)
        begin
          subsop(rel, BO = eval(BOR(rel)))
        end_proc:

Rel::float:=
        proc(rel)
          local f;
        begin
          if type((f:= float(BOR(rel)))) <> "float" then
            subsop(rel, BO = f)
          else
            rel
          end_if
        end_proc:

Rel::Integer:=
        proc(rel)
        begin
          if contains({1, 2}, OPR(rel)) then
            property::Rel(_leequal, floor(BOR(rel)))
          elif contains({5, 6}, OPR(rel)) then
            property::Rel(-_leequal, ceil(BOR(rel)))
          else
            property::Null
          end_if
        end_proc:

// Attribute
Rel::min:=
        proc(XX)
        begin
          if args(0) = 1 then
            return(XX)
          else
            return(Rel::_min(XX, Rel::min(args(2..args(0)))))
          end_if;
          FAIL
        end_proc:

Rel::_min:=
        proc(rel1, rel2)
          local _op;
        begin
          if (_op:= Rel::_minop(OPR(rel1), OPR(rel2))) <> 0 then
            if property::_is(BOR(rel1) = BOR(rel2)) = TRUE then
              return(subsop(rel1, OP = _op))
            elif property::_is(BOR(rel1) >= BOR(rel2)) = TRUE then
              return(rel2)
            elif property::_is(BOR(rel1) <= BOR(rel2)) = TRUE then
              return(rel1)
            else
              return(subsop(rel1, OP = _op, BO = hold(min)(BOR(rel1), BOR(rel2))))
            end_if
          end_if;
          return(hold(min)(rel1, rel2))
        end_proc:

Rel::max:=
        proc(XX)
        begin
          if args(0) = 1 then
            return(XX)
          else
            return(Rel::_max(XX, Rel::max(args(2..args(0)))))
          end_if;
          FAIL
        end_proc:

Rel::_max:=
        proc(rel1, rel2)
          local _op;
        begin
          if (_op:= Rel::_maxop(OPR(rel1), OPR(rel2))) <> 0 then
            if property::_is(BOR(rel1) = BOR(rel2)) = TRUE then
              return(subsop(rel1, OP = _op))
            elif property::_is(BOR(rel1) >= BOR(rel2)) = TRUE then
              return(rel1)
            elif property::_is(BOR(rel1) <= BOR(rel2)) = TRUE then
              return(rel2)
            else
              return(subsop(rel1, OP = _op, BO = hold(max)(BOR(rel1), BOR(rel2))))
            end_if
          end_if;
          return(hold(max)(rel1, rel2))
        end_proc:

Rel::sin:=
        proc(rel)
        begin
          if contains({3, 4}, OPR(rel)) then
            subsop(rel, BO = sin(BOR(rel)))
          else
            property::IV([-1, 1])
          end_if
        end_proc:

Rel::cos:=
        proc(rel)
        begin
          if contains({3, 4}, OPR(rel)) then
            subsop(rel, BO = cos(BOR(rel)))
          else
            property::IV([-1, 1])
          end_if
        end_proc:

Rel::tan:=
        proc(rel)
        begin
          if contains({3, 4}, OPR(rel)) then
            subsop(rel, BO = tan(BOR(rel)))
          else
            property::IV([-1, 1])
          end_if
        end_proc:

Rel::sqrt:=
        proc(rel)
        begin
          if Rel::_sign(rel) > 0 then
            subsop(rel, BO = sqrt(BOR(rel)))
          else
            Type::Imaginary
          end_if
        end_proc:

Rel::ln:=
        proc(rel)
        begin
          if property::_is(BOR(rel) >= 1) = TRUE then
            if OPR(rel) >= 4 then
              return(subsop(rel, BO = (BOR(rel) - 1)))
            end_if
          elif property::_is(BOR(rel) >= 0) = TRUE then
            return(Type::Real)
          end_if;
          hold(ln)(rel)
//        if Rel::_sign(subsop(rel, BO = BOR(rel) - 1)) <> 0 then
//          subsop(rel, BO = 0)
//        else
//          Type::Real
//        end_if
        end_proc:

Rel::exp:=
        proc(rel)
        begin
          if Rel::neq(OPR(rel), 1) or contains({3, 4}, OPR(rel)) then // >/>=
            return(subsop(rel, BO = exp(BOR(rel))))
          else //if Rel::neq(OPR(rel), 6) then // </<=
            property::IV(0, exp(BOR(rel)))
          end_if
        end_proc:

Rel::abs:=
        proc(rel)
          local s;
        begin
          if OPR(rel) = 4 then // =
            subsop(rel, BO = abs(BOR(rel)))
          elif OPR(rel) = 3 then // <>
            rel and property::Rel(-_leequal, 0)
          elif contains({5, 6}, OPR(rel)) then // >/>=
            if contains((s:= Rel::sign(rel)), -1) then
              property::Rel(-_leequal, 0)
            else
              rel
            end_if
          else //elif contains({1, 2}, OPR(rel)) then // </<=
            if contains((s:= Rel::sign(rel)), 1) then
              property::Rel(-_leequal, 0)
            else
              -rel
            end_if
          end_if
        end_proc:

// Konvertierungen
Rel::NRange:=
        proc(rel1, Opt = hold(Exact))
        begin
          if Rel::real(rel1) = FALSE then
            return(FAIL)
          end_if;
          // 
          if Opt = hold(Exact)
             and (not(contains({-infinity, 0, infinity}, OPR(rel1)))
                  or contains({DOM_EXPR, DOM_IDENT}, domtype(OPR(rel1)))) then
            if property::_is(BOR(rel1) = 0) = TRUE then
              return(property::NRange(op(["Negative", "Negative", "NonZero",
                                          "Zero", "NonNegative", "Positive"], OPR(rel1))))
            end_if;
            return(Rel::IV(rel1))
          else
            case OPR(rel1)
              of 4 do
                return((BOR(rel1))::dom::NRange(BOR(rel1)))
              of 1 do // < 
                if property::_is(BOR(rel1) <= 0) = TRUE then
                  return(property::NRange("Negative"))
                end_if
              of 2 do // <= 
                if property::_is(BOR(rel1) < 0) = TRUE then
                  return(property::NRange("Negative"))
                end_if
              of 3 do
                if property::_is(BOR(rel1) = 0) = TRUE then
                  return(property::NRange("NonZero"))
                else
                  return(FAIL)
                end_if
              of 5 do // >= 
                if property::_is(BOR(rel1) > 0) = TRUE then
                  return(property::NRange("Positive"))
                elif property::_is(BOR(rel1) >= 0) = TRUE then
                  return(property::NRange("NonNegative"))
                end_if
              of 6 do // > 
                if property::_is(BOR(rel1) >= 0) = TRUE then
                  return(property::NRange("Positive"))
                end_if
            end_case;
            if domtype(BOR(rel1)) = DOM_IDENT then return(FAIL) end_if;
            //return(FAIL);
            //if property::propend then
              property::NRange("Real")
            //else
            //  FAIL
            //end_if
          end_if
        end_proc:

Rel::Residue:=
        proc(rel, OPT = hold(Exact))
        begin
          if OPT = hold(Exact) then
            if OPR(rel) = EQU then
              if domtype(BOR(rel)) = DOM_INT then
                property::Residue(BOR(rel), 0)
              elif domtype(BOR(rel)) = DOM_RAT then
                hold(_divide)(property::Residue(op(BOR(rel), 1), 0),
                              property::Residue(op(BOR(rel), 2), 0))
              else
                FAIL
              end_if
            else
              FAIL
            end_if
          else
            Rel::NRange(rel)
          end_if
        end_proc:

Rel::IVnat:=
        proc(rel1)
        begin
          if Rel::real(rel1) = FALSE then
            return(FAIL)
          end_if;
          
          case OPR(rel1)
            of 1 do return(property::IVnat(-infinity, BOR(rel1)))
            of 2 do return(property::IVnat(-infinity, [BOR(rel1)]))
            of 3 do
              return(hold(_or)(property::IVnat(-infinity, BOR(rel1)),
                               property::IVnat(BOR(rel1), infinity)))
            of 4 do return(property::IVnat([BOR(rel1)], [BOR(rel1)]))
            of 5 do return(property::IVnat([BOR(rel1)], infinity))
            of 6 do return(property::IVnat(BOR(rel1), infinity))
          end_case
        end_proc:

Rel::IV:=
        proc(rel1)
          local ret;
        begin
          if Rel::real(rel1) = FALSE then
            return(FAIL)
          end_if;
          ret:= Rel::IVnat(rel1);
          if type(ret) = "IVnat" then
            property::IV::check(ret, Type::Real)
          else
            map(ret, property::IV::check, Type::Real)
          end_if
        end_proc:

Rel::Complex:=
        proc(rel)
        begin
          case Rel::real(rel)
            of TRUE do
              property::Complex(rel, Type::Zero);
              break
            of FALSE do // = Complex constant
              //if OPR(rel) = EQU then
                property::Complex(property::Rel(hold(_equal), Re(BOR(rel))),
                                  property::Rel(hold(_equal), Im(BOR(rel))));
              //else
              //  property::Complex()
              //end_if;
              break
            otherwise
              property::Complex()
          end_case
        end_proc:

// Relation -> Ausdruck 
Rel::expr:=
        proc(rel, ident = null())
        begin
          if domtype(ident) <> DOM_NULL then
            if type(rel) = "Rel" then 
              if OPR(rel) > 4 then
                eval(op(Rel::__op, 7 - OPR(rel)))(BOR(rel), ident)
              else
                eval(op(Rel::__op, OPR(rel)))(ident, BOR(rel))
              end_if
            else
              rel
            end_if
          else
            rel
          end_if
        end_proc:

//Rel::expr2text:=
//        proc(rel1)
//        begin
//          "property::Rel(".
//          expr2text(op(Rel::__op, OPR(rel1))).
//          ", ".expr2text(BOR(rel1)).")"
//        end_proc:

Rel::prop2set:=
        proc(rel)
        begin
          Dom::Interval::convert(Rel::IVnat(rel))
        end_proc:

Rel::Test:=
        proc(ex, ident = hold(x))
          //option hold;
        begin
          if testtype(ex, Type::Relation) then
            if op(ex, 1) = ident then
              return(Rel(op(ex, 0), op(ex, 2)))
            else
              return(Rel(-op(ex, 0), op(ex, 1)))
            end_if
          else
            map(ex, Rel::Test, ident)
          end_if
        end_proc:

//Rel::Z:= "":

Rel::print:=
        proc(rel1)
        begin
          _if(contains({1,2,3,4,5,6}, OPR(rel1)),
              op(Rel::__po, OPR(rel1)), "?").
          " ".expr2text(BOR(rel1))
        end_proc:

unalias(BO):
unalias(BOR):
unalias(OP):
unalias(OPR):
unalias(Rel):
