// $Date: 2000/11/28 11:38:21 $ $Author: bij $ $Revision: 1.18 $
      
//
//   Residue - domain of residue Modes
//

alias(Residue = property::Residue):
alias(NRange = property::NRange):
alias(Rem(x) = op(extop(x, 5), 1)):
alias(REM = [5, 1]):
alias(Mod(x) = op(extop(x, 5), 2)):
alias(MOD = [5, 2]):
alias(Set(x) = op(extop(x, 5), 3)):
alias(SET = [5, 3]):
alias(Range(x) = DOM_INT::NRange(x, hold(Sloppy))):

Residue:= newDomain("property::Residue"):
Residue::info:= "Domain of residue classes":
Residue::domtype:= ()->Type:
Residue::type:= "Residue":
Residue::prop:= TRUE:
//Residue::autoload:= TRUE:
Residue::testtype:= Type::testtype:

Residue::noTrace:= {hold(print), hold(testtype), hold(domtype)}:

Residue::ax:= []:

// remainder, modulus
// modulus '0' = integer 'RM'
Residue::new:=
  proc(RM = 0, CL = 0, ST = Type::Integer)
  begin
    if CL = 0 then
      if domtype(RM) = DOM_INT then
        new(Residue, "Residue", FALSE, FALSE, FALSE,
            [RM, CL, Range(RM)])
      elif domtype(RM) = DOM_RAT then
        property::Rel(hold(_equal), RM)
      else
        error("wrong remainder '".expr2text(RM)."'('DOM_INT' expected)")
      end_if
    elif domtype(RM) <> DOM_INT then
      error("wrong remainder '".expr2text(RM)."'('DOM_INT' expected)")
    elif domtype(CL) <> DOM_INT then
      error("wrong class '".expr2text(CL)."'('DOM_INT' expected)")
    else
      if CL < 0 then
        CL:= -CL
      end_if;
      if type(ST) <> "NRange" then
        if ((ST:= property::_convert(hold(Sloppy), ST, property::NRange))) = FAIL then
          // Verallgemeinerung
          ST:= Type::Integer
        end_if
      //elif property::NRange::is(ST, Type::Integer) <> TRUE then
        //error("wrong range '".expr2text(ST)."'(subset of 'Type::Integer' expected)")
      end_if;
      new(Residue, "Residue", FALSE, FALSE, FALSE,
          [modp(RM, CL), CL, ST + RM]) // + RM ??
    end_if
  end_proc:

Residue::remainder:=
  proc(rs : Residue)
  begin
    Rem(rs)
  end_proc:

Residue::modulus:=
  proc(rs : Residue)
  begin
    Mod(rs)
  end_proc:

Residue::range:=
  proc(rs : Residue)
  begin
    Set(rs)
  end_proc:

Residue::constant:=
  proc(rs : Residue)
  begin
    bool(Mod(rs) = 0)
  end_proc:

Residue::Numeric:=
  proc(rs : Residue, OPT = FAIL)
  begin
    if OPT <> FAIL then
      Rem(rs)
    else
      bool(Mod(rs) = 0)
    end_if
  end_proc:

Residue::is:=
  proc(rs1, rs2)
    local GGT;
  begin
    if type(rs1) <> "Residue" then
      return(UNKNOWN)
    elif contains({"NRange", "IV", "Complex"}, type(rs2)) then
      return(property::implies(Set(rs1), rs2))
    elif Mod(rs1) = 1 and type(rs2) <> "Residue" then
      return(property::implies(Set(rs1), rs2))
    elif domtype(rs1) = DOM_INT then
      return(Residue::is(Residue(rs1, Mod(rs2), Range(rs1)), rs2))
    elif type(rs2) <> "Residue" or type(rs1) <> "Residue" then
      return(UNKNOWN)
    end_if;
    if Mod(rs1) = Mod(rs2) then
      return(bool(Rem(rs1) = Rem(rs2) and property::implies(Set(rs1), Set(rs2))))
    elif Mod(rs1)*Mod(rs2) = 0 then
      if Mod(rs2) <> 0 then
        return(bool(modp(Rem(rs1), Mod(rs2)) = Rem(rs2)) and property::implies(Set(rs1), Set(rs2)))
      else // Mod(rs1) <> 0, sonst Mod(rs1) = Mod(rs2)
        return((bool(Rem(rs1) = modp(Rem(rs2), Mod(rs1))) and bool(Rem(rs1) <> 0 or UNKNOWN))
               and UNKNOWN) // rs2 const!
      end_if
    elif type(Mod(rs1)/Mod(rs2)) = DOM_INT then
      if Rem(rs2) = 0 then
        return(bool(domtype(Rem(rs1)/Mod(rs2)) = DOM_INT and property::implies(Set(rs1), Set(rs2))))
      else
        return(bool(modp(Rem(rs1), Mod(rs2)) = Rem(rs2) and property::implies(Set(rs1), Set(rs2))))
      end_if
    elif modp(Rem(rs1), (GGT:= igcd(Mod(rs1), Mod(rs2)))) <> modp(Rem(rs2), GGT) then
      return(FALSE)
    end_if;
    UNKNOWN
  end_proc:

Residue::sign:=
  proc(rs : Residue)
  begin
    if Mod(rs) = 0 then
      sign(Rem(rs))
    elif Mod(rs) = 1 then
      sign(Set(rs))
    else
      {1, 1} union {_if(Rem(rs) = 0, 0, null())}
    end_if
  end_proc:

Residue::_plus:=
  proc()
    local ARGS, RET, const;
  begin
    if has((ARGS:= map([args()], type)), DOM_INT) then
      ARGS:= split([args()], X -> domtype(X) <> DOM_INT);
      RET:= Residue::_plus(op(op(ARGS, 1)));
      const:= op(op(ARGS, 2));
      if type(RET) = "Residue" then
        if Mod(RET) > 1 then
          return(Residue((Rem(RET) + const) mod Mod(RET), Mod(RET),
                         Set(RET)/* + const*/)) // const?
        else
          return(Residue((Rem(RET) + const), Mod(RET),
                         Set(RET)/* + const*/)) // const?
        end_if
      //else
      //  return(RET*const)
      end_if
    elif has(ARGS, DOM_RAT) then
      ARGS:= split([args()], X -> domtype(X) <> DOM_RAT);
      RET:= Residue::_plus(op(op(ARGS, 1)));
      const:= op(op(ARGS, 2));
      if type(RET) = "Residue" then
        if Mod(RET) = 0 then
          // const
          return(property::Rel(hold(_equal), Rem(RET) + const))
        else
          return(NRange::_and(NRange::_invert(Set(RET)), not Set(RET)))
        end_if
      end_if
    end_if;
//     if has(map([args()], type), DOM_INT) then
//       ARGS:= split([args()], X -> domtype(X) <> DOM_INT);
//       RET:= Residue::_plus(op(op(ARGS, 1)));
//       if type(RET) = "Residue" then
//         return(Residue((Rem(RET) + op(op(ARGS, 2))) mod Mod(RET),
//                        Mod(RET), Set(RET) + op(op(ARGS, 2))))
//       else
//         return(RET + op(op(ARGS, 2)))
//       end_if
//     end_if;
    property::_Op
    (Residue, _plus,
     proc(rs1, rs2)
       local GGT;
     begin
       if Mod(rs1)*Mod(rs2) = 0 then
         if Mod(rs1) = 0 then
           if Mod(rs2) = 0 then
             return(Residue::new(Rem(rs1) + Rem(rs2)))
           else
             return(Residue::new(Rem(rs1) + Rem(rs2), Mod(rs2),
                                 Set(rs2) + Range(Rem(rs1))))
           end_if;
         elif Mod(rs2) = 0 then
           return(Residue::new(Rem(rs1) + Rem(rs2), Mod(rs1),
                               Set(rs1) + Range(Rem(rs2))))
         end_if
       end_if;
       GGT:= igcd(Mod(rs1), Mod(rs2));
       if GGT = 1 then
         Set(rs1) + Set(rs2)
       else
         Residue::new(Rem(rs1) + Rem(rs2), GGT, Set(rs1) + Set(rs2))
       end_if
     end_proc, args())   
  end_proc:

Residue::_negate:=
  proc(rs)
  begin
    if Mod(rs) = 0 then
      Residue::new(-Rem(rs), 0, -Set(rs))
    else
      Residue::new(Mod(rs) - Rem(rs), Mod(rs), -Set(rs))
    end_if
  end_proc:

Residue::_mult:=
  proc()
    local ARGS, RET, const;
  begin
    if has((ARGS:= map([args()], type)), DOM_INT) then
      ARGS:= split([args()], X -> domtype(X) <> DOM_INT);
      RET:= Residue::_mult(op(op(ARGS, 1)));
      const:= op(op(ARGS, 2));
      if type(RET) = "Residue" then
        if Mod(RET) > 1 then
          return(Residue((Rem(RET)*const) mod (Mod(RET)*const), Mod(RET)*const,
                         Set(RET)*sign(const)))
        else
          return(Residue((Rem(RET)*const), Mod(RET)*const,
                         Set(RET)*sign(const)))
        end_if
      //else
      //  return(RET*const)
      end_if
    elif has(ARGS, DOM_RAT) then
      ARGS:= split([args()], X -> domtype(X) <> DOM_RAT);
      RET:= Residue::_mult(op(op(ARGS, 1)));
      const:= op(op(ARGS, 2));
      if type(RET) = "Residue" then
        if Mod(RET) > 1 then
// fprint(0, "1", (Rem(RET)*op(const, 1)) mod (Mod(RET)*op(const, 1)),
//                                           Mod(RET)*op(const, 1),
//                                           Set(RET)*sign(const), op(const, 2));
          return(Residue::_divide(Residue((Rem(RET)*op(const, 1)) mod (Mod(RET)*op(const, 1)),
                                          Mod(RET)*op(const, 1),
                                          Set(RET)*sign(const)), op(const, 2)))
        else
// fprint(0, "2", (Rem(RET)*op(const, 1)), Mod(RET)*op(const, 1),
//                                           Set(RET)*sign(const), op(const, 2));
          return(Residue::_divide(Residue((Rem(RET)*op(const, 1)), Mod(RET)*op(const, 1),
                                          Set(RET)*sign(const)), op(const, 2)))
        end_if
      end_if
//     elif has(ARGS, "NRange") then
//       ARGS:= split([args()], X -> type(X) = "NRange");
//       RET:= NRange::_mult(op(op(ARGS, 1)));
//       if NRange::is(RET, NRange("Integer")) <> TRUE then
//         return(_mult(RET, map(op(op(ARGS, 2)), X -> if type(X) = "Residue" then Set(X) else X end_if)))
//       end_if
    end_if;
    property::_Op
    (Residue, _mult,
     proc(rs1, rs2)
       local GGT;
     begin
       if Mod(rs1)*Mod(rs2) = 0 then
         if Mod(rs1) = 0 then
           if Mod(rs2) = 0 then
             return(Residue::new(Rem(rs1)*Rem(rs2)))
           else
             return(Residue::new(Rem(rs1)*Rem(rs2), Rem(rs1)*Mod(rs2), Set(rs2)*Range(Rem(rs1))))
           end_if;
         elif Mod(rs2) = 0 then
           return(Residue::new(Rem(rs1)*Rem(rs2), Mod(rs1)*Rem(rs2), Set(rs1)*Range(Rem(rs2))))
         end_if
       end_if;
       if TRUE then
         GGT:= igcd(Mod(rs1)*Rem(rs2), Rem(rs1)*Mod(rs2), Mod(rs1)*Mod(rs2));
         Residue(Rem(rs1)*Rem(rs2), GGT, Set(rs1)*Set(rs2))
       else
         hold(_mult)(rs1, rs2)
       end_if
     //hold(_mult)(rs1, rs2)
     end_proc, args())
    end_proc:

Residue::_divide:=
  proc(rs1, rs2)
  begin
//fprint(0, "div", args());
    property::_Op
    (Residue, _divide,
     proc(rs1, rs2)
     begin
       if Mod(rs1)*Mod(rs2) = 0 then
         if Mod(rs1) = 0 then
           if Mod(rs2) = 0 then // const/const
             if type(Rem(rs1)/Rem(rs2)) = DOM_INT then
               return(Residue::new(Rem(rs1)/Rem(rs2)))
             else
               return(property::Rel(_equal, Rem(rs1)/Rem(rs2)))
             end_if
           else // const/res
             return(Residue::new(Rem(rs1)*Rem(rs2), Rem(rs1)*Mod(rs2), Set(rs2)*Range(Rem(rs1))))
           end_if;
         elif Mod(rs2) = 0 then // res/const
           if Rem(rs1) = 0 then
             if type(Mod(rs1)/Rem(rs2)) = DOM_INT then
               return(Residue::new(0, Mod(rs1)/Rem(rs2), Set(rs1)*Set(rs2)))
             else
               return(NRange::_mult(Set(rs1), 1/Rem(rs2))); //?
               return(NRange::_divide(Set(rs1), Rem(rs2)))
             end_if
           elif type(Rem(rs1)/Rem(rs2)) = DOM_INT and type(Mod(rs1)/Rem(rs2)) = DOM_INT then
             return(Residue::new(Rem(rs1)/Rem(rs2), Mod(rs1)/Rem(rs2), Set(rs1)*Set(rs2)))
           else
             return(NRange::_mult(Set(rs1), NRange::_invert(Set(rs2)))); //?
             return(NRange::_divide(Set(rs1), Set(rs2)))
           end_if
         end_if
       end_if;
       // Mod1 und Mod2 <> 0
       if Mod(rs1) = Mod(rs2) then
         if Rem(rs2) = 0 then
           return(NRange::_mult(Set(rs1), NRange::_invert(Set(rs2)))); //?
           return(NRange::_divide(Set(rs1), Set(rs2)))
         elif type(Rem(rs1)/Rem(rs2)) = DOM_INT then
           return(Residue::new(Rem(rs1)/Rem(rs2), Rem(rs1), NRange::_or(Set(rs1), Set(rs2))))
         end_if
       elif type(Mod(rs1)/Mod(rs2)) = DOM_INT then
         if Rem(rs1) = 0 then
           if Rem(rs2) = 0 then
             return(Residue::new(0, Mod(rs1)/Mod(rs2), Set(rs1)*Set(rs2)))
           elif type(Rem(rs1)/Rem(rs2)) = DOM_INT then
             return(Residue::new(Rem(rs1)/Rem(rs2), Mod(rs1)/Mod(rs2), Set(rs1)*Set(rs2)))
           end_if
         elif Rem(rs2) = 0 then
           if type(Rem(rs1)/Mod(rs2)) = DOM_INT then
             return(Residue::new(Rem(rs1)/Mod(rs2), Mod(rs1)/Mod(rs2), Set(rs1)*Set(rs2)))
           end_if
         end_if
       //elif Rem(rs1) = 0 and Rem(rs2) = 0 then
       //  NRange::_mult(NRange::_divide(Set(rs1), Set(rs2)), Rem(rs1)/Rem(rs2))
       end_if;
       return(NRange::_mult(Set(rs1), NRange::_invert(Set(rs2)))); //?
       NRange::_divide(Set(rs1), Set(rs2))
     end_proc, args())
  end_proc:

Residue::_invert:=
  proc(rs)
  begin
    if Mod(rs) = 0 then
      property::Rel(hold(_equal), 1/Rem(rs))
    //elif Mod(rs) = 1 then
    //  NRange::_invert(Set(rs))
    else
      NRange::_invert(Set(rs))
    end_if
  end_proc:

Residue::_power:=
  proc(rs, pow)
    local EVEN;
  begin
    if type(rs) <> "Residue" then
      if testtype(rs, Type::Real) then
        return(Residue::_power(Residue::new(rs), pow))
      elif domtype(rs) = DOM_COMPLEX then
        return(property::Complex::_power(rs, pow))
      elif type(rs) = "NRange" then
        return(NRange::_power(Set(rs), Set(pow)))
      else
        return(hold(_power)(rs, pow))
      end_if
    end_if;
    // type(rs) = "Residue"
    if testtype(pow, Type::Numeric) = TRUE then
      if domtype(pow) = DOM_INT then
        // power mod Mod(rs)
        if pow >= 0 then
          Residue::new(Rem(rs)^pow, Mod(rs), Set(rs)^pow)
        else // pow < 0
          Residue::_invert(Residue::_power(rs, -pow))
        end_if
      else
        Set(rs)^pow + Mod(rs)
      end_if
    elif type(pow) = "Residue" then
      if Residue::is(pow, Residue(0, 2)) = TRUE then
        EVEN:= NRange::abs
      else
        EVEN:= id
      end_if;
      if Mod(pow) = 0 then
        Residue::_power(rs, Rem(pow))
      elif Mod(pow) = 1 then
        NRange::_power(Set(rs), EVEN(Set(pow)) + Rem(rs))
      elif Rem(pow) = 0 then
        Residue::new(Rem(rs)^Mod(pow), Mod(rs)^Mod(pow), EVEN((Set(rs)^Set(pow))^Mod(pow)))
      else
        EVEN(NRange::_power(Set(rs), Set(pow))) + Rem(rs)
      end_if
    elif type(pow) = "NRange" then
      NRange::_power(Set(rs), pow)
    else
      hold(_power)(rs, pow)
    end_if
  end_proc:

Residue::NRange:=
  proc(rs, OPT = hold(Exact))
  begin
    if OPT = hold(Exact) then
      if Mod(rs) = 1 then
        Set(rs)
      elif Mod(rs) = 0 then
        Range(Rem(rs))
      else
        FAIL
      end_if
    else
      Set(rs)
    end_if
  end_proc:

Residue::Complex:=
  proc(rs, OPT = hold(Exact))
  begin
    if OPT = hold(Exact) then
      if Mod(rs) = 1 then
        property::Complex(Set(rs), Type::Zero)
      else
        FAIL
      end_if
    else
      property::Complex(Set(rs), Type::Zero)
    end_if
  end_proc:

Residue::Interval:=
  proc(rs, OPT = hold(Exact))
  begin
    if OPT = hold(Exact) then
      FAIL
    else
      Type::Interval(-infinity, infinity, Set(rs))
    end_if
  end_proc:

Residue::Rel:=
  proc(rs, OPT = hold(Exact), EQ = FAIL)
  begin
    if OPT = hold(Exact) then
      if Mod(rs) = 0 then
        return(property::Rel(hold(_equal), Rem(rs)))
      else
        return(FAIL)
      end_if
    end_if;
    if domtype(EQ) = DOM_INT then
      if (EQ mod Mod(rs)) = Rem(rs) then
        property::Rel(_equal, EQ)
      else
        property::Rel(_unequal, EQ)
      end_if
    elif Mod(rs) = 0 then
      property::Rel(_equal, Rem(rs))
    else
      if not contains(sign(Set(rs)), -1) then
        if not contains(sign(Set(rs)), 0) then
          NRange::Rel(Set(rs)) + Rem(rs) + (Mod(rs) - 1)
        else
          NRange::Rel(Set(rs)) + Rem(rs)
        end_if
      elif not contains(sign(Set(rs)), 1) then
        NRange::Rel(Set(rs)) - Rem(rs)
      else
        NRange::Rel(Set(rs))
      end_if
    end_if
  end_proc:

// 
Residue::Z:= "is":

Residue::print:=
  proc(rs)
  begin
//     if Mod(rs) = 2 then
//       hold(_and)(op([hold(Type::Even), hold(Type::Odd)], Rem(rs) + 1), Set(rs))
//     else
//       hold(_and)(hold(_mod)(Rem(rs), Mod(rs)), Set(rs))
//     end_if
    if Rem(rs) <> 0 then
      if Mod(rs) = 1 then
        Set(rs)
      elif Mod(rs) = 0 then
        "= ".expr2text(Rem(rs))
      else
        hold(_plus)(hold(_mult)(Mod(rs), Set(rs)), Rem(rs))
      end_if
    else
      if Mod(rs) = 1 then
        Set(rs)
      elif Mod(rs) = 0 then
        "= ".expr2text(Rem(rs))
      else
        hold(_mult)(Mod(rs), Set(rs))
      end_if
    end_if
  end_proc:

unalias(Set):
unalias(Mod):
unalias(Rem):
unalias(Residue):
