// $Date: 2000/05/21 14:59:07 $ $Author: bij $ $Revision: 1.4 $ 

/*
  The function attribute series for RootOf

  series(RootOf(f,x),y,ord) -> slot(RootOf, "series")(f,x,y,ord)                                

  returns a series s such that f(s)=O(y^n) for n as large as possible

  If there are several expansions, a sequence of all expansions is
    returned


*/

                                  
RootOf:=slot
(RootOf,"series",
 proc(f, x, y, ord)
   local i, j, k, sol,sols,g, currentindex,eq, polys, newpolys,
   newcurrentindex, newsol, X; 
 begin

   userinfo(3, "Series attribute of RootOf called with args ".
            expr2text(args()));
   if x=y then
     error("Cannot expand w.r.t. RootOf variable")
   end_if;

 // solve f(z(x)) \equiv 0 \bmod x^{k+1} for z (for k=ord) 

   X:=[genident() $i=1..ord];

   g:=subs(f, x=_plus(X[i+1]*y^i $i=0..ord-1 ));

 
   // system (coeff(g,y,i)_i) is in triangular form ! 

   // compute a list of all possible coefficient lists sol 

   /* for each branch j, sol[j] always contains the list of coefficients
       computed for this branch so far; polys[j] contains the poly
       obtained from g by substituting the X's by these coeffs;
       and currentindex[j] contains the smallest index k for which
       coeff(polys[j], y, k) may be nonzero

   */

   
                                                           
   currentindex:=[0];
   sol:=[[]];
   polys:=[g];
   
   for i from 1 to ord do
     userinfo(15, "Determining ".expr2text(i-1)."th coefficient");

     newpolys:=[];
     newsol:=[];
     newcurrentindex:=[];
     
     for j from 1 to nops(sol) do

       while iszero((eq:=coeff(polys[j], y, currentindex[j]))) do
         currentindex[j]:=currentindex[j]+1
       end_while;

       userinfo(15, "Solving ".expr2text(eq)." for ".expr2text(X[i]));
       
       sols:=solve(eq, X[i]);

       userinfo(15, "Set of solutions: ".expr2text(sols));
       
       if type(sols)<>DOM_SET then
         // can happen if solution is a RootOf itself 
         warning("Possible loss of solution (branch ".expr2text(j).")")
       end_if;
 
       if sols={} or type(sols)<>DOM_SET then
         userinfo(10, "No solution found in branch ".expr2text(j));
       else
         if nops(sols)>1 then
           userinfo(10, "Splitting branch ".expr2text(j).
                    "into ".expr2text(nops(sols))." new branches");
         end_if;

         // each solution corresponds to a new branch 
 
         newsol:=append(newsol,append(sol[j],
                 op(sols,k)) $k=1..nops(sols) );

         // in each branch, substitute the coefficient found 
 
         newpolys:=append(newpolys, expand(subs(polys[j], X[i]=op(sols,k)))
                         $k=1..nops(sols));

 
         newcurrentindex:=append(newcurrentindex, currentindex[j]+1 $nops(sols));
       end_if;

     end_for;

     polys:=newpolys;

     userinfo(15, "List of polys is ".expr2text(polys));
 
     sol:=newsol;
     currentindex:=newcurrentindex;
 
     userinfo(15, "List of new approximations is: ".expr2text(sol));
 
   end_for;

   if nops(sol)=0 then
     warning("No series expansion found (no roots?)");
     return(FAIL)
   end_if;

 
   op(map(sol, u -> normal(Series::Puiseux::create(1,0,ord, u, y ))))
 
   end_proc   
):

