/*--
        besselI/Series -- the function attribut "series" for besselI
--*/

besselI := slot(besselI,"series",proc(v,f,x,n,dir)
   local fv,k,k2,l,mu,s,sc,sw,P,x2,x8,t,vabs,tt;
begin
   fv:=float(v):
   if not(testtype(fv,Type::Complex)) or has(v,x) then FAIL 
   else
      // recursively expand the argument
      tt := series(f, x, n, if dir = FAIL then null() else dir end_if);

      if dir <> FAIL then // directional expansion
        l:=limit(f,x, if dir = Real then null() else dir end_if);
      elif domtype(tt) = Series::Puiseux then
        if ldegree(tt) = FAIL then // order too small
          l := FAIL;
        elif ldegree(tt) >= 0 then
          l := coeff(tt, x, 0);
        else
          return(FAIL)
        end_if
      else
        l := FAIL
      end_if;

      if iszero(l) then // expansion in 0: use definition 9.1.10 
         x2:=x^2;
         if iszero(v) then
            //Abramowitz and Stegun 9.6.12
            t:=1;
            s:=_plus(1,(t:=t*1/4*x2/k^2)$ k=1..trunc(n/2));
         else
            //Abramowitz and Stegun 9.6.10 combined with
            //reflection formula of 9.6.6
            vabs:=abs(v);
            s:=1/gamma(vabs+1):
            t:=s;
            s:=_plus(s,(t:=t*1/4*x2/k/(k+vabs))$ k=1..trunc(n/2));
            s:=((x/2)^vabs)*s;
         end_if;
// More efficient: use create !!!
         s:=series(s,x,n);
         if f=x then return(s) else return(s @ tt) end_if
      elif l=infinity or l=-infinity then
         // Abramowitz and Stegun 9.7.1
         // changed z into 1/x (z->infinity when x->0, Right) 
         // We keep in mind a reflection formula which applies
         // to the complex plane for "v" integer:
         // 1) I_(-v) (z) = I_(v) (z)  reflection formula    (9.6.6)
         //
         // For expansions about -infinity, we use the same phase 
         // factor "sw" as for the besselJ functions.
         // WARNING: expansion is only valid for the half-plane!
         if type(v)=DOM_INT then
          sc:=sign(l)^v;
         else
          sc:=1;
         end_if;
         x8:=8/x;
         mu:=4*v^2;
         t:=1;
// More efficient: use create !!!
         P:=series(_plus(1,(k2:=2*k;t:=-t*(mu-(k2-1)^2)/k/x8)$k=1..(n-1)),x,n):
         if sign(l) > 0 or domtype(v)=DOM_INT then
          if f <> 1/x and f <> -1/x then
            s:=series(sign(l)/f,x,n,
                      if dir = FAIL then null() else dir end_if); 
            P:=P @ s; 
          end_if:
          P := P * series(sqrt(sign(l)/f), x, n,
                          if dir = FAIL then null() else dir end_if); 
          P := Series::gseries::convert(P);
          return(Series::gseries::scalmult(P,
                   sc/2^(1/2)/PI^(1/2), exp(sign(l)*f)))
         else
          if f <> -1/x then
            s:=series(sign(l)/f,x,n,
                      if dir = FAIL then null() else dir end_if);
            P:=P @ s;
          end_if:
          P := P * series(f^(-1/2), x, n,
                          if dir = FAIL then null() else dir end_if);
          P := Series::gseries::convert(P);
          sw:= (-1)^((v+1/2)*signIm(f));
          return(Series::gseries::scalmult(P,
                   1/2^(1/2)/PI^(1/2), sw*exp(sign(l)*f)))
         end_if: 
      end_if;

      Series::unknown(besselI(v,f),x,n,dir)

   end_if
end_proc):
