/*--
        besselJ/Series -- the function attribut "series" for besselJ
--*/

besselJ := slot(besselJ,"series",proc(v,f,x,n,dir)
   local fv,k,k2,k4,l,s,sc,P,Q,xi,t,vabs,v2,x2,s0,sw,tt;
begin
   fv:=float(v):
   if not(testtype(fv,Type::Complex)) or has(v,x) then FAIL
   else
      // recursively expand the argument
      tt := series(f, x, n, if dir = FAIL then null() else dir end_if);

      if dir <> FAIL then // directional expansion
        l:=limit(f,x,if dir = Real then null() else dir end_if);
      elif domtype(tt) = Series::Puiseux then
        if ldegree(tt) = FAIL then // order too small
          l := FAIL;
        elif ldegree(tt) >= 0 then
          l := coeff(tt, x, 0);
        else
          return(FAIL)
        end_if
      else
        l := FAIL
      end_if;

      if iszero(l) then // expansion in 0: use definition 9.1.10 
         x2:=x^2;
         if iszero(v) then
            //Abramowitz and Stegun 9.1.12
            t:=1;
            s:=_plus(1,(t:=-t*1/4*x2/k^2)$ k=1..trunc(n/2));
         else
            //Abramowitz and Stegun 9.1.10 combined with
            // reflection formula 9.1.5
            vabs:=abs(v);
            s:=1/gamma(vabs+1):
            t:=s;
            s:=_plus(s,(t:=-t*1/4*x2/k/(k+vabs))$ k=1..trunc(n/2));
            s:=(x/2)^vabs*(sign(v)^v)*s;
         end_if:
// More efficient: use create !!!
         s:=series(s,x,n);
         if f=x then return(s) else return(s @ tt) end_if
      elif l=infinity or l=-infinity then 
         // formula 9.2.5 with 9.2.9 and 9.2.10 
         // changed z into 1/x (z->infinity when x->0, Right) 
         // 9.2.5 is Hankel's asymptotic formula which applies when
         // |arg(z)| < PI (i.e. excludes negative real axis)
         //
         // We keep in mind two reflection formulae which apply
         // to the complex plane for "v" integer:
         // 1) J_(-v) (z) = (-1)^v *J_(v) (z)  reflection formula    (9.1.5)
         // 2) J_v(-z) = (-1)^v * J_(v) (z)    analytic continuation (9.1.35)
         //
         // and in general, analytic continuation as expressed in 9.1.35:
         // J_(v) (z exp(m*PI*I) = exp(m*v*PI*I) J_(v) (z) ["m" is an integer]
         // MUST BE CAREFULLY WORKED OUT.
         // i.e. in the non-integer case, we have to be careful about expansions
         // about -infinity, a phase factor "sw" must be worked out.
         x2:=x^2;
         if type(v)=DOM_INT then
          vabs:=abs(v);
          v2:=2*vabs;  
          sc:=sign(l)^v;
          if v < 0 then
           sc:=sc*((-1)^v);
          end_if:
         else
          sc:=1;
          v2:=2*v;  
          vabs:=v; 
         end_if;
         t:=1;
// More efficient: use create!
         P:=series(_plus(1,(k2:=2*k; k4:=4*k;
         t:=-t*1/128*(k4+v2-3)*(k4+v2-1)*(k4-v2-1)*(k4-v2-3)*x2/(k2-1)/k)
             $ k=1..trunc((n-1)/2)),x,n);
         s0:=1/8*(v2-1)*(v2+1)*x;
         t:=s0;
         Q:=series(_plus(s0,(k2:=2*k; k4:=4*k;
         t:=-t*1/128*(k4+v2-1)*(k4+v2+1)*(k4-v2+1)*(k4-v2-1)*x2/k/(k2+1))
             $ k=1..trunc((n-3)/2)),x,n);
         if sign(l) > 0 or type(v)=DOM_INT then
          if f<>1/x and f<>-1/x then
              s:=series(sign(l)/f,x,n,
                        if dir = FAIL then null() else dir end_if); 
              P:=P @ s;
              Q:=Q @ s
          end_if;
          xi:=sign(l)*f-(vabs/2+1/4)*PI;
          s := series((1/sign(l)/f)^(1/2),x,n,
                      if dir = FAIL then null() else dir end_if);
          P := P * s;
          Q := Q * s;
// This is not very nice, since the sin and cos become coefficients
// of Series::Puiseux, but gseries currently cannot handle the sum
// and since it's a directional expansion, it is ok as a workaround.
// Similarly at the "return" below.
          return(sc*2^(1/2)/PI^(1/2)*(P*expand(cos(xi))-Q*expand(sin(xi))));
         else
          if f<>-1/x then
            s:=series(sign(l)/f,x,n,
                      if dir = FAIL then null() else dir end_if);
            P:=P @ s;
            Q:=Q @ s
          end_if;
          xi:=sign(l)*f-(v/2+1/4)*PI;
          // phase factor sw represents f^v/((-f)^v) =(-1)^(v*signIm(f));
          // However, we make use of the identity:
          // (f^v/(-f)^v)*(-1/f)^(-1/2) = (-1)^((v+1/2)*signIm(f))*f^(-1/2)
          // i.e. the 1/sqrt(z) of Hankel's asymptotic expansion is modified
          // to ensure a clean result with the transformation z -> 1/x
          s := series(f^(-1/2),x,n,
                      if dir = FAIL then null() else dir end_if);
          P := P * s;
          Q := Q * s;
          sw:= (-1)^((v+1/2)*signIm(f));
          return(2^(1/2)/PI^(1/2)*sw*(P*expand(cos(xi))-Q*expand(sin(xi))));
         end_if
      end_if
   end_if;

   Series::unknown(besselJ(v,f),x,n,dir)

end_proc):
