/*--
        besselK/Series -- the function attribut "series" for besselK
--*/

besselK := slot(besselK,"series",proc(v,f,x,n,dir)
   local fv,h,k,k2,l,mu,P,x8,s,t,x2,res1,res2,vabs,tt;
begin
   fv:=float(v):
   if not(testtype(fv,Type::Complex)) or has(v,x) then FAIL
   else
      // recursively expand the argument
      tt := series(f, x, n, if dir = FAIL then null() else dir end_if);

      if dir <> FAIL then // directional expansion
        l:=limit(f,x,if dir = Real then null() else dir end_if);
      elif domtype(tt) = Series::Puiseux then
        if ldegree(tt) = FAIL then // order too small
          l := FAIL;
        elif ldegree(tt) >= 0 then
          l := coeff(tt, x, 0);
        else
          return(FAIL)
        end_if
      else
        l := FAIL
      end_if;

      if iszero(l) then 
      // expansion in 0: use definition 9.1.10 
        if type(v) <> DOM_INT then
          // Abramowitz and Stegun 9.6.2
          t := (PI/2)*(besselI(-v,x)-besselI(v,x))/sin(v*PI);
        else 
          x2:=x^2;
          if iszero(v) then
          // Abramowitz and Stegun 9.6.13
           if n > 2 then
            res2:=x2/4;
            h:=1; 
            t:=res2;
            for k from 2 to trunc(n/2) do
             h := h+1/k;
             t := t*1/4*x2/k:
             res2 := res2+t*h;
            end_for: 
           else
            res2:=0;
           end_if:
           t:=-((ln(x/2)+EULER)*besselI(0,x)+res2);
          else
           // Abramowitz and Stegun 9.1.11 combined with
           // reflection formula of 9.6.6
           vabs:=abs(v);
           res1:=fact(vabs-1);
           t:=res1;
           res1:=_plus(res1,(t:=-t*1/4/k*x2/(vabs-k)) $ k=1..vabs-1);
           t:=1/fact(vabs);
           res2:=(psi(vabs+1)-EULER)*t;
           for k from 1 to max(-1,trunc((n-vabs-1)/2)) do
             t:= t*1/4*x2/k/(vabs+k);
             res2:=res2+(psi(k+1)+psi(k+vabs+1))*t; 
           end_for: 
           t := (1/2)*((x/2)^(-vabs)*res1+((-1)^vabs)*(x/2)^vabs*res2) + 
                  ((-1)^(vabs+1))*ln(x/2)*besselI(vabs,x);
          end_if:
        end_if:
// More efficient: use create !!!
        s:=series(t,x,n);
        if f=x then return(s) else return(s @ tt) end_if
      elif l=infinity then
         // Abramowitz and Stegun 9.7.2
         // changed z into 1/x (z->infinity when x->0, Right)
         // We keep in mind a reflection formula which applies
         // to the complex plane for "v" integer:
         // 1) K_(-v) (z) = K_(v) (z)  reflection formula    (9.6.6)
         x8:=8/x;
         mu:=4*v^2;
         t:=1;
         P:=_plus(1,(k2:=2*k;t:=t*(mu-(k2-1)^2)/k/x8)$k=1..(n-1));
// More efficient: use create !!!
         P:=series(P,x,n);
         if f<>1/x then
           s:=series(1/f,x,n,
                     if dir = FAIL then null() else dir end_if);
           P:=P @ s;
         end_if:
         P := P * series(sqrt(1/f), x, n,
                         if dir = FAIL then null() else dir end_if);
         P := Series::gseries::convert(P);
         return(Series::gseries::scalmult(P, PI^(1/2)/2^(1/2), exp(-f))):
      end_if
   end_if;

   Series::unknown(besselI(v,f),x,n,dir)

end_proc):
