/*
 * series slot of dirac - computes series(dirac(f, n), x, ord, dir)
 *
 * Computes l := limit(f, x = 0, dir), or l := f(0) if dir = FAIL
 * returns symbolic "series" call if l cannot be determined
 *
 * If l is not real (so dirac(l,n) = undefined) ==> returns error
 * otherwise, if dir = FAIL ==> symbolic "series" call
 *
 * otherwise, if l is real and dir <> FAIL:
 * If l = 0 (expansion around 0) ==> try formal Taylor expansion
 * If l <> 0 is real (so dirac(l,n) = 0) ==> returns 0
 *
 * Author: Juergen Gerhard, 19.5.00, 17.8.00
 */


dirac := slot(dirac, "series", proc(f, n, x, ord, dir)
  local l, d;
begin

  if dir = FAIL then
    // undirected expansion, compute f(0)
    if traperror((l := eval(subs(f, x = 0)))) <> 0 then
      userinfo(1, "could not compute limit of first argument of dirac");
      return(FAIL);
    end_if
  else
    // directional expansion, compute limit(f, x = 0)
    l := limit(f, x = 0, if dir = Real then null() else dir end_if);
    if type(l) = "limit" or l = FAIL then
      userinfo(1, "could not compute limit of first argument of dirac");
      return(FAIL);
    end_if;
  end_if;
 
  d := dirac(l, n);
  if l <> 0 and d = undefined then
    error("dirac is not defined for non-real expansion points");
  elif l = 0 and dir <> FAIL then // directed expansion around zero
    // symbolic expansion
    return(Series::unknown(dirac(f, n), x, ord, dir))
  elif d = 0 and dir <> FAIL then
    // directional expansion around real l <> 0
    return(Series::Puiseux::new(0, x, ord))
  else
    return(FAIL)
  end_if

end_proc):
