// $Date: 2000/06/29 11:52:26 $ $Author: jngerhar $ $Revision: 1.10 $ 

/*--
	erfc/Series -- the function attribut "series" for erfc
--*/

erfc:=slot(erfc,"series",proc(f,x,n,dir)
  local t,s,d;
begin

   // recursively expand the argument
   t:=Series::series(f,x,n,dir);

   if domtype(t) = Series::Puiseux then
     d := ldegree(t);
     if d = FAIL then // t = O(..)
	d:=Series::Puiseux::order(t);
        if d >= 0 then return(1+t)
        else
	  error("order too small");
          return(FAIL);
        end_if
     elif d < 0 then // f goes to +/-infinity 
       s:=sign(lcoeff(t));
// This is wrong if lcoeff(t) depends on x!
       if s=1 then // +infinity 
         s:=Series::Puiseux::create(1,1,n+1,Series::gen["erfc"](n),x);
         s:=Series::Puiseux::_fconcat(s, 1/t);
         s:=Series::gseries::convert(extsubsop(s,5=x));
         return(Series::gseries::scalmult(s,1,exp(-f^2)))
       elif s=-1 then // -infinity 
         s:=Series::Puiseux::create(1,1,n+1,Series::gen["erfc"](n),x);
         s:=Series::Puiseux::_fconcat(s, -1/t);
         // This is a very crude hack, since _plus does not work correctly
         // for gseries
         s := -s + Series::Puiseux::create(1, 0, n + 1, [2/exp(-f^2)], x);
         s:=Series::gseries::convert(extsubsop(s,5=x));
         return(Series::gseries::scalmult(s,1,exp(-f^2)))
       else 
	 userinfo(2,"unable to compute expansion");
         return(FAIL)
       end_if
     elif d > 0 then // expansion around 0
       // s := series(erfc(x), x)
       // = 1 - 2/sqrt(PI) * sum( (-1)^k/(2k+1)/k! * x^(2k+1) )
       s := -2/sqrt(PI);
       s := [1, s, (0, (s := -s*(2*k - 1)/(2*k + 1)/k))
                   $ k = 1..((n - 2) div 2)];
       s := Series::Puiseux::create(1, 0, n, s, x);
       if f = x then
         return(s)
       else
         return(Series::Puiseux::_fconcat(s, t))
       end_if
     end_if
   end_if;

   Series::unknown(erfc(f),x,n,dir)

end_proc):

// ensure that domain Series is loaded
Series:

// expansion of erfc(x)*exp(x^2) around infinity, leading order is 1
Series::gen["erfc"]:=proc(n) local t,i;
begin
   t:=1/sqrt(PI);
   [t,(0,(t:=-t*(2*i-1)/2))$i=1..(n-1) div 2]
end_proc:

// end of file 
