/*--
	exp/Series -- the function attribut "series" for exp
--*/


exp:=slot(exp,"series",proc(f,x,n,dir)
local s, f0, a, t, tt, v, l, p, i, k, q, li;
begin
   s:=Series::Puiseux::create(1,0,n,Series::gen["exp"](n),x);
   if f=x then s
   else
      t:=Series::series(f,x,n,dir);
      if domtype(t) <> Series::Puiseux then return(FAIL) end_if;
      v:=Series::Puiseux::ldegree(t);
      if v=FAIL then // exp(O(...)) 
         if (v:=Series::Puiseux::order(t))>0 then 1+t	// exp(O(x))=1+O(x) 
         elif v=0 then t			// exp(O(1))=O(1) 
         else // precision too low
           error("order too small");
         end_if
      elif v<0 then // exp(1/x^a) 
        if contains({Left, Right}, dir) then // directional expansion
          // f0 := negative order part of t
          f0 := expr(t + Series::Puiseux::new(1, x, 0));
          if normal(f - f0) = 0 then
            tt := null();
          else
            tt := series(exp(f-f0), x, n, dir);
          end_if;
          tt * Series::gseries::new(exp(f0),x,n,dir) // try MRVasympt 
        else // undirected expansion
          FAIL
        end_if;
      elif v=0 then // exp(f0+(f-f0)) = exp(f0)*exp(f-f0) 
	 f0:=Series::Puiseux::lcoeff(t);

         // tt := series of exp(f-f0)
         tt:=Series::Puiseux::_fconcat(s,Series::Puiseux::lmonomial(t,Rem)[2]);
         if tt = FAIL then return(FAIL) end_if;

         // p := series(1, x, n)
         p := Series::Puiseux::create(1,0,n,[1],x,0);
         a := 0;
         if has(f0, x) and has(f0, ln) then
           // try to get rid of logs in f0

           // extract constant summands a from f0
           if type(f0) = "_plus" then
             f0 := split(f0, has, x);
             a := f0[2];
	     f0 := f0[1];
           end_if;
           if type(f0) = "_plus" then
             l := [op(f0)];
           else
             l := [f0];
           end_if;
           // now lcoeff(t) = a + f0 = a + _plus(op(l)), a does not
           // contain x, and every element of l contains x

           // check whether all summands of f0 are
           // of the form k*ln(m), with rational k and m containing x.
           // In the latter case, pull m^k out of exp(f0)
           for i from 1 to nops(l) do
             li := l[i];
             k := 1;
             if type(li) = "ln" then
               li := op(li);
             elif type(li) = "_mult" then
               li := split(li, has, x);
               if type(li[1]) = "ln" then
                 k := li[2];
                 li := op(li[1]);
                 if testtype(k, Type::Rational) then
                 elif nops(l) = 1 then 
                   // handle case lcoeff(t) = const + k * ln(foo)
                   // with non-rational k separately. This is
                   // only a very crude hack to maintain backward
                   // compatibility, to compensate for missing features
                   // in gseries, and to make "limit" still work!
                   // return exp(a) * exp(f0) * exp(f - f0) as a gseries
                   tt := Series::gseries::convert(tt);
                   if tt = FAIL then return(FAIL) end_if;
                   if normal(f - f0) = 0 then // f = f0 
                     return(Series::gseries::create(
                              [[exp(a), li^k]], 0, x = 0));
                   else
                     return(map(Series::gseries::scalmult(tt, exp(a), li^k),
                            combine, exp));
                   end_if;
                 else
                   return(FAIL);
                 end_if;
               else
                 return(FAIL);
               end_if;
             else
               return(FAIL);
             end_if;
             // now l[i] = k * ln(li) with rational k, and li containing x

             q := Series::series(li^k, x, n, dir);
             if domtype(p) = Series::gseries then
               q := Series::gseries::convert(q);
             end_if;
             if traperror((p := p * q)) <> 0 then
               return(FAIL);
             end_if;
           end_for;
         elif has(f0, x) then // f0 contains x, but not ln
           return(FAIL);
         else // f0 does not contain x
           a := f0;
         end_if;
         // now lcoeff(t) = p * exp(a), where a does not contain x

         // final result := tt * p * exp(a)
         if domtype(p) = Series::gseries then
           p := Series::gseries::scalmult(p, exp(a), 1);
           tt := Series::gseries::convert(tt);
           if tt = FAIL then return(FAIL) end_if;
         else
           p := Series::Puiseux::scalmult(p, exp(a), 0);
         end_if;

         if traperror((tt := tt * p)) <> 0 then
           return(FAIL);
         end_if;
         return(map(tt, combine, exp));

// old code, now deactivated
//         # try to recognize a+K*ln(x) #
//         if has(f0, ln(x)) then
//            if not has(f0/ln(x),x) then a:=0
//            else
//               if type(f0)="_mult" then f0:=expand(f0) end_if;
//               if type(f0)="_plus" then ([f0,a,K]):=split(f0,has,x)
//               else a:=0 end_if;
//            end_if;
//            if not has((K:=f0/ln(x)),x) then
//               t:=Series::Puiseux::_fconcat(s,Series::Puiseux::lmonomial(t,Rem)[2]);
//               if type(K)=DOM_INT then t:=Series::Puiseux::scalmult(t,exp(a),K)
//               else # generalized series #
//                  t:=Series::gseries::convert(t); # extsubsop(t,5=x)); #
//                  t:=Series::gseries::scalmult(t,exp(a),x^K);
//               end_if;
//               return(t)
//            end_if;
//            error("invalid expansion") # to avoid exp(ln(x)^2) #
//         end_if;
//         Series::Puiseux::scalmult(Series::Puiseux::_fconcat(s,Series::Puiseux::lmonomial(t,Rem)[2]),exp(f0))
      else // v>0 
         Series::Puiseux::_fconcat(s,t)
      end_if;
   end_if
end_proc):

// ensure that domain Series is loaded
Series:

Series::gen["exp"]:=proc(n) local t,i;
begin
   t:=1; [1,(t:=t/i)$i=1..n-1]
end_proc:
Series::gen["exp"](0):=[]:

// end of file 
