// $Date: 2000/05/24 14:44:24 $ $Author: jngerhar $ $Revision: 1.10 $ 

/*--
	gamma/Series -- the function attribut "series" for gamma
--*/

gamma:= slot( gamma, "series", proc(f,x,n,dir)
   local l,i,a,t,d;
begin

   if dir <> FAIL then // directional expansion
     l:=limit(f,x,if dir = Real then null() else dir end_if);
     if testtype(-l, Type::NonNegInt) then // simple pole 
//       # use gamma(x+1)=x*gamma(x), formula 6.1.15 #
//       p:=_mult(f+i $ i=0..-l);
//       return(series(gamma(f-l+1)/p,x,n,dir))
// better: use reflection formula
         return(series(PI / sin(PI*f) / gamma(1 - f), x, n, dir))
     elif l=infinity then // Stirling formula, see 6.1.40 
       a:=new(Series::gseries,[[1,f^f*exp(-f)*(2*PI/f)^(1/2)]],0,1/x);
// This is probably not what gseries::new would return!
// More efficient: use create and concat !!!
       l:=bernoulli(2*i)/2/i/(2*i-1)/f^(2*i-1)$i=1..max(1,n-2);
       return(Series::gseries::_mult(a,Series::series(exp(_plus(l)),x,n,dir)))
     elif l=-infinity then 
       return(FAIL);
     end_if;
   end_if;
 
   // recursively expand the argument into a series
   t := Series::series(f,x,n,dir);

   if domtype(t) = Series::Puiseux then
     d := ldegree(t);
     if d = FAIL then
       error("order too small");
     elif d >= 0 then
       l := coeff(t, x, 0);
       if testtype(-l, Type::NonNegInt) then // simple pole, use refl. formula
         return(series(PI / sin(PI*f) / gamma(1 - f), x, n,
                       if dir = FAIL then null() else dir end_if))
       end_if;
     else // ldegree(t) < 0
       return(FAIL)
     end_if;
   end_if;

   Series::unknown(gamma(f),x,n)

end_proc):

// end of file 
