Series::Puiseux:=newDomain("Series::Puiseux"):
Series::interface:= {}:

alias(Puiseux=Series::Puiseux):

alias(BranchOrder(a)=extop(a,1)):
alias(valuation(a)=extop(a,2)): // valuation 
alias(err(a)=extop(a,3)):
alias(ListCoeffs(a)=extop(a,4)):
alias(variable(a)=op(extop(a,5),1)): // series variable 
alias(point(a)=op(extop(a,5),2)): // expansion point 

Puiseux::indet:=proc(s) begin variable(s) end_proc:
Puiseux::point:=proc(s) begin point(s) end_proc:

Puiseux::match_x_power_k:=proc(e,x) // matches x^k with k a rational 
// returns k if match successful, FALSE otherwise 
begin
   if e=1 then 0
   elif e=x then 1
   elif type(e)="_power" and op(e,1)=x and contains({DOM_INT,DOM_RAT},type(op(e,2))) then op(e,2)
   else FALSE
   end_if
end_proc:

/* these procedures are to be used from outside to
  create Puiseux expansions efficiently without knowing their
  internal structure */
Puiseux::create := proc(bo,valuation,ord,l,x,x0)
begin
   if args(0)<6 then x0:=0 end_if;
   new(Puiseux,bo,valuation,ord,l,x=x0)
end_proc:

Puiseux::ldegree:=proc() local l, n; 
 begin 
   l:=ListCoeffs(args(1));
   n:=valuation(args(1));
   while l<>[] do
      if Puiseux::iszero(l[1]) then delete l[1]; n:=n+1
      else return(n/BranchOrder(args(1)))
      end_if
   end_while;
   FAIL
 end_proc :

Puiseux::order := proc(a) begin err(a)/BranchOrder(a) end_proc:

// default test to zero 
//Puiseux::iszero:=proc() begin iszero(expr(args(1))) or Puiseux::fiszero(args(1)) end_proc :
Puiseux::iszero:=proc() begin iszero(normal(expr(args(1)))) end_proc:

// fiszero has been removed since it is too unreliable

/*
// test to zero using float for constants 
Puiseux::fiszero:=proc() begin (
   if contains({DOM_FLOAT,DOM_COMPLEX}, float(args(1))) then
      bool(numeric::isnonzero(args(1)) <> TRUE)
// old code deactivated, since it is far more unreliable
//      float(args(1)); 
//      if type(%)=DOM_FLOAT and abs(%)<10.0^(-DIGITS) then TRUE 
//      else FALSE end_if
   else FALSE end_if) end_proc :
*/

Puiseux::normal:=proc(a) // remove zero coefficients in head 
local l,v;
begin
   l:=ListCoeffs(a); // removes a0 
   if nops(l)=0 then a
   elif not Puiseux::iszero(l[1]) then a
   else
      v:=valuation(a);
      while l<>[] do
         if Puiseux::iszero(l[1]) then delete l[1]; v:=v+1 else break end_if
      end_while;
      extsubsop(a,2=v,4=l)
   end_if
end_proc:

// [branch order, valuation, ord, [coeffs]] 
Puiseux::new:= proc(e,x,Ord=ORDER)
   local i,big_O,ord,k,d,bo,l,combineexponent;
   begin
      if type(1/x)=DOM_IDENT then // x=1/t for expansions at t=infinity 
         return(series(e,1/x=infinity,Ord))
      end_if;
      ord:=proc(t) begin if type(t)="_power" and op(t,1)=x and
	type(op(t,2))=DOM_RAT then denom(op(t,2)) else 1 end_if end_proc:
      l:=indets(e,PolyExpr);
      bo:=ilcm(ord(op(l,i))$i=1..nops(l));
      // find the O-term if any, and the branching order 
      if type(e)="_plus" then e:=[op(e)] else e:=[e] end_if;
      big_O:=0;
      for i from 1 to nops(e) do
         if type(op(e,i))=O then big_O:=op(e,i); e:=subsop(e,i=null()); break end_if
      end_for;
      if big_O<>0 then 
         if nops(big_O)<>1 or (ord:=Puiseux::match_x_power_k(op(big_O),x))=FALSE
         then error("invalid big-O") end_if;
         bo:=ilcm(bo,denom(ord));
         ord:=ord*bo;
      end_if;
      e:=_plus(op(e));
      e:=subs(e,x=x^bo); // should have integer exponents now 
/*fix*/ // However, this does not work, e.g., bo=4: (1/x)^(1/2) --> (1/x^4)^(1/2)
/*fix*/ // The following routine is a crude way of fixing this.
/*fix*/ // combineexponent: (a*(b*x)^p1)^p2) -> (a*b^p1)^p2*x^(p1*p2)
/*fix*/ combineexponent:= proc(b,expo) local c;
/*fix*/ begin
/*fix*/    if not has(b, x) then return(b^expo) end_if:
/*fix*/    if b = x then return(b^expo) end_if:
/*fix*/    if type(b)= "_power" then //((a*x)^p)^expo --> (a*x)^(p*expo)
/*fix*/       return( op(b,1)^(op(b,2)*expo));
/*fix*/    end_if:
/*fix*/    if type(b)="_mult" then   // (a*x^p)^expo -> a^expo*(x^p)^expo
/*fix*/       c:= select(b, has, x);
/*fix*/       if b = c // e.g.,  b = x^(1/2)*(1/x)^(1/2)
/*fix*/                // Each factor of the product contains x
/*fix*/         then return(map(b, combineexponent, expo));
/*fix*/         else return(combineexponent(c,expo)*(b/c)^expo);
/*fix*/       end_if;
/*fix*/    end_if; 
/*fix*/    //error("should not arrive here");
/*fix*/    return(b^expo);
/*fix*/ end_proc;
/*fix*/ e:=subs(e, hold(_power) = combineexponent);
/*fix*/ e:= eval(e):

      // now find the valuation and the degree 
      k:=ldegree(e,[x]);
      if k=FAIL then return(FAIL) end_if; // not a Puiseux 
      if type(ord)<>DOM_INT then
         ord:=k+Ord
      end_if;
      d:=min(degree(e,[x]),ord-1);
      new(Puiseux,bo,k,ord,[coeff(e,[x],i)$ i=k..d],x=0);
   end_proc:

// specify variable of expansion 
Puiseux::set_var:= proc(a,X,x,x0) 
    local s, L, i;
begin
    s:= (if domtype(x0)=stdlib::Infinity then sign(x0)/x else x-x0 end_if);

    // If x0 = infinity, then X = 1/x,
    // if x0 =-infinity, then X = -1/x.
    // We  need to undo this transformation by subs(.., X=s): 

    // The coeffs may contain trancendental functions such as
    // a= (bo,v,err,[exp(X), ln(X),..])
    //   -> exp(X)*X^(v/bo) + ln(X)*X^((v+1)/bo) +...

    // First, substitute the arguments of the trancendental
    // functions in the list.
    L:= eval(subs(ListCoeffs(a), X=s));

    // For finite x0 and x0 = infinity this does the job.
    // However, for x0 = -infinity we also need to replace
    //     L[i] (-> L[i]*X^((v+i-1)/bo))
    // with v = extop(a,2), bo=extop(a,1) by
    //    (-1)^((v+i-1)/bo)*L[i]  --> (-1)^((v+i-1)/bo)*L[i]*1/x^((v+i-1)/p)):

    if x0 = -infinity then
       L:= [(-1)^( (extop(a,2)+i-1)/extop(a,1) )*L[i] $ i=1..nops(L)];
    end_if:

    extsubsop(extsubsop(a, 4 = L), 5 = (x=x0) )
end_proc:

Puiseux::subs:= proc(s) 
    local l, p, x;
begin 
    l:= subs( extop(s,5),args(2..args(0)) );
    x:= op(l,1);
    if domtype(x) <> DOM_IDENT then
        if (p:= poly(x)) = FAIL then error("invalid series variable") end_if;
        x:= op(p,2);
        if nops(x) <> 1 or domtype((x:= x[1])) <> DOM_IDENT then
            error("invalid series variable")
        end_if;
        // convert s to an expression, substitute arguments and
        // compute a series approximation of this result. This
        // ensures that the result of the given substitution 
        // is again of type Series.
        return(Series::main( subs(Puiseux::expr(s),args(2..args(0))),x=op(l,2),extop(s,3) ))
    else
        return(new(Puiseux, extop(s,1..3), subs(ListCoeffs(s),args(2..args(0))), l))
    end_if
end_proc:

Puiseux::printmult := proc(x, y) begin
  if x = 0 then null()
  elif x = 1 then y
  elif y = 1 then x
  else hold(_mult)(x, y)
  end_if
end_proc:

Puiseux::print:= proc(a,x) 
   local l,i,bo,k;
begin
      bo:=BranchOrder(a);
      k:=valuation(a);
      l:=ListCoeffs(a);
      x:=extop(a,5); 
      if domtype(op(x,2))=stdlib::Infinity then x:=1/op(x,1)
      else x:=op(x,1)-op(x,2)
      end_if;
      l:=[_mult(l[i],x^((k+i-1)/bo)) $ i=1..nops(l)];
      l:=map(l,(x->(if x=0 then null() else x end_if)));
      i:=hold(O)(x^(err(a)/bo));
      if l=[] then i else hold(_plus)(op(l),i) end_if
end_proc:

Puiseux::expr := proc(a)
local l,x,i,bo,k;
begin
   bo:=BranchOrder(a);
   k:=valuation(a);
   l:=ListCoeffs(a);
   if type(point(a))=stdlib::Infinity then x:=1/variable(a)
   else x:=variable(a)-point(a)
   end_if;
   _plus(l[i]*x^((k+i-1)/bo) $ i=1..nops(l))
end_proc:

Puiseux::multbo := proc(a,n) // multiply branching order of a by n (integer) 
local l,i,z;
begin
   l:=ListCoeffs(a);
   if l<>[] then 
      z:=0$(n-1);
      l:=[(l[i],z)$i=1..nops(l)-1,l[nops(l)]]
   end_if;
   extsubsop(a,1=BranchOrder(a)*n,2=valuation(a)*n,3=err(a)*n,4=l)
end_proc:

Puiseux::plus := proc(a,b) local ka,kb,qa,qb,la,lb,i,dif,da,db,lc,boa,bob;
begin
   if type(a)<>Puiseux then
      qb:=err(b);
      Puiseux::new(a, variable(b), qb);
      if % <> FAIL
      then a:= %;
      else series(a, extop(b,5), qb);
           if % <> FAIL 
           then a:= %;
           else error("could not convert first argument");
           end_if:
      end_if:
      qa:=err(a);
   elif type(b)<>Puiseux then
      // is b in a more general domain ? 
      b::dom::convert(a); 
      if % <> FAIL then return(%+b) end_if;
      qa:=err(a); 
      Puiseux::new(b, variable(a), qa);
      if % <> FAIL
      then b:= %;
      else series(b, extop(a,5), qa);
           if % <> FAIL 
              then b:= %
              else error("could not convert second argument");
           end_if;
      end_if:
      qb:=err(b);
   else
      qa:=err(a); qb:=err(b);
   end_if;
   boa:=BranchOrder(a); bob:=BranchOrder(b);
   if boa<>bob then
      i:=ilcm(boa,bob);
      Puiseux::plus(Puiseux::multbo(a,i/boa),Puiseux::multbo(b,i/bob))
   else
      ka:=valuation(a); kb:=valuation(b);
      if kb>=qa then a
      elif ka>=qb then b
      elif ka<=kb then
         dif:=kb-ka; // dif>=0 
         la:=ListCoeffs(a); lb:=ListCoeffs(b);
         da:=nops(la); db:=nops(lb)+dif;
         if da<=db then
            if dif<=da then 
               lc:=[la[i]          $i=1..dif,
                    la[i]+lb[i-dif]$i=dif+1..da,
                    lb[i-dif]      $i=da+1..min(db,qa-ka)]
            else // no overlapping 
               lc:=[la[i]$   i=1..da,
                      0      $(dif-da),
                    lb[i-dif]$i=dif+1..db]
            end_if
         else // da>db thus da>dif ==> always overlapping 
            lc:=[la[i]          $i=1..dif,
                 la[i]+lb[i-dif]$i=dif+1..db,
                 la[i]          $i=db+1..min(da,qb-ka)]; // <- version 2
              // la[i]          $i=db+1..min(da,qb-kb)]; // <- version 1
              // la[i]          $i=db+1..da]; // <- original code = version 0
         end_if;
         while nops(lc)<>0 do
            if lc[1]=0 then
               lc:=subsop(lc,1=null()); ka:=ka+1; if lc=[] then break end_if
            else break
            end_if
         end_while;
         extsubsop(a,2=ka,3=min(qa,qb),4=lc)
      else Puiseux::plus(b,a)
      end_if
   end_if
end_proc:

Puiseux::_plus:=proc() local i; 
begin 
    case args(0)
    of 1 do return(args(1))
    of 2 do return(Puiseux::plus(args(1), args(2)))
    of 3 do return((if type(args(1))=Puiseux then
	Puiseux::plus(Puiseux::plus(args(1), args(2)),args(3))
        else Puiseux::plus(args(1),Puiseux::plus(args(2),args(3))) end_if))
    otherwise
      _plus(_plus(args(i) $ i = 1..args(0) div 2),
      _plus(args(i) $ i = (args(0) div 2) + 1..args(0)))
  end_case
end_proc :

Puiseux::negate := proc(a) local l,i;
begin
   l:=ListCoeffs(a);
   extsubsop(a,4=[-l[i]$i=1..nops(l)])
end_proc:

Puiseux::_subtract:= (x,y) -> x + (-y):

Puiseux::mult:= proc(a,b)
local aa,bb,ka,kb,qa,qb,da,db,la,lb,i,j,i1,i2,l1,l2,l3,imax,boa,bob;
begin
   if type(a)<>Puiseux then // -> type(b)=Puiseux
      if a=1 then return(b) end_if:
      if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT,DOM_COMPLEX},domtype(a)) then
         // Try to convert via new. This only works, if a is an expression
         // that is a Puiseux series (mathematically)
         aa:=Puiseux::new(a,variable(b),err(b));
         if aa = FAIL
           then a:= series(a, extop(b, 5), extop(b,3)/extop(b,1));
           else a:= aa:
         end_if:
      end_if;
      // a cannot be of a more general domain such as Series::geseries,
      // otherwise the corresponding _mult method would have been called.
      // So this is all we can do about conversion.
   elif type(b)<>Puiseux then  // -> type(a)=Puiseux
      if b=1 then return(a) end_if:
      if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT,DOM_COMPLEX},domtype(b)) then
         // Try to convert via new. This only works, if b is an expression
         // that is a Puiseux series (mathematically)
         bb:=Puiseux::new(b,variable(a),err(a));
         if bb = FAIL
           then b:= series(b, extop(a, 5), extop(a,3)/extop(a,1));
           else b:= bb:
         end_if:
      else // b may be of a more general domain (Series::gseries)
           return(b::dom::_mult(b::dom::convert(a),b))
      end_if
   end_if;
   if type(a)<>Puiseux or type(b)<>Puiseux then
      error("incompatible types, cannot multiply");
   end_if:
   boa:=BranchOrder(a); bob:=BranchOrder(b);
   if boa<>bob then
      i:=ilcm(boa,bob);
      Puiseux::mult(Puiseux::multbo(a,i/boa),Puiseux::multbo(b,i/bob))
   else
      ka:=valuation(a); kb:=valuation(b);
      qa:=err(a); qb:=err(b);
      la:=ListCoeffs(a); lb:=ListCoeffs(b);
      da:=nops(la); db:=nops(lb);
      imax:=min(qa-ka,qb-kb); // maximal number of terms in the product 
      if da<=db then i1:=da; i2:=db
      else i1:=db; i2:=da
      end_if;
      l1:=_plus(la[j]*lb[i+1-j]$ j=1..i)$ i=1..i1;
      if da<=db then    
	 l2:=_plus(la[j]*lb[i+1-j]$ j=1..da)$ i=da+1..min(db,imax);
      else
	 l2:=_plus(la[i+1-j]*lb[j]$ j=1..db)$ i=db+1..min(da,imax);
      end_if;
      l3:=_plus(la[j]*lb[i+1-j]$ j=i+1-db..da)$ i=i2+1..min(da+db-1,imax);
//      extsubsop(a,2=ka+kb,3=min(ka+qb,kb+qa),4=map([l1,l2,l3],expand))
// CHANGED (Juergen) 17.11.00: removed expand
      extsubsop(a,2=ka+kb,3=min(ka+qb,kb+qa),4=[l1,l2,l3])
   end_if
end_proc:


Puiseux::homog_mult:=misc::genassop(Puiseux::mult, 1):

Puiseux::_mult:=
proc()
begin
split([args()], x-> type(x)=Puiseux);
Puiseux::mult(Puiseux::homog_mult(op(op(%,1))), _mult(op(op(%,2))))
end_proc:


Puiseux::square := proc(a) local ka,qa,da,la,i,j,l1,l3,imax;
begin
   ka:=valuation(a);
   qa:=err(a);
   la:=ListCoeffs(a);
   da:=nops(la); 
   imax:=qa-ka; // maximal number of terms in the product 
   l1:=_plus(la[j]*la[i+1-j]$ j=1..i)$ i=1..da;
   l3:=_plus(la[j]*la[i+1-j]$ j=i+1-da..da)$ i=da+1..min(2*da-1,imax);
   extsubsop(a,2=2*ka,3=ka+qa,4=[l1,l3])
end_proc:

Puiseux::_power := proc(a,n)
begin
   if a=FAIL then a
   elif type(n)=DOM_RAT then Puiseux::power(a,n)
   elif n<0 then 
      a:=normal(a);
      if n=-1 then Puiseux::_invert(a)
      else Puiseux::_invert(Puiseux::_power(a,-n))
      end_if
   elif n=0 then Puiseux::one(n,variable(a))
   elif n=1 then a
   elif n mod 2 = 0 then Puiseux::_power(Puiseux::square(a),n div 2)
   else Puiseux::mult(a,Puiseux::_power(Puiseux::square(a),n div 2))
   end_if
end_proc:

// from Zippel, page 165 
Puiseux::power := proc(a,s)
local v,p,ps,k,q,l,d,p0,C,bo;
begin
   // works only for valence 0 
   v:=valuation(a);
   if v<>0 then 
      /* a = x^(v/bo) * (1 + ...) thus a^s = B * C
        where B = x^(s*v/bo) and C = (1 + ...)^s */
      // B has branching order denom(s*v/bo) and C has branching order bo 
      C:=Puiseux::power(extsubsop(a,2=0,3=err(a)-v),s); 
      bo:=BranchOrder(a);
      p:=s*v/bo;
      if bo mod denom(p) <> 0 then
         k:=ilcm(bo,denom(p));
         C:=Puiseux::multbo(C,k/bo);
      else k:=bo
      end_if;
      if domtype(s)=DOM_INT
      then extsubsop(C,2=p*k,3=err(C)+p*k);
      else // The code above transformed
           //    ( a0*x^(v/bo) + a1*x^((v+1)/bo) + ...)^ s
           // into  x^(s*v/bo)*(a0 + a1*x^(1/bo) + ..)^s
           //     = a0^s*x^(s*v/bo)* (1 + a1/a0*x^(1/bo) + ..)^s
           // This is wrong!! The correct answer is
           // (a0*x^(v/bo))^s* (1 + a1/a0*x + ..)^s !
           // I.e., so far the result is incorrect by the factor
           // a0^s*x^(s*v/bo) / (a0*x^(v/bo))^s

           // We cannot repair this within the existing Puiseux data
           // structure. So return the incorrect result for the time
           // being.
           extsubsop(C,2=p*k,3=err(C)+p*k);
      end_if:
   else
      q:=err(a); // O(x^q) 
      p:=ListCoeffs(a);
      d:=nops(p);
      if d=0 then return(extsubsop(a,3=s*q)) end_if;
      p0:=p[1];
      ps[1]:=p0^s;
      for k from 1 to q-1 do
         ps[k+1]:=_plus((l*s-k+l)*p[l+1]*ps[k+1-l]$l=1..min(k,d-1))/p0/k
      end_for;
      extsubsop(a,4=[ps[k]$ k=1..q])
   end_if
end_proc:

// from Zippel, page 167 
Puiseux::_fconcat:=proc(P,Q)
local q,R,i,l,d,k,vQ,lR,boP,boQ;
begin
      boP:=BranchOrder(P);
      // P(x<-Q) = P(x<-x^boP)(x<-Q^(1/boP)) 
      if boP<>1 then P:=extsubsop(P,1=1); Q:=Puiseux::power(Q,1/boP) end_if; 
      boQ:=BranchOrder(Q);
      vQ:=valuation(Q);
      if vQ<0 then error("invalid composition") end_if;

      if point(P) = 0 then
        if vQ = 0 then
          error("invalid composition")
        end_if
      else
        // point(P)=a<>0 therefore Q must tend to a 
        Q:=lmonomial(Q,Rem);
        if Q = FAIL or Q[1]<>point(P) then error("invalid composition") end_if;
        Q:=Q[2]; vQ:=valuation(Q);
      end_if;

      // P = a[k]*x^k + ... + a[p]*x^p + O(x^q) 
      k:=valuation(P);
      l:=ListCoeffs(P);
      d:=nops(l);
      q:=err(P);
      if d = 0 then // P = O(x^q)
	if nops(ListCoeffs(Q)) <> 0 then
          // Q := O(x^(vQ/boQ))
          Q := extsubsop(Q, 3 = vQ, 4 = [])
        end_if;
        if q < 0 or err(Q) <= 0 then
          error("invalid composition")
        elif q = 0 then
	  // R := O(1)
          R := Puiseux::create(1, 0, 0, [], variable(Q), point(Q))
        else // q > 0
	  // R := O(Q^q)
          R := Puiseux::power(Q, q)
        end_if;
        return(R)
      else
        // initialize with a[p] + O(x^(vQ*(q-p))) 
        R:=extsubsop(Q,2=0,3=vQ*(q-k-d+1),4=[l[d]]);
      end_if;
      for i from d-1 downto 1 do
	 R:=Puiseux::mult(R,Q); // now R = b*x^vQ + ... 
	 lR:=[l[i],0$(vQ-1),op(ListCoeffs(R))];
         R:=extsubsop(R,2=0,4=lR);
      end_for;
      // now R = a[k] + a[k+1]*Q + ... 
      if k=0 then R else Puiseux::_mult(R,Puiseux::_power(Q,k)) end_if
end_proc:

Puiseux::scalmult:=proc(p,a,k) // a*x^k*p 
local l,i;
begin
   l:=ListCoeffs(p);
   if args(0)<3 then extsubsop(p,4=[a*l[i]$i=1..nops(l)])
   else
      extsubsop(p,4=[a*l[i]$i=1..nops(l)],2=valuation(p)+k*BranchOrder(p),
	3=err(p)+k*BranchOrder(p))
   end_if
end_proc:

Puiseux::one:=proc(n,x) begin new(Puiseux,1,0,n,[1],x=0) end_proc:

Puiseux::frompoly:= proc(p,x,n=ORDER)
local i,v,X;
begin
   X:= op(p,[2,1]);
   if args(0) > 1 then
      if X <> x then 
         if domtype(x) <> DOM_IDENT then
             return(Series::main( expr(p),x,n,args(4..args(0)) ))
         end_if;
         X:= x;
         p:= poly(expr(p),[X]);
         if p = FAIL then
            error("not a polynomial")
         end_if
      elif nops(op(p,2)) > 1 then
          p:= poly(expr(p),[X])
      end_if;
      if args(0) > 2 then
         if not testtype( n,Type::NonNegInt ) then 
            error("order must be a positive integer")
         end_if
      end_if
   elif nops(op(p,2)) <> 1 then
       error("expecting a univariate polynomial")
   elif domtype(X) <> DOM_IDENT then
       error("invalid series variable")
   end_if;

   // to avoid coefficients like 0.0
   p:=mapcoeffs(p,proc() begin (if iszero(args(1)) then 0 else args(1) end_if) end_proc );
   if iszero(p) then 
      return(new(Puiseux,1,n,n,[],X=0))
   else
      v:=ldegree(p);
      return(new(Puiseux,1,v,max(n,v),[coeff(p,X,i)$i=v..min(degree(p,X),n-1)],X=0))
   end_if
end_proc:

Puiseux::_divide:= proc(a,b) begin a*b::dom::_invert(b) end_proc:

// the following is about twice faster than Puiseux::power(a,-1) 
Puiseux::_invert := proc(a) local k,q,d,l,i,a1,b,j;
begin
   l:=ListCoeffs(a);
   d:=nops(l);
   q:=err(a);
   if d=0 then // 1/O(x^q) -> division by zero 
      error("order too small")
   else // 1/(a1*x^k+...+ad*x^(k+d-1)+O(x^q)) = x^(-k)/(a1+...+ad*x^(d-1)+O(x^(q-k))) 
      a1:=l[1];
      k:=valuation(a);
      b:=table(); b[1]:=1/a1;
      for i from 3 to d+1 do b[i-1]:=-_plus(l[j]*b[i-j]$j=2..i-1)/a1 end_for;
      for i from d+2 to q-k+1 do b[i-1]:=-_plus(l[j]*b[i-j]$j=2..d)/a1 end_for;
      extsubsop(a,2=-k,3=q-2*k,4=[b[i]$ i=1..q-k])
   end_if;
end_proc:

Puiseux::equalx := proc(a) // is a x + O(...) 
begin
   if valuation(a)=1 and nops(ListCoeffs(a))>=1 and ListCoeffs(a)[1]=1 then TRUE else FALSE end_if
end_proc:

// gives the coefficient of x^n
Puiseux::coeff:= proc(a,n)
   local k,q,l,bo;
begin
   case args(0)
   of 0 do
       error("missing argument")
   of 1 do
       return( op(ListCoeffs(a)) )
   of 3 do
      if n <> variable(a) then
          error("invalid variable")
      else
          n:= args(3)
      end_if;
   of 2 do
      if args(0) = 2 then
          if n = variable(a) then
              return( op(ListCoeffs(a)) )
          elif not testtype( n,Type::Numeric ) then
              error("illegal argument")
          end_if
      end_if;

      bo:= BranchOrder(a);
      k:= valuation(a);
      if n<k/bo then 
          return( 0 )
      else
          q:= err(a);
          if n>=q/bo then 
              return( FAIL )
          else 
              // x^k/bo corresponds to l[1], thus x^n corresponds to l[n*bo+1-k]
              n:= n*bo+1-k;
              if type(n)<>DOM_INT then return(0) end_if;
              l:= ListCoeffs(a);
              if n<=nops(l) then return( l[n] ) else return( 0 ) end_if
          end_if
      end_if
   otherwise
      error("wrong number of arguments")
   end_case
end_proc:

Puiseux::nthterm := proc(a, n)
  local l, t, k;
begin
   l:=ListCoeffs(a);
   if domtype(n) <> DOM_INT or n<=0 or n>nops(l) then FAIL else
     k := 0;
     while k < nops(l) and n > 0 do
       k := k + 1;
       t := simplify(l[k]);
       if not Puiseux::iszero(t) then
         n := n - 1;
       end_if;
     end_while;
     if n > 0 then
       return(FAIL);
     end_if;
     if domtype(point(a)) = domtype(infinity) then
// change this for new Puiseux representation !!!
       variable(a)^(-(valuation(a)+k-1)/BranchOrder(a))
     else
       variable(a)^((valuation(a)+k-1)/BranchOrder(a))
     end_if
   end_if
end_proc:

Puiseux::nthmonomial := proc(a, n)
  local l, t, k;
begin
   l:=ListCoeffs(a);
   if domtype(n) <> DOM_INT or n<=0 or n>nops(l) then FAIL else
     k := 0;
     while k < nops(l) and n > 0 do
       k := k + 1;
       t := simplify(l[k]);
       if not Puiseux::iszero(t) then
         n := n - 1;
       end_if;
     end_while;
     if n > 0 then
       return(FAIL);
     end_if;
     if domtype(point(a)) = domtype(infinity) then
// change this for new Puiseux representation !!!
       t*variable(a)^(-(valuation(a)+k-1)/BranchOrder(a))
     else
       t*variable(a)^((valuation(a)+k-1)/BranchOrder(a))
     end_if
   end_if
end_proc:

Puiseux::nthcoeff := proc(a, n)
  local l, t, k;
begin
   l:=ListCoeffs(a);
   if domtype(n) <> DOM_INT or n<=0 or n>nops(l) then FAIL else
     k := 0;
     while k < nops(l) and n > 0 do
       k := k + 1;
       t := simplify(l[k]);
       if not Puiseux::iszero(t) then
         n := n - 1;
       end_if;
     end_while;
     if n > 0 then
       return(FAIL)
     else
// change this for new Puiseux representation !!!
       t
     end_if;
   end_if
end_proc:

Puiseux::lcoeff := proc(_a) local l;
begin
   l:=ListCoeffs(_a);
   while l<>[] do
      simplify(l[1]);
      if Puiseux::iszero(%) then delete l[1]
      else return(%)
      end_if
   end_while;
   FAIL
end_proc:

/* lmonomial(s) returns the leading term of s
  lmonomial(s,Rem) returns [u,r] where u is the leading non-zero term and r=s-u,
  both return FAIL if there is no non-zero leading term, i.e. s = O(...) */
Puiseux::lmonomial := proc(_a,opt=FAIL) local l,n,u;
begin
   l:=ListCoeffs(_a);
   n:=0; while l<>[] do
      if Puiseux::iszero(l[1]) then delete l[1]
      else 
         u:=l[1]*variable(_a)^((valuation(_a)+n)/BranchOrder(_a));
         if opt=hold(Rem) then
            _a:=Puiseux::normal(extsubsop(_a,2=valuation(_a)+n+1,4=subsop(l,1=null())));
            return([u,_a])
         else return(u) 
         end_if;
      end_if;
      n:=n+1;
   end_while;
   FAIL
end_proc:

Puiseux::lterm := proc(a) 
   local l,n;
begin
   l:=ListCoeffs(a);
   n:=0; while l<>[] do
      if Puiseux::iszero(l[1]) then delete l[1]
      else return(variable(a)^((valuation(a)+n)/BranchOrder(a)))
      end_if;
      n:=n+1;
   end_while;
   FAIL
end_proc:

/* Diese Methode ueberlaedt NICHT die Funktion 'eval' !! (daher auskommentiert, frankp)
Puiseux::evaluate := proc(_a_)
begin
   ListCoeffs(_a_);
   extsubsop(_a_,4=eval(%))
end_proc:
*/

Puiseux::func_call:= proc(s,p)
   local k, l, i, X, bo;
begin
   if args(0) <> 2 then error("invalid function call") end_if;
   p:= context(p);

   bo:= BranchOrder(s);
   k:= valuation(s);
   l:= ListCoeffs(s);
   s:= p^( k/bo );

   X:= genident("X");
   s:= poly( _plus(l[i]*s*X^(i-1) $ i=1..nops(l)), [X] );
   return( evalp( s,X=p^(1/bo) ) )
end_proc:

/*
    reverse(a[k]*x^(k/b) + a[k+1]*x^((k+1)/b) + ... ) = 
    reverse(a[k]*x^k + a[k+1]*x^(k+1) + ...)^b
*/
Puiseux::revert := proc(a) 
local k,l,n,m,i,p,q,eq,X,C;
begin
   if BranchOrder(a)<>1 then
      Puiseux::revert(extsubsop(a,1=1))^BranchOrder(a)
   else // branching order is 1 
      k:=valuation(a); // y = x^k + ... ==> x = y^(1/k) + y^(2/k) + ... 
      if k<0 then // y = 1/x^j + ... ==> 1/y = x^j + x^(j+1) + ... 
         1/Puiseux::revert(1/a)
      elif k=0 then // y = c + x^j + ... ==> (y-c) = x^j + ... 
         C:=lcoeff(a);
         a:=Puiseux::lmonomial(a,Rem)[2];
         extsubsop(Puiseux::revert(a),5=(variable(a)=C))
      else // k>0 
         n:=err(a)-k; // number of knowns terms of y 
         m:=ListCoeffs(a);
         if nops(m)<n then m:=m . [0$(n-nops(m))] end_if;
         l[1]:=1/m[1]^(1/k); // coefficient of X^(1/k) 
	 X:= genident("X"); 
	 C:= genident("C"); // coefficients of polynomials 
         p:=l[1]*X; q:=poly(m[1]*X^k,[X]);
         for i from 2 to n do
            // compute the coefficient of X^(i/k) 
            // we need only the coefficients m[1]...m[i] 
            p:=p+C*X^i;
            q:=q+poly(m[i]*X^(i+k-1),[X]);
            eq:=coeff(evalp(q,X=p),[X],i+k-1); // this is the expensive step 
            l[i]:=-coeff(eq,[C],0)/coeff(eq,[C],1);
            p:=subs(p,C=l[i]);
         end_for;
         extsubsop(a,1=k,2=1,3=1+n,4=[l[i]$ i=1..n])
      end_if
   end_if
end_proc:

Puiseux::map := proc() local i;
begin
     extsubsop( args(1),4=map(ListCoeffs(args(1)),args(i) $ i=2..args(0)) )
end_proc :

Puiseux::float:=proc(x) begin
   map(x,proc() begin (if args(1)=0 then 0 else float(args(1)) end_if) end_proc )
end_proc:

Puiseux::contfrac := proc(a)
local l,bo,n,j,v,k,a1,b1,z,q;
begin
   z:=variable(a);
   a:=Puiseux::normal(a);
   bo:=BranchOrder(a);
   v:=valuation(a);
   q:=err(a);
   l:=ListCoeffs(a);
   n:=nops(l);
   if n=0 then return(O(z^(q/bo)))
   elif n=1 then return(l[1]*z^(v/bo)+O(z^(q/bo)))
   end_if;
   // a is normalized ==> l[1]<>0 
   // find second non zero term 
   for j from 2 to n do if l[j]<>0 then break end_if end_for;
   if j > n then // only one nonzero term
     return(l[1]*z^(v/bo)+O(z^(q/bo)))
   elif j=n then // exactly two nonzero terms
     return(l[1]*z^(v/bo)+l[j]*z^((v+j-1)/bo)+O(z^(q/bo)))
   end_if;
   // is there a third non zero term ? 
   for k from j+1 to n do if l[k]<>0 then break end_if end_for;
   if k > n then // there are only two non zero terms 
      l[1]*z^(v/bo)+l[j]*z^((v+j-1)/bo)+O(z^(q/bo))
   else
      a1:=l[1]*z^(v/bo);
      b1:=z^((v+j-1)/bo);
      a1+b1/Puiseux::contfrac(b1/(a-a1))
   end_if
end_proc:

Puiseux::series:=proc()
begin
// The eval is necessary since print returns the error term as hold(O)
  series(eval(Puiseux::print(args(1)),args(2..args(0))))
end_proc:

Puiseux::diff := proc(a,x)
local l,i,k,bo,ord;
begin
   case args(0)
   of 0 do error("wrong number of arguments")
   of 1 do return(a)
   of 2 do break // normal case 
   otherwise return(Puiseux::diff(Puiseux::diff(a,x),args(3..args(0))))
   end_case;
   bo:=BranchOrder(a);
   k:=valuation(a);
   ord:=err(a);
   l:=ListCoeffs(a);
   if not has(variable(a)-x,x) then

     if has(l, x) then // coeffs depend on x as well, may contain log
       series(diff(expr(a), x), x = point(a), ord - k + bo) +
       if domtype(point(a)) <> domtype(infinity) then
         Puiseux::create(bo, ord - bo, ord - bo, [], x, point(a))
       else
         Puiseux::create(bo, ord + bo, ord + bo, [], x, point(a))
       end_if

     // coeffs do not contain x
     elif domtype(point(a)) <> domtype(infinity) then
       extsubsop(a,2=k-bo,3=ord-bo,4=[l[i]*(k+i-1)/bo$i=1..nops(l)])
     else
       extsubsop(a,2=k+bo,3=ord+bo,4=[l[i]*(1-k-i)/bo$i=1..nops(l)])       
     end_if

   elif has(variable(a),x) then FAIL
   else // only differentiate coefficients 
      map(a,diff,x)
   end_if
end_proc:

Puiseux::laplace := proc(a,t,s) local x,l,i,bo,ord,k;
begin
   x:=variable(a);
   if x=t then
      bo:=BranchOrder(a); k:=valuation(a); ord:=err(a); l:=ListCoeffs(a);
      extsubsop(a,2=k+bo,3=ord+bo,4=[l[i]*gamma((k+i-1)/bo+1)$i=1..nops(l)],5=1/s)
   else map(a,transform::laplace,t,s)
   end_if
end_proc:

Puiseux::invlaplace := proc(a,s,t) local x,l,i,bo,ord,k;
begin
   x:=variable(a);
   if x=1/s then
      bo:=BranchOrder(a); k:=valuation(a); ord:=err(a); l:=ListCoeffs(a);
      extsubsop(a,2=k-bo,3=ord-bo,4=[l[i]/gamma((k+i-1)/bo)$i=1..nops(l)],5=t)
   else map(a,transform::invlaplace,s,t)
   end_if
end_proc:

Puiseux::has:=proc() begin has([extop(args(1),4..5)],args(2)) end_proc :

Puiseux::int:=
proc(a,x)
  local A, k, bo, ord, l;
begin
  if domtype(x)=DOM_IDENT then // indefinite integration
    bo := BranchOrder(a);
    k := valuation(a);
    ord := err(a);
    l := ListCoeffs(a);
    if x<>variable(a) then map(a,int,x)
    else //series(int(expr(a),x),extop(a,5),err(a)+1);
      A:=int(expr(a),x);
      if has(A,int) then hold(int)(a,x,args(3..args(0)));
      else series(A,extop(a,5), ord - k + 1)
           + Puiseux::create(bo, ord + bo, ord + bo, [],
                             variable(a), point(a));
      end_if
    end_if;
  else // definite integration
    // I do not like to do the same tests as in STDLIB/int.mu
    // thus errors can occur!!!
    if domtype(op(x,1))=DOM_IDENT then 
      if op(x,1)<>variable(a) then map(a,int,x,args(3..args(0)));
      else
        int(expr(a),x,args(3..args(0)))+
        int(O(variable(a)^err(a)),x,args(3..args(0)));
      end_if;
    else // case something is wrong, should I return here an error?
      hold(int)(a,x,args(3..args(0)));
    end_if;
  end_if;
end_proc:


Puiseux::testtype:= proc(x, T)
		      local Ttesttype;
		    begin
		      if domtype(x)=T then
			TRUE
		      else
			if domtype(T) = DOM_DOMAIN then 
			  Ttesttype := T::testtype
			else
			  Ttesttype := T::dom::testtype
			end_if;
			if Ttesttype<>FAIL then
			  Ttesttype(x, T)
			else
			  FALSE
			end_if:
		      end_if;
		    end_proc:


// map expand to the coefficients and normalize the result
// overloads expand
Puiseux::expand := proc(P)
begin
// *** 4 has to be changed for flag = 1
  Puiseux::normal(extsubsop(P, 4 = map(ListCoeffs(P), expand, args(2..args(0)))))
end_proc:


unalias(BranchOrder, valuation, err, ListCoeffs, variable):

unalias( Puiseux ):
