/*====================================================================*/
/*   polylog/Series -- the function attribute "series" for polylog    */
/*====================================================================*/
/*
 *  Version 0.2.06 (1998.06.17)
 *  Contributed by Daniel Duparc <dduparc@club-internet.fr>
 *  29 av. de la Commune de Paris
 *  94400 Vitry sur Seine (France)
 */

/**********************************************************************/
/*
 *  For series(polylog(p,f(x))) at the order n.
 */
/**********************************************************************/
polylog:=slot(polylog,"series", proc(p,f,x,n,dir) 
  local l,t,d;
begin

   // The series attribute of dilog is quite sophisticated.
   // Use it, if p = 2 (polylog(2,x) = dilog(1-x))
   if p=2 then return(series(dilog(1-f), x, n,
                      if dir = FAIL then null() else dir end_if)) end_if:

   if dir <> FAIL then // directional expansion
     l:= limit(f, x, if dir = Real then null() else dir end_if):
     if has(l, infinity) then
       return(FAIL) // <- insert code for asymptotic expansions here
     end_if:
     if iszero(l) then
       l:= Series::Puiseux::create(1,1,n,Series::gen["polylog"](p,n),x):
       if f=x
       then return(l)
       else t:=Series::series(f,x,n,dir);
           if type(t)<>Series::Puiseux then return(FAIL) end_if;
           return(Series::Puiseux::_fconcat(l, t));
       end_if
     end_if
   end_if;

   // recursively expand the argument
   t := Series::series(f, x, n, dir);

   if domtype(t) = Series::Puiseux then
     d := ldegree(t);
     if d = FAIL then // t = O(..)
       d := Series::Puiseux::order(t);
       if d > 0 then
         return(t);
       else
         error("order too small");
       end_if       
     elif d > 0 then // expansion around 0
       l:= Series::Puiseux::create(1,1,n,Series::gen["polylog"](p,n),x):
       if f=x
       then return(l)
       else return(Series::Puiseux::_fconcat(l, t));
       end_if
     elif d < 0 then
       return(FAIL)
     end_if
   end_if;

   Series::unknown(polylog(p,f),x,n,dir)

end_proc):

// ensure that domain Series is loaded
Series:

//-------------------------------------------------------------------
//  Expansion for polylog(p,t) at the order n. 
//-------------------------------------------------------------------
Series::gen["polylog"]:=proc(p,n) local t,i; begin
   t:=1; [((t:=t)/(i^p)$i=1..n-1)]
end_proc:
//-------------------------------------------------------------------
