// $Date: 2000/05/24 14:42:27 $ $Author: jngerhar $ $Revision: 1.12 $ 

/*--
	psi/Series -- the function attribut "series" for psi
--*/

psi:=slot(psi,"series",proc(f,n,x,ord,dir) // series(psi(f,n),x,ord,dir) 
  local l,lim,k,s,f0,d;
begin

   if dir <> FAIL then // directional expansion
     if (lim := limit(f,x=0,
          if dir = Real then null() else dir end_if)) = infinity then
       // expansion at infinity 
       if n=0 then // formula 6.3.18 in Abramowitz & Stegun 
         f0:=lmonomial(Series::series(f,x,2,dir));
         // ln(f) = ln(f0+(f-f0)) = ln(f0)+ln(1+(f-f0)/f0) 
         l:=[-1/2/f,-bernoulli(2*k)/2/k*f^(-2*k)$k=1..(ord-1) div 2];
         if nops(l)>ord then delete l[ord+1] end_if;
         s:=Series::series(_plus(ln(1+(f-f0)/f0),op(l)),x,ord,dir);
//         s:=Series::gseries::convert(s);
//         return(extsubsop(s,1=[[1,ln(f0)],op(extop(s,1))]))
         return(Series::series(ln(f0), x, ord, dir) + s);
       else // formula 6.4.11 in Abramowitz & Stegun 
         s:=(-1)^(n-1);
         l:=[s*fact(n-1),s*fact(n)/2,
                (s*bernoulli(2*k)*fact(2*k+n-1)/fact(2*k),0)
		    $ k=1..(ord-1) div 2];
         if nops(l)>ord then delete l[ord+1] end_if;
         return(Series::Puiseux::create(1,n,n+ord,l,x))
       end_if
     elif lim = -infinity then // expansion at -infinity
       return(FAIL);
     end_if;
   end_if;

   // recursively expand the argument
   s:=Series::series(f, x, ord, dir);

   if domtype(s) = Series::Puiseux then
     d := ldegree(s);
     if d = FAIL then // s = O(..)
       error("order too small");
//     elif d > 0 then // expansion in zero 
//       // use formula 6.4.6 of Abramowitz & Stegun 
//       return(series(-(-1)^n*fact(n)/f^(n+1)+psi(f+1,n),x,ord,
//                     if dir = FAIL then null() else dir end_if))
     elif d < 0 then
       return(FAIL)
     elif /* d >= 0 and */ testtype(-coeff(s, x, 0), Type::NonNegInt) then
       // expansion around simple pole, use nth derivative of reflection
       // formula psi(f) = psi(1 - f, n) - PI * cot(PI*f)
       if domtype(n) = DOM_INT then
         return(series((-1)^n*psi(1 - f, n) - PI * diff(cot(PI * f), x $ n),
                       x, ord, if dir = FAIL then null() else dir end_if))
       else
         // unfortunately, we cannot do this for a symbolic second
         // exponent, since there is no means to express an nth formal
         // derivative for symbolic n
         userinfo(1, "cannot compute series expansion for symbolic second argument of psi");
         return(FAIL);
       end_if
     end_if
   end_if;
 
   Series::unknown(psi(f,n),x,ord,dir)

end_proc):

// end of file 
