alias( Puiseux = Series::Puiseux ):

Series::series:= proc(f,x,n,dir = FAIL)
local tf,p,opf0;
begin   
   userinfo(2,"computing expansion of",f);
   if testtype(f,Type::PolyExpr(x,Type::AnyType)) then Puiseux::frompoly(poly(f,[x]),x,n)
   else
      tf:=type(f); // "exp" or "tan" or ... 
      // first catch functions with variable number of arguments 
      if tf="_plus" then Series::plus(args())
      elif tf="_mult" then Series::mult(args())
      elif tf="_power" then Series::power(op(f),args(2..args(0)))

      // function environments with one or two args
      elif tf="psi" and nops(f) = 1 then
        psi::series(op(f), 0, x, n, dir)
      elif tf="dirac" and nops(f) = 1 then
        dirac::series(op(f), 0, x, n, dir)

      else
         opf0 := op(f,0);
         // the eval is necessary since DOM_VARs are evaluated with
         // depth 1 only and the object that opf0 refers to may change
         // after the following slot command
         if domtype(eval(opf0)) = DOM_FUNC_ENV then
            p:=slot(eval(opf0), "series");
            if contains({DOM_PROC,DOM_EXEC},domtype(p))
            then return(p(op(f),x,n,dir)) end_if;
         end_if;
         if traperror((p:=Series::unknown(args())))=0 then p else FAIL end_if
      end_if
   end_if
end_proc:

Series::unknown := proc(f,x,n,dir = FAIL)
local g,fa,i,g0,l;
begin
   userinfo(3, "Series::unknown called for ".
	    expr2text(f));
   fa:=1;
   g:=normal(f);
   userinfo(8, "Normalizing gives ".expr2text(g));
   if bool(type(g)="int") and has(op(g,2),x) then g0:=0 
   elif traperror((g0:=eval(subs(g,x=0)))) <> 0 or
      contains({FAIL, undefined}, g0) then return(FAIL);
   end_if;
   if n = 0 then return(Puiseux::create(1,0,0,[],x)) end_if;
   userinfo(5, "Function value is ".expr2text(g0));
   l:=[g0];
   for i from 1 to n-1 do
      g:=diff(g,x);
      userinfo(5, expr2text(i)."th derivative is ".expr2text(g));
      g:=Series::diff2D(g);
      userinfo(15, "Converting diff to D gave ".expr2text(g));
      fa:=fa*i;      
      if traperror((g0 := eval(subs(g, x=0)))) <> 0 or
         contains({FAIL, undefined}, g0) then
        return(FAIL);
      end_if;
      l := append(l, g0/fa)
   end_for;
   normal(Puiseux::create(1,0,n,l,x))
end_proc:

Series::diff2D := proc(f) 
begin
   userinfo(15, "diff2D called for ".expr2text(f));
   case type(f)
   of "_plus" do
   of "_mult" do
   of "_power" do return(map(f,Series::diff2D))
   of "diff" do return(_fconcat(D$nops(f)-1)(op(f,[1,0]))(op(f,[1,1]))*
				Series::diff2D(diff(op(f,[1,1]),op(f,2))))
   end_case;
   f
end_proc:

Series::plus:=proc(f,x,n,dir = FAIL) // f is a sum 
local s,i;
begin
   userinfo(3, "Computing series of the sum ".
	    expr2text(f));
   s:=Series::series(op(f,1),x,n,dir);
   for i from 2 to nops(f) do s:=s+Series::series(op(f,i),x,n,dir) end_for;
end_proc:

Series::mult:=proc(f,x,n,dir = FAIL) // f is a product 
local s,i,k,j;
begin
   userinfo(3, "Computing series of the product ".
            expr2text(f));
   ([f,k,s]):=split(f,has,x); j:=0;
   if f=1 then Puiseux::create(1,0,n,[k],x) // does not depend on x 
   else
      if type(f)<>"_mult" then s:=Series::series(f,args(2..args(0)))
      else
         ([j,f,s]):=split(f,proc() begin args(1)=x or type(args(1))="_power"
		and op(args(1),1)=x and type(op(args(1),2))=DOM_INT end_proc );
         if j=1 then j:=0 elif j=x then j:=1 else j:=op(j,2) end_if;
         if type(f)<>"_mult" then s:=Series::series(f,args(2..args(0)))
         else
	    s:=Series::series(op(f,1),args(2..args(0)));
	    for i from 2 to nops(f) do
	       s:=s*Series::series(op(f,i),args(2..args(0)))
	    end_for;
         end_if
      end_if;
      if domtype(s)=Puiseux then Puiseux::scalmult(s,k,j)
      else s::dom::scalmult(s,k,x^j) end_if
   end_if
end_proc:

Series::power:=proc(a,b,x,n,dir = FAIL)
begin
   userinfo(3, "Computing series of a power");
   case domtype(b)
   of DOM_INT do
   of DOM_RAT do return(Series::series(a,args(3..args(0)))^b)
   otherwise 
       if iszero(a) then 
           return( Series::series(0,args(3..args(0)))  )
       else 
           return( Series::series(hold(exp)(b*ln(a)),args(3..args(0))) )
       end_if
   end_case
end_proc:

unalias( Puiseux ):

// end of file 
