/*
 * series slot of trunc - computes series(trunc(f), x, n, dir)
 * analogous to floor and ceil
 *
 * Author: Juergen Gerhard, 18.5.00
 */

trunc:=slot(trunc,"series",proc(f,x,n,dir)
  local l, t, d;
begin

  if dir <> FAIL then // directional expansion
    l:=limit(f,x,if dir = Real then null() else dir end_if);
    if Re(l) = 0 or domtype(Re(l)) <> DOM_INT then
      l := trunc(l);
      if domtype(Re(l)) = DOM_INT then
        return(Series::Puiseux::new(l, x, n)) // l + O(x^n)
      end_if
    else // Re(l) is an integer
      userinfo(1,
        "no real series expansion of trunc(x) around discontinuity x = "
        . expr2text(l));
      return(FAIL);
    end_if;
  end_if;

  // recursively expand the argument
  t := Series::series(f, x, n, dir);

  if domtype(t) = Series::Puiseux then
    d := ldegree(t);
    if d = FAIL then
      error("order too small")
    elif d >= 0 then // expansion around a finite point 
      l := coeff(t, x, 0);
      if l = 0 or 
         (domtype(Re(l)) <> DOM_INT and domtype(Im(l)) <> DOM_INT) then
        l := trunc(l);
        if domtype(Re(l)) = DOM_INT and domtype(Im(l)) = DOM_INT then
          return(Series::Puiseux::new(l, x, n)) // l + O(x^n)
        end_if
      else
        userinfo(1,
          "no series expansion of trunc(x) around discontinuity x = "
          . expr2text(l));
        return(FAIL);
      end_if
    else // ldegree(t) < 0
      return(FAIL)
    end_if
  end_if;

  Series::unknown(trunc(f),x,n,dir)

end_proc):
