/* $Date: 1999/10/13 17:51:18 $ $Author: stefanw $ $Revision: 1.5 $ */

/* solvelib::Union


   Given a set1 whose elements contain a parameter x,
   and a set2, return the set of all objects that can
   be obtained by substituting x in some element of set1
   by some element of set2

*/

solvelib::Union:=
proc(set1, x, set2)
local i;

begin
  if set2={} then
    return({})
  end_if;

  if type(set2)=piecewise then
     // do case analysis w.r.t. parameter set
    if has(set2, x) then 
      error("Index set of Union must not depend on index variable")
    end_if;
    return(map(set2, z-> solvelib::Union(set1, x, z)))
  end_if;
  
  // try overloading
  if set1::dom::Union <> FAIL then
    return(set1::dom::Union(args()))
  end_if;
  
  case type(set1)
    of DOM_SET do
      return(_union(solvelib::substituteBySet(op(set1,i), x, set2)
                    $i=1..nops(set1)))
    of "solve" do
    of "RootOf" do
      return(procname(args()))
    of "_union" do
    of "_intersect" do
      return(map(set1, solvelib::Union, x, set2))
    of "_minus" do
      return(procname(args()))
    otherwise
      error("unknown set type ".expr2text(type(set1)))
  end_case
 end_proc:











