/* */
/* $Date: 2000/12/05 12:57:21 $ $Author: stefanw $ $Revision: 1.22 $ */
/* */
/* zimmerma, 14/12/95 */

/*++
        solvelib::algebraic  --  solve algebraic equations 

	algebraic(sys,vars <,domain>)

	sys   : set of algebraic equations
	vars  : set of unknowns
	domain: (optional) coefficient domain
++*/

solvelib::algebraic :=
proc(syss,unkk,domm)
  local t,sys,unk,dm, options;
begin
  if testargs() then
    if domtype(syss) <> DOM_SET then
      error("expecting set of equations")
    end_if;
    case args(0)
      of 1 do
        break
      of 2 do
        if domtype(unkk) <> DOM_SET and domtype(unkk) <> DOM_DOMAIN then
          error("invalid second argument")
        end_if;
        break
      of 3 do
        if domtype(unkk) <> DOM_SET or domtype(domm) <> DOM_DOMAIN then
          error("invalid arguments")
        end_if;
        break
      otherwise
        error("wrong no of args")
    end_case
  end_if;

  sys:=syss;

  if args(0) = 1 then
    dm:= null();
    unk:= indets(sys);
    options:=solvelib::getoptions()
  elif args(0) = 2 then
    unk:=unkk;
    if domtype(unk) = DOM_SET then
      t:= indets(sys);
      if nops(unk) <> nops(t) then
        userinfo(1,"found ",t," as unknowns");
        error("wrong no of unknowns given")
      end_if;
      dm:= null()
    else
      dm:= unk;
      unk:= indets(sys)
    end_if;
    options:=solvelib::getoptions()
  else
    unk:=unkk;
    dm:=domm;
    options:=solvelib::getoptions(args(4..args(0)))
  end_if;


  sys:=map
        (sys,
         proc(x)
           name equationsToExpressions;
         begin
           if type(x)="_equal" then
             op(x,1)-op(x,2)
           else
             x
           end_if
         end_proc
         );

  solvelib::solve_algebraic(sys,unk,dm, options)
end_proc:


/*--
	solvelib::solve_algebraic(sys,unk,dm, options) solves the algebraic
	system 'sys' with respect to the unknowns 'unk' over the
	coefficient field 'dm'.

	If 'unk' is a list, then it gives the elimination order.
--*/

solvelib::solve_algebraic:=
proc(syss,unkk,domm) 
  local g,reord, sys, unk, dm, options;
  
begin
  sys:=syss;
  unk:=unkk;
  if args(0)>=3 then
    dm:=domm
  else
    dm:=NIL
  end_if;
  options:=solvelib::getoptions(args(4..args(0)));
  if type(unk)=DOM_SET then
    unk:=[op(unk)];
    reord:=Reorder
  else
    reord:=null()
  end_if;
  userinfo(3,"Entering solver for algebraic systems");
  if dm=NIL then
    if map(sys,testtype,Type::PolyExpr(unk,Type::Rational))={TRUE}
      then
      dm:=Expr
    else
      dm:=Dom::ExpressionField(normal, iszero@normal)
    end_if
  end_if;
  sys:=map([op(sys)],poly,unk,dm);
  /* first compute a basis wrt DegInvLexOrder */
  g:=groebner::gbasis(sys,DegInvLexOrder,reord);
  /* then transform it in a LexOrder basis (the Grobner walk) */
  g:=groebner::gbasis(g,LexOrder,reord);
  userinfo(2,"Groebner basis is",map(g,expr));
  if nops(g)=1 and degree(g[1])=0 then
    null() /* no solution */
  else
    g:=groebner::factor(g, reord);
    g:=map(g,op@groebner::triangular_set);
    op(map(g,solvelib::allvals,{op(unk)},options))
  end_if
end_proc:


solvelib::allvals :=
proc(g,unkk,options) 
  local x,sol,i,j,r,n,foo,unk;
begin
  unk:=unkk;
  sol:=[[]];
  /* foo(l,x,j) adds x=j in front of partial solution l */
  if options["BackSubstitution"]=TRUE then
    foo:=proc(l, x, j)
           name makeEqAndBackSubstitute;
         begin
           select([op(l)], not hastype, "RootOf");
           [x=subs(j,op(%)),op(l)]
         end_proc
  else
    foo:=proc(l,x,j)
           name makeEq;
         begin
           // better: ?
           // if type(j)="RootOf" then
           //  [x in j, op(l)]
           // else  
             [x=j,op(l)]
           // end_if  
         end_proc
  end_if;

  // if the ideal is <1>, there is no solution
  if nops(g)=1 and degree(g[1])=0 then
    return(null())
  end_if;

  
  for i from nops(g) downto 1 do
    r:=g[i];
    x:=indets(op(r,1)) intersect unk;
    if nops(x)=0 then
      next
    end_if;
    /* choose the largest unknown */
    for j from 1 to nops(op(r,2)) do
      if contains(x,op(r,[2, j])) then
        x:=op(r, [2, j]);
        break
      end_if
    end_for;

    unk:=unk minus {x};  // !!!! modified 4.12.00 Walter

    // remove not only x, but also all unknowns after x

    //  unk:= unk minus {op(op(r,2), k) $k=j..nops(op(r,2))}; //!!! modified 4.12.00 Walter

    j:=solvelib::solve_poly(poly(op(r,1),[x]),x,options);
    if type(j)=DOM_SET and property::hasprop(x) then
      // select only those solutions that do not contradict
      // the properties of x
      j:=select(j, s -> is(s, getprop(x))<>FALSE )
    end_if;
    userinfo(5, "Solving polynomial ".expr2text(op(r,1))." gives ".
             expr2text(j));
    // assume that the remaining unknowns can be chosen suitably
    // do not take into account zeros of denominators etc. (to be changed)

    if type(j) = piecewise then
      j:= piecewise::disregardPoints(j);
      if type(j) = piecewise then
        j:= piecewise::assumeIdentsTRUE(j, unk)
      end_if;
    end_if;
    if type(j)="RootOf" then
      sol:=map(sol, foo, x, j)
    elif type(j)=solvelib::BasicSet then
      // nothing to do here !
    elif (n:=nops(j))=1 then
       sol:=map(sol,foo,x,op(j))
    else
       sol:=_concat(map(sol,foo,x,op(j,i)) $i=1..n)
    end_if;
    userinfo(5, "Solutions so far are ".expr2text(sol));
  end_for;
  op(sol)
end_proc:

/* end of file */

