
// solvelib::Bound2Free

// take a solution from the system solver
// and make the elements of vars to free parameters  
// the output is (usually) an object of type piecewise


solvelib::bound2Free:=
proc(sol, vars:DOM_SET)
  local f;
begin

  // local procedure f working on lists

  f:=
  proc(l)
    local condition, s;
  begin
    s:= split(l, eq -> contains(vars, op(eq,1)) );
    // now the equations in s[1] are constraints to the free parameters
    // while the equations of s[2] are part of the solutions

    // due to solver bug, strip the conditions whose right hand side
    // contains the left hand side
    s[1]:= select(s[1], eq -> not has(op(eq,2), op(eq, 1)));
    condition:= _and(op(s[1]));
    piecewise([condition, {s[2]}], [not condition, {}])
  end_proc;

  case type(sol)
    of piecewise do
      // flattening is done by piecewise automatically
      map(sol, solvelib::bound2Free, vars); break
    of "solve" do
      // eliminate free params from variables to solve for
      subsop(sol, 2=select(op(sol,2), v -> not has(vars, v))); break
    of DOM_SET do
      _union(op(map(sol, f))); break
    otherwise
      error("Unknown type of solution")
  end_case
  
end_proc:

