
/* $Date: 2000/11/30 16:31:45 $ $Author: stefanw $  $Revision: 1.20 $ */


/*

    solvelib::domsolve - solve over a domain

    solvelib::domsolve(eqs, var, options)

    eqs - equation, expression, or polynomial
    var - identifier
    options - table of options

*/


solvelib::domsolve:=
proc(eqss, var)
  local dm, i, p, options, eqs, newvar, sol;

begin
  eqs:=eqss;
  options:=solvelib::getoptions(args(3..args(0)));
  dm:=options["Domain"];

  if type(dm)=DOM_DOMAIN and dm::domsolve <> FAIL then
    return(dm::domsolve(eqs, var, options))
  end_if;

  if type(var)=DOM_LIST or type(var)=DOM_SET then
    if nops(var)<>1 then
      error("Solving over a domain works only for exactly one variable")
    end_if;
    var:=op(var);
    solve(eqs, var, options);
    if type(%)="_in" or type(%)=DOM_SET and map(%, domtype)={DOM_LIST} then
      return(%)
    else
      return(hold(_in)(var, %))
    end_if;
  end_if;

  
  if dm::dom::hasProp(Cat::Set)=TRUE then
    // solve over C and intersect with dm
    newvar:=genident();
    assume(newvar, solvelib::set2prop(dm));
    options["Domain"]:=hold(Expr);
    sol:=subs(solve(subs(eqs, op(var,1)=newvar), newvar, options),
              newvar=op(var,1)); 
    case type(sol)
      of "_in" do
        return(hold(_in)(op(sol,1), op(sol,2) intersect dm ))
      of DOM_SET do
        if map(sol, domtype)={DOM_LIST} then
          // handle set-of-lists format
          // keep all entries that could be in dm
          return(select(sol, l -> _and((op(l,i) in dm) $i=1..nops(l) )<>FALSE))
          // else fall through
        end_if;
    end_case;
    return(sol intersect dm);
  end_if;
  if type(dm)<>DOM_DOMAIN then
    error("Right hand side of second argument must be a domain")
  end_if;

  if contains({DOM_LIST, DOM_SET}, type(eqs)) then
    map(eqs, solvelib::domsolve, var, options);
    select(%, _unequal, dm);
    if nops(%)=0 then
      return(dm)
    else
      return(_intersect(op(%)))
    end_if
  end_if;
    
  if contains( Type::ConstantIdents,var ) then
    error("Invalid variable (mathematical constant was given)")
  elif type(eqs)="_equal" then
    eqs:=op(eqs,1)-op(eqs,2)
  end_if;

  if iszero(eqs) then
    return(dm)
  end_if;


  if contains({DOM_INT, Dom::Integer}, dm) then
    return(solvelib::diophantine(eqs, var, options))
  end_if;

/* if contains({DOM_RAT, Dom::Rational}, dm) then
   error("Not yet implemented")
end_if; */ 


/* try to find roots by factoring */

  p:=poly(eqs, [var], dm);

  userinfo(3, "Trying to factor");
  
  if traperror((p:=factor(p)))<>0 then
    error("Cannot solve over that domain")
  end_if;

  if options["Multiple"]=FALSE then
    p:=select({op(p,2*i) $i=1..nops(p) div 2},
              proc() begin degree(args(1))=1 end_proc );
    [op(map(p, _negate@(u->coeff(u,0))*_invert@lcoeff))];
    return({op(select(%, proc() begin args(1)<>FAIL end_proc ))})
  else
    p:=select({[op(p,2*i),op(p,2*i+1)] $i=1..nops(p) div 2},
              proc() begin degree(op(args(1),1))=1 end_proc );
    p:=map(p, proc() begin subsop(args(1),
                                  1=_negate(coeff(args(1)[1],0))) end_proc );
    return(Dom::Multiset((op(p,[i,1]) $op(p,[i,2]))
                         $i=1..nops(p)))
  end_if;

  error("No special method called, should not happen");

end_proc:


