/* */
/* $Date: 2000/12/07 18:16:42 $ $Author: jngerhar $ $Revision: 1.31 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
	solvelib::solve_eq
--*/

solvelib::solve_eq:=
proc(eq,x)
  local options, l, r, dummy;
begin
  userinfo(3, "solve_eq called with equation ".expr2text(eq));

  if eq::dom::solve <> FAIL then
    return(eq::dom::solve(args()))
  end_if;
  
  options:=solvelib::getoptions(args(3..args(0)));
  
  case domtype(eq)
    of DOM_INT do
    of DOM_FLOAT do
    of DOM_COMPLEX do  
      return((if iszero(eq) then
                if options["Multiple"]=TRUE then
                  error("Infinitely many solutions, cannot create multiset")
                else
                  solvelib::BaseSet
                end_if
              else
                {}
              end_if))
    of DOM_RAT do
      // there is no DOM_RAT representing zero
      return({})
    of DOM_IDENT do /* solve(x,x) or maybe solve(y,x) */
      return(solvelib::solve_poly_irred(poly(eq,[x]), x, options))
    of DOM_EXPR do
      if contains( Type::ConstantIdents,x ) then
        error("Invalid variable (mathematical constant was given)")
      elif type(x)=DOM_FUNC_ENV then
        error("cannot solve for ".expr2text(x))
      end_if;

      case type(eq)
        of "_mult" do
        // do *not* immediately call the polynomial solver
        // because expanding a product would be the worst thing
        // one could do
          userinfo(5, "Solving factors ".expr2text(op(eq))." separately");
          return(_union(map(op(eq), solvelib::solve_eq, x, options))
                  minus discont(eq, x, undefined))
        of "_power" do
          if is(op(eq,2)<=0)=TRUE then
            return({})
          end_if;
          if not has(op(eq,2), x) then
            userinfo(5, "Equation ".expr2text(eq)." reduces to ".
                     expr2text(op(eq,1)));
            l:= solvelib::solve_eq(op(eq,1), x, options);
            if options["Multiple"]=TRUE then
              if type(op(eq, 2)) = DOM_INT then 
                return(_union(l $op(eq,2)))
              else
                error("Option multiple is only allowed for polynomial equations")
              end_if
            else
              return(piecewise([op(eq, 2) > 0 or not op(eq,2) in R_, l],
                               [op(eq, 2) <= 0, {}]))
            end_if
          end_if
        of "_plus" do
          if testtype(eq,Type::PolyExpr(x)) then
            // test for special case f(x)^n = C
            // in this case, return the union of the sets
            // of solutions for f(x)=C^(1/n) + exp(2*PI*I*j/n)
            // for 0 <= j <= n-1

            // taken back because of ODE-Problem
            // if degree(eq, [x], x) > options["MaxDegree"] then
              [l, r, dummy]:= split(eq, has, x);
              if type(l)="_power" then
                return(_union
                       (solvelib::solve_eq(op(l,1)-(-r)^(1/op(l,2))*
                                           exp(2*PI*I*j/op(l,2)), x, options)
                        $ j=0..op(l,2)-1))
              end_if;
            // end_if;
            // degree too small, or polynomial has not special form
            return(solvelib::solve_poly(poly(eq,[x]),x,options))
          end_if;
          break
        of "function" do
          return(hold(solve)(eq, x))
      end_case;
        
     
      // type(eq) is none of _mult, _plus, or _power, or eq is a sum
      // but not polynomial in x
      
      if options["Multiple"]=TRUE then
        error("Option multiple is only allowed for polynomial equations")
      end_if;


      // handle floating-point input
  
      if hastype(eq, DOM_FLOAT, {DOM_COMPLEX}) then
        eq:=numeric::rationalize(eq);
        solvelib::solve_eq(eq, x, options);
        return(misc::maprec(%, {Dom::ImageSet, DOM_INT, DOM_RAT, DOM_COMPLEX,
                                DOM_IDENT}= float))
      end_if;


      
      // do *not* normalize here, because this would destroy
      // sums where some summands do not contain x
      
      return(solvelib::isolate(eq, 0, x, options))
      
    of DOM_POLY do
      return(solvelib::solve_poly(eq,x,options))
  end_case;
  // NOT REACHED 
  error("No appropriate method found in equation solver")
end_proc:



/* end of file */

