/* */
/* $Date: 2000/11/30 16:32:55 $ $Author: stefanw $ $Revision: 1.62 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
        solvelib::isolate  --  isolate an identifier to the lhs
--*/


solvelib::isolate :=
proc(lhside,Rhs,x,options, reclev)
  local Lhs, reclevel, l,i,a,s,arg,lev,
  pw, u:DOM_IDENT, v:DOM_IDENT,
  p, q,
  checkConditions: DOM_PROC,
  selectsols:DOM_PROC;

begin

// local method checkConditions
  
checkConditions:=
proc(sol, a)
  name checkConditions;
  local sols:DOM_SET;
begin
  userinfo(10, "Checking solution ".expr2text(sol));
                // in the list of conditions a, insert the solution
  sols:={op(map(a, subs, sol))};
  userinfo(20, "Substituting gives ".expr2text(sols));
  _and(op(sols))
end_proc;

// begin of main program

  Lhs:=lhside;
  if args(0)<5 then
    reclevel:=1
  else
    reclevel:=reclev
  end_if;

  if testargs() then
    if has(Rhs, x) then
      error("Right hand side must not contain the variable ". expr2text(x))
    end_if
  end_if;

  lev:=options["MaxDegree"];

  pw:=if options["IgnoreSpecialCases"] then
        solvelib::ignoreSpecialCases
      else
        (piecewise::flatten)@(piecewise::new)
      end_if;
 
  
  if reclevel>=30 then
    userinfo(2, "Maximum recursion level exceeded in solvelib::isolate");
    return(hold(solve)(Lhs=Rhs,x))
  end_if;
  userinfo(3,"trying to isolate ".expr2text(x)." in ".expr2text(Lhs=Rhs));
  userinfo(5,"Recursion level is ".expr2text(reclevel));
  if Lhs=x then
    return({Rhs})
  elif not has(Lhs,x) then
    return(pw([Lhs=Rhs, solvelib::BaseSet],
                      [Lhs<>Rhs, {}]))
  end_if;

  // case analysis, depending on the type of left hand side
  // solve f(x) = C for x, for different types of f
  
  case type(Lhs)
    of "_plus" do
      a:=split(Lhs, has, x);
      if not iszero(a[2]) then
        userinfo(5, "Subtracting ".expr2text(a[2])." on both sides gives ".
                 expr2text(a[1]). " = ". expr2text(Rhs-a[2]));
        return(solvelib::isolate(a[1], Rhs-a[2], x, options, reclevel+1))
      end_if;
     
      if iszero(Rhs) then
          /* try to recognize equations of the form f(a)-f(b)=0
            where f has an inverse function */
        if nops(Lhs)=2 then
          if type(op(Lhs,1))="_mult" then
            a:=2
          else
            a:=1
          end_if;
          i:=-op(Lhs, 3-a);
          a:=op(Lhs, a); /* equation is a=i */
          if type(a)=type(i) and type(a)="exp" then
            i:=op(i);
            return(solvelib::preImage(op(a), x, i+2*I*PI*Z_, options));
          end_if;
        end_if;
          /* try to factor the left hand side, but only over Q */
        l:=maprat(numer(Lhs), expr@factor);
        if type(l)="_mult" then
          return(_union(solvelib::isolate(op(l,i),0,x,options, reclevel+1)
                        $i=1..nops(l)))
        end_if
      end_if;
      /* try to recognize a polynomial */
      if testtype(Lhs,Type::PolyExpr(x)) then
        return(solvelib::solve_poly(poly(Lhs-Rhs,[x]),x,options))
      end_if;
      if has(Lhs,ln) then
        combine(Lhs,ln);
        if %<>Lhs then
          return(solvelib::isolate(%,Rhs,x,options, reclevel+1))
        end_if
      end_if;
      break;
    of "_mult" do
      a:=split(Lhs, has, x);
      if a[2]<>1 then
        userinfo(5, "Dividing both sides by ".expr2text(a[2])." gives ".
                 expr2text(a[1]). " = ". expr2text(Rhs/a[2]));
        return(pw
               ([a[2]<>0,
                 solvelib::isolate(a[1], Rhs/a[2], x, options, reclevel+1)],
                [a[2]=0, solve(Rhs,x)]))
      end_if;
      
      if iszero(Rhs) then /* one factor must be zero */
        userinfo(10, "Solving each factor separately");
          return(solvelib::solve_minus
                 (_union(solvelib::isolate(op(Lhs,i),0,x,options, reclevel+1)
                        $ i=1..nops(Lhs)), discont(Lhs, x, undefined),
                  options))
      end_if;
      break;
    of "_power" do
      s:=op(Lhs, 2);
      
      if not has(s,x) then
        // f(x)^C = 0 iff f(x)=0 for positive C; this is not solvable
        // for negative C or C=0
        // in general (complex C) one has f(x)^C = exp(C *ln(f(x)))
        // for nonzero f(x), which cannot be zero
        if iszero(Rhs) then
          return(piecewise([s>0 or Im(s)<>0,
                            solvelib::isolate(op(Lhs,1), 0, x, options,
                                              reclevel+1)],
                           [s<=0, {}]))
        end_if;
      
        case type(s)
          of DOM_INT do
            if s>0 then
              // solving u^s = a for u
              a:=solvelib::solve_poly(poly(x^s-Rhs,[x]),x,options);
              if op(Lhs,1)=x then
                return(a)
              else
                // it remains to solve op(Lhs,1)=u for x
                return(solvelib::preImage(op(Lhs,1), x, a, options))
              end_if
            else // s<0 -> invert both sides iff Rhs <> 0
              return(pw
                     ([Rhs=0, {}],
                      [Rhs<>0,
                       solvelib::isolate(op(Lhs,1)^(-s), 1/Rhs, x, options,
                                         reclevel+1)]))
            end_if
            // NOT REACHED
          of DOM_RAT do

            p:= op(s,1);
            q:= op(s,2);
            
            // note that base^(p/q) is the same as (base^(1/q))^p
            // and not the same as (base^p)^(1/q)

            // let u be the solutions of v^p=Rhs for v
            v:=genident();
            u:=solvelib::solve_eq(v^p-Rhs, v, options );
            
            /* u:=_union(solvelib::solve_eq(op(Lhs,1)-(Rhs)^(1/p)*
                                         exp(2*PI*I*j/p),
                                      x, options)
                      $ j=0..p-1); */

            // base^(1/q) is in u iff base is some u^q and u has small
            // argument
            // the latter is not always checked at the moment

            
            
            // warning("Possibly additional solutions");

            userinfo(1, "Possibly additional solutions");
            return(solvelib::preImage(op(Lhs,1), x, u^q,
                                      options))
            
          otherwise
            /* for complex numbers and symbolic expressions,
             write the Lhs as exp(...*ln(..))
             to solve: exp(s*ln(base) = Rhs
             this becomes (Rhs<>0) s*ln(base)=ln(Rhs)+2*k*PI*I */
            return(pw
                  ([Rhs=0 and (s>0 or Im(s)<>0),
                   solvelib::isolate(op(Lhs,1), 0, x, options,reclevel+1)],
                   [Rhs=0 and s<=0, {}],
                   [Rhs<>0,
                   solvelib::preImage(s*ln(op(Lhs,1)),x, ln(Rhs+2*PI*I*Z_),
                   options)]
                   ))
        end_case;

      
      else // x occurs in the exponent s
        if iszero(Rhs) then
          // solving a(x)^b(x) for x
          // this is zero iff a(x)=0 and not b(x)<=0
          return(solvelib::solve_minus
                 (solvelib::isolate(op(Lhs,1),0,x,options, reclevel+1),
                  solve(s<=0, x, options), options))           
        else
          return(solvelib::isolate(exp(s*ln(op(Lhs,1))),Rhs,
                                   x,options, reclevel+1))
        end_if
      end_if;
      break;
    of "arctan" do
      // new: can only have one argument 
        return(solvelib::isolate(op(Lhs),tan(Rhs),x,options, reclevel+1))
    of "arcsin" do
      return(pw([Re(Rhs) > PI/2 or Re(Rhs) < - PI/2, {}],
                       [Re(Rhs) <= PI/2 and Re(Rhs)>=-PI/2,
                        solvelib::isolate(op(Lhs), sin(Rhs), x,
                                          options, reclevel+1)]))
    of "arccos" do
      return(pw([Re(Rhs) < 0 or Re(Rhs) > PI, {}],
                       [Re(Rhs)>=0 and Re(Rhs)<=PI,
                        solvelib::isolate(op(Lhs), cos(Rhs), x, options,
                                          reclevel+1)]))
    of "ln" do // the imaginary part of a log must be  > -PI and <= PI 
      return(pw([Im(Rhs) <= -PI or Im(Rhs)>PI, {}],
                       [Im(Rhs)>-PI and Im(Rhs)<=PI,
                        solvelib::isolate(op(Lhs),exp(Rhs),x,options,
                                          reclevel+1)]))
    of "exp" do /* exp(a)=b <==> a = ln(b) + 2*I*k*PI */
      return(pw
             ([Rhs=0, {}],
              [Rhs<>0, solvelib::preImage(op(Lhs), x, ln(Rhs) +2*I*PI*Z_,
               options)]))
    of "sinh" do
      return(solvelib::preImage(op(Lhs),x ,arcsinh(Rhs)+2*I*PI*Z_, options))
    of "cosh" do
      l:=arccosh(Rhs);
      if l=0 then 
         return(solvelib::preImage(op(Lhs),x, 2*I*PI*Z_, options))
      else 
         return(solvelib::preImage(op(Lhs),x, l+2*I*PI*Z_,options) union
                solvelib::preImage(op(Lhs),x, -l+2*I*PI*Z_, options))
      end_if
    of "coth" do
    of "tanh" do
      return(solvelib::isolate(rewrite(Lhs, sincos), Rhs, x, options,
             reclevel+1))
    of "arcsinh" do
      return(solvelib::isolate(op(Lhs),sinh(Rhs),x,options, reclevel+1))
    of "arccosh" do
      return(solvelib::isolate(op(Lhs),cosh(Rhs),x,options, reclevel+1))
    of "cos" do /* cos(a)=b <==> a = +-arccos(b) + 2*k*PI */
      l:=arccos(Rhs);
      if l=0 then 
         return(solvelib::preImage(op(Lhs),x,2*PI*Z_, options))
      elif l=PI/2 then 
         return(solvelib::preImage(op(Lhs),x, l+PI*Z_, options))
      elif l=PI then 
         return(solvelib::preImage(op(Lhs),x, l+2*PI*Z_, options))
      else 
         return(solvelib::preImage(op(Lhs),x, l+2*PI*Z_, options) union
                solvelib::preImage(op(Lhs),x,-l+2*PI*Z_, options))
      end_if
    of "sin" do /* sin(a)=b <==> a = {arcsin(b),PI-arcsin(b)} + 2*k*PI */
      if Rhs=0 then 
         return(solvelib::preImage(op(Lhs),x,PI*Z_, options))
      elif Rhs=1 then 
         return(solvelib::preImage(op(Lhs),x, arcsin(Rhs)+2*PI*Z_, options))
      elif Rhs=-1 then 
         return(solvelib::preImage(op(Lhs),x,-PI/2+2*PI*Z_, options))
      else 
         return(solvelib::preImage(op(Lhs),x,arcsin(Rhs)+2*PI*Z_, options) union
               solvelib::preImage(op(Lhs),x, PI-arcsin(Rhs)+2*PI*Z_, options))
      end_if
    of "tan" do /* tan(a)=b <==> a=arctan(b)+k*PI */
      arg:=genident();
      return(
      piecewise([Rhs<>I and Rhs<>-I,
      solvelib::Union(
        solvelib::isolate(op(Lhs), arg, x, options, reclevel+1),
        arg, arctan(Rhs)+PI*Z_)], [Rhs=I or Rhs=-I, {}])
      )
    of "abs" do // abs(a)=b <=> (a=b or -a=b) and b>=0 if a is real
      a:=op(Lhs);
      if is(a, Type::Real)=TRUE then
        return(
        piecewise([Rhs>=0,
            solvelib::isolate(a,Rhs,x, options, reclevel+1)
                   union solvelib::isolate(-a, Rhs, x, options, reclevel+1)],
          [not Rhs>=0, {}]))
      else
        // fall through (substitute the abs using rectform below)
        break
      end_if
    of "sign" do
    of "Re" do
    of "Im" do
      // handle below by rewriting
      break
    of "heaviside" do
      a:=op(Lhs);
      if Rhs=heaviside(0) then
        return(solvelib::isolate(a, 0, x, options, reclevel+1))
      end_if;
      return(piecewise
            ([Rhs=0, solve(a<0, x, options)],
             [Rhs=1, solve(a>0, x, options)],
             [Rhs<>0 and Rhs<>1, {}]))
    // of "fact" do
    // of "gamma" do
    // of "function" do
    otherwise // cannot solve f(x)=C if f is unknown
      return(hold(solve)(Lhs=Rhs,x))
  end_case;

  // rewriting by piecewise should eliminate abs and sign

  Lhs:= rewrite(Lhs, piecewise);
  if type(Lhs)=piecewise then
    return(piecewise::solve(map(Lhs, _subtract, Rhs), x, options))
  end_if;

 // some changes on the left hand side
  // expand expressions like Re(x^2) using rectform 

  Lhs:=misc::maprec(Lhs,
                    {"Re", "Im"}=
                    proc(t)
                      name applyRectform;
                    begin
                      if has(t, x) then
                        expr(rectform(t))
                      else
                        t
                      end_if
                    end_proc); 
  
  userinfo(10, "Applying rectform to certain special functions gives ".
           expr2text(Lhs)." = ". expr2text(Rhs));

  if has(Lhs, hold(Re)(x)) or has(Lhs, hold(Im)(x)) then
    // substitute both Re(x) and Im(x) by new variables u and v, x=u+Iv
    u:=genident("U");
    v:=genident("V");
    // assume does not evaluate its arguments, we need a trick
    eval(hold(assume)(u, Type::Real));
    eval(hold(assume)(v, Type::Real));
    // immediately substitute Re(x) and Im(x) in order
    // to avoid the computation of Re(u+I*v) etc.
    Lhs:= subs(Lhs, hold(Re)(x)=u, hold(Im)(x)=v);
    Lhs:= eval(subs(Lhs, x=u+I*v));
    userinfo(10, "Substituting ".expr2text(hold(Re)(x))." by ".expr2text(u).
             " and ".expr2text(hold(Im)(x))." by ".expr2text(v)." gives ".
             expr2text(Lhs=Rhs));
    l:=solve(Lhs-Rhs, [u,v], Domain=Dom::Real );
    userinfo(10, "Solving the system gives ".expr2text(l));
    if type(l)="solve" then
      return(hold(solve)(lhside=Rhs, x))
    end_if;
    // now l is either an "_in" or a set of lists of equations
    // l contains solutions (u_0,v_0) for u and v; for each such solution,
    // u_0 + I*v_0 is a solution of the orig. equation for x
    l:=piecewise::mapMap(solvelib::vectorForm(l, [u,v]), t->t[1]+I*t[2]);
    userinfo(10, "Back substituting gives ".expr2text(l));
    // indeterminates in the solution must be understood as
    // ranging over all reals
    if (a:=indets(l) intersect {u,v})={} then
      return(l)
    elif nops(a)=2 then
      // should not happen !
      return(solvelib::Union(solvelib::Union(l, u, R_), v, R_))
    else
      return(solvelib::Union(l, op(a), R_))
    end_if;
  end_if;
    
  // try to see the expression Lhs as an element of a field
  // extension of Q(x)

  l:= solvelib::makeSystem(Lhs, Rhs, x);
  if nops(op(l,3))>1 then
    // there was at least one variable created by a substitution
    a:=op(l,4);
    userinfo(10, "Conditions are ".expr2text(a));
    u:=op(l,2);
    userinfo(10, "Nonalgebraic part is ".expr2text(u));
    l:=op(l,1), op(l,3);
    userinfo(10, "Algebraic system is  ".expr2text(l));

    // choose variable to solve for
    case nops(u)
      of 0 do
        v:=x;
        break
      of 1 do
        // only one non-alg. condition
        v:=lhs(op(u,1))
    end_case;

    if nops(u)<= 1 then
      userinfo(10, "Solving the system for ".expr2text(v))
    else
      userinfo(10, "Cannot solve system with more than one non-algebraic".
               " equation")
    end_if;
    
    if nops(u)=0
      // purely algebraic case
      or nops(u)=1 and not has(op(l,1),x) then
      // not really an eq. in x, but in f(x) (with f(x)=rhs(op(u,1)) )
      l:=solve(l, options);
      userinfo(10, "Solving gives ".expr2text(l));


      // local method selectsols

      // given the solutions l produced by the system solver,
      // select those that satisfy the conditions given by a
      
      selectsols:=
      proc(l)
        local checkConds;
        
      begin
        if type(l)="solve" then
          return(hold(solve)(lhside=Rhs, x))
        end_if;
        // for the set l of solutions, attach --- for each solution ---
        // the condition under which it is valid
        l:=[op(l)];
        checkConds:=map(l, checkConditions, a);
        _union(piecewise([checkConds[i], {l[i]}], [not checkConds[i], {}])
                  $i=1..nops(l));
      end_proc; // selectsols

      if type(l)=piecewise then
        l:= simplify(l); // often helps ..
        l:=map(l, selectsols)
      else
        l:=selectsols(l)
      end_if;


      // now l is the set of all tuples [v, var1, var2, ...]
      // that solve the system created by makeSystem
      // Here, var1,  var2, etc. are variables newly created by makeSystem
      // We apply a projection to the variable v:

      l:= solvelib::project(l, v,
                            indets({Lhs, Rhs}) minus Type::ConstantIdents
                            minus {v});

      if l=FAIL then
        return(hold(solve)(lhside=Rhs, x))
      end_if;
      
      if nops(u)=0 then
        return(l)
      else
        // we have solved for v= f(x); return the set of all x
        // such that f(x) is in that set
        return(solvelib::preImage(rhs(op(u,1)), x, l, options))
      end_if;
    end_if;
  end_if;


   /* try to expand the left hand side if it contains tan, cot, ... */
  l:=expand(Lhs);
  if indets(l,RatExpr)<>indets(Lhs,RatExpr) then
     userinfo(5, "Expanding the left hand side gives ".
              expr2text(l). " = ".expr2text(Rhs));
     return(solvelib::isolate(l,Rhs,x,options, reclevel+1))
   end_if;
   if testtype(Lhs,Type::RatExpr(x)) then
     return(solvelib::solve_minus
            (solvelib::solve_poly(poly(numer(Lhs-Rhs),[x]), x, options),
             solvelib::solve_poly(poly(denom(Lhs-Rhs), [x]), x, options),
             options))
   end_if;
   /* try to decompose */
  l:=rationalize(Lhs, NIL, NIL, {"sin","exp"});
  userinfo(3,"after rationalize, equation is",l[1]=Rhs,"with",l[2]);
  l[1]:=l[1]-Rhs;

   // special case: only one substitution and equations remains univariate
  if nops(l[2])=1 and not has(l[1],x) then
    i:=op(l[2],[1,1]);
     a:=solvelib::solve_eq(l[1],i,options);
     userinfo(3,"Set of solutions of ",l[1]," for ",i," is ",a);
     if not has(a,"solve")
       and a<>{i-l[1]} then
       /* to avoid inf. loops, i.e. in solve(exp(x)^(1/4)=C,x) */
       return(solvelib::preImage(op(l[2],[1,2]), x, a, options))
     end_if
  end_if;
  // end of special case

  simplify(Lhs);
  if %<>Lhs then
    // solve the simplified version
    // note that the singularities may be different now!
    return(solvelib::isolate(%, Rhs, x, options, reclevel+1 ))       
  end_if;


  // give up
  hold(solve)(Lhs=Rhs, x, solvelib::nonDefaultOptions(options))

end_proc:



/* end of file */

