/* $Date: 1999/12/15 18:31:30 $ $Author: stefanw $Revision: 1.3 $ */



// solvelib::solve_minus(S,T)

// computes S minus T correctly, even if S and T contain
// parameters

solvelib::solve_minus:=
proc(S, T, options:DOM_TABLE)
  local constS, paramS, constT, paramT, fconstT, pw;
begin

  // simple case
  
  if T={} or S={} then
    return(S)
  end_if;

  if S::dom::_minus <> FAIL then
    return(S::dom::_minus(S,T))
  elif T::dom::_minus <> FAIL then
    return(T::dom::_minus(S,T))
  end_if;

  if type(S)<>DOM_SET or type(T)<>DOM_SET then
    return(hold(_minus)(S,T))
  end_if;

  pw:=if options["IgnoreSpecialCases"] then
        solvelib::ignoreSpecialCases
      else
        piecewise
      end_if;

  // now S and T are sets
  // split into constant elements and elements with parameters
  
  paramS:=split(S, u -> contains({DOM_COMPLEX, DOM_FLOAT}, type(float(u))));
  constS:=op(paramS,1);
  paramS:=op(paramS,2);

  paramT:=split(T, u -> contains({DOM_COMPLEX, DOM_FLOAT}, type(float(u))));
  constT:=op(paramT,1);
  paramT:=op(paramT,2);

  fconstT:= map(constT, float);
  
  // S = constS union paramS
  // T = constT union paramT
  // to compute: S minus T = constS minus (constT union paramT) union
  //                         paramS minus T
  //                       = (constS minus constT) minus paramT union
  //                         paramS minus T

  
  constS:= select(constS, u-> not contains(fconstT, float(u)));

  map(constS, u -> pw([u in paramT, {}], [not (u in paramT), {u}]));
  constS:=_union(op(%));

  paramS:=map(paramS, u -> pw([u in T, {}], [not (u in T), {u}]));
  paramS:=_union(op(%));
  
  constS union paramS
    
end_proc:

