/* */
/* $Date: 2000/09/07 14:58:38 $ $Author: stefanw $ $Revision: 1.25 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
	solvelib::solve_poly_irred
--*/

solvelib::solve_poly_irred :=
proc(p,x) /* p is irreducible */
  local a,b,c,d,f,l, s,
  lev:DOM_INT,
  multi: DOM_BOOL,
  options: DOM_TABLE,
  // flattenSets: DOM_PROC,
  solve_inner: DOM_PROC,
  pw;
  
begin
  options:=solvelib::getoptions(args(3..args(0)));
  lev:=options["MaxDegree"];
  multi:=options["Multiple"];
  pw:= if options["IgnoreSpecialCases"] then
         solvelib::ignoreSpecialCases
       else
         (piecewise::flatten)@(piecewise::new)
       end_if;

  
  d:=degree(p,x);
  userinfo(2,"solving irreducible polynomial of degree ".expr2text(d));
  userinfo(10, "Polynomial is ".expr2text(p));
  userinfo(10, "Variable to solve for: ".expr2text(x));
  userinfo(10, "Options are ".expr2text(op(options)));
  if multi then
    s:=Dom::Multiset
  else
    s:=DOM_SET
  end_if;
  if d=0 then
    expr(coeff(p,x,0));
    // multi: deliver an error instead of BaseSet ? 
    pw([%=0, solvelib::BaseSet],[%<>0, {}])
  elif d=1 then
    if op(p,2)=[x] then
      l:=-coeff(p,x,0); d:=coeff(p,x,1)
    else
      l:=-expr(coeff(p,x,0)); d:=expr(coeff(p,x,1))
    end_if;
    pw([d<>0, s(l/d)], [d=0 and l=0,solvelib::BaseSet],
               [d=0 and l<>0, {}])
  elif d=2 and lev>=2 then
    a:=coeff(p,x,2); b:=coeff(p,x,1); c:=coeff(p,x,0);
    if op(p,2)<>[x] then
      a:=expr(a); b:=expr(b); c:=expr(c)
    end_if;
    d:=expand(b*b-4*a*c);
    /* if d=k*a^2 we can choose d=k^(1/2)*a */
    if type(d)="_mult" and type(op(d,1))="_power" and op(d,[1,2])=2 then
      d:=op(d,[1,1])*sqrt(subsop(d,1=1))
    else
      d:=sqrt(d)
    end_if;
    if is(b=0)<>TRUE then
      // to avoid division by zero 
      pw([a=0 and b=0 and c=0, solvelib::BaseSet],
                 [a=0 and b=0 and c<>0, {}],
                 [a=0 and b<>0, s(-c/b)],
                 [a<>0, s((-b+d)/2/a,(-b-d)/2/a)])
    else
      pw([a=0 and c=0, solvelib::BaseSet],
                 [a=0 and c<>0, {}],
                 [a<>0, s((-b+d)/2/a,(-b-d)/2/a)])
    end_if
  elif d=3 and lev>=3 then
    pw([coeff(p,x,3)=0, solvelib::solve_poly(divide(p, poly(x^3),
                                                    hold(Rem)), x, options) ],
       [coeff(p,x,3)<>0, solvelib::solve_poly3(p,x,options)  ])
  else /* d>=4 */
    userinfo(1,"trying polynomial decomposition");
    /* do not try decomposition if lev=1 */
    if lev<2 or nops((a:=[polylib::decompose(p,x)]))=1 then
/* no decomposition */
      userinfo(1,"decomposition failed");
      if d=4 and lev>=4 then
         pw([coeff(p,x,4)=0, solvelib::solve_poly(divide(p, poly(x^4),
                                                    hold(Rem)), x, options)],
            [coeff(p,x,4)<>0, solvelib::solve_poly4(p,x,options)  ])
      elif hastype((a:=expr(p)),DOM_FLOAT) and
        indets(a)={x} then
        s(op(numeric::polyroots(p))) 
      elif nterms(p)=2 then
           // polynom is of the form a*x^d + b
           // of course this can be solved by radicals also for high d
        pw([coeff(p,d)=0, solvelib::solve_poly(poly(coeff(p,0), [x]),
                                               x, options)],
           [coeff(p,d)<>0,
        s((-coeff(p,0)/coeff(p,d))^(1/d)* exp(2*k*PI*I/d) $k=0..d-1)])
      else
        userinfo(5, "Cannot solve by radicals");
        genident();
        RootOf(subs(a, x=%),%)
      end_if
    else
      userinfo(1,"decomposition succeeded");
      userinfo(5,"decomposed input into ".expr2text(nops(a)).
               " polynomials");

      // now a=[f1,..,fk]
      // This means we have to solve f1(f2(...(x)..))=0 for x
      // start by solving f1(y)=0 for y; for each solution y,
      // solve f2(z)=y for z, and so on
      
      b:=solvelib::solve_poly(a[1],x,options);
      delete a[1];
      for f in a do
        // we have solved f1(f2(..f_{i-1}(y)..))=0; the solutions
        // are stored in b . For each such solution y, we now have to solve
        // f(x) = y and store all of these x in b.
        

        // piecewise - case: uncommented because of efficiency problems
        
        if type(b)<>s /* and type(b)<>piecewise */ then
          /* cannot solve innermost -> return a RootOf */
          genident();
          return(RootOf(subs(expr(p),x=%), %))
        end_if;

        // b may be a set or a piecewise of sets of y's
        // we map a function that returns, for each y,
        // the set of solutions of f(x)=y
        
     
                             
        solve_inner:=
        proc(S)
          local solve_el;

        begin
          
          solve_el:=
          proc(y)
            local g;
          begin
            g:=f-poly(y, [x]);
            userinfo(10, "Polynomial to solve:".
                     expr2text(g));
            g:=solvelib::solve_poly(g, x, options);
            userinfo(10,"Set of solutions is ".
                     expr2text(g));
            g
          end_proc;

          case type(S)
            of piecewise do
              map(S, solve_inner); break
            of DOM_SET do
            of Dom::Multiset do  
              Dom::Multiset::nestedUnion(map(S, solve_el)); break
            of solvelib::BasicSet do
              // S=C_
              // but g(x) in C_ is true for every x anyway
              S; break
            otherwise
              error("Unknown type of solution ".expr2text(S))
          end_case;

        end_proc;
              

        b:= solve_inner(b);

        if type(b)=piecewise then b:=simplify(b) end_if;          
         
        // b might be set of piecewise objects, or a piecewise of such sets

        /*
        flattenSets:=
        proc(S)
          local uncond, cond, dummy;               
        begin
          if type(S)=piecewise then
            return(map(S, flattenSets))
          end_if;
          if type(S)<>DOM_SET then
            return(S)
          end_if;
          
          [cond, uncond, dummy]:=
          split(S, a-> type(a)=piecewise);
          // assertion: dummy={} must hold
          // now cond are piecewise objects,
          // and uncond are others

          // make every piecewise object
          // obj= a1 if c1,..., ak if ck
          // to a piecewise obj
          // {a1} if c1, ..., {ak} if ck
          // and compute their union
          cond:=map(cond, piecewise::map, el->{el});
          _union(op(cond)) union uncond
        end_proc;
        
        b:= flattenSets(b);

        */
        
        // now be is a set of sets: each of its elements contains the
        // solutions for one particular y
        // we have to compute the union of all these sets
        
        // Dom::Multiset can also handle sets
        // only one call necessary
        /* b:= Dom::Multiset::nestedUnion(b) */
      end_for;
      userinfo(5, "Return value is ".expr2text(b));
      b
    end_if
  end_if
end_proc:

/* end of file */



