/* $Date: 1999/12/07 15:25:19 $ $Author: stefanw $ $Revision: 1.14 $ */

/*--
	solvelib::solve_poly --  solve a polynomial

	solve_poly( p,x,options )

	p  : polynomial
	x  : indeteminate
	
	Solve a polynomial in explicit form, or returns RootOf(...) 
	for factors of degree>solve::maxdeg.
        Returns
	1) either {x=1}
	2) either {x=1,x=2,...}
	3) either {x=x}
	4) or {} if there is no solution

--*/

solvelib::solve_poly :=
proc(pp:DOM_POLY,x)
/* pp is supposed to be of type DOM_POLY, x is the variable */
local p,d,l,s,i,multi, options;
begin
  if testargs() then
    if args(0) < 2 then
      error("expecting at least two arguments")
    end_if
  end_if;

  p:=pp;
  
  options:=solvelib::getoptions(args(3..args(0)));
  userinfo(3, "solve_poly called with ".expr2text(p));
  userinfo(10, "solving for ".expr2text(x));
  userinfo(10, "Options are ".expr2text(options));
  multi:=options["Multiple"];
  /*  p:=poly(numer(op(p,1)),op(p,2..3)); */ /* why ? */
  if hastype(op(p,1),DOM_FLOAT) then /* remove 0.0 terms */
    userinfo(4, "Removing floating-point zeroes");
    p:=mapcoeffs(p,proc()
                   begin
                     (if iszero(args(1)) then 0 else args(1) end_if)
                   end_proc );
  end_if;
  d:=degree(p,x);
  if d<=1 then
    userinfo(10,  "poly must be irreducible because of its degree");
    return(solvelib::solve_poly_irred(p, x, options))
  else /* d >=2 */
    userinfo(1,"trying to factor polynomial of degree ".d);
    userinfo(2,"polynomial is ".expr2text(p));
    // do not try factor because there could be many irrational
    // coefficients
    l:=maprat(expr(p), factor);
    userinfo(5, "Factorization is ".expr2text(l));
    if multi then
      s:=Dom::Multiset()
    else  
      s:={}
    end_if;
    
    for i from 1 to nops(l) div 2 do
      userinfo(4, "Processing ".expr2text(i)."th irreducible factor");
      if multi then
        s:= s union
        (Dom::Multiset)::_union(
                 solvelib::solve_poly_irred(poly(l[2*i],[x]),x,options)
                                $l[2*i+1]);
        
      else
        s:=s union
            _union(solvelib::solve_poly_irred(poly(l[2*i],[x]), x,options))
      end_if;
    end_for;
    s
  end_if;
  userinfo(5, "Return value is ".expr2text(%)); %2
end_proc:

/* end of file */

