/* */
/* $Date: 1999/09/21 19:26:30 $ $Author: stefanw $ $Revision: 1.8 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
	solvelib::solve_poly3
	Reference: "Handbook of mathematical functions"
		edited by M. Abramowitz and I. Stegun, Dover, New York,
		page 17.

   rewritten by walter (was causing problems for complex coeffs)


--*/

solvelib::solve_poly3 :=
proc(p:DOM_POLY, x:DOM_IDENT, options: DOM_TABLE) 
  local numeric_mode,a,b,c,d,q,r,s1,s2,result, s;
begin
  userinfo(2,"entering degree 3 poly solver");
  if options["Multiple"]=TRUE then
    s:=Dom::Multiset
  else
    s:=DOM_SET
  end_if;
  if hastype(expr(p), DOM_FLOAT, {DOM_COMPLEX})
    then p:= mapcoeffs(p, float);
    numeric_mode:= TRUE;
    if {op(map(poly2list(p), domtype@op, 1))}
             // include DOM_INT, because of future float(0)=0
      minus {DOM_INT, DOM_FLOAT, DOM_COMPLEX} = {}
          then // no symbolic coefficients remain
               // Use numeric solver, because the symbolic
               // formulas may be numerically unstable.
               // E.g., try p:= (x-1)*(x-1/10^50)*(x-10^50)+10^(-100) !!
      return(s(op(numeric::polyroots(p))));
    else
      p:= subs(p, float(0)=0);
    end_if:
  else // there are no float entries
    numeric_mode:= FALSE;
  end_if:

  // potential problem for poly( float(0)*x^3 + ..., [x]):

  if degree(p)<>3 then
    error("not a polynomial of degree 3")
  end_if;

  /*---------------------------------------*/
  /* input form p(x) = a*x^3+b*x^2+c*x+d   */
  /*---------------------------------------*/
  a:=coeff(p,3); b:=coeff(p,2); c:=coeff(p,1); d:=coeff(p,0);
  /*---------------------------------------*/
  /* convert to p(x) = x^3+b*x^2+c*x+d     */
  /*---------------------------------------*/
  b:=b/a; c:=c/a; d:=d/a;
  /*---------------------------------------*/
  /* replace b,c,d by parameters b,q,r     */
  /*---------------------------------------*/
  q:=expand(c/3-b^2/9);
  r:=expand(1/6*(c*b-3*d)-1/27*b^3);
  /*---------------------------------------*/
  /* now p(x-b/3)= x^3 + 3*q*x - 2*r       */
  /*---------------------------------------*/

  /*---------------------------------------*/
  /* Solve  Q(x) = x^3 + 3*q*x - 2*r       */
  /* If z is a root of Q(x) = p(x-b/3),    */
  /* then z-b/3 is a root of p!            */
  /*---------------------------------------*/
  //  Ansatz: roots z1,z2,z3 of Q(x):
  //  [z1, z2, z3]:= [     (s1+s2),
  //                  -1/2*(s1+s2)+I*3^(1/2)/2*(s1-s2),
  //                  -1/2*(s1+s2)-I*3^(1/2)/2*(s1-s2)]:
  //  ->  (x-z1)*(x-z2)*(x-z3) = x^3 -3*x*s1*s2 -s1^3 -s2^3
  //  So s1, s2 are determined by the two equations:
  //          s1*s2       = -q ,   (1) 
  //          s1^3 + s2^3 = 2*r    (2)
  // Case 1:  q=0  -> s2=0, s1 = (2*r)^(1/3)
  // Case 2:  q<>0 -> s1<>0, s2=-q/s1<>0 
  //          s2 must satisfy:
  //              s1^3 - q^3/s1^3 = 2*r           (2')
  //          --> (s1^3)^2 - 2*r*(s1)^3 -q^3 = 0  (2'')
  //
  //          Solve this quadratic equation in s1^3 !
  //          s1:= (r +- sqrt(r^2+q^3))^(1/3)  (2'')
  //          s2:= -q/s1                       (1)
  
  if iszero(q) then
    s1:= (2*r)^(1/3); 
    s2:= 0;
  else
    d:=expand(q^3+r^2);
    s1:=(r+d^(1/2))^(1/3); 
    // Do not use  s2:=(r-d^(1/2))^(1/3) to make sure that
    // the symbolic result is correct after substituting
    // floats for the symbols. We do need s1*s2 = - q !!
    s2:= -q/s1;
  end_if;
  result:= s(     (s1+s2)-b/3,
            -1/2*(s1+s2)-b/3+I*3^(1/2)/2*(s1-s2),
            -1/2*(s1+s2)-b/3-I*3^(1/2)/2*(s1-s2) ):
  if numeric_mode // there are both symbolic entries as well as floats
    then
    return(subs(map(result, float), float(0)=0))
  else
    return(result);
  end_if:
end_proc:

/* end of file */

