/* */
/* $Date: 2000/12/06 13:46:37 $ $Author: stefanw $ $Revision: 1.31 $ */
/* */
/* zimmerma, 14/12/95 */

/*--
	solvelib::solve_sys
--*/

// Sys - set or list of equations
// Unk - set or list of unknowns

solvelib::solve_sys :=
proc(Sys, Unk:Type::Union(DOM_SET, DOM_LIST))
  local
  sys:DOM_LIST,
  newsys,eq,i,R,
  unk: DOM_LIST,
  options: DOM_TABLE,
  eqtypes, ring,
  substitutions: DOM_SET,
  params: DOM_SET;
  
begin
  case type(Sys)
    of DOM_SET do
      sys:=[op(Sys)];
      break
    of DOM_LIST do
      sys:=Sys;
      break
    otherwise
      error("First argument must be list or set")
  end_case;

  options:=solvelib::getoptions(args(3..args(0)));


  if nops(sys)=1 and nops(Unk)=1 then
    // really only one equation in one unknown
    R:=solve(op(sys, 1), op(Unk, 1), options);
    case type(R)
      of "solve" do
        return(hold(solve)(Sys, Unk, solvelib::nonDefaultOptions(options)))
      of DOM_SET do
        return({[op(Unk, 1)= op(R, i)] $i=1..nops(R)})
      otherwise
        return(hold(_in)(op(Unk, 1), R))
    end_case
  end_if;
  
  R:= options["Domain"];
  
  // convert all equations and polys to expressions

  
  sys:=map(sys,proc(equation)
                 name makeExpression;
                 local ring;
               begin
                 case type(equation)
                   of "_equal" do
                     op(equation,1)-op(equation,2);
                     break
                   of DOM_POLY do
                     ring:=op(equation, 3);
                     if has(ring, IntMod) then
                       ring:=Dom::IntegerMod(op(ring,1))
                     end_if;
                     if R=hold(Expr) then
                       R:=ring
                     else
                       // try to understand equation
                       // as a polynomial over the previously chosen ring
                       equation:=poly(equation, R);
                       if equation=FAIL then
                         error("Coefficient ring of polynomial does not ".
                               "match previously chosen domain for solving".
                               " or other coefficient ring");
                       end_if;
                     end_if;
                     expr(equation);
                     break
                   otherwise
                     equation
                 end_case
               end_proc );

  eqtypes:= map(sys, domtype);
  eqtypes:= select(eqtypes, c -> (slot(c, "solve_sys")<>FAIL));
  if nops(eqtypes) > 0 then
    userinfo(50, "System solver has been overloaded");
    ring:= eqtypes[1];
    sys:= map(sys, ring::convert);
    if has(sys, FAIL) then
      error("Could not convert all of the equations")
    end_if;
    options["Domain"]:=R;
    return(ring::solve_sys(sys, Unk, options))
  end_if;

  userinfo(3, "System solver called for ".expr2text(sys)." in unknowns ".
	expr2text(Unk));
  userinfo(10, "Options are ".expr2text(options));

  if hastype(sys, DOM_FLOAT, {DOM_COMPLEX}) then
    sys:=numeric::rationalize(sys);
    solvelib::solve_sys(sys, Unk);
    return(misc::maprec(%, {Dom::ImageSet, DOM_INT, DOM_RAT, DOM_COMPLEX,
                            DOM_IDENT}= float))
  end_if;
    
  
  if options["Multiple"]=TRUE then
    error("Option multiple is not allowed in the system solver")
  end_if;
  
  unk:=[op(Unk)];
  
  if map({op(sys)}, type) intersect
    {"_less", "_leequal", "_unequal"} <> {} then
    if nops(Unk)>1 then
      // cannot really solve system in more than one unknown
      return(hold(solve)(Sys, Unk))
    else  
      return(hold(_in)(op(Unk,1), solve(sys, op(Unk,1), options)))
    end_if
  end_if;
  
  if {op(Unk)} intersect Type::ConstantIdents <> {} then
    error("Invalid variable (mathematical constant was given)")
  elif nops(unk)=0 then
    error("Need variable(s) to solve for")
  end_if;
   /* remove unknowns not appearing in equations */
  unk:=select(unk,proc()
                  begin
                    has(sys,args(1))
                  end_proc );

  // no unknowns remaining after selection ?
  
  if unk=[] then
    return((if {op(sys)}={0} then {[]} else {} end_if))
  end_if;
  
  // substitute unknowns that are not identifiers
  eq:=select(unk,not testtype,DOM_IDENT);
  if eq<>[] then
    eq:=[(op(eq,i)=genident()) $ i=1..nops(eq)];
    newsys:=solvelib::solve_sys(subs(sys,eq), subs(Unk,eq), options);
    subs(newsys,(op(eq,[i,2])=op(eq,[i,1])) $i=1..nops(eq));
    return(%)
  end_if;

  // handle linear systems (do not handle parameters in a special way!)


  if (newsys:=solvelib::solve_islinear(sys,unk,options))<>FALSE then
    /* solve returns a set of solutions
       whereas linsolve always returns one solution */
    if R=hold(Expr) then
      i:= numeric::linsolve(newsys,unk,hold(Symbolic))
    else
      i:= linsolve(newsys,unk,hold(Domain)=R)
    end_if;
    if i=FAIL then
      return({}) /* no solution */
    else
      return({i})
    end_if
  end_if;


  // handle parametrized systems

  params:= solvelib::indets(sys) minus {op(unk)};
  if params<>{} and options["IgnoreSpecialCases"]=FALSE then
    // rationalize subexpressions that depend on params
    [newsys, substitutions] := [rationalize
    (sys, NIL,
     /* argument stop of rationalize */
     subexpr -> indets(subexpr) intersect params = {})];

    params:= params union map(substitutions, lhs);
    
    // solve system for unknowns and params
    solvelib::solve_sys(newsys, unk.[op(params)]);                       
    return(subs(solvelib::bound2Free(%, params), substitutions))
  end_if;
  
   // handle algebraic systems (*do* handle parameters in a special way)

  if (newsys:=solvelib::solve_isalgebraic({op(sys)},unk))<>FALSE then
    return({solvelib::solve_algebraic({op(newsys)},unk,NIL,options)})
  end_if;
  
 
 
  if nops(sys)=1 then /* only one equation */
    eq:=op(sys);
      /* look for the optimal unknown */
    if nops(unk)=1 then
      // this  can happen if some equations (e.g. 0=0) have
      // been removed since the head of solve_system was processed
      i:=1
    else
      for i from 1 to nops(unk) do
        if Type::Linear(eq,[op(unk,i)])<>FALSE then
          break
        end_if
      end_for;
      if i>nops(unk) then
        i:=1
      end_if
    end_if;
    newsys:=solvelib::solve_eq(eq,op(unk,i), options); 
    if type(newsys)<>"solve" then
      return(subsop(hold(dummy)(op(unk, i), newsys), 0=_in, Unsimplified))
    end_if
  end_if;
  userinfo(1,"Could not solve the non-algebraic system");
  hold(solve)(sys,Unk)
 
end_proc:

/* end of file */

