// $Date: 2000/10/31 14:28:14 $ $Author: stefanw $ $Revision: 1.18 $ 

/*++
        Ci -- the cosine integral

	Ci(x):= EULER + ln(x) + int((cos(t)-1)/t, t=0..x)
++*/

Ci:=
proc(x)
    option noDebug;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::Ci <> FAIL then return(x::dom::Ci(args())) 
  elif args(0) <> 1 then error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, Ci))
  end_case;
  
  if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  if iszero(x) then error("singularity") end_if;
      
  case domtype(x)
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(Ci(-x)+I*PI) end_if; break
    of DOM_FLOAT do
      if x < 0 // workaround for bug in Ci::float
         then return(slot(Ci,"float")(-x)+I*float(PI))
         else return(slot(Ci,"float")(x))
      end_if;
      break;
    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
        domtype(op(x,2)) = DOM_FLOAT then
        return(slot(Ci,"float")(x))
      end_if;
  end_case;

  procname(x)
end_proc:

Ci(infinity):=0:
Ci(-infinity):= I*PI: // = limit(Ci(x), x=-infinity, Right)
                      // along the negative real axis
Ci:= funcenv(Ci):
Ci:= subsop(Ci, 2=op(specfunc::Ci, 2)):
Ci:= slot(Ci,"print","Ci"):
Ci:= slot(Ci,"type","Ci"):
Ci:= slot(Ci,"info",
   "Ci(x) -- the Cosine integral EULER + ln(x) + int((cos(t)-1)/t, t=0..x)"):
Ci:= slot(Ci, "float", specfunc::Ci):

Ci:= slot(Ci, "diff",
              proc()
              begin  // e=Ci(f) 
                op(args(1),1);
                cos(%)/% * diff(%,args(2))
              end_proc ):

Ci:= slot(Ci, "undefined", {0}):
Ci:= slot(Ci, "realDiscont", {0}):
Ci:= slot(Ci, "complexDiscont", Dom::Interval(-infinity, [0])):

// end of file 
