// 
// $Date: 2000/10/31 14:28:15 $ $Author: stefanw $ $Revision: 1.22 $ 

// zimmerma, 10/08/95 

/*++
        Si -- the Sine integral

	Si(x):= int(sin(t)/t,t=0..x)

	Reference: Abramowitz & Stegun, pages 231-233
++*/

Si:=
proc(x)
  option noDebug;
  local f;
begin
  if args(0) < 1 then error("no arguments given")
  elif x::dom::Si <> FAIL then return(x::dom::Si(args()))
  elif args(0) <> 1 then error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, Si))
  end_case;
      
  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case domtype(x)
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(-Si(-x)) end_if; break
    of DOM_FLOAT do return(slot(Si,"float")(x))
    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
         domtype(op(x,2)) = DOM_FLOAT
      then return(slot(Si,"float")(x))
      end_if;
    //also normalize complex numbers ?
    //if op(x,1)<0 or (op(x,1)=0 and op(x,2)<0) then
    //   return(-Si(-x))
    //end_if;
  end_case;

  // in analogy to sin, cos:
  if type(x)= "_mult" then
      f:= op(x, nops(x));
      if testtype(f, Type::Real) and f < 0
         then return(-Si(-x))
      end_if;

    //also normalize complex factors ?
    //if testtype(f, Type::Complex) and 
    //   (Re(f) < 0 or (Re(f)=0 and Im(f)<0))
    //   then return(-Si(-x))
    //end_if;
  end_if;

  procname(x)
end_proc:

Si(0):=0:
Si(infinity):=PI/2:
Si(-infinity):=-PI/2:

Si:= funcenv(Si):
Si:= subsop(Si, 2=op(specfunc::Si, 2)):
Si:= slot(Si,"print","Si"):
Si:= slot(Si,"type","Si"):
Si:= slot(Si,"info","Si(x) -- the sine integral int(sin(t)/t,t=0..x)"):
Si:= slot(Si, "float", specfunc::Si):

Si:= slot(Si, "diff", proc() begin  // e=Si(f) 
   op(args(1),1);
   sin(%)/%*diff(%,args(2))
 end_proc ):

Si:= slot(Si, "complexDiscont", {}):
Si:= slot(Si, "realDiscont", {}):
Si:= slot(Si, "undefined", {}):

Si:= slot(Si, "series",
    loadproc(slot(Si,"series"),pathname("SERIES"),"Si")
):

// end of file 
