// $Date: 2000/10/31 14:28:16 $ $Author: stefanw $ $Revision: 1.18 $ 

// kg, 13/01/94 

/*++
arccosh -- the inverse hyperbolic cosine

arccosh(x)

x - expression
++*/

arccosh:= proc(x)
  option noDebug;
  name arccosh;
  local k;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::arccosh <> FAIL then return(x::dom::arccosh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
  of DOM_SET do
  of "_union" do
      return( map(x,arccosh) )
  end_case;

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_FLOAT do
	return(slot(arccosh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(arccosh,"float")(x))
	end_if;
	break;

    of "cosh" do // arccosh(cosh(x))=x when 0 <= Im x <= PI 
        k:=round(-Im(op(x))/2/PI);
        return((if type(k)=DOM_INT then
           k:=op(x)+2*k*PI*I;
           if sign(Im(k))=-1 then -k else k end_if;
        else procname(x) end_if))

    end_case;

    procname(x)
end_proc:

arccosh(1):= 0:
arccosh(0):=1/2*I*PI:

arccosh:= funcenv(arccosh):
arccosh:= subsop(arccosh, 2=op(specfunc::arccosh, 2)):
arccosh:= slot(arccosh,"print","arccosh"):
arccosh:= slot(arccosh,"info","arccosh -- the inverse hyperbolic cosine"):
arccosh:= slot(arccosh,"type","arccosh"):
arccosh:= slot(arccosh, "float", specfunc::arccosh):
// potential library fix for PARI's arccosh:
//proc(x) begin
//      x:= float(x):
//      ln(x + sqrt(x^2-1))
// end_proc):

// Folgendes complexDiscont ist richtig!  Fehler in  Abramowitz/Stegun!
arccosh:= slot(arccosh, "complexDiscont", Dom::Interval(-infinity, -1) union
               Dom::Interval(1, infinity)):
arccosh:= slot(arccosh, "realDiscont", {}):
arccosh:= slot(arccosh, "undefined", {}):

arccosh:= slot(arccosh,"inverse", "cosh"):

arccosh:= slot(arccosh,"diff",proc() begin 
    op(args(1),1);
    diff(%, args(2..args(0))) / sqrt(%^2-1)
 end_proc ):

arccosh:=slot(arccosh,"series",proc(f,x,n,dir)
begin
   series(ln(f+sqrt(f^2-1)),x,n,
          if dir = FAIL then null() else dir end_if)
end_proc):

// end of file 
