
// ok, 27/01/94 

/*++
arccot -- the inverse cotangens

arccot(x)

x - expression
++*/

arccot:=
proc(x)
  option noDebug;
  local f;
  name arccot;
begin
  if args(0) = 0 then
    error("no arguments given")
  elif x::dom::arccot <> FAIL then
    return(x::dom::arccot(args()))
  elif args(0) <> 1 then
    error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, arccot))
  end_case;
  
  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(-arccot(-x)) end_if;
      break;
 
    of DOM_FLOAT do
      return(slot(arccot,"float")(x));
      
    of DOM_COMPLEX do
      if x = I then
        error("singularity");
      elif domtype(op(x,1)) = DOM_FLOAT or
        domtype(op(x,2)) = DOM_FLOAT then
        return(slot(arccot,"float")(x))
      elif op(x, 1) = 0 then
        return( -I*arccoth(-I*x) );
      elif op(x, 1) < 0 and op(x, 2) < 0 then
        return(-arccot(-x));
      end_if;
      break;
 
    of "_mult" do
      f:= op(x, nops(x));
      if testtype(f, Type::Real) then
        if f < 0 then return(-arccot(-x)) end_if;
      end_if;
      break;

    of "cot" do // the real part of arctan is in ]-PI/2,PI/2] 
        if (is(op(x)>-PI/2) and is(op(x)<=PI/2))=TRUE then return(op(x))
        elif type((f:=round(-Re(op(x))/PI)))=DOM_INT then return(op(x)+f*PI)
        end_if;
        break;

    end_case;
 
    procname(x)
end_proc:

arccot( infinity) := 0:
arccot(-infinity) := 0:

arccot(0) := PI/2:
arccot( (3^(1/2))/3 ):= PI/3:
arccot(-(3^(1/2))/3 ):=-PI/3:
arccot( 1):=  PI/4:
arccot(-1):= -PI/4:
arccot( 1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2) ):=   PI/5:
arccot( cos(PI/5)/sin( PI/5) )             :=   PI/5:
arccot(-cos(PI/5)/sin(-PI/5) )             :=   PI/5:
arccot( 1/tan( PI/5))                      :=   PI/5:
arccot(-1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2) ):= - PI/5:
arccot(-cos(PI/5)/sin(PI/5) )              := - PI/5:
arccot( cos(PI/5)/sin(-PI/5) )             := - PI/5:
arccot( cos(4*PI/5)/sin(4*PI/5) )          := - PI/5:
arccot(-1/tan( PI/5))                      := - PI/5:
arccot( 1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2) ):= 2*PI/5:
arccot( cos(2*PI/5)/sin( 2*PI/5) )         := 2*PI/5:
arccot(-cos(2*PI/5)/sin(-2*PI/5) )         := 2*PI/5:
arccot(-cos(3*PI/5)/sin( 3*PI/5) )         := 2*PI/5:
arccot( 1/tan(2*PI/5))                     := 2*PI/5:
arccot(-1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2) ):=-2*PI/5:
arccot(-cos(2*PI/5)/sin(2*PI/5) )          :=-2*PI/5:
arccot( cos(2*PI/5)/sin(-2*PI/5) )         :=-2*PI/5:
arccot( cos(3*PI/5)/sin( 3*PI/5) )         :=-2*PI/5:
arccot(-1/tan(2*PI/5))                     :=-2*PI/5:
arccot( (3^(1/2)) ):= PI/6:
arccot(-(3^(1/2)) ):=-PI/6:
arccot( 2^(1/2)+1 )              :=   PI/8:
arccot( cos(PI/8)/sin( PI/8))    :=   PI/8:
arccot( 1/tan( PI/8))            :=   PI/8:
arccot(-1/tan(-PI/8))            :=   PI/8:
arccot(-cos(PI/8)/sin(-PI/8))    :=   PI/8:
arccot(-2^(1/2)-1 )              := - PI/8:
arccot(-cos(PI/8)/sin( PI/8))    := - PI/8:
arccot( cos(PI/8)/sin(-PI/8))    := - PI/8:
arccot(-1/tan( PI/8))            := - PI/8:
arccot( 1/tan(-PI/8))            := - PI/8:
arccot( 2^(1/2)-1 )              := 3*PI/8:
arccot( cos(3*PI/8)/sin(3*PI/8)) := 3*PI/8:
arccot(-cos(3*PI/8)/sin(-3*PI/8)):= 3*PI/8:
arccot( 1/tan( 3*PI/8))          := 3*PI/8:
arccot(-1/tan(-3*PI/8))          := 3*PI/8:
arccot(-2^(1/2)+1 )              :=-3*PI/8:
arccot(-cos(3*PI/8)/sin(3*PI/8)) :=-3*PI/8:
arccot( cos(3*PI/8)/sin(-3*PI/8)):=-3*PI/8:
arccot(-1/tan(   PI/8))          :=-3*PI/8:
arccot( 1/tan(-3*PI/8))          :=-3*PI/8:
arccot( (5+2*5^(1/2))^(1/2) )      :=   PI/10:
arccot( cos(PI/10)/sin( PI/10) )   :=   PI/10:
arccot(-cos(PI/10)/sin(-PI/10) )   :=   PI/10:
arccot( 1/tan(   PI/10))           :=   PI/10:
arccot(-(5+2*5^(1/2))^(1/2) )      := - PI/10:
arccot(-cos(PI/10)/sin( PI/10) )   := - PI/10:
arccot( cos(PI/10)/sin(-PI/10) )   := - PI/10:
arccot(-1/tan(   PI/10))           := - PI/10:
arccot( (5-2*5^(1/2))^(1/2) )      := 3*PI/10:
arccot( cos(3*PI/10)/sin( 3*PI/10)):= 3*PI/10:
arccot(-cos(3*PI/10)/sin(-3*PI/10)):= 3*PI/10:
arccot( 1/tan( 3*PI/10))           := 3*PI/10:
arccot(-(5-2*5^(1/2))^(1/2) )      :=-3*PI/10:
arccot(-cos(3*PI/10)/sin( 3*PI/10)):=-3*PI/10:
arccot( cos(3*PI/10)/sin(-3*PI/10)):=-3*PI/10:
arccot(-1/tan( 3*PI/10))           :=-3*PI/10:
arccot( 2+3^(1/2) )                  :=   PI/12:
arccot( cos(PI/12)/sin( PI/12))      :=   PI/12:
arccot(-cos(PI/12)/sin(-PI/12))      :=   PI/12:
arccot(-cos(11*PI/12)/sin( 11*PI/12)):=   PI/12:
arccot( cos(11*PI/12)/sin(-11*PI/12)):=   PI/12:
arccot( 1/tan( PI/12))               :=   PI/12:
arccot(-1/tan(-PI/12))               :=   PI/12:
arccot(-2-3^(1/2) )                  := - PI/12:
arccot(-cos(PI/12)/sin(PI/12))       := - PI/12:
arccot( cos(PI/12)/sin(-PI/12))      := - PI/12:
arccot( cos(11*PI/12)/sin( 11*PI/12)):= - PI/12:
arccot(-cos(11*PI/12)/sin(-11*PI/12)):= - PI/12:
arccot(-1/tan( PI/12))               := - PI/12:
arccot( 1/tan(-PI/12))               := - PI/12:
arccot( 2-3^(1/2) )                  := 5*PI/12:
arccot( cos(5*PI/12)/sin( 5*PI/12))  := 5*PI/12:
arccot(-cos(5*PI/12)/sin(-5*PI/12))  := 5*PI/12:
arccot(-cos(7*PI/12)/sin( 7*PI/12))  := 5*PI/12:
arccot( cos(7*PI/12)/sin(-7*PI/12))  := 5*PI/12:
arccot( 1/tan( 5*PI/12))             := 5*PI/12:
arccot(-1/tan(-5*PI/12))             := 5*PI/12:
arccot(-2+3^(1/2) )                  :=-5*PI/12:
arccot(-cos(5*PI/12)/sin( 5*PI/12))  :=-5*PI/12:
arccot( cos(5*PI/12)/sin(-5*PI/12))  :=-5*PI/12:
arccot( cos(7*PI/12)/sin( 7*PI/12))  :=-5*PI/12:
arccot(-cos(7*PI/12)/sin(-7*PI/12))  :=-5*PI/12:
arccot(-1/tan( 5*PI/12))             :=-5*PI/12:
arccot( 1/tan(-5*PI/12))             :=-5*PI/12:

arccot:= funcenv(arccot):
arccot:= slot(arccot,"type", "arccot"):
arccot:= slot(arccot,"print", "arccot"):
arccot:= slot(arccot,"info","arccot -- the inverse cotangent"):

arccot:= slot(arccot,"inverse", "cot"):


arccot:= slot(arccot, "undefined", {0}):
arccot:= slot(arccot,"realDiscont", {0}):
arccot:= slot(arccot, "complexDiscont",
        loadproc(arccot::complexDiscont, pathname("STDLIB","DISCONT"),
                     "arccot")):


arccot:= slot(arccot,"diff",proc() begin 
    op(args(1),1);
    diff(%, args(2..args(0))) * ( -1/(1+%^2) )
 end_proc ):

arccot:= slot(arccot, "float", proc(e)
begin
  if iszero(e) then float(PI/2)
  else slot(arctan, "float")(1/e)
  end_if
end_proc):

arccot:= slot(arccot, "series",
    loadproc(slot(arccot,"series"),pathname("SERIES"),"arccot")
):

// end of file 
