
// ok, 27/01/94 

/*++
arccoth -- the inverse hyperbolic cotangens

arccoth(x)

x - expression
++*/

arccoth:= proc(x)
  option noDebug;
  name arccoth;
begin
  if args(0) < 1 then error("no arguments given")
  elif x::dom::arccoth <> FAIL then return(x::dom::arccoth(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
  of DOM_SET do
  of "_union" do                                                                            
      return(map(x, arccoth))
  end_case;

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  case domtype(x)
    of DOM_INT do
    of DOM_RAT do
	if x = 1 then
           error("singularity")
	elif x < 0 then
           return(-arccoth(-x))
        end_if;
	break;
    
    of DOM_FLOAT do
	return(slot(arccoth,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(arccoth,"float")(x))
//      elif op(x,1) = 0 then
//          return(-I*arccot(-I*x))
//      elif op(x,1) < 0 and op(x,2) < 0 then
//          return(-arccoth(-x))
	end_if;
	break;

    end_case;

    procname(x)
end_proc:

arccoth(0) := PI/2*I:

arccoth:= funcenv(arccoth):
arccoth:= slot(arccoth,"type","arccoth"):
arccoth:= slot(arccoth,"print","arccoth"):
arccoth:= slot(arccoth,"info","arccoth -- the inverse hyperbolic cotangent"):
arccoth:= slot(arccoth,"inverse", "coth"):

arccoth:= slot(arccoth, "undefined", {-1, 1}):
arccoth:= slot(arccoth, "realDiscont", {-1, 1}):
arccoth:= slot(arccoth, "complexDiscont", Dom::Interval([-1, 1])):

arccoth:= slot(arccoth, "diff", proc() begin 
    op(args(1),1);
    diff(%, args(2..args(0))) * ( -1/(%^2-1) )
 end_proc ):

arccoth:= slot(arccoth, "float", proc(e)
begin
    // arccoth(0) = PI/2*I              
    // arccoth(1) = singularity         
    // arccoth(e) = arctanh(1/e) , e <> 0 
    if e = 1 or e = float(1) then
       error("singularity");
    elif iszero(e) 
         then float(PI/2*I)
         else slot(arctanh, "float")(1/e)
    end_if
end_proc):

arccoth:=slot(arccoth,"series",proc(f,x,n,dir)
begin
   series(1/2*(ln(1+1/f) - ln(1-1/f)), x, n,
          if dir = FAIL then null() else dir end_if)
end_proc):

// end of file 
