// $Date: 2000/10/31 14:28:17 $ $Author: stefanw $ $Revision: 1.23 $ 

// kg, 13/01/94 

/*++
arcsin -- the inverse sine

arcsin(x)

x - expression

NB:  The branch cuts are the real intervals (-infinity, -1) and
     (1, infinity). 
     Note: for x >=1 :  arcsin(x) =  limit(arcsin(x-eps*I),eps=0, eps>0)
           for x <=-1:  arcsin(x) =  limit(arcsin(x+eps*I),eps=0, eps>0)

     I.e., on the right real line the values coincide with the 
     limits from below, on the left real line with the limit from
     above. OK.
++*/

arcsin:= proc(x)
  option noDebug;
  local f;
  name arcsin;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::arcsin <> FAIL then return(x::dom::arcsin(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  elif domtype(x) = DOM_SET then
      return(map(x,arcsin))
  elif not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-arcsin(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(slot(arcsin,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(arcsin,"float")(x))
	end_if;
	if op(x,1) = 0 then return(arcsinh(-x*I)*I) end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-arcsin(-x)) end_if;
	end_if;
	break;

    of "sin" do // arcsin : [-1..1] --> [-PI/2..PI/2] 
        if (is(op(x)>=-PI/2) and is(op(x)<=PI/2))=TRUE then return(op(x))
        elif type((f:=round(Re(op(x))/PI)))=DOM_INT then
           return((-1)^f*(op(x)-f*PI))
        end_if; break
    end_case;

    procname(x)
end_proc:

arcsin(0):= 0:
arcsin( 1):= PI/2:
arcsin(-1):=-PI/2:
arcsin( (3^(1/2))/2):= PI/3:
arcsin(-(3^(1/2))/2):=-PI/3:
arcsin( (2^(1/2))/2):= PI/4:
arcsin(-(2^(1/2))/2):=-PI/4:
arcsin( 1/4*2^(1/2)*(5-5^(1/2))^(1/2)):= PI/5:
arcsin(-1/4*2^(1/2)*(5-5^(1/2))^(1/2)):=-PI/5:
arcsin( 1/4*2^(1/2)*(5+5^(1/2))^(1/2)):= 2*PI/5:
arcsin(-1/4*2^(1/2)*(5+5^(1/2))^(1/2)):=-2*PI/5:
arcsin( 1/2):= PI/6:
arcsin(-1/2):=-PI/6:
arcsin( ((2-2^(1/2))^(1/2))/2):= PI/8:
arcsin(-((2-2^(1/2))^(1/2))/2):=-PI/8:
arcsin( ((2+2^(1/2))^(1/2))/2):= 3*PI/8:
arcsin(-((2+2^(1/2))^(1/2))/2):=-3*PI/8:
arcsin( 1/4*(5^(1/2)-1)):= PI/10:
arcsin(-1/4*(5^(1/2)-1)):=-PI/10:
arcsin( 1/4*(5^(1/2)+1)):= 3*PI/10:
arcsin(-1/4*(5^(1/2)+1)):=-3*PI/10:
arcsin( (6^(1/2)-2^(1/2))/4):= PI/12:
arcsin(-(6^(1/2)-2^(1/2))/4):=-PI/12:
arcsin( (6^(1/2)+2^(1/2))/4):= 5*PI/12:
arcsin(-(6^(1/2)+2^(1/2))/4):=-5*PI/12:

arcsin( infinity):=  PI/2 - I*infinity:
arcsin(-infinity):= -PI/2 + I*infinity:
arcsin(I*infinity):=   I*infinity:
arcsin(-I*infinity):= -I*infinity:


arcsin:= funcenv(arcsin):
arcsin:= subsop(arcsin, 2=op(specfunc::arcsin, 2)):
arcsin:= slot(arcsin,"print","arcsin"):
arcsin:= slot(arcsin,"info","arcsin -- the inverse sine" ):
arcsin:= slot(arcsin,"type","arcsin"):
arcsin:= slot(arcsin, "float", specfunc::arcsin):

arcsin:= slot(arcsin,"inverse", "sin"):

arcsin:= slot(arcsin,"diff",proc(f) begin
    op(f,1);
    diff(%, args(2..args(0))) / sqrt(1-%^2)
end_proc):


// the arcsin has a branch cut at the negative real axis
arcsin:= slot(arcsin, "complexDiscont",
              Dom::Interval(-infinity,-1) union Dom::Interval(1, infinity)):
arcsin:= slot(arcsin, "realDiscont", {}):
arcsin:= slot(arcsin, "undefined", {}):

arcsin:= slot(arcsin,"rectform",
    loadproc(slot(arcsin,"rectform"),pathname("STDLIB","RECTFORM"),"asin")
):

arcsin:= slot(arcsin, "series",
    loadproc(slot(arcsin,"series"),pathname("SERIES"),"arcsin")
):

// end of file 
