// $Date: 2000/10/31 14:28:17 $ $Author: stefanw $ $Revision: 1.19 $ 

// kg, 13/01/94 

/*++
arcsinh -- the inverse hyperbolic sine

arcsinh(x)

x - expression
++*/

arcsinh:= proc(x)
  option noDebug;
  local f;
  name arcsinh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::arcsinh <> FAIL then return(x::dom::arcsinh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
  of DOM_SET do
  of "_union" do
     return(map(x,arcsinh))
  end_case; 

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-arcsinh(-x)) end_if;
	break;
    of DOM_FLOAT do
	return(slot(arcsinh,"float")(x));
    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(arcsinh,"float")(x))
	end_if;
	break;
    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-arcsinh(-x)) end_if;
	end_if;
	break;
    of "sinh" do // arcsinh(sinh(x))=x when -PI/2 < Im x <= PI/2 
        f:=round(-Im(op(x))/PI);   
        if type(f)=DOM_INT then return((-1)^f*(op(x)+f*I*PI)) end_if;
        break;
    end_case;

    procname(x)
end_proc:

arcsinh(0):= 0:

arcsinh:= funcenv(arcsinh):
arcsinh:= subsop(arcsinh, 2=op(specfunc::arcsinh, 2)):
arcsinh:= slot(arcsinh, "print", "arcsinh"):
arcsinh:= slot(arcsinh,"info","arcsinh -- the inverse hyperbolic sine"):
arcsinh:= slot(arcsinh, "type", "arcsinh"):
arcsinh:= slot(arcsinh, "float", specfunc::arcsinh):
// potential library fix for PARI's arcsinh:
// proc(x) begin
//    x:= float(x);
//    ln(x +sqrt(1+x^2));
//  end_proc):

arcsinh:= slot(arcsinh, "inverse", "sinh"):

arcsinh:= slot(arcsinh, "undefined", {}):
arcsinh:= slot(arcsinh, "realDiscont", {}):
arcsinh:= slot(arcsinh, "complexDiscont",
          loadproc(arcsinh::complexDiscont, pathname("STDLIB","DISCONT"),
                     "arcsinh")):

arcsinh:= slot(arcsinh, "diff", proc(f) begin
    op(f,1);
    diff(%, args(2..args(0))) / sqrt(%^2+1)
end_proc):

arcsinh:=slot(arcsinh,"series", proc(f, x, n, dir)
begin
   // formula 4.6.20 of Abramowitz & Stegun 
  //Series::Puiseux(ln(f+sqrt(f^2+1)),x,n);
   series(ln(f+sqrt(f^2+1)),x,n,
          if dir = FAIL then null() else dir end_if)
end_proc):

// end of file 
