// $Date: 2000/10/31 14:28:18 $ $Author: stefanw $ $Revision: 1.22 $ 

// kg, 13/01/94 

/*++
arctanh -- the inverse hyperbolic tangens

arctanh(x)

x - expression
++*/

arctanh:= proc(x)
  option noDebug;
  local f;
  name arctanh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::arctanh <> FAIL then return(x::dom::arctanh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do                                                                            
      return(map(x, arctanh))
  end_case;

  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
        if x = 1 then
           error("singularity");
	elif x < 0 then return(-arctanh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(slot(arctanh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(arctanh,"float")(x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-arctanh(-x)) end_if;
	end_if;
	break;

    of "tanh" do // arctanh(tanh(x))=x iff -PI/2 < Im(x) < PI/2 
        f:=round(Im(op(x))/PI);
        if type(f)=DOM_INT then return(op(x)-f*I*PI) end_if;
        break;
    end_case;

    procname(x)
end_proc:

arctanh(0):= 0:
arctanh:= funcenv(arctanh):
arctanh:= subsop(arctanh, 2=op(specfunc::arctanh, 2)):
arctanh:= slot(arctanh, "info", "arctanh - the inverse hyperbolic tangens"):
arctanh:= slot(arctanh, "type", "arctanh"):
arctanh:= slot(arctanh, "print", "arctanh"):
arctanh:= slot(arctanh, "float", specfunc::arctanh):
// potential library fix for PARI's arctanh:
// proc(x) begin
//   x:= float(x):
//   ln( (1+x)^(1/2) ) - ln( (1-x)^(1/2) ) ;
// end_proc):  

arctanh:= slot(arctanh, "inverse", "tanh"):

arctanh:= slot(arctanh, "undefined", {-1, 1}):
arctanh:= slot(arctanh, "realDiscont", {-1, 1}):
arctanh:= slot(arctanh, "complexDiscont",
               Dom::Interval(-infinity, [-1]) union
               Dom::Interval([1], infinity)):


arctanh:= slot(arctanh, "diff", proc() begin 
    op(args(1),1);
    diff(%, args(2..args(0))) / (1-%^2)
 end_proc ):

arctanh:= slot(arctanh, "series", proc(f,x,n,dir)
 //local s,i;
begin
 /*
   s:=(if n<2 then [] else [1,(0,1/(2*i+1))$i=1..(n div 2)-1] end_if);
   s:=Series::Puiseux::create(1,1,n,s,x);
   if f=x then s
   else
      Series::Puiseux::_fconcat(s,Series::Puiseux(f,x,n))
   end_if
 */
   series(1/2*(ln(1+f)-ln(1-f)), x, n,
          if dir = FAIL then null() else dir end_if)
end_proc):

// end of file 
