
// ok, 21/01/94 

/*++
Bernoulli numbers and polynomials

bernoulli(n)
bernoulli(n, x)

n - a nonnegative integer
x - an expression
++*/

bernoulli := proc(n, x)
   option noDebug;
   local t, k, s, k0, x2;
begin
  case args(0)
    of 1 do
       if domtype(n) = DOM_INT and n >= 0 then
          return(specfunc::bernoulliNumber(n));
       end_if;
       if testtype(float(n), Type::Numeric) then
          error("first argument must be symbolic or a nonnegative integer")
       end_if;
       if not testtype(n,Type::Arithmetical) then
          error("first argument must be of type 'Type::Arithmetical'")
       end_if;
       return(procname(n));
    of 2 do
        // most probable case first: n is a nonnegative integer
        if domtype(n) = DOM_INT and n>=0
        then if n = 0 then return(1)
             elif n = 1 then return(x-1/2)
             // special cases, no need to evaluate the polynomial:
             elif x = 0  then return(specfunc::bernoulliNumber(n))
             elif x = 1 then return((-1)^n*specfunc::bernoulliNumber(n))
             elif x = 1/2 then return((2^(1-n)-1)*specfunc::bernoulliNumber(n)) 
             elif x = 1/3 and domtype(n/2)=DOM_INT then
                  return((3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             elif x = 2/3 and domtype(n/2)=DOM_INT then
                  return((3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             elif x = 1/4 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)/2^n*specfunc::bernoulliNumber(n)) 
             elif x = 3/4 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)/2^n*specfunc::bernoulliNumber(n)) 
             elif x = 1/6 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)*(3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             elif x = 5/6 and domtype(n/2)=DOM_INT then
                  return((2^(1-n)-1)*(3^(1-n)-1)/2*specfunc::bernoulliNumber(n)) 
             else // evaluate the polynomial, use
                  // B(n,x) = sum(binomial(n,k)*x^k*B(n-k), k=0..n)
                  if n mod 2 = 0
                  then  t:= 1;
                        s:= specfunc::bernoulliNumber(n);
                       k0:= 2;
                  else  t:= n*x;
                        s:= t*specfunc::bernoulliNumber(n-1):
                       k0:= 3;
                  end_if;
                  //compute  sum(binomial(n,k)*x^k*B(n-k), k=0..n-2)
                  x2:= x^2:
                  for k from k0 to n-2 step 2 do
                    t:= t*(n-k+2)*(n-k+1)/k/(k-1)*x2;
                    s:= s + t*specfunc::bernoulliNumber(n - k);
                  end_for:
                  // add the last 2 terms:
                  t:= t*x/(n-1);
                  s:= s - t;
                  t:= 2*x*t/n;
                  return(s+t);
             end_if;
        // Various simplifications, if n is not a nonnegative integer.
        // The call bernoulli(n) checks, if n is valid:
        elif x = 0 then return(bernoulli(n))
        elif x = 1 then return((-1)^n*bernoulli(n))
        elif x =-1 then return(bernoulli(n) + n*(-1)^n)
        elif x = 1/2 then return((2^(1-n)-1)*bernoulli(n)) 
        elif x = 1/3 and domtype(n/2)=DOM_INT then
             return((3^(1-n)-1)/2*bernoulli(n)) 
        elif x = 2/3 and domtype(n/2)=DOM_INT then
             return((3^(1-n)-1)/2*bernoulli(n)) 
        elif x = 1/4 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)/2^n*bernoulli(n)) 
        elif x = 3/4 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)/2^n*bernoulli(n)) 
        elif x = 1/6 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)*(3^(1-n)-1)/2*bernoulli(n)) 
        elif x = 5/6 and domtype(n/2)=DOM_INT then
             return((2^(1-n)-1)*(3^(1-n)-1)/2*bernoulli(n)) 
        elif testtype(x, Type::Real) and x > 1/2 and x < 1 then
             return((-1)^n*bernoulli(n,1-x))
        elif testtype(x, Type::Real) and x < 0 then
             return((-1)^n*bernoulli(n,-x) + n*(-1)^n*(-x)^(n-1))
        // errors: 
        elif testtype(float(n), Type::Numeric) then
             error("first argument must be symbolic or a nonnegative integer")
        elif not testtype(n,Type::Arithmetical) then
             error("first argument must be of type 'Type::Arithmetical'")
        elif not testtype(x,Type::Arithmetical) then
             error("second argument must be of type 'Type::Arithmetical'")
        // symbolic return
        else return(procname(n, x))
        end_if;
        break;
    otherwise
        error("1 or 2 arguments expected.")
  end_case
end_proc:

bernoulli:= funcenv(bernoulli):
bernoulli:= slot(bernoulli, "type", "bernoulli"):
bernoulli:= slot(bernoulli, "print", "bernoulli"):
bernoulli:= slot(bernoulli, "conjugate",
            proc(n, x) begin
                if args(0) = 1 
                   then bernoulli(n)
                   else bernoulli(n, conjugate(x))
                end_if;
                end_proc):

// auxiliary routine for the Bernoulli numbers with option remember
specfunc::bernoulliNumber:=
proc(n)
local a, b, i;
option remember;
begin
   if n mod 2 <> 0
   then return( 0 )
   else a := 1;
        b := 1/2-1/2*n;
        for i from 2 to n-2 step 2 do
            a :=  (a * (n+3-i) * (n+2-i)) div ((i-1)*i);
            b := b + a * specfunc::bernoulliNumber(i)
        end_for; 
        return( -b / (n+1) )
   end_if
end_proc:
     
specfunc::bernoulliNumber(0):= 1;
specfunc::bernoulliNumber(1):= -1/2;

/* end of file */
