//    Differentiate the Bessel and related functions w.r.t the second argument
//
//    Function derivatives coded here:
//      BesselJ, BesselY, BesselI, BesselK
//

specfunc::besselDiffMain := proc(f,g,v,z,C,x)
    // f : Bessel or Bessel-related function being differentiated
    // g : Auxiliary function (0 if not required)
    // v : order expression
    // z : argument expression
    // C : list of coefficients used in the 2 forms of the recurrence
    //                 relations (A&S 9.1.27, 9.6.26, 9.9.14/15, 12.1.9/10,
    //                 12.2.4/5, and related formulas for the Anger and
    //                 Weber functions (see, e.g., Watson, A Treatise on
    //                 Bessel Functions)
    // x : differentiation variable
    option noDebug;
    begin
    if has(v,x) then
	return(diff(f(v,z),x));
    elif (type(v)=DOM_INT or type(v)=DOM_RAT) and v > 1/2 then
	   return( (C[1]*(f(v-1,z)+g(v-1,z))+C[2]*v/z*f(v,z)+C[3])*diff(z,x) );
    end_if;
    (C[4]*(f(v+1,z)+g(v+1,z))+C[5]*v/z*f(v,z) + C[6])*diff(z,x);
end_proc:
