// T.C. Scott,  Jan 2000

besselI:=
proc(vv,z)
local f1,f2,fv,fz,v,r,r2,s0,t1,t2,res,z2,sv,sz,Vv,zz,dummy;
begin
  v:=vv;
  if args(0) = 0 then
    error("no arguments given")
  elif z::dom::besselI <> FAIL then
    return( z::dom::besselI(args()) )
  elif contains( {DOM_BOOL,DOM_FAIL,DOM_NULL,DOM_NIL},domtype(v) ) then
    error("invalid operand")
  end_if;

  // ----------  map to sets -------------------------
  // This is quite complicated, because both arguments have
  // to be handled
  case [type(v),type(z)]
    of [DOM_SET, DOM_SET] do
        return({besselI(sv, sz) $ sv in v $ sz in z});
    of [DOM_SET, "_union"] do
    of ["_union", DOM_SET] do
        return(_union((besselI(sv, sz) $ sv in v) $ sz in z));
    of ["_union", "_union"] do

        // besselI({a} union A, C union D)
        //  -> besselI({a},C) union besselI({a},D) union besselI(A,C) union besselI(A,D)
        // Make sure that A, C, D are interpreted as sets, i.e.,
        // besselI({a},C) --> besselI(a, C), not {besselI(a,C)} !

        [v, Vv, dummy]:= split(v, testtype, DOM_SET):
        if type(Vv) <> "_union" then Vv:= [Vv] end_if:
        [z, zz, dummy]:= split(z, testtype, DOM_SET):
        if type(zz) <> "_union" then zz:= [zz] end_if:

        return(_union({(besselI(sv, sz) $ sv in v ) $ sz in z},
                       (besselI(sv, sz) $ sv in Vv) $ sz in z,
                       (besselI(sv, sz) $ sv in v ) $ sz in zz,
                       (besselI(sv, sz) $ sv in Vv) $ sz in zz ));
  end_case;

  case type(v)
    of DOM_SET do
    of "_union" do
       // z cannot be a set, if v is a set
       return(map(v, besselI, z));
  end_case;

  case type(z)
    of DOM_SET do
    of "_union" do
       // v cannot be a set, if z is a set
       return(map(z, (z,v) -> besselI(v,z), v))
  end_case;
  // --------------------------------------------------
  if not testtype(z,Type::Arithmetical) or
    not testtype(v,Type::Arithmetical)
    then
    error("arguments must be of type 'Type::Arithmetical'")
  end_if;

  fv:= domtype(float(v)):
  fz:= domtype(float(z)):
  if (fv=DOM_FLOAT or fv=DOM_COMPLEX) and (fz=DOM_FLOAT or fz=DOM_COMPLEX)
   and has([map((op(v),op(z)),type)],DOM_FLOAT) then
         return(slot(besselI,"float")(v,z))
  end_if:

  if iszero(z) then
     if iszero(v) then
            return(1)
      elif (fv=DOM_FLOAT or fv=DOM_COMPLEX) and (Re(v) > 0 or domtype(v)=DOM_INT) then
            return(0):
      elif (fv=DOM_FLOAT or fv=DOM_COMPLEX) and not(Re(v) > 0 or domtype(v)=DOM_INT) then
            error("singularity"):
      else
            procname(v,z);
      end_if:
  end_if:

  if type(v) <> DOM_INT and type(2*v)=DOM_INT then
   // might want to limit this expansion to say abs(v) <= 10
   // and let "expand" take care of cases > 10.
   // Formulae worked out from Abramowitz and Stegun 10.2.9 and 10.2.10
   // Note: divides into even and odd cases where exp(z) and exp(-z) 
   // are rewritten in terms of sinh(z) and cosh(z)
   if v < 1/2 then
       v := -v-1/2;
       if iszero(v mod 2) then
         f2:=sinh(z);
         f1:=cosh(z);
       else
         f1:=sinh(z);
         f2:=cosh(z);
       end_if:
    else
        v := v-1/2;
        if iszero(v mod 2) then
         f1:=sinh(z);
         f2:=cosh(z);
        else
         f2:=sinh(z);
         f1:=cosh(z);
        end_if:
    end_if:
    z2:=z^2;
    s0:=1/2*v*(v+1)/z;
    t1:=1;
    t2:=s0;
    res:= f1*_plus(1,(r2:=2*r;t1:=t1*1/8*(v+r2-1)*(v+r2)/(r2-1)/r*(v-r2+1)*(v-r2+2)/z2)
             $ r=1..trunc(v/2))
            -f2*_plus(s0,(r2:=2*r;t2:=t2*1/8*(v+r2)*(v+r2+1)/r/(r2+1)/z2*(v-r2)*(v-r2+1))
             $ r=1..trunc((v-1)/2));
    sqrt(2/(PI*z))*res;
    else procname(v,z)
  end_if
end_proc:

besselI := funcenv(besselI):
besselI:= slot(besselI,"type","besselI"):
besselI:= slot(besselI,"print","besselI"):
besselI:= slot(besselI,"info",
  "besselI(v,z) -- bessel function of the second kind, order v and argument z"
):
besselI::Content      := (Out, data) -> Out::Capply(Out::CbesselI, map(op(data), Out)):


besselI := slot(besselI, "float",
    loadproc(slot(besselI, "float"), pathname("STDLIB","FLOAT"),"besselI")
):

besselI := slot(besselI,"diff",
    loadproc(slot(besselI,"diff"),pathname("STDLIB","DIFF"),"besselI")
):

besselI := slot(besselI,"series",
    loadproc(slot(besselI,"series"),pathname("SERIES"),"besselI")
):

// end of file 
