// Updated T.C. Scott Feb 2000

/*++
        besselY -- bessel function of the second kind, order v and argument z

        besselY(v,z)
++*/

besselY:= proc(v,z)
local fv,fz,n,sv,sz,vv,zz,dummy;
begin
  if args(0) = 0 then error("no arguments given")
  elif z::dom::besselY <> FAIL then
    return( z::dom::besselY(args()) )
  elif args(0) <> 2 then
    error("expecting two arguments")
  end_if;

  // ----------  map to sets -------------------------
  // This is quite complicated, because both arguments have
  // to be handled
  case [type(v),type(z)]
    of [DOM_SET, DOM_SET] do
        return({besselY(sv, sz) $ sv in v $ sz in z});
    of [DOM_SET, "_union"] do
    of ["_union", DOM_SET] do
        return(_union((besselY(sv, sz) $ sv in v) $ sz in z));
    of ["_union", "_union"] do

        // besselY({a} union A, C union D)
        //  -> besselY({a},C) union besselY({a},D) union besselY(A,C) union besselY(A,D)
        // Make sure that A, C, D are interpreted as sets, i.e.,
        // besselY({a},C) --> besselY(a, C), not {besselY(a,C)} !

        [v, vv, dummy]:= split(v, testtype, DOM_SET):
        if type(vv) <> "_union" then vv:= [vv] end_if:
        [z, zz, dummy]:= split(z, testtype, DOM_SET):
        if type(zz) <> "_union" then zz:= [zz] end_if:

        return(_union({(besselY(sv, sz) $ sv in v ) $ sz in z},
                       (besselY(sv, sz) $ sv in vv) $ sz in z,
                       (besselY(sv, sz) $ sv in v ) $ sz in zz,
                       (besselY(sv, sz) $ sv in vv) $ sz in zz ));
  end_case;

  case type(v)
    of DOM_SET do
    of "_union" do
       // z cannot be a set, if v is a set
       return(map(v, besselY, z));
  end_case;

  case type(z)
    of DOM_SET do
    of "_union" do
       // v cannot be a set, if z is a set
       return(map(z, (z,v) -> besselY(v,z), v))
  end_case;
  // --------------------------------------------------
  if not testtype(z,Type::Arithmetical) or
    not testtype(z,Type::Arithmetical)
    then
    error("arguments must be of type 'Type::Arithmetical'")
  end_if:

  fv:= domtype(float(v)):
  fz:= domtype(float(z)):
  if (fv=DOM_FLOAT or fv=DOM_COMPLEX) and (fz=DOM_FLOAT or fz=DOM_COMPLEX)
   and has([map((op(v),op(z)),type)],DOM_FLOAT) then
         return(slot(besselY,"float")(v,z))
  end_if:

  if iszero(z) then
        error("singularity"):
  end_if:
  if type(v) <> DOM_INT and type(2*v)=DOM_INT then
    // might want to limit this expansion to say abs(v) <= 10
    // and let "expand" take care of cases > 10.
    // Abramowitz and Stegun eq. 10.1.15
    n := v-1/2;
    return((-1)^(n+1) * besselJ(-v,z));
  end_if:
  procname(v,z)
end_proc:

besselY := funcenv(besselY):
besselY:=slot(besselY,"type","besselY"):
besselY:=slot(besselY,"info",
"besselY(v,z) -- bessel function of the second kind, order v and argument z"):
besselY:=slot(besselY,"print","besselY"):
besselY::Content      := (Out, data) -> Out::Capply(Out::CbesselY, map(op(data), Out)):

besselY:=slot(besselY,"float",
    loadproc(slot(besselY,"float"),pathname("STDLIB","FLOAT"),"besselY")
):

besselY := slot(besselY,"diff",
    loadproc(slot(besselY,"diff"),pathname("STDLIB","DIFF"),"besselY")
):

besselY := slot(besselY,"series",
    loadproc(slot(besselY,"series"),pathname("SERIES"),"besselY")
):

// end of file 
