binomial := proc(n, k)

   option noDebug;
   local b, i;
   name binomial;

begin
    if args(0) <> 2 then error("2 arguments expected") end_if;

    if k = 0 or k = n then    // (1) 
       return( 1 )
    elif k = 1 or k = n-1 then
       return( n )
    end_if;

    if not testtype(n, Type::Numeric) or
       not testtype(k, Type::Numeric) 
    then
        if not testtype(n,Type::Arithmetical) or
           not testtype(k,Type::Arithmetical) 
           then error("arguments must be of type 'Type::Arithmetical'")
           else return( procname(n, k) )
        end_if
    end_if;

    case domtype(k)
    of DOM_INT do
       case domtype(n)
         of DOM_INT do
            if k < 0 then
               if 0 <= n then 
                  return( 0 )
// n > 0, k < 0  => n-k > 0
//               elif n-k < 0 then 
//                  return( 0 )
               else 
                  return( binomial(n, n-k) )
               end_if
            else // k > 0, see (1) 
               if n < 0 then 
                  return( (-1)^k*binomial(k-n-1, k) )
               elif n-k < k then 
                  return( binomial(n, n-k) )
               end_if
            end_if;
            break;
         of DOM_RAT     do
         of DOM_FLOAT   do
         of DOM_COMPLEX do
            if k < 0 then 
               return( 0 )
            end_if;
            break;
       end_case;

       b := n;   
       for i from k downto 2 do  b := b * (n+i-k-1)/i end_for;
       break;

    of DOM_RAT do
        case domtype(n)
          of DOM_FLOAT do
             return( gamma(n+1)/gamma(float(k)+1)/gamma(n-k+1) );
          of DOM_COMPLEX do
             if domtype(op(n,1)) = DOM_FLOAT 
                or domtype(op(n,2)) = DOM_FLOAT then
                return( gamma(n+1)/gamma(float(k)+1)/gamma(n-k+1) )
             end_if;
        end_case;
        procname(n, k); break;

    of DOM_FLOAT do
        gamma(float(n)+1)/gamma(k+1)/gamma(n-k+1);
        break;

    of DOM_COMPLEX do
        if domtype(k) = DOM_FLOAT or 
           domtype(op(n,1)) = DOM_FLOAT or
           domtype(op(n,2)) = DOM_FLOAT then
           gamma(float(n+1))/gamma(float(k+1))/gamma(float(n-k+1))
        else
           procname(n, k);
        end_if;
        break;
   end_case
end_proc:

binomial := funcenv(binomial):
binomial := slot(binomial, "type", "binomial"):
binomial := slot(binomial, "print", "binomial"):

binomial := slot(binomial, "float", 
                 proc(n, k) begin binomial(float(n), float(k)) end_proc
                    ):

binomial := slot(binomial,"expand",
   proc(n,k) 
   begin 
      if testtype(k,Type::NegInt) or 
         testtype(n-k,Type::NegInt) 
      then 0
  //  elif contains(map({k,n-k,n},type),DOM_FLOAT) then
  //       expand(gamma(n+1)/gamma(k+1)/gamma(n-k))
  //  else expand(fact(n)/fact(k)/fact(n-k))         // do not use fact
      else expand(gamma(n+1)/gamma(k+1)/gamma(n-k+1))
      end_if
   end_proc):

binomial := slot(binomial,"simplify",
   proc(e) 
      local n, k;
   begin 
      n:= op(e,1); k:= op(e,2);

      if testtype(k,Type::NegInt) or testtype(n-k,Type::NegInt) then 0
      else e
      end_if
   end_proc):

// series(binomial(f1, f2), x, n, dir)
binomial := slot(binomial,"series",proc(f1, f2, x, n, dir) begin 
   series(gamma(f1+1)/gamma(f2+1)/gamma(f1-f2+1), x, n,
          if dir = FAIL then null() else dir end_if)
end_proc):
