// $Date: 2000/05/21 15:02:42 $ $Author: bij $ $Revision: 1.22 $ 

// kg, 13/01/94 

/*++
cos -- the cosine

cos(x)

x - expression
++*/

cos:= proc(x)
  option noDebug;
  local f, y;
  name cos;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::cos <> FAIL then return(x::dom::cos(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;
  
  case type(x)
  of DOM_SET do
  of "_union" do
      return(map(x, cos))
  end_case; 

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of type 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(cos(-x)) end_if;
	break;

    of DOM_FLOAT do
	return(slot(cos,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	       return(slot(cos,"float")(x))
	   end_if;
	if op(x,1) = 0 then return(cosh(-x*I)) end_if;
	break;

    of "_mult" do
/*--
due to performance reasons, move this code to cos::simplify
--*/
/*
       if nops(x)>nops(x/PI) then // x = multiple of PI
          if is(x/(2*PI), Type::Integer) = TRUE then
               return(1)
          elif is( (f:= x/PI), Type::Integer) = TRUE then
               // f = n*k with integer n and assume(k, Type::Integer)
               // i.e., x = n*k*PI. We may reduce n modulo 2.
               if type(f)="_mult" then
                  f:= map(f, proc(n) begin
                             if domtype(n)=DOM_INT
                                then n mod 2
                                else n
                             end_if
                             end_proc);
               end_if:
               return((-1)^f)
          elif is( (f:= 2*x/PI), Type::Even) = TRUE then
               // f = n*k with integer n and assume(k, Type::Even)
               // i.e., x = n*k*PI/2. We may reduce n modulo 4.
               if type(f)="_mult" then
                  f:= map(f, proc(n) begin
                             if domtype(n)=DOM_INT
                                then n mod 4
                                else n
                             end_if
                             end_proc);
               end_if:
               return((-1)^(f/2))
          elif is(2*x/PI, Type::Odd) = TRUE then
               return(0)
 	end_if;
       end_if;
*/
//--
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(cos(-x)) end_if;
	    if domtype(f) = DOM_FLOAT then break end_if;
	else
	    break
	end_if;

	if nops(x) = 2 then
	    if op(x,1) = PI then
		if f < 1/2 then break end_if;
		if f < 1 then return(-cos((1-f)*PI)) end_if;
		if f < 2 then return(cos((2-f)*PI)) end_if;
		return(cos((f-2*(floor(f) div 2))*PI));
	    end_if
	end_if;
	break;
    of "_plus" do
       // due to performance: move this code to cos::simplify
       // if sysorder(-x,x)=TRUE then return(cos(-x)) else break end_if

       // handle cos(y + integer*PI) -> (-1)^integer*cos(y)
       // We could also do cos(y + PI/2) -> -sin(y).
       // At the moment, we decide not to do it.
       if has(x, PI) then
          for y in x do
              f:= y/PI;
              if testtype(f, DOM_INT) then
                 return((-1)^f*cos(x-y))
              end_if;
          end_for:
       end_if:
       break;
    of "arcsin" do return(sqrt(1-op(x)^2));
    of "arccos" do return(op(x));
    of "arctan" do return(1/sqrt(1+op(x)^2));
    of "arccot" do x:= 1/op(x);
                   return(1/sqrt(1+x^2));
    end_case;

    procname(x)
end_proc:

cos(I):= cosh(1):
cos(0):= 1:
cos(PI):= -1:
cos(PI/2):= 0:
cos(PI/3):= 1/2:
cos(PI/4):= (2^(1/2))/2:
cos(PI/  5):= (5^(1/2)+1)/4:
cos(PI*2/5):= (5^(1/2)-1)/4:
cos(PI/6):= (3^(1/2))/2:
cos(PI/  8):= ((2+2^(1/2))^(1/2))/2:
cos(PI*3/8):= ((2-2^(1/2))^(1/2))/2:
cos(PI  /10):= 2^(1/2)/4*(5+5^(1/2))^(1/2):
cos(PI*3/10):= 2^(1/2)/4*(5-5^(1/2))^(1/2):
cos(PI/  12):= (6^(1/2)+2^(1/2))/4: // simpler than ((2+3^(1/2))^(1/2))/2: 
cos(PI*5/12):= (6^(1/2)-2^(1/2))/4: // = ((2-3^(1/2))^(1/2))/2:

cos:= funcenv(cos):
cos:= subsop(cos, 2=op(specfunc::cos, 2)):
cos:= slot(cos, "print", "cos"):
cos:= slot(cos, "type", "cos"):
cos:= slot(cos, "info", "cos -- the cosine" ):
cos:= slot(cos, "float", specfunc::cos):

cos:= slot(cos, "inverse", "arccos"):

cos:= slot(cos, "expand",
    loadproc(slot(cos,"expand"),pathname("STDLIB","EXPAND"),"cos")
):

cos:= slot(cos, "conjugate",
    loadproc(slot(cos,"conjugate"),pathname("STDLIB","CONJ"),"cos")
):

cos:= slot(cos,"rectform",
    loadproc(slot(cos,"rectform"),pathname("STDLIB","RECTFORM"),"cos")
):

cos:= slot(cos,"simplify",
    loadproc(slot(cos,"simplify"),pathname("STDLIB","SIMPLIFY"),"cos")
):

cos:= slot(cos, "series",
    loadproc(slot(cos,"series"),pathname("SERIES"),"cos")
):

cos:= slot(cos,"Re",
    loadproc(slot(cos,"Re"),pathname("STDLIB","RE"),"cos")
):

cos:= slot(cos,"Im",
    loadproc(slot(cos,"Im"),pathname("STDLIB","IM"),"cos")
):

cos:= slot(cos, "diff",
    loadproc(slot(cos,"diff"),pathname("STDLIB","DIFF"),"cos")
):

// end of file 
