// $Date: 2000/05/21 15:02:42 $ $Author: bij $ $Revision: 1.19 $ 

// kg, 13/01/94 

/*++
cosh -- the hyperbolic cosine

cosh(x)

x - expression
++*/

cosh:= proc(x)
  option noDebug;
  local f;
  name cosh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::cosh <> FAIL then return(x::dom::cosh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, cosh))
  end_case;

  if not testtype(x,Type::Arithmetical) then
      error("argument must be of type 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(cosh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(slot(cosh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(cosh,"float")(x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(cosh(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then return((-1)^f) // cosh(n*I*PI)= (-1)^n 
        elif type(2*f)=DOM_INT then return(0) // cosh( (n+1/2)*I*PI ) = 0 
	end_if;
	break;

    // cf Table 4.5.60 p. 85 in Abramowitz and Stegun 
    of "arccosh" do return(op(x))
    of "arcsinh" do return(sqrt(1+op(x)^2))
    of "arctanh" do return(1/(1-op(x))^(1/2)/(1+op(x))^(1/2))
    of "arccoth" do x:= 1/op(x):
                    return(1/(1-x)^(1/2)/(1+x)^(1/2))
    end_case;

    procname(x)
end_proc:

cosh(0):= 1:
cosh(I*PI):= -1:
cosh(I*PI/2):= 0:
cosh(I*PI/3):= 1/2:
cosh(I*PI/4):= (2^(1/2))/2:
cosh(I*PI/  5):= (5^(1/2)+1)/4:
cosh(I*PI*2/5):= (5^(1/2)-1)/4:
cosh(I*PI/6):= (3^(1/2))/2:
cosh(I*PI/  8):= ((2+2^(1/2))^(1/2))/2:
cosh(I*PI*3/8):= ((2-2^(1/2))^(1/2))/2:
cosh(I*PI  /10):= 2^(1/2)/4*(5+5^(1/2))^(1/2):
cosh(I*PI*3/10):= 2^(1/2)/4*(5-5^(1/2))^(1/2):
cosh(I*PI/  12):= (6^(1/2)+2^(1/2))/4: // simpler than ((2+3^(1/2))^(1/2))/2: 
cosh(I*PI*5/12):= (6^(1/2)-2^(1/2))/4: // = ((2-3^(1/2))^(1/2))/2:

cosh( infinity):= infinity:
cosh(-infinity):= infinity:

cosh:= funcenv(cosh):
cosh:= subsop(cosh, 2=op(specfunc::cosh, 2)):
cosh:= slot(cosh, "print", "cosh"):
cosh:= slot(cosh, "type", "cosh"):
cosh:= slot( cosh, "info", "cosh -- the hyperbolic cosine"):
cosh:= slot(cosh, "float", specfunc::cosh):

cosh:= slot( cosh, "inverse", "arccosh"):

cosh:= slot(cosh, "diff", proc(f) begin
    op(f,1);
    diff(%, args(2..args(0))) * sinh(%)
end_proc):

cosh := slot(cosh, "expand",
    loadproc(slot(cosh,"expand"),pathname("STDLIB","EXPAND"),"cosh")
):

cosh:= slot(cosh, "conjugate",
    loadproc(slot(cosh,"conjugate"),pathname("STDLIB","CONJ"),"cosh")
):

cosh:= slot(cosh,"rectform",
    loadproc(slot(cosh,"rectform"),pathname("STDLIB","RECTFORM"),"cosh")
):

cosh:= slot(cosh, "series",
    loadproc(slot(cosh,"series"),pathname("SERIES"),"cosh")
):

cosh:= slot(cosh,"simplify",
    loadproc(slot(cosh,"simplify"),pathname("STDLIB","SIMPLIFY"),"cosh")
):

cosh:= slot(cosh,"Re",
    loadproc(slot(cosh,"Re"),pathname("STDLIB","RE"),"cosh")
):

cosh:= slot(cosh,"Im",
    loadproc(slot(cosh,"Im"),pathname("STDLIB","IM"),"cosh")
):

// end of file 
