// $Date: 2000/05/21 15:02:43 $ $Author: bij $ $Revision: 1.22 $ 

// kg, 13/01/94 

/*++
cot -- the cotangens

cot(x)

x - expression
++*/

cot:= subsop(tan, [2,4]="cot", 1=proc(x)
    option noDebug;
    local f, y;
    name cot;
begin
    if args(0) = 0 then error("no arguments given")
    elif x::dom::cot <> FAIL then return(x::dom::cot(args()))
    elif args(0) <> 1 then error("wrong no of args") 
    elif domtype(x) = DOM_SET then
        return(map(x,cot))
    end_if;

    if x = 0 or x = PI then error("singularity") end_if;

    if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
    end_if;

    case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-cot(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(slot(cos,"float")(x)/slot(sin,"float")(x));


    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
           domtype(op(x,2)) = DOM_FLOAT then
	       return(slot(cos,"float")(x)/slot(sin,"float")(x));
	   end_if;
	if op(x,1) = 0 then return(-coth(-x*I)*I) end_if;
	break;

    of "_mult" do
/*--
due to performance reasons, move this code to cot::simplify
--*/
/*
       if nops(x)>nops(x/PI) then // x = multiple of PI
          if is(x/PI, Type::Integer) = TRUE then
               error("singularity");
          elif is(2*x/PI, Type::Even) = TRUE then
               error("singularity");
          elif is(2*x/PI, Type::Odd) = TRUE then
               return(0)
        end_if;
       end_if;
*/
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-cot(-x)) end_if;
	    if domtype(f) = DOM_FLOAT then break end_if;
	else
	    break
	end_if;

	if nops(x) = 2 then
	    if op(x,1) = PI then
		if f = 0 then error("singularity") end_if;
		if f < 1/2 then break end_if;
		if f < 1 then return(-cot((1-f)*PI)) end_if;
		if f = 1 then error("singularity") end_if;
		if f < 2 then return(cot((f-1)*PI)) end_if;
		return(cot((f-2*(floor(f) div 2))*PI));
	    end_if
	end_if;
	break;
    of "_plus" do
       // due to performance: move this code to cot::simplify
       // if sysorder(-x,x)=TRUE then return(-cot(-x)) else break end_if

       // handle cot(y + integer*PI) -> cot(y)
       // We could also do cot(y + PI/2) -> -tan(y).
       // At the moment, we decide not to do it.
       if has(x, PI) then
          for y in x do
              f:= y/PI;
              if testtype(f, DOM_INT) then
                 return(cot(x-y))
              end_if;
          end_for:
       end_if:
       break;
    of "arcsin" do return(sqrt(1-op(x)^2) / op(x));
    of "arccos" do return(op(x) / sqrt(1-op(x)^2));
    of "arctan" do return(1/op(x));
    of "arccot" do return(op(x));
    end_case;

    procname(x)
end_proc):

cot(  PI/2):= 0:
cot(  PI/3):= (3^(1/2))/3:
cot(  PI/4):= 1:
cot(  PI/5):= 1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2):
cot(2*PI/5):= 1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2):
cot(  PI/6):= (3^(1/2)):
cot(  PI/8):= 2^(1/2)+1:
cot(3*PI/8):= 2^(1/2)-1:
cot(  PI/10):= (5+2*5^(1/2))^(1/2):
cot(3*PI/10):= (5-2*5^(1/2))^(1/2):
cot(  PI/12):= 2+3^(1/2):
cot(5*PI/12):= 2-3^(1/2):
cot(I):= - coth(1)*I:


cot:= funcenv(cot):
cot:= slot(cot, "type", "cot"):
cot:= slot(cot, "print", "cot"):
cot:= slot(cot, "info", "cot -- the cotangent"):

cot:= slot(cot, "inverse", "arccot"):

cot:= slot(cot, "float", proc(e)
begin
   // cot(e) = cos(e)/sin(e) 

   slot(sin,"float")(e);
   if iszero(%)
      then error("singularity")
      else slot(cos,"float")(e)/%
   end_if
end_proc):

cot:= slot(cot, "diff", proc() begin 
    op(args(1),1);
    - diff(%, args(2..args(0))) / sin(%)^2
 end_proc ):

cot:= slot(cot, "expand", proc() begin expand(cos(args(1))/sin(args(1))) end_proc  ):

cot:= slot( cot, "conjugate",
    loadproc(slot(cot,"conjugate"),pathname("STDLIB","CONJ"),"cot")
):

cot:= slot(cot,"rectform",
    loadproc(slot(cot,"rectform"),pathname("STDLIB","RECTFORM"),"cot")
):

cot:= slot(cot,"simplify",
    loadproc(slot(cot,"simplify"),pathname("STDLIB","SIMPLIFY"),"cot")
):

cot:=slot(cot,"series",proc(f,x,n,dir)
begin
   f:=Series::series(tan(f),x,n,dir);
   Series::Puiseux::_invert(f)
end_proc):

cot:= slot(cot,"Re",
    loadproc(slot(cot,"Re"),pathname("STDLIB","RE"),"cot")
):

cot:= slot(cot,"Im",
    loadproc(slot(cot,"Im"),pathname("STDLIB","IM"),"cot")
):

// end of file 
