
// ok, 27/01/94 

/*++
coth -- the hyperbolic cotangens

coth(x)

x - expression
++*/

coth:= proc(x)
  option noDebug;
  local f;
  name coth;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::coth <> FAIL then return(x::dom::coth(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  elif domtype(x) = DOM_SET then
     return(map(x,coth))
  elif not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
        if x = 0 then
           error("singularity")
        elif x < 0 then
          return(-coth(-x))
        end_if;
      break;
    
    of DOM_FLOAT do
	return(slot(coth,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(coth,"float")(x))
//      elif op(x, 1) = 0 then
//           return(-I*cot(-I*x))
//      elif op(x, 1) < 0 and op(x, 2) < 0 then
//          return(-coth(-x))
        end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
             if f < 0 then return(-coth(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then
             error("singularity");
        elif type(2*f)=DOM_INT then
             return(0);
        end_if;
	break;

    of "arccoth" do return(op(x));
    of "arcsinh" do return(sqrt(op(x)^2+1)/op(x));
    of "arccsch" do return(sqrt(op(x)^2+1));
    of "arccosh" do return(op(x)/sqrt(op(x)^2-1));
    of "arcsech" do return(1/sqrt(1-op(x)^2));
    of "arctanh" do return(1/op(x));

    end_case;

    procname(x)
end_proc:

coth(  I*PI/2):= 0:
coth(  I*PI/3):= I*(3^(1/2))/3:
coth(  I*PI/4):= I:
coth(  I*PI/5):= I*1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2):
coth(2*I*PI/5):= I*1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2):
coth(  I*PI/6):= I*(3^(1/2)):
coth(  I*PI/8):= I*(2^(1/2)+1):
coth(3*I*PI/8):= I*(2^(1/2)-1):
coth(  I*PI/10):= I*(5+2*5^(1/2))^(1/2):
coth(3*I*PI/10):= I*(5-2*5^(1/2))^(1/2):
coth(  I*PI/12):= I*(2+3^(1/2)):
coth(5*I*PI/12):= I*(2-3^(1/2)):

coth( infinity):= 1:
coth(-infinity):=-1:

coth:= funcenv(coth):
coth:= slot(coth, "type", "coth"):
coth:= slot(coth, "print", "coth"):
coth:= slot( coth, "info", "coth -- the hyperbolic cotangent"):

coth:= slot(coth, "inverse", "arccoth"):

coth:= slot(coth, "realDiscont", {0}):
coth:= slot(coth, "complexDiscont",
            loadproc(coth::complexDiscont, pathname("STDLIB","DISCONT"),
                     "coth")):
coth:= slot(coth, "undefined", coth::complexDiscont):

coth:= slot(coth, "diff", proc() begin 
    op(args(1),1);
    diff(%, args(2..args(0))) * (1-coth(%)^2)
 end_proc ):

coth:= slot(coth, "float",proc() begin 
    // coth(e) = 1/tanh(e) 
    if args(1) = 0 or iszero(args(1)) 
       then error("singularity")
       else 1/slot(tanh, "float")(args(1))
    end_if
 end_proc ):

coth:= slot(coth, "expand", proc() begin expand(cosh(args(1))/sinh(args(1))) end_proc  ):

coth:= slot( coth, "conjugate",
    loadproc(slot(coth,"conjugate"),pathname("STDLIB","CONJ"),"coth")
):

coth:= slot(coth,"rectform",
    loadproc(slot(coth,"rectform"),pathname("STDLIB","RECTFORM"),"coth")
):

coth:=slot(coth,"series",proc(f,x,n,dir)
begin
  f:=Series::series(tanh(f),x,n,dir);
  Series::Puiseux::_invert(f)
end_proc):

coth:= slot(coth,"simplify",
    loadproc(slot(coth,"simplify"),pathname("STDLIB","SIMPLIFY"),"coth")
):

coth:= slot(coth,"Re",
    loadproc(slot(coth,"Re"),pathname("STDLIB","RE"),"coth")
):

coth:= slot(coth,"Im",
    loadproc(slot(coth,"Im"),pathname("STDLIB","IM"),"coth")
):

// end of file 
