// $Date: 2000/08/15 19:42:45 $ $Author: walter $ $Revision: 1.19 $ 

// zimmerma, 01/08/95 

/*++
        dirac -- the Dirac delta function

        dirac(x)
        dirac(x, n)   = n-th derivative of dirac(x)

        x - expression

	dirac(x) := diff(heaviside(x),x)
++*/

dirac := proc(x)
  local n, X, a, b;
  option noDebug;
  begin
    if args(0) = 0 then error("no arguments given")
    elif x::dom::dirac <> FAIL then return(x::dom::dirac(args()))
    elif args(0) > 2 then error("expecting at most two arguments")
    end_if;

    n:= 0:
    if args(0) = 2 then
      n:= args(2);
      if n = 0 then return(dirac(x)); end_if;
      if testtype(n, Type::Numeric) then
        if type(n) <> DOM_INT or n < 0 then
          error("second argument must be a nonnegative integer");
        end_if;
      end_if;
    end_if;

    case type(x)
      of DOM_SET do
      of "_union" do
         if args(0) = 1
            then return(map(x, dirac))   
            else return(map(x, dirac, n))
          end_if;
      of "_intersect" do
      of "_minus" do 
         return(procname(args()));
    end_case;

    if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
    end_if;

    if (is(x<0) or 
        is(x>0) or
        (is(x, Type::Real) and is(x <> 0) )
       ) = TRUE then 
      return(0);
    end_if;
    if domtype(x)=DOM_COMPLEX then 
      return(undefined);
    end_if;
    // Dirac(a*X + b, n) = sign(a)/a^(n+1) * Dirac(X + b/a)
    // Identify symbolic X and numerical a, b
    // Use numeric::indets rather than indets, because
    // indets(2*x[1]+3) -> {x}, but one needs
    // numeric::indets(2*x[1]+3) -> {x[1]}
    // We can do this simplification only if there is
    // exactly one indeterminate X. Otherwise we do 
    // not know, which indeterminate is to represent
    // the distribution variable:
    X:= numeric::indets(float(x)):
    if nops(X)=1 then
       X:= op(X);
       a:= diff(x, X):
       // Extract real numerical coefficients only.
       // Well, not has(a, I) is a bit crude ..
       if (not has(a, X)) and (not has(a, I)) then
          b:= eval(subs(x, X=0));
          // x = a * X + b;
          if n = 0 then 
             return(sign(a)/a*procname(X + b/a))
          else 
             return(sign(a)/a^(n+1)*procname(X + b/a, n))
          end_if:
       end_if;
    end_if;
    procname(args()) 
  end_proc:

dirac:= funcenv(dirac):
dirac:= slot(dirac,"type","dirac"):
dirac:= slot(dirac,"info", "dirac -- the Dirac function dirac(x) = diff(heaviside(x),x)"):
dirac:= slot(dirac,"print","dirac"):

dirac:= slot(dirac,"diff", proc(f, x)
local y, n;
begin
   y:= op(f, 1); // f=dirac(y, n)
   if nops(f)=1 then
      return(hold(dirac)(op(f,1), 1)*diff(y, x));
   else
      n:= op(f, 2);
      if has(n, x) then return(hold(diff)(f, x)) end_if;
      return(hold(dirac)(y, n+1)*diff(y,x))
   end_if;
end_proc):

dirac:= slot(dirac,"int",proc(f,x) local t;
                         begin
                           t:=op(f);
                           //if op(f)=x then heaviside(x) else FAIL end_if
                           if not has(t-x,x) then heaviside(t) else FAIL end_if
                         end_proc):

dirac:= slot(dirac,"series",
	loadproc(slot(dirac,"series"),pathname("SERIES"),"dirac")):

dirac:= slot(dirac,"sign",proc(x)
                           begin
                           case is(op(x,1)=0)
                              of TRUE do
                                 return(1)
                              of FALSE do
                                 return(0)
                              of UNKNOWN do
                                 return(hold(sign)(x))
                           end_case;
                           error("Unexpected result returned by 'is'")
                           end_proc):

// end of file 
