// ok, 25/01/94 

/*++
The Error function

erf(x)

x - an expression
++*/

erf :=
proc(x) 
  option noDebug;
  local f;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::erf <> FAIL then return(x::dom::erf(args()))
  elif args(0) <> 1 then error("1 argument expected");
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, erf))
  end_case;
  
  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)

    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(-erf(-x)) end_if;
      break;

    of DOM_FLOAT do
      return(slot(erf,"float")(x));
    of DOM_COMPLEX do
      if type(op(x,1)) = DOM_FLOAT or type(op(x,2)) = DOM_FLOAT then
        return(slot(erf,"float")(x));
      end_if;
      break;
   
    of "_mult" do
      f:= op(x, nops(x));
      if testtype(f, Type::Real) then
        if f < 0 then
          return(-erf(-x))
        end_if;
        if domtype(f) = DOM_FLOAT then
          break
        end_if;
      else
        break
      end_if;

  end_case;
  procname(x)
end_proc:

erf(0) := 0:
erf(infinity):=1:
erf(-infinity):=-1:


erf:= funcenv(erf):
erf:= slot(erf,"print","erf"):
erf:= slot(erf,"type","erf"):
erf:= slot(erf,"info","erf -- the error function [try ?erf for details]"):

erf:= slot(erf,"conjugate",
    loadproc(slot(erf,"conjugate"),pathname("STDLIB","CONJ"),"erf")
):

erf:= slot(erf,"diff",proc() begin 
    op(args(1),1);
    2/PI^(1/2) * exp(-%^2) * diff(%, args(2..args(0)))
 end_proc ):

erf:= slot(erf,"float",
    proc(x) 
    local fx, absfx;
    save DIGITS;
    begin 
      fx:= float(x):
      case domtype(fx)
      of DOM_FLOAT do
      of DOM_COMPLEX do
         // erf(x) = 2/sqrt(PI) * (x - x^3/3 + x^5/10 - ..)
         absfx:= specfunc::abs(fx):
         if absfx < 10.0^(-DIGITS/2) then
            return(float(2)/float(PI)^(1/2)*fx)
         end_if;
         if absfx < 10.0^(-DIGITS/4) then
            return(float(2)/float(PI)^(1/2)*(fx - fx^3/3))
         end_if;
         if absfx < 10.0^(-DIGITS/6) then
            return(float(2)/float(PI)^(1/2)*(fx - fx^3/3 + fx^5/10))
         end_if;
         // Beware: for small |x| the following erf(x) = 1 - erfc(x)
         // is numerically unstable.
         // For this reason the above truncated series were
         // used to stabilize floating point evaluation for very small x.
         // Now we may assume |x| >= 10^(-DIGITS/6).
         // The formula erf(x) = 1 - erfc(x) amplifies the
         // relative error
         //  Delta(erf(x))/erf(x)) = Delta(erfc(x))/erf(x) = macheps* erfc(x)/erf(x))
         // by 1/erf(x) = 10^(DIGITS/6) in the worst case x = 10^(-DIGITS/6).
         // So increase DIGITS -> DIGITS*(1+1/6):
         if specfunc::abs(fx) < 1/2 then
            DIGITS:= round(DIGITS*7/6):
         end_if:
      end_case:
      float(1) - erfc::float(x)
    end_proc ):

erf:= slot(erf,"sign",
    loadproc(slot(erf,"sign"),pathname("SPECFUNC","SIGN"),"erf")
):

erf:=slot(erf,"series",
    loadproc(slot(erf,"series"),pathname("SERIES"),"erf")
):

// end of file 
