// $Date: 2000/05/21 15:02:48 $ $Author: bij $ $Revision: 1.25 $ 

// kg, 13/01/94 

/*++
exp -- the exponential function

exp(x)

x - expression
++*/

exp:= proc(x)
  option noDebug;
  name exp;
  local y, lny, c;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::exp <> FAIL then return(x::dom::exp(args()))
  elif args(0) <> 1 then error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, exp))
  end_case;

  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_FLOAT do
      return(slot(exp,"float")(x));

    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
         domtype(op(x,2)) = DOM_FLOAT 
      then return(slot(exp,"float")(x))
      end_if;
      break;

    of "_mult" do
      // handle case exp(y*PI*I) for rational y  
      y:=x/PI/I;
      if testtype(y, DOM_INT) then
        return((-1)^y)
      end_if;
      if testtype(y, DOM_RAT) then
        if y>=1 or y<0 then
          // reduce to case exp(y*PI*I) with 0<=y<1
          /* recursive call is necessary to use the
            predefined values below */
          y:=2*frac(y/2); // this is >=0 and < 2, and its difference to the
                         // previous y is a multiple of 2
          if %<1 then
            return(exp(PI*I*%))
          else
            // exp(y*PI*I) = exp(-PI*I)*exp((y-1)*PI*I)
            //             =   (-1)    *exp((y-1)*PI*I)
            return(-exp(PI*I*(%-1)))
          end_if;
        else
          // do not determine the function value except for the
          // predefined cases below
          break
        end_if
      end_if;
     
      // exp(ln(y)*Constant) = y^Constant 
      if has(x, hold(ln)) then
         // split x = ln(y)*Constant
         lny:= select(x, testtype, "ln");
         // select can return 1 --> test again 
         if testtype(lny, "ln") then
            // x = ln(y)* Constant ??
            c:= x/lny;
            if testtype(c, Type::Constant) then
               return(op(lny,1)^c)
            end_if;
         end_if;
      end_if:
      break;

    of "_plus" do
      // handle exp(y + integer*PI*I)
      if has(x, PI) and has(x, I) then
         for y in x do
             if testtype(y/2/PI/I, DOM_INT) then
                return(exp(x-y))
             end_if;
             if testtype(y/PI/I, DOM_INT) then
                return(-exp(x-y))
             end_if;
         end_for:
      end_if:
      break;

    of "ln" do
      return(op(x,1));
    of "lambertV" do
    of "lambertW" do
      return(op(x,1)/x)
  end_case;

  procname(x)
end_proc:

// standard simplifications 

exp(infinity):= infinity:
exp(-infinity):= 0:
exp(0):= 1:
exp(I*PI):= -1:

exp(I*PI  /2):=  I:

exp(I*PI  /3):=  1/2 + I/2*3^(1/2):
exp(I*PI*2/3):= -1/2 + I/2*3^(1/2):

exp(I*PI  /4):= (1/2 + 1/2*I)*2^(1/2):
exp(I*PI*3/4):=-(1/2 - 1/2*I)*2^(1/2):

exp(I*PI  /5):=  (5^(1/2)+1)/4 + (5-5^(1/2))^(1/2)*2^(1/2)/4*I:
exp(I*PI*2/5):=  (5^(1/2)-1)/4 + (5+5^(1/2))^(1/2)*2^(1/2)/4*I:
exp(I*PI*3/5):= (-5^(1/2)+1)/4 + (5+5^(1/2))^(1/2)*2^(1/2)/4*I:
exp(I*PI*4/5):= -(5^(1/2)+1)/4 + (5-5^(1/2))^(1/2)*2^(1/2)/4*I:

exp(I*PI   /6):=  3^(1/2)/2 + I/2:
exp(I*PI* 5/6):= -3^(1/2)/2 + I/2:

exp(I*PI   /8):= (2+2^(1/2))^(1/2)/2 + (2-2^(1/2))^(1/2)/2*I:
exp(I*PI* 3/8):= (2-2^(1/2))^(1/2)/2 + (2+2^(1/2))^(1/2)/2*I:
exp(I*PI* 5/8):=-(2-2^(1/2))^(1/2)/2 + (2+2^(1/2))^(1/2)/2*I:
exp(I*PI* 7/8):=-(2+2^(1/2))^(1/2)/2 + (2-2^(1/2))^(1/2)/2*I:

exp(I*PI   /10):=  (5+5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)-1)/4*I:
exp(I*PI* 3/10):=  (5-5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)+1)/4*I:
exp(I*PI* 7/10):= -(5-5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)+1)/4*I:
exp(I*PI* 9/10):= -(5+5^(1/2))^(1/2)*2^(1/2)/4 + (5^(1/2)-1)/4*I:

exp(I*PI   /12):=  (1-I)/4*2^(1/2) + (1+I)/4*6^(1/2):
exp(I*PI* 5/12):= -(1-I)/4*2^(1/2) + (1+I)/4*6^(1/2):
exp(I*PI* 7/12):=  (1+I)/4*2^(1/2) - (1-I)/4*6^(1/2):
exp(I*PI*11/12):= -(1+I)/4*2^(1/2) - (1-I)/4*6^(1/2):

// function attributes 

exp:= funcenv(exp):
exp:= subsop(exp, 2=op(specfunc::exp, 2)):
exp:= slot(exp,"print","exp"):
exp:= slot(exp,"info","exp -- the exponential function" ):
exp:= slot(exp,"type","exp"):
exp:= slot(exp, "float",
           proc(x) local err, tmp;
           begin
            err:= traperror( (tmp:= specfunc::exp(x)) );
            if err = 0
            then return(tmp)
            elif err = 20 // Error: Overflow/underflow
            then x:= float(x);
                 if domtype(op(x,1)) = DOM_FLOAT and
                    op(x,1) < -5.81*10^(6) // PARI underflow for Re(x)<-5.81*10^6;
                 then return(float(0))
                 end_if
            end_if;
            // error erneut ausloesen
            lasterror();
          end_proc):

exp:= slot(exp,"inverse", "ln"):

exp:= slot(exp,"abs", proc(x) begin exp(Re(x)) end_proc):

exp:= slot(exp,"sign",
    loadproc(slot(exp,"sign"),pathname("SPECFUNC","SIGN"),"exp")
):

exp:= slot(exp,"expand",
    loadproc(slot(exp,"expand"),pathname("STDLIB","EXPAND"),"exp")
):

exp:= slot(exp,"conjugate",
    loadproc(slot(exp,"conjugate"),pathname("STDLIB","CONJ"),"exp")
):

exp:= slot(exp,"rectform",
    loadproc(slot(exp,"rectform"),pathname("STDLIB","RECTFORM"),"exp")
):

exp:=slot(exp,"simplify",
    loadproc(slot(exp,"simplify"),pathname("STDLIB","SIMPLIFY"), "exp")
):

exp:= slot(exp,"series",
    loadproc(slot(exp,"series"),pathname("SERIES"),"exp")
):

exp:= slot(exp,"Re",
    loadproc(slot(exp,"Re"),pathname("STDLIB","RE"),"exp")
):

exp:= slot(exp,"Im",
    loadproc(slot(exp,"Im"),pathname("STDLIB","IM"),"exp")
):

exp := slot(exp,"diff",
    loadproc(slot(exp,"diff"),pathname("STDLIB","DIFF"),"exp")
):

// end of file 
