
// ok, 20/04/94 

/*++
The Gamma function

gamma(x)

x - an expression
++*/

gamma :=
proc(x)
  option noDebug;
begin
  if args(0) = 0 then
    error("no arguments given")
  elif x::dom::gamma <> FAIL then
    return(x::dom::gamma(args()))
  elif args(0) <> 1 then
    error("1 argument expected");
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, gamma))
  end_case;
  
  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case domtype(x)

    of DOM_FLOAT do

      if x <= 0 and iszero(frac(x)) then
        error("singularity");
      end_if;
      return( slot(gamma, "float")(x) );

    of DOM_INT do

      if x <= 0 then
        error("singularity");
      elif x < 1000 then
        // gamma(x) = (x-1)! ,                 
        // but compute (x-1)! only if x < 1000 
		return( fact(x-1) )
      end_if;
      break;

    of DOM_COMPLEX do
 
      if domtype(op(x, 1)) = DOM_FLOAT or
        domtype(op(x, 2)) = DOM_FLOAT then
        return( slot(gamma, "float")(x) );
      end_if;
      break;
	   
    of DOM_RAT do

      if 1 < x and x <= 500 then
        // compute (x-1)*gamma(x-1) only if x <= 500 
        return( (x-1) * gamma(x-1) )
      elif x < 1/2 and
        (domtype(4*x) = DOM_INT or domtype(6*x) = DOM_INT) then
        return( level( PI * csc(PI*x) / gamma(1-x) ) )
      end_if;
      break;

    otherwise

      if is(x, Type::PosInt) = TRUE then
        return( (x-1)! )
      end_if;
                 
  end_case;

  procname(x)
end_proc:

gamma(1/2) := PI^(1/2):
gamma(infinity) := infinity:

gamma:= funcenv(gamma):
gamma:= subsop(gamma, 2=op(specfunc::gamma, 2)):
gamma:= slot(gamma,"print","gamma"):
gamma:= slot(gamma,"info","gamma -- the Gamma function [try ?gamma for details]"):
gamma:= slot(gamma,"type","gamma"):
gamma:= slot(gamma, "float", specfunc::gamma):

gamma:= slot(gamma, "complexDiscont",
loadproc(gamma::complexDiscont, pathname("STDLIB","DISCONT"),
                     "gamma")):
gamma:= slot(gamma, "realDiscont", gamma::complexDiscont):
gamma:= slot(gamma, "undefined", gamma::complexDiscont):

gamma:= slot(gamma,"diff",proc() begin 
    op(args(1),1);
    diff(%, args(2..args(0))) * psi(%) * gamma(%)
 end_proc ):

gamma:= slot(gamma, "conjugate",
    loadproc(slot(gamma,"conjugate"),pathname("STDLIB","CONJ"),"gamma")
):

gamma:= slot(gamma, "expand", 
    loadproc(slot(gamma,"expand"),pathname("STDLIB","EXPAND"),"gamma")
):

gamma:= slot(gamma,"rectform",
    loadproc(slot(gamma,"rectform"),pathname("STDLIB","RECTFORM"),"gamma")
):

gamma:= slot(gamma, "series",
    loadproc(slot(gamma,"series"),pathname("SERIES"),"gamma")
):

// end of file 
