// $Date: 2000/07/25 19:26:16 $ $Author: walter $ $Revision: 1.14 $ 

// zimmerma, 01/08/95 

/*++
	heaviside -- the Heaviside function

	heaviside(x)

	x - expression

	heaviside(x) :=      1            if x>0
                        0            if x<0
                        heaviside(0) if x=0
                        undefined    if x is a non-real complex number
                        heaviside(x) otherwise.
++*/

heaviside := proc(x)
  option noDebug;
  begin
    if args(0) = 0 then error("no arguments given")
    elif x::dom::heaviside <> FAIL then return(x::dom::heaviside(args()))
    elif args(0) <> 1 then
        error("expecting one argument")
    end_if;

    case type(x)
      of DOM_SET do // fall through
      of "_union" do
         return(map(x, heaviside))
      of "_intersect" do
      of "_minus" do
         return(procname(x))
    end_case;    

    if not testtype(x,Type::Arithmetical) then
      error("argument must be of 'Type::Arithmetical'")
    elif is(x>0)=TRUE then 
      1
    elif is(x<0)=TRUE then 
      0
    elif domtype(x)=DOM_COMPLEX then 
      undefined
    else 
      procname(x)
    end_if
  end_proc:

heaviside:= funcenv(heaviside):
heaviside:= slot(heaviside,"type","heaviside"):
heaviside:= slot(heaviside,"info",
  "heaviside -- the Heaviside step function"
):
heaviside:= slot(heaviside, "print", "heaviside"):

heaviside:= slot(heaviside,"series",
	loadproc(slot(heaviside,"series"),pathname("SERIES"),"heaviside")):

heaviside:= slot(heaviside, "diff", proc(f,x) begin
   eval(diff(subsop(f,0=dirac)*diff(op(f),x),args(3..args(0))))
end_proc):


// end of file 
