// $Date: 2000/05/21 15:02:54 $ $Author: bij $ $Revision: 1.12 $ 

// stefanw, 14/08/98 

/*++
lambertW - the main branch of the functional inverse of
           x -> x*exp(x)

lambertW(x)

x - expression
++*/

lambertW:=
proc(x)
  option noDebug;
  local fx,i,f;
begin
  if args(0) <> 1 then 
     error("wrong no of args")
  elif x::dom::lambertW <> FAIL then 
     return(x::dom::lambertW(args()))
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, lambertW))
  end_case;
  
  if not testtype(x,Type::Arithmetical) then
     error("argument must be of 'Type::Arithmetical'")
  end_if;

  fx:= float(x):
  if domtype(fx) <> DOM_FLOAT or
     fx < float(-1/E) 
  then return(procname(args()))
  end_if:

  case type(x)
    of DOM_FLOAT do
       return(slot(lambertW,"float")(x));
    of "_mult" do
      for i from 1 to nops(x) do
        if op(x,i)=exp((f:=_mult(op(x,1..i-1),op(x,i+1..nops(x))))) then
          return(f)
        end_if;
      end_for;
      break
  end_case;
	
  procname(x)
end_proc:

lambertW(0):=0:
lambertW(exp(1)):=1:
lambertW(-exp(-1)):=-1:
lambertW(infinity):= infinity:

lambertW:= funcenv(lambertW):
lambertW:= subsop(lambertW, 2=op(specfunc::lambertW, 2)):
lambertW:= slot(lambertW, "print", "lambertW"):
lambertW:= slot(lambertW, "info",
                    "lambertW -- main branch of the inverse of x*exp(x)" ):
lambertW:= slot(lambertW, "type", "lambertW"):
lambertW:= slot(lambertW, "float", specfunc::lambertW):

lambertW:= slot(lambertW, "inverse", "id*exp"):


lambertW:= slot(lambertW, "diff",
                    proc(f,x)
                    begin
                      f*diff(op(f,1),x)/(op(f,1)*(1+f))
                    end_proc):


// end of file 
