// $Date: 2000/10/31 14:28:20 $ $Author: stefanw $ $Revision: 1.22 $ 

// kg, 13/01/94 

/*++
ln -- the natural logarithm

ln(x)

x - expression
++*/

ln:= proc(x)
  option noDebug;
  name ln;
  local k;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::ln <> FAIL then return(x::dom::ln(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  if iszero(x) then error("singularity") end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, ln))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)

    of DOM_FLOAT do
	return(slot(ln,"float")(x))

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT 
	then
	    return(slot(ln,"float")(x))
	end_if;
	break

    of DOM_INT do
        if x < 0 then return(ln(-x) + I*PI) end_if;
	break

    of DOM_RAT do
        if x < 0 then return(ln(-x) + I*PI) end_if;
        if op(x,1)=1 then return(-ln(op(x,2))) end_if;
	break

    of "exp" do
	if testtype( op(x), Type::Numeric ) then
	    k:=round( -Im(op(x))/2/PI );
	    return( op(x)+2*k*PI*I )
	end_if;
	break

    of "lambertV" do
    of "lambertW" do
	return(ln(op(x,1))-x)

  end_case;

  procname(x)
end_proc:

ln(1):= 0:
ln(E):= 1:
ln(1/E):= -1:
ln(-1):= I*PI:
ln(I):= I*PI/2:
ln(-I):=-I*PI/2:
ln(infinity):= infinity:
ln(-infinity):= I*PI + infinity:

ln:= funcenv(ln):
ln:= subsop(ln, 2=op(specfunc::ln, 2)):
ln:= slot(ln,"print","ln"):
ln:= slot(ln,"type","ln"):
ln:= slot(ln,"info","ln -- the natural logarithm"):
ln:= slot(ln, "float", specfunc::ln):

ln:= slot(ln,"inverse", "exp"):

ln:= slot(ln, "undefined", {0}):
ln:= slot(ln, "realDiscont", {0}):
ln:= slot(ln, "complexDiscont",
    loadproc(slot(ln,"complexDiscont"),pathname("STDLIB","DISCONT"),"ln")
):

ln:= slot(ln,"expand",
    loadproc(slot(ln,"expand"),pathname("STDLIB","EXPAND"),"ln")
):

ln:= slot(ln,"simplify",
    loadproc(slot(ln,"simplify"),pathname("STDLIB","SIMPLIFY"),"ln")
):

ln:= slot(ln,"sign",
    loadproc(slot(ln,"sign"),pathname("SPECFUNC","SIGN"),"ln")
):

ln:= slot(ln,"rectform",
    loadproc(slot(ln,"rectform"),pathname("STDLIB","RECTFORM"),"ln")
):

ln:= slot(ln,"series",
    loadproc(slot(ln,"series"),pathname("SERIES"),"ln")
):

ln:= slot(ln,"Re",
    loadproc(slot(ln,"Re"),pathname("STDLIB","RE"),"ln")
):

ln:= slot(ln,"Im",
    loadproc(slot(ln,"Im"),pathname("STDLIB","IM"),"ln")
):

ln:= slot(ln,"diff",
    loadproc(slot(ln,"diff"),pathname("STDLIB","DIFF"),"ln")
):

// end of file 
