
// ok, 14/02/00 

/*++
log -- the general logarithm

log(b, x)

b - a real number (b > 0, b <> 1) or an identifier
x - an expression
++*/

log:= proc(b, x)
  option noDebug;
  name log;
  local k, dk, bk, mmin, mmax;
begin
  if args(0) = 0 then error("no arguments given")
  elif args(0) <> 2 then error("wrong no of args") 
  elif x::dom::log <> FAIL then return(x::dom::log(args()))
  end_if;

// Computations with symbols should be allowed!

  if testtype(b, Type::Real) then
     if not (b > 0 and b <> 1) then
        error("base must be greater than zero and not equal one")
     end_if;
     if domtype(b) = DOM_FLOAT then 
        x:= float(x):
     end_if:
  elif b = exp(1) then
     return( ln(x) )
  elif domtype(b) <> DOM_IDENT then
     error("base must be an identifier or of Type::Positive")
  end_if;

  if iszero(x) then error("singularity") end_if;

  case type(x)
    of DOM_SET do
      return( { log(b, k) $ k in x } )
    of "_union" do
      return( _union( log(b, k) $ k in x ) )
  end_case;

  case type(x)

    of DOM_FLOAT do
        if domtype(b) <> DOM_IDENT then
	   return(slot(log,"float")(b, x))
	end_if;
	break;

    of DOM_COMPLEX do
	if domtype(b) <> DOM_IDENT and
	   (domtype(op(x,1)) = DOM_FLOAT or
	    domtype(op(x,2)) = DOM_FLOAT) 
	then
	    return(slot(log,"float")(b, x))
	elif x = I or x = -I then
	    // log(b, I)  =  I*PI/2/ln(b)
            // log(b, -I) = -I*PI/2/ln(b)
	    return( x*PI/2/ln(b) )
	end_if;
	break

    of DOM_INT do
    of DOM_RAT do
        if x < 0 then return(log(b, -x) + I*PI/ln(b)) end_if;
	if x = 1 then return( 0 ) end_if; // log(b, 1) = 0
        // log(b, 1/n) = -log(b, n)
        if op(x,1)=1 then return(-log(b, op(x,2))) end_if;
        // identify integer values k = log(b, b^k):
	if domtype(b) <> DOM_IDENT then   

           // The following k is the candidate for x=b^k
	   k := round(log::float(b, x)); 
           bk:= b^k:
           if bk = x then return(k) end_if;

           // The last 3 lines of code should have identified
           // all integers x of the form b^k, k integer.
           // Unfortunately, k may be affected by round-off.
           // Without roundoff we must have (assuming b>1):
           //   b^(k-1) < x < b^(k+1)
           // If x is in this range and b^k <> x, then x is
           // definitely not of the form x = b^integer:

           mmin:= min(b, 1/b):
           mmax:= max(b, 1/b):
           while not (mmin < x/bk and x/bk < mmax)  do
              // k is marred by round-off. 
              // The correction dk should be ok:
              dk:= round(log::float(b, x/bk));
              if dk=0 then break end_if; // give up, cannot recover
                                         // from round-off problems
              k:= k + dk; 
              bk:= bk*b^dk:
              if bk = x then return(k) end_if;
           end_while;

           // If we arrive here, then there is no integer k such that x = b^k.

	end_if;
	break


    of "_power" do
      // log(b, b^y) = y + k*2*PI*I/ln(b) 
      if op(x, 1) = b then
      // The following lines are correct and operational code
      // for complex exponents in x = b^exponent. However,
      // if the exponent is not real, then a symbolic
      // round( number*ln(b)/2/PI) is returned, which looks
      // odd. So do not simplify for non-real exponents:
      //if testtype( op(x,2), Type::Numeric ) then
            // make sure that Im(log(b,x)) is in the range
            //     (-PI/ln(b), PI/ln(b)]   for b>1
            //     [PI/ln(b), -PI/ln(b))   for b<1
          //k:=  round(-Im(op(x,2))*ln(b)/2/PI );
          //return( op(x,2)+2*k*PI*I/ln(b) )
      //end_if;
      // Do simplify log(b, b^real_exponent):
        if testtype( op(x,2), Type::Real ) then
            return( op(x,2) )
        end_if;
      end_if;
      break;

  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  if   b = x   then return( 1 )  // log(b, b)   = 1
  elif b = 1/x then return( -1 ) // log(b, 1/b) = -1
  end_if;

  procname(b, x)

end_proc:

log:= funcenv(log):
log:= slot(log, "print", "log"):
log:= slot(log, "type",  "log"):
log:= slot(log, "info",  "log -- the general logarithm"):

log:= slot(log, "float", proc(b, x)
local fb, fx;
begin
    fb:= float(b):
    fx:= float(x):
    // If at least one of the arguments is numeric, then
    //  log(b,x) -> ln(x)/ln(b) = float or ln(x)/float or float/ln(b)
    // If both b and x and symbolic, then return log(b,x)
    case domtype(fb) 
    of DOM_FLOAT do
    of DOM_COMPLEX do
       return(ln::float(fx)/ln::float(fb));
    otherwise
       case domtype(fx) 
       of DOM_FLOAT do
       of DOM_COMPLEX do
          return(ln::float(fx)/ln::float(fb));
       otherwise // both b and x are symbolic
          return(log(fb, fx))
       end_case;
    end_case
end_proc):

log:= slot(log, "series", (b,x,t,n,dir) -> series(ln(x)/ln(b),t,n,dir) ):

// The following discont slots of ln do not work: functions
// with 2 arguments have to be handled directly in discont
//
// log:= slot(log, "complexDiscont", Dom::Interval(-infinity, [0])):
// log:= slot(log, "realDiscont", {0}):

log:= slot(log, "expand",
    loadproc(slot(log,"expand"),pathname("STDLIB","EXPAND"),"log")
):

log:= slot(log,"diff",
    loadproc(slot(log,"diff"),pathname("STDLIB","DIFF"),"log")
):

log:= slot(log, "simplify",
    loadproc(slot(log,"simplify"),pathname("STDLIB","SIMPLIFY"),"log")
):

log:= slot(log, "sign", 
     loadproc(slot(log,"sign"),pathname("SPECFUNC","SIGN"),"log")
):

log:= slot(log, "rectform",
    loadproc(slot(log,"rectform"),pathname("STDLIB","RECTFORM"),"log")
):

log:= slot(log,"Re",
    loadproc(slot(log,"Re"),pathname("STDLIB","RE"),"log")
):

log:= slot(log,"Im",
    loadproc(slot(log,"Im"),pathname("STDLIB","IM"),"log")
):

// end of file 
