// ok, 29/10/94 

/*++
The Digamma and Polygamma functions

psi(x)
psi(x, n)

x - an expression
n - an non negative integer
++*/

psi:=
proc(x, n)
  option noDebug;
  name psi;
  local fx, i;
begin
  if args(0) = 0 then
    error("no arguments given")
  elif x::dom::psi <> FAIL then
    return(x::dom::psi(args()))
  end_if;

  if testtype(x, Type::Numeric) then
    fx:= float(x);
    if domtype(fx) = DOM_FLOAT and iszero(frac(fx)) and fx <= 0 
      then error("singularity");
    end_if
  end_if;

  case args(0)
  of 1 do
     case type(x)
       of DOM_FLOAT do
         return( specfunc::psi(x) );
       of DOM_COMPLEX do
         if type(op(x,1)) = DOM_FLOAT or type(op(x,2)) = DOM_FLOAT then
           return( specfunc::psi(x) );
         end_if;
         break;
       of DOM_INT do
         if 1 <= x and x < 1000 then // See apendix 
           return(psi(1) + _plus(1/i $ i=1..x-1) );
         end_if;
         break;
       of DOM_RAT do
         if domtype(2*x) = DOM_INT and specfunc::abs(x) < 1000 then // See apendix 
           if x < 0 
           then return(psi(1/2)+2*_plus(1/(2*i-1)$i=1..(op(-x,1)+1) div 2))
           else return(psi(1/2)+2*_plus(1/(2*i-1)$i=1..op(x,1) div 2))
           end_if
         end_if;
         break
       of DOM_SET do
       of "_union" do
         return(map(x, psi))
       otherwise
         if not testtype(x,Type::Arithmetical) then
           error("argument must be of 'Type::Arithmetical'")
         end_if
     end_case;
     break;
  of 2 do 
     if testtype(n, Type::Numeric) then
       if type(n) <> DOM_INT or n < 0 then
         error("second argument must be a nonnegative integer");
       elif n = 0 then return( psi(x) );
       end_if;
       case type(x)
         of DOM_FLOAT do
           return( specfunc::floatpsi(x, n) );
         of DOM_COMPLEX do
           if type(op(x,1)) = DOM_FLOAT or type(op(x,2)) = DOM_FLOAT then
             return( specfunc::floatpsi(x, n) );
           end_if;
           break;
         of DOM_INT do
           if 1 <= x and x < 1000 then
             return((-1)^n*fact(n)*(-zeta(n+1) + _plus(1/i^(n+1) $ i=1..x-1)));
           end_if;
           break;
         of DOM_RAT do
           if x = 1/2 then
             return( (-1)^(n+1)*fact(n)*(2^(n+1)-1)*zeta(n+1) );
//         elif type(2*x) = DOM_INT and abs(x) < 1000-n then # See apendix #
//           if x < 0 
//              then return( psi(x+1, n)-(-1)^n*fact(n)/x^(n+1) );
//              else return( (-1)^n*fact(n)/(x-1)^(n+1)+psi(x-1, n) );
//           end_if
//         end_if;
           elif domtype(2*x) = DOM_INT and specfunc::abs(x) < 1000 then // See apendix 
             if x < 0 
             then return(psi(1/2,n) + fact(n)*2^(n+1)*
                                 _plus(1/(2*i-1)^(n+1)$i=1..(op(-x,1)+1) div 2))
             else return(psi(1/2,n) + (-1)^n*fact(n)*2^(n+1)*
                                 _plus(1/(2*i-1)^(n+1)$i=1..op(x,1) div 2))
             end_if
           end_if;
           break;
          of DOM_SET do
          of "_union" do
            return(map(x, psi, n))
          otherwise
            if x = infinity then return(0) end_if;
        end_case;
     else // n is not Type::Numeric
          case type(x)
          of DOM_SET do
          of "_union" do
            return(map(x, psi, n))
          end_case;
          if not testtype(n,Type::Arithmetical) or
             not testtype(x,Type::Arithmetical) then
             error("argument must be of 'Type::Arithmetical'");
          end_if;
      end_if;
      break;

    otherwise

      error("expecting two arguments")

  end_case;

  procname(args())

end_proc:

psi(1) := -EULER:
psi(1/2) := -EULER-2*ln(2):
psi(infinity) := infinity:


psi:= funcenv(psi):
psi:= subsop(psi, 2=op(specfunc::psi, 2)):
psi:= slot(psi,"type","psi"):
psi:= slot(psi,"print","psi"):
psi:= slot(psi,"info","psi -- the Digamma and Polygamma functions [try ?psi for options]"):

psi:= slot(psi,"diff",proc() 
    local f, n, x;
    begin 
    f:= op(args(1),1);
    x:= args(2..args(0));
    if nops(args(1)) > 1 then
       n:= op(args(1),2):
       // do not try to diff w.r.t. the 2nd argument n of psi(f,n)
       if has(n, args(2)) 
        then hold(diff)(psi(f,n),x)
        else psi(f, n+1)*diff(f,x)
       end_if;
    else psi(f, 1)*diff(f, x)
    end_if
 end_proc ):

// Appendix:                                        
// Due to the complexity to compute psi(x), where x 
// is an integer, compute psi(x) only if x < 100    
// Also restrict the computation of psi(x,n), where 
// 2*x is an integer. Compute psi(x,n) only if      
// abs(x) < 100-n. This restrict the number of      
// recursive calls of psi and the fractional part   
// of the result.                                   

psi := slot(psi, "float", proc(x, n) begin
    case args(0) 
      of 1 do return(specfunc::psi(x));
      of 2 do return(specfunc::floatpsi(x,n));
    end_case;
    error("wrong no of arguments");
end_proc):

specfunc::floatpsi:= loadproc(
    specfunc::floatpsi,pathname("STDLIB","FLOAT"),"psi"
):

psi:= slot(psi, "expand",
    loadproc(slot(psi,"expand"),pathname("STDLIB","EXPAND"),"psi")
):

psi:=slot(psi,"series",
    loadproc(slot(psi,"series"),pathname("SERIES"),"psi")
):

// end of file 
