/*+++++++++++++++++++    Author: W. Oevel, 4.2.2000

signIm -- indicates the position of a number in the complex plane : 
             signIm(z) = +1  <--> z is in the upper half plane
             signIm(z) = -1  <--> z is in the lower half plane
             signIm(z) =  0  <--> z = 0

Im(z)>0               --> signIm(z) = 1        +  |   +
Im(z)=0 and Re(z)<0   --> signIm(z) = 1           |______
Im(z)=0 and Re(z)>0   --> signIm(z) =-1      -----o
Im(z)<0               --> signIm(z) =-1        -  |   -
z = 0                 --> signIm(z) = 0           |

Call:      signIm(z)
Parameter: z - arithmetical expression representing a complex number
Returns:   +1 or -1 or 0 or an unevaluated call

Following attributes are missing:
 -- rectform
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

signIm:= proc(x)
  option noDebug;
  local Rex, Imx;
  name signIm;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::signIm <> FAIL then return(x::dom::signIm(args()))
  elif args(0) <> 1 then error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, signIm))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
    of DOM_FLOAT do
      if x < 0 then 
           return(1)
      elif x > 0 then
           return(-1)
      else return(0);
      end_if:
    of DOM_COMPLEX do
      Rex:= op(x, 1):
      Imx:= op(x, 2):
      if Imx > 0 then
           return(1)
      elif Imx < 0 then
           return(-1)
      elif Rex <0  then // Im(x)=0 and Re(x)<0 -->  1
           return(1)
      elif Rex >0  then // Im(x)=0 and Re(x)>0 --> -1
           return(-1)
      else return(0)    // z = 0 --> 0
      end_if:
  end_case;
  // Now process symbolic input
  Imx:= Im(x):
  if is(Imx>0)= TRUE or is(x<0)=TRUE then 
     return(1)
  elif is(Imx<0)=TRUE or is(x>0)=TRUE then
     return(-1)
  elif is(x=0)=TRUE then
     return(0)
  else
     return(procname(x));
  end_if:
end_proc:

signIm:= funcenv(signIm):
signIm:= slot(signIm, "print", "signIm"):
signIm:= slot(signIm, "info", "signIm -- sign of the imaginary part of a number" ):
signIm:= slot(signIm, "type", "signIm"):
signIm:= slot(signIm, "float", proc(x) begin signIm(float(x)) end_proc):
signIm:= slot(signIm, "conjugate", signIm):
signIm:= slot(signIm, "Re", signIm):
signIm:= slot(signIm, "Im", 0): 
signIm:= slot(signIm, "diff", 0): // regard signIm(x) as a constant function
signIm:= slot(signIm, "series", // treat signIm(x) as leading order term
        proc(f, x, n) begin return(signIm(f)*series(1,x,n)) end_proc):

// minimal property
signIm:= slot(signIm, "minprop", Type::Interval([-1, 1], Type::Integer)):

//rectform attribute still missing
//signIm:= slot(signIm,"rectform",
//    loadproc(slot(signIm,"rectform"),pathname("STDLIB","RECTFORM"),"signIm")):

// end of file 
