// $Date: 2000/05/21 15:02:59 $ $Author: bij $ $Revision: 1.23 $ 

// kg, 13/01/94 

/*++
sin -- the sine

sin(x)

x - expression
++*/

sin:= proc(x)
  option noDebug;
  local f, y;
  name sin;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::sin <> FAIL then return(x::dom::sin(args()))
  elif args(0) <> 1 then error("wrong no of args")
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, sin))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(-sin(-x)) end_if;
      break;

    of DOM_FLOAT do
      return(slot(sin,"float")(x));
      
    of DOM_COMPLEX do
      if domtype(op(x,1)) = DOM_FLOAT or
        domtype(op(x,2)) = DOM_FLOAT then
        return(slot(sin,"float")(x))
      end_if;
      if op(x,1) = 0 then return(sinh(-x*I)*I) end_if;
      break;

    of "_mult" do
/*--
due to performance reasons, move this code to sin::simplify
--*/
/*
       if nops(x)>nops(x/PI) then // x = multiple of PI
          if is(x/PI, Type::Integer) = TRUE then
               return(0)
          elif is(2*x/PI, Type::Even) = TRUE then
               return(0)
          elif is((f:=2*x/PI), Type::Odd) = TRUE then
               // f = n*k with integer n and assume(k, Type::Odd)
               // i.e., x = n*k*PI/2. We may reduce n modulo 4.
               if type(f)="_mult" then
                  f:= map(f, proc(n) begin 
                             if domtype(n)=DOM_INT 
                                then n mod 4
                                else n
                             end_if
                             end_proc);
               end_if: 
               return((-1)^((f-1)/2))
          end_if;
       end_if;
*/
//--
      f:= op(x, nops(x));
      if testtype(f, Type::Real) then
        if f < 0 then return(-sin(-x)) end_if;
        if domtype(f) = DOM_FLOAT then break end_if;
      else
        break
      end_if;

      if nops(x) = 2 then
        if op(x,1) = PI then
          if f < 1/2 then break end_if;
          if f < 1 then return(sin((1-f)*PI)) end_if;
          if f < 2 then return(-sin((2-f)*PI)) end_if;
          return(sin((f-2*(floor(f) div 2))*PI));
        end_if
      end_if;
      break;
    of "_plus" do
       // due to performance: move this code to sin::simplify
       // if sysorder(-x,x)=TRUE then return(-sin(-x)) else break end_if

       // handle sin(y + integer*PI) -> (-1)^integer*sin(y)
       // We could also do sin(y + PI/2) -> -cos(y).
       // At the moment, we decide not to do it.
       if has(x, PI) then
          for y in x do
              f:= y/PI;
              if testtype(f, DOM_INT) then
                 return((-1)^f*sin(x-y))
              end_if;
          end_for:
       end_if:
       break;
    of "arcsin" do return(op(x));
    of "arccos" do return(sqrt(1-op(x)^2));
    of "arctan" do return(op(x)/sqrt(1+op(x)^2));
 // of "arctan" do return(op(x)/sqrt(1+op(x)*I)/sqrt(1-op(x)*I));
    of "arccot" do x:= 1/op(x):
                   return(x/sqrt(1+x^2));
    end_case;

    procname(x)
end_proc:

sin(I):= sinh(1)*I:
sin(0):= 0:
sin(PI):= 0:
sin(PI/2):= 1:
sin(PI/3):= (3^(1/2))/2:
sin(PI/4):= (2^(1/2))/2:
sin(PI/  5):= 1/4*2^(1/2)*(5-5^(1/2))^(1/2):
sin(PI*2/5):= 1/4*2^(1/2)*(5+5^(1/2))^(1/2):
sin(PI/6):= 1/2:
sin(PI/  8):= ((2-2^(1/2))^(1/2))/2:
sin(PI*3/8):= ((2+2^(1/2))^(1/2))/2:
sin(PI/  10):= 1/4*(5^(1/2)-1):
sin(PI*3/10):= 1/4*(5^(1/2)+1):
sin(PI/  12):= (6^(1/2)-2^(1/2))/4: // simpler than ((2-3^(1/2))^(1/2))/2: 
sin(PI*5/12):= (2^(1/2)+6^(1/2))/4: // = ((2+3^(1/2))^(1/2))/2:

sin:= funcenv(sin):
sin:= subsop(sin, 2=op(specfunc::sin, 2)):
sin:= slot(sin, "print", "sin"):
sin:= slot(sin, "info", "sin -- the sine" ):
sin:= slot(sin, "type", "sin"):
sin:= slot(sin, "float", specfunc::sin):

sin:= slot(sin, "inverse", "arcsin"):

sin:= slot(sin, "expand",
    loadproc(slot(sin,"expand"),pathname("STDLIB","EXPAND"),"sin")
):

sin:= slot(sin, "conjugate",
    loadproc(slot(sin,"conjugate"),pathname("STDLIB","CONJ"),"sin")
):

sin:= slot(sin,"rectform",
    loadproc(slot(sin,"rectform"),pathname("STDLIB","RECTFORM"),"sin")
):

sin:= slot(sin,"simplify",
    loadproc(slot(sin,"simplify"),pathname("STDLIB","SIMPLIFY"),"sin")
):

sin:= slot(sin, "series",
    loadproc(slot(sin,"series"),pathname("SERIES"),"sin")
):

sin:= slot(sin,"Re",
    loadproc(slot(sin,"Re"),pathname("STDLIB","RE"),"sin")
):

sin:= slot(sin,"Im",
    loadproc(slot(sin,"Im"),pathname("STDLIB","IM"),"sin")
):

sin:= slot(sin, "diff",
    loadproc(slot(sin,"diff"),pathname("STDLIB","DIFF"),"sin")
):

// end of file 
