// $Date: 2000/05/21 15:02:59 $ $Author: bij $ $Revision: 1.18 $ 

// kg, 13/01/94 

/*++
sinh -- the hyperbolic sine

sinh(x)

x - expression
++*/

sinh:= proc(x)
  option noDebug;
  local f;
  name sinh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::sinh <> FAIL then return(x::dom::sinh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;
  
  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, sinh))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-sinh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(slot(sinh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(sinh,"float")(x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-sinh(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then return(0) // sinh(n*I*PI)=0 
        elif type(2*f)=DOM_INT then return((-1)^(f-1/2)*I)
	end_if;
	break;
    
    // cf Table 4.5.60 p. 85 of Abramowitz and Stegun 
    of "arcsinh" do return(op(x))
    of "arccosh" do return(sqrt(op(x)^2-1))
    of "arctanh" do return(op(x)/sqrt(1-op(x))/sqrt(1+op(x)))
  //of "arccoth" do return(sqrt(1/(op(x)^2-1)))
    of "arccoth" do 
           x:= 1/op(x): // note that op(x) <> 0, because arccoth(0) is
                        // simplified to I*PI/2
           return(x/sqrt(1-x)/sqrt(1+x))

    end_case;

    procname(x)
end_proc:

sinh(0):= 0:
sinh(I*PI):= 0:
sinh(I*PI/2):= I:
sinh(I*PI/3):= I*(3^(1/2))/2:
sinh(I*PI/4):= I*(2^(1/2))/2:
sinh(I*PI/  5):= I*1/4*2^(1/2)*(5-5^(1/2))^(1/2):
sinh(I*PI*2/5):= I*1/4*2^(1/2)*(5+5^(1/2))^(1/2):
sinh(I*PI/6):= I*1/2:
sinh(I*PI/  8):= I*((2-2^(1/2))^(1/2))/2:
sinh(I*PI*3/8):= I*((2+2^(1/2))^(1/2))/2:
sinh(I*PI/  10):= I*1/4*(5^(1/2)-1):
sinh(I*PI*3/10):= I*1/4*(5^(1/2)+1):
sinh(I*PI/  12):= I*(6^(1/2)-2^(1/2))/4: // simpler th. ((2-3^(1/2))^(1/2))/2: 
sinh(I*PI*5/12):= I*(2^(1/2)+6^(1/2))/4: // = ((2+3^(1/2))^(1/2))/2:

sinh( infinity):= infinity:
sinh(-infinity):=-infinity:

sinh:= funcenv(sinh):
sinh:= subsop(sinh, 2=op(specfunc::sinh, 2)):
sinh:= slot(sinh,"info","sinh -- the hyperbolic sine"):
sinh:= slot(sinh, "print", "sinh"):
sinh:= slot(sinh, "type", "sinh"):
sinh:= slot(sinh, "float", specfunc::sinh):

sinh:= slot(sinh, "inverse", "arcsinh"):

sinh:= slot(sinh, "diff", proc(f) begin
    op(f,1);
    diff(%, args(2..args(0))) * cosh(%)
end_proc):

sinh := slot(sinh, "expand",
    loadproc(slot(sinh,"expand"),pathname("STDLIB","EXPAND"),"sinh")
):

sinh:= slot(sinh,"conjugate",
    loadproc(slot(sinh,"conjugate"),pathname("STDLIB","CONJ"),"sinh")
):

sinh:= slot(sinh,"rectform",
    loadproc(slot(sinh,"rectform"),pathname("STDLIB","RECTFORM"),"sinh")
):

sinh:= slot(sinh, "series",
    loadproc(slot(sinh,"series"),pathname("SERIES"),"sinh")
):

sinh:= slot(sinh,"simplify",
    loadproc(slot(sinh,"simplify"),pathname("STDLIB","SIMPLIFY"),"sinh")
):

sinh:= slot(sinh,"Re",
    loadproc(slot(sinh,"Re"),pathname("STDLIB","RE"),"sinh")
):

sinh:= slot(sinh,"Im",
    loadproc(slot(sinh,"Im"),pathname("STDLIB","IM"),"sinh")
):

// end of file 
