// $Date: 2000/05/21 15:03:00 $ $Author: bij $ $Revision: 1.16 $ 

// kg, 13/01/94 
// ok, 21/01/94 

/*++
sqrt -- the square root

sqrt(x)

x - expression
++*/

sqrt:=
proc(xx)
  option noDebug;
  name sqrt;
  local l,e,f,i,p,k,s,x;
begin
  if args(0) = 0 then
    error("no arguments given")
  end_if;

  if xx::dom::sqrt <> FAIL then
    return(xx::dom::sqrt(args()))
  elif args(0) <> 1 then
    error("one argument expected")
  end_if;

  case type(xx)
    of DOM_SET do
    of "_union" do
      return(map(xx, sqrt))
  end_case;

  
  if not testtype(xx,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  x:= xx;
  case type(x)
    of DOM_FLOAT do
      return(slot(sqrt,"float")(x));
    of DOM_COMPLEX do
      if type(op(x, 1)) = DOM_FLOAT or type(op(x, 2)) = DOM_FLOAT then
        return( slot(sqrt,"float")(x) )
      end_if;
      break;
    of DOM_RAT do
      return(sqrt(op(x,1))*sqrt(op(x,2))/op(x,2))
    of DOM_INT do
    of "_plus" do
    of "_mult" do // do not simplify sqrt(x*y) to sqrt(x)*sqrt(y) 
      i:=abs(specfunc::icontent_expr(x));
        // try to factor, but don't try too hard 
      l:=stdlib::ifactor(i, UsePrimeTab);
      e:=1; x:=l[1]*(x/i); f:=1;
      for i from 1 to nops(l) div 2 do
        p:=op(l,2*i); k:=op(l,2*i+1);
        f:=f*p^(k div 2); if k mod 2 = 1 then e:=e*p end_if
      end_for;
      // look for squares of reals 
      for l in (if type(x)="_mult" then x else hold(_mult)(x) end_if) do
        if type(l)="_power" then
          if type((i:=op(l,2)/2))=DOM_INT then
            if contains({"abs"},type((k:=op(l,1)))) then
              x:=x/l; f:=f*k^i
            elif contains({-1,1},(s:=sign(k^i))) then
              x:=x/l; f:=f*k^i*s
            end_if;
          end_if
        end_if;
      end_for;
      return(f*e^(1/2)*x^(1/2))
    of "_power" do
      e := op(x,2); f := op(x,1);
      if type(e)=DOM_INT then
        if modp(e,2)=0 then
          if is((s:= f^(e/2)) >= 0) = TRUE then
            return(s)
          elif is(s < 0) = TRUE then
            return(-s)
          end_if
        end_if
      end_if;
      if f <> E then break end_if;
      x := exp(e); // sqrt(x) = sqrt(E^e) -> sqrt(exp(e)) 
      // step into the next switch           
    of "exp" do // exp(x)^(1/2) is exp(x/2) if -PI/2<arg(exp(x/2))<=PI/2, -exp(x/2) otherwise 
      e:=exp(op(x)/2);
      case sign(Re(e))
        of 1 do
          return(e)
        of 0 do
          case sign(Im(e))
            of 1 do
            of 0 do
              return(e)
            of -1 do
              return(-e)
          end_case;
          break
        of -1 do
          return(-e)
      end_case;
      break
    end_case;

  x^(1/2)
end_proc:

sqrt(I) := (-1)^(1/4):
sqrt(0) := 0:


sqrt:= funcenv(sqrt):
sqrt:= subsop(sqrt, 2=op(specfunc::sqrt, 2)):
sqrt:= slot(sqrt, "print", "sqrt"):
sqrt:= slot(sqrt, "info", "sqrt -- the square root"):
sqrt:= slot(sqrt, "type", "sqrt"):
sqrt:= slot(sqrt, "float", specfunc::sqrt):

/*--
specfunc::icontent_expr -- get integer contents of an expression

specfunc::icontent_expr(e)

e - expression
--*/

specfunc::icontent_expr:=
proc(e)
begin
  case type(e)
    of DOM_INT do
      return(e);
    of "_plus" do
      return(igcd(map(op(e), specfunc::icontent_expr)));
    of "_mult" do
      op(e, nops(e));
      if domtype(%) = DOM_INT then return(%) end_if;
      break;
    of "_power" do
      if domtype(op(e,2)) = DOM_INT then
        if op(e,2) > 0 then
          return(specfunc::icontent_expr(op(e,1))^op(e,2))
        end_if
      end_if;
  end_case;
  1
end_proc:

sqrt:= slot(sqrt, "diff",
    loadproc(slot(sqrt,"diff"),pathname("STDLIB","DIFF"),"sqrt")
):

// end of file 
