// $Date: 2000/05/21 15:03:04 $ $Author: bij $ $Revision: 1.22 $ 

// kg, 13/01/94 

/*++
tan -- the tangens

tan(x)

x - expression
++*/

tan:= proc(x)
  option noDebug;
  local f, y;
  name tan;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::tan <> FAIL then return(x::dom::tan(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;

  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, tan))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
      if x < 0 then return(-tan(-x)) end_if;
      break;
    
    of DOM_FLOAT do
	return(slot(tan,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
           domtype(op(x,2)) = DOM_FLOAT then
	       return(slot(tan,"float")(x))
	   end_if;
	if op(x,1) = 0 then return(tanh(-x*I)*I) end_if;
	break;

    of "_mult" do
/*--
due to performance reasons, move this code to tan::simplify
--*/
/*
       if nops(x)>nops(x/PI) then // x = multiple of PI
          if is(x/PI, Type::Integer) = TRUE then
               return(0)
          elif is(2*x/PI, Type::Even) = TRUE then
               return(0)
          elif is(2*x/PI, Type::Odd) = TRUE then
               error("singularity");
          end_if;
       end_if;
*/
//--
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-tan(-x)) end_if;
	    if domtype(f) = DOM_FLOAT then break end_if;
	else
	    break
	end_if;

	if nops(x) = 2 then
	    if op(x,1) = PI then
		if f < 1/2 then break end_if;
		if f = 1/2 then error("singularity") end_if;
		if f < 1 then return(-tan((1-f)*PI)) end_if;
		if f < 2 then return(tan((f-1)*PI)) end_if;
		return(tan((f-2*(floor(f) div 2))*PI));
	    end_if
	end_if;
	break;
    of "_plus" do
       // due to performance: move this code to tan::simplify
       // if sysorder(-x,x)=TRUE then return(-tan(-x)) else break end_if

       // handle tan(y + integer*PI) -> tan(y)
       // We could also do tan(y + PI/2) -> -cot(y).
       // At the moment, we decide not to do it.
       if has(x, PI) then
          for y in x do
              f:= y/PI;
              if testtype(f, DOM_INT) then
                 return(tan(x-y))
              end_if;
          end_for:
       end_if:
       break;
    of "arctan" do return(op(x));
    of "arccot" do return(1/op(x));
    of "arcsin" do return(op(x) / sqrt(1-op(x)^2));
    of "arccos" do return(sqrt(1-op(x)^2) / op(x));
    end_case;

    procname(x)
end_proc:

tan(   0   ):= 0:
tan(  PI/3 ):= (3^(1/2)):
tan(  PI/4 ):= 1:
tan(  PI/5 ):= (5 - 2*5^(1/2))^(1/2):
tan(2*PI/5 ):= (1 + 5^(1/2)) * (2*5^(1/2) + 10)^(1/2)/4:
tan(  PI/6 ):= (3^(1/2))/3:
tan(  PI/8 ):= 2^(1/2)-1:
tan(3*PI/8 ):= 2^(1/2)+1:
tan(  PI/10):= 1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2):
tan(3*PI/10):= 1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2):
tan(  PI/12):= 2-3^(1/2):
tan(5*PI/12):= 2+3^(1/2):
tan(PI):= 0:
tan(I):= tanh(1)*I:

tan:= funcenv(tan):
tan:= subsop(tan, 2=op(specfunc::tan, 2)):
tan:= slot(tan,"print","tan"):
tan:= slot(tan,"info","tan -- the tangens"):
tan:= slot(tan,"type","tan"):
tan:= slot(tan, "float", specfunc::tan):

tan:= slot(tan,"inverse", "arctan"):

tan:= slot(tan,"diff",proc(f) begin
    op(f,1);
    diff(%, args(2..args(0))) *(1+tan(%)^2)
    // tan is better than cos because it does not introduce a new function 
end_proc):

tan:= slot(tan, "expand",
	proc(f) begin expand(sin(f)/cos(f)) end_proc ):

tan:= slot(tan, "conjugate",
    loadproc(slot(tan,"conjugate"),pathname("STDLIB","CONJ"),"tan")
):

tan:= slot(tan,"rectform",
    loadproc(slot(tan,"rectform"),pathname("STDLIB","RECTFORM"),"tan")
):

tan:= slot(tan,"simplify",
    loadproc(slot(tan,"simplify"),pathname("STDLIB","SIMPLIFY"),"tan")
):

tan:= slot(tan,"series",proc(f,x,n,dir)
begin
  if dir = FAIL then
    series(sin(f),x,n)/series(cos(f),x,n)
  else
    series(sin(f),x,n,dir)/series(cos(f),x,n,dir)
  end_if
end_proc):

tan:= slot(tan,"Re",
    loadproc(slot(tan,"Re"),pathname("STDLIB","RE"),"tan")
):

tan:= slot(tan,"Im",
    loadproc(slot(tan,"Im"),pathname("STDLIB","IM"),"tan")
):

// end of file 
