// $Date: 2000/10/31 14:28:22 $ $Author: stefanw $ $Revision: 1.23 $ 

// kg, 13/01/94 

/*++
tanh -- the hyperbolic tangens

tanh(x)

x - expression
++*/

tanh:= proc(x)
  option noDebug;
  local f;
  name tanh;
begin
  if args(0) = 0 then error("no arguments given")
  elif x::dom::tanh <> FAIL then return(x::dom::tanh(args()))
  elif args(0) <> 1 then error("wrong no of args") 
  end_if;
  
  case type(x)
    of DOM_SET do
    of "_union" do
      return(map(x, tanh))
  end_case;

  if not testtype(x,Type::Arithmetical) then
    error("argument must be of 'Type::Arithmetical'")
  end_if;

  case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-tanh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(slot(tanh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(slot(tanh,"float")(x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::Real) then
	    if f < 0 then return(-tanh(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then return(0) // tanh(k*I*PI)=0 
        elif type(2*f)=DOM_INT then
           //return((-1)^(f-1/2)*I*infinity)
             error("singularity");
	end_if;
	break;

    // Table 4.5.60 p. 85 of Abramowitz and Stegun 
    of "arctanh" do return(op(x))
    of "arccoth" do return(1/op(x))
    of "arcsinh" do return(op(x)/sqrt(1+op(x)^2))
    of "arccosh" do return(sqrt(op(x)^2-1)/op(x))

    end_case;

    procname(x)
end_proc:

tanh(0):= 0:
tanh(I*PI):= 0:
tanh(  I*PI/3 ):= I*(3^(1/2)):
tanh(  I*PI/4 ):= I:
tanh(  I*PI/5 ):= I*(5 - 2*5^(1/2))^(1/2):
tanh(2*I*PI/5 ):= I*(1 + 5^(1/2)) * (2*5^(1/2) + 10)^(1/2)/4:
tanh(  I*PI/6 ):= I*(3^(1/2))/3:
tanh(  I*PI/8 ):= I*(2^(1/2)-1):
tanh(3*I*PI/8 ):= I*(2^(1/2)+1):
tanh(  I*PI/10):= I*1/5*5^(1/2)*(5 - 2*5^(1/2))^(1/2):
tanh(3*I*PI/10):= I*1/5*5^(1/2)*(5 + 2*5^(1/2))^(1/2):
tanh(  I*PI/12):= I*(2-3^(1/2)):
tanh(5*I*PI/12):= I*(2+3^(1/2)):

tanh(infinity):=1:
tanh(-infinity):=-1:

tanh:= funcenv(tanh):
tanh:= subsop(tanh, 2=op(specfunc::tanh, 2)):
tanh:= slot(tanh,"info","tanh -- the hyperbolic tangent"):
tanh:= slot(tanh,"print","tanh"):
tanh:= slot(tanh,"type","tanh"):
tanh:= slot(tanh, "float",
           proc(x) local err, tmp;
           begin
            err:= traperror( (tmp:= specfunc::tanh(x)) );
            if err = 0 then
                 return(tmp)
            elif err = 20 then // Error: Overflow/underflow
                 x:= float(x);
                 if domtype(op(x,1)) = DOM_FLOAT then
                    // PARI underflow for |Re(x)|>-5.81*10^6;
                    if op(x,1)<-10^(6) then return(float(-1)) end_if:
                    if op(x,1)> 10^(6) then return(float( 1)) end_if:
                 end_if
            end_if;
            // error erneut ausloesen
            lasterror();
          end_proc):

tanh:= slot(tanh,"inverse", "arctanh"):

tanh:= slot(tanh, "undefined",
            loadproc(tanh::undefined, pathname("STDLIB","DISCONT"), 
                     "tanh")):
tanh:= slot(tanh, "realDiscont", {}):
tanh:= slot(tanh, "complexDiscont",
            loadproc(tanh::complexDiscont, pathname("STDLIB","DISCONT"),
                     "tanh")):

tanh:= slot(tanh,"diff",proc(f) begin
    op(f,1);
    diff(%, args(2..args(0))) / cosh(%)^2
end_proc):

tanh:= slot(tanh, "expand",
	    proc(f) begin expand(sinh(f)/cosh(f)) end_proc ):

tanh:= slot(tanh, "conjugate",
    loadproc(slot(tanh,"conjugate"),pathname("STDLIB","CONJ"),"tanh")
):

tanh:= slot(tanh,"rectform",
    loadproc(slot(tanh,"rectform"),pathname("STDLIB","RECTFORM"),"tanh")
):

tanh:= slot(tanh,"series",proc(f,x,n,dir)
begin
  if dir = FAIL then
    series(sinh(f),x,n)/series(cosh(f),x,n)
  else
    series(sinh(f),x,n,dir)/series(cosh(f),x,n,dir)
  end_if
end_proc):

tanh:= slot(tanh,"simplify",
    loadproc(slot(tanh,"simplify"),pathname("STDLIB","SIMPLIFY"),"tanh")
):

tanh:= slot(tanh,"Re",
    loadproc(slot(tanh,"Re"),pathname("STDLIB","RE"),"tanh")
):

tanh:= slot(tanh,"Im",
    loadproc(slot(tanh,"Im"),pathname("STDLIB","IM"),"tanh")
):

// end of file 
