// $Date: 2000/05/21 15:03:24 $  $Author: bij $   $Revision: 1.5 $ 

/*--
usage:
BPCorr(l) / BPCorr(s, c, ...)

Bravais-Pearson coefficient of correlation
--*/
/*--
parameters:
l  list of rows with 2 elements each or sequence of 2 columns
s  stats::sample
c  sequence / list of data-column indices
   (optional if sample contains only 2 data-columns)
--*/

stats::BPCorr := proc()
  local l, x1_, x2_, g;
begin
  l:=stats::getdata(testargs(), "all_data", 2, args());
  if testargs() then
    if domtype(l) = DOM_STRING then error(l) end_if
  end_if;

  // calculation 
  l:=zip(l, proc() begin [args(1), args(2)] end_proc );
  if l = [] then
    FAIL // division by zero 
  else
    x1_:=_plus(op(map(l, op, 1)))/nops(l); // mean of 1st column 
    x2_:=_plus(op(map(l, op, 2)))/nops(l); // mean of 2nd column 
    g:=sqrt((_plus(op(map(map(l, op, 1), _power, 2))) - nops(l)*(x1_^2))
           *(_plus(op(map(map(l, op, 2), _power, 2))) - nops(l)*(x2_^2)));
    if iszero(g) then
      FAIL // division by zero 
    else
      (_plus(op(map(l, proc() begin op(args(1), 1)*op(args(1), 2) end_proc )))
       - nops(l)*x1_*x2_)/g
    end_if
  end_if
end_proc:


// end of file 
 
