// $Date: 2000/05/30 09:35:22 $  $Author: walter $   $Revision: 1.9 $ 

/*--
usage:
testdata(opt, l) / testdata(opt, s, c)

internal procedure, required by procedures like getdata to
test arguments / elements of arguments.
if a data-column of a sample is specified, the column index is verified
option "all_data":
                allows DOM_INT, DOM_FLOAT, DOM_COMPLEX, DOM_RAT, DOM_EXPR, DOM_IDENT
option "numeric_only":
                allows DOM_INT, DOM_FLOAT, DOM_COMPLEX, DOM_RAT
if called by a procedure or function, all errors are returned as
strings. if no errors are found, TRUE is returned.
--*/
/*--
parameters:
opt option, "all_data" or "numeric_only"
l   list or sequence
s   stats::sample
c   data-column index (optional if sample contains only one row)
--*/

stats::testdata := proc(opt, li, co)
  local l, c, err, floatconvert;
begin
  l:=li; 
  if testargs() then // return errors as errors 
    err:=error;
    if args(0) < 2 then error("Wrong number of arguments") end_if;
    if opt <> "all_data" and opt <> "numeric_only" then 
      error("Invalid arguments")
    end_if
  else // return errors as strings (if testdata is called by procedure) 
    err:=return
  end_if;

  floatconvert:= proc(x)
  begin
      if domtype(x) = DOM_EXPR or contains(Type::ConstantIdents, x)
      then float(x)
      else x;
      end_if;
  end_proc;

  if domtype(l) = stats::sample then
    case args(0)
    of 2 do // 1 + opt : no column specified 
      if nops(op(l, 1)) <> 1 then
        err("Wrong number of arguments")
      end_if; c:=1; break;
    of 3 do // 2 + opt : column specified 
      c:=co;
      if not testtype(c, DOM_INT) or c < 1 or nops(op(l, 1)) < c
           or domtype(op(l, [1, c])) = DOM_STRING
      then
        err("Invalid data-column index")
      end_if; break;
    otherwise
      err("Wrong number of arguments")
    end_case;
    if opt = "numeric_only" then
      // convert expressions to floats
      l:= map(l, map, floatconvert);
      // double check that all entries are numerical
      if has(map(stats::sample2list(l), proc() begin 
                        testtype(op(args(1), c), DOM_INT) 
                     or testtype(op(args(1), c), DOM_FLOAT) 
                     or testtype(op(args(1), c), DOM_COMPLEX) 
                     or testtype(op(args(1), c), DOM_RAT) 
                                        end_proc ), FALSE) then 
        err("Invalid operand: integers, rationals or floats expected")
      end_if;
    end_if
  else
    if domtype(l) = DOM_LIST then
      if args(0) <> 2 then // 1 + opt 
        err("Wrong number of arguments")
      end_if;
    else // sequence to list 
      l:=[args(2..args(0))]; // exclude opt from sequence 
    end_if;
    case opt
    of "all_data" do
      if has(map(l, proc() begin
                        testtype(args(1), DOM_INT) 
                     or testtype(args(1), DOM_FLOAT) 
                     or testtype(args(1), DOM_COMPLEX) 
                     or testtype(args(1), DOM_EXPR)
                     or testtype(args(1), DOM_IDENT)
                     or testtype(args(1), "_index")
                     or testtype(args(1), DOM_RAT) end_proc ), FALSE) then 
        err("Invalid operand: integers, rationals, floats, expressions or identifiers expected");
      end_if; break;
    of "numeric_only" do
      // convert expressions to floats
      if domtype(l[1]) = DOM_LIST then // l is a list of lists
           l:= map(l, map, floatconvert);
      else l:= map(l, floatconvert);
      end_if:
      if has(map(l, proc() begin
                        testtype(args(1), DOM_INT) 
                     or testtype(args(1), DOM_FLOAT) 
                     or testtype(args(1), DOM_COMPLEX) 
                     or testtype(args(1), DOM_RAT) end_proc ), FALSE) then 
        err("Invalid operand: integers, rationals or floats expected")
      end_if;
    end_case
  end_if; 
  return(TRUE)
end_proc:

// end of file 
