// $Date: 2000/11/03 14:40:18 $  $Author: stefanw $  $Revision: 1.24 $ 

RootOf:=
proc(eqq,zz)
  local eq,z;
begin
   if args(0)=1 then
     z:=indets(eqq);
     if nops(z)<>1 then
       error("you should specify the RootOf variable")
     end_if;
     z:=op(z)
   else
     z:=zz
   end_if;
   if indets(z, PolyExpr)<>{z} then
     error("Illegal variable")
   end_if;
   if type(eqq)=DOM_POLY then
     eq:=expr(eqq)
   else
     eq:=eqq
   end_if;
   eq:=numer(eq);
   if testtype(eq,Type::PolyExpr(z,Type::Rational)) then 
      eq:=polylib::primpart(eq,[z]) 
   end_if;
   procname(eq,z)
end_proc:

RootOf:= funcenv(RootOf):
RootOf:= slot(RootOf, "print", "RootOf"):
RootOf:= slot(RootOf, "type",  "RootOf"):

RootOf:= slot(RootOf, "Re",
              proc()
              begin
                hold(Re)(hold(RootOf)(args()))
              end_proc
              ):

RootOf:= slot(RootOf, "Im",
              proc()
              begin
                hold(Re)(hold(RootOf)(args()))
              end_proc
              ):

RootOf:=slot(RootOf, "series",
                 loadproc(slot(RootOf,"series"),pathname("SERIES"),
                          "RootOf")):

// normalizes the fraction f in x, where x=RootOf(p) 
stdlib::RootOf_evala := proc(f,p,_x)
local l;
begin
   userinfo(1,"f=",f,"p=",p,"x=",_x);
   l:=gcdex(denom(f),p,_x); 
   // l[1]=denom(f)*l[2]+p*l[3] thus 1/denom(f) = l[2]/l[1] mod p 
   if has(l[1],_x) then error("singular denominator") end_if;
   numer(f)*l[2]/l[1]
end_proc:

// computes the sum of the terms in f, where x goes through the roots of p 
stdlib::RootOf_polysum :=
proc(f,p,x)
  local d,y,f1;
begin
  f1:=divide(f,p,[x],Rem);
  d:=degree(p,[x]);
  if not has(f1,x) then
    d*f1
  else
    y:= genident("y");
    f1:=polylib::resultant(p,numer(y-f1),x);
    normal(-coeff(f1,[y],d-1)/coeff(f1,[y],d))
  end_if
end_proc:

RootOf:=slot(RootOf, "diff",
proc(f,x) 
local P,y,yp;
begin
   if args(0)=1 then f
   elif args(0)=2 then
      y:=op(f,2);
      if x=y then error("cannot differentiate with respect to RootOf variable")
      end_if;
      P:=op(f,1);
      if testtype(P,Type::PolyExpr(y)) then
         yp:=(if type(y)=DOM_IDENT then genident(expr2text(y))
	 elif type(y)="function" then genident(expr2text(op(y,0)))(op(y))
	 else genident() end_if);
         // P:=subs(P,y=yy); # to avoid problems if y is not a name #
         RootOf(polylib::resultant(P,diff(P,x)+yp*diff(P,y),y),yp)
      else error("can differentiate only algebraic equations") 
      end_if
   else slot(RootOf,"diff")(slot(RootOf, "diff")(f,x),args(3..args(0)))
   end_if
end_proc):

// RootOf::has:=proc(x,s) begin has(extop(x,1),s) end_proc:

// RootOf::int:=proc() begin hold(int)(args()) end_proc:

/* 
>> float(RootOf(x^4+x+1));

0.7271360845 + 0.9340992894 I, 0.7271360845 - 0.9340992894 I,

- 0.7271360845 - 0.4300142883 I, - 0.7271360845 + 0.4300142883 I

>> float(RootOf(-y^2+RootOf(0.7271360845+0.9340992894*I+y^2+1,y),y));

- 0.9340992894 + 0.7271360845 I, 0.9340992894 - 0.7271360845 I,

- 0.7271360845 - 0.9340992894 I, 0.7271360845 + 0.9340992894 I
*/

RootOf:= slot
(RootOf, "float", 
 proc(f, var)
   local l,x,v,i;
 begin
   userinfo(3, "RootOf's float-attribute called with ".expr2text(args()));
   l:=misc::subExpressions(f, "RootOf");
   if nops(l)=0 then
     userinfo(10, "No nested RootOf");
     if numeric::indets(f) minus {var} = {} then
       userinfo(10, "Calling numeric method");
       slot(solve, "float")(args())
     else
       userinfo(10, "Equation contains parameters");
       hold(RootOf)(args())
     end_if
   else
     x:=op(l,1);
     for i from 2 to nops(l) do
       if length(op(l,i))<length(x) then x:=op(l,i) end_if
     end_for;
     v:=[op(float(x))];
     {op(slot(RootOf,"float")(subs(f,x=v[i]), args(2..args(0))))
     $ i=1..nops(v)}
   end_if
 end_proc):

// expand a^i where a is a RootOf 
stdlib::RootOf_expand_power:=proc(a,i) local y;
begin
   if type(i)=DOM_INT then
      y:=op(a,2); 
      return(subs(divide(y^i,op(a,1),Rem),y=a))
   end_if;
   a^i
end_proc: 

// end of file 
