// $Date: 2000/09/15 15:28:57 $ $Author: frankp $ $Revision: 1.5 $
// frankp, 15.03.1995

/*
        asympt  --  computing asymptotic expansions

        asympt(ex,x [=v] [,ord])

        ex : expression in x
        x  : ident
        v  : (optional) expression
        ord: (optional) positive integer

        Compute the asymptotic series of ex of order ord with respect
        to at v. If v is missing then x is assumed to goes to infinity.
        If ord is missing then the value of ORDER is used (6 by default).

        If an expansion could not be computed, then the expression
        ex will be returned.
        Otherwise an element of the domain 'gseries' (defined in
        the library "Series") is returned, that can be manipulated 
        with the standard arithmetic operations.

        See gseries.mu for detailed informations.
*/

asympt:= proc(f,x,ord)
    local s;
begin 
    if args(0) = 0 then
        error("no arguments given")
    elif f::dom::asympt <> FAIL then 
        // allow overloading of the first argument of asympt:
        return(f::dom::asympt( args() )) 
    elif args(0) < 2 or args(0) > 4 then
        error("wrong no of args")
    elif domtype(x) = DOM_IDENT then
        s:= x = infinity
    elif type(x) <> "_equal" or domtype(op(x,1)) <> DOM_IDENT then
        error("invalid 2nd argument")
    else
        s:= op(x,2);
        if not testtype(s,Type::Numeric) and s <> infinity and s <> -infinity then
            // Note: is(infiniy,Type::Real) gives UNKNOWN!?
            case is(s,Type::Real)
            of TRUE    do break
            of FALSE   do error("expansion point must be a real arithmetical expression")
            of UNKNOWN do error("expansion point cannot be checked on being real")
            end_case;
        end_if;
        s:= x
    end_if;

    s:= Series::gseries::new(f,s,args(3..args(0)));
    if s = FAIL then return(procname(args())) else return(s) end
end_proc:

asympt:= slot(funcenv(asympt), "type", "asympt"):

