
/*++
complexInfinity -- representing the 'complex infinity' in the compactification of C

There exists only one element 'complexInfinity' of this domain (stdlib::CInfinity)
which has no operands.
++*/

stdlib::CInfinity:= newDomain("stdlib::CInfinity");
stdlib::CInfinity::info:=
    "domain 'stdlib::CInfinity': representing the complex infinity";
stdlib::CInfinity::interface:= {};

stdlib::CInfinity::expr:= proc() begin hold(complexInfinity) end_proc;
stdlib::CInfinity::print:= stdlib::CInfinity::expr;
stdlib::CInfinity::TeX:= proc() begin "\\infty" end_proc;

stdlib::CInfinity::_negate:= id;
stdlib::CInfinity::_invert:= proc() begin 0 end_proc;
stdlib::CInfinity::_not:= proc() begin error("invalid operand") end_proc;

stdlib::CInfinity::_power:= proc() begin
    case domtype(args(2))
    of stdlib::CInfinity do
        if domtype(args(1)) = stdlib::CInfinity or
           domtype(args(1)) = domtype(infinity) or
           args(1) = undefined then 
            return(undefined)
        end_if;
        if is(abs(args(1))>1) = TRUE then complexInfinity
        elif is(abs(args(1))<1) = TRUE then 0
        else hold(_power)(args())
        end_if;
        break;
    of DOM_FLOAT do
        if iszero(args(2)) then return(1.0) end_if;
    of DOM_INT do
        if args(2) = 0 then return(1) end_if;
    of DOM_RAT do
	if args(2) < 0 then 0 else complexInfinity end_if;
	break;
    otherwise
        if is(args(2)<0) = TRUE then return(0) end_if;
	hold(_power)(args())
    end_case
end_proc:

stdlib::CInfinity::_plus:= proc() begin
    if map({args()}, domtype) intersect
	{stdlib::Undefined, stdlib::CInfinity, stdlib::Infinity} <> {}
    then
        undefined
    else 
        complexInfinity
    end_if
end_proc;

stdlib::CInfinity::_mult:= proc() begin
    _mult(op(select([args()], not (stdlib::CInfinity)::thisDomain)));
    if % = undefined or iszero(%) then undefined else complexInfinity end_if
end_proc;

stdlib::CInfinity::intmult:= proc() begin
    if iszero(args(2)) then undefined else complexInfinity end_if
end_proc;

stdlib::CInfinity::subs:= proc() begin
    if op(args(2),1) = complexInfinity then
        op(args(2),2)
    else
        complexInfinity
    end_if
end_proc;

stdlib::CInfinity::subsop:= proc() begin
    if args(2) = 1 then args(3) else FAIL end_if
end_proc;

stdlib::CInfinity::op:= proc() begin
    if args(0) = 2 then
        if args(2) = 1 then complexInfinity else FAIL end_if
    else 
        complexInfinity
    end_if
end_proc;

stdlib::CInfinity::has:= proc() begin bool(args(2) = complexInfinity) end_proc;

stdlib::CInfinity::O:= proc() begin undefined end_proc;

stdlib::CInfinity::thisDomain:= proc() begin
    bool(domtype(args(1)) = stdlib::CInfinity)
end_proc;

complexInfinity:= new(stdlib::CInfinity);

// end of file 
