// 
// $Date: 2000/11/03 13:28:13 $ $Author: jngerhar $ $Revision: 1.28 $ 

// frankp, 15.02.1995 

/*++
   combine.mu

	combine  --  combine terms of the same algebraic structure

	combine(expr) combines terms of the same algebraic
	structure.

	For expressions of type DOM_FUNC_ENV the function attribute
	"combine" will be called with the operands of expr.
	If such a function attribute does not exist the expression will
	be returned without changes.
++*/

combine :=
proc(e,p)
  local _t, f, l, i, n, b, _e;
begin
    // is overloaded ? 
  if e::dom::combine <> FAIL then
    return( e::dom::combine(e,args(2..args(0))) )
  end_if;

  case args(0)
    of 1 do
      break
    of 2 do
      if type(p)=DOM_LIST then
        for _t in p do
          e:=combine(e,_t)
        end_for;
        return(e)
      elif traperror((p:=slot(combine,expr2text(args(2)))))=0 then
        if p <> FAIL then
           return(p(e))
        end_if:
        error("invalid second argument")
      else
        error("invalid second argument")
      end_if
    otherwise
      error("wrong no of args")
  end_case;

  case domtype(e)
    // default is to return e, thus we do not need to consider these cases 
    of DOM_POLY	   	do return( mapcoeffs(e,combine) )
    of Series::Puiseux	do return( map(e,combine) )
    of DOM_LIST    	do
    of DOM_ARRAY   	do
    of DOM_SET     	do return( map(e,combine) )
    of DOM_EXPR    	do
      _t:= op(e,0);
      if domtype(eval(%)) = DOM_FUNC_ENV and
        (p:= slot(eval(%),"combine")) <> FAIL then
        return( p(op(e)) )
      end_if;
      case _t
        of hold(_mult) do
          // rewrite x^a*x^b --> x^(a+b)
          e:=map(e,combine);
          if type(e)<>"_mult" then
            return(e)
          end_if;

          // f(ausdr, b)
          // tests whether ausdr has the form b^something
          // where "something" may also equal 1
          f := proc()
               begin
                 if type(args(1))="_power" then
                   if op(args(1),1) = args(2) then
                     TRUE
                   else
                     FALSE
                   end_if
                 else
                   bool(args(1)=args(2))
                 end_if
               end_proc ;
          p := 1;
          _e := {op(e)};
          i := 1;
          repeat
            _t := op(_e,i);
            if type(_t) = "_power" then
              b := op(_t,1); n := op(_t,2)
            else
              b := _t; n := 1
            end_if;
            _e := _e minus {_t};
            l := select(_e, f, b);
            _e := _e minus l;
            for _t in l do
              if type(_t)="_power" then
                n := n + op(_t,2)
              else
                n:=n+1
              end_if;
            end_for;
            p := p*b^normal(n);
          until nops(_e) = 0 end_repeat;
          if hastype( p,"_power" ) then
            return( slot(combine,"_power")(p) )
          elif p <> e then
            return( combine(p) )
          else
            return( p )
          end_if
        of hold(_power) do
          l:=combine(op(e,1)); f:=combine(op(e,2));
          // rewrite (a^b)^c --> a^(b*c)
          // *** BUG !!! *** This rule is not universally valid!
          if type(l)="_power" and type(f)<>DOM_RAT then
            return( op(l,1)^(op(l,2)*f) )
          end_if;

          if (p:= slot(combine,type(l))) <> FAIL then
            return( p(e) )
          end_if
      end_case;

      return( map(e,combine) )
  end_case;
  e
end_proc:

combine:= funcenv(combine):
combine::print:= "combine":

// sqrt(a)*sqrt(b) ==> sqrt(a*b) 
combine:=slot(combine,"sqrt",
proc(e) 
name combine::sqrt;
local t,a,f,g,matchsqrt,n;
begin
   matchsqrt := proc(e) // matches sqrt(f) and returns f, or FAIL 
   begin
      if type(e)="_power" then if op(e,2)=1/2 then return(op(e,1)) end_if
      elif type(e)="sqrt" then return(op(e))
      end_if;
      FAIL
   end_proc;

   case type(e)
   of "_mult" do
      f:=1; g:=1; n:= 0;
      for t in e do
         if (a:=matchsqrt(t))<>FAIL then 
	    if is(a>0) = TRUE then n:= n+1 end_if;
	    f:=f*a 
	 else 
	    g:=g*t 
   	 end_if
      end_for;
      if n+1 >= nops(e) then return(f^(1/2)*g)
      else return( e )
      end_if
   of "_plus" do return(map(e,slot(combine,"sqrt")))
   otherwise e
   end_case;
end_proc):

// cos(x)^2 -> cos(2*x)/2+1/2 
combine:=slot(combine,"sincos",proc(ee)
name combine::sincos;
local l,k,f, e;
begin
   e:=ee; // to avoid warnings 
   case domtype(e) // default is to return e 
   of DOM_POLY	  do return( mapcoeffs(e,slot(combine,"sincos")) )
   of Series::Puiseux	  do
   of DOM_LIST    do 
   of DOM_ARRAY   do
   of DOM_SET     do return( map(e,slot(combine,"sincos")) )
   of DOM_EXPR    do
      case type(e)
      of "_mult" do e:=[op(e)]; break
      of "_power" do e:=[e]; break
      otherwise return(map(e,slot(combine,"sincos")))
      end_case;
      // now deal with _mult and _power case 
      k:=1; l:=[];
      for f in e do
         case type(f)
         of "sin" do
         of "cos" do l:=append(l,f); break
         of "_power" do
            if contains({"sin","cos"},type(op(f,1))) and testtype(op(f,2),Type::PosInt) then
               l:=append(l,op(f,1)$op(f,2)); break
            end_if
         otherwise k:=k*map(f,slot(combine,"sincos"))
         end_case;
      end_for;
      return(stdlib::combine_sin_cos(l,k))
   end_case;
   e
end_proc):

// k is a scalar, l is a list of sin or cos expressions 
stdlib::combine_sin_cos := proc(ll,kk)
local l, k;
begin
   l:=ll; k:=kk; // to avoid warnings 
   if l=[] then return(k) end_if;
   while nops(l)>=2 do
      if type(l[1])="_plus" then
         l[1]:=map(l[1],stdlib::combine_sincos,l[2])
      else l[1]:=stdlib::combine_sincos(l[1],l[2])
      end_if;
      delete l[2];
   end_while;
   if type(l[1])="_plus" then map(l[1],_mult,k) else k*l[1] end_if
end_proc:

// for x*y where x and y can be cos(..) or a*cos(..) or a where a is a rational 
stdlib::combine_sincos := proc(xx,yy) 
local a,b,m, x, y;
begin
   x:=xx; y:=yy;
   if type(x)="_mult" then x:=split(x,testtype,Type::Union("cos","sin"));
	m:=x[2]; x:=x[1]; else m:=1 end_if;
   if type(y)="_mult" then y:=split(y,testtype,Type::Union("cos","sin"));
	m:=m*y[2]; y:=y[1] end_if;
   a:=op(x); b:=op(y);
   case type(x),type(y)
   of "sin","sin" do m/2*cos(a-b)-m/2*cos(a+b); break
   of "sin","cos" do m/2*sin(a+b)+m/2*sin(a-b); break
   of "cos","sin" do
     if stdlib::hasmsign(a-b) then
       m/2*sin(a+b)+m/2*sin(b-a)
     else
       m/2*sin(a+b)-m/2*sin(a-b)
     end_if;
     break
   of "cos","cos" do m/2*cos(a-b)+m/2*cos(a+b); break
   otherwise m*x*y /* it can happen that x or y is a constant, 
			for example cos(0) -> 1 for cos(t)^3 */
   end_case
end_proc:

// arctan(x)+arctan(y) = arctan((x+y)/(1-x*y)) 
combine:=slot(combine,"arctan",proc(e)
name combine::arctan;
local s,f,x,y,i;
begin
   case domtype(e) // default is to return e 
   of DOM_POLY	  do return( mapcoeffs(e,slot(combine,"arctan")) )
   of Series::Puiseux	  do
   of DOM_LIST    do 
   of DOM_ARRAY   do
   of DOM_SET     do return( map(e,slot(combine,"arctan")) )
   of DOM_EXPR    do
      case type(e)
      of "_plus" do
         s:=0; x:=0;
         for f in e do
            if type(f)="arctan" then
               y:=op(f);
               x:=(x+y)/(1-x*y)
            elif type(f)="_mult" then // n*arctan(y) 
               if nops(f)=2 and type(op(f,1))="arctan" and type((i:=op(f,2)))=DOM_INT then
                  if i>0 then y:=op(op(f,1)) else y:=-op(op(f,1)); i:=-i end_if;
                  while i>0 do x:=(x+y)/(1-x*y); i:=i-1 end_while
               else s:=s+f
               end_if
            else s:=s+f
            end_if
         end_for;
         return(s+arctan(x))
      otherwise return(map(e,slot(combine,"arctan")))
      end_case
   end_case;
   e
end_proc):

/* exp(a)*exp(b) ==> exp(a+b)
  exp(a)^b      ==> exp(a*b) */
combine:=slot(combine,"exp",proc(e) 
name combine::exp;
local x;
begin
   case type(e)
   of "_plus" do return(map(e,slot(combine,"exp")))
   of "_mult" do
      e:=map(e,slot(combine,"exp"));
      if type(e)="_mult" then
         x:=select(e,testtype,"exp");
         if type(x)="_mult" then
            return(e/x*exp(_plus(map(op(x),op))))
         end_if
      end_if; break
   of "_power" do 
      if type((x:=op(e,1)))="exp" then return(exp(op(x)*op(e,2))) end_if; break
   end_case;
   e
end_proc):

/* ln(a)+ln(b) ==> ln(a*b)
  k*ln(a) ==> ln(a^k) */
combine := slot
(combine, "ln",
 proc(ee)
   name combine::ln;
   local e, f, fpos, fneg, isnegativelog, fpos_arguments, fpos_posarguments,
         fpos_otherarguments, fneg_arguments, fneg_posarguments,
         fneg_otherarguments;
 begin
   isnegativelog := proc(ee) begin
     /* decide whether ee is of the form -ln(something) */
     if (testtype(ee, "_mult") and (nops(ee) = 2)) then
       bool(op(ee, 2) = -1)
     else
       FALSE
     end_if
   end_proc;
 
   e := ee; // to avoid warnings 

   /* recursively combine all operands */
   if type(e) = "_plus" or type(e) = "_mult" then
     e := map(e, slot(combine, "ln"));
   end_if;

   case type(e)
     of "_plus" do

       /* extract all summands of the form ln(a)+ln(b)+... and their args */
       fpos := select(e, testtype, "ln");
       fpos_arguments :=
         if (fpos = 0) then []
         elif testtype(fpos, "ln") then [op(fpos)]
         else [map(op(fpos), op)] end_if;

       /* extract all summands of the form -ln(c)-ln(d)-... and their args */
       fneg := select(e, isnegativelog);	
       fneg_arguments :=
         if (fneg = 0) then []
         elif testtype(fneg, "_mult") then [op(fneg, [1, 1])]
         else [map(op(fneg), op, [1, 1])] end_if;

       /* do nothing if there is only one ln(..) at all */
       if nops(fpos_arguments) + nops(fneg_arguments) < 2 then
         break;
       end_if;

       e := e - fpos - fneg;

       /* multiply all positive arguments */
       fpos_arguments := split(fpos_arguments, is, Type::Positive);
       fpos_posarguments := _mult(op(fpos_arguments[1]));
       fpos_otherarguments := fpos_arguments[2].fpos_arguments[3];
       fneg_arguments := split(fneg_arguments, is, Type::Positive);
       fneg_posarguments := _mult(op(fneg_arguments[1]));
       fneg_otherarguments := fneg_arguments[2].fneg_arguments[3];

       /* if there are any arguments that are not positive, then multiply
	  the first of them by what has just been computed */
       if fpos_otherarguments = [] then
	 if fneg_otherarguments = [] then
           return(e + ln(fpos_posarguments / fneg_posarguments));
         end_if;
         fneg_otherarguments[1] :=
           fneg_otherarguments[1] / fpos_posarguments * fneg_posarguments;
         return(e - _plus(op(map(fneg_otherarguments,ln))));
       end_if;
       fpos_otherarguments[1] :=
         fpos_otherarguments[1] * fpos_posarguments / fneg_posarguments;
       if fneg_otherarguments = [] then
         return(e + _plus(op(map(fpos_otherarguments,ln))));
       end_if;
       return(e + _plus(op(map(fpos_otherarguments,ln)))
                - _plus(op(map(fneg_otherarguments,ln))));

   of "_mult" do
      f:=select(e,testtype,"ln");
      if type(f)="ln"
        and is(e/f, Type::Real)=TRUE
        and is(op(f,1)>0)=TRUE then
        // only one factor ln(..) 
        return(ln(op(f)^(e/f)))
      end_if; break
   end_case;
   e
end_proc):

// 5^x*6^x/10^x/3^x ==> (5*6/10/3)^x ==> 1 // tonner 
combine:=slot
(combine,"_power",
 proc(X)
   name combine::_power;
   local a,f,s,p,x,t;
 begin
   userinfo(5, "Attribute combine::_power called");
   if type(X)<>"_mult" then X
   else
     a:=1; // product of non-powers 
     s:={}; // set of exponents 
     p:=table(); // p[x] is the list of all factors being to power x 
     for f in X do
       if type(f)="_power" then
         x:=op(f,2);
         if not contains(s,x) then
           if not contains(s,-x) or
             /* second case: a^(-x)=(1/a)^x does not hold e.g.
                          for a=I, x=1/2 */
             (is(x, Type::Integer) or is(op(f,1) > 0))<>TRUE then
             p[x]:=[op(f,1)]; s:=s union {x}
           else
             p[-x]:=p[-x] . [1/op(f,1)]
           end_if
         else
           p[x]:=p[x] . [op(f,1)]
         end_if
       else
         a:=a*f
       end_if
    end_for;
    userinfo(10, "Table of powers (by exponent): ",p);
      for f in s do
	 // x>0 or y>0  =>  x^b * y^b = (x*y)^b 
	 // Re(x)>0 or (Re(x)=0 and Im(y)>0)  =>  x^b * y^b = (x*y)^b 
	 t:= map( p[f],
            proc(y)
            begin 
              if testtype(y,Type::Numeric) then
                bool(sign(y)=1)
              else
                is(y>0)
              end_if
            end_proc);
	 // x>0 or y>0  =>  x^b * y^b = (x*y)^b 
	 if nops(select(t,_equal,TRUE)) + 1 >= nops(t) then
	     t:= _mult( op(p[f]) );
             if length(t^f)<=length((1/t)^(-f)) or
               not (is(f, Type::Integer) or is(t>0))<>FALSE then 
               a:=a*t^f
             else
               a:=a*(1/t)^(-f)
             end_if
	 else
            /* Re(x)>0 or (Re(x)=0 and Im(x)>0)  AND
	      Re(y)>0 or (Re(y)=0 and Im(y)>0)      =>  x^b * y^b = (x*y)^b 
	    */ 
            t:= select( p[f], proc() begin 
                (testtype(args(1),Type::Numeric) and (sign(Re(args(1)))=1
                   or sign(Re(args(1)))=0 and sign(Im(args(1)))=1))
                or (is(Re(args(1))>0)=TRUE or
                     (is(Re(args(1))=0) and is(Im(args(1))>0)) = TRUE)
             end_proc  );
            if t = p[f] then
		t:= _mult( op(p[f]) );
                if length(t^f)<=length((1/t)^(-f)) or
                  not (is(f, Type::Integer) or is(t>0))<>FALSE then
                  a:=a*t^f
                else 
                  a:=a*(1/t)^(-f)
                end_if
	    else
	        a:= _mult(a,op(map(p[f],hold(_power),f)))
	    end_if
	end_if
      end_for;
      a
   end_if
end_proc):

/*
|    The following routines are used to combine expressions including
|    sinh and cosh. The algorithms are the same as in case of combining
|    expressions involving sin and cos. Therefore the above procedures
|    combine:=slot(combine,"sincos",...), sdtlib::combine_sin_cos()
|    and stdlib::combine_sincos() have been renamed to
|    combine:=slot(combine,"sincos",...), stdlib::combine_sinh_cosh
|    and stdlib::combine_sinhcosh. 
|    T.S. 03.05.1996
*/

combine:=slot(combine,"sinhcosh",proc(ee)
name combine::sinhcosh;
local l,k,f,e;
begin
   e:=ee;
   case domtype(e) // default is to return e 
   of DOM_POLY    do return( mapcoeffs(e,slot(combine,"sinhcosh")) )
   of Series::Puiseux     do
   of DOM_LIST    do
   of DOM_ARRAY   do
   of DOM_SET     do return( map(e,slot(combine,"sinhcosh")) )
   of DOM_EXPR    do
      case type(e)
      of "_mult" do e:=[op(e)]; break
      of "_power" do e:=[e]; break
      otherwise return(map(e,slot(combine,"sinhcosh")))
      end_case;
      // now deal with _mult and _power case 
      k:=1; l:=[];
      for f in e do
         case type(f)
         of "sinh" do
         of "cosh" do l:=append(l,f); break
         of "_power" do
            if contains({"sinh","cosh"},type(op(f,1))) and testtype(op(f,2),Type
::PosInt) then
               l:=append(l,op(f,1)$op(f,2)); break
            end_if
         otherwise k:=k*map(f,slot(combine,"sinhcosh"))
         end_case;
      end_for;
      return(stdlib::combine_sinh_cosh(l,k))
   end_case;
   e
end_proc):

// k is a scalar, l is a list of sinh or cosh expressions 
stdlib::combine_sinh_cosh := proc(l,k)
begin
   if l=[] then return(k) end_if;
   while nops(l)>=2 do
//print(l);
      if type(l[1])="_plus" then
         l[1]:=map(l[1],stdlib::combine_sinhcosh,l[2])
      else l[1]:=stdlib::combine_sinhcosh(l[1],l[2])
      end_if;
      delete l[2];
   end_while;
//print(l);
   if type(l[1])="_plus" then map(l[1],_mult,k) else k*l[1] end_if
end_proc:

// for x*y where x and y can be cos(..) or a*cos(..) or a where a is a rational 
stdlib::combine_sinhcosh := proc(xx,yy)
local a,b,m, x, y;
begin
   x:=xx; y:=yy;
   if type(x)="_mult" then
        m:=op(x,2);
        x:=op(x,1):
   else
        m:=1
   end_if;
   if type(y)="_mult" then
       m:=m*op(y,2);
       y:=op(y,1):
   end_if;
   a:=op(x); b:=op(y);
   case type(x),type(y)
   of "sinh","sinh" do -m/2*cosh(a-b)+m/2*cosh(a+b); break
   of "sinh","cosh" do m/2*sinh(a+b)+m/2*sinh(a-b); break
   of "cosh","sinh" do
        if stdlib::hasmsign(a-b) then
            m/2*sinh(a+b)+m/2*sinh(b-a);
        else
            m/2*sinh(a+b)-m/2*sinh(a-b);
        end_if:
   break
   of "cosh","cosh" do m/2*cosh(a-b)+m/2*cosh(a+b); break
   otherwise m*x*y /* it can happen that x or y is a constant,
                        for example cos(0) -> 1 for cos(t)^3 */
   end_case:
end_proc:

_plus:=slot(_plus,"combine",
	loadproc(slot(_plus,"combine"),pathname("STDLIB","COMBINE"),
	"_plus")):

// end of file 

