// $Date: 2000/11/16 15:55:47 $ $Author: frankp $ $Revision: 1.10 $
// kg, 05/10/95
// unfreeze modified: frankp, 03/30/00

domain stdlib::frozen
    evaluate:= proc(e) begin extop(e,2) end;
    print:= proc(e) begin extop(e,1) end;
    domtype:= proc(e) begin domtype(extop(e,1)) end;
    type:= proc(e) begin type(extop(e,1)) end;
    func_call:= proc(e) begin subsop(hold(F)(args(2..args(0))), 0=e) end;
end_domain:

/*
freeze -- make function inactive

freeze(f)

f - function

freeze(f) returns a function which is an 'inactive' version of f.
The function returned by freeze:
- only evaluates its arguments, but does not compute anything else,
- prints like f and has the same expression type as f,
- has all the functional attributes of f.

freeze is used for example to create inactive functions to be
returned instead of expensive functions like 'int'. The inactive 'int'
is returned if the expression can not be integrated further in order to
avoid re-evaluation of 'int'.

Use 'unfreeze' to re-activate the function.
*/

freeze:= proc(f)
    option escape, hold;
    local v;
begin
    v:= context(f);
    if domtype(v) <> DOM_FUNC_ENV then
        if domtype(v) <> DOM_PROC then 
            error("expecting a procedure or a function environment")
        else
            v := funcenv(v)
        end_if
    end_if;
    v:= slot(v, "unfreeze", v);
    v:= subsop(v, 1=proc() begin subsop(hold(F)(args()), 0=f) end);
    f:= new(stdlib::frozen, f, v);
    v
end_proc:

/*
unfreeze -- re-activate inactive funtion

unfreeze(f)

f - inactive function or functional expression created by 'freeze'

unfreeze(f) returns the active version of an inactive function
which has been created by 'freeze'. Use 'subsop(e,
0=unfreeze(op(e,0)))' to unfreeze an expression where the operator
has been frozen.
*/

unfreeze:= proc(e) 
    local f; 
begin
    if args(0) <> 1 then error("expecting one argument") end_if;
    if (f:= stdlib::unfreeze_apply(e)) = FAIL then
        // apply unfreeze recursively to the operands of e:
        return(
            misc::maprec( e,(x -> bool(stdlib::unfreeze_apply(x,hold(Check)) <> FAIL)) = stdlib::unfreeze_apply, hold(PostMap) )
        )
    else
        return( f )
    end_if
end_proc:

stdlib::unfreeze_apply:= proc(e)
    local f;
begin
    if domtype(e) <> DOM_FUNC_ENV then
        if domtype(e) = DOM_EXPR and domtype((f:= eval(op(e,0)))) = DOM_FUNC_ENV then
            f:= slot(f, "unfreeze");
            if f <> FAIL then 
                if args(2..args(0)) = hold(Check) then
                    return( TRUE )
                else
                    return(f(op(e))) 
                end_if
            end_if
        end_if;
        return( FAIL )
    else
        return( slot(e, "unfreeze") )
    end_if
end_proc:

