// $Date: 2000/11/29 14:31:48 $  $Author: bij $  $Revision: 1.23 $ 
// kg, 11.05.93 

/*++
info -- print short information

info(d)
info(f)

d - domain
f - function

info(d) prints a short information about the domain d. d must have a
method 'info' or an text attribute 'info'. Either the method 'd::info'
is executed or the text 'info' is printed. If the domain has the attibute
'interface', which must be a set of interface names, these names are printed
also.

info(f) prints a short information about the function f. f must have
an string attribute "info". The string is printed unquoted.
++*/

info:= proc(dat) begin
    if args(0) = 0 then 
        stdlib::infoLibraries()     
    elif args(0) = 1 then 
        if slot(domtype(dat), "printInfo") = FAIL then
           print(NoNL, "Sorry, no information available.\n")
        else
           slot(domtype(dat), "printInfo")(dat)
        end_if
    else
        error("wrong no of args") 
    end_if
end_proc:

stdlib::infoLibraries:=
  proc()
  begin
    print(NoNL, "-- Libraries:\n");
    output::tableForm(stdlib::LIBRARIES, ", ", Unquoted);
    null()
end_proc:

DOM_DOMAIN::printInfo:= proc(dm)
    local dom_name, tab, expo;
begin
    dom_name:= (if dm::Name = FAIL then dm::key else dm::Name end_if);
    if dom_name = FAIL then dom_name:="<<unknown>>" end_if;

    if dm::info = FAIL then
      print(NoNL, "Sorry, no information available.\n");
      return()
    end_if;

    if domtype(dm::info) = DOM_STRING then
        print(NoNL, dm::info);
        print(NoNL, "\n")
    else
        dm::info();
    end_if;

    tab:= dm::interface;
    if tab = FAIL then
        return()
    elif not testtype(tab, Type::SetOf(DOM_IDENT)) then
        error("in attribute \"interface\"")
    end_if;

    expo:= dm::exported;
    if domtype(expo) <> DOM_SET then expo:= {} end_if;
    if domtype(dm::_exported) = DOM_SET then
      expo:= expo union dm::_exported
    end_if;

    if nops(tab) > 0 then
      tab:= map(tab, f->if not contains(expo, f) then dom_name."::".expr2text(f) else null() end_if);
      if nops(tab) > 0 then
        print(NoNL, "\n-- Interface:\n");
        output::tableForm(tab, ", ", Unquoted)
      end_if
    else
        print(NoNL, "No Interface.\n");
    end_if;

    if nops(expo) <> 0 then
        print(NoNL, "\n-- Exported:\n");
        output::tableForm(expo, ", ", Unquoted);
    end_if;

    null()
end_proc:

DOM_FUNC_ENV::printInfo:=
  proc(f)
  begin
    if slot(f, "info") = FAIL then
      print(NoNL, "Sorry, no information available.\n")
    else
      print(NoNL, slot(f, "info")(f));
      print(NoNL, "\n")
    end_if
  end_proc:

DOM_IDENT::printInfo:=
  proc(ID)
  begin
    if contains(stdlib::OPTIONS, ID) then
      print(NoNL, "'" . ID . "' is a protected option.\n")
    else
      print(NoNL, "Sorry, no information available.\n")
    end_if
  end_proc:

// end of file 
