/*++ ---------------- package.mu ---------------------

Author: Winfried Fakler, Ralf Hillebrand
Date Created:  16 March   1999
Last Update :   2 April   1999

- package("path",<Forced,Quiet>)
  inserts a new library package. This package should have the same
  hierarchically structure as mupad's library. In contrast to mupad's
  library where the domains paths are already defined in the file .mupadsysinit
  in a package library this file is called init.mu and needs to be located
  in the directory lib, e.g. for UNIX operating system the new library
  package mypack should have the following structure:
    mypack/lib/init.mu
    mypack/lib/LIBFILES/mypack.mu
    mypack/lib/MYPACK/ ...
  A library package can be placed everywhere in the filesystem and is called
  via the whole path (up to the package name) given as a string. If no option
  is given, information about the package is printed out by a info call at
  the first try for loading the package.
  This can be suppressed with the option Quiet. A package will be loaded only
  once. If the option Forced is given, the package is loaded new by every call.
                                                                             
++*/

package :=
  proc(str:DOM_STRING) // str, options: Forced, Quiet
    option noDebug;
    local fid,readpath,libpath,status,opt,loaded,knownLibraries,newlib,
          lib,t;
  begin
    // detect options
    opt:={args(2..args(0))}; 
    loaded:=TRUE;
    knownLibraries:=stdlib::LIBRARIES;
    status:=FAIL;
    t:=0;
    
    // test, if package exists
    if (fid:=fopen(str.stdlib::PathSep."lib.tar#init.mu")) <> FAIL then
      libpath:=str.stdlib::PathSep."lib.tar#"
    elif (fid:=fopen(str.stdlib::PathSep."lib".stdlib::PathSep."init.mu"))
         <> FAIL then
      libpath:=str.stdlib::PathSep."lib"
    else
      return(FAIL)
    end_if;
    
    // package exists
    // test, if path already exists
    readpath:=  // NOTE! no test if readpath is valid!
        str.stdlib::PathSep."modules".stdlib::PathSep.sysname(hold(Arch)); 
    if not contains({LIBPATH},libpath.stdlib::PathSep) then
      LIBPATH:=libpath,LIBPATH;
      loaded:=FALSE;
    end_if;
    if not contains({READPATH},readpath) then
      if READPATH=hold(READPATH) then READPATH:=readpath
      else
        READPATH:=readpath,READPATH;
      end_if;
    end_if;
    
    // load package
    if not loaded or contains(opt,hold(Forced)) then
      t:=traperror((status:=stdlib::syseval(fread(fid, Plain, Quiet))));
      if loaded then
        warning("Package redefined");
      end_if;
    else // package already loaded
      warning("Package already defined. For redefinition use option Forced");
      status:=null();
    end_if;
    fclose(fid);
    if t<>0 then lasterror() end_if;
      
    // return load status
    if contains(opt,hold(Quiet)) then return(status)
    else
      newlib:=stdlib::LIBRARIES minus knownLibraries;
      for lib in newlib do
        info(eval(text2expr(lib)));
      end_for;
      status
    end_if;
  end_proc:

