
/*
plotfunc3d -- plot 3D function graphs

plotfunc(f1,..., fn, x=xl..xu, y=yl..yu])

f1,...,fn      - expression
x, y           - identifiers
xl, xu, yl, yu - real numbers

The fi are interpreted as functions in x and y from the range 
[xl,xu] x [yl,yu] into the real numbers.

The graphs of these functions are then plotted by the procedure plot3d.

Examples:

// Draw x,y -> sin(x*y)^2 on [0,1]x[0,1]
plotfunc(sin(x*y)^2, x=0..1, y=0..1);	
// Draw x,y -> sin(x), sin(x*y) on [-PI,PI]x[-1,1]
plotfunc(sin(x), sin(x*y), x=-PI..PI, y=-1..1)
 
*/

/*

plotlib::Options[SceneOptions3d] := { Scaling=UnConstrained, Labeling=TRUE }:
plotlib::Options[DefaultXRange] := -5..5: 
plotlib::Options[DefaultYRange] := -5..5:
                
*/

plotfunc3d:= proc()
    // option hold;
    // Hold verursacht nur massive Probleme auf Anwenderseite.
    option escape;
    local a, b, DefaultX, DefaultY, createrange,
          createtitle, gen_x, gen_y, getrange, grid, 
	  ind, nr, pf, plot3dObjects, plot3dOptions, 
	  s, t, ti, x, y, xr, yr;
    save DIGITS;
begin
    if args(0) < 1 then return(plot3d()) end_if;

    /* prepare creating of titles: objects of DOM_FLOAT should'nt be converted
      to strings via expr2text because of ending zeroes
    */
    
    createtitle:= proc()
        name plotfunc3d::createtitle; 
        local s,i;
    begin
        s:= expr2text( args(1) );
	if type(args(1)) = DOM_FLOAT then
           i:= length(s)-1;
           while i > 0 and s[i] = "0" and s[i-1] <> "." do i:= i - 1 end_while;
           s := substring( s,0..i )
	end_if;
	return( s )
    end_proc:

    getrange:= proc(rr)
      name plotfunc3d::getrange;
      local r;
    begin
      r:=rr; // to avoid warnings 
      if domtype(op(r,1)) <> DOM_IDENT then error("no variable") end_if;
      if type(op(r,2)) = "_range" then
        r:= (op(r,1) = [op(op(r,2))])
      end_if;
      if domtype(float(op(r,[2,1]))) <> DOM_FLOAT then
        error("illegal plot range")
      end_if;
      if domtype(float(op(r,[2,2]))) <> DOM_FLOAT then
        error("illegal plot range")
      end_if;
      op(r,1) = op(r,2)
    end_proc;

    createrange := proc(ind, nr, xr, yr)
       name plotfunc3d::createrange;
       local ni;
    begin
       if (ni := nops(ind)) > 2 then 
             error("Functions contain more than 2 different variables.")
       end_if;
       if nr = 0 then
          if ni = 0 then
	     gen_x := TRUE; gen_y := TRUE;
	     return( (x = DefaultX) , (y = DefaultY) )
	  elif ni = 1 then
	     gen_y := TRUE;
	     return( (op(ind, 1) = DefaultX) , (y = DefaultY) )
          else
             return( (op(ind, 1) = DefaultX) , (op(ind, 2) = DefaultY) )
	  end_if;
       elif nr = 1 then
          if ni = 0 then
	     gen_y := TRUE;
	     return( xr , (y = DefaultY) )
	  elif ni = 1 then
	     if op(ind, 1) = op(xr, 1) then
	        gen_y := TRUE;
	        return( xr, (y = DefaultY ) )
	     else
	        return( xr, (op(ind, 1) = DefaultY) )
	     end_if
	  else // ni = 2
	     if has(ind, op(xr,1)) then
                return( xr, (op(ind minus {op(xr, 1)}) = DefaultY) )
  	     else
	        error("More than 2 different variables defined.")
	     end_if
	  end_if
       else // nr = 2
          return( xr, yr )
       end_if;
    end_proc;

    a := []; pf := null(); ti := ""; 
    plot3dOptions := {};  plot3dObjects := {}; 
    ind := {}; xr := null(); yr := null(); nr := 0; 

    DefaultX := plotlib::Options[DefaultXRange]; 
    DefaultX := [op(DefaultX, 1), op(DefaultX, 2)];
    DefaultY := plotlib::Options[DefaultYRange]; 
    DefaultY := [op(DefaultY, 1), op(DefaultY, 2)];
    x := genident("__X"); y := genident("__Y");
    gen_x := FALSE; gen_y := FALSE;

    grid := [20, 20];
    DIGITS:=10;

    for t in [args()] do
        case type(t)
	  of "_equal" do
	     if type(op(t, 2)) = "_range" then
	        if nr = 0 then
	           xr := getrange(t); nr := nr + 1;
	        elif nr = 1 then
	           yr := getrange(t); nr := nr + 1;
	        else
	           error("vertical and horizontal plotting range already specified.")
	        end_if;
	     elif op(t, 1) = Grid then
	        grid := op(t, 2);
	        if not (testtype(grid, Type::ListOf(DOM_INT, 2, 2)) and
		   grid[1] > 1 and grid[2] > 1) then
	           error("Grid=[x,y], x,y integers > 1 expected.")
	        end_if;
	     else
	        // plot3d-Option wurde spezifiziert
		if type(t) <> "_equal" then
		   error("Illegal option: ".expr2text(t))
		end_if;
	        plot3dOptions := plot3dOptions union {t};   
	     end_if;
	     break;
	 of DOM_LIST do
	     // plot3d-Objekt wurde spezifiziert
	     plot3dObjects := plot3dObjects union {t}; 
	     break;
	 otherwise
	     a := append(a, t);
	     ind := ind union (indets(t) minus Type::ConstantIdents);
       end_case
    end_for;

    s := createrange(ind, nr, xr, yr); 
    xr := op(s, 1); yr := op(s, 2);
        
    nr := nops(a); 
    for t in a do
        pf := pf, [Mode=Surface, [ op(xr,1), op(yr,1), t ], xr, yr, Grid=grid];
        ti := ti . createtitle(t); 
	if (nr := nr - 1) > 0 then ti := ti.", " end_if
    end_for;

    // define scene attributes 
    if gen_x then xr := "x" else xr:= expr2text(op(xr,1)) end_if;
    if gen_y then yr := "y" else yr:= expr2text(op(yr,1)) end_if;
    b := {op(plotlib::Options[Scene3d] union 
          {Labels=[xr, yr, "z"], Title=ti})};
    a := b;
    for t in b do
        if has(plot3dOptions, op(t, 1)) then
           a := a minus {t}
        end_if
    end_for;

    a := op(a), op(plot3dOptions), op(plot3dObjects), pf;

    // plot the stuff 
    plot3d(a);

end_proc:

// end of file 
