// $Date: 2000/07/20 10:11:29 $  $Author: frankp $  $Revision: 1.7 $

/*++
    taylor -- expand expression into taylor series

    taylor(f, x < =a> <, order>)

    f     - expression to expand
    x     - identifier
    a     - arithmetical expression
    order - nonnegative integer

    taylor(f, x=a <, order>) expands f according to the variable 
    x at point p into a taylor series of order o.

    taylor(f, x <, order>) expands f at x=0.

    'order' may be missing, in this case the global variable ORDER
    determines the order of expansion. The default value of ORDER 
    is 6.

    An error is reported, of a Taylor series expansion does
    not exist.
++*/

taylor:= proc(f,x,order=ORDER)
   local s;
begin
   if args(0) <> 0 and (s:= f::dom::taylor) <> FAIL then
      return( s(args()) )
   elif args(0) < 2 then
      error("expecting at least two arguments")
   elif args(0) > 3 then
      error("too many arguments")
   elif domtype(order) <> DOM_INT or order < 0 then
      error("3rd argument, the order of the series computation, must be a nonnegative integer")
   end_if;

   s:= series(args(),hold(NoWarning));
   if type(s) = "series" then
      warning("could not compute Taylor series expansion; try 'series' with option 'Left', 'Right', or 'Real' for a more general expansion");
      return(procname(args()))
   elif testtype(s, Type::Series(hold(Taylor),x)) then
      return(s)
   else
      error("does not have a Taylor series expansion, try 'series'")
   end_if
end_proc:

taylor:= slot(funcenv(taylor), "type", "taylor"):

