// $Date: 2000/11/29 10:00:30 $ $Author: jngerhar $ $Revision: 1.15 $ 

// kg, 27/12/94 

/*++
_print_userinfo -- print information for user

_print_userinfo(pnam, <hold(Text), > lev, epxr...)

pnam - procedure name
Text - output strings/exprsequ without quotes/commas
lev  - level of information (positive integer)
expr - expression to print

'_print_userinfo' prints its arguments 'expr...' to the screen if either the
"information level" of the procedure or domain calling 'userinfo' or
the "global information level" is greater or equal to 'lev'.

The information level (of a procedure or domain or the global one) is defined 
via 'setuserinfo'. The default level is 0, ie. no information is printed.
++*/

_print_userinfo:=
  proc(pname, Opt = hold(Text))
    option hold;
    local info_level, TEXT, ISIN, PRINT, LEV, dname, lev, nam, INFO, EXPR2TEXT;
  begin
    // This is necessary because expr2text quotes strings
    EXPR2TEXT := proc(s) begin
      if domtype(s) = DOM_STRING then s else expr2text(s) end_if
    end_proc;
    if args(0) < 3 then return(null()) end_if;
    if context(Opt) = hold(Text) then
      TEXT:= 1;
      PRINT:= proc() begin
        print(NoNL, _concat(INFO, EXPR2TEXT(args(i)) $ i=1..args(0) - 1,
                            _if(nam, " ", null()), EXPR2TEXT(args(args(0))),
                            "\n")) end_proc
    else
      TEXT:= 0;
      PRINT:= proc() begin
        print(NoNL, _concat(INFO, EXPR2TEXT(args(1)),
                            (", ", EXPR2TEXT(args(i))) $ i = 2..args(0) - 1,
                            _if(nam, " ", null()), EXPR2TEXT(args(args(0))),
                            "\n")) end_proc
    end_if;
    LEV:= context(args(2 + TEXT));
    info_level:= _userinfo_level();

    ISIN:= (case type(LEV)
	      of DOM_INT do
		(X->bool(LEV <= X));
		break
	      of "_range" do
		(BR->bool(op(LEV, 1) <= BR and op(LEV, 2) >= BR));
		break
	      otherwise
		(BR->FALSE)
	    end_case
	    );

    INFO:= "Info: ";
    nam:= FALSE;
    if contains(info_level, pname) then
      lev:= info_level[pname];
      if type(lev) <> DOM_INT then
        if op(lev, 2) = hold(Quiet) then
          INFO:= ""
        else
          nam:= TRUE
        end_if;
        lev:= op(lev, 1)
      end_if;
      if ISIN(lev) then
	PRINT(context(args((3 + TEXT)..args(0))),
                (if nam then [pname] else "" end));
	return()
      end_if
    end_if;

    if type(pname)="slot" then
      dname:=pname;
      repeat
	dname:=op(dname,1);
      until type(dname)<>"slot" end_repeat;
      if contains(info_level, dname) then
        lev:= info_level[dname];
        if type(lev) <> DOM_INT then
          if op(lev, 2) = hold(Quiet) then
            INFO:= ""
          else
            nam:= TRUE
          end_if;
          lev:= op(lev, 1)
        end_if;
        if ISIN(lev) then
	  PRINT(context(args((3 + TEXT)..args(0))),
                (if nam then [pname] else "" end));
	  return()
        end_if
      end_if
    end_if;

    if contains(info_level, hold(Any)) then
      lev:= info_level[hold(Any)];
      if type(lev) <> DOM_INT then
        if op(lev, 2) = hold(Quiet) then
          INFO:= ""
        else
          nam:= TRUE
        end_if;
        lev:= op(lev, 1)
      end_if;
      if ISIN(lev) then
	PRINT(context(args((3 + TEXT)..args(0))),
              (if nam then [pname] else "" end));
	return()
      end_if
    end_if;

    null()
end_proc:

// end of file 
