/*-

stringlib::contains -- tests if a substring exists

Calling sequence:

stringlib::contains(s, t)
stringlib::contains(s, t, Index)
stringlib::contains(s, t, IndexList)

Parameter:

s         - non-empty string
t         - non-empty string
Index     - an optional name
IndexList - an optional name

Summary:

stringlib::contains(s, t) returns TRUE, if the string s contains the
string t. Otherwise FALSE is returned.

stringlib::contains(s, t, Index) returns an integer i.
//a list [n, m, i]. n is the length of s and m is the length of t.
//The length of a string is the number of characters.
If i = FAIL, then the string s doesnt 
contain the string t. Otherwise i is an integer and 
s[i..i+m-1] = substring(s, i, m) = t holds (m = length(t)). 

stringlib::contains(s, t, IndexList) returns a list l
//[n, m, l].
l is a list. If l is the empty list, i.e. l = [], then the string s doesnt 
contain the string t. Otherwise s[l[i]..l[i]+m-1] = t for all
i = 1..nops(l) holds (m = length(t)).

Examples:

>> stringlib::contains("abcdabe", "ab");

           TRUE
		   
>> stringlib::contains("abcdabe", "ab", Index);

           0 // 1.4.2 yields [7, 2, 0]
		   
>> stringlib::contains("abcdabe", "ab", IndexList);

           [0, 4] // 1.4.2 yields [7, 2, [0, 4]]
		   
>> stringlib::contains("abcdabe", "abd");

           FALSE
		   
>> stringlib::contains("abcdabe", "abd", Index);

           FAIL // 1.4.2 yields [7, 3, FAIL]
		   
>> stringlib::contains("abcdabe", "abd", IndexList);

           [] // 1.4.2 yields [7, 3, []]
		   
See also:

contains, substring, stringlib::match

-*/

stringlib::contains:=
  proc(s, t, opt=FAIL)
    local i, k, l, m, n;
  begin
    if testargs() then
      case args(0)
        of 3 do
          if opt <> Index and opt <> IndexList then
            error("Illegal 3rd argument")
          end_if;
        of 2 do
          if not (testtype(s, DOM_STRING) and s <> "" and
                  testtype(t, DOM_STRING) and t <> "") then
            error("Non-empty string expected")
          end_if;
          break;
        otherwise
          error("Wrong number of arguments")
      end_case
    end_if;
    
    m := length(t); n := length(s);
    k := stringlib::FirstIndex;
    case opt
      of Index do
        //return( [ n, m, stringlib::match(s, t, n, m, k) ] );
        return(stringlib::match(s, t, n, m, k))
      of IndexList do
        //r := [ n, m ];
        l:= [];
        while (i:= stringlib::match(s, t, n, m, k)) <> FAIL do
          l:= append(l, i); k := i + m
        end_while;
        //return(append(r, l));
        return(l)
      otherwise
        return( strmatch(s, "\*".t."\*") )
    end_case
  end_proc:

