/*-

stringlib::remove -- deletes substrings

Calling sequence:

stringlib::remove(s, t)
stringlib::remove(s, t, First)
stringlib::remove(s, a)
stringlib::remove(s, a..b)

Parameter:

s, t  - strings
a, b  - non-negative integers
First - name

Examples:

>> stringlib::remove("abcdab", "ab", First);

                                  "cdab"

>> stringlib::remove("abcdab", "ab");       

                                   "cd"

>> stringlib::remove("abcdab", 3);

                                  "abcab"

>> stringlib::remove("abcdab", 1..3);
 
                                   "aab"
-*/

stringlib::remove := proc(s, r, opt=FAIL)
   local a, b, len;
begin

   if testargs() then

      if type(s) <> DOM_STRING then
         error("String expected");
      end_if;
      len := length(s);
      if type(r) = "_range" then
         a := op(r, 1); b := op(r, 2);
         if not (testtype(a, Type::NonNegInt) and testtype(b, Type::NonNegInt)) 
            or b < a or b >= len then
            error("Illegal range")
         end_if;
         stringlib::subsop(s, r="")
      elif testtype(r, Type::NonNegInt) then
           if r >= len then
              error("Illegal index")
           end_if;
           stringlib::subsop(s, r="") 
      elif type(r) = DOM_STRING then
           if args(0) = 3 then
              if opt <> First then
                 error("Illegal option")
              end_if;
              stringlib::subs(s, r="", First)
           else
              stringlib::subs(s, r="")
           end_if
      else
           error("Illegal argument")
      end_if

   else

     if type(r) = DOM_STRING then
        stringlib::subs(args())
     else
        stringlib::subsop(s, r="")
     end_if

   end_if
        
end_proc:
