// $Date: 2000/04/10 15:01:17 $ $Author: frankp $ $Revision: 1.9 $
// frankp, 7.09.95

/*++
    student::Kn  --  the vectorspace of n-tupels

    Kn( <n,> F )

    n: (optional) positive integer.
    F: a field, i.e., a domain of category Cat::Field.

    This contructor creates a domain representing the vectorspaces
    of n-tupels over F.

    The operator + is defined to add two n-tupels component-wise.
    Kn together with this operation forms an Abelian group.

    This domain constructor is simply build up by the domain
    constructor 'Dom::MatrixGroup'.

    The scalar multiplication (by default realized by
    Product["_mult"]) can be overloaded by the method
    "scalarMult".
++*/

domain student::Kn( n,F )
    local MatF, MatrixF;

    inherits Dom::MatrixGroup( n,1,F );
    category Cat::VectorSpace( F ), Cat::Matrix( F );
    axiom 
        (if F::hasProp( Ax::canonicalRep ) then Ax::canonicalRep end_if);

    // methods:
    new:= proc()
        local A;
    begin
        // Some argument testing here because we don't want thew options
        // Diagonal and Banded here, which are useful for the Dom::MatrixGroup
        // but not for student::Kn
        if args(0) > 1 then
            error("expecting not more than one argument") ;
        elif traperror( (A:=MatF::new(n,1,args())) ) <> 0 then
            error("invalid argument(s)")
        else
            extsubsop(A,0=dom)
        end_if
    end_proc;

    _mult:= proc(x,y)
        local T;
    begin
        if args(0) > 2 then
            return( dom::_mult(x,
                dom::_mult( y,args(i) $ i=3..args(0) )
            ) )
        end_if;
        if domtype(x) = dom and domtype(y) = dom then
            return( FAIL )
        elif domtype(x) = dom then
            if dom::scalarMult <> FAIL then
                dom::scalarMult( y,x )
            else
                T:= x::dom::getSuperDomain(x);
                T:= T::_mult( extsubsop(x,0=T),y );
                if T <> FAIL then 
                    return( extsubsop(T,0=dom) )
                else
                    return( FAIL )
                end_if
            end_if
        elif domtype(y) = dom then
            if dom::scalarMult <> FAIL then
                dom::scalarMult( x,y )
            else
                T:= y::dom::getSuperDomain(y);
                T:= T::_mult( x,extsubsop(y,0=T) );
                if T <> FAIL then 
                    return( extsubsop(T,0=dom) )
                else 
                    return( FAIL )
                end_if
            end_if
        else
            return( FAIL )
        end_if
    end_proc;

    delRow:= proc()
        local r;
    begin
        r:= slot(MatrixF,"delRow")(args());
        if domtype(r) = DOM_NULL then 
            return(r)
        else 
            return( extsubsop(r,0=MatrixF) )
        end_if
    end_proc;

    delCol:= proc()
        local r;
    begin
        r:= slot(MatrixF,"delCol")(args());
        if domtype(r) = DOM_NULL then 
            return(r)
        else 
            return( extsubsop(r,0=MatrixF) )
        end_if
    end_proc;

    stackMatrix:= proc()
        local r;
    begin
        r:= slot(MatrixF,"stackMatrix")(args());
        if domtype(r) = DOM_NULL then return(r)
        else return( extsubsop(r,0=MatrixF) )
        end_if
    end_proc;

    concatMatrix:= proc()
        local r;
    begin
        r:= slot(MatrixF,"concatMatrix")(args());
        if domtype(r) = DOM_NULL then return(r)
        else return( extsubsop(r,0=MatrixF) )
        end_if
    end_proc;

    row:= proc()
    begin
        extsubsop( slot(MatrixF,"row")(args()), 0=MatrixF )
    end_proc;

    col:= proc()
    begin
        extsubsop( slot(MatrixF,"col")(args()), 0=MatrixF )
    end_proc;

    // must not be inherited, because dom does not contain the n x n identity matrix!
    identity:= () -> error("invalid method call (there is no identity matrix)");

/* body of the domain */
begin
    if args(0) <> 1 and args(0) <> 2 then
        error("expecting one or two arguments")
    end_if;
    if args(0) = 1 then 
        F:= n; 
        n:= 1 
    end_if;

    if domtype(F) <> DOM_DOMAIN or F::hasProp( Cat::Field ) <> TRUE then
        error("coefficient domain must be a field, i.e., of category 'Cat::Field'")
    elif not testtype( n,Type::PosInt ) then
        error("number of components must be a positive integer")
    end_if;

    MatF:= Dom::MatrixGroup( n,1,F );
    MatrixF:= Dom::Matrix( F );
end_domain:

