/*++
  frankp, 7/09/95

        student::equateMatrix  --  return a matrix equation

	equateMatrix( A,vars )

	A   : augmented coefficient matrix
	vars: list of unknowns

	equateMatrix( A,vars ) returns the equation
	    A[1..r,1..c-1] * X = A[1..r,c..c]

	where r and c are the row and column number of A and
	X is the vector formed by the unknowns in vars.
++*/

student::equateMatrix:= proc(eA,vars)
    local M, MeA, X, b, d;
begin
    if testargs() then
        if args(0) <> 2 then
	    error("wrong no of args")
        end_if;
        if eA::dom::hasProp( Cat::Matrix ) <> TRUE then
 	    error("expecting a matrix")
        end_if;
        if domtype(vars) <> DOM_LIST then
	    error("expecting a list of unknowns") 
        end_if ;
        if nops(vars) <> op( eA::dom::matdim(eA), 1 ) then
          error("dimension of matrix and number of vars don't match")
        end_if ;
    end_if;

    // Convert eA to a matrix over ExpressionField() 
    M:= Dom::Matrix();
    MeA:= M( eA );

    // Create X = [ x_i ]_i, x_i in vars 
    X:= M( nops(vars),1,vars );

    d:= M::matdim(MeA);

    // Extract the right-side vector of eA 
    b:= M::col( MeA, d[2] );

    // Return the matrix equation 
    MeA[1..d[1], 1..d[2]-1] * X = b
end_proc:

// end of file 
