// $Date: 2000/11/24 16:20:34 $  $Author: frankp $   $Revision: 1.6 $
// igel, 6.03.2000

student::plotRiemann := proc(f, range, N = 10, via = hold(Middle))
    local color, z, fterm, tmp, numerical, p, i, a, b, x, boxes, frames;
begin
    if args(0) = 0 then error("no arguments given") end_if;
    if f::dom::plotRiemann <> FAIL then return( f::dom::plotRiemann(args()) ) end_if;

    if testargs() then
        if args(0) = 1 or args(0) > 4 then
            error("two or at most four arguments expected");
        elif not testtype(range, Type::Equation(DOM_IDENT, "_range"))
             or domtype(float(op(range,[2,1]))) <> DOM_FLOAT or domtype(float(op(range,[2,2]))) <> DOM_FLOAT
        then
            error("2nd argument: illegal range definition ('ident = a..b') expected");
        elif not testtype(f, Type::Arithmetical) then
            error("1st argument: arithmetical expression expected");
        elif not testtype(N, Type::PosInt) and not testtype(N, Type::ListOf(Type::Arithmetical, 1))
           and args(0) = 3 and not contains({hold(Middle), hold(Left), hold(Right)}, N)
        then
            error("3rd argument: illegal type");
        end_if
    end_if;

    if domtype(f) <> DOM_PROC then
        fterm := f;
        z := op(range, 1);
        f := X -> eval(subs(fterm, z = X));
    end_if;

    x:= table();
    if domtype(N) <> DOM_INT then
        if domtype(N) <> DOM_LIST then
            via := N; N := 10;

            a := op(range, [2, 1]);
            b := op(range, [2, 2]);
            (x[i] := a + i * (b-a)/N) $ i=0..N;
        else
            a := op(range, [2, 1]);
            b := op(range, [2, 2]);
            
            N := sort(float(N));
            if float(a) > N[1] or float(b) < op(N, nops(N)) then
                error("range must contain all elements of the node set");
            end_if;
          
            tmp := [a, op(N), b];
            N := nops(tmp) - 1;
            (x[i-1] := tmp[i]) $ i = 1..N+1
         end_if
    else
        a := op(range, [2, 1]);
        b := op(range, [2, 2]);

        (x[i] := a + i * (b-a)/N) $ i=0..N
    end_if:

    tmp:= table();
    case via 
    of hold(Middle) do 
        color := RGB::Orange;
        via:= "middle";

        p := [[plot::Point(x[i], 0), plot::Point(x[i+1], (tmp[i] := f((x[i] + x[i+1])/2)))] $ i = 0..N-1];
        // doesn't work with non-aequidistant nodes!? 
        // numerical := _plus( numeric::quadrature( f(op(f, 1)), op(f,1) = x[i]..x[i+1], NC = 1, Adaptive = FALSE ) ):
        numerical := _plus( ( x[i+1] - x[i] )*tmp[i] $ i = 0..N-1 );
        break:
    of hold(Right) do
        color := RGB::Yellow;
        via:= "right";

        p := [[plot::Point(x[i], (tmp[i] := f(x[i+1]))), plot::Point(x[i+1], 0)] $ i = 0..N-1];
        numerical := _plus( ( x[i+1] - x[i] )*tmp[i] $ i = 0..N-1 );
        break:
    of hold(Left) do
        color := RGB::Gray;
        via:= "left";

        p := [[plot::Point(x[i], 0), plot::Point(x[i-1], (tmp[i] := f(x[i-1])))] $ i = 1..N];
        numerical := _plus( ( x[i] - x[i-1] )*tmp[i] $ i = 1..N)
    end_case;

    tmp := map( p, x -> plot::Rectangle2d(op(x, 1), (op(x, [2,1])- op(x, [1,1])), op(x, [2,2])-op(x, [1,2]), hold(Color) = RGB::EnglishRed) );

    frames := plot::Group(op(tmp));
    boxes  := frames::dom::modify(frames, hold(Color) = color, hold(Filled) = TRUE);

    f:= plot::Function2d( 
        f(op(range, 1)), range, hold(Color) = RGB::Black
    ); 

    return( plot::Group(boxes, frames, f) ) // take care: first print the boxes, then the frames!
end_proc:

