student::plotSimpson := proc(f, range, N = 10, grid = 75)
    local z, fterm, colors, whole, n, i, a0, b0, a, b, x, y, parabs, areas;
begin
    if args(0) = 0 then error("no arguments given") end_if;
    if f::dom::plotSimpson <> FAIL then return( f::dom::plotSimpson(args()) ) end_if;

    if testargs() then
        if args(0) = 1 or args(0) > 4 then
            error("two up to four arguments expected");
        elif not testtype(range, Type::Equation(DOM_IDENT, "_range"))
             or domtype(float(op(range,[2,1]))) <> DOM_FLOAT or domtype(float(op(range,[2,2]))) <> DOM_FLOAT
        then
            error("2nd argument: illegal range definition ('ident = a..b') expected")
        elif not testtype(f, Type::Arithmetical) then
            error("1st argument: arithmetical expression or procedure expected")
        elif not testtype(N, Type::PosInt) then
            error("3rd argument: illegal number of nodes (expecting a positive integer)")
        elif not testtype(grid,Type::PosInt) then
            error("4th argument: expecting a positive integer")
        end_if
    end_if;

    if domtype(f) <> DOM_PROC then
        fterm := f;
        z := op(range, 1);
        f := X -> eval(subs(fterm, z = X)); 
    end_if;

    range := op(range, 2);
    a0 := op(range, 1);
    b0 := op(range, 2);
    x:= table();
    (x[i] := a0 + i*(b0-a0)/N) $ i=0..N;

    parabs := [];
    y := genident("Y");
    for i from 0 to N-1 do
        parabs := append(parabs, [
            op(numeric::lagrange([[ x[i],    (x[i] + x[i+1])/2,    x[i+1] ]],
            array(1..3, [ f(x[i]), f((x[i] + x[i+1])/2), f(x[i+1])]), y),1), y = x[i]..x[i+1]
        ]);
    end_for;

    whole := b0 - a0;
    colors := [RGB::Yellow, RGB::LightGray];

    i := 0;
    areas := [];
    for y in parabs do 
        // approximative Flaechen via gefuellte Polygone:
        i := i+1;
        y := op(y,[2, 2]);
        a:= op(y,1); b:= op(y,2);
        n := ceil((b - a) * grid / whole);
        b := (b -a) / n;
        areas := append(areas, 
            plot::area([(a + i * b) $ i = 0..n],f,x->0, Color = colors[(i mod 2) + 1], Filled = TRUE)
        )
    end_for:

    y := op(f, 1);    
    range := y = range;    
    parabs := _plus(numeric::quadrature(f(y), y = x[i]..x[i+1], hold(NewtonCotes) = 3, hold(Adaptive) = FALSE) $ i = 0..N-1);
    
    f:= plot::Function2d(
      f(op(range, 1)), range, hold(Color) = RGB::Black, hold(LineWidth) = 1
    );

    return(plot::Group( op(areas), f))
end_proc:

