student::plotTrapezoid := proc(f, range, N = 10)
    local z, fterm, tmp, numerical, p, i, a, b, x, frames, traps, y;
begin
    if args(0) = 0 then error("no arguments given") end_if;
    if f::dom::plotTrapezoid <> FAIL then return( f::dom::plotTrapezoid(args()) ) end_if;

    if testargs() then
        if args(0) = 1 or args(0) > 3 then
            error("two or three arguments expected")
        elif not testtype(range, Type::Equation(DOM_IDENT, "_range"))
             or domtype(float(op(range,[2,1]))) <> DOM_FLOAT or domtype(float(op(range,[2,2]))) <> DOM_FLOAT
        then
            error("2nd argument: illegal range definition ('ident = a..b') expected");
        elif not testtype(f, Type::Arithmetical) then
            error("1st argument: arithmetical expression expected");
        elif not testtype(N, Type::PosInt) and not testtype(N, Type::ListOf(Type::Arithmetical, 1)) then
            error("3rd argument: illegal type (either positive integer or list of arithmetical expressions expected)");
        end_if
    end_if;

    if domtype(f) <> DOM_PROC then
        fterm := f;
        z := op(range, 1);
        f := X -> eval(subs(fterm, z = X));
    end_if;

    if domtype(N) <> DOM_INT then
        a := op(range, [2, 1]);
        b := op(range, [2, 2]);
        N := sort(float(N));
        if float(a) > op(N, 1) or float(b) < op(N, nops(N)) then
            error("range must contain all elements of the node set");
        end_if:
    
        tmp := [a, op(N), b];
        (x[i-1] := tmp[i]) $ i = 1..nops(tmp);
        N := nops(tmp) - 1;
    else
        a := op(range, [2, 1]);
        b := op(range, [2, 2]);

        (x[i] := a + i * (b-a)/N) $ i=0..N
    end_if;

    y := genident("y"):    
    numerical := float(_plus( numeric::quadrature(f(y), y = x[i]..x[i+1], hold(NewtonCotes) = 2, hold(Adaptive) = FALSE) $ i = 0..N-1 ));

    p := [[plot::Point(x[i], 0), plot::Point(x[i], f(x[i])), plot::Point(x[i+1], f(x[i+1])), plot::Point(x[i+1], 0)] $ i = 0..N-1];
    
    traps := map(p, x -> plot::Polygon(op(x)));

    frames := plot::Group( op(traps, 1..nops(traps)), hold(Color) = RGB::Yellow, hold(Filled) = TRUE );
    traps  := frames::dom::modify( frames, hold(Color) = RGB::Red, hold(Closed) = TRUE, hold(Filled) = FALSE );

    f:= plot::Function2d( 
        f(op(range, 1)), range, Color = RGB::Black
    ); 

    return(plot::Group(frames, traps, f)) // take care: first plot the frames, then the traps!
end_proc:

