// Dirk Mueller, olorien, 28/10/96, v1.0 

alias( IntTrans = transform::IntTrans ):
alias( invlaplace = transform::invlaplace ):

invlaplace:=newDomain("transform::invlaplace"):
invlaplace::type:="transform::invlaplace":

invlaplace::has:=proc() local T, v;
begin
  T:=args(1); v:=args(2);
  bool( v=extop(T,2) or v=extop(T,3) or 
        has(extop(T,1), v) or v=hold(invlaplace)
  )
end_proc:

invlaplace::new:=proc(_f, var1, var2) 
  local _f2, back;
begin
   if args(0) = 0 then error("no arguments given") end_if,

   // allow overloading 
   if _f::dom::invlaplace<>FAIL then
     return(_f::dom::invlaplace(args()))
   end_if ;
  if testargs() then
    if args(0)<>3 then error("wrong number of arguments") end_if;
 
    if type(_f) = "_equal" then
        if not testtype(_f,Type::Arithmetical=Type::Arithmetical) then
            error("invalid 1st argument")
        end_if
    elif not testtype(_f,Type::Arithmetical) then
        error("1st argument must be of 'Type::Arithmetical'")
    end_if;

    if domtype(var1)<>DOM_IDENT then
      error("second argument must be of type DOM_IDENT");
    end_if;
  end_if;

  if domtype(_f)=transform::laplace then
    return(subs(extop(_f,1), extop(_f,2)=var2))
  end_if;

  back:=invlaplace::compute(_f, var1, var2);

  if back=FAIL and testtype(_f,Type::RatExpr(var1)) then

      back:=invlaplace::compute(partfrac(_f, var1), var1, var2);
  end_if;

  if back=FAIL then
    _f2:=expand(_f);

    if (not (_f=_f2)) then

      back:=invlaplace::compute(_f2, var1, var2);

    end_if;
  end_if;

  if back=FAIL then
    return(new(invlaplace, _f, var1, var2)) else return(back)
  end_if;

end_proc:


invlaplace::print:=proc(T)
begin
  hold(invlaplace)(extop(T,1), extop(T,2), extop(T,3))
end_proc:


//this part computes the laplacian transform
invlaplace::compute:=proc(_g, var1, var2) local _f, back, typ;
begin
  _f:=_g;
  if not has(_f, var1) then
    return(_f*dirac(var2));
  elif _f=var1 then
    return(dirac(1, var2));
  end_if;

  typ:=type(_f);

  if typ="_equal" then
    return(_equal(invlaplace(op(_f,1), var1, var2), invlaplace(op(_f,2), var1, var2)))
  end_if;


  if typ="_mult" or typ="_plus" then
    back:=IntTrans::linear(new(invlaplace, _f, var1, var2));
    if not (back=FAIL) then
      return(back);
    end_if;
  end_if;

  if typ="function" and type(op(_f,0))="D" then
    _f:=IntTrans::D2diff(_f)
  end_if;

  back:=IntTrans::builtin(new(invlaplace, _f, var1, var2));
  if not (back=FAIL) then
    return(back);
  end_if;

 return(FAIL);
end_proc:

unalias( invlaplace ):
unalias( IntTrans ):

// end of file 
