// Dirk Mueller, olorien, 28/10/96, v1.0 

alias( IntTrans = transform::IntTrans ):
alias( laplace = transform::laplace ):

laplace:=newDomain("transform::laplace"):
laplace::type:="transform::laplace":

laplace::has:=proc() local T, v;
begin
  T:=args(1); v:=args(2);
  bool( v=extop(T,2) or v=extop(T,3) or 
        has(extop(T,1), v) or v=hold(laplace)
  )
end_proc:

laplace::new:=proc(_f, var1, var2)
   local back, dummy;
begin
   if args(0) = 0 then error("no arguments given") end_if;

   if _f::dom::laplace<>FAIL then
     return(_f::dom::laplace(args()))
   end_if ;

   if testargs() then
    if args(0)<>3 then error("wrong number of arguments") end_if;
 
    if type(_f) = "_equal" then
        if not testtype(_f,Type::Arithmetical=Type::Arithmetical) then
            error("invalid 1st argument")
        end_if
    elif not testtype(_f,Type::Arithmetical) then
        error("1st argument must be of 'Type::Arithmetical'")
    end_if;

    if domtype(var1)<>DOM_IDENT then
      error("second argument must be of type DOM_IDENT");
    end_if;
   end_if;

   if domtype(_f)=transform::invlaplace then
     return(subs(extop(_f,1), extop(_f,2)=var2))
   end_if;

   // laplace::compute may use differentiation w.r.t. var2,
   // so make sure, that we can differentiate symbolically:
   if domtype(var2)<> DOM_IDENT 
     then dummy:= genident():
     else dummy:= var2;
   end_if;
    
   back:=laplace::compute(_f, var1, dummy);
 
   if back=FAIL then
     return(new(laplace, _f, var1, var2))
   else
     if dummy <> var2 then
        back:= eval(subs(back, dummy = var2));
     end_if:
     return(back)
   end_if;
end_proc:


laplace::print:=proc(T)
begin
  hold(laplace)(extop(T,1), extop(T,2), extop(T,3));
end_proc:

//this part computes the laplacian transform
laplace::compute:=proc(_g, var1, var2) local _f, back, typ, i, tmp;
begin 

  _f:=_g;
  if not has(_f, var1) then
    return(_f/var2);
  elif _f=var1 then
    return(1/var2^2);
  end_if;

  typ:=type(_f);

  if typ="_equal" then
    return(_equal(laplace(op(_f,1), var1, var2),
            laplace(op(_f,2), var1, var2)))
  end_if;

  if typ="_mult" or typ="_plus" then

    back:=IntTrans::linear(new(laplace, _f, var1, var2));

    // -------------------------------------------------
    // Walter, 4.5.2000
    // If no success, then try again with normalization 
    // inside IntTrans::linear. This is triggered by the
    // second (dummy) argument 0:
    if back = FAIL or hastype(back, "transform::laplace") then
       back:=IntTrans::linear(new(laplace, _f, var1, var2), 0);
    end_if:
    // -------------------------------------------------

    if not (back=FAIL) then
      return(back);
    end_if;
  end_if;

/*
  if typ="_mult" and type(op(_f,1))="_power" and type(op(_f,2))="_plus" then
    if op(op(_f,1),2)<0 then
      back:=_plus(
          laplace( op(op(_f,2), i)*op(_f, 1), var1, var2)
             $ i=1..nops(op(_f,2)));
    end_if;
    if back<>FAIL then return(back) end_if;
  end_if;
*/
  if typ="function" and type(op(_f,0))="D" then
    _f:=IntTrans::D2diff(_f);
  end_if;

  back:=IntTrans::builtin(new(laplace, _f, var1, var2));
  if not (back=FAIL) then
    return(back);
  end_if;

  if typ="_mult" and type(op(_f,1))="_power" and type(op(_f,2))="_plus" then
    tmp:= op(op(_f,1),2);
    if testtype(tmp, Type::Real) then
      if op(op(_f,1),2)<0 then
        back:=_plus(
            laplace( op(op(_f,2), i)*op(_f, 1), var1, var2)
               $ i=1..nops(op(_f,2)));
      end_if;
      if back<>FAIL then
        return(back)
      end_if;
    end_if;
  end_if;

 return(FAIL);
end_proc:


laplace::diff:=proc() local f, e, x, S, vars, var;
begin
 
  // diff(anything) = anything
  if args(0) = 1 then return(args(1)) end_if;

  // analyze the arguments
  f:=args(1); // f = transform::laplace(e, x, S)
  e:= extop(f, 1); // this can be a multivariate expression 
  x:= extop(f, 2); // this is an identifier
  S:= extop(f, 3); // this can be some expression S(some variables)!
                 
  vars:= [args(2..args(0))]; // we are differentiating w.r.t. these vars 

  // diff(transform::laplace(expr, x, S), x) = 0
  if has(vars, x) then return(0) end_if;

  // vars cannot have x any more

  for var in vars do
      e:= -x*diff(S,var)*e + diff(e, var);
  end_for;

  return(laplace(e, x, S)); 
end_proc:


laplace::int:=proc() local _F, _f, _a, _b, _t, var1, var2;
begin
  _F:=args(1); _t:=op(args(2),1); _a:=op(op(args(2),2),1); _b:=op(op(args(2),2),2);
  _f:=extop(_F,1); var1:=extop(_F,2); var2:=extop(_F,3);
  if _t=var2 and _a=0 and _b=infinity then
    return(_F/var1)
  end_if;
  hold(int)(_F, _t=_a.._b);
end_proc:

arctan:= slot( arctan, "transform::invlaplace",
    loadproc(slot( arctan, "transform::invlaplace"),
        pathname("TRANS","LAPLACE"), "IL_atan"
    )
):

_mult:= slot( _mult, "transform::laplace",
    loadproc(slot( _mult, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_mult"
    )
):

_mult:= slot( _mult, "transform::invlaplace",
    loadproc(slot( _mult, "transform::invlaplace"),
        pathname("TRANS","LAPLACE"), "IL_mult"
    )
):

_power:= slot( _power, "transform::laplace",
    loadproc(slot( _power, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_power"
    )
):
_power:= slot( _power, "transform::invlaplace",
    loadproc(slot( _power, "transform::invlaplace"),
        pathname("TRANS","LAPLACE"), "IL_power"
    )
):

sin:= slot( sin, "transform::laplace",
    loadproc(slot( sin, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_sin"
    )
):

sinh:= slot( sinh, "transform::laplace",
    loadproc(slot( sinh, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_sinh"
    )
):

cos:= slot( cos, "transform::laplace",
    loadproc(slot( cos, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_cos"
    )
):

cosh:= slot( cosh, "transform::laplace",
    loadproc(slot( cosh, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_cosh"
    )
):

diff:= slot( diff, "transform::laplace",
    loadproc(slot( diff, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_diff"
    )
):

ln:= slot( ln, "transform::laplace",
    loadproc(slot( ln, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_ln"
    )
):

heaviside:= slot( heaviside, "transform::laplace",
    loadproc(slot( heaviside, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_heavis"
    )
):

exp:= slot( exp, "transform::laplace",
    loadproc(slot( exp, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_exp"
    )
):

int:= slot( int, "transform::laplace",
    loadproc(slot( int, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_int"
    )
):

dirac:= slot( dirac, "transform::laplace",
    loadproc(slot( dirac, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_dirac"
    )
):

besselJ:= slot( besselJ, "transform::laplace",
    loadproc(slot( besselJ, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_bessJ"
    )
):

abs:= slot( abs, "transform::laplace",
    loadproc(slot( abs, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_abs"
    )
):

Ei:= slot( Ei, "transform::laplace",
    loadproc(slot( Ei, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_Ei"
    )
):

unalias( laplace ):
unalias( IntTrans ):

// end of file 
