

//--------------------------------------------------------------------
// MuPAD - Systeminitialisierungsdatei
//--------------------------------------------------------------------

(// trick to suppress all output

//--------------------------------------------------------------------
// Initialize domains for buit-in types
//--------------------------------------------------------------------

old_assign := _assign;
protected(hold(_assign), None);
sysassign( _assign, sysassign );

sysdelete( Dpoly ):   // use polylib::Dpoly 

funcattr:= proc() begin error("Use slot() instead of funcattr()") end_proc:
domattr:=  proc() begin error("Use slot() instead of domattr()") end_proc:

// -------- DOM_ARRAY --------
DOM_ARRAY:= domtype(array(1..1));
DOM_ARRAY::func_call:= proc(arr) name func_call; begin
    map(context(arr),
        proc(elem) begin elem(args(2..args(0))) end_proc,
        context(args(2..args(0)))
    )
end_proc;
DOM_ARRAY::new:= array;
DOM_ARRAY::new_extelement:= proc(d) name new_extelement; begin
    d::new(args(2..args(0)))
end_proc;

// -------- DOM_BOOL --------
DOM_BOOL:= domtype(TRUE);
DOM_BOOL::new:= proc() name new; begin args() end_proc;
DOM_BOOL::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_COMPLEX --------
DOM_COMPLEX:= domtype(I);
DOM_COMPLEX::D := 0 ;
DOM_COMPLEX::new:= proc() name new; begin args(1)+I*args(2) end_proc;
DOM_COMPLEX::new_extelement:= DOM_ARRAY::new_extelement;
 
// -------- DOM_DOMAIN --------
DOM_DOMAIN:= domtype(newDomain("DOM_DOMAIN"));
DOM_DOMAIN::func_call:= _domainfunccall ;
DOM_DOMAIN::new:= newDomain;
DOM_DOMAIN::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_EXEC --------
DOM_EXEC:= domtype(op(_assign, 1));
DOM_EXEC::new:= builtin;
DOM_EXEC::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_EXPR --------
DOM_EXPR:= domtype(XX+1);
DOM_EXPR::testtype:= proc(x,T) name testtype; option noDebug; 
begin
    if domtype(T) = DOM_EXPR then
        if domtype(x) <> DOM_EXPR then
            FALSE
        elif op(x, 0) <> op(T, 0) then
            FALSE
        elif nops(x) <> nops(T) then
            FALSE
        elif nops(x) = 0 then
            TRUE
        elif nops(x) = 1 then
            testtype(op(x, 1), op(T, 1))
        else
            testtype(op(x), Type::Product(op(T)))
        end_if
    else
        case T
        of domtype(x) do
        //of DOM_STRING do
        //of DOM_POLY do
            return( TRUE )
        otherwise
            return( FAIL )
        end_case
    end_if
end_proc;
DOM_EXPR::coerce:= proc(x) 
    local T;
begin
    T:= x::dom;
    case T
    of DOM_STRING do
        return(text2expr(x))
    of DOM_POLY do
        return(op(x,1))
    otherwise
        return(T::convert_to(x,DOM_EXPR))
    end_case
end_proc;
DOM_EXPR::new:= proc() name new; begin args(1)(args(2..args(0))) end_proc;
DOM_EXPR::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_FAIL --------
DOM_FAIL:= domtype(FAIL);
DOM_FAIL::D := proc() name D; begin FAIL end_proc;
DOM_FAIL::new:= proc() name new; begin FAIL end_proc;
DOM_FAIL::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_FLOAT --------
DOM_FLOAT:= domtype(1.0);
DOM_FLOAT::D := 0 ;
DOM_FLOAT::new:= DOM_BOOL::new;
DOM_FLOAT::new_extelement:= DOM_ARRAY::new_extelement;
DOM_FLOAT::coerce:= proc(x)
    local T;
begin
    T:= x::dom;
    case T
    of DOM_STRING do
        return(expr2text(x))
    otherwise
        return(T::convert_to(x,DOM_FLOAT))
    end_case
end_proc:

// -------- DOM_FUNC_ENV --------
DOM_FUNC_ENV:= domtype(_assign);
DOM_FUNC_ENV::new:= funcenv;
DOM_FUNC_ENV::new_extelement:= DOM_ARRAY::new_extelement;
 
// -------- DOM_IDENT --------
DOM_IDENT:= domtype(XX);
DOM_IDENT::new:= DOM_BOOL::new;
DOM_IDENT::new_extelement:= DOM_ARRAY::new_extelement;
DOM_IDENT::coerce:= proc(x)
    local T;
begin
    T:= x::dom;
    case T
    of DOM_STRING do
        return(expr2text(x))
    otherwise
        return(T::convert_to(x,DOM_IDENT))
    end_case
end_proc:

// -------- DOM_INT -------- 
DOM_INT:= domtype(1);
DOM_INT::D := 0 ;
DOM_INT::new:= DOM_BOOL::new;
DOM_INT::new_extelement:= DOM_ARRAY::new_extelement;
DOM_INT::phi:= phi;
sysdelete( phi ):   // use numlib::phi 
DOM_INT::coerce:= proc(x)
    local T;
begin
    T:= x::dom;
    case T
    of DOM_STRING do
        return(expr2text(x))
    of DOM_FLOAT do
        return(float(x))
    otherwise
        return(T::convert_to(x,DOM_INT))
    end_case
end_proc:

// -------- DOM_NIL --------
DOM_NIL:= domtype(NIL);
DOM_NIL::new:= proc() name new; begin NIL end_proc;
DOM_NIL::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_NULL --------
DOM_NULL:= domtype(null());
DOM_NULL::new:= proc() name new; begin null() end_proc;
DOM_NULL::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_POLY --------
DOM_POLY:= domtype(poly(0, [XX]));
DOM_POLY::new:= poly;
DOM_POLY::new_extelement:= DOM_ARRAY::new_extelement;
DOM_POLY::coerce:= proc(x)
    local T, p;
begin
    T:= x::dom;
    case T
    of DOM_EXPR do
        if traperror( (p:= poly(x) ) ) <> 0 then
            return( FAIL )
        else
            // t could be FAIL
            return( p )
        end_if
    of DOM_LIST do
        p:= [op(indets(x))];
        if traperror((p:= poly(x,p))) <> 0 then
            return( FAIL )
        else
            return( p )
        end_if
    of DOM_STRING do
        return(expr2text(x))
    otherwise
        return(T::convert_to(x,DOM_POLY))
    end_case
end_proc:

// -------- DOM_PROC --------
DOM_PROC:= domtype(proc() begin end_proc);
DOM_PROC::new:= _procdef;
DOM_PROC::print :=
proc(p)
  local Name, param;
  option noDebug;
begin 
  param := op(p,1);
  if param = NIL then param := "" else param := expr2text(param) end;
  Name := op(p,6);
  if Name = NIL then Name := "" else Name := " ".expr2text(Name) end;
  "proc".Name."(".param.") ... end"
end_proc:

// -------- DOM_PROC_ENV --------
DOM_PROC_ENV:= domtype(op(
    proc(x) option escape; begin proc() begin x end_proc end_proc(1), 12));
DOM_PROC_ENV::new:= proc() name new; begin FAIL end_proc;
DOM_PROC_ENV::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_VAR --------
DOM_VAR:= domtype(op(proc(x) begin x end_proc, 4));
DOM_VAR::new:= proc(d,i) name new; begin 
    subsop(op(proc(x) begin x end_proc, 4), 1=d, 2=i, Unsimplified);
end_proc;
DOM_VAR::new_extelement:= DOM_ARRAY::new_extelement;
DOM_VAR::func_call:= proc(v) name func_call; begin
    subsop(hold(XX)(context(args(2..args(0)))), 0=v)
end_proc:

// -------- DOM_RAT --------
DOM_RAT:= domtype(1/2);
DOM_RAT::D := 0 ;
DOM_RAT::new:= proc() name new; begin args(1)/args(2) end_proc;
DOM_RAT::new_extelement:= DOM_ARRAY::new_extelement;
DOM_RAT::coerce:= proc(x)
    local T;
begin
    T:= x::dom;
    case T
    of DOM_STRING do
        return(expr2text(x))
    of DOM_FLOAT do
        return(float(x))
    otherwise
        return(T::convert_to(x,DOM_RAT))
    end_case
end_proc:

// -------- DOM_LIST --------
DOM_LIST:= domtype([]);
DOM_LIST::func_call:= DOM_ARRAY::func_call;
DOM_LIST::new:= proc() name new; begin [args()] end_proc;
DOM_LIST::new_extelement:= DOM_ARRAY::new_extelement;
DOM_LIST::coerce:= proc(x)
    local T;
begin
    T:= x::dom;
    case T
    of DOM_SET do
        return( [op(x)] )
    of DOM_ARRAY do
        return( [op(x)] )
    of DOM_POLY do
        return(poly2list(x))
    of DOM_STRING do
        return(expr2text(x))
    otherwise
        return(T::convert_to(x,DOM_LIST))
    end_case
end_proc:

// -------- DOM_SET --------
DOM_SET:= domtype({});
DOM_SET::func_call:= DOM_ARRAY::func_call;
DOM_SET::new:= proc() name new; begin {args()} end_proc;
DOM_SET::new_extelement:= DOM_ARRAY::new_extelement;
DOM_SET::sort:= proc(s : DOM_SET) : DOM_LIST 
  option noDebug;
begin sort([op(s)]) end_proc:
DOM_SET::_index := 
proc(s : DOM_SET, i) 
  option noDebug;
begin 
  if i = 0 then DOM_SET 
  else op(DOM_SET::sort(s), i) end_if;
end_proc:
DOM_SET::_plus:= proc(S:DOM_SET, a)
  local j;
begin
  case domtype(a)
    of DOM_INT do
    of DOM_RAT do
    of DOM_COMPLEX do
    of DOM_FLOAT do       
    of DOM_EXPR do
    of DOM_IDENT do
    of DOM_VAR do
      return(map(S, _plus, a))
    of DOM_SET do
      return(_union(DOM_SET::_plus(S,j) $j in a))
  end_case;
  if a::dom::hasProp(Cat::Set)=TRUE then
    a::dom::inhomog_plus[DOM_SET];
    if type(%)<>"_index" then       
      return(%(a, S))
    end_if       
  end_if;         
  error("Illegal argument ".expr2text(a))
end_proc;
DOM_SET::_mult:= proc(S:DOM_SET, a)
  local j;
begin
  case domtype(a)
    of DOM_INT do
    of DOM_RAT do
    of DOM_COMPLEX do
    of DOM_FLOAT do       
    of DOM_EXPR do
    of DOM_IDENT do
    of DOM_VAR do
      return(map(S, _mult, a))
    of DOM_SET do
      return(_union(DOM_SET::_mult(S,j) $j in a))
  end_case;
  if a::dom::hasProp(Cat::Set)=TRUE then
    a::dom::inhomog_mult[DOM_SET];
    if type(%)<>"_index" then       
       return(%(a, S))
    end_if;       
  end_if;         
  error("Illegal argument '".expr2text(a)."' of type '".expr2text(domtype(a))."'")
end_proc:
DOM_SET::_power:= proc(b, e)
begin
  if domtype(b)= DOM_SET then
    if domtype(e)= DOM_SET then
      return(DOM_SET::bin_power(b, e))
    else
      DOM_SET::inhomogleft_power[domtype(e)];
      if type(%)="_index" then
        return(hold(_power)(b,e))
      else
        return(%(b,e))
      end_if
    end_if
  end_if;
  // domtype(e)=DOM_SET
  DOM_SET::inhomogright_power[domtype(b)];
  if type(%)="_index" then
    return(hold(_power)(b,e))
  else
    return(%(b,e))
  end_if
end_proc:
DOM_SET::bin_power:= proc(b, e)
 local a;
begin
  {(if traperror((a:=i^j))=0 then
     a
   else
     null()
   end_if)
     $i in b $j in e} 
end_proc:
DOM_SET::inhomogleft_power:= table():
DOM_SET::inhomogleft_power[DOM_INT]:=
DOM_SET::inhomogleft_power[DOM_RAT]:=  
DOM_SET::inhomogleft_power[DOM_FLOAT]:=
DOM_SET::inhomogleft_power[DOM_COMPLEX]:=
  (S, e) -> map(S,
                proc(s)
                  local a;
                begin
                  if traperror((a:=s^e))=0 then
                    a
                  else
                    null()
                  end_if
                end_proc):
DOM_SET::inhomogright_power:= table():  
DOM_SET::inhomogright_power[DOM_INT]:=
DOM_SET::inhomogright_power[DOM_RAT]:=  
DOM_SET::inhomogright_power[DOM_FLOAT]:=
DOM_SET::inhomogright_power[DOM_COMPLEX]:=
  (b, S) -> map(S,
                proc(s)
                  local a;
                begin
                  if traperror((a:=b^s))=0 then
                    a
                  else
                    null()
                  end_if
                end_proc):
DOM_SET::coerce:= proc(x)
    local T;
begin
    T:= x::dom;
    case T
    of DOM_LIST do
        return( {op(x)} )
    of DOM_ARRAY do
        return( {op(x)} )
    of DOM_STRING do
        return(expr2text(x))
    otherwise
        return(T::convert_to(x,DOM_SET))
    end_case
end_proc:

// -------- DOM_STRING --------
DOM_STRING:= domtype("");
DOM_STRING::testtype:= proc(x,T) name testtype; option noDebug; local Op0;
begin
  if domtype(T) = DOM_STRING then
    if domtype(x) = DOM_EXPR then
      Op0 := eval(op(x, 0));
      bool(type(x) = T 
           or (T = "function" and domtype(Op0) = DOM_FUNC_ENV
       // std functional output
               and (op(Op0, 2) = NIL or op(Op0, [2,1]) = 1101)))
    else
        FALSE
    end_if;
  elif domtype(x) = T then
    TRUE
  else
    FAIL
  end_if
end_proc;
DOM_STRING::coerce:= x -> expr2text(x):
DOM_STRING::_less := proc(x:DOM_STRING, y:DOM_STRING) name _less; begin
  bool(sort([x, y])=[x, y] and x <> y)
end_proc;
DOM_STRING::_leequal := proc(x:DOM_STRING, y:DOM_STRING) name _leequal; begin
  bool(sort([x, y])=[x, y])
end_proc;
DOM_STRING::new:= DOM_BOOL::new;
DOM_STRING::new_extelement:= DOM_ARRAY::new_extelement;
// for the internal Print methods
DOM_STRING::Print := id:

// -------- DOM_TABLE --------
DOM_TABLE:= domtype(table());
DOM_TABLE::func_call:= DOM_ARRAY::func_call;
DOM_TABLE::new:= table;
DOM_TABLE::new_extelement:= DOM_ARRAY::new_extelement;
DOM_TABLE::coerce:= proc(x)
    local T;
begin
    T:= x::dom;
    case T
    of DOM_STRING do
        return(expr2text(x))
    otherwise
        return(T::convert_to(x,DOM_TABLE))
    end_case
end_proc:

// -------- DOM_POINT --------
DOM_POINT:= domtype(point(1,2,3));
DOM_POINT::new:= point;
DOM_POINT::new_extelement:= DOM_ARRAY::new_extelement;

// -------- DOM_POLYGON --------
DOM_POLYGON:= domtype(polygon(point(1,2,3)));
DOM_POLYGON::new:= polygon;
DOM_POLYGON::new_extelement:= DOM_ARRAY::new_extelement;

//--------------------------------------------------------------------
// only needed for output library                                     
//--------------------------------------------------------------------

_mult_intern := funcenv(_mult_intern):
_mult_intern::type := "_mult_intern":
old_assign::Content := () -> error("not yet implemented"):

//--------------------------------------------------------------------
// Remember values for special functions                              
//--------------------------------------------------------------------

abs(I):= 1: abs(PI):=PI: abs(E):=E: abs(EULER):=EULER;
abs(CATALAN):= CATALAN;

//--------------------------------------------------------------------
// Operator rules for +, *, ^ and @                                   
//--------------------------------------------------------------------

// (f+g)(x) --> f(x)+g(x) 
_plus := slot(_plus, "operator", _op_plusmult) ;

// (f*g)(x) --> f(x)*g(x) 
_mult := slot(_mult, "operator", _op_plusmult) ;

// ((f=g))(x) --> f(x)=g(x) 
_equal := slot(_equal, "operator", _op_plusmult) ;

// (f<g)(x) --> f(x)<g(x) 
_less := slot(_less, "operator", _op_plusmult) ;

// (f<=g)(x) --> f(x)<=g(x) 
_leequal := slot(_leequal, "operator", _op_plusmult) ;

// (f<>g)(x) --> f(x)<>g(x) 
_unequal := slot(_unequal, "operator", _op_plusmult) ;

// (f and g)(x) --> f(x) and g(x) 
_and := slot(_and, "operator", _op_plusmult) ;

// (f or g)(x) --> f(x) or g(x) 
_or := slot(_or, "operator", _op_plusmult) ;

// (not f)(x) --> not f(x) 
_not := slot(_not, "operator", _op_plusmult) ;

// (f^n)(x) --> f(x)^n 
_power := slot(_power, "operator", _op_power) ;

// (f@g)(x) --> f(g(x)) 
_fconcat := slot(_fconcat, "operator", _op_fconcat) ;

//--------------------------------------------------------------------
// Define "float" attributes for basic arithmetic operators           
//--------------------------------------------------------------------

_plus:=  slot(_plus, "float", _float_plus);
_mult:=  slot(_mult, "float", _float_mult);
_power:= slot(_power, "float", _float_power);
sysdelete(_float_plus, _float_mult, _float_power);

_equal:=    slot(_equal,   "float", (x, y)->(float(x) =  float(y)));
_unequal:=  slot(_unequal, "float", (x, y)->(float(x) <> float(y)));
_less:=     slot(_less,    "float", (x, y)->(float(x) <  float(y)));
_leequal:=  slot(_leequal, "float", (x, y)->(float(x) <= float(y)));

_union:= slot(_union, "float",
              proc()
                name float_union;
                local argv;
              begin
                argv:=[args()];
                hold(_union)(op(map(argv, float)))
              end_proc);

_intersect:= slot(_intersect, "float",
              proc()
                name float_intersect;  
                local argv;
              begin
                argv:=[args()];
                hold(_intersect)(op(map(argv, float)))
              end_proc); 

//--------------------------------------------------------------------
// Define shortcuts for basic arithmetic operators                    
//--------------------------------------------------------------------

`+` := hold(_plus):
`-` := hold(_negate):
`*` := hold(_mult):
`/` := hold(_divide):
`^` := hold(_power):
`**` := hold(_power):

//--------------------------------------------------------------------
// Define standard prefix for 'genident'                              
//--------------------------------------------------------------------

genident := funcenv(builtin(1056, NIL, "genident", NIL, "X" ),
		    builtin(1101, 0, NIL, "genident" ),
		    NIL );

//--------------------------------------------------------------------
// Polynomials as operators                                           
//--------------------------------------------------------------------

DOM_POLY::func_call:= proc()		// func_call ueberladen 
    name func_call;
begin
    if args(0) <> nops(op(args(1),2))+1 then
        error("wrong no of args")
    end_if;
    evalp(args(1), op(zip(op(args(1),2), context([args(2..args(0))]), _equal)))
end_proc;

//--------------------------------------------------------------------
// Erzeugung des stdlib-Domains (wird von loadproc benoetigt)	     
// The 'stdlib' library domain serves only as name space for utilies  
//--------------------------------------------------------------------

stdlib:= newDomain("stdlib");
stdlib::info:= "Library 'stdlib': the basic functionality of MuPAD":

stdlib::ifactor:= ifactor: // keep the kernel function 'ifactor'
sysdelete( ifactor ):      // use stdlib::ifactor


//--------------------------------------------------------------------
// Some attributes for kernel functions                               
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Funktionsattribute fuer ``diff''				     
//--------------------------------------------------------------------

id:= slot(id, "diff", 
    proc(f) name diff; begin diff(op(f, 1), args(2..args(0))) end_proc);

_negate:= slot(_negate, "diff",
    proc(f) name diff; begin diff(-op(f,1), args(2..args(0))) end_proc);

_subtract:= slot(_subtract, "diff",
    proc(f) name diff; begin
        diff(op(f,1) - op(f,2), args(2..args(0)))
    end_proc);

_invert:= slot(_invert, "diff",
    proc(f) name diff; begin diff(1/op(f,1), args(2..args(0))) end_proc);

_divide:= slot(_divide, "diff",
    proc(f) name diff; begin
        diff(op(f,1) / op(f,2), args(2..args(0)))
    end_proc);

_equal:= slot(_equal, "diff",
    proc(f) name diff; begin
        _equal( diff(op(f,1),args(2..args(0))),
            diff(op(f,2),args(2..args(0))))
    end_proc);

//--------------------------------------------------------------------
// Funktionsattribute fuer ``_plus''                                  
//--------------------------------------------------------------------

_equal:= slot
 (_equal, "_plus",
  proc(a, b)
    name _plus;
  begin
    case type(b)
      of "_equal" do
        _equal( op(a, 1)+op(b, 1), op(a, 2)+op(b,2) );
        break;
      of "_unequal" do
        _unequal( op(a, 1)+op(b, 1), op(a, 2)+op(b,2) );
        break;      
      of DOM_INT     do
      of DOM_RAT     do
      of DOM_FLOAT   do
      of DOM_COMPLEX do
      of DOM_IDENT   do
        _equal( op(a, 1)+b, op(a, 2)+b);
        break;
      otherwise
        error("")
    end_case;
  end_proc);

_unequal:= slot
 (_unequal, "_plus",
  proc(a, b)
    name _plus;
  begin
    case type(b)
      of "_equal" do
        _unequal( op(a, 1)+op(b, 1), op(a, 2)+op(b,2) );
        break
      of DOM_INT do
      of DOM_RAT do
      of DOM_FLOAT do
      of DOM_COMPLEX do
      of DOM_IDENT do
        _unequal( op(a, 1)+b, op(a, 2)+b);
        break
      otherwise
        error("")
    end_case    
    end_proc);

_leequal:= slot
 (_leequal, "_plus",
  proc(a, b)
    name _plus;
  begin
   case type(b)
     of "_equal" do
     of "_leequal" do  
       _leequal( op(a, 1)+op(b, 1), op(a, 2)+op(b,2) );
       break
     of "_less" do
       _less( op(a, 1)+op(b, 1), op(a, 2)+op(b,2) );
       break
     of DOM_INT do
     of DOM_RAT do
     of DOM_FLOAT do
     of DOM_IDENT do  
       _leequal( op(a, 1)+b, op(a, 2)+b);
       break
     otherwise
       error("")
   end_case
  end_proc);

_less:= slot
 (_less, "_plus",
  proc(a, b)
    name _plus;
  begin
    case type(b)
      of "_equal" do
      of "_leequal" do
      of "_less" do  
       _less( op(a, 1)+op(b, 1), op(a, 2)+op(b,2) );
       break
     of DOM_INT do
     of DOM_RAT do
     of DOM_FLOAT do
     of DOM_IDENT do  
       _less( op(a, 1)+b, op(a, 2)+b);
       break
     otherwise
       error("")
   end_case
  end_proc);

_union:= slot(_union, "_plus",
    proc(a, b) name _plus; begin
        // (A_1 union ... union A_n)+B = (A_1+B) union ... (A_n+B)
        map(a, _plus, b)
    end_proc);

_intersect:= slot(_intersect, "_plus",
    proc(a, b) name _plus; begin
        hold(_plus)(a,b)
    end_proc);

_minus:= slot(_minus, "_plus",
    proc(a, b) name _plus; begin
        hold(_plus)(a,b)
    end_proc);

_and:= slot(_and, "_plus",
    proc(a, b) name _plus; begin
        hold(_plus)(a,b)
    end_proc);

_or:= slot(_or, "_plus",
    proc(a, b) name _plus; begin
        hold(_plus)(a,b)
    end_proc);

_not:= slot(_not, "_plus",
    proc(a, b) name _plus; begin
        hold(_plus)(a,b)
    end_proc);

//--------------------------------------------------------------------
// Funktionsattribute fuer ``_mult''                                  
//--------------------------------------------------------------------

_equal:= slot
 (_equal, "_mult",
  proc(a, b)
    name _mult;

  begin
    case type(b)
      of "_equal" do
        _equal( op(a, 1)*op(b, 1), op(a, 2)*op(b,2) );
        break;
      of DOM_INT     do
      of DOM_RAT     do
      of DOM_FLOAT   do
      of DOM_COMPLEX do
      of DOM_IDENT   do
        _equal( op(a, 1)*b, op(a, 2)*b);
        break;
      of "_power"    do
        case type(op(b, 1))
          of "_equal" do
            _equal( op(a, 1) * op(b, [1,1]) ^ op(b,2),
                   op(a, 2) * op(b ,[1,2]) ^ op(b,2));
            break;
          of DOM_IDENT do
            _equal( op(a, 1) * op(b, 1) ^ op(b,2),
                   op(a, 2) * op(b, 1) ^ op(b,2));
            break;
          otherwise
            error("");
        end_case;
        break
    otherwise
  error("")
  end_case;
  end_proc);

_unequal:= slot(_unequal, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a, b)
    end_proc);

_leequal:= slot(_leequal, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a,b)
    end_proc);

_less:= slot(_less, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a,b)
    end_proc);

_union:= slot(_union, "_mult",
    proc(a, b) name _mult; begin
        map(a, _mult, b)
    end_proc);

_intersect:= slot(_intersect, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a,b)
    end_proc);

_minus:= slot(_minus, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a,b)
    end_proc);

_and:= slot(_and, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a,b)
    end_proc);

_or:= slot(_or, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a,b)
    end_proc);

_not:= slot(_not, "_mult",
    proc(a, b) name _mult; begin
        hold(_mult)(a,b)
    end_proc);

//--------------------------------------------------------------------------
// Kern-Funktionen sichern (werden in specfunc bzw. polylib umdefiniert)    
//--------------------------------------------------------------------------

specfunc:= newDomain("specfunc");
specfunc::info:= "Library 'specfunc': elementary and special functions";

specfunc::sin:= sin;
specfunc::cos:= cos;
specfunc::tan:= tan;
specfunc::arcsin:= arcsin;
specfunc::arccos:= arccos;
specfunc::arctan:= arctan;
specfunc::sinh:= sinh;
specfunc::cosh:= cosh;
specfunc::tanh:= tanh;
specfunc::arcsinh:= arcsinh;
specfunc::arccosh:= arccosh;
specfunc::arctanh:= arctanh;
specfunc::exp:= exp;
specfunc::ln:= ln;
specfunc::sqrt:= sqrt;
specfunc::gamma:= gamma;
specfunc::igamma:= igamma;
specfunc::zeta:= zeta;
specfunc::erfc:= erfc;
specfunc::sign:= sign;
specfunc::abs:= abs;
specfunc::fact:= fact;
specfunc::psi:=psi;
specfunc::lambertV:= funcenv(builtin(1001, 1326, "lambertV", NIL),
		    builtin(1101, 0, NIL, "lambertV" ),
		    NIL );
delete lambertv;
specfunc::lambertW:= funcenv(builtin(1001, 1327, "lambertW", NIL),
		    builtin(1101, 0, NIL, "lambertW" ),
		    NIL );
delete lambertw;
specfunc::Ei:= funcenv(builtin(1001, 1325, "Ei", NIL),
		    builtin(1101, 0, NIL, "Ei" ),
		    NIL );
delete eint;
specfunc::Ci:=Ci;
specfunc::Si:=Si;
specfunc::floor:= floor;
specfunc::ceil:= ceil;
specfunc::frac:= frac;
specfunc::round:= round;
specfunc::trunc:= trunc;

// keep the kernel function 'plot' as a function of the plot library:
sysdelete(plot);
plot:= newDomain("plot"); // see also LIBFILES/plot.mu
plot::_plot:=  builtin(2020, NIL, "plot", NIL):

polylib:=newDomain("polylib");
polylib::lterm:=lterm; 
polylib::lmonomial:=lmonomial;
polylib::lcoeff:=lcoeff;
polylib::tcoeff:=tcoeff;
polylib::nthterm:=nthterm;
polylib::nthcoeff:=nthcoeff;
polylib::nthmonomial:=nthmonomial;
polylib::degreevec:=degreevec;


stdlib::powermod:= powermod;
powermod:=NIL;
stdlib::ithprime:= ithprime;
ithprime:=NIL;
stdlib::max:= max;
stdlib::min:= min;
stdlib::anames:= anames;

stdlib::expr:= expr:
delete expr:

stdlib::ShowPlot        := TRUE:  // Do not delete this initialization!
stdlib::ShowPlotWarning := FALSE: // Needed for plot2d and plot3d

newpurefunc:= () -> error("newpurefunc(...) is no longer available");
newfuncarg:= () -> error("newfuncarg(...) is no longer available");

//--------------------------------------------------------------------
// Anonymous proc to hide aux variables				     
//--------------------------------------------------------------------

proc()
    local path, aux_globals, path_sep, filedescr, lib_path, sysread;
begin

// the global aux variables should not be protected 
aux_globals := { hold(old_assign), hold(XX) };

//--------------------------------------------------------------------
// if tar-lib exists, use it                                          
// since the library is not yet loaded, we have no pathname()         
// therefore we must construct the correct path by hand.              
//--------------------------------------------------------------------

case sysname()
of "UNIX" do
    path_sep := "";
    stdlib::PathSep:= "/";
    path:= LIBPATH;
    break;
of "MACOS" do
    path_sep := ":";
    stdlib::PathSep:= ":";
    path:= LIBPATH.":";
    break;
of "MSDOS" do
    path_sep := "";
    stdlib::PathSep:= "\\";
    path:= LIBPATH;
    break;
end_case;

sysread := proc(filename)
             local filedescr,res;
           begin
             filedescr := fopen(LIBPATH.path_sep.filename.".mb");
             if filedescr <> FAIL then
               res := fread(filedescr, Plain);
               fclose(filedescr);
               return(res);
             end_if;
             filedescr := fopen(LIBPATH.path_sep.filename.".mu");
             if filedescr <> FAIL then
               res := fread(filedescr, Plain);
               fclose(filedescr);
               return(res);
             end_if;
             error("Can't open file'".LIBPATH.path_sep.filename.".{mb,mu}'") 
           end_proc:

lib_path:= LIBPATH;
filedescr := fopen(LIBPATH.path_sep."lib.tar");

if filedescr <> FAIL then
    fclose(filedescr);
    delete filedescr;
    path:= LIBPATH.path_sep."lib.tar#lib";
    LIBPATH := path;
end_if;
sysread("STDLIB".stdlib::PathSep."read");

//--------------------------------------------------------------------
// Initialisierung der vordefinierten Standardprozeduren, die bei     
// Bedarf automatisch nachgeladen werden sollen.                      
//--------------------------------------------------------------------

stdlib::LoadlibState:= table();

// use library patch archive first if one exists 

filedescr := fopen(lib_path.path_sep."plib.tar");
if filedescr <> FAIL then
    fclose(filedescr);
    delete filedescr;
    LIBPATH := lib_path.path_sep."plib.tar#plib", LIBPATH;
    sysread("STDLIB".stdlib::PathSep."patchlev");
else
    patchlevel:= proc() name patchlevel; begin 0 end_proc;
end_if;

sysread("STDLIB".stdlib::PathSep."alias");
sysread("STDLIB".stdlib::PathSep."unalias");
sysread("STDLIB".stdlib::PathSep."pathname");
sysread("STDLIB".stdlib::PathSep."loadproc");

sysread("LIBFILES".stdlib::PathSep."Pref");
sysread("PREF".stdlib::PathSep."verbread");

sysread("LIBFILES".stdlib::PathSep."stdlib");
sysread("LIBFILES".stdlib::PathSep."misc");
sysread("LIBFILES".stdlib::PathSep."specfunc");
sysread("LIBFILES".stdlib::PathSep."polylib");
sysread("LIBFILES".stdlib::PathSep."plot");

delete sysread;

//--------------------------------------------------------------------
// initialize packages                                                
//--------------------------------------------------------------------

path:= pathname("LIBFILES");

Ax       := loadproc(Ax,        path, "Ax");
Cat      := loadproc(Cat,       path, "Cat");
Dom      := loadproc(Dom,       path, "Dom");
Network  := loadproc(Network,   path, "Network");
RGB      := loadproc(RGB,       path, "RGB");
Series   := loadproc(Series,    path, "Series");
property := loadproc(property,  path, "property");
matchlib := loadproc(matchlib,  path, "matchlib");
Simplify := loadproc(Simplify,  path, "Simplify");
Type     := loadproc(Type,      path, "Type");
adt      := loadproc(adt,       path, "adt");
combinat := loadproc(combinat,  path, "combinat");
detools  := loadproc(detools,	path, "detools");
domains  := loadproc(domains,   path, "domains");
faclib   := loadproc(faclib,    path, "faclib");
fp       := loadproc(fp,        path, "fp");
gcdlib   := loadproc(gcdlib,    path, "gcdlib");
generate := loadproc(generate,  path, "generate");
groebner := loadproc(groebner,  path, "groebner");
intlib   := loadproc(intlib,    path, "intlib");
import   := loadproc(import,    path, "import");
linalg   := loadproc(linalg,    path, "linalg");
listlib  := loadproc(listlib,   path, "listlib");
linopt   := loadproc(linopt,    path, "linopt");
module   := loadproc(module,    path, "module");
numeric  := loadproc(numeric,   path, "numeric");
numlib   := loadproc(numlib,    path, "numlib");
ode      := loadproc(ode,       path, "ode");
orthpoly := loadproc(orthpoly,  path, "orthpoly");
output   := loadproc(output,    path, "output");
prog     := loadproc(prog,      path, "prog");
rec      := loadproc(rec,       path, "rec");
solvelib := loadproc(solvelib,  path, "solvelib");
stats    := loadproc(stats,     path, "stats");
stringlib:= loadproc(stringlib, path, "stringlib");
student  := loadproc(student,   path, "student");
transform:= loadproc(transform, path, "transform");
plotlib  := loadproc(plotlib,   path, "plotlib"); // Note: not longer supported! See 'plot'.

//////////////////////////////////////////////////////////////////////
//       initialize basic sets 
//////////////////////////////////////////////////////////////////////

Z_:= loadproc(Z_, path,"solvelib"):
Q_:= loadproc(Q_, path,"solvelib"):
R_:= loadproc(R_, path,"solvelib"):
C_:= loadproc(C_, path,"solvelib"):
 

// typesetting library
Content  := loadproc(Content, pathname("OUTPUT"), "Content"):
MathContent  := loadproc(MathContent, pathname("OUTPUT"), "MathContent"):

// wurden bisher mit 'domains' geladen: 
path:= pathname("DOMAINS", "CONSTR");
_constructor:=	      loadproc(_constructor,        path, "constr");
DomainConstructor:=   loadproc(DomainConstructor,   path, "DomCons");
Category:=            loadproc(Category,            path, "Category");
CategoryConstructor:= loadproc(CategoryConstructor, path, "CatCons");
Axiom:=               loadproc(Axiom,               path, "Axiom");
AxiomConstructor:=    loadproc(AxiomConstructor,    path, "AxCons");

//--------------------------------------------------------------------
//        Methods for basic data types                                
//--------------------------------------------------------------------
path:= pathname("INTLIB");
DOM_POLY::int:= loadproc( DOM_POLY::int, path, "DOM_POLY" ):

_intersect():=universe: 
/* not possible earlier because stdlib must be loaded first */


//--------------------------------------------------------------------
// Warnung bei unbekannten globalen Variablen			     
//--------------------------------------------------------------------

_check_global:= proc(v, p) option hold, noDebug; local s; begin
    if contains({hold(ERRORLEVEL),hold(PRINTLEVEL),hold(EVAL_STMT),
                 hold(LIB_PATH), hold(READ_PATH), hold(WRITE_PATH), 
                 hold(PRETTY_PRINT)}, v)
    then s:= "Obselete environment variable '".v."' used"
    elif v = hold(HISTORY)
    then s:= "HISTORY may no longer be changed for procedures"
    elif contains({hold(fun),hold(func),hold(newpurefunc),
        hold(newfuncarg)}, v)
    then s:= "Pure functions no longer available [".v."]"
//  Bleibt dieser Zweig drin, so wird die ueberfluessige Warnung:
//
//     >> proc(y) begin (x->x^2)(y) end
//     Warning: Unknown global identifier 'x'
//
//  ezeugt. 
/*  elif protected(v) <> hold(Error) and
    not contains(indexval(stdlib, "ENVIRONMENT_VARIABLES"), v) and
    not contains(indexval(stdlib, "SYSTEM_CONSTANTS"), v) and
    not contains(indexval(stdlib, "LIBRARY_CONSTANTS"), v) and
    not contains(indexval(stdlib, "OPTIONS"), v) and
    (_parser_config() = null() or not contains(_parser_config()[3],v)) and
    not contains(indexval(Type, "interface"),v)
    then s:= "Unknown global identifier '".v."'" 
*/
    else return(null())
    end_if;
    if p <> NIL then
        s:= s." in procedure '".expr2text(p)."'"
    end;
    warning(s);
end_proc:

//--------------------------------------------------------------------
// path definitions for kernel secure mode
//--------------------------------------------------------------------

//if _pref(SecureKernel) then // andi, 12.10.2000 (future feature)
//    WRITEPATH:="/tmp":
//end_if:

//--------------------------------------------------------------------
// initialize modules                                                 
//--------------------------------------------------------------------

//if loadmod() = TRUE then
//   stdmod  := hold(module)("stdmod"):
//   util    := hold(module)("util"  ):
//
//   if sysname() = "UNIX" then
//      cdebug  := hold(module)("cdebug"):
//   end_if:
//end_if:

//--------------------------------------------------------------------
// Gewaehrleistung der Kompatibilitaet zu SWP/SNB 3.5.x
//--------------------------------------------------------------------

Plot := newDomain("Plot"):
Plot::riemann := proc()
  local i,r;
begin
  r := student::plotRiemann(args());
  return( plot::Group(r[3], r[2], r[1], r[i] $ i=4..nops(r)) )
end_proc:
Plot::show := r -> 
  plot( plot::Group(r[3], r[2], r[1], r[i] $ i=4..nops(r)) ):


//--------------------------------------------------------------------
// Remember protected identifiers				     
//--------------------------------------------------------------------

stdlib::PROTECTED := 
    (stdlib::anames(3)
      minus indexval(stdlib, "ENVIRONMENT_VARIABLES")
      minus indexval(stdlib, "SYSTEM_CONSTANTS")
      minus aux_globals)
      union indexval(stdlib, "LIBRARY_CONSTANTS")
      union indexval(stdlib, "OPTIONS"):

map((stdlib::anames(3)
      minus indexval(stdlib, "ENVIRONMENT_VARIABLES")
      minus indexval(stdlib, "SYSTEM_CONSTANTS")
      minus aux_globals)
      union indexval(stdlib, "LIBRARY_CONSTANTS"),
    protected, Error);

map(indexval(stdlib, "OPTIONS"), protected, Warning);

end_proc(); //------------------------- end of anonymous aux proc 

sysassign( _assign, old_assign ):
sysdelete( old_assign ):

//--------------------------------------------------------------------
// Init random generator          				     
//--------------------------------------------------------------------
SEED:= 1:

//--------------------------------------------------------------------
// Versionsnummern von Kern und Library vergleichen.                  
//--------------------------------------------------------------------

if( _pref(hold(Kernel)) <> version() ) then
    print( Unquoted, 
           "Warning: Kernel and library release number differ!\n".
           "Kernel : ".expr2text(_pref(hold(Kernel)))."\n".
           "Library: ".expr2text(version())."\n"
    );
end_if;

//--------------------------------------------------------------------
// only for beta test:  activate prog::changes() after protection
//--------------------------------------------------------------------
//prog::changes(Warning, Quiet):
sysassign(Factor,() -> (warning("this function is obsolete, use 'factor' instead!"); factor(args()))):
protect(Factor,Error):

//--------------------------------------------------------------------
// only for first releases of MuPAD 2.0: raise an error message
// if Factor is called instead of factor.
// !!! UNCOMMENT THE FOLLOWING TWO LINES !!!
// !!! IF prog::changes IS ACTIVATED     !!!
//--------------------------------------------------------------------
sysassign(Factor,() -> error("call 'factor' instead!")):
protect(Factor,Error):

):// trick to suppress any output

//- the end ----------------------------------------------------------
