/* $Date: 2000/05/08 11:15:33 $  $Author: kg $  $Revision: 1.16 $ */

/*
 * MAM_ass_mem.h
 * In diesem File sind Konstanten und Makros definiert, die von den
 * Funktionen fuer assoziativen Speicher benoetigt werden.
 * Erstellungsdatum  : 22/08/91
 * Letzte Aenderung  : 16/07/92
 * Autor             : Holger Naundorf
 */

#ifndef __MAM_ass_mem__
#define __MAM_ass_mem__

/** Get number of entries of assoziative memory.
 * @param aM s-pointer to assoziative memory
 * @return number of entries in #aM#
 */
inline long MAM_number_of_entries(S_Pointer aM)
{
    return MMMsignature(aM);
}

/*
 * Im folgenden werden die Funktionen deklariert, die vom assoziativen
 * Speicher als Schnittstelle bereitgestellt werden.
 */
S_Pointer MAM_create      ( long size = 0 ) ;
void      MAM_free        ( S_Pointer *aM );
bool      MAM_in          ( S_Pointer  aM, S_Pointer arg, S_Pointer  *erg );
bool      MAM_insert      ( S_Pointer *aM, S_Pointer arg, S_Pointer   erg );
bool      MAM_delete      ( S_Pointer *aM, S_Pointer arg );
S_Pointer MAM_content     ( S_Pointer  aM );
void      MAM_cond_insert ( S_Pointer *aM, S_Pointer arg, S_Pointer erg );
void	  MAM_filter	  ( S_Pointer *aM,
			    bool (*)(S_Pointer, S_Pointer, void *), void *);


/** Iterates over the entries of an assoziative memory.
 *
 * The contents of the memory may be read only using an iterator.
 * One must not change the assoziative memory while iterating over it!
 */

class MAMT_iterator {
public:
    /** Create iterator and attach it to the associative memory #aM#.
     * Moves the iterator to the first entry of the memory.
     * @param aM s-pointer to associative memory */
    MAMT_iterator(S_Pointer aM=MMMNULL);

    /// Destroy iterator.
    ~MAMT_iterator();

    /** Attach iterator to the associative memory #aM#.
     * Moves the iterator to the first entry of the memory.
     * @param aM s-pointer to associative memory */
    void attach(S_Pointer aM);

    /// Detach iterator from the associative memory.
    void detach();

    /** Moves the iterator to the next entry of the memory.
     * @return false if no more entries exist. */
    bool next();

    /** Test if iterator is located behind last entry.
     * @return true if no more entries exist to iterate over. */
    bool at_end();

    /** Get key of actual entry.
     * The s-pointer returned is not copied!
     * @return a s-pointer to the actual entries key */
    S_Pointer key();

    /** Get value of actual entry.
     * The s-pointer returned is not copied!
     * @return a s-pointer to the actual entries value */
    S_Pointer value();

private:
    // Move to next non-empty bucket list. Return true if any exists.
    bool next_first();

    S_Pointer mem_;        	// the associative memory
    S_Pointer* elem_;	        // the actual entry
    long hash_;                 // the actual slot
    long left_;	        	// no of entries in actual bucket list left
    bool is_asslst_;            // is associative memory stored as list?
};

inline bool MAMT_iterator::next()
{
    if (mem_ == MMMNULL || elem_ == 0)
	return false;

    if (left_ != 0 && *(elem_+2) != MMMNULL)
    {
	left_ -= 2;
	elem_ += 2;
	return true;
    }
    if (is_asslst_)
    {
        elem_ = 0;
        left_ = 0;
        return false;
    }
    hash_++;
    return next_first();
}
inline bool MAMT_iterator::at_end()
{
    return elem_ == 0;
}
inline S_Pointer MAMT_iterator::key()
{
    assert(elem_ != 0);
    return *elem_;
}
inline S_Pointer MAMT_iterator::value()
{
    assert(elem_ != 0);
    return *(elem_+1);
}

#endif /* __MAM_ass_mem__ */
