#ifndef __MAS_assoc_h__
#define __MAS_assoc_h__

/*
 * MAS_assoc.h -- Assoziations-Listen
 *
 * Assoziations-Listen sind dynamisch verkettete Listen, die
 * einem Schluessel einen Wert zuordnen.
 */

#include "MMMstorage_lean.h"

/* Fuegt eine neue Assoziation in eine vorhandenen ein, indem das Paar
 * (key,val) vorne in die Liste list eingetragen werden.
 * Es wird nicht geprueft, ob der Schluessel key bereits vorhanden ist!
 * Keines der Argumente wird kopiert! */
S_Pointer MAS_insert_at_front(S_Pointer key, S_Pointer val, S_Pointer list);

/* Liefert den Schluessel des ersten Elementes einer Assoziations-Liste.
 * Der Rueckgabewert ist nicht kopiert! */
S_Pointer MAS_get_key(S_Pointer list);

/* Liefert den Wert des ersten Elementes einer Assoziations-Liste.
 * Der Rueckgabewert ist nicht kopiert! */
S_Pointer MAS_get_value(S_Pointer list);

/* Liefert das naechste Element der Assoziations-Liste, auf die das
 * gegebene Argument verweist (kann MMMNULL sein).
 * Der Rueckgabewert ist nicht kopiert! */
S_Pointer MAS_get_next(S_Pointer list);

/* Loescht die nachfolgenden Elemente der Assoziations-Liste als
 * Seiteneffekt. */
void MAS_delete_remainder(S_Pointer list);

//----------------------- Implementation --------------------------------------

inline S_Pointer MAS_get_key(S_Pointer list)
{
    S_Pointer res = *MMMP(&list, 0);
    return res;
}

inline S_Pointer MAS_get_value(S_Pointer list)
{
    S_Pointer res = *MMMP(&list, 1);
    return res;
}

inline S_Pointer MAS_get_next(S_Pointer list)
{
    S_Pointer res = *MMMP(&list, 2);
    return res;
}

inline void MAS_delete_remainder(S_Pointer list)
{
    // die Signatur haengt nicht vom 3ten Eintrag ab, daher
    // koennen wir in place loeschen.
    MMMfree(*MMMP(&list, 2));
    *MMMP(&list, 2) = MMMNULL;
}

#endif /* __MAS_assoc_h__ */

