#ifndef __MCL_classic_dbg_switch_h__
#define __MCL_classic_dbg_switch_h__

#define MDBC_IN_OUT_FORMAT "%c %s %ld %s\n"

/** Classic client switch.
 *
 * Classic client switch implementing ASCII i/o and debugging protocol
 * a'la' OLDCOMMUNICATION (via files).
 */
class MCLT_classic_dbg_switch : public MCLT_classic_switch {
public:
    /// Create new switch object.
    MCLT_classic_dbg_switch();

    /// Destroy switch.
    virtual ~MCLT_classic_dbg_switch();

    /// Query if client really wants to stop command execution.
    virtual bool ack_stop(MCLT_client* client);

    /// Called if debugging mode is stopped by client.
    virtual void on_stop_debug(MCLT_client* client);

    //@name Sending Messages to the Client
    /**
     * These methods return true if the message was sent successfully.
     */
    //@{
    /// Send debug command wanted message.
    virtual bool send_dbg_cmd_wanted(MCLT_client* client, void* cmd_cookie);

    /// Send finished message.
    virtual bool send_finished(MCLT_client* client, void* cmd_cookie);

    /// Send debug information message.
    virtual bool send_dbg_info(MCLT_client* client, unsigned char cmd, const char* fname,
                               long pos, const char* procname, long lev, long depth);

    /// Send debug display list message.
    virtual bool send_display_list(MCLT_client* client, size_t num_vars, const char* names[],
                                   const MTmemoryData* values[]);

    /// Send debug breakpoint list message.
    virtual bool send_breakpoint_list(MCLT_client* client, size_t num_bpts, const char* fnames[],
                                      long lines[], const char* conditions[]);

    /// Send debug call stack message.
    virtual bool send_call_stack(MCLT_client* client, size_t level, size_t num_calls,
                                 const char* fnames[], long lines[], const char* calls[]);
    //@}

protected:
    virtual void init_dbg();
    virtual void await_command(MCLT_client* client);
    virtual bool await_dbg_cmd(MCLT_client* client);
    virtual bool send_dbg_prompt();
    virtual void read_command() = 0;

    virtual void deactivate_dbg_ui();
    virtual bool open_dbg_comm_file(const char* fname);
    virtual void close_dbg_comm_file(bool info);
    virtual void write_dbg_info(unsigned char cmd, const char* fname, long pos,
                                const char* procname);
    virtual void write_dbg_display_var(const char* var, const char* val);

    char dbg_next_symbol(const char* input, unsigned long& pos, char delimiter);
    void dbg_scan_to_eol(const char* input, unsigned long& pos);
    bool parse_dbg_cmd(MCLT_client* client, const char* input);

    MTheapStream input_;
    bool dbg_initialized_;
    char stop_file_[256];
    char display_file_[256];
    unsigned char dbg_last_cmd_;
    unsigned long dbg_scan_num_;
    char dbg_var_[MDBC_BUF_SIZE];
    MCLT_iostream dbg_comm_file_;
    int dbg_pid_;
};

#endif
