#ifndef __MCL_classic_switch_h__
#define __MCL_classic_switch_h__

/** Classic client switch.
 *
 * Classic client switch implementing ASCII i/o.
 */
class MCLT_classic_switch : public MCLT_switch {
public:
    /// Create new switch object.
    MCLT_classic_switch();

    /// Destroy switch.
    virtual ~MCLT_classic_switch();

    /// Create new single client.
    virtual bool new_client();

    /// Remove our single client.
    virtual void remove_client();

    /// Initialize kernel.
    virtual void do_init(MCLT_client* client, size_t argc, char* argv[]);

    /// Evaluate command.
    virtual void do_cmd(MCLT_client* client, void* cookie, MTmemoryData* cmd,
                        MTexecutionFlag flags, const char* format);

    /// Evaluate plot command.
    virtual void do_plot_cmd(MCLT_client* client, void* cookie, MTmemoryData* cmd);

    /// Parse input.
    S_Pointer do_input(MCLT_client* client, bool canceled, MTmemoryData* input);

    /// Called after a command has been executed.
    virtual void cmd_done(MCLT_client* client);

    /// Called after a plot command has been executed.
    virtual void plot_cmd_done(MCLT_client* client);

    /// Called after a debug command has been executed.
    virtual void dbg_cmd_done(MCLT_client* client);

    //@name Sending Messages to the Client
    /**
     * The class implements this methods by writing the messages to
     * the client. All output must be linear or 2d ascii text.
     *
     * These methods return true if the message was sent successfully.
     */
    //@{
    /// Send client id message.
    virtual bool send_client_id(MCLT_client* client);

    /// Send kernel ready message.
    virtual bool send_kernel_ready(MCLT_client* client);

    /// Send finished message.
    virtual bool send_finished(MCLT_client* client, void* cmd_cookie);

    /// Sent input wanted message.
    virtual bool send_input_wanted(MCLT_client* client, void* cmd_cookie, bool text_input,
                                   MTmemoryData* prompt);

    /// Sent callback message.
    virtual bool send_callback(MCLT_client* client, void* cmd_cookie, MTmemoryStream* call);

    /// Send select format message.
    virtual bool send_select_format(MCLT_client* client, const char* output_type,
                                    size_t num_formats, const MTformat* formats);

    /// Send input completion message
    virtual bool send_input_completion(MCLT_client* client, const char* completionList, const char* completion);

    /** Return value of last completion / only needed for TTY.
     * The string returned must be freed by caller. */
    virtual char *completion();

    /** Return value of last completion list / only needed for TTY.
     * The string returned must be freed by caller. */
    virtual char *all_possible_completions();

    /// Send debug command wanted message.
    virtual bool send_dbg_cmd_wanted(MCLT_client* client, void* cmd_cookie);

    /// Send debug position message.
    virtual bool send_dbg_info(MCLT_client* client, unsigned char cmd, const char* fname,
                               long pos, const char* procname, long lev, long depth);

    /// Send debug display list message.
    virtual bool send_display_list(MCLT_client* client, size_t num_vars, const char* names[],
                                   const MTmemoryData* values[]);

    /// Send debug breakpoint list message.
    virtual bool send_breakpoint_list(MCLT_client* client, size_t num_bpts, const char* fnames[],
                                      long lines[], const char* conditions[]);

    /// Send debug call stack message.
    virtual bool send_call_stack(MCLT_client* client, size_t level, size_t num_calls,
                                 const char* fnames[], long lines[], const char* calls[]);

    /// Send init wanted message.
    virtual bool send_init_wanted(MCLT_client* client);

    /// Send reset message.
    virtual bool send_reset(MCLT_client* client);

    /// Send quit message.
    virtual bool send_quit();
    //@}

protected:
    virtual void show_banner() = 0;
    virtual void await_command(MCLT_client* client) = 0;
    virtual void await_input(MCLT_client* client, const char* prompt) = 0;
    virtual void await_textinput(MCLT_client* client, const char* prompt) = 0;
    virtual bool send_prompt() = 0;
    bool post_message(MCLT_mssg* mssg);
};

#endif
