#ifndef __MCL_dde_switch_h__
#define __MCL_dde_switch_h__

#include "MCL_classic_switch.h"

/** Windows DDE protocol client switch.
 *
 * Client switch implementing i/o for old (pre 1.5) DDE clients.
 */
class MCLT_dde_switch : public MCLT_classic_switch {
public:
    /// Create new tty switch object.
    MCLT_dde_switch();

    /// Destroy switch.
    virtual ~MCLT_dde_switch();

    /// Create new single DDE client.
    virtual bool new_client();

    /// Remove our single client. Will eventually exit the kernel if stay_online=false.
    virtual void remove_client(bool stay_online);

    /// Send plotting data stored in S-Pointer to UI
    virtual bool put_plot_data(MCLT_client* client, S_Pointer data);

    //@name Sending Messages to the Terminal
    /**
     * The class implements this methods by writing the messages to
     * the terminal.
     *
     * These methods return true if the message was sent successfully.
     */
    //@{
    /// Send output message.
    virtual bool send_output(MCLT_client* client, void* cmd_cookie, const char* output_type,
                             MTmemoryData* output);

    /// Send help message.
    virtual bool send_help(MCLT_client* client, const char* topic);

    /// Send system message.
    virtual bool send_system(MCLT_client* client, const char* cmd);

    /// Send textwidth changed message.
    virtual bool send_textwidth_changed(MCLT_client* client, long width);

    /// Send pretty-print changed message.
    virtual bool send_pretty_print_changed(MCLT_client* client, bool pretty);

    /// Send format changed message.
    virtual bool send_format_changed(MCLT_client* client, const char* output_type,
                                     const MTformat* format);

    /// Send status message.
    virtual bool send_status(MCLT_client* client);

    /// Send plot result file message
    virtual bool send_plot_result_file(MCLT_client* client, void* cmd_cookie, const char* fname);
    //@}

protected:
    virtual void show_banner();
    virtual void await_command(MCLT_client* client);
    virtual void await_input(MCLT_client* client, const char* prompt);
    virtual void await_textinput(MCLT_client* client, const char* prompt);
    virtual bool send_prompt();
    virtual bool quit_without_clients();

    bool stay_online_;
};

#endif
