#ifndef __MCL_mac_switch_h__
#define __MCL_mac_switch_h__

#include "MCL_classic_dbg_switch.h"

/** Mac client switch.
 *
 * Client switch implementing i/o for Mac MuPAD.
 */
class MCLT_mac_switch : public MCLT_classic_dbg_switch {
public:
    /// Create new tty switch object.
    MCLT_mac_switch();

    /// Destroy switch.
    virtual ~MCLT_mac_switch();

    /// Test if client has a certain feature.
    virtual bool has_feature(const MCLT_client* client, MTclientFeature feature) const;

    /// Send plotting data stored in S-Pointer to UI
    virtual bool put_plot_data(MCLT_client* client, S_Pointer data);

    /// Query if client really wants to stop command execution.
    virtual bool ack_stop(MCLT_client* client);

    //@name Sending Messages to the Terminal
    /**
     * The class implements this methods by writing the messages to
     * the terminal.
     *
     * These methods return true if the message was sent successfully.
     */
    //@{
    /// Send output message.
    virtual bool send_output(MCLT_client* client, void* cmd_cookie, const char* output_type,
                             MTmemoryData* output);

    /// Send help message.
    virtual bool send_help(MCLT_client* client, const char* topic);

    /// Send textwidth changed message.
    virtual bool send_textwidth_changed(MCLT_client* client, long width);

    /// Send format changed message.
    virtual bool send_format_changed(MCLT_client* client, const char* output_type,
                                     const MTformat* format);

    /// Send system message (shell-command)
	virtual bool send_system(MCLT_client*,const char*)		{ return false; }

    /// Send status message.
    virtual bool send_status(MCLT_client* client);

    /// Send input completion message
    virtual bool send_input_completion(MCLT_client* client, const char* completionList, const char* completion);

	/// Send plot result file message
    virtual bool send_plot_result_file(MCLT_client* client, void* cmd_cookie, const char* fname);

    /// Send debug breakpoint list message.
    virtual bool send_breakpoint_list(MCLT_client* client, size_t num_bpts, const char* fnames[],
                                      long lines[], const char* conditions[]);

    /// Send debug call stack message.
    virtual bool send_call_stack(MCLT_client* client, size_t level, size_t num_calls,
                                 const char* fnames[], long lines[], const char* calls[]);
    //@}

    bool post_message(MCLT_mssg* mssg);

private:
    virtual void show_banner();
    virtual void init_dbg();
    virtual void await_input(MCLT_client* client, const char* prompt);
    virtual void await_textinput(MCLT_client* client, const char* prompt);
    virtual bool await_dbg_cmd(MCLT_client* client);
    virtual bool send_prompt();
    virtual bool send_long_prompt();
    virtual bool send_dbg_prompt();
    virtual void read_command();

    virtual bool open_dbg_comm_file(const char* fname);
    virtual void close_dbg_comm_file(bool info);
    virtual void write_dbg_info(unsigned char cmd, const char* fname, long pos,
                                const char* procname);
    virtual void write_dbg_display_var(const char* var, const char* val);
};

#endif
