#ifndef __MCL_mxmupad_switch_h__
#define __MCL_mxmupad_switch_h__

/** Motiv MuPAD client switch.
 *
 * Client switch implementing i/o for UNIX Motiv frontend.
 */
class MCLT_mxmupad_switch : public MCLT_classic_dbg_switch {
public:
    /// Create new tty switch object.
    MCLT_mxmupad_switch();

    /// Destroy switch.
    virtual ~MCLT_mxmupad_switch();

    /// Test if client has a certain feature.
    virtual bool has_feature(const MCLT_client* client, MTclientFeature feature) const;

    /// Send plotting data stored in S-Pointer to UI
    virtual bool put_plot_data(MCLT_client* client, S_Pointer data);

    /// Refresh debugger display variable list.
    virtual void refresh_display_vars(MCLT_client* client);

    /// Refresh debugger breakpoint list.
    virtual void refresh_breakpoints(MCLT_client* client);

    /// Refresh debugger call stack list.
    virtual void refresh_call_stack(MCLT_client* client);

   /// Query if client really wants to stop command execution.
   virtual bool ack_stop(MCLT_client* client);
    /// Called if debugging mode is stopped by client.
    virtual void on_stop_debug(MCLT_client* client);

    //@name Sending Messages to the Terminal
    /**
     * The class implements this methods by writing the messages to
     * the terminal.
     *
     * These methods return true if the message was sent successfully.
     */
    //@{
    /// Send output message.
    virtual bool send_output(MCLT_client* client, void* cmd_cookie, const char* output_type,
                             MTmemoryData* output);

    /// Send help message.
    virtual bool send_help(MCLT_client* client, const char* topic);

    /// Send system message.
    virtual bool send_system(MCLT_client* client, const char* cmd);

    /// Send textwidth changed message.
    virtual bool send_textwidth_changed(MCLT_client* client, long width);

    /// Send pretty-print changed message.
    virtual bool send_pretty_print_changed(MCLT_client* client, bool pretty);

    /// Send format changed message.
    virtual bool send_format_changed(MCLT_client* client, const char* output_type,
                                     const MTformat* format);

    /// Send input completion(s).
    virtual bool send_input_completion(MCLT_client* client, const char* completionList,
				       const char* completion);

    /// Send status message.
    virtual bool send_status(MCLT_client* client);

    /// Send plot result file message
    virtual bool send_plot_result_file(MCLT_client* client, void* cmd_cookie, const char* fname);

    virtual bool send_display_list(MCLT_client*, size_t num_vars,
	            const char* names[], const MTmemoryData* values[]);

    virtual bool send_breakpoint_list(MCLT_client* client, size_t num_bpts, const char* fnames[],
		long lines[], const char* conditions[]);

    /// Send debug call stack message.
    bool send_call_stack(MCLT_client* client, size_t level, size_t num_calls,
                 const char* fnames[], long lines[], const char* calls[]);

    /// Send debug information message.
    virtual bool send_dbg_info(MCLT_client* client, unsigned char cmd, const char* fname,
                               long pos, const char* procname, long lev, long depth);
    //@}

private:
    virtual void show_banner();
    virtual void init_dbg();
    virtual void await_command(MCLT_client* client);
    virtual bool await_dbg_cmd(MCLT_client* client);
    virtual void await_input(MCLT_client* client, const char* prompt);
    virtual void await_textinput(MCLT_client* client, const char* prompt);
    virtual bool send_prompt();
    virtual bool send_dbg_prompt();
    virtual void read_command();
    virtual void deactivate_dbg_ui();

    MPRTtype input_type_;
    bool dbg_initialized_;
    int dbg_pid_;
};

#endif
