#ifndef __MCL_tty_switch_h__
#define __MCL_tty_switch_h__

/** ASCII terminal client switch.
 *
 * Client switch implementing terminal i/o for UNIX kernels.
 */
class MCLT_tty_switch : public MCLT_classic_dbg_switch {
public:
    /// Create new tty switch object.
    MCLT_tty_switch();

    /// Destroy switch.
    virtual ~MCLT_tty_switch();

    /// Test if client has a certain feature.
    virtual bool has_feature(const MCLT_client* client, MTclientFeature feature ) const;

    /// Evaluate command.
    virtual void do_cmd(MCLT_client* client, void* cookie, MTmemoryData* cmd, 
                        MTexecutionFlag flags, const char* format);

    /// Send plotting data stored in S-Pointer to UI
    virtual bool put_plot_data(MCLT_client* client, S_Pointer data);

    /// Query if client really wants to stop command execution.
    virtual bool ack_stop(MCLT_client* client);

    //@name Sending Messages to the Terminal
    /**
     * The class implements this methods by writing the messages to
     * the terminal.
     *
     * These methods return true if the message was sent successfully.
     */
    //@{
    /// Send output message.
    virtual bool send_output(MCLT_client* client, void* cmd_cookie, const char* output_type,
                             MTmemoryData* output);

    /// Send a finished message.
    virtual bool send_finished(MCLT_client* client, void* cmd_cookie);

    /// Send complete command message.
    virtual bool send_complete_cmd(MCLT_client* client, void* cmd_cookie);

    /// Send help message.
    virtual bool send_help(MCLT_client* client, const char* topic);

    /// Send system message.
    virtual bool send_system(MCLT_client* client, const char* cmd);

    /// Send textwidth changed message.
    virtual bool send_textwidth_changed(MCLT_client* client, long width);

    /// Send pretty-print changed message.
    virtual bool send_pretty_print_changed(MCLT_client* client, bool pretty);

    /// Send format changed message.
    virtual bool send_format_changed(MCLT_client* client, const char* output_type,
                                     const MTformat* format);

    /// Send status message.
    virtual bool send_status(MCLT_client* client);

    /// Send plot result file message
    virtual bool send_plot_result_file(MCLT_client* client, void* cmd_cookie, const char* fname);

    /// Send input completition
    virtual bool send_input_completion(MCLT_client* client, const char* completionList, const char* completion);

    /// Return value of last completion / only needed for TTY
    virtual char *completion();

    /// Return value of last completion list / only needed for TTY
    virtual char *all_possible_completions();

    //@}

protected:
    virtual void show_banner();
    virtual void await_input(MCLT_client* client, const char* prompt);
    virtual void await_textinput(MCLT_client* client, const char* prompt);
    virtual bool send_prompt();
    virtual bool send_long_prompt();
    virtual void read_command();
    virtual bool read_input(MTheapStream& input, bool type);
    bool complete_;
    char *completion_;               //  buffer for completion
    char *all_possible_completions_; //  buffer for full completion list
};

#endif
