#ifndef __MCL_xmupad_switch_h__
#define __MCL_xmupad_switch_h__

/** Xmupad client switch.
 *
 * Client switch implementing communication with Xmupad.
 */
class MCLT_xmupad_switch : public MCLT_classic_dbg_switch {
public:
    /// Create new tty switch object.
    MCLT_xmupad_switch();

    /// Destroy switch.
    virtual ~MCLT_xmupad_switch();

    /// Test if client has a certain feature.
    virtual bool has_feature(const MCLT_client* client, MTclientFeature feature) const;

    /// Send plotting data stored in S-Pointer to UI
    virtual bool put_plot_data(MCLT_client* client, S_Pointer data);

    //@name Sending Messages to the Terminal
    /**
     * The class implements this methods by writing the messages to
     * the terminal.
     *
     * These methods return true if the message was sent successfully.
     */
    //@{
    /// Send output message.
    virtual bool send_output(MCLT_client* client, void* cmd_cookie, const char* output_type,
                             MTmemoryData* output);

    /// Send help message.
    virtual bool send_help(MCLT_client* client, const char* topic);

    /// Send system message.
    virtual bool send_system(MCLT_client* client, const char* cmd);

    /// Send textwidth changed message.
    virtual bool send_textwidth_changed(MCLT_client* client, long width);

    /// Send pretty-print changed message.
    virtual bool send_pretty_print_changed(MCLT_client* client, bool pretty);

    /// Send format changed message.
    virtual bool send_format_changed(MCLT_client* client, const char* output_type,
                                     const MTformat* format);

    /// Send status message.
    virtual bool send_status(MCLT_client* client);

    /// Send plot result file message
    virtual bool send_plot_result_file(MCLT_client* client, void* cmd_cookie, const char* fname);
    //@}

private:
    virtual void show_banner();
    virtual void await_input(MCLT_client* client, const char* prompt);
    virtual void await_textinput(MCLT_client* client, const char* prompt);
    virtual void read_command();
    virtual bool read_input();
    bool send_prompt();
    void send_signal();
};

#endif
