//
// DATEI:         MCO_kernparser.h
// AUTOR:         Torsten Metzner
// ERSTELLT:      03/09/98
// GEAENDERT:     03/09/98
// INHALT:        Konkrete Parserklasse die einen S_Pointer in einen
//                MCode-Bytesstream zerlegt und einen S_Pointer aus einem
//                MCode-Bytestream wieder aufbauen kann.
//
// Diese Headerdatei kann derzeit unter folgenden Betriebssystemen eingesetzt
// werden (hier wurde es getestet):
//    SPARC/Solaris 2.5 , PC/Linux 2.0
//

#ifndef __MCO_kernparser__
#define __MCO_kernparser__

MTR_DECL_LITERAL(MCOV_str_create_dom)

void MCO_init();
void MCO_exit();

#ifndef NO_MCODE

#include "MCO_parser.h"

//
// Klasse MTkernParser -- Konkreter Parser S_Pointer <--> MCodeBytestream
//
class MTkernParser : public MTmcodeParser
{
private:
   static const unsigned long highest_bit_ ;
   unsigned long reference_ ;  // Referenzzhler
   S_Pointer assmem_ ;         // Assoziativer Speicher
   static const long assmemtyp_ ; // Typ des assoziativen Speichers
   //void mydataheader_( (S_Pointer *) sp, MTdataheader head) ;
   Stat buildCatIdentString( S_Pointer *spointer ) ;
#ifndef GRAPHIC
   Stat writeCatIdentString( const S_Pointer *spointer ) ;
   Stat buildCatPariString_( void *spointer ) ;
   Stat writeCatPariString_( const void *spointer ) ;
   Stat writePariNumber_( const GEN parinumber, const long len ) ;
#endif
   Stat buildSonsGeneric_( S_Pointer *spointer, unsigned long sons ) ;
   Stat buildSonsGeneric_( S_Pointer *spointer, unsigned long anfang,
                           unsigned long ende ) ;
   Stat writeCatGeneric_(  const S_Pointer *spointer, const char *str ) ;
   Stat writeSonsGeneric_( const S_Pointer *spointer, const char *str ) ;
   Stat writeSonsGeneric_( const S_Pointer *spointer, unsigned long anfang,
                           unsigned long ende, const char *str ) ;
   Stat getstring_( char ** chpt ) ;

   Stat putstring_( char * chpt ) ;

   S_Pointer new_cat_poly_ ( long t, long u ) ;
   S_Pointer new_cat_point_( S_Pointer s1, S_Pointer s2,
                                     S_Pointer s3, S_Pointer color ) ;
   S_Pointer new_cat_polygon_ ( long num );

   Stat writeRgbColorFlat(S_Pointer color);
   Stat readRgbColorFlat(S_Pointer* color);
   Stat writePoint3dFlat(S_Pointer point);
   Stat readPoint3dFlat(long nobj, S_Pointer* point);

   // Hilfsroutinen fr Grafikobjekte
   Stat read_( long *l ) ;
   Stat read_( double *d ) ;
   Stat read_( char *chpt, size_t howmuch ) ;
   Stat read_( long *l, size_t howmuch ) ;
   Stat read_( double *d, size_t howmuch ) ;
   Stat read_( double *d, size_t dim1, size_t dim2 ) ;
   Stat read_( char * chpt, size_t d1, size_t d2, size_t d3 );


   Stat read_( MGRT_Scene *scene ) ;
   Stat read_( MGRT_Object *object ) ;
   Stat read_( MGRT_Defaults *defau ) ;
   Stat read_( MGRT_Color *color ) ;
   Stat read_( MGRT_Color *color, size_t howmuch ) ;
   Stat read_( MGRT_DrawStyle *draw ) ;
   Stat read_( MGRT_Font *font ) ;
   Stat read_( MGRT_Viewbox *box ) ;
   Stat read_( MGRT_Scal *scal) ;
   Stat read_( MGRT_Axes *axes ) ;
   Stat read_( MGRT_Ticks *ticks ) ;

   Stat write_( const long *l ) ;
   Stat write_( const double *d ) ;
   Stat write_( const char *chpt, size_t howmuch ) ;
   Stat write_( const long *l, size_t howmuch ) ;
   Stat write_( const double *d, size_t howmuch ) ;
   Stat write_( const double *d, size_t dim1, size_t dim2 ) ;
   Stat write_( const char * chpt, size_t d1, size_t d2, size_t d3 );

   Stat write_( const MGRT_Scene *scene ) ;
   Stat write_( const MGRT_Object *object ) ;
   Stat write_( const MGRT_Defaults *defau ) ;
   Stat write_( const MGRT_Color *color ) ;
   Stat write_( const MGRT_Color *color, size_t howmuch ) ;
   Stat write_( const MGRT_DrawStyle *draw ) ;
   Stat write_( const MGRT_Font *font ) ;
   Stat write_( const MGRT_Viewbox *box ) ;
   Stat write_( const MGRT_Scal *scal) ;
   Stat write_( const MGRT_Axes *axes ) ;
   Stat write_( const MGRT_Ticks *ticks ) ;


public:
   MTkernParser();
   MTkernParser(MTscanner *scan);
   ~MTkernParser() ;

   void scanner(MTscanner *scan);
   bool test_and_send_reference( const void *spointer ) ;
   void remember_write( const void *spointer ) ;
   void remember_read( const void *spointer ) ;

   Stat read( S_Pointer *spointer ) ;

   Stat buildHeader(      const MTmcodeHeader *head ) ;
   Stat buildCatString(   void *spointer, unsigned long special ) ;
   Stat buildCatStatList( void *spointer, unsigned long sons    ) ;
   Stat buildCatIdent(    void *spointer, unsigned long special ) ;
   Stat buildCatInt(      void *spointer, unsigned long special ) ;
   Stat buildCatApm(      void *spointer, unsigned long special ) ;
   Stat buildCatFloat(    void *spointer, unsigned long special ) ;
   Stat buildCatRat(      void *spointer, unsigned long special ) ;
   Stat buildCatComplex(  void *spointer, unsigned long special ) ;
#ifndef GRAPHIC
   Stat buildCatExec(     void *spointer, unsigned long sons ) ;
   Stat buildCatProc(     void *spointer, unsigned long sons ) ;
   Stat buildCatDom(      void *spointer, unsigned long special ) ;
   Stat buildCatPoly(     void *spointer, unsigned long special ) ;
   Stat buildCatExt(      void *spointer, unsigned long special ) ;
   Stat buildCatTable(    void *spointer, unsigned long special ) ;
   Stat buildCatSetFinite(void *spointer, unsigned long special ) ;
   Stat buildCatProcEnv(  void *spointer, unsigned long special ) ;
   Stat buildCatFuncEnv(  void *spointer, unsigned long sons ) ;
   Stat buildCatArray(    void *spointer, unsigned long special ) ;
#endif
   Stat buildCatBool(     void *spointer, unsigned long special ) ;
   Stat buildCatNull(     void *spointer, unsigned long special ) ;
   Stat buildCatNil(      void *spointer, unsigned long special ) ;
   Stat buildCatFailed(   void *spointer, unsigned long special ) ;
   Stat buildCatMmmnull( void *spointer, unsigned long special ) ;
   Stat buildCatExpr(     void *spointer, unsigned long sons ) ;
   Stat buildCatPoint(    void *spointer, unsigned long special ) ;
   Stat buildCatPolygon(  void *spointer, unsigned long sons ) ;
   Stat buildCatRef(      void *spointer, unsigned long special ) ;
   Stat buildCatVar(      void *spointer, unsigned long special ) ;

   Stat write( const S_Pointer spointer, const MTmcodeHeader *mcodeheader ) ;
   Stat writeCatString(    const void *spointer, unsigned long special ) ;
   Stat writeCatStatList(  const void *spointer, unsigned long sons    ) ;
#ifndef GRAPHIC
   Stat writeCatIdent(     const void *spointer, unsigned long special ) ;
   Stat writeCatInt(       const void *spointer, unsigned long special ) ;
   Stat writeCatApm(       const void *spointer, unsigned long special ) ;
   Stat writeCatFloat(     const void *spointer, unsigned long special ) ;
   Stat writeCatRat(       const void *spointer, unsigned long special ) ;
   Stat writeCatComplex(   const void *spointer, unsigned long special ) ;
   Stat writeCatExec(      const void *spointer, unsigned long special ) ;
   Stat writeCatProc(      const void *spointer, unsigned long special ) ;
   Stat writeCatDom(       const void *spointer, unsigned long special ) ;
   Stat writeCatPoly(      const void *spointer, unsigned long special ) ;
   Stat writeCatExt(       const void *spointer, unsigned long special ) ;
   Stat writeCatTable(     const void *spointer, unsigned long special ) ;
   Stat writeCatSetFinite( const void *spointer, unsigned long special ) ;
   Stat writeCatProcEnv(   const void *spointer, unsigned long special ) ;
   Stat writeCatFuncEnv(   const void *spointer, unsigned long special ) ;
   Stat writeCatArray(     const void *spointer, unsigned long special ) ;
#endif
   Stat writeCatBool(      const void *spointer, unsigned long special ) ;
   Stat writeCatNull(      const void *spointer, unsigned long special ) ;
   Stat writeCatNil(       const void *spointer, unsigned long special ) ;
   Stat writeCatFailed(    const void *spointer, unsigned long special ) ;
   Stat writeCatMmmnull(   const void *spointer, unsigned long special ) ;
   Stat writeCatExpr(      const void *spointer, unsigned long special ) ;
   Stat writeCatPoint(     const void *spointer, unsigned long special ) ;
   Stat writeCatPolygon(   const void *spointer, unsigned long special ) ;
   Stat writeCatRef(       const void *spointer, unsigned long special ) ;
   Stat writeCatVar(       const void *spointer, unsigned long special ) ;

   // Und jetzt gibt es doch noch Routinen fr MMM-Typen
#ifndef GRAPHIC
   Stat buildMmmProcEnv(  void *spointer, unsigned long sons ) ;
#endif

   Stat buildMmmString(   void *spointer, unsigned long special ) ;

   Stat buildMmmGraphic(  void *spointer, unsigned long sons ) ;
   Stat buildMmmScene3D(  void *spointer, unsigned long sons ) ;
   Stat buildMmmObject3D( void *spointer, unsigned long sons ) ;
   Stat buildMmmDefaults( void *spointer, unsigned long sons ) ;
   Stat buildMmmPointlist(void *spointer, unsigned long sons ) ;
   Stat buildMmmGrlist(   void *spointer, unsigned long sons ) ;
   Stat buildMmmPoint3D(  void *spointer, unsigned long sons ) ;
   Stat buildMmmLine(     void *spointer, unsigned long sons ) ;
   Stat buildMmmTriangle( void *spointer, unsigned long sons ) ;
   Stat buildMmmRgbColor( void *spointer, unsigned long sons ) ;
   Stat buildMmmGraphicCommand( void *spointer, unsigned long sons ) ;
   Stat buildMmmPolygon(  void *spointer, unsigned long sons ) ;

#ifndef GRAPHIC
   Stat writeMmmProcEnv(   const void *spointer, unsigned long special ) ;
#endif

   Stat writeMmmString(   const void *spointer, unsigned long special ) ;

   Stat writeMmmGraphic(  const void *spointer, unsigned long special ) ;
   Stat writeMmmScene3D(  const void *spointer, unsigned long special ) ;
   Stat writeMmmObject3D( const void *spointer, unsigned long special ) ;
   Stat writeMmmDefaults( const void *spointer, unsigned long special ) ;
   Stat writeMmmPointlist(const void *spointer, unsigned long special ) ;
   Stat writeMmmGrlist(   const void *spointer, unsigned long special ) ;
   Stat writeMmmPoint3D(  const void *spointer, unsigned long special ) ;
   Stat writeMmmLine(     const void *spointer, unsigned long special ) ;
   Stat writeMmmTriangle( const void *spointer, unsigned long special ) ;
   Stat writeMmmRgbColor( const void *spointer, unsigned long special ) ;
   Stat writeMmmGraphicCommand( const void *spointer, unsigned long sons ) ;
   Stat writeMmmPolygon(  const void *spointer, unsigned long special ) ;

} ;

#endif // ! NO_MCODE

#endif // __MCO_kernparser__
