#ifndef ___MCT_tarfile__
#define ___MCT_tarfile__


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef MSDOS
#include <unistd.h>
#endif

#ifdef MACINTOSH
#include <stat.h>
#else
#ifdef MSDOS
#include <stdlib.h>
#include <time.h>
#else
#ifdef linux
#include <time.h>
#else
#include <sys/stat.h>
#endif
#endif
#endif

#include "MCT_tar_table.h"
#include "MUP_constants.h"

#ifndef MUPC_NEW
#  ifdef SOLARIS
#    define MUPC_NEW new
#    define MUPC_DELETE delete
#  else
#    ifdef SGI
#       include <new.h>
#    else
         // std::... should be only defined in new, NOT in new.h
#       include <new>
#    endif
#    define MUPC_NEW new(std::nothrow)
#    define MUPC_DELETE delete
#    ifdef WIN32
// VC++ defines no operator delete(std::nothrow), thus...
inline void __cdecl operator delete(void * ptr, const std::nothrow_t&) throw()
    { ::operator delete(ptr); }
#    endif
#  endif
#endif

#define MCLC_UNIX_PATH_SEPARATOR		'/'
#define MCLC_MAC_PATH_SEPARATOR			':'
#define MCLC_DOS_PATH_SEPARATOR			'\\'

#ifdef MACINTOSH
#define MCLC_SYS_PATH_SEPARATOR			MCLC_MAC_PATH_SEPARATOR
#else
#ifdef MSDOS
#define MCLC_SYS_PATH_SEPARATOR			MCLC_DOS_PATH_SEPARATOR
#else
#define MCLC_SYS_PATH_SEPARATOR			MCLC_UNIX_PATH_SEPARATOR
#endif
#endif

typedef
struct _FileInfo
{
	long	fStart, fEnd;
	char	*fName;
} MCLT_FileInfo;

class MCLT_TarFile
{
private:
	FILE				*itsTarFile;

	char				itsPathSep;
	long				itsNumFiles;
	long				itsLastFileUsed;
	MCLT_FileInfo		*itsFileNameList;
	char				*itsFileNames;
	char				itsArchivePath[MUPC_MAX_PATH];
	char				itsArchiveName[32];
	char				itsErrorMsg[64];
	long				itsTotalStrLen;
	time_t				itsLastModDate;

	MCLT_LibFileTable	itsFileTable;

public:
				MCLT_TarFile();
				MCLT_TarFile(char pathSeparator);
				~MCLT_TarFile();

	int			Open(const char *fileName );
	int			Close(void);

	char		*GetLastError(void)				{ return( *itsErrorMsg? itsErrorMsg : (char *) NULL ); }

	void		ListTarFile(FILE *stream);
	short		NumFilesOpen()					{ return itsFileTable.NumOpenFiles(); }

	MCLT_FP		lfopen(const char *fileName, const char *theMode);
	int			lfclose(MCLT_FP libFile);

	int			lfeof(MCLT_FP libFile);
	long		lftell(MCLT_FP libFile)			{ return( itsTarFile? itsFileTable.GetFilePos(libFile)	: 0 ); }
	int			lferror()						{ return( itsTarFile? ferror(itsTarFile) 				: 1 ); }
	void		lclearerr()						{ clearerr(itsTarFile); }
	long 		lgetlength(MCLT_FP libFile)		{ return( itsTarFile? itsFileTable.GetFileLength(libFile): -1); }


	int			lfgetc(MCLT_FP libFile);
	int			lgetc(MCLT_FP libFile)			{ return(lfgetc(libFile)); }
	char		*lfgets(char *aBuffer, int numBytes, MCLT_FP libFile);
	size_t		lfread(void *aBuffer, size_t itemSize, size_t numItems, MCLT_FP libFile);

	int			lfseek(MCLT_FP libFile, long filePos, int fromWhere);

private:

	int			FindLibFile( char *fName, long *fStart, long *fEnd );
	int			XFindLibFile( char *fName, long *fStart, long *fEnd );
	int			ScanTarFile(void);
	int			ReadTarIndex(void);
	int			WriteTarIndex(void);
	void		PrepareForRead(MCLT_FP libFile);
	void		PathConvert(char *destPathName, char *srcPathName );
	void		SetErrorMsg(char *aString)		{ strcpy( itsErrorMsg, aString ); }
};


#endif
