/* $Date: 2000/02/08 16:40:45 $  $Author: tonner $  $Revision: 1.30 $ */

/**************************************************************************/
/* FILE   : MDB_debug.h                                                   */
/* AUTHOR : Oliver Kluge                                                  */
/* CREATED: 19.12.1991                                                    */
/* CHANGED: 07.09.1999                                                    */
/**************************************************************************/

#ifndef __MDB_debug__
#define __MDB_debug__

#ifdef MACINTOSH
#  include "MAC_interface.h"
#  include "MCL_client.h"
#endif

#include "MUP_env.h"

#undef DEBUG_ON
#ifdef DBX
#define DEBUG_ON
#else
#ifdef MDX
#define DEBUG_ON
#endif
#endif

#define MDBC_BUF_SIZE 512

#ifdef MDX
extern char MDBV_mode ;
#endif

extern bool MDBV_HIST_OF_PROCS;

MTR_DECL_LITERAL(MDBV_mmm_str_varname)

#ifdef DEBUG_ON

/****************************************************************/
/* Z.Z. wird davon ausgegangen, dass ein long 32 Bit ist, d.h.  */
/* BITS_IN_LONG = 32                                            */
/****************************************************************/

#define DBX_TYPE unsigned long


/* Es muss gelten:                                              */
/*                                                              */
/* (1) MAX_FILES = 2 ^ n , 0 < n < BITS_IN_LONG - 1             */
/*                                                              */
/* (2) SHIFT     = n + 1                                        */
/*                                                              */
/* (3) MASK      = hex(MAX_FILES - 1)                           */

/*****************************************************************/
/* STOP_LIST_SIZE, DISPLAY_LIST_SIZE, STACK_SIZE m"ussen >0 sein.*/
/*****************************************************************/

#define STOP_LIST_SIZE	  50	/* Max. Anzahl von Stop-Marken	*/

#define DISPLAY_LIST_SIZE 50	/* Max. Anzahl von Display-Var.	*/

#define MAX_FILES	1024	/* Max. Anzahl von Dateinamen	*/

#define SHIFT             11

#define MASK            0x000003ff

#define MAX_INFO	  10    /* Max. Anzahl der rekursiven   */
                                /* Aufrufen von add_debug_info  */

#ifdef USE_PRAGMA
#pragma GLOBAL_PRAGMA( MCLT_iostream , MDBV_covfile )
#endif
GLOBAL_DECL( MCLT_iostream , MDBV_covfile );

#define MDB_make_debug_info(line, name, valid) \
	(line << SHIFT | (MDB_get_key(name) << 1) | valid)

#define MDB_get_line_num(dbx) \
	(dbx >> SHIFT)

#define MDB_get_filekey(dbx) \
	((dbx >> 1) & MASK)

#define MDB_valid(dbx) \
	((dbx & 1) && (dbx >> SHIFT))

/* Debug-Modus eingeschaltet ? */

#define MDB_debug() \
	(MUPV_env.isDebugInteractive())

/* Verbose-Modus eingeschaltet ? */

#define MDB_verbose() \
	(MUPV_env.isDebugVerbose())

/* Wird Debugger "uber das Frontend gesteuert ? */

#define MDB_connected() \
        (MUPV_env.isDebugConnect())

#define MDB_gcov() \
	(MDBV_covfile.io_isopen())

/* used in the parser to set last debug node */
#define MDB_set_last_info(db_info) 
//	{ if (db_info != MMMNULL) MDBV_last_dbx = MDB_get_info(db_info); }


void MDB_init(  );
void MDB_init_2(  );
void MDB_reset(  );
void MDB_exit(  );
void MDB_error_handling( long );
void MDB_close_tmp_file(  );
void MDB_display_variables( MCLT_client* client );
void MDB_breakpoints( MCLT_client* client );
void MDB_call_stack( MCLT_client* client );
void MDB_reset_last_info(  );
void MDB_interact_with_user(S_Pointer s, long cmd_mode);
void MDB_cmd_dispatcher(  );
bool MDB_do_cmd(MCLT_client* client, unsigned char cmd, const char* fname, unsigned long fpos, const char* expr);
void MDB_enter_proc(S_Pointer name, S_Pointer args, DBX_TYPE dbx);
void MDB_leave_proc(S_Pointer name, S_Pointer result, bool remember);
void MDB_display_file(long status);
void MDB_show_buttons(char activate);

void MDB_add_debug_info(S_Pointer *root);

void MDB_print_error(char *s);

bool MDB_has_debug_infos(S_Pointer s);
DBX_TYPE MDB_first_debug_info(S_Pointer s);
bool MDB_no_debug(S_Pointer proc);
bool MDB_step_mode(  );

bool MDB_is_finished(  );

S_Pointer MDB_create_debug_info(DBX_TYPE mark_dbx);
S_Pointer MDB_eval_debug(S_Pointer root, long prev_func,long eval_type, S_Pointer exec);
S_Pointer MDB_eval_gcov(S_Pointer root, long prev_func,long eval_type, S_Pointer exec);
S_Pointer MDB_eval_predecessor(S_Pointer root, long prev_func,long eval_type, S_Pointer exec);
S_Pointer MDB_append_info(S_Pointer stmt, S_Pointer info);
S_Pointer MDB_prepand_info(S_Pointer stmt, S_Pointer info);
S_Pointer MDB_execute(S_Pointer cmd);
S_Pointer MDB_get_name_slot(S_Pointer proc);

DBX_TYPE MDB_get_info(S_Pointer dbx);

DBX_TYPE MDB_get_proc_entry_point(char* procname);

long MDB_get_key(char* name);

char* MDB_get_direct_filename(long index);

inline char* MDB_get_filename(DBX_TYPE dbx)
{ return(MDB_get_direct_filename(MDB_get_filekey(dbx))); }

#ifdef MACINTOSH
Handle	MDB_mac_get_stop_marks(char *fileName);
void	MDB_mac_get_callstack(size_t &level, size_t &num_calls, char **fileNames[], long *lineNumbers[], char **funcCalls[]);
#endif

#endif /* DEBUG_ON */

#endif /* __MDB_debug__ */
