/* $Date: 2000/01/21 17:50:54 $  $Author: olli $  $Revision: 1.14 $ */

/**************************************************************************/
/* FILE:             MGS_simplify.h                                       */
/* Autor:            Oliver Kluge                                         */
/* Erstellt:         01/07/91                                             */
/* Letzte Aenderung: 03/03/97                                             */
/**************************************************************************/

#ifndef __MGS_simplify__
#define __MGS_simplify__


#ifdef DOS_FILENAMES
#  include "MGS_simh.h"
#else
#  include "MGS_simplify_help.h"
#endif


/* Wichtige Konstanten des Moduls ******************************************/

#define MGSC_FLAT_OP      1      /* Zur Bedeutung dieser Konstanten siehe:  */
#define MGSC_FLAT_ARG     2      /* MGS_simplify.c, Routine MGS_flat().     */
#define MGSC_FLAT_NO_ARG  0
#define MGSC_FLAT_LIST	  4
#define MGSC_FLAT_ARG_2   8      /* Muss MGSC_FLAT_ARG beinhalten !	     */


// See lib/PROP/Propsys. The constants defined 
// in this file !must! be identical!!!!!!!!!!! 

#define MGSC_SYS_PROP_COMPLEX       1   // 2^0
#define MGSC_SYS_PROP_REAL          2   // 2^1
#define MGSC_SYS_PROP_NONNEGATIVE   4   // 2^2
#define MGSC_SYS_PROP_POSITIVE      8   // 2^3
#define MGSC_SYS_PROP_NEGATIVE     16   // 2^4
#define MGSC_SYS_PROP_INTEGER      32   // 2^5
#define MGSC_SYS_PROP_ZERO         64   // 2^6
#define MGSC_SYS_PROP_NONZERO     128   // 2^7
#define MGSC_SYS_PROP_ONE         256   // 2^8


// MFcat(expr) must be CAT_IDENT!
// 0 <= prop_mask < 4096 (12 Bit)

inline bool
MGS_check_sys_prop( S_Pointer expr, unsigned int prop_mask)
{
   unsigned int code;

   if ((code = MID_read_sys_property(expr)) == 0) {
      // Identifier has no properties
      return(FALSE);
   }

   if (code & prop_mask)
      return(TRUE);
   else
      return(FALSE);
}

#ifdef IGNORE_SYS_PROP

# define MGSM_check_sys_prop(cat, expr, is_ident, prop_mask)    \
         ((is_ident = FALSE))

  inline unsigned int
  MGS_read_sys_prop(short cat, S_Pointer expr, bool *is_ident)
  {
      *is_ident = FALSE;
      return( 0 );
  }

# define MGSMS_check_sys_prop(code, prop_mask)                  \
         FALSE

#else

  // S_Pointer expr, cat = MFcat(expr)
# define MGSM_check_sys_prop(cat, expr, is_ident, prop_mask)    \
         ((is_ident = (cat == CAT_IDENT)) &&                    \
          MGS_check_sys_prop(expr, prop_mask))

  inline unsigned int
  MGS_read_sys_prop(short cat, S_Pointer expr, bool *is_ident)
  {
      if ((*is_ident = (cat == CAT_IDENT)))
         return( MID_read_sys_property(expr) );
      else
         return( 0 );
  }

# define MGSMS_check_sys_prop(code, prop_mask)                  \
         (code & prop_mask)

#endif

// S_Pointer expr, cat = MFcat(expr)
#define MGSM_is_number(cat, expr, is_ident)                     \
        (cat < MUPC_num ||                                      \
         MGSM_check_sys_prop(cat, expr, is_ident,               \
             (MGSC_SYS_PROP_COMPLEX | MGSC_SYS_PROP_REAL)))

// S_Pointer expr, cat = MFcat(expr)
#define MGSM_is_real(cat, expr, is_ident)                       \
        (cat < MUPC_real ||                                     \
         MGSM_check_sys_prop(cat, expr, is_ident,               \
                             MGSC_SYS_PROP_REAL))

// S_Pointer expr, cat = MFcat(expr)
#define MGSM_is_pos_real(cat, expr, is_ident)                   \
        ((cat < MUPC_real && MCA_gt(expr, MCAC_zero)) ||        \
         MGSM_check_sys_prop(cat, expr, is_ident,               \
                             MGSC_SYS_PROP_POSITIVE))

// S_Pointer expr, cat = MFcat(expr)
#define MGSM_is_integer(cat, expr, is_ident)                    \
        (cat == CAT_INT || cat == CAT_APM ||                    \
         MGSM_check_sys_prop(cat, expr, is_ident,               \
                             MGSC_SYS_PROP_INTEGER))

// S_Pointer expr, cat = MFcat(expr), is_int0 if expr = 0, not 0.0
#define MGSM_is_zero(cat, expr, is_float0, is_ident)            \
        ((is_float0 = MCM_equal(expr, MCAC_r_zero)) ||          \
         MCM_equal(expr, MCAC_zero)                 ||          \
         MGSM_check_sys_prop(cat, expr, is_ident,               \
                             MGSC_SYS_PROP_ZERO))

// The following macros should be used if the system properties
// of an identifier are already known. These macros avoid to
// read again the system properties. 

#define MGSMS_is_number(cat, code)                              \
        (cat < MUPC_num ||                                      \
         MGSMS_check_sys_prop(code,                             \
             (MGSC_SYS_PROP_COMPLEX | MGSC_SYS_PROP_REAL)))

#define MGSMS_is_real(cat, code)                                \
        (cat < MUPC_real ||                                     \
         MGSMS_check_sys_prop(code, MGSC_SYS_PROP_REAL))        

#define MGSMS_is_pos_real(cat, expr, code)                      \
        ((cat < MUPC_real && MCA_gt(expr, MCAC_zero)) ||        \
         MGSMS_check_sys_prop(code, MGSC_SYS_PROP_POSITIVE))    

#define MGSMS_is_integer(cat, code)                             \
        (cat == CAT_INT || cat == CAT_APM ||                    \
         MGSMS_check_sys_prop(code, MGSC_SYS_PROP_INTEGER))

#define MGSMS_is_zero(expr, code, is_float0)                    \
        ((is_float0 = MCM_equal(expr, MCAC_r_zero)) ||          \
         MCM_equal(expr, MCAC_zero)                 ||          \
         MGSMS_check_sys_prop(code, MGSC_SYS_PROP_ZERO))

#define MGSMS_is_one(expr, code)                                \
        (MCM_equal(expr, MCAC_one) ||                           \
         MGSMS_check_sys_prop(code, MGSC_SYS_PROP_ONE))


/* Unterhalb dieser Grenze wird per Ein-   */
/* fuegen sortiert, dar"uber per Quicksort.*/
#define MGSC_QUICKSORT   25     

extern bool MGSV_flat_table[];

/* Deklaration der Funktionen, die Schnittstelle des Moduls sind ***********/

#ifdef SET_LENGTH
S_Pointer MGS_eval_set_length     ( S_Pointer s,
                                    long      prev_func,
                                    long      eval_type,
                                    S_Pointer exec );
#endif

void MGS_init                      (  );
void MGS_exit                      (  );

void MGS_quicksort                 ( S_Pointer *s, long l, long *r, long op_typ,
                                     long prev_func, long eval_type, 
                                     S_Pointer function, bool *float_context );

void MGS_quicksort_generic         ( S_Pointer *s, long le, long *rp, long op_typ,
                                     long (*cmp_func)(S_Pointer,S_Pointer),
                                     void (*sum_func)(S_Pointer*,long,long,long*,long,long*),
                                     long prev_func, long eval_type, S_Pointer function );

void MGS_straightinsertion         ( S_Pointer *s, long l, long *r, long op_typ,
                                     long prev_func, long eval_type, 
                                     S_Pointer function, bool *float_context );

void MGS_straightinsertion_generic ( S_Pointer *s, long l, long *r, long op_typ,
                                     long (*cmp_func)(S_Pointer, S_Pointer),
                                     void (*sum_func)(S_Pointer*,long,long,long*,long,long*),
                                     long prev_func, long eval_type, S_Pointer function );

void MGS_summarize_and_or_union_intersect ( S_Pointer *s, long i, long j, long *r,
                                            long op, long *dummy );

void MGS_compress                  ( S_Pointer s, long *upperbound, long count_blank, long op_typ,
                                     long prev_func, long eval_type, S_Pointer function );

long MGS_cmp                       ( long i, long j, S_Pointer sp_i, S_Pointer sp_j,
                                     long op_typ );

bool MGS_flat                   ( S_Pointer *r, long flat_flag );

#ifdef OLD_SET

bool MGS_combine_constants      ( S_Pointer *s, long lower_const_typ, long upper_const_typ,
                                     long op_typ, long *new_counter );

#else

bool MGS_combine_constants      ( S_Pointer *s, long lower_const_typ, long upper_const_typ,
                                     long op_typ, long *new_counter, S_Pointer *set_constant );

S_Pointer MGS_call_dom_slot (char* chr_dom, long sys_dom, long slot, 
                             bool is_sys, long num Mva_arglist);
inline S_Pointer MGS_build_chr_dom_slot (char *, long);
inline S_Pointer MGS_build_sys_dom_slot (long, long);

#endif

bool      MGS_contains_not   ( S_Pointer s );
S_Pointer MGS_normalize_not  ( S_Pointer s );
S_Pointer MGS_negate_        ( S_Pointer s, bool Unsimplified );
S_Pointer MGS_reciprocal_    ( S_Pointer s, bool Unsimplified );
S_Pointer MGS_divide_        ( S_Pointer s, S_Pointer t, bool Unsimplified );
S_Pointer MGS_minus_         ( S_Pointer s, S_Pointer t, bool Unsimplified );
long      MGS_special_expr   ( S_Pointer s );

/***********************************************************/
/* Berechne 1/s. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_reciprocal( s )     MGS_reciprocal_( s, false )

/***********************************************************/
/* Berechne -s. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_negate( s )         MGS_negate_( s, false )

/***********************************************************/
/* Berechne s/t. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_divide( s, t )      MGS_divide_( s, t, false )

/***********************************************************/
/* Berechne s-t. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_minus( s, t )       MGS_minus_( s, t, false )


/* Definition von Makros ***************************************************/

#define MGS_sort_generic(s, l, r, op_typ, cmp_func, sum_func, prev_func, eval_type, func)       \
 {                                                                                              \
   if( *(r)-(l) >= MGSC_QUICKSORT )                                                             \
        MGS_quicksort_generic        (s,l,r,op_typ,cmp_func,sum_func,prev_func,eval_type,func); \
   else MGS_straightinsertion_generic(s,l,r,op_typ,cmp_func,sum_func,prev_func,eval_type,func); \
 }


#define MGS_sort(s, l, r, op_typ, prev_func, eval_type, func, float_context)\
 {                                                                          \
   if( *(r)-(l) >= MGSC_QUICKSORT )                                         \
        MGS_quicksort        (s, l, r, op_typ, prev_func,                   \
                              eval_type, func, float_context);              \
   else MGS_straightinsertion(s, l, r, op_typ, prev_func,                   \
                              eval_type, func, float_context);              \
 }


#define MGS_op_is(a, b) \
        MCM_compare_MMMstring_ident(MTRV_SYS_FUNC_LIST[a], b)

#define MGS_flat_allowed(func_nr) \
	MGSV_flat_table[func_nr]

#endif /* __MGS_simplify__ */
