/* $Date: 2001/01/31 13:13:01 $  $Author: ccr $  $Revision: 1.25 $ */

/*
 * MID_ident.h -- Prozeduren zum Zugriff auf Bezeichner
 */

#ifndef __MID_ident_h__
#define __MID_ident_h__

#include "MTR_tree.h"
#include "MUP_constants.h"

#ifdef DOS_FILENAMES
#  include "MUP_typ.h"
#else
#  include "MUP_types.h"
#endif


/**************************************************************************/
/* MIDT_option                                                             */
/* Mit Hilfe dieser Option k"onnen den Stack-Funktionen weitere           */
/* Informationen "ubergeben werden.                                       */
/**************************************************************************/
typedef long MIDT_option ;

#define MIDC_option_non            0
#define MIDC_option_standard       0
#define MIDC_option_force          4
#define MIDC_option_delete         8

/**************************************************************************/
/* Im Mem-Teil der obersten Stack-Schachtel werden Informationen          */
/* gespeichert. Mit den folgenden Konstanten k"onnen diese Informationen  */
/* ausgelesen werden.                                                     */
/**************************************************************************/
#define MIDC_protected_flags     3
#define MIDC_not_protected       0
#define MIDC_warn_protected      1
#define MIDC_err_protected       2
#define MIDC_user_protected      3

/* Informationen zum Identifier: Namespace-ID, Protection, Property
 * Wird im Mem-Teil des MMM_VALUE-Sohns gespeichert.
 */
struct MIDT_ident {
    unsigned int namespace_idx : 16; // Index des Namespaces <= 65536
    unsigned int protection    :  3; // Protection-Flags
    unsigned int is_property   :  1; // Wert oder Property?
    unsigned int sysprop       : 12; // Kodierte System-Property 
                                     // (max. 12 verschiedene). Siehe
                                     // auch lib/PROP/Propsys.mu
};

/* Der Ausdruck exp(1) */
MTR_DECL_LITERAL( MIDV_exp1 )

// global MMM-strings
MTR_DECL_LITERAL( MSTR_DIGITS )
MTR_DECL_LITERAL( MSTR_ORDER )
#ifdef USE_ROUNDING_VAR
MTR_DECL_LITERAL( MSTR_ROUNDING )
#endif
MTR_DECL_LITERAL( MSTR_PI )
MTR_DECL_LITERAL( MSTR_EULER )
MTR_DECL_LITERAL( MSTR_CATALAN )

/**************************************************************************/
/* Deklaration der Prozeduren zur Behandlung des Stacks.                  */
/**************************************************************************/

void       MID_init();
void       MID_reset();
void       MID_exit();
void       MID_set_environment_variables();

unsigned int MID_protect_flags (S_Pointer ident);
void       MID_set_protect_flags(S_Pointer ident, unsigned int flags);

S_Pointer  MID_replace_ident_val (S_Pointer   ident,
                              S_Pointer   value,
                              MIDT_option );

void       MID_write_ident_val  (S_Pointer   string,
                             S_Pointer   value,
                             MIDT_option);

void       MID_write_string_val (S_Pointer   string,
                             S_Pointer   value,
                             MIDT_option);

S_Pointer  MID_read_ident_val   (S_Pointer  ident);

S_Pointer  MID_read_string_val  (S_Pointer  string);

S_Pointer  MID_read_ident_float_val(S_Pointer ident);

S_Pointer  MID_read_ident_property(S_Pointer ident);

void       MID_write_ident_property(S_Pointer ident,
			    S_Pointer prop);

S_Pointer  MID_eval_properties (S_Pointer s,
                                long      prev_func,
                                long      eval_type,
                                S_Pointer exec);

bool       MID_properties_valid( );

/** Get value of special ident ORDER.
 * @return actual value of ORDER */
long MID_get_order();

/** Set value of special ident ORDER.
 * @param ord new value of ORDER, default is 6. */
void MID_set_order(long ord=6);

/**
   Tests if an identifier is no longer used.

   This function is used during garbage collection to remove unused idents.
   An identifier is unused if it is not referenced, has no value and is not
   protected.
   @param name name of the identifier
   @param ident identifier to test
   @param cd call-data passed through via #MAM_filter#
   @return true if the ident is unused
 */
bool MID_is_unused_ident(S_Pointer name, S_Pointer ident, void* cd);

/**
    Return the name, this means the underlying C-String of the given S\_Pointer
    of type CAT\_IDENT
    @param   stringi  A S\_Pointer of type CAT\_IDENT
    @return  A pointer to the name (the underlying C-String) of the identifier
*/
const char* MID_get_name( const S_Pointer ident ) ;


// Die Lese/Schreibroutinen muessen global deklariert werden, da sie in
// MLI_literals.cpp vom generierten Code benoetigt werden.

S_Pointer MID_change_digits       ( S_Pointer value );
S_Pointer MID_change_order        ( S_Pointer value );
S_Pointer MID_change_history      ( S_Pointer value );
S_Pointer MID_change_level        ( S_Pointer value );
S_Pointer MID_change_maxlevel     ( S_Pointer value );
S_Pointer MID_change_maxdepth     ( S_Pointer value );
S_Pointer MID_change_textwidth    ( S_Pointer value );
S_Pointer MID_change_lib_path     ( S_Pointer value );
S_Pointer MID_change_read_path    ( S_Pointer value );
S_Pointer MID_change_write_path   ( S_Pointer value );
S_Pointer MID_change_pretty_print ( S_Pointer value );
S_Pointer MID_change_pi           ( S_Pointer value );
S_Pointer MID_change_euler        ( S_Pointer value );
S_Pointer MID_change_catalan      ( S_Pointer value );
S_Pointer MID_change_rounding     ( S_Pointer value );

S_Pointer MID_read_digits         ( bool eval_float );
S_Pointer MID_read_order          ( bool eval_float );
S_Pointer MID_read_pi             ( bool eval_float );
S_Pointer MID_read_euler          ( bool eval_float );
S_Pointer MID_read_catalan        ( bool eval_float );
S_Pointer MID_read_level          ( bool eval_float );
S_Pointer MID_read_history        ( bool eval_float );
S_Pointer MID_read_textwidth      ( bool eval_float );
S_Pointer MID_read_pretty_print   ( bool eval_float );
S_Pointer MID_read_maxlevel       ( bool eval_float );
S_Pointer MID_read_maxdepth       ( bool eval_float );
S_Pointer MID_read_rounding       ( bool eval_float );
S_Pointer MID_read_lib_path       ( bool eval_float );
S_Pointer MID_read_read_path      ( bool eval_float );
S_Pointer MID_read_write_path     ( bool eval_float );

/**************************************************************************
 * Implementation
 */

inline void
MID_write_ident_val(S_Pointer ident, S_Pointer value, MIDT_option opt)
{
    S_Pointer res = MID_replace_ident_val(ident, value, opt);
    MMMfree(res);
}

inline unsigned int
MID_protect_flags(S_Pointer ident)
{
    S_Pointer val = *MMMP(&ident, 1);
    unsigned int flags = MMMZ(&val, MIDT_ident)->protection;
    return flags;
}

inline void
MID_set_protect_flags(S_Pointer ident, unsigned int flags)
{
    S_Pointer val = *MMMP(&ident, 1);
    MMMZ(&val, MIDT_ident)->protection  = flags;
}

inline const char* MID_get_name( const S_Pointer ident )
{
   assert( MTR_cat(ident) == CAT_IDENT ) ;
   const S_Pointer *sohn = MMMP( (S_Pointer *) &ident, 0) ;
   return( MTR_get_string( *sohn ) ) ;
}

/*
 * MID_read_sys_property -- liefert eine ganze > 0, falls dem
 * Bezeichner System-Properties zugeordnet sind, 0 sonst.
 * Der Rueckgabewert enthaelt die System-Properties in
 * kodierter Form (siehe lib/PROP/Propsys.mu).
 */

inline unsigned int
MID_read_sys_property(S_Pointer ident)
{
    MIDT_ident *val;

    // MFcat(ident) must be CAT_IDENT !!!
    val = MMMZ(MMMP(&ident, 1), MIDT_ident);
    if (val->is_property) 
       return(val->sysprop);
    else
       return(0);
}

#endif /* __MID_ident_h__ */

