/* $Date: 2000/08/07 14:52:25 $  $Author: kg $  $Revision: 1.37 $ */

/**************************************************************************/
/* FILE:             MIO_io.h                                             */
/* Autor:            Oliver Kluge, Andreas Kemper, Ralf Hillebrand        */
/* Erstellt:         27/06/91                                             */
/* Letzte Aenderung: 31/08/94                                             */
/**************************************************************************/

#ifndef __MIO_io__
#define __MIO_io__

#ifdef DOS_FILENAMES
#  include "MPR_prot.h"
#else
#  include "MPR_protocol.h"
#endif

#ifndef GRAPHIC
#  include "MKT_mssg.h"
#endif

#if !(defined GRAPHIC || defined MOTIF)

/**************************************************************************/
/* Dieses File enthaelt die Routinen zur Ausgabe.                         */
/**************************************************************************/

extern bool MIOV_PRETTY_PRINT;    /* Steuerung des Pretty - Printers */
extern bool MIOV_STRING_UNQUOTED; /* Steuerung der Stringausgabe */
extern MCLT_bytestream* MIOV_OUTPUT; /* Ausgabefile */

/* Strings fuer intern bekannte Ausgabetypen, vgl. MKT_mssg.h */
MTR_DECL_LITERAL(MIOV_ot_result)
MTR_DECL_LITERAL(MIOV_ot_warning)
MTR_DECL_LITERAL(MIOV_ot_print)
MTR_DECL_LITERAL(MIOV_ot_error)
MTR_DECL_LITERAL(MIOV_ot_debug)
MTR_DECL_LITERAL(MIOV_ot_dbg_err)
MTR_DECL_LITERAL(MIOV_ot_dbg_pretty)
MTR_DECL_LITERAL(MIOV_ot_other)
MTR_DECL_LITERAL(MIOV_ot_SYSERROR)

MTR_DECL_LITERAL(MIOV_fn_mcode_content)
MTR_DECL_LITERAL(MIOV_fn_mcode_tree)
MTR_DECL_LITERAL(MIOV_fn_mupad_lang)
MTR_DECL_LITERAL(MIOV_fn_null)
MTR_DECL_LITERAL(MIOV_fn_text)

MTR_DECL_LITERAL(MIOV_DOMAIN_ASS_EXPR2TEXT)
MTR_DECL_LITERAL(MIOV_DOMAIN_ASS_OUTPUT)

MTR_DECL_LITERAL(MIOV_S_STMTSEQ)
MTR_DECL_LITERAL(MIOV_S_EXPRSEQ)
MTR_DECL_LITERAL(MIOV_S_POWER)
MTR_DECL_LITERAL(MIOV_S_MULT)
MTR_DECL_LITERAL(MIOV_S_DIVIDE)
MTR_DECL_LITERAL(MIOV_S_PLUS)
MTR_DECL_LITERAL(MIOV_S_MINUS)


/************** Die offizielle Schnittstelle der Ausgabe ******************/

void     /* Initialisierung der Ausgabe */
MIO_init();

void     /* Gibt bei Programmende dynamische Strukturen frei */
MIO_exit();

void     /* Zuruecksetzen der Ausgabe nach Error */
MIO_error_reset(bool newline=true);

void     /* Zuruecksetzen der Ausgabe nach reset() */
MIO_reset();

int      /* typisierte Ausgabe von 1-String-Messages */
MIO_printf(S_Pointer outputType, char *format Mva_arglist);

int      /* typisierte Ausgabe von 1-String-Messages */
MIO_printf1(S_Pointer outputType, char *format, Mva_list);

void     /* Ausgabe eines MuPAD-Datums */
MIO_out(S_Pointer, S_Pointer outputType = MIOV_ot_result,
        bool newline=true);

bool  /* Ausgabe des Endergebnisses in der Main Command Loop. */
MIO_print_result(S_Pointer res, const char* fmt, bool old_pretty_print);

MTformat /* Liefert zu Ausgabe-Datenformat-Namen das Client-Format */
MIO_get_output_format(S_Pointer fmt_name);

void	/* Aendert Datenformat fuer Ausgabetyp. */
MIO_change_output_format(const char* type, const char* fmt);

void	/* Aendert Datenformat fuer Ausgabetyp. */
MIO_change_output_format(S_Pointer type, S_Pointer fmt);

bool    /* Prueft, ob fmt gueltiges Ausgabeformat fuer Ergebnisse ist */
MIO_is_valid_result_format(const char* fmt);

void	/* Schickt alle Ausgabetypen zum Client. */
MIO_send_all_output_types(MCLT_client* client);

/* Ausgabe einer Prozedur und gleichzeitiges Einfuegen von
   Debug-Informationen.                                    */
void
MIO_create_debug(S_Pointer *, MCLT_bytestream*, long, long *);


S_Pointer /* erledigt die Drecksarbeit fuer MSY_eval_expr2text  */
MIO_expr2text(S_Pointer);

/* Client-Context fuer die Ausgabe erzeugen */
S_Pointer MIO_new_client_context();

/* Client-Context fuer die Ausgabe loeschen */
void MIO_delete_client_context(S_Pointer ctx);

/* Client-Context fuer die Ausgabe (de)aktivieren */
void MIO_activate_client_context(S_Pointer ctx, bool activate);

/* List mit aktuell ausgegebenen Prozeduren */
extern S_Pointer MIOV_ACT_PROC;


/**** Die folgenden Funktionen kontrollieren den Zugriff auf die
      Variable MIOV_PRETTY_PRINT, die IMMER auch im Stack erfolgen
      MUSS.                                                        */

/* Schreiben eines neuen Pretty-Werts auf den Stack */

#define MIO_push_pretty(val) \
    bool __act_pretty__ = MIOV_PRETTY_PRINT; \
    MIOV_PRETTY_PRINT = ((val) ? true : false);
#define MIO_pop_pretty() \
    MIOV_PRETTY_PRINT = __act_pretty__;

#define MIO_push_pretty_true() MIO_push_pretty(1)
#define MIO_push_pretty_false() MIO_push_pretty(0)

/* zuerst kommen die Standard-Ausgabe-Funktionen  */
S_Pointer
MIO_out_stand_op(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_stand_sys_func(S_Pointer *, S_Pointer, long, S_Pointer);

/* nun diejenigen Ausgaberoutinen spezieller Ausdruckstypen */
S_Pointer
MIO_out_slot(          S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_index(         S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_not(           S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_divex(         S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_negate(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_invert(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_subtract(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_seqgen(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_seqin(         S_Pointer *, S_Pointer, long, S_Pointer);

/* und schliesslich die Ausgaberoutinen der Anweisungen */
S_Pointer
MIO_out_assign(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_break(         S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_case(          S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_delete(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_for(           S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_for_down(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_for_in(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_if(            S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_next(          S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_quit(          S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_repeat(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer
MIO_out_while(         S_Pointer *, S_Pointer, long, S_Pointer);


/******* Vordefinierte Konstanten fuer Ausgabeprioritaeten *******/

#define MIOC_P_NULL             0
#define MIOC_P_STMTSEQ          1
#define MIOC_P_EXPRSEQ          2
#define MIOC_P_OR               3
#define MIOC_P_AND              4
#define MIOC_P_SEQGEN           5
#define MIOC_P_NOT             MIOC_P_SEQGEN
#define MIOC_P_LESS             6
#define MIOC_P_LEEQUAL          6
#define MIOC_P_EQUAL            6
#define MIOC_P_UNEQUAL          6
#define MIOC_P_RANGE            7
#define MIOC_P_UNION            8
#define MIOC_P_MINUS            9
#define MIOC_P_INTERSECT       10
#define MIOC_P_MOD             11
#define MIOC_P_PLUS            12

/* die beiden folgenden Eintraege sind nur fuer den PRETTY_PRINTer */
#define MIOC_P_UNARY_MINUS     13
#define MIOC_P_COMPLEX         14

#define MIOC_P_MULT            15
#define MIOC_P_POWER           16
/* hier fehlen eigentlich selbst-def Operatoren */
#define MIOC_P_FCONCAT	       17
/* 18 wird in der lib fuer _fnest gesetzt */
#define MIOC_P_CONCAT          19
#define MIOC_P_SLOT            20
#define MIOC_P_STMT            21


#define MIOC_P_max             22

#endif

/* Dies wird von GRAPHIC bentigt */

#define MIOC_DONT_SPLIT        32
#define MIOC_BADNESS           64

#define MIOT_FIELDSIZE long       /* Datenfeldgroesse im Ausgabebaum */

#if !(defined GRAPHIC || defined MOTIF)
/* Tabelle Ausgabetyp --> Liste moeglicher Datenformate */
extern S_Pointer MIOV_output_types;

/* Tabelle Ausgabetyp --> aktuelles Datenformat */
extern S_Pointer MIOV_act_formats;
#else

/* Dies wird von GRAPHIC bentigt */

extern void         MIO_print_string
                       (   char    *string
                       );

extern void         MIO_output_string
                       (   char    *string
                       );

extern void         MIO_print_format_string
                       (   char   *format,
                           long   toprint
                       );

extern void         MIO_print_char(char c);

#endif

#endif /* __MIO_io__ */
