/* $Date: 1999/11/17 14:18:13 $  $Author: andi $  $Revision: 1.26 $ */
#ifndef __MMMstorage_lean__
#define __MMMstorage_lean__

#ifdef MACINTOSH
#pragma once
#endif

/*
Definements fuer die Speicherverwaltung:
MMMDEBUG           - Debug-Version
MMMD_COMPACT       - kompakte S-Pointer-Header
MMMD_STATISTICS    - ermoeglicht Statistiken ueber Speicherverbrauch
MMMD_LIT_SP        - erlaubt literale S-Pointer (nicht dynamisch alloziiert)
MMMD_GEN_LIT_SP    - ermoeglicht die Generierung literaler S-Pointer
*/

#define MEXDSTANDARD
#ifdef DOS_FILENAMES
#  include "MEX_ext.h"
#  include "MDE_decl.h"
#else
#  include "MEX_extern.h"
#  include "MDE_declare.h"
#endif

#ifdef PARALLEL
#  error Sequential only, sorry.
#endif
#if (defined MMMD_LIT_SP) && (defined MMMD_GEN_LIT_SP)
#  error Both MMMD_LIT_SP and MMMD_GEN_LIT_SP are defined.
#endif
#if (defined MMMD_STATISTICS) && (! defined MMMDEBUG)
#  error MMMDEBUG is needed for MMMD_STATISTICS.
#endif

////////////////////////////////////////////////////////////////////////
// Makros

#define MMMZ(s,T) \
           ((T *)MMMgetMemAdr(*(s)))

#define MMMarr(s,i,T) \
           ((T *)MMMgetMemAdr(*(s), (i)*sizeof(T)))

#define MMMmv(s,mv,T) \
           ((T *)MMMgetMemAdr(*(s), (mv)))

#define MMMmvarr(s,mv,i,T) \
           ((T *)MMMgetMemAdr(*(s), (mv) + (i)*sizeof(T)))

#define MMMP(s,i) \
           MMMgetPtrAdr(*(s), (i))

////////////////////////////////////////////////////////////////////////
// Typen

class MMMTheader;
typedef MMMTheader* S_Pointer;
typedef void (*MMMTno_memory)();
typedef void (*MMMTevent_handler)();

#ifdef MMMDEBUG
struct _MMMT_stats {
    long used_page_blocks;
    long used_big_blocks;
    long reserved_blocks;
    long max_reserved_memory;
    long frees;
    long realfrees;
    long mallocs;
};
typedef struct _MMMT_stats MMMT_stats;
#endif // MMMDEBUG

typedef unsigned char MTbool;   // Fuer Module

////////////////////////////////////////////////////////////////////////
// Oeffentliche Schnittstelle

const S_Pointer MMMNULL = 0;

S_Pointer MMMmalloc(long typ, long size, long count);
S_Pointer MMMpalloc(long typ, long size, long count);
S_Pointer MMMcalloc(long typ, long size, long count);

void MMMresize(S_Pointer *s, long size);
void MMMnewcounter(S_Pointer *s, long count);
void MMMrealloc(S_Pointer *s, long size, long count);
void MMMchange(S_Pointer *s);

void MMMfree(S_Pointer s);
void MMMlfree(S_Pointer s);
long MMMrefs(S_Pointer s);

char *MMMEmalloc(long size);
void MMMEfree(char *);

int MMMequal(S_Pointer s, S_Pointer t);
void MMMset(S_Pointer *p, S_Pointer s);
void MMMreplace(S_Pointer *p, S_Pointer s);
void MMMshift(S_Pointer sour, long spos,
	      S_Pointer dest, long dpos, long count);

long MMMsize(S_Pointer s);
long MMMcounter(S_Pointer s);
long MMMtype(S_Pointer s);
long MMMsignature(S_Pointer s);
void MMMnewsignature(S_Pointer s, long sig);

void MMMinsert_global(S_Pointer *s);
void MMMmark_globals();
void MMMmark_tree(S_Pointer s, bool mark=true);
bool MMMfree_unmarked(S_Pointer s);
void MMMfreeUnmarked(S_Pointer s);
void MMMfree_unused();
void MMMshare_all();

void MMMfirst_init(MMMTevent_handler eventHandler = 0);
MMMTno_memory MMMout_of_memory_handler(MMMTno_memory hdl);
MMMTno_memory MMMmemory_limit_handler(MMMTno_memory hdl);
void MMMinit(long memory_limit);
void MMMexit();


long MMMused_memory();
long MMMreserved_memory();
#ifdef MMMDEBUG
void MMMget_stats(MMMT_stats* stats);
#endif

////////////////////////////////////////////////////////////////////////
// Debuggen

#ifdef MMMDEBUG
#define MMMASSERT(e) assert(e)
#else
#define MMMASSERT(e)
#endif

#if (defined MMMDEBUG || defined ALPHA) && (! defined MACINTOSH)
void MMMtrace(const char*, ...);
#  define MMMTRACE MMMtrace
#  define MMMTRACE2 MMMtrace
#else
#define MMMTRACE(f)
#define MMMTRACE2(f,p)
#endif

////////////////////////////////////////////////////////////////////////
// Deklaration der Hilfsfunktionen

S_Pointer *MMMgetPtrAdr(S_Pointer s, long i);
char *MMMgetMemAdr(S_Pointer s);
char *MMMgetMemAdr(S_Pointer s, long mv);

extern void MMMrecFree(S_Pointer s);
extern void MMMfreeBlock(S_Pointer s);

#if (defined MSDOS) || (defined MACINTOSH)
void *MMMsysIrrMalloc(long len);
void MMMsysIrrFree(void *p);
#else
inline void *MMMsysIrrMalloc(long len);
inline void MMMsysIrrFree(void *p);
#endif

////////////////////////////////////////////////////////////////////////
// MMMTheader
// Jede MAMMUT-Zelle enth"alt in einem Header Verwaltungsinformationen.
// Dieser Header wird am Anfang der Zelle gespeichert.
// Die Zelle kann frei oder belegt sein. Ist sie frei, so wird in next
// ein Zeiger auf die naechste freie Zelle gespeichert.

#ifdef MMMD_COMPACT

/** Header for compact nodes.

Node headers come in two versions: compact and non-compact ones.
Their usage is controlled by the definement #MMMD_COMPACT#: If this
definement exists, compact headers are used.

Compact headers have varying length to minimize the memory usage of
the header, non-compact headers have fixed size. Accessing compact
headers is slower than accessing non-compact ones.

The memory layout of the compact headers is as follows:
\begin{itemize}
\item Status information (1 unsigned long).
\item Signature (1 long).
\item Optionally: Count, i.e. no of childrens of the node (1 long).
  The count is stored as a long only if it is >= 16. Smaller counts
  are stored in the status part.
\item Optionally: Size, i.e. length of memory part in bytes (1 long).
  The size is stored as a long only if it is >= 16. Smaller sizes
  are stored in the status word.
\item Optionally: Refs, i.e. number of counted references to the node (1
  long). The refs are stored only for nodes allocated on the heap.
\end{itemize}
Thus the length of a compact header may vary between 2 and 5 words.
*/

class MMMTheader
{
private:
    unsigned long count_ : 4;       // no of children if < 16
    unsigned long big_count_ : 1;   // 1 iff more than 15 children
    unsigned long static_ : 1;      // 1 iff not allocated on the heap
    unsigned long const_ : 1;       // 1 iff read-only node
    unsigned long big_size_ : 1;    // 1 iff memory part has > 15 bytes
    unsigned long type_ : 8;        // type of node

    unsigned long header_len_ : 3;  // length of header (in words)
    unsigned long is_blk_ : 1;      // 1 iff node is stored on page
    unsigned long size_ : 4;        // size of memory part if < 16
    unsigned long blk_idx_ : 4;     // idx of page size where node is stored
    unsigned long mark_ : 1;        // mark
    unsigned long filler_ : 3;

    union {
        long signature_[1];	   // signature
	MMMTheader *next_free_;	   // pointer to next free block
    };

    enum { MAX_TYPE = 0xff };    // max no of types
    enum { MAX_BLK = 0xf };     // max block index
    enum { MAX_SCOUNT = 0xf };  // max count for 'small count headers'
    enum { MAX_SSIZE = 0xf };   // max size for 'small size headers'

public:
    static long getLength(long count, long size);

    void init(long s, long c, long t, bool b, unsigned long i);
    void initFree(MMMTheader *next_free, unsigned long block_idx);
    long getSignature() const;
    void setSignature(long sig);
    long getSize() const;
    bool setSize(long s);
    long getCount() const;
    long headerLen() const;
    int release();
    long& refs();
    const long& refs() const;
    void addRef();
    long getRefs() const;
    long getType() const;
    void setType(long t);
    bool isMarked() const;
    void setMark(bool is_marked=true);
    bool isBlock() const;
    unsigned long getBlockIdx() const;
    MMMTheader *getNextFree() const;
    void setNextFree(MMMTheader *s);
    void signDead();
    bool isDead();
#ifdef MMMD_LIT_SP
    bool isStatic() const;
    bool isConst() const;
#endif
#ifdef MMMD_GEN_LIT_SP
    void printLiteralHeader(const char*, bool, FILE*);
#endif
};

#else // !MMMD_COMPACT

class MMMTheader
{
private:
    unsigned long type_ : 16;     // MMMM-Typ
    unsigned long static_ : 1;    // 1 falls _nicht_ auf dem Heap alloziiert
    unsigned long const_ : 1;     // 1 falls konstant (im ROM)
    unsigned long mark_ : 1;      // fuer Markierung
    unsigned long is_blk_ : 1;    // 1 falls der Block regulaer in einer Page
			          // gespeichert ist
    unsigned long blk_idx_ : 4;   // Block-Index (falls is_blk_=1)
    unsigned long filler_ : 8;

    union {
        long signature_;	   // Signatur
	MMMTheader *next_free_;	   // Zeiger auf naechsten freien Block
    };
    long size_;		           // Groesse des Mem-Teils
    long count_;		   // Anzahl Kinder (S_Pointer)
    long refs_;		           // Referenz-Zaehler

    enum { MAX_TYPE = 0xff };      // max no of types
    enum { MAX_BLK = 0xf };        // max block index

public:
    static long getLength(long count, long size);

    void init(long s, long c, long t, bool b, unsigned long i);
    void initFree(MMMTheader *next_free, unsigned long block_idx);
    long getSignature() const;
    void setSignature(long sig);
    long getSize() const;
    bool setSize(long s);
    long getCount() const;
    int release();
    void addRef();
    long getRefs() const;
    long getType() const;
    void setType(long t);
    bool isMarked() const;
    void setMark(bool is_marked=true);
    bool isBlock() const;
    unsigned long getBlockIdx() const;
    MMMTheader *getNextFree() const;
    void setNextFree(MMMTheader *s);
    void signDead();
    bool isDead();
#ifdef MMMD_LIT_SP
    bool isStatic() const;
    bool isConst() const;
#endif
#ifdef MMMD_GEN_LIT_SP
    void printLiteralHeader(const char*, bool, FILE*);
#endif
};

#endif // !MMMD_COMPACT

#ifdef MMMD_LIT_SP

inline bool
MMMTheader::isStatic() const
{
    MMMASSERT(this != NULL);
    return (bool) static_;
}

inline bool
MMMTheader::isConst() const
{
    MMMASSERT(this != NULL);
    return (bool) const_;
}

#endif // MMMD_LIT_SP

#ifdef MMMD_COMPACT

inline long
MMMTheader::getSignature() const
{
    MMMASSERT(this != NULL);
    return signature_[0];
}

inline void
MMMTheader::setSignature(long sig)
{
    MMMASSERT(this != NULL);
#ifndef MMMD_LIT_SP
    signature_[0] = sig;
#else // MMMD_LIT_SP
    // some functions (MSU_eval_subs_intern for example) wildly
    // call MSG_sig even if the object was not created from
    // scratch. so we do not throw an error if they try to set
    // the signature of constant literals
    if (! isConst()) {
	signature_[0] = sig;
    }
#endif // MMMD_LIT_SP
}

inline long
MMMTheader::getSize() const
{
    MMMASSERT(this != NULL);
    if (big_size_) {
        // is either the 3rd or 4th word in the header
        return signature_[ big_count_ ? 2 : 1 ];
    }
    return (long) size_;
}

inline long
MMMTheader::getCount() const
{
    MMMASSERT(this != NULL);
    if (big_count_) {
        // is allways the 3rd word in the header
        return signature_[1];
    }
    return (long) count_;
}

inline long MMMTheader::headerLen() const
{
    return header_len_;
}

inline long& MMMTheader::refs()
{
#ifdef MMMD_LIT_SP
    MMMASSERT(! isStatic());
#endif
    return signature_[header_len_-2];
}

inline const long& MMMTheader::refs() const
{
#ifdef MMMD_LIT_SP
    MMMASSERT(! isStatic());
#endif
    return signature_[header_len_-2];
}

inline int
MMMTheader::release()
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    if (isStatic())
        return false;
#endif
    // is allways the last word in the header
    long& r = refs();

    MMMASSERT(r != 0);
    if (r == 0) {
	return false;		// object has already been freed,
				// avoid infinite recursion
    } else {
	r--;
        return (r == 0);
    }
}

inline void
MMMTheader::addRef()
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    if (isStatic())
        return;
#endif
    refs()++;
}

inline long
MMMTheader::getRefs() const
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    if (isStatic())
        return 0xffff;
#endif
    return refs();
}

#else // !MMMD_COMPACT

inline long
MMMTheader::getSignature() const
{
    MMMASSERT(this != NULL);
    return signature_;
}

inline void
MMMTheader::setSignature(long sig)
{
    MMMASSERT(this != NULL);
#ifndef MMMD_LIT_SP
    signature_ = sig;
#else // MMMD_LIT_SP
    // some functions (MSU_eval_subs_intern for example) wildly
    // call MSG_sig even if the object was not created from
    // scratch. so we do not throw an error if they try to set
    // the signature of constant literals
    if (! isConst()) {
	signature_ = sig;
    }
#endif // MMMD_LIT_SP
}

inline long
MMMTheader::getSize() const
{
    MMMASSERT(this != NULL);
    return size_;
}

inline long
MMMTheader::getCount() const
{
    MMMASSERT(this != NULL);
    return count_;
}

inline int
MMMTheader::release()
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    if (isStatic())
        return false;
#endif
    MMMASSERT(refs_ != 0);
    if (refs_ == 0) {
	return false;		// object has already been freed,
				// avoid infinite recursion
    } else {
	refs_--;
        return (refs_ == 0);
    }
}

inline void
MMMTheader::addRef()
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    if (! isStatic())
        refs_++;
#else
    refs_++;
#endif
}

inline long
MMMTheader::getRefs() const
{
    MMMASSERT(this != NULL);
    return refs_;
}

#endif // !MMMD_COMPACT

inline long
MMMTheader::getType() const
{
    MMMASSERT(this != NULL);
    return type_;
}

inline void
MMMTheader::setType(long t)
{
    MMMASSERT(this != NULL);
    MMMASSERT(t <= MAX_TYPE);
#ifdef MMMD_LIT_SP
    MMMASSERT(! isConst());
#endif
    type_ = t;
}

inline bool
MMMTheader::isMarked() const
{
    MMMASSERT(this != NULL);
    return (bool) mark_;
}

inline void
MMMTheader::setMark(bool is_marked)
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    MMMASSERT(! isConst());
#endif
    mark_ = is_marked;
}

inline bool
MMMTheader::isBlock() const
{
    MMMASSERT(this != NULL);
    return (bool) is_blk_;
}

inline unsigned long
MMMTheader::getBlockIdx() const
{
    MMMASSERT(this != NULL);
    return blk_idx_;
}

inline MMMTheader *
MMMTheader::getNextFree() const
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    MMMASSERT(! isStatic());
#endif
    return next_free_;
}

inline void
MMMTheader::setNextFree(MMMTheader *s)
{
    MMMASSERT(this != NULL);
#ifdef MMMD_LIT_SP
    MMMASSERT(! isStatic());
#endif
#ifdef MMMDEBUG
    setType(0);
#endif
    next_free_ = s;
}

////////////////////////////////////////////////////////////////////////
// MMMTbigHeader
// envelopes headers which are not stored on pages

class MMMTbigHeader {
public:
    static const long bigHeaderOverhead;
    MMMTbigHeader* prev;
    MMMTbigHeader* next;
    MMMTheader header;

    void init(MMMTbigHeader* n, long s, long c, long t);
};

////////////////////////////////////////////////////////////////////////
// Zugriff auf den Inhalt

#ifdef MMMD_COMPACT

inline S_Pointer*
MMMgetPtrAdr(S_Pointer s, long i)
{
    MMMASSERT(s != NULL);
    return (S_Pointer*)s + s->headerLen() + i;
}

inline char*
MMMgetMemAdr(S_Pointer s)
{
    MMMASSERT(s != NULL);
    return (char*)((S_Pointer*)s + s->headerLen() + s->getCount());
}

#else // !MMMD_COMPACT

inline S_Pointer*
MMMgetPtrAdr(S_Pointer s, long i)
{
    MMMASSERT(s != NULL);
    S_Pointer *p = (S_Pointer *)(s+1);
    return p+i;
}

inline char*
MMMgetMemAdr(S_Pointer s)
{
    MMMASSERT(s != NULL);
    S_Pointer *q = (S_Pointer *)(s + 1);
    char *p = (char *)(q + s->getCount());
    return p;
}

#endif // !MMMD_COMPACT

inline char*
MMMgetMemAdr(S_Pointer s, long mv)
{
    return MMMgetMemAdr(s) + mv;
}

////////////////////////////////////////////////////////////////////////
// Allokieren von Speicher

inline char*
MMMEmalloc(long size)
{
    extern MMMTno_memory MMMVno_memory;

    char *p = (char *) MMMsysIrrMalloc(size);
    if (p == NULL)
	(*MMMVno_memory)();
    return p;
}

inline void
MMMEfree(char *p)
{
    MMMsysIrrFree((void *) p);
}


////////////////////////////////////////////////////////////////////////
// Freigabe

inline void
MMMfree(S_Pointer s)
{
    if (s != MMMNULL && s->release()) {
	MMMrecFree(s);
    }
}

inline void
MMMlfree(S_Pointer s)
{
    if (s != MMMNULL && s->release()) {
	MMMfreeBlock(s);
    }
}

inline bool
MMMfree_unmarked(S_Pointer p)
{
    if (p == MMMNULL)
	return true;
#ifdef MMMD_LIT_SP
    else if (p->isStatic() || p->isMarked())
#else
    else if (p->isMarked())
#endif
        return false;
    else {
        MMMfreeUnmarked(p);
        return true;
    }
}

////////////////////////////////////////////////////////////////////////
// Vergleichen und Kopieren                                           //
////////////////////////////////////////////////////////////////////////

inline int
MMMequal(S_Pointer s, S_Pointer t)
{
    return s == t;
}

inline void
MMMcopy(S_Pointer s)
{
    if (s != NULL) s->addRef();
}

inline void
MMMset(S_Pointer *p, S_Pointer s)
{
    MMMASSERT(p != NULL);
    MMMcopy(s);
    *p = s;
}

inline void
MMMreplace(S_Pointer *p, S_Pointer s)
{
    MMMASSERT(p != NULL);
    MMMcopy(s);
    MMMfree(*p);
    *p = s;
}

////////////////////////////////////////////////////////////////////////
// Zugriff auf Statusinformationen                                    //
////////////////////////////////////////////////////////////////////////

inline long
MMMsize(S_Pointer s)
{
    return ((s == MMMNULL) ? 0 : s->getSize());
}

inline long
MMMcounter(S_Pointer s)
{
    return ((s == MMMNULL) ? 0 : s->getCount());
}

inline long
MMMtype(S_Pointer s)
{
    MMMASSERT(s != NULL);
    return s->getType();
}

inline long
MMMsignature(S_Pointer s)
{
    MMMASSERT(s != NULL);
    return s->getSignature();
}

inline void
MMMnewsignature(S_Pointer s, long sig)
{
    MMMASSERT(s != NULL);
    s->setSignature(sig);
}

inline bool
MMMunmoveable(S_Pointer, bool)
{
    return true;
}

inline long
MMMrefs(S_Pointer s)
{
    MMMASSERT(s != NULL);
    return s->getRefs();
}

////////////////////////////////////////////////////////////////////////
// Poolverwaltung                                                     //
////////////////////////////////////////////////////////////////////////

inline void MMMpoolinit() {}
#define MMMpoolinsert(p)
#define MMMpooldelete(p)

////////////////////////////////////////////////////////////////////////
// Hilfsfunktionen                                                    //
////////////////////////////////////////////////////////////////////////

#if (! defined MSDOS) && (! defined MACINTOSH)

// MMMsysIrrMalloc -- alloziiert irregulaeren (Non-Page) Bereich
// mit Hilfe der "uebergeordneten" Speicherverwaltung.

inline void *
MMMsysIrrMalloc(long len)
{
    return malloc(len);
}

inline void
MMMsysIrrFree(void *p)
{
    free(p);
}

#endif

////////////////////////////////////////////////
// Generieren von Literalen

#ifdef MMMD_GEN_LIT_SP

// generate code for literal S-Pointers
void MMMgen_lit();

/// Linked list of literal S-Pointer variable references, used to
/// generate code to initialize them.
class MMMT_literal
{
public:
    /// Create literal S-Pointer variable reference given the address of
    /// the S-Pointer and its variable name.
    MMMT_literal(S_Pointer* spp, const char* name);

    static MMMT_literal* first();
    S_Pointer s_pointer() const;
    S_Pointer* spp() const;
    const char* name() const;
    MMMT_literal* next() const;
private:
    static MMMT_literal* first_;    // first reference in list
    S_Pointer* spp_;                // address of S-Pointer which is referenced
    const char* name_;              // C-variable name of the S-Pointer
    MMMT_literal* next_;            // next reference in list
};

inline MMMT_literal* MMMT_literal::first() { return first_; }
inline S_Pointer MMMT_literal::s_pointer() const { return *spp_; }
inline S_Pointer* MMMT_literal::spp() const { return spp_; }
inline const char* MMMT_literal::name() const { return name_; }
inline MMMT_literal* MMMT_literal::next() const { return next_; }

#endif // MMMD_GEN_LIT_SP

////////////////////////////////////////////////
// Iterator ueber alle Knoten (fuer Statistiken)
////////////////////////////////////////////////

// Iterate ofer all allocated nodes which are not freed, wether they are
// available from the root set or not.
void MMMvisit_all(void (*)(S_Pointer, bool, void*), void*);

#ifdef MMMD_STATISTICS

// Iterate depth-first over all nodes which are alive, ie. may be reached
// from the root set.
void MMMvisit_live(void (*)(S_Pointer, bool, void*), void*);

#endif // MMMD_STATISTICS

#endif /* !__MMMstorage_lean__ */

