/*************************************************************************
**                         MNS_namespace.h                              **
** In diesem File werden die Routinen definiert, mit denen Namespaces   **
** benutzt werden k"onnen.                                              **
** Autor            :  Holger Naundorf                                  **
** Erstellungsdatum :  04/09/95                                         **
** Letzte Aenderung :  27/09/95                                         **
**                                                                      **
** 'namespace' scheint fuer den g++ ein reservierter Bezeichner zu sein **
** und darf daher nicht verwendet werden (==> 'namspace').              **
*************************************************************************/

#ifndef __MNS_namespace__
#define __MNS_namespace__

#define MNS_STD_IDENT 1
#define MNS_WRITE_IDENT 2
#define MNS_READ_WRITE_IDENT 3

#ifdef MMMD_LIT_SP
extern S_Pointer MNSV_global_lit_namespace;
#endif

void MNS_init();
void MNS_reset();
void MNS_exit();
void MNS_free_unused_idents();

S_Pointer MNS_new_ident(S_Pointer name, long ident_typ = MNS_STD_IDENT);
S_Pointer MNS_get_ident(S_Pointer name);

S_Pointer MNS_new_client_context();
void MNS_delete_client_context(S_Pointer ctx);
void MNS_activate_client_context(S_Pointer ctx, bool activate);

/*************************************************************************
** S_Pointer MNS_ident_name(S_Pointer ident)                            **
** Die Funktion liefert zu einem Identifier den Namen. Bei diesem       **
** Namen kann es sich um einen beliebigen S-Zeiger handeln.             **
*************************************************************************/
inline S_Pointer MNS_ident_name(S_Pointer ident)
{
   S_Pointer name = *MMMP(&ident, 0) ;
   MMMcopy(name) ;
   return name ;
}


/** Iterates over all identifiers of the global namespace.
 *
 * May only be used for reading the namespace entries.
 */

class MNST_ident_iterator {
public:
    /** Create iterator for identifiers in the global namespace.
     * Moves the iterator to the first identifier. */
    MNST_ident_iterator();

    /// Destroy iterator.
    ~MNST_ident_iterator();

    /** Moves the iterator to the next identifier.
     * @return false if no more identifiers exist. */
    bool next();

    /** Test if iterator is located behind last identifier.
     * @return true if no more identifiers exist to iterate over. */
    bool at_end();

    /** Get the actual identifier.
     * The s-pointer returned is not copied!
     * @return a s-pointer to the actual identifier. */
    S_Pointer ident();

private:
    // iterator for the ass. memory holding the idents
    MAMT_iterator mam_iterator_;
#ifdef MMMD_LIT_SP
    bool literals_;
#endif
};

#ifndef MMMD_LIT_SP
inline bool MNST_ident_iterator::next()
{
    return mam_iterator_.next();
}

inline bool MNST_ident_iterator::at_end()
{
    return mam_iterator_.at_end();
}
#endif

inline S_Pointer MNST_ident_iterator::ident()
{
    return mam_iterator_.value();
}

#endif /* __MNS_namespace__ */
