#ifndef __MPE_environ_h__
#define __MPE_environ_h__
/*
 * MPE_proc_env.h -- Routinen fuer Umgebungen, Variablen und Bezeichner
 */

#include "MDB_debug.h"


/* Struct mit den Informationen im Mem-Teil einer Umgebung.
 */
typedef struct MPET_info_tag {
    long local_idx;		/* Index der ersten lokalen Var. */
    long args_idx;		/* Index des ersten Parameters */
    long args0;			/* Anzahl aktueller Parameter */
    long hist_idx;		/* Index des naechten History-Eintrags */
#if (! defined GRAPHIC && defined DEBUG_ON)
    DBX_TYPE last_dbx;		/* Debug-Info */
#endif
} MPET_info;

/* Routinen zur Erfragung der entspr. Eintraege in einer Umgebung.
 * Rueckgabewerte sind NICHT kopiert!
 */
S_Pointer MPE_get_proc(S_Pointer env);
S_Pointer MPE_get_procname(S_Pointer env);
S_Pointer MPE_get_lex_env(S_Pointer env);
S_Pointer MPE_get_dyn_env(S_Pointer env);
S_Pointer MPE_get_domain(S_Pointer env);

/* Liefern die Werte von Variablen bzw. Argumenten einer Umgebung.
 * Rueckgabewerte sind kopiert.
 */
S_Pointer MPE_env_get_var(S_Pointer env, long idx);
long MPE_env_num_actuals(S_Pointer env);
S_Pointer* MPE_env_args1_ptr(S_Pointer env);
S_Pointer* MPE_env_saved1_ptr(S_Pointer env);
S_Pointer MPE_env_get_arg(S_Pointer env, long idx);
#if (! defined GRAPHIC && defined DEBUG_ON)
DBX_TYPE MPE_env_get_dbx(S_Pointer env);
void MPE_env_set_dbx(S_Pointer env, DBX_TYPE dbx);
#endif

/* History-Verwaltung
 */
S_Pointer MPE_env_get_last(S_Pointer env, long idx);
void MPE_env_set_history(S_Pointer env, S_Pointer val);
void MPE_env_get_history(S_Pointer env, S_Pointer *hist1, 
			 S_Pointer *hist2, S_Pointer *hist3,
			 long *idx);
void MPE_env_restore_history(S_Pointer env, S_Pointer hist1, 
			 S_Pointer hist2, S_Pointer hist3,
			 long idx);


/* Setzen von Umgebungs-Werten.
 */
S_Pointer MPE_replace_dyn_env(S_Pointer env, S_Pointer dom);
S_Pointer MPE_replace_domain(S_Pointer env, S_Pointer dom);

/* Restaurieren von gesicherten Bezeichnern
 */
void MPE_restore_saved(S_Pointer env);

/* Freigabe der aktuellen Umgebung.
 */
bool MPE_free_env(S_Pointer env, bool allow_escape);

/* Routinen fuer Variablen.
 */
void MPE_var_get_dist_idx(S_Pointer v, long* dist, long* idx);
S_Pointer MPE_var_replace(S_Pointer var, S_Pointer val,
			  S_Pointer env = MMMNULL);
bool MPE_var_assign(S_Pointer var, S_Pointer val,
		       S_Pointer env = MMMNULL);
S_Pointer MPE_var_get_val(S_Pointer var, S_Pointer env = MMMNULL);
S_Pointer MPE_var_get_val_nowarn(S_Pointer var, bool* err,
				 S_Pointer env = MMMNULL);
S_Pointer MPE_var_idx2ident(long idx, S_Pointer formals, S_Pointer locals);
S_Pointer MPE_var_idx2ident(long idx, S_Pointer proc);

/* Routinen zur Zuweisung / zum Lesen von Variablen und Bezeichnern
 */
void MPE_write_val(S_Pointer var, S_Pointer val,
		   MIDT_option option = MIDC_option_standard);
S_Pointer MPE_read_val(S_Pointer var);
S_Pointer MPE_read_val_nowarn(S_Pointer var);

/* Hilfsroutinen
 */
bool MPE_want_unitialized_var_warnings(bool warn);
S_Pointer MPE_env_warn_unitialized_var(S_Pointer env, long idx);

//---------------------------- Implementation ------------------------------

inline S_Pointer MPE_get_proc(S_Pointer env)
{
    S_Pointer s;
    if (env == MMMNULL) {
	s = MMMNULL;
    } else {
	s = *MMMP(&env, 0);
    }
    return s;
}

inline S_Pointer MPE_get_procname(S_Pointer env)
{
    S_Pointer s;
    if (env == MMMNULL) {
	s = MMMNULL;
    } else {
	long idx = (MMMZ(&env, MPET_info)->local_idx);
	s = *MMMP(&env, idx);
    }
    return s;
}

inline S_Pointer MPE_get_lex_env(S_Pointer env)
{
    S_Pointer s;
    if (env == MMMNULL) {
	s = MMMNULL;
    } else {
	s = *MMMP(&env, 1);
    }
    return s;
}

inline S_Pointer MPE_get_dyn_env(S_Pointer env)
{
    S_Pointer s;
    if (env == MMMNULL) {
	s = MMMNULL;
    } else {
	s = *MMMP(&env, 2);
    }
    return s;
}

inline S_Pointer MPE_replace_dyn_env(S_Pointer env, S_Pointer dyn)
{
    S_Pointer s;
    if (env == MMMNULL) {
	s = MMMNULL;
    } else {
	s = *MMMP(&env, 2);
	*MMMP(&env, 2) = dyn;
	if (dyn != MMMNULL) MMMcopy(dyn);
    }
    return s;
}

inline S_Pointer MPE_get_domain(S_Pointer env)
{
    S_Pointer s;
    if (env == MMMNULL) {
	s = MMMNULL;
    } else {
	long idx = (MMMZ(&env, MPET_info)->local_idx) + 1;
	s = *MMMP(&env, idx);
    }
    return s;
}

inline S_Pointer MPE_replace_domain(S_Pointer env, S_Pointer dom)
{
    S_Pointer d;
    if (env == MMMNULL) {
	d = MMMNULL;
    } else {
	long idx = (MMMZ(&env, MPET_info)->local_idx) + 1;
	d = *MMMP(&env, idx);
	MMMset(MMMP(&env, idx), dom);
    }
    return d;
}

inline S_Pointer MPE_env_get_var(S_Pointer env, long idx)
{
    S_Pointer v;
    if (env == MMMNULL) {
	v = MMMNULL;
    } else {
	idx += (MMMZ(&env, MPET_info)->local_idx);
	if (idx >= (long) MMMcounter(env)) {
	    v = MMMNULL;
	} else {
	    v = *MMMP(&env, idx);
	    if (v == MMMNULL)
		v = MPE_env_warn_unitialized_var(env, idx);
	    MMMcopy(v);
	}
    }
    return v;
}

inline long MPE_env_num_actuals(S_Pointer env)
{
    long n = MMMZ(&env, MPET_info)->args0;
    return n;
}

#if (! defined GRAPHIC && defined DEBUG_ON)
inline DBX_TYPE MPE_env_get_dbx(S_Pointer env)
{
    DBX_TYPE dbx;
    if (env == MMMNULL) {
	dbx = 0;
    } else {
	dbx = MMMZ(&env, MPET_info)->last_dbx;
    }
    return dbx;
}

inline void MPE_env_set_dbx(S_Pointer env, DBX_TYPE dbx)
{
    assert(env != MMMNULL);
    if (env == MMMNULL) {
	return ;
    } else {
        MMMZ(&env, MPET_info)->last_dbx = dbx;
    }
}
#endif

inline S_Pointer* MPE_env_args1_ptr(S_Pointer env)
{
    long n = MMMZ(&env, MPET_info)->args_idx;
    S_Pointer* ap = MMMP(&env, n);
    return ap;
}

inline S_Pointer* MPE_env_saved1_ptr(S_Pointer env)
{
    S_Pointer* sp = MMMP(&env, 6);
    return sp;
}

inline S_Pointer MPE_env_get_arg(S_Pointer env, long idx)
{
    S_Pointer v;
    if (env == MMMNULL) {
	v = MMMNULL;
    } else {
	idx += (MMMZ(&env, MPET_info)->args_idx);
	if (idx >= (long) MMMcounter(env)) {
	    v = MMMNULL;
	} else {
	    v = *MMMP(&env, idx);
	    if (v == MMMNULL)
		v = MPE_env_warn_unitialized_var(env, idx);
	    MMMcopy(v);
	}
    }
    return v;
}

inline S_Pointer MPE_env_get_last(S_Pointer env, long idx)
{
    S_Pointer v;
    if (env == MMMNULL) {
	v = MMMNULL;
    } else {
	idx = ((MMMZ(&env, MPET_info)->hist_idx + 3 - idx) % 3) + 3;
	MMMset(&v, *MMMP(&env, idx));
    }
    return v;
}

inline void MPE_env_set_history(S_Pointer env, S_Pointer val)
{
    long idx = MMMZ(&env, MPET_info)->hist_idx;
    MMMreplace(MMMP(&env, 3 + idx), val);
    MMMZ(&env, MPET_info)->hist_idx = (idx + 1) % 3;
}

inline void MPE_env_get_history(S_Pointer env, S_Pointer *hist1, 
			 S_Pointer *hist2, S_Pointer *hist3,
			 long *idx)
{
  *idx = MMMZ(&env, MPET_info)->hist_idx;
  *hist1 = *MMMP(&env, 3);  MMMcopy(*hist1);
  *hist2 = *MMMP(&env, 4);  MMMcopy(*hist2);
  *hist3 = *MMMP(&env, 5);  MMMcopy(*hist3);
}


inline void MPE_env_restore_history(S_Pointer env, S_Pointer hist1, 
			 S_Pointer hist2, S_Pointer hist3,
			 long idx)
{
  MMMZ(&env, MPET_info)->hist_idx = idx;
  MMMfree(*MMMP(&env, 3));  *MMMP(&env, 3) = hist1; 
  MMMfree(*MMMP(&env, 4));  *MMMP(&env, 4) = hist2; 
  MMMfree(*MMMP(&env, 5));  *MMMP(&env, 5) = hist3; 
}


inline void MPE_var_get_dist_idx(S_Pointer v, long* dist, long* idx)
{
    S_Pointer t;
    t = *MMMP(&v, 0);
    *dist = *MMMZ(&t, long);
    *idx = *MMMarr(&t, 1, long);

    MMMASSERT(*dist >= 0 && *idx >= 0);
}

inline bool MPE_var_assign(S_Pointer var, S_Pointer val, S_Pointer env)
{
    S_Pointer res = MPE_var_replace(var, val, env);
    if (res == MMMNULL) {
	return false;
    } else {
	MMMfree(res);
	return true;
    }
}

#endif
