/* $Date: 1999/09/29 08:09:21 $  $Author: kg $  $Revision: 1.10 $ */

/**************************************************************************/
/* FILE:             MPO_poly_help.h                                      */
/* Autor:            Klaus Gottheil                                       */
/* Erstellt:         23/04/93                                             */
/* Letzte Aenderung: 20/12/94                                             */
/**************************************************************************/

#ifndef __MPO_poly_help__
#define __MPO_poly_help__


/*----------------------------------------------------------------------
 * Konstanten
 */

#define MPOC_EXPR        0     /* Koeffizienten-Ring Expr */
#define MPOC_INTMOD      1     /* Koeffizienten-Ring IntMod(...) */
#define MPOC_DOM         2     /* Koeffizienten-Ring ist ein Domain */

#define MPOC_NOORD       0	/* unbekannte Ordnung */
#define MPOC_LEXORD      1	/* lexikographische Ordnung */
#define MPOC_DEGORD      2	/* totale Gradordnung */
#define MPOC_DEGINVLEX   3	/* umgek. lexikographische Ordnung */

#define MPOC_MAX_NORM	 0	/* Maximums-Norm */
#define MPOC_ONE_NORM	 1	/* 1-Norm */
#define MPOC_N_NORM	 2	/* n-Norm */

#define MPOC_NOT_ZERO      0    /* Koeffizient ist nicht 0 */
#define MPOC_IS_ZERO       1    /* Koeffizient ist 0 (aber nicht 0.0 "uber Expr) */
#define MPOC_IS_FLOAT_ZERO 2    /* Koeffizient ist 0.0 "uber Expr */

#define MPOC_no_name "<<unknown>>"


/*----------------------------------------------------------------------
 * Typen
 */

class MPOT_poly_type;

/* Struktur mit Zeigern auf die Grund-Operationen fuer Koeffizienten-Ringe. */

typedef struct _MPOT_coeff_ops {

    /* Umwandlung von Ausdruecken in Koeffizienten */
    S_Pointer (*expr2coeff)(S_Pointer, MPOT_poly_type *);
    /* Umwandlung von Koeffizienten in Ausdruecken */
    S_Pointer (*coeff2expr)(S_Pointer, MPOT_poly_type *);
    /* Addition von Koeffizienten */
    S_Pointer (*plus)(S_Pointer, S_Pointer, MPOT_poly_type *);
    /* Multiplikation von Koeffizienten */
    S_Pointer (*mult)(S_Pointer, S_Pointer, MPOT_poly_type *);
    /* Division von Koeffizienten */
    S_Pointer (*divide)(S_Pointer, S_Pointer, MPOT_poly_type *);
    /* Potenzieren von Koeffizienten */
    S_Pointer (*power)(S_Pointer, long, MPOT_poly_type *);
    /* Negieren von Koeffizienten */
    S_Pointer (*negate)(S_Pointer, MPOT_poly_type *);
    /* Null-Test von Koeffizienten */
    int (*iszero)(S_Pointer, MPOT_poly_type *);

} MPOT_coeff_ops;

/* Typ-Struktur fuer Polynome, enthaelt ua. den Koeffizienten-Ring.
 * Die originale Polynom-Struktur wird in diese Struktur umgewandelt, damit
 * ein schnellerer Zugriff auf die Informationen moeglich ist. */

class MPOT_poly_type {
public:
    MPOT_poly_type();
    ~MPOT_poly_type();
    void init_expr(long nind);
    void init_imod(long nind, S_Pointer mod);
    void init_dom(long nind, S_Pointer dom);

    long nindets() const;
    void add_nindets(long inc);

    long ring();
    S_Pointer dom();
    S_Pointer mod();

    GEN gmod();

    S_Pointer Zero();
    S_Pointer one();

    /* coefficient operations */
    S_Pointer expr2coeff(S_Pointer);
    S_Pointer coeff2expr(S_Pointer);
    S_Pointer plus(S_Pointer, S_Pointer);
    S_Pointer mult(S_Pointer, S_Pointer);
    S_Pointer divide(S_Pointer, S_Pointer);
    S_Pointer power(S_Pointer, long);
    S_Pointer negate(S_Pointer);
    int iszero(S_Pointer);

    /* get dummy-expressions */
    S_Pointer* plus();
    S_Pointer* mult();
    S_Pointer* power();
    S_Pointer* negate();
    S_Pointer* divide();
    S_Pointer* intmult();
    S_Pointer* norm();
    S_Pointer* convert();
    S_Pointer* expr();
    S_Pointer* iszero();

private:
    long nindets_;              /* Anzahl der Unbestimmten */
    long ring_;                 /* der Koeffizienten-Ring */
    MPOT_coeff_ops* ops_;       /* coefficient ops func pointers */
    S_Pointer dom_;             /* das Domain */
    S_Pointer mod_;             /* der Modulus bei IntMod */

    GEN gmod_;                  /* der Modulus als PARI-Zahl */

    S_Pointer Zero_;            /* Die 0 im Koeffizienten-Ring */
    S_Pointer one_;             /* Die 1 im Koeffizienten-Ring */

    S_Pointer plus_;		/* Dummy-Aufruf der Methode _plus */
    S_Pointer mult_;		/* Dummy-Aufruf der Methode _mult */
    S_Pointer power_;		/* Dummy-Aufruf der Methode _power */
    S_Pointer negate_;		/* Dummy-Aufruf der Methode negate */
    S_Pointer divide_;		/* Dummy-Aufruf der Methode divide */
    S_Pointer intmult_;		/* Dummy-Aufruf der Methode intmult */
    S_Pointer norm_;		/* Dummy-Aufruf der Methode norm */
    S_Pointer convert_;		/* Dummy-Aufruf der Methode convert */
    S_Pointer expr_;   		/* Dummy-Aufruf der Methode expr */
    S_Pointer iszero_;   	/* Dummy-Aufruf der Methode iszero */
};

inline MPOT_poly_type::MPOT_poly_type() : ring_(-1) { }

inline long MPOT_poly_type::nindets() const { 
    assert(ring_ != -1);
    return nindets_; 
}
inline long MPOT_poly_type::ring() {
    assert(ring_ != -1);
    return ring_; 
}
inline S_Pointer MPOT_poly_type::dom() { 
    assert(ring_ == MPOC_DOM);
    return dom_; 
}
inline S_Pointer MPOT_poly_type::mod() { 
    assert(ring_ == MPOC_INTMOD);
    return mod_; 
}
inline GEN MPOT_poly_type::gmod() { 
    assert(ring_ == MPOC_INTMOD);
    return gmod_; 
}
inline S_Pointer MPOT_poly_type::Zero() {
    assert(ring_ != -1);
    return Zero_; 
}
inline S_Pointer MPOT_poly_type::one() {
    assert(ring_ != -1);
    return one_; 
}

inline S_Pointer MPOT_poly_type::expr2coeff(S_Pointer s) {
    assert(ring_ != -1);
    return (*(ops_->expr2coeff))(s, this);
}
inline S_Pointer MPOT_poly_type::coeff2expr(S_Pointer s) {
    assert(ring_ != -1);
    return (*(ops_->coeff2expr))(s, this);
}
inline S_Pointer MPOT_poly_type::plus(S_Pointer s, S_Pointer p) {
    assert(ring_ != -1);
    return (*(ops_->plus))(s, p, this);
}
inline S_Pointer MPOT_poly_type::mult(S_Pointer s, S_Pointer p) {
    assert(ring_ != -1);
    return (*(ops_->mult))(s, p, this);
}
inline S_Pointer MPOT_poly_type::divide(S_Pointer s, S_Pointer p) {
    assert(ring_ != -1);
    return (*(ops_->divide))(s, p, this);
}
inline S_Pointer MPOT_poly_type::power(S_Pointer s, long n) {
    assert(ring_ != -1);
    return (*(ops_->power))(s, n, this);
}
inline S_Pointer MPOT_poly_type::negate(S_Pointer s) {
    assert(ring_ != -1);
    return (*(ops_->negate))(s, this);
}
inline int MPOT_poly_type::iszero(S_Pointer s) {
    assert(ring_ != -1);
    return (*(ops_->iszero))(s, this);
}


/* Position von Exponenten einer Unbestimmten im Datenteil eines Polynoms */

typedef struct _MPOT_expos {
    long exp;                           /* Exponent */
    long pos;                           /* Position im Datenteil */
} MPOT_expos;


/*----------------------------------------------------------------------
 * globale Variablen
 */

/* Funktionsname fuer Fehlermeldungen */

extern const char *MPOV_fname;

/* MMM-Strings fuer Optionen und Namen von Domain-Attributen */

MTR_DECL_LITERAL(MSTR_IntMod)
MTR_DECL_LITERAL(MSTR_Expr)
MTR_DECL_LITERAL(MSTR_LexOrder)
MTR_DECL_LITERAL(MSTR_DegreeOrder)
MTR_DECL_LITERAL(MSTR_DegInvLexOrder)
MTR_DECL_LITERAL(MSTR_Rem)
MTR_DECL_LITERAL(MSTR_Quo)
MTR_DECL_LITERAL(MSTR_Exact)

/* Strings fuer Namen von Domain-Attributen */

MTR_DECL_LITERAL(MPOV_str_zero)
MTR_DECL_LITERAL(MPOV_str_one)
MTR_DECL_LITERAL(MPOV_str_norm)
MTR_DECL_LITERAL(MPOV_str_plus)
MTR_DECL_LITERAL(MPOV_str_mult)
MTR_DECL_LITERAL(MPOV_str_divide)
MTR_DECL_LITERAL(MPOV_str_power)
MTR_DECL_LITERAL(MPOV_str_negate)
MTR_DECL_LITERAL(MPOV_str_intmult)
MTR_DECL_LITERAL(MPOV_str_convert)
MTR_DECL_LITERAL(MPOV_str_expr)
MTR_DECL_LITERAL(MPOV_str_diff)
MTR_DECL_LITERAL(MPOV_str_iszero)
MTR_DECL_LITERAL(MPOV_DOMAIN_ASS_POLY)

/* Identifier (CAT_IDENT) */

MTR_DECL_LITERAL(MPOV_id_IntMod)
MTR_DECL_LITERAL(MPOV_id_Expr)

/* Expressions (CAT_EXPR) */

extern S_Pointer MPOV_indet_call;

/*----------------------------------------------------------------------
 * Inlines
 */

/* aktueller Funktionsname fuer Fehlermeldungen setzen */
inline void MPO_fname(const char* s) { 
    MPOV_fname = s;
}

/* Makro fuer Fehlerausgaben mit MPOV_fname */
inline void MPO_error(long e) { 
    MEV_set_error(e, MPOV_fname);
}

/* TRUE falls 's' eine Zahl ist */
inline bool MPO_is_number(S_Pointer s) { 
    return MTR_cat(s) < CAT_STRING; 
}

/* TRUE falls MMM-String 's' derjenige fuer System-Funktion 'f' ist */
inline bool MPO_is_sys_func(S_Pointer s, int f) {
    return MCM_compare_MMMstring_ident(MTRV_SYS_FUNC_LIST[f], s) != 0;
}

// Tests if the data operand (op 1) of a polynomial represents the zero polynomial.
inline int MPO_is_zero_data(S_Pointer p, MPOT_poly_type * ptype) {
    if (MMMtype(p) == MMM_DIST_POLY) 
        return MPOC_NOT_ZERO;
    if (ptype->ring() == MPOC_EXPR && MTR_cat(p) == CAT_FLOAT)
        return MPOC_IS_FLOAT_ZERO;
    return MPOC_IS_ZERO;
}

// Tests if a polynomial coefficient is zero.
inline int MPO_is_zero_coeff(S_Pointer p, MPOT_poly_type * ptype) {
    return ptype->iszero(p);
}

/*----------------------------------------------------------------------
 * lokale Funktionen
 */

/* Funktionen in MPO_poly.c */

S_Pointer MPO_convert_coeffs(S_Pointer,
		S_Pointer (*)(S_Pointer, MPOT_poly_type *),
		MPOT_poly_type *, long, 
                MPOT_poly_type *);
S_Pointer MPO_poly(S_Pointer, S_Pointer, MPOT_poly_type *, bool *, S_Pointer);
S_Pointer MPO_nth_coeff_univar(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_nth_coeff_multivar(S_Pointer, long, long, S_Pointer *,
				 MPOT_poly_type *);
S_Pointer MPO_list2poly(S_Pointer, long, long, MPOT_poly_type *);

/* Funktionen in MPO_utils.c */

S_Pointer MPO_parse_indets(S_Pointer, long *);
bool   MPO_is_indet(S_Pointer);
bool   MPO_is_legal_indet(S_Pointer);
S_Pointer MPO_get_indets(S_Pointer *, long *, bool);
long	  MPO_parse_order(S_Pointer);

short     MPO_arg2poly(S_Pointer, long *, S_Pointer *, S_Pointer *,
		       MPOT_poly_type *);
short     MPO_2args2poly(S_Pointer, long *, S_Pointer *, S_Pointer *,
			 S_Pointer *, MPOT_poly_type *);
long      MPO_find_lterm(S_Pointer, MPOT_poly_type *, long);
void      MPO_get_type(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_new_poly_from_data(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_new_const_poly_data(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_new_lin_poly_data(long, MPOT_poly_type *);
S_Pointer MPO_new_type_expr(MPOT_poly_type *);
S_Pointer MPO_remove_coeff(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_remove_list_elem(S_Pointer, long);


/* Funktionen in MPO_arithm.c */

S_Pointer MPO_divide_univar(S_Pointer, S_Pointer, S_Pointer *, bool,
			    MPOT_poly_type *);
S_Pointer MPO_divide_multivar(S_Pointer, S_Pointer, S_Pointer *, bool,
                              MPOT_poly_type *);
S_Pointer MPO_pdivide_univar(S_Pointer, S_Pointer, long *, S_Pointer *,
			     bool, MPOT_poly_type *);
S_Pointer MPO_evalp_univar(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_evalp_multivar(S_Pointer, S_Pointer, long, S_Pointer *,
			     MPOT_poly_type *);
void      MPO_sort_expos(MPOT_expos *, long, long);
S_Pointer MPO_genpoly(S_Pointer, S_Pointer, long *, MPOT_poly_type *);
S_Pointer MPO_diff_coeffs(S_Pointer, MPOT_poly_type *, S_Pointer, S_Pointer, S_Pointer*);
S_Pointer MPO_diff_terms(S_Pointer, long, MPOT_poly_type *);

/* Funktionen in MPO_basics.c */

S_Pointer MPO_dist_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dist_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dist_power(S_Pointer, long, MPOT_poly_type *);
void      MPO_dist_negate(S_Pointer, MPOT_poly_type *);
long	  MPO_mult_long(long, long);
S_Pointer MPO_convert_float(S_Pointer, MPOT_poly_type *);

S_Pointer MPO_expr_expr2coeff(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_coeff2expr(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_divide(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_divide_exact(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_power(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_expr_negate(S_Pointer, MPOT_poly_type *);
int       MPO_expr_iszero(S_Pointer, MPOT_poly_type *);

S_Pointer MPO_imod_expr2coeff(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_coeff2expr(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
bool      MPO_imod_is_invertible(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_divide(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_power(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_imod_negate(S_Pointer, MPOT_poly_type *);
int       MPO_imod_iszero(S_Pointer, MPOT_poly_type *);

S_Pointer MPO_dom_expr2coeff(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_coeff2expr(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_intmult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_divide(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_power(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_dom_negate(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_norm(S_Pointer, MPOT_poly_type *);
int       MPO_dom_iszero(S_Pointer, MPOT_poly_type *);

/* werden zZt. nicht benoetigt:
long      MPO_dist_lex_excmp();
long      MPO_dist_deg_excmp();
*/

#endif /* __MPO_poly_help__ */
