/**************************************************************************
**                         MPR_protocol.h                                **
** In diesem File werden die Typen f"ur das Protokoll MPR... definiert.  **
** Au\3erdem werden die Schnittstellenfunktionen deklariert.             **
**									                                     **
** Werden nur die Typ-Konstanten fuer die Meldungen benoetigt, so ist    **
** das Makro MPR_TYPE_CONST zu definieren.				                 **
**                                                                       **
**   ALLE AENDERUNGEN MUESSEN IN DOC/PAPER/KERNEL/MPR_protocol.tex       **
**   DOKUMENTIERT WERDEN !!!!                                            **
**                                                                       **
**   DIESE DATEI WIRD AUCH IN DIE XVIEW-TOOLS EINGEBUNDEN.               **
**     =>  nur C-Syntax erlaubt, insbesondere keine C++ Kommentare !!!!  **
**************************************************************************/

#ifndef __MPR_protocol__
#define __MPR_protocol__

/**************************************************************************
** Typ-Konstanten fuer Meldungen                                         **
**************************************************************************/

#define MPRCillegal                0    /* unzulaessiger Typ fuer Fehlerpruefung */
#define MPRCinput                  1
#define MPRCoutput                 2
#define MPRCsystem                 3
#define MPRCgettextinput           4
#define MPRCgetinput               5
#define MPRCtextwidth              6
#define MPRCprettyprint            7
#define MPRChelp                   8
#define MPRCerror                  9
#define MPRCexample               10
#define MPRCmdxcommand            11
#define MPRCdisplayvariables      12
#define MPRCprompt				  13
#define MPRClongprompt      	  14
#define MPRCdebuggermode    	  15
#define MPRCmdxprompt             16
#define MPRC_mv_graphic_cmd       17
#define MPRC_vx_plot_cmd          18
#define MPRC_vm_plot_cmd          19
#define MPRC_mv_graphic_error     20
#define MPRC_vx_graphic_error     21
#define MPRCsignal                22      /* Nachricht ist Signal (s.u.) */
#define MPRCmdxoutput			  23
#define MPRCmdxerror			  24
#define MPRCsyntaxerror			  25
#define MPRCincompleteinput		  26
#define MPRCgetstatus             27
#define MPRCstop                  28      /* stoppt die aktuelle Berechnung,
                                           * der Kern kehrt wie bei Ctl-C auf interaktive
                                           * Ebene zurueck (zZt nur unter Windows) */
#define MPRCquit                  29      /* beendet den Kern wie mit Ctl-C und 'quit' */
#define MPRCconnect               30      /* koordiniert den Zugriff auf den Kern, falls
                                           * gleichzeitig mehrere Clients mit dem Kern
                                           * verbunden sind (zZt. nur unter Windows fuer
                                           * Cornelsen-Version) */

 /* Protokollerweiterung f"ur Tab-Complete */

 #define MPRCcompl_request			31
 #define MPRCcompl_list				32
 #define MPRCcompl_newinput			33

/* Spezifikation:

MPRCcompl_request (char *input)
	Initiert die Vervollst"andigung. "input" sollte der Inhalt des Eingabepuffers sein,
	und zwar vom Start bis zur Position des Cursors. Beispiel:

	>> fun1(arg) + pl<TAB>
	   [............] = input

MPRCcompl_list (char *list)
	Liefert eine kommaseparierte Liste von Bezeichnern an das Frontend. Diese Liste
	ist NULL, wenn es keine Vervollst"andigungen gab.

MPRCcompl_newinput (char *newinput)
	Liefert die vervollst"andigte Eingabe. Diese ersetzt den Eingabepuffer des
	Frontends vom Start bis zur Position des Cursors.

*/

 /* Protokollerweiterung MDB (Mupad DeBugger) */

 #define MPRCmdb_file_pos          34
 #define MPRCmdb_disp_list         35
 #define MPRCmdb_disp_list_begin   36
 #define MPRCmdb_disp_list_end     37
 #define MPRCmdb_disp_set          38
 #define MPRCmdb_disp_clear        39
 #define MPRCmdb_disp_clearall     40
 #define MPRCmdb_proc_list         41
 #define MPRCmdb_proc_list_begin   42
 #define MPRCmdb_proc_list_end     43
 #define MPRCmdb_proc_up           44
 #define MPRCmdb_proc_down         45
 #define MPRCmdb_proc_goto         46
 #define MPRCmdb_bkpt_list         47
 #define MPRCmdb_bkpt_list_begin   48
 #define MPRCmdb_bkpt_list_end     49
 #define MPRCmdb_bkpt_set          50
 #define MPRCmdb_bkpt_clear        51
 #define MPRCmdb_bkpt_clearall     52
 #define MPRCmdb_next              53
 #define MPRCmdb_step              54
 #define MPRCmdb_cont              55
 #define MPRCmdb_execute           56
 #define MPRCmdb_quit              57
 #define MPRCmdb_print             58
 #define MPRCmdb_help              59
 #define MPRCmdb_status            60
 #define MPRCmdb_output            61
 #define MPRCmdb_prompt            62
 #define MPRCmdb_error             63
 #define MPRCmdb_start             64

 #define MPRCmdb_finish            65		/* new */
 #define MPRCmdb_where             66
 #define MPRCmdb_pprint            67
 #define MPRCmdb_stop_at           68
 #define MPRCmdb_goto_proc         69
 #define MPRCmdb_proc_level        70		/* new */

/*  this is a direct example, not only a reference */
 #define MPRCnewexample            71

#define MPRCno_of_messages	  72


/* Spezifikation:
**
** Typ [Kommunikationsrichtung] Nachrichtenformat
** (M->X = MuPAD-Kernel an Frontend, X->M = andere Richtung, M<>X = beide Richtungen)
**
** Neu: MPRCmdb_???_list_begin
**                       ^^^^^
** Ein GUI kann eine evt. Liste lschen, danach kommen die Eintrge erneut.
**
**
** MPRCmdb_file_pos [M->X] "%s\n%ld\n%s" z.B. "file.mu\n10\nfunc"
**	Sendet den Namen der aktuellen Datei sowie die aktuelle Zeilennummer an das Frontend,
**	der Name und die Nummer sind durch ein Komma getrennt. Der Dateiname ist kann auch
**	ein URL bzw. ein TAR-URL sein, er muss vom Frontend mit den Kernel Funktionen
**	ge:offnet und gelesen werden. Erweiterung: Enth"alt auch noch den Prozedurnamen.
**
**
** MPRCmdb_disp_list [M->X] "%s\n%s\n..."
**	Sendet die Liste aller Display-Variablen. Die Liste ist ein String durch \n's
**	getrennt. Kann Unter Unix sofort weiterverwendet werden, unter Windows.. ? (\r ?)
**	Diese Liste wird immer gesendet, wenn im Kern eine Ver"anderung eintritt. Eine
**	explizite Abfrage halte ich f"ur unn"otig. Diese Liste ersetzt jede alte Liste!
**	Wird nach jedem Step usw. ausgel"ost! Wenn die Nachricht NULL ist, so ist die
** Liste leer.
**
**
** MPRCmdb_disp_set [X->M] "%s" z.B. "varname"
**	Setzen einer Display-Variablen. Existiert diese Variable nicht so sollte ein
**	Fehler zur"uckgesendet werden ? Enth"alt der String Trennzeichen (\t\n ) so
**	gilt jedes Teilwort als Variablenname.
**	-> l:ost MPRCmdb_disp_list aus !
**
**
** MPRCmdb_disp_clear [X->M] "%s" z.B. "varname"
**	L"oscht die angegebene Variable explizit. Evt. Fehlermeldung, falls nicht vorhanden.
**	(Kann nicht passieren, da das Frontend den Namen aus der Liste nimmt, die der Kern
**	gesendet hat. Existiert der Name nicht, sind Kern,Debugger und Frontend kaputt!)
**	-> l:ost MPRCmdb_disp_list aus !
**
**
** MPRCmdb_disp_clearall [X->M] NULL
**	L"oscht die interne Liste von Display-Variablen.
**	-> l:ost MPRCmdb_disp_list aus !
**
**
** MPRCmdb_proc_list [M->X] "%s\n%s\n%s..."
**	Gibt den Procstack aus. NULL bedeutet einen leeren Procstack.
**
**
** MPRCmdb_proc_up [M<>X] NULL
**	"Cursor-Up" im Procstack. Kann auf beiden Seiten ausgel:ost werden; Wenn das
**	Frontend dies ausl:ost, so sendet der Kern dies jedoch nochmal zur:uck; hier
**	muss man evt. aufpassen beim Programmieren.
**
**
** MPRCmdb_proc_down [M<>X] NULL
**	Identisch mit ..._up
**
**
** MPRCmdb_proc_goto [X->M] "%s" z.B. "procname"
**	MuPAD soll die Prozedur suchen. Dies l:ost dann ein MPRCmdb_file_pos aus,
**	d.h. die Datei mit der entsprechenden Prozedur wird angezeigt.
**	(Prozedur an Zeile 'pos')
**
**
** MPRCmdb_bkpt_list [M->X] "%s\n%s\n%s..."
**	Gibt die Liste der Breakpoints aus. NULL = keine Breakpoints.
**	Evt. Abfrage in R:uckrichtung ?
**
**
** MPRCmdb_bkpt_set [X->M] "%s,%ld[,%s]" z.B. "file.mu,10" oder "file.mu,10,cond=1"
**	Setzt einen Breakpoint, l:ost ein MPRCmdb_bkpt_list aus. Wenn das Frontend eine
**	Eingabezeile f"ur eine 'Condition' hat, so wird diese zus"atzlich :ubertragen.
**
**
** MPRCmdb_bkpt_clear [X->M] "%s,%ld" z.B. "file.mu,10"
**	L:oschen eines Breakpoints, l:ost ein MPRCmdb_bkpt_list aus.
**
**
** MPRCmdb_bkpt_clearall [X->M] NULL
**	L:oscht die komplette Liste von Breakpoints. L:ost ein MPRCmdb_bkpt_list aus,
**	mit Nachricht NULL.
**
**
** MPRCmdb_next [X->M] NULL
**	F:uhre NEXT aus
**
**
** MPRCmdb_step [X->M] NULL
**	F:uhre STEP aus
**
**
** MPRCmdb_cont [X->M] NULL
**	F:uhre CONT aus
**
**
** MPRCmdb_execute [X->M] "%s" z.B. "mupad expression"
**	Die angegebene 'expression' wird ausgef:uhrt. Das Ergebnis wird mit
**	MPRCmdb_output ausgegeben.
**
**
** MPRCmdb_quit [X->M] NULL
**	Beenden des Debuggermodus.
**
**
** MPRCmdb_print [X->M] "%s" z.B. "mupad expression to be printed"
**	Druckt den Inhalt einer 'expression' mit MPRCmdb_output. Ich weiss nicht
**	wo der Unterschied zu 'execute' liegen soll..
**
**
** MPRCmdb_help [X->M] NULL
**	Ruft den internen Hilfetext des Debuggers auf. Besser: 'Bubblehelp' auf
**	allen GUI Elementen ?
**
**
** MPRCmdb_status [M->X] "%s ??"
**	Kompakte Nachricht mit dem Status des Debuggers (running, waiting, line, file, ..)
**	Evt. nutzlos ?
**
**
** MPRCmdb_output [M->X] "%s" z.B. "stop at line ..."
**	Ausgabefunktion. Text ist steuerzeichenlos bis auf \n und \t.
**	(Frontend schaltet 'Wartecursor' ein..)
**
**
** MPRCmdb_prompt [M->X] "%s" z.B. "\nmdx> "
**	Prompt ausgeben.
**	(Frontend schaltet 'Wartecursor' aus..)
**
** MPRCmdb_start [X->M] "%s" z.B. "functionname"
**	Starten des Debuggermodus.
*/

/**************************************************************************
** MPRTtype                                                              **
** Instanzen dieses Typs geben an, um was f"ur einen Typ von Nachricht   **
** es sich bei dieser Nachricht handelt.                                 **
**************************************************************************/
typedef unsigned char MPRTtype ;
/**************************************************************************
** Dieser Typ ist fuer die unterste Protokollebene                       **
**************************************************************************/

typedef unsigned char MPRTuchar;


#ifndef MPR_TYPE_CONST

#define MEXDSTANDARD
#ifdef DOS_FILENAMES
#  include "MEX_ext.h"
#else
#  include "MEX_extern.h"
#endif

typedef int MPRTbool;
typedef	int MPRTstatus;

/**************************************************************************
** MPRTchannel                                                           **
** Auf Instanzen diesen Typs setzen die Funktionen des Protokolls auf.   **
** Eine Instanz besteht aus einem Zeiger auf ein FILE, aus dem die Daten **
** gelesen bzw. auf den sie geschrieben werden, einem Puffer zum         **
** zwischenpuffern von gelesenen Daten. Dieser wird ben"otigt, da einige **
** Bytes nur im Kontext mit den darauffolgenden Bytes interpretiert      **
** werden k"onnen. Au\3erdem beinhaltet die Instanz einen Z"ahler der    **
** angibt, wieviele Bytes im Puffer gespeichert sind.                    **
**************************************************************************/

typedef struct _MPRSchannel
{
	FILE	*file ;
	MPRTuchar	readbuffer[1] ;
	short	rsize ;
	int	bytes;
	MPRTtype last_type;
	int use_last;
} MPRSchannel ;

typedef MPRSchannel *MPRTchannel ;


/**************************************************************************
** MPRTchar                                                              **
** Dies ist der Typ, der aus einer Instanz des Typs MPRTchannel          **
** herausgelesen wird. Eine Instanz dieses Typs enth"alt mehr            **
** Informationen als nur den Wert des Bytes, den er repr"asentiert.      **
** Sie enth"alt z.B. die Information, ob es sich "uberhaupt um ein Byte  **
** handelt oder um das Ende einer Nachricht. Sie kann auch weitere       **
** Informationen zum Ende der Nachricht enthalten. Es w"are auch         **
** m"oglich, in eine Nachricht Informationen einzuschieben, dies m"u\3te **
** das entsprechende Protokoll dann aber verstehen.                      **
**************************************************************************/
typedef signed short MPRTchar ;


/**************************************************************************
** Sonstige Konstanten                                                   **
**************************************************************************/
#define MPRCfail		30	  /* ioctl() fail auf Deskribtor */
#define MPRCnone		31	  /* keine bytes auf Deskribtor  */
#define MPRCread		32	  /* bytes auf Deskribtor        */


/* Moegliche Signale des Typs MPRCsignal *******************************/
#define MPRC_SIG_break            'b'
#define MPRC_SIG_quit             'q'
#define MPRC_SIG_kill             'k'


/*
 * Konstanten f"ur die Kommunikation mit VCAM // "Anderungen bitte auch in
 *                          DOC/PAPER/KERNEL/MPR_protocol.tex aktualisieren
 *
 * Dazu einige Erl"auterungen:
 *
 * MPRC_mv_graphic_cmd      Kennzeichnet eine Botschaft von MuPAD an VCam,
 *                          in der ein Grafik-Befehl (= Grafikdaten + Befehl)
 *                          geschickt wird.
 *
 * MPRC_vx_plot_cmd         Kennzeichnet eine Botschaft von VCam an XMuPAD,
 *                          in der ein Plot-Befehl an XMuPAD geschickt wird,
 *                          der ins XMuPAD-Fenster geschrieben werden soll.
 *
 * MPRC_vm_plot_cmd         Kennzeichnet eine Botschaft von VCam an MuPAD,
 *                          in der dem Kern MuPAD ein Plot-Befehl geschickt
 *                          wird. Zur Zeit wird dieser Befehl wohl noch "uber
 *                          XMuPAD geschickt.
 *
 * MPRC_vx_graphic_error    Kennzeichnet eine Botschaft von VCam an XMuPAD,
 *                          in der XMuPAD mitgeteilt wird, dass das Grafiktool
 *                          Probleme hat (bspw. konnte eine Datei nicht ge"off-
 *                          net werden).
 *
 * MPRC_mv_graphic_error    Kennzeichnet eine Botschaft vom Kern MuPAD an das
 *                          Grafiktool VCam, in der diesem mitgeteilt wird, das
 *                          es bei der Bearbeitung eines Plot-Befehles, welcher
 *                          von VCam an den Kern geschickt wurde, ein Problem
 *                          gab. Bisher kam dann immer die Fehlermeldung: 'An
 *                          error was detected in the definition of the objects.'
 */

/**************************************************************************
** Konstanten des  Protokolls MPR...                                     **
**************************************************************************/


#define MPRCEOM             1000  /* Ende einer Nachricht */
#define MPRCNOCHAR          1001  /* Kein Zeichen auf dem Kanal */

/**************************************************************************
** Deklaration der Funktionalen Schnittstelle des Protokolls MPR...      **
**************************************************************************/
#ifndef MSDOS

void	        MPR_openFP	(MPRTchannel ch, FILE *fp);
void		MPR_close	(MPRTchannel ch) ;
MPRTtype	MPR_getstart	(MPRTchannel ch) ;
MPRTchar	MPR_get		(MPRTchannel ch) ;
MPRTchar	MPR_try		(MPRTchannel ch) ;
MPRTbool	MPR_EOM		(MPRTchar c) ;
MPRTbool	MPR_nochar	(MPRTchar c) ;
MPRTbool	MPR_ready       (MPRTchannel ch);
MPRTuchar       MPR_convert	(MPRTchar c) ;
void		MPR_flush	(MPRTchannel ch) ;
MPRTstatus	MPR_checkChannel(MPRTchannel ch) ;
void		MPR_putstart	(MPRTchannel ch, MPRTtype type) ;
void		MPR_putend	(MPRTchannel ch) ;
void		MPR_putchar	(MPRTchannel ch, MPRTuchar c) ;
void		MPR_putstring	(MPRTchannel ch, const char *s) ;
void		MPR_putmessage	(MPRTchannel ch, MPRTtype type, const char *s) ;
void		MPR_putsignal	(MPRTchannel ch, char signal) ;
void        MPR_pipemessage(MPRTchannel ich,MPRTchannel och,MPRTtype type);
void        MPR_ungetstart(MPRTchannel ch);

#else
/* Unter Windows wird nur die Ausgabe-Routine MPR_putmessage verwendet.
 * Eingaben werden vom Windows-Rahmen verarbeitet. Alle Ausgabe gehen auf
 * einen Kanal, nur der Typ ist entscheidend. */

#define MPR_putmessage(ch,type,str) { MSW_pr_puts((type), (str)); }

#endif /* ! MSDOS */

#endif /* ! MPR_TYPE_CONST */
#endif /* __MPR_protocol__ */
