/*
 * kg, 5.12.95
 *
 * MSW_int.h -- Interface-Routinen des Kerns zu Win32
 */

#ifndef _MSW_INT_H_
#define _MSW_INT_H_

#ifdef DOS_FILENAMES
#include "MMMstora.h"
#include "MPR_prot.h"
#include "MCT_iost.h"
#else
#include "MMMstorage_lean.h"
#include "MPR_protocol.h"
#include "MCT_iostream.h"
#endif

/* MSW_init -- initializes Win32-specific stuff which is independent from
 * actual frame implementation */
void MSW_init();

/* MSW_exit -- sorgt fuer korrekten Programmabbruch des Servers.
 * s gibt Statuswert an, 0 = fehlerfreier Abbruch */
void MSW_exit(int s);

/* MSW_cleanup -- interne Routine fuer Cleanup bei exit unter Windows. */
void MSW_cleanup(int s);

/* MSW_fatal -- Fehlermeldung und Abbruch nach fatalem Fehler. */
void MSW_fatal(char *s);

/* MSW_message -- Fehlermeldung nach internem Fehler, falls MEV_error
 * nicht verwendet werden kann. */
void MSW_message(char *s);

/* MSW_message_box -- show message in dialog box or write it to file in 
 * batch mode. */
void MSW_message_box(char *s, unsigned int type);

/* MSW_handle_events -- arbeitet evtl. anstehende Windows-Events waehrend
 * laengerer Rechnungen ab, wird von MAMMUT und PARI regelmaessig aufgerufen.
 * Dies ist notwendig, damit andere Applikationen auch mal 'and die Reihe
 * kommen' und nicht haengen. */
void MSW_handle_events(bool main_loop, bool wait);

/* MSW_alarm -- installiert die Alarm-Funktion tf, die in s Sekunden 1mal
 * aufgerufen werden soll. Ein bestehender Alarm wird geloescht. Ist s = 0,
 * so wird nur ein ggf. bestehender Alarm geloescht.
 * Zurueckgeliefert werden sollte eigentlich die Zeit bis zum geloeschten
 * Alarm, zZt. aber immer 0. */
int MSW_alarm(int s, void (*tf)(void));

/* MSW_clock -- liefert User-Zeit des Task's in Millisekunden. */
long MSW_clock(void);

/* MSW_get_tmp_filename -- liefert in Puffer buff Name fuer temporaeres
 * File. Die Pufferlaenge ist len. nr dient zur Unterscheidung mehrerer
 * temporaerer Files und wird in den Namen integriert. */
char *MSW_get_tmp_filename(char *buff, int len, short nr);

/* MSW_is_registered -- liefert TRUE falls der Benutzer registriert ist. */
int MSW_is_registered(void);

/* MSW_register -- registriert Benutzer
 * Liefert FALSE, falls der Benutzer nicht registriert werden konnte. */
int MSW_register(char *name);

/* MSW_register_campus -- registriert Campus
 * Liefert FALSE, falls der Campus nicht registriert werden konnte. */
int MSW_register_campus(char *name);

/* MSW_open_plot_file -- liefert Fileptr fuer temp. File, das zur Uebertragung
 * von Plot-Daten ans Frontend benutzt wird.
 * Der Name des Files wird in den Buffer buff kopiert. */
int MSW_open_plot_file(char *buff, MCLT_iostream &file);

/* MSW_close_plot_file -- schliesst temp. File zur Uebertragung von Plot-Daten,
 * damit das Frontend das File zum Lesen oeffnen kann. */
void MSW_close_plot_file(void);

/* MSW_user_init_file -- liefert Name des User-Init-File
 * Liefert NULL, falls der Benutzer kein User-Init-File angegeben hat. */
char *MSW_user_init_file(void);

/* MSW_lib_path -- liefert Pfad der Standard-Library
 * Liefert 0, falls der Benutzer keinen Pfad angegeben hat. */
int MSW_lib_path(char* buff, int buff_len);

#endif /* ! _MSW_INT_H_ */

