///////////////////////////////////////////////////////////////////////////////
// TOOLBOX : MTBpari.h
// AUTHOR  : Andreas Sorgatz (Andreas.Sorgatz@sciface.com)
// CREATED : 10.Nov.1999 
// CHANGED : 10.Nov.1999 - andi
// CONTENTS: Support of accessing PARI number in dynamic modules
//
// This header file contains extension of the MuPAD Application Programming
// Interface (MAPI)  and facilitates the  development of dynamic modules in
// MuPAD. This file must only refer to MuPAD objects listed in  MDM_mod.h
// and MKT_mapi.h.
///////////////////////////////////////////////////////////////////////////////

#define MuPAD                                               /* PARI adaption */
#include "genpari.h"
#include "pari-arithmetic.h" 

///////////////////////////////////////////////////////////////////////////////
#if   (defined MACINTOSH) 
#elif (defined WIN32) 
#else  
#endif

#ifdef GMP ////////////////////////////////////////////////////////////////////

#include "gmp.h"

///////////////////////////////////////////////////////////////////////////////
// FUNC: MFgmp2pariInt
// ARGS: gmpInt  - GMP  integer
//       pariInt - PARI integer
// DESC: Converts a GMP integer number into a PARI integer number.
///////////////////////////////////////////////////////////////////////////////
inline void MFgmp2pariInt ( mpz_t gmpInt, GEN &pariInt )
{ long        glen = abs(gmpInt->_mp_size);
  long        plen = glen+2;
  mp_limb_t  *gptr = gmpInt->_mp_d +glen -1;
  GEN         pptr;

  pariInt = MFparii( plen );
  setlgef( pariInt, plen );
  if      ( gmpInt->_mp_size < 0 ) setsigne( pariInt, -1 );
  else if ( gmpInt->_mp_size > 0 ) setsigne( pariInt,  1 );
  else                             setsigne( pariInt,  0 );
  pptr = pariInt +2;

  for( long i=2 ; i<plen ; i++, pptr++, gptr-- ) *pptr = (long) *gptr;
}

///////////////////////////////////////////////////////////////////////////////
// FUNC: MFpari2gmpInt
// ARGS: gmpInt  - GMP  integer
//       pariInt - PARI integer
// DESC: Converts a PARI number integer into a GMP integer number.
///////////////////////////////////////////////////////////////////////////////
inline void MFpari2gmpInt ( mpz_t &gmpInt, GEN pariInt )
{ long        plen = lgef(pariInt);
  long        glen = plen -2;
  GEN         pptr = pariInt +2;
  mp_limb_t  *gptr;

  mpz_init( gmpInt );
  _mpz_realloc( gmpInt, (mp_size_t) glen );
  gmpInt->_mp_size = glen *signe(pariInt);
  gptr = gmpInt->_mp_d +glen -1;

  for( long i=2 ; i<plen ; i++, pptr++, gptr-- ) *gptr = (mp_limb_t) *pptr;
}

#endif /* USE_GMP */ //////////////////////////////////////////////////////////

