/* $Date: 1999/06/02 07:22:28 $  $Author: kg $  $Revision: 1.23 $ */

/**************************************************************************/
/* FILE:             MTR_tree.h                                           */
/* Autor:            Andreas Kemper, Andreas Sorgatz                      */
/* Erstellt:         25/05/91                                             */
/* Letzte Aenderung: 28/12/95                                             */
/*                                                                        */
/* Dieses File deklariert alle zum Aufbau der Datenstrukturen noetigen    */
/* Prozeduren und Macros.                                                 */
/**************************************************************************/

#ifndef __MTR_tree__
#define __MTR_tree__


#ifdef DOS_FILENAMES
#  include "MMMstorl.h"
#else
#  include "MMMstorage_lean.h"
#endif


/* Struktur zur Definition von Kategorien (MuPAD-Basis-Typen)        */

typedef struct {
	short cat;              /* Enthaelt den Kategorietyp         */
	short spec;             /* Die beruehmten MuPAD special-bits */
} CATEGORY ;

/**************************************************************************/
/* 			Typen	                                          */
/**************************************************************************/

/* STRSIZE darf _kein_ unsigned sein, da Funktionen wie MSR_strcmp auf
 * STRSIZE len >= 0 abfragen und der Fall len < 0 in diesen Routinen
 * auftreten kann! (kg) */

// Because basic routines which handles MMM_STRING should be contained
// in this "Module", it's also necessary that this type is defined
// here and not in MSR_string.h

typedef long STRSIZE;		/* Typ fuer Stringlaenge		  */


/* Typdefinition fuer Zeiger auf MuPAD-Systemfunktionen MEV_eval_... */

typedef S_Pointer (FUNC)         ( S_Pointer s,
                                   long      prev_func,
                                   long      eval_type,
                                   S_Pointer exec );

typedef S_Pointer (MSYT_eval_func)( S_Pointer s,
                                    long      prev_func,
                                    long      eval_type,
                                    S_Pointer exec );

/* Makro zum Zugriff auf den Funktionszeiger in MMM_CODE Objekten */

#define MTRC_CODE(s,T) ((T *) *MMMZ((s), T *))

//---------------------------------------------------------------------------

#ifndef MMMD_LIT_SP

/**
 Declares a kernel C-function for which a MMM\_CODE object is to be created.

 The declarations are inserted into a linked list which is used to create the
 MMM\_CODE objects when the kernel is started.

 When literals are to be generated the names of the C-functions are also defined.
 They are needed in the generated code for MMM\_CODE literals.
 */
class MTRT_code_decl
{
public:
#ifdef MMMD_GEN_LIT_SP
    /// Create a declaration given the function pointer, its code number and its name.
    MTRT_code_decl(void* func, long number, const char* name);
#else
    MTRT_code_decl(void* func, long number);
#endif
    /// Get first declaration in list
    static const MTRT_code_decl* first();
    /// Get next declaration
    const MTRT_code_decl* next() const;
    /// Get pointer to C-function
    const void* func() const;
    /// Get identifying code number of function
    long number() const;
#ifdef MMMD_GEN_LIT_SP
    /// Get C-name of function
    const char* name() const;
#endif

private:
    static const MTRT_code_decl* first_;  // first declaration
    const void* func_;                    // pointer to kernel C-function
    const long num_;                      // number (id) of function
#ifdef MMMD_GEN_LIT_SP
    const char *const name_;              // C-name of function
#endif
    const MTRT_code_decl* next_;          // pointer to next declaration
};

inline const MTRT_code_decl* MTRT_code_decl::first() { return first_; }
inline const MTRT_code_decl* MTRT_code_decl::next() const { return next_; }
inline const void* MTRT_code_decl::func() const { return func_; }
inline long MTRT_code_decl::number() const { return num_; }
#ifdef MMMD_GEN_LIT_SP
inline const char* MTRT_code_decl::name() const { return name_; }
#endif

#endif // !MMMD_LIT_SP

//---------------------------------------------------------------------------
// Macros used to declare, define, create and delete literal S-Pointers

#ifdef MMMD_GEN_LIT_SP

/// Declare a literal S-Pointer variable #var#.
#define MTR_DECL_LITERAL(var) \
    extern S_Pointer var;

/// Define a literal S-Pointer variable #var#.
#define MTR_DEF_LITERAL(var) \
    S_Pointer var; \
    static MMMT_literal MMMV_lit_##var(&var, #var );

/// Delete the S-Pointer of the variable #var#.
#define MTR_DELETE_LITERAL(var) \
    MMMfree(var);

/// Create literal S-Pointer variable.
/// #var# is the name of the variable, #val# its value (a S-Pointer).
#define MTR_NEW_LITERAL(var, val) \
    var = (val); \
    MMMinsert_global(&var);

/// Create literal S-Pointer variable containing MMM-string.
/// #var# is the name of the variable, #val# a const C-string
/// containing the strings content.
/// The string is copied, so it must be deleted somewhere!
#define MTR_NEW_MMM_STR_LITERAL(var, val) \
    var = MTR_set_string((val)); \
    MMMcopy(var); \
    MMMinsert_global(&var);

/// Create literal S-Pointer variable containing CAT-string.
/// #var# is the name of the variable, #val# a const C-string
/// containing the strings content.
/// The string is copied, so it must be deleted somewhere!
#define MTR_NEW_CAT_STR_LITERAL(var, val) \
    var = MTR_new_cat_string((val)); \
    MMMinsert_global(&var);

/// Create literal S-Pointer variable containing CAT_IDENT.
/// #var# is the name of the variable, #val# a const C-string
/// containing the name of the ident.
/// The string is copied, so it must be deleted somewhere!
#define MTR_NEW_CAT_IDENT_LITERAL(var, val) \
    var = MTR_new_cat_ident((val)); \
    MMMinsert_global(&var);

/// Declares C-function for which a MMM\_CODE object is to be created.
/// #var# ist the function, #num# its identiying code number.
#define MTR_DECL_MMM_CODE(var, num) \
    static MTRT_code_decl MTRV_code_##var((void*)(&var), (num), #var);

#elif MMMD_LIT_SP

#define MTR_DECL_LITERAL(var) \
    extern S_Pointer var;

#define MTR_DEF_LITERAL(var) \
    S_Pointer var;

#define MTR_DELETE_LITERAL(var)

#define MTR_NEW_LITERAL(var, val)
#define MTR_NEW_MMM_STR_LITERAL(var, val)
#define MTR_NEW_CAT_STR_LITERAL(var, val)
#define MTR_NEW_CAT_IDENT_LITERAL(var, val)

#define MTR_DECL_MMM_CODE(var, num)

#else // !MMMD_GEN_LIT_SP && !MMMD_LIT_SP

#define MTR_DECL_LITERAL(var) \
    extern S_Pointer var;

#define MTR_DEF_LITERAL(var) \
    S_Pointer var;

#define MTR_DELETE_LITERAL(var) \
    MMMfree(var);

#define MTR_NEW_LITERAL(var, val) \
    var = (val); \
    MMMinsert_global(&var);

#define MTR_NEW_MMM_STR_LITERAL(var, val) \
    var = MTR_set_string((val)); \
    MMMcopy(var); \
    MMMinsert_global(&var);

#define MTR_NEW_CAT_STR_LITERAL(var, val) \
    var = MTR_new_cat_string((val)); \
    MMMinsert_global(&var);

#define MTR_NEW_CAT_IDENT_LITERAL(var, val) \
    var = MTR_new_cat_ident((val)); \
    MMMinsert_global(&var);

#define MTR_DECL_MMM_CODE(var, num) \
    static MTRT_code_decl MTRV_code_##var((void*)(&var), (num));

#endif // !MMMD_GEN_LIT_SP && !MMMD_LIT_SP

/****** Makros zur Anforderung von Speicherplatz fuer Grundtypen **********/

/* Typ MMM_LIST */
#define MTR_new_list() MMMmalloc(MMM_LIST,0,2)

/* Typ MMM_STRING */
#define MTR_new_string(n) MMMpalloc(MMM_STRING,(long)(sizeof(char)*n+sizeof(STRSIZE)),0)

/* Typ MMM_Category mit 'n' Verweisen */
#define MTR_new_category(n) MMMmalloc(MMM_CATEGORY,(long)sizeof(CATEGORY),(long) n)

/* Typ MMM_DIST_POLY mit 'n' Termen in 'k' Unbestimmten */
#define MTR_new_dist_poly(n,k) MMMmalloc(MMM_DIST_POLY,(long)(sizeof(long)*((n)*(k))),(long)(n))


/* Variablen dieses Modules *********************************************/

extern S_Pointer MTRV_STRING_ASS_MEM;
extern S_Pointer MTRV_VAR_ASS_MEM;

#ifdef MMMD_LIT_SP
extern S_Pointer MTRV_LIT_STRING_ASS_MEM;
extern S_Pointer MTRV_LIT_VAR_ASS_MEM;
#endif

MTR_DECL_LITERAL( MTRV_code_table )

MTR_DECL_LITERAL( MUPV_CAT_NULL )
MTR_DECL_LITERAL( MUPV_CAT_NIL )
MTR_DECL_LITERAL( MUPV_CAT_BOOL_TRUE )
MTR_DECL_LITERAL( MUPV_CAT_BOOL_UNKNOWN )
MTR_DECL_LITERAL( MUPV_CAT_BOOL_FALSE )
MTR_DECL_LITERAL( MUPV_CAT_FAILED )

/******  Deklaration der im File MTR_tree.c definierten Prozeduren  *******/

void      MTR_init 		( );
void      MTR_exit              ( );
S_Pointer MTR_new_code 		( const void* f, long num );
S_Pointer MTR_set_int 		( long num );
S_Pointer MTR_new_cat_exec 	( long n );
S_Pointer MTR_new_cat_bool 	( long n );
S_Pointer MTR_new_cat_null 	( );
S_Pointer MTR_new_cat_nil 	( );
S_Pointer MTR_new_cat_failed 	( );
S_Pointer MTR_new_cat_set_finite( );
S_Pointer MTR_new_cat_point     ( S_Pointer  s1,
			          S_Pointer  s2,
			          S_Pointer  s3,
			          S_Pointer  color);

S_Pointer MTR_new_cat_polygon   ( long       num,
				  S_Pointer  *sp,
				  S_Pointer  color,
				  S_Pointer  closed,
				  S_Pointer  filled);
S_Pointer MTR_new_cat_stat_list ( long n );
S_Pointer MTR_new_cat_expr 	( long n );
S_Pointer MTR_new_cat_func_env 	( );
S_Pointer MTR_new_proc 		( );
S_Pointer MTR_new_user_func 	( S_Pointer	name,
				  long		num
				  Mva_arglist );
S_Pointer MTR_new_sys_expr_empty( long		type,
				  long		num );
S_Pointer MTR_new_sys_expr0     ( long		type,
				  long		num );
S_Pointer MTR_new_sys_expr 	( long		type,
				  long		num
				  Mva_arglist );
S_Pointer MTR_build_sys_expr 	( long		type,
				  long		num
				  Mva_arglist );
S_Pointer MTR_shift_sys_expr    ( long          type,
                                  S_Pointer     args,
				  long          start,
				  long          end );
S_Pointer MTR_new_cat_ident_sys ( long type );
S_Pointer MTR_new_cat_string_sys( long type );
S_Pointer MTR_new_cat_ident 	( const char *s );
S_Pointer MTR_new_cat_ident_MMMstring ( S_Pointer mmmstring ) ;
S_Pointer MTR_new_cat_string 	( const char *s );
S_Pointer MTR_set_string 	( const char *s );
S_Pointer MTR_new_tmp_string    ( const char *s );
S_Pointer MTR_new_cat_dom 	( long count );
S_Pointer MTR_new_cat_ext 	( long count );
S_Pointer MTR_new_cat_debug 	( unsigned long value );
S_Pointer MTR_new_cat_bytecode  ( S_Pointer s );
S_Pointer MTR_new_cat_var 	( long lexdist, long idx );
S_Pointer MTR_new_cat_proc_env  ( S_Pointer env );

S_Pointer MTR_new_mmm_proc_env  ( S_Pointer proc,
				  S_Pointer procname,
				  S_Pointer lexEnv,
				  S_Pointer dynEnv,
				  S_Pointer dom,
				  long saved,
				  long locals,
				  long formals,
				  long actuals);

void      MTR_load_func_env     ( unsigned long func);

S_Pointer MTR_find_number_for_code(S_Pointer s) ;
S_Pointer MTR_find_code_for_number(S_Pointer s) ;

/**
   Returns the MMM\_CODE object for the function with number #num#
   @param num the number of the kernel function (as defined in MUP_funcs.h)
   @return the MMM\_CODE object for the function or MMMNULL if the function
           does not exist
*/
S_Pointer MTR_find_code_for_number(long num);


/**
   Returns the category type of a MMM\_CATEGORY S\_Pointer
   @param   sp  A s-pointer of type MMM\_CATEGORY
   @return  A value representing the category type CAT\_XXX of the given
            S\_Pointer {\tt sp}.
*/
short MTR_cat( S_Pointer sp );

/**
   Returns the special bits of a MMM\_CATEGORY S\_Pointer
   @param   sp  A s-pointer of type MMM\_CATEGORY
   @return  The special bits of the given S\_Pointer {\tt sp}.
*/
short MTR_spec( S_Pointer sp );

/**
    Return the underlying C-String of the given S\_Pointer of type MMM\_STRING
    @param   stringi  A S\_Pointer of type MMM\_STRING
    @return  A pointer to the underlying C-String
*/
const char* MTR_get_string( const S_Pointer stringi ) ;

/**
    Return the long value stored in a MMM\_INT
    @param   value  A S\_Pointer of type MMM\_INT
    @return  The long value stored in the MMM\_INT
*/
long MTR_int( const S_Pointer value ) ;

/**
    Return the approbriate MuPAD constant  MUPC_constants_CAT\_BOOL\_XXX which
    is hold in the S\_Pointer booli. Remeber the result is one of
    3 values (true, false, unknown) !
    @param   booli  A S\_Pointer of type CAT\_BOOL
    @return  The "boolean" value of type bool
*/
MTbool MTR_bool( const S_Pointer booli ) ;

//---------------------------------------------------------------------------
// Implementation

inline short MTR_cat( S_Pointer sp )
{
    assert(MMMtype(sp) == MMM_CATEGORY);
    return MMMZ(&sp, CATEGORY)->cat ;
}

inline short MTR_spec( S_Pointer sp )
{
    assert(MMMtype(sp) == MMM_CATEGORY);
    return MMMZ(&sp, CATEGORY)->spec ;
}

inline const char* MTR_get_string( const S_Pointer stringi )
{
   assert( MMMtype(stringi) == MMM_STRING ) ;
   return MMMmv( &stringi, sizeof(STRSIZE), const char) ;
}

inline long MTR_int( const S_Pointer value )
{
   assert( MMMtype(value) == MMM_INT ) ;
   return (*MMMZ( &value, long)) ;
}

inline MTbool MTR_bool( const S_Pointer booli )
{
   assert( MTR_cat(booli) == CAT_BOOL ) ;
   const S_Pointer *sohn = MMMP( (S_Pointer *) &booli, 0) ;
   return( (MTbool)  MTR_int( *sohn ) ) ;
}

#endif /* __MTR_tree__ */
