/* $Date: 2000/05/25 08:23:37 $  $Author: kg $  $Revision: 1.37 $ */

/**************************************************************************/
/* FILE   : MUP_extern.h                                                  */
/* AUTHOR : Andreas Kemper, Oliver Kluge, Karsten Morisse                 */
/* CREATED: 12.03.1991                                                    */
/* CHANGED: 13.09.1997                                                    */
/**************************************************************************/

#ifndef __MUP_extern__
#define __MUP_extern__

#include "MTR_tree.h"
#include "MUP_env.h"


extern MUPT_env  MUPV_env;            // MuPAD Kernel Environment (MUP_main)


/**************************************************************************/

class MCLT_bytestream;
class MCLT_iostream;

/* Variable zur Speicherung der realen Prozedurtiefe.           */

GLOBAL_DECL( long, MUPV_REAL_PROC_DEPTH );


/* Gloabale Verweise auf MuPAD-Konstanten *********************************/

MTR_DECL_LITERAL(MCAC_ci);
MTR_DECL_LITERAL(MCAC_one);
MTR_DECL_LITERAL(MCAC_r_one);
MTR_DECL_LITERAL(MCAC_one_);
MTR_DECL_LITERAL(MCAC_two );
MTR_DECL_LITERAL(MCAC_two_);
MTR_DECL_LITERAL(MCAC_zero);
MTR_DECL_LITERAL(MCAC_r_zero);
MTR_DECL_LITERAL(MCAC_half);
MTR_DECL_LITERAL(MCAC_rd_inf);
MTR_DECL_LITERAL(MCAC_rd_ninf);
MTR_DECL_LITERAL(MCAC_rd_nan);

/* Diese sind nun im Modul 'MTR_tree.c' defininiert, damit  */
/* sie auch in VCAM (MSV_save.c) zur Verfuegung stehen.     */

extern MCLT_iostream MFOV_PROT_FILE;    // Protocol file pointer or NULL

extern bool MFOV_PROT_ALL;      // true = input+output, false = input only

GLOBAL_DECL( short, MEVV_SUBSTITUTE );
GLOBAL_DECL( long, MEVV_ACT_DEPTH );

GLOBAL_DECL( long,           MEVV_LEVEL );
GLOBAL_DECL( long,           MEVV_PREV_LEVEL );
GLOBAL_DECL( long,           MEVV_MAXLEVEL );
GLOBAL_DECL( long,           MEVV_MAXDEPTH );
GLOBAL_DECL( long,           MEVV_TEXTWIDTH );

GLOBAL_DECL( S_Pointer, MPAV_parser_config );

extern S_Pointer MUT_LIB_PATH;
extern S_Pointer MUT_READ_PATH;
extern S_Pointer MUT_WRITE_PATH;

extern S_Pointer MPAV_CALLBACKS;

/** Initializes the kernel.
 * Command line arguments may be given here and also may be given later 
 * by the first client which connects to the kernel. (See the method
 * #MTkernel::send_init#.) Please note that #argv[0]# {\em must} hold
 * a string with the name of the kernel. One should not try to re-init
 * the kernel once this function call failed.
 * @param argc number of command line arguments
 * @param argv array of #argc+1# strings: the first one defines the
 *    name of the kernel, the remaining strings hold command
 *    line arguments 
 * @return false if an error occured */
bool MUP_maininit( int argc, char *argv[] ) ;

/** Execute commands from command queue.
 * Executes all commands currently stored in the command queue.
 * If #runUntilQuit# is true the kernel then waits for commands 
 * to come and executes them until 'quit' is executed.
 * @param runUntilQuit the kernel waits for commands and executes 
 *      them until 'quit' is executed. */
void MUP_mainexecute( bool runUntilQuit=true );

/** Terminate the kernel.
 * Terminates the kernel and frees its resources. May only be called
 * when the kernel is regulary quit, not on errors.
 * @param stat exit status (if exit is called)
 * @param terminateProgram if true #exit# is called with the given status */
void MUP_mainexit( int stat, bool terminateProgram );

/** Display fatal error message and exit.
 * The message depends on the current error state of the kernel. */
void MUP_fatal_mssg( );

void MUP_print_banner( bool info );

#ifndef MACINTOSH
#if !defined MSDOS && !defined __EMX__
void  MUP_catchstop (  );
#endif
void  MUP_catchint (  );
void  MUP_catch_system_error (  );
void  MUP_catch_alarm (  );
#else /* MACINTOSH */
void  MUP_catchint  ( char c );
#endif /* MACINTOSH */

#ifdef MAINEXECUTE1
long MUP_mainexecute1 ( char *stmt, MCLT_iostream &out ) ;
#endif

void MUP_out_of_memory();

inline void MUP_swap(long* v1, long* v2)
{
    long t = *v1; *v1 = *v2; *v2 = t;
}

inline void MUP_swap(bool* v1, bool* v2)
{
    bool t = *v1; *v1 = *v2; *v2 = t;
}

inline void MUP_swap(S_Pointer* v1, S_Pointer* v2)
{
    S_Pointer t = *v1; *v1 = *v2; *v2 = t;
}

#endif /* __MUP_extern__ */

