#ifndef __MVC_vector_h__
#define __MVC_vector_h__

/*
 * MVC_vector.h -- dynamische Arrays
 *
 * ACHTUNG: Dynamische Arrays duerfen nur intern verwendet werden
 * und nicht in Ausdruecken! Sie verwenden keinen eigenen MMM-Typ,
 * koennen nicht verglichen werden und sind zB. nicht in die Ausgabe
 * integriert.
 *
 * Ein Vektor ist wie ein C-Array mit 0 beginnend indiziert.
 * Nur beim Einfuegen mit MVC_..._push_back wird der Vektor ggf.
 * verlaengert. (Die Laenge des Vektors wird verdoppelt.)
 */

#include "MMMstorage_lean.h"

// Erzeugt einen Vektor von S-Pointern, initiale Groesse ist size.
S_Pointer MVC_point_vector(long size=8);

// Erzeugt einen Vektor von Speicherbereichen. Jeder Speicherbereich
// muss die gleiche Groesse len (in Bytes) haben. Die initiale Groesse
// des Vektors ist size.
S_Pointer MVC_mem_vector(long len, long size=8);

// Liefert die Anzahl der Elemente des Vektors.
long MVC_size(S_Pointer vector);

// Liefert die Groesse der Elemente des Vektors in Bytes.
long MVC_mem_size(S_Pointer vector);

// Fuegt einen S-Pointer am Ende des Vektors ein. Der Pointer wird NICHT
// kopiert!
void MVC_point_push_back(S_Pointer* vector, S_Pointer elem);

// Fuegt den Speicherbereich, auf den elem_ptr verweist, am Ende des
// Vektors ein.
void MVC_mem_push_back(S_Pointer* vector, void* elem_ptr);

// Entfernt das letzte Element aus dem Vektor.
// Das Element wird NICHT freigegeben.
void MVC_pop_back(S_Pointer vector);

// Liefert den S_Pointer mit Index idx. Der Pointer wird NICHT kopiert!
S_Pointer* MVC_point_at(S_Pointer* vector, long idx);

// Liefert die Adresse des Speicherbereichs mit Index idx.
void* MVC_mem_at(S_Pointer* vector, long idx);

// Setzt die Groesse des Vektors auf 0. Speicher wird NICHT neu
// initialisiert!
void MVC_reset(S_Pointer vector);

//----------------------- Implementation --------------------------------------

inline S_Pointer MVC_point_vector(long size) {
    S_Pointer s = MMMpalloc(MMM_RW_LIT, sizeof(long), size);
    *MMMZ(&s, long) = 0;
    return s;
}

inline S_Pointer MVC_mem_vector(long len, long size) {
    S_Pointer s = MMMmalloc(MMM_RW_LIT,
                            (2 * sizeof(long) + len * size), 0);
    *MMMZ(&s, long) = 0;
    *MMMarr(&s, 1, long) = len;
    return s;
}

inline long MVC_size(S_Pointer vector) {
    long s = *MMMZ(&vector, long);
    return s;
}

inline long MVC_mem_size(S_Pointer vector) {
    long ms = *MMMarr(&vector, 1, long);
    return ms;
}

inline void MVC_reset(S_Pointer vector) {
    *MMMZ(&vector, long) = 0;
}

inline void MVC_pop_back(S_Pointer vector) {
    (*MMMZ(&vector, long))--;
}

#endif
